#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

use FindBin;
use lib "$FindBin::Bin/tools";
use BuildTimeConfig;

=pod "

=head1 NAME

B<configure.pl> - IDX-PKI configuration script, which generates F<config.status>

=head1 SYNOPSIS

B<configure.pl> [I<options>]

=head1 DESCRIPTION

This script allows to configure IDX-PKI for build and installation. It can
be called both with command-line arguments or interactively, or any mixing
of the two methods (if a mandatory parameter is not supplied, it will ask
for it).

It reads and propose the values found in a previously generated
F<config.status> file.  Command-line arguments take precedence over
previously entered values, which take precedence over built-in default
values.

=head1 OPTIONS

The name of the options correspond to the values in the F<config.status>
file, lower-cased and with C<-> instead of C<_>.

=head2 Generic Options

=over 4

=item B<--batch>

Do not process the old F<config.status>, and do not ask questions. In this
mode, a missing mandatory parameter is a fatal error.

=item B<--full-featured>

This option activates some "private" IDX-PKI functionalities, which are
not provided in the published GPL version. Don't use it unless you are
sure that your version of IDX-PKI supports them.


=back

=cut

our %opts;

=pod

=head2 Installation Options

=over 4

=item B<--install-all>

If true, installs all IDX-PKI entities (the other B<--install-*> options
become optional).

=item B<--install-ee> I<(mandatory)>

Install IDX-PKI's EE on this machine?

=item B<--install-ra> I<(mandatory)>

Install IDX-PKI's RA on this machine?

=item B<--install-ca> I<(mandatory)>

Install IDX-PKI's OpCA on this machine?

=item B<--install-rootca> I<< (available only if B<--full-featured> is
activated; mandatory in this case) >>

Install IDX-PKI's RootCA on this machine?

=item B<--install-renewal> I<(mandatory)>

Install the renewal module on this machine?

=cut "


=pod "

=item B<--install-scep>

Install tools to do SCEP requests? Default is false.

=cut "


=pod "

=item B<--install-publish> I<(mandatory)>

Install IDX-PKI's publication module on this machine?

=cut "


=pod "

=item B<--ca-keysize>

CAs keysize (defaults to 4096 if B<--test-keys> is not set, 512
otherwise).

=item B<--keysize>

End-user certificates keysize (defaults to 1024 if B<--test-keys> is not
set, 512 otherwise).

=item B<--rootca-validity> I<< (available only if B<--full-featured> and
B<--install-rootca> are set) >>

Root CA validity in days. Defaults to 10958 (30 years).

=item B<--ca-validity>

Operational CA validity in days. Defaults to 1826 (5 years).

=item B<--validity>

Default standard certificates validity in days. Defaults to 365 (1 years).

=cut "


=pod "

=item B<--ca-crl-validity>

Operational CA CRL validity in days. Defaults to 30 (1 month).

=item B<--target-distribution> I<(mandatory)>

The target distribution to create configuration files for.

=item B<--test> I<(mandatory)>

Activates test setup options.  All B<--test-*> options are empty if
B<--test> is not set.

=item B<--test-privs> I<< (mandatory if B<--test> is set) >>

Unprivileged installation.  Default is true if B<--test> is set.

=item B<--test-keys> I<< (mandatory if B<--test> is set) >>

Use testing small-size keys.  Default is true if B<--test> is set.

=item B<--test-autosigned-opca> I<< (true if B<--full-featured> is not
set, otherwise mandatory with a true default if B<--test> and
B<--install-ca> are set, otherwise false) >>

Use a self-signed operational CA?

=item B<--test-rootca> I<< (available only if B<--full-featured> and
B<--test> are set but not B<--test-autosigned-opca>, then mandatory with a
true default) >>

Use a semi-automatic root CA? It is mostly useful for testing purposes,
when you don't want to play with floppies (like you would with the real
rootCA) but still don't want a self-signed OpCA.

Don't use this option for production purposes, as I<nothing> is done here
to take care about security. After all, it is a B<test> option.

=item B<--install-webserver> I<< (mandatory if B<--test> is not set) >>

Install a fully-configured web server? Otherwise you will have to manually
integrate the generated Apache configuration file in your existing
configuration. Say no if you already have Apache installed and configured.
Default is true.

=back

Hostnames for various PKI entities (yes, we may want different hostnames
for different entities, for example if the webserver is to be named
F<www.mydomain.net> when the LDAP server is to be named
F<ldap.mydomain.net> and the primary name for this host is
F<myhost.mydomain.net>).

This is mostly useful to generate webserver certificates which bear the
proper hostname, so as to avoid browsers warnings.

=over 4

=item B<--webserver-hostname> I<< (mandatory) >>

Hostname to use for the webserver. Defaults to the local hostname.

=item B<--repository-ldap-hostname> I<< (only if B<--install-ca> is set,
mandatory in this case) >>

Hostname to use for the repository LDAP. Defaults to the local hostname.

=cut "


=pod "

=back

IDX-PKI DNs are defined here for the sake of centralising informations,
but are not supposed to be changed.

=over 4

=item B<--idxpki-subdn>

Reserved sub DN for IDX-PKI.

=item B<--admin-cn>

CN of the administrator of the PKI, for the bootstrap certificate.

=cut "


=pod "

=item B<--ca-cn>

CN of IDX-PKI Operational CA.

=item B<--ra-cn>

CN of IDX-PKI RA.

=item B<--ee-cn>

CN of IDX-PKI EE.

=item B<--publish-cn>

CN of IDX-PKI publication module.

=item B<--repository-ldap-cn>

CN of IDX-PKI repository LDAP.

=cut "


=pod "

=back

=cut "


# Shortcuts for further simplification of dependent options

our @opttemplates = (
	"full-featured" =>
	{
		"type"    => "bool",
		"default" => 0,
	},

	'install-all' =>
	{
		'question' => "install all IDX-PKI entities on this machine",
	    'default' => 1,
	},

	'install-ee' => install_all_implies ("install IDX-PKI's EE on this ".
										 "machine"),

	'install-ra' => install_all_implies ("install IDX-PKI's RA on this ".
										 "machine"),

	'install-ca' => install_all_implies("install IDX-PKI's OpCA on this ".
										"machine"),

	'install-renewal' => install_all_implies ("install the renewal module ".
											  "on this machine"),
	'install-publish' => install_all_implies ("install IDX-PKI's ".
		"publication module on this machine"),

	"install-scep" => {
		"default"   => 0,
	},


	"ca-keysize" => {
		"type"    => "integer",
		"default" => sub {valueof ("test-keys") ? 512 : 4096},
	},

	"keysize" => {
		"type"    => "integer",
		"default" => sub {valueof ("test-keys") ? 512 : 1024},
	},



	"ca-validity" => {
		"type"    => "integer",
		# 5 years x 365.25 days = 1826.25 days.
		"default" => 1826,
	},

	"validity" => {
		"type"    => "integer",
		"default" => 365,
	},

	"ca-crl-validity" => {
		"type"    => "integer",
		"default" => 30,
	},

    'target-distribution' => {
		"mandatory" => 1,
		"question"  => "Target distribution",
		"default"   => sub {
			my %files = ("redhat"   => "/etc/redhat-release",
						 "debian"   => "/etc/debian_version",
						 "mandrake" => "/etc/mandrake-release",
						 "suse"     => "/etc/SuSE-release");
			my ($distro, $ver);
			foreach my $key (keys (%files)) {
				next unless (-f $files{$key});
				$distro = $key;
				local *FILE;
				open (FILE, "<", $files{$distro});
				$ver = <FILE>;
				chomp ($ver);
				close FILE;
				if ($ver =~ m/.*([0-9][0-9]*\.[0-9][0-9]*[a-z]*).*/) {
					# From http://distro.pipfield.ca/
					$ver = $1;
				}
			}
			return undef unless ($distro);
			return $distro.(defined ($ver) ? "-".$ver : "");
		},
    },

	'test' =>
	{
		"type"     => "boolean",
		'question' => "test setup options",
		'default'  => 0,
	},

	'test-privs' => test_enables("unprivileged installation"),

	'test-keys' => test_enables ("use testing small-size keys"),

	'test-autosigned-opca' => {
		"mandatory" => sub {
			return 0 if (! valueof ("full-featured"));
			return undef if (! valueof ("test"));
			return undef if (! valueof ("install-ca"));
			return 1;
		},
		"default"   => sub {
			return 1 if (! valueof ("full-featured"));
			return undef if (! valueof ("test"));
			return undef if (! valueof ("install-ca"));
			return 1;
		},
		"question"  => sub {
			return undef if (! valueof ("full-featured"));
			return undef if (! valueof ("test"));
			return undef if (! valueof ("install-ca"));
			return "use a testing self-signed operational CA";
		},
	},

	"test-rootca" =>
	{
		"mandatory" => sub {
			return undef if (! valueof ("full-featured"));
			return undef if (! valueof ("test"));
			return undef if (valueof ("test-autosigned-opca"));
			return 1;
		},
		"default"   => sub {
			return undef if (! valueof ("full-featured"));
			return undef if (! valueof ("test"));
			return undef if (valueof ("test-autosigned-opca"));
			return 1;
		},
		"question"  => sub {
			return undef if (! valueof ("full-featured"));
			return undef if (! valueof ("test"));
			return undef if (valueof ("test-autosigned-opca"));
			return "use a testing semi-automatic root CA";
		},
	},

	'install-webserver' => {
		"mandatory" => sub {valueof ("test") ? undef : 1},
		"question"  => sub {valueof ("test") ? undef :
								"do the Apache installation automatically"},
		"default"   => 1,
	},

# Servers hostnames

	"webserver-hostname" =>
	{
		"mandatory" => 1,
		"question"  => "Hostname to use for the webserver",
		"default"   => BuildTimeConfig::hostname (),
	},

	"repository-ldap-hostname" =>
	{
		"mandatory" => sub {
			valueof ("install-ca") ? 1 : undef;
		},
		"question"  => sub {
			valueof ("install-ca") ?
				"Hostname to use for the repository LDAP" :
				undef;
		},
		"default"   => sub {
			valueof ("install-ca") ? BuildTimeConfig::hostname () : undef;
		},
	},


# DNs
	'idxpki-subdn' =>
	{
		'default'  => "OU=IDX-PKI",
	},

	'admin-cn' =>
	{
		'default'  => "Administrator",
	},


	'ca-cn' =>
	{
		'default'  => "IDX-PKI Operational CA",
	},

	'ra-cn' =>
	{
		'default'  => "RA",
	},

	'ee-cn' =>
	{
		'default'  => "EE",
	},

	'publish-cn' =>
	{
		'default'  => "publish",
	},


	'repository-ldap-cn' =>
	{
		'default'  => "LDAP-repository",
	},


	'transporttype-ee' =>
	{
		'default'  => sub {valueof ("full-featured") ? "HTTPS" : "local"},
	},

	'transporttype-ra' =>
	{
		'default'  => sub {valueof ("full-featured") ? "HTTPS" : "local"},
	},

	'transporttype-ca' =>
	{
		'default'  => sub {valueof ("full-featured") ? "HTTPS" : "local"},
	},

	'transporttype-publish' =>
	{
		'default'  => sub {valueof ("full-featured") ? "HTTPS" : "local"},
	},

);


=pod "

=head2 Directory and File Names

If the given path is relative, the value of the prefix is prepended.  All
these values can only be set through command-line options.

=over 4

=item B<--prefix>

Prefix directory (to be prepended to the default values for directory
locations).

=item B<--shelllibdir>

Directory for shell include files.

=item B<--perllibdir>

Directory for perl modules.

=item B<--docdir>

Directory for documentation (subdir doc) and manual pages (subdir
man/man<n>).

=item B<--sbindir>

Directory for cron and administrative scripts.

=item B<--bootstrapdir>

Directory for semi-graphical post-installation ("bootstrap") programs

=item B<--etcdir>

Top directory for configuration files and such.

=item B<--configdir>

Directory for configuration files.

=item B<--sysvdir>

Directory for System-V startup scripts.

=item B<--vardir>

Top directory for work files.

=item B<--logsdir>

Top directory for IDX-PKI logs.

=item B<--htdocsdir>

Directory for web pages.

=item B<--securedir>

Directory for secure secrets (must not be backuped without care).

=item B<--crldir>

Directory for IDX-PKI CRLs.

=item B<--cacertsdir>

Directory for IDX-PKI CAs certificates.

=back

=cut "

push @opttemplates,(
	'prefix' =>
	{
	    'question' => "Installation prefix",
		'default'  => sub {
			return "/opt/idxpki" if (! valueof ("test-privs"));
			require FindBin;
			return $FindBin::RealBin."/install_image";
		},
	},

	'perllibdir' =>
	{
		'default'  => sub {valueof("prefix")."/lib/perl5/site_perl"},
	},

	'shelllibdir' =>
	{
		'default'  => sub {valueof("prefix")."/lib/idxpki"},
	},

	'docdir' =>
	{
		'default'  => sub {valueof("prefix")},
	},

	'sbindir' =>
	{
		'default'  => sub {valueof("prefix")."/sbin"},
	},

	'bootstrapdir' =>
	{
	 'default' => sub {valueof("prefix")."/bin"},
	},

	'etcdir' =>
	{
		'default'  => sub {valueof("prefix")."/etc/idxpki"},
		'temporary' => 1,	# Only used to compute some defaults values.
	},

	'configdir' =>
	{
		'default'  => sub {valueof("etcdir")."/conf"},
	},

	'sysvdir' =>
	{
		'default'  => sub {valueof("prefix")."/etc/init.d"},
	},

	'vardir' =>
	{
		'default'  => sub {valueof("prefix")."/var/idxpki"},
	},

	'logsdir' =>
	{
		'default'  => sub {valueof("vardir")."/log"},
	},

	'htdocsdir' =>
	{
		'default'  => sub {valueof("vardir")."/www"},
	},

	'securedir' =>
	{
		'default'  => sub {valueof("vardir")."/secure"},
	},

	'scepdir' =>
	{
		'default'  => sub {
			valueof ("install-scep") ? valueof("vardir")."/scep" : undef},
	},

	'crldir' =>
	{
		'default'  => sub {valueof("vardir")."/crl"},
	},

	'cacertsdir' =>
	{
		'default'  => sub {valueof("etcdir")."/ca"},
	},
);


=pod "

=head2 Network Addresses and Ports

=over 4

=item B<--port-https> I<< (mandatory if B<--test-privs> is set) >>

Port for IDX-PKI's secure WWW server.

=item B<--port-http> I<< (mandatory if B<--test-privs> is set) >>

Port for IDX-PKI's insecure WWW server (used for publication purposes).

=item B<--port-ldap-repository> I<< (mandatory if both B<--test-privs> and
B<--install-ca> are set) >>

Port for the IDX-PKI's repository LDAP server (defaults to 636 if
B<--test-privs> is not set).

If B<--install-ca> is not set, this is a bootstrap-time value, not a
configure-time value.

=item B<--port-ldap-escrow> I<< (mandatory if both B<--test-privs> and B<--install-escrow> are set) >>

Port for the IDX-PKI's escrow LDAP server (defaults to 1636 if
B<--test-privs> is not set).

If B<--install-escrow> is not set but B<--use-escrow> is, this is a
bootstrap-time value, not a configure-time value.

=item B<--port-ldap-publishing> I<< (mandatory if both B<--test-privs> and
B<--install-ldap-publishing> are set) >>

Port for the IDX-PKI's publishing LDAP server (defaults to 389 if
B<--test-privs> is not set).

=item B<--port-db-logs> I<< (available only if B<--install-logs> is
set) >>

Port for the IDX-PKI XML logs database server (defaults to 5432).

If B<--install-ldap-publishing> is not set but B<--use-ldap-publishing>
is, this is a bootstrap-time value, not a configure-time value.

=item B<--default-language>

Default language to use for web pages.  Defaults to "en" in the published
version, "fr" otherwise.

=back

=cut "

push @opttemplates,(
# Port numbers
	'port-https' => {
	    'type'      => "int",
		"mandatory" => sub {valueof ("test-privs") ? 1 : 0},
		'question'  => "port for IDX-PKI's secure WWW server",
		'default'   => sub {valueof ("test-privs") ? undef : 443},
	},

	'port-http' => {
	    'type'      => "int",
		"mandatory" => sub {valueof ("test-privs") ? 1 : 0},
		'question'  => "port for IDX-PKI's insecure WWW server (used for ".
			"publication purposes)",
		'default'   => sub {valueof ("test-privs") ? undef : 80},
	},

	'port-ldap-repository' => {
		'type'      => "int",
		"mandatory" => sub {
			my $ca   = valueof ("install-ca");
			my $priv = valueof ("test-privs");
			($priv && $ca) ? 1 : 0},
		'question'  => sub {
			valueof ("install-ca") ?
				"port for the IDX-PKI's repository LDAP server" : undef},
		'default'   => sub {
			my $ca   = valueof ("install-ca");
			my $priv = valueof ("test-privs");
			($ca && ! $priv) ? 636 : undef},
	},


	'default-language' =>
	{
		'default'  => "en",
	},
);

=pod "

=head2 Unix privileges

The Unix privileges are all empty if the B<--test-privs> option is set.

=over 4

=item B<--privs-group-main> I<< (mandatory if B<--install-webserver> and
B<--test-privs> are not set) >>

Group name or GID for main IDX-PKI elements.

=item B<--privs-group-ldap-repository> I<< (only if B<--install-ca> is set
but not B<--test-privs>) >>

Group name or GID for the repository LDAP server.  LDAP servers are just
databases, UNIX-isolated from the rest of the PKI.  So we offer a private
group for them.

=cut "


=pod "

=item B<--privs-group-ldap-publishing> I<< (only if
B<--install-ldap-publishing> is set but not B<--test-privs>) >>

Group name or GID for the publishing LDAP server.

=item B<--privs-user-www> I<< (mandatory if B<--install-webserver> and
B<--test-privs> are not set) >>

User name or UID for the WWW server (and cron scripts, as they do the same
job than web CGIs, just automatically :-).

=item B<--privs-user-ldap-repository> I<< (only if B<--install-ca> is set
but not B<--test-privs>) >>

User name or UID for the repository LDAP server.  LDAP servers are just
databases, UNIX-isolated from the rest of the PKI.  So we offer a private
user for them.

=cut "


=pod "

=item B<--privs-user-ldap-publishing> I<< (only if
B<--install-ldap-publishing> is set but not B<--test-privs>) >>

User name or UID for the external LDAP server.

=item B<--privs-group-db-logs> I<< (only if B<--install-logs> is set
but not B<--test-privs>) >>

Group name or GID for the XML logs database server.

=item B<--privs-user-db-logs> I<< (only if B<--install-logs> is set but
not B<--test-privs>) >>

User name or UID for the XML logs database server.

=back

=cut "

push @opttemplates,(
	'privs-group-main' => {
		"mandatory" => sub {
			my $web  = valueof ("install-webserver");
			my $priv = valueof ("test-privs");
			($priv || $web) ? 0 : 1},
		"question"  => sub {
			valueof ("test-privs") ? undef :
				"group name or GID for IDX-PKI elements (must be ".
				"the same as your Apache server)"},
		'default'   => sub {valueof ("test-privs") ? undef : "idxpki"},
	},

	'privs-group-ldap-repository' => {
		'question'  => sub {
			my $ca   = valueof ("install-ca");
			my $priv = valueof ("test-privs");
			($priv || (! $ca)) ? undef :
				"group name or GID for the LDAP repository server"},
		'default'   => sub {
			my $ca   = valueof ("install-ca");
			my $priv = valueof ("test-privs");
			# Warning, "idxpki-repository" is too long for adduser.
			($priv || (! $ca)) ? undef : "idxpki-repos"},
	},


	'privs-user-www' => {
		"mandatory" => sub {
			my $web  = valueof ("install-webserver");
			my $priv = valueof ("test-privs");
			($priv || $web) ? 0 : 1},
		"question"  => sub {
			valueof ("test-privs") ? undef :
				"user name or UID for the WWW server (must be ".
				"the same as your Apache server)"},
		'default'   => sub {valueof ("test-privs") ? undef : "idxpki-www"},
	},

	'privs-user-ldap-repository' => {
		'question'  => sub {
			my $ca   = valueof ("install-ca");
			my $priv = valueof ("test-privs");
			($priv || (! $ca)) ? undef :
				"user name or UID for the LDAP repository server"},
		'default'   => sub {
			my $ca   = valueof ("install-ca");
			my $priv = valueof ("test-privs");
			# Warning, "idxpki-repository" is too long for adduser.
			($priv || (! $ca)) ? undef : "idxpki-repos"},
	},

);


=pod "

=head2 Binaries and libraries location

These values can only be set through the command-line option.

=over 4

=item B<--with-openssl-root>

The OpenSSL development kit to use.  Include files are expected in F<<
<dir>/include >> or F<< <dir>/include/openssl >>, and linkable libraries
in F<< <dir>/lib >>.  Default is F</usr>.

=item B<--with-openldap-root>

The OpenLDAP development kit to use.  Include files are expected in F<<
<dir>/include >>, and linkable libraries in F<< <dir>/lib >>.  Default is
F</usr>.

=item B<--with-apache>

Apache binary, F<apache> or F<httpd>.

=item B<--with-openssl>

=item B<--with-stunnel>

=item B<--with-slapd>

=item B<--with-slapadd>

=item B<--with-ldapmodify>

=item B<--with-slappasswd>

Paths for components of IDX-PKI's required software.  Augmented PATH (with
the standard F<sbin> directories) is searched by default.

=item B<--with-openssl-include>

Path to openssl includes directory.

=item B<--with-schemadir>

The OpenLDAP schema directory.

=back

=cut "

%opts = runOpts ();		# Sets %opts.

@loctemplates = (
	'with-openssl-root' =>
	{
		'directory'   => "/usr",
	},

	'with-openldap-root' =>
	{
		'directory'   => "/usr",
	},

	'with-apache' =>
	{
		# When will Linux distributions use same names?... *sigh*
		'executable'   => ["apache", "httpd"],
	},

	'with-openssl' =>
	{
		'executable'   => "openssl",
	},

	'with-openssl-include' =>
	{
		'directory'   => ["/usr/include/openssl", "include/openssl"],
	},

	'with-stunnel' =>
	{
		'executable'   => "stunnel",
	},

	'with-slapd' =>
	{
		'executable'   => "slapd",
	},

	'with-slapadd' =>
	{
		'executable'   => "slapadd",
	},

	'with-ldapmodify' =>
	{
		'executable'   => "ldapmodify",
	},

	'with-slappasswd' =>
	{
		'executable'   => "slappasswd",
	},

	'with-schemadir' =>
	{
		# When will Linux distributions use same names?... *sigh*
		'directory'   => [
						  "/etc/openldap/schema",		# RedHat 7.2
						  "/usr/share/openldap/schema",	# Mandrake 8.2
						  "/etc/ldap/schema",			# Debian 3.0
						  "openldap/schema",
						  "ldap/schema",
						  ],
	},
);
if ($opts{"install-logs"}) {
	push (@loctemplates,
		  "with-pg-postmaster" => {
			  "executable" => [
							   # Debian 3.0
							   "/usr/lib/postgresql/bin/postmaster",
							   "postmaster",	# Std path
							   ],
			  },
		  "with-pg-initdb" => {
			  "executable" => [
							   # Debian 3.0
							   "/usr/lib/postgresql/bin/initdb",
							   "initdb",	# Std path
							   ],
			  },
		  );
}

%opts = runLocate ();	# Updates %opts.


####################  PLATFORM AUTO-DETECTION  ####################

# To maintain type:
# grep -r 'use .*::' sources/src/ | perl -pe 's/^.*(use (?:\w|:)*).*?$/$1/g' \
#    | sort | uniq | grep -v IDX::PKI
# But don't forget the dependencies... They are indented in the list below.

# Please don't enforce a version dependancy if unnecessary.

our %needed_perl_modules = (
	'Apache'          => 0,			# Interactions with request object
	'Carp'            => 0,			# Here and there
	'CGI'             => 0,			# Obsolete IDX::PKI::CGI
	'Config::IniFiles'=> "1.07",	# IDX::Config
	'Data::Dumper'    => 0,			# Mostly for tests, but not only
	'Errno'           => 0,			# Symbolic errors (LDAP & transport)
	'Error'           => "0.14",	# Everywhere!
	'Exporter'        => 0,			# Ditto
	'Fatal'           => 0,			# LDAP init.d script
	'IO::Scalar'      => 0,			# For Config & CertDB
	'Fcntl'           => 0,			# Config & transport file locking
	'File::Sync'      => "0.09",	# For IDX::PKI::Transport
	'FindBin'         => 0,			# Installation and tests
	'IO::Handle'      => 0,			# Here and there
	'IO::Pipe'        => 0,			# Ditto
	'IO::Socket'      => 0,			# Ditto
# Don't use 0.92, it will break things.
	'IO::Socket::SSL' => "0.93",	# SSL connections
	'IO::Socket::UNIX'=> 0,			# LDAP
#	'IPC::Open3'      => 0,			# Tests only
	'LWP'             => "5.48",	# SSL connections, HTTPS transport
	'MIME::Base64'    => "2.11",	# Crypto & transport
#	'Net::HTTP'       => 0,			# Test webserver
	'Net::LDAP'       => "0.26",	# LDAPs ;-)
	'Net::SSLeay'     => "1.08",	# SSL connections
	'Pod::Usage'	  => 0,			# Standalone scripts
	'POSIX'	  		  => 0,			# Here and there
	'Socket'	  	  => 0,			# LDAP and PostgreSQL
	'Sys::Hostname'	  => 0,			# Bootstrap-time
	'Sys::Syslog'	  => 0,			# Default / standard logging method
	'Taint'	  		  => "0.07",	# For IDX::PKI::Registry
#	'Test::Harness'	  => 0,			# Tests only
	'Tie::IxHash'     => "1.21",	# Convenience for the view
);

if ($opts{"use-verification-database"}) {
	$needed_perl_modules{"DBI"} = 0;
	$needed_perl_modules{"DBIx::Abstract"} = "1.003";
	$needed_perl_modules{"DBD::".$opts{"database-type"}} = 0;
}

if ($opts{"use-xml-logs"}) {
	$needed_perl_modules{"IO::Dir"} = 0;
	$needed_perl_modules{"Unicode::String"} = 0;
	# These modules aren't needed for writing logs, only for parsing them.
	if ($opts{"install-logs"}) {
		$needed_perl_modules{"DBI"} = 0;
		$needed_perl_modules{"DBD::Pg"} = 0;
		$needed_perl_modules{"XML::Grove::Builder"} = 0;
		$needed_perl_modules{"XML::Parser::PerlSAX"} = 0;
	}
}

our %needed_software = (
						"perl"     => '5.006',
						"suidperl" => 0,
						"slapd"    => '2.0.23',
						"apache"   => '1.3.19',
						"openssl"  => '0.9.6',
						);

runVersions();	# Writes final config.status.
