#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<mkcrontab.pl> - Makes the crontab for IDX-PKI

=head1 SYNOPSIS

B<mkcrontab.pl> [B<--remove>] [B<--user> I<user>] [B<--stdout>]

=head1 DESCRIPTION

B<mkcrontab.pl> installs (or uninstalls if the B<--remove> switch is
present) the crontab or crontabs appropriate for IDX-PKI, according to the
configuration. If B<--stdout> is set, the computed crontab is printed to
STDOUT rather than being directly installed.

=over 4

=item *

In test mode for privileges, installs or prints only one crontab that
shall be installed as the user's crontab.

=item *

If no B<--user> is specified, outputs all the crontabs in Vixie Cron's
F</etc/crontab> format (that is, with usernames prefixed); in this case,
B<--stdout> is always assumed. This has no meaning with B<--remove>.

=item *

If B<--user> is specified, installs or prints this user's crontab.

=back

=cut "

use strict;

use IDX::PKI::Config;

use Getopt::Long;
use Pod::Usage;

use vars qw ($remove $user $stdout $config);
GetOptions ("remove" => \$remove,
			"stdout" => \$stdout,
			"user=s" => \$user) ||
	pod2usage ("-verbose" => 1, "-exitval" => 1);

$config = the IDX::PKI::Config;

my $sbindir = $config->getval ("paths", "sbindir");
my $sysvdir = $config->getval ("paths", "sysvdir");
my $wwwuser = $config->val ("privileges", "user_www");
my $testprivs = $config->val ("_components", "test_privs");

# Sanity checks.
$user = undef if ($testprivs);
pod2usage ("-verbose" => 1, "-exitval" => 1)
	if ((! $testprivs) && (! $user) && $remove);
$stdout = 1 if ((! $testprivs) && (! $user));

####################
# Computes entries that will be put or removed.

=pod "

=head2 Cron entries computation

Cron entries are put into the I<@cronentries> array. Its elements are
anonymous arrays (C<[user, command, frequency, comments]> where comments
are optional). Multiple comments means multiple lines.

=cut "

my @cronentries;


# Operational CA.
if ($config->val ("_components", "ca")) {
	push (@cronentries,
		  [$wwwuser, "$sbindir/auto-ca.pl", "1-59/5 * * * *", "Automatic CA."],
		  [$wwwuser, "$sbindir/ca-crl-gen.pl", "01 04 * * *", "Issue CRL."],
		  );
}; # End if install CA.

# Publication module.
if ($config->val ("_components", "publish")) {
	push (@cronentries,
		  [$wwwuser, "$sbindir/publish.pl", "2-59/5 * * * *",
		   "Sends emails for new issued & revoked certificates."],
		  );
}

# Renewal module.
if ($config->val ("_components", "renewal")) {
	push (@cronentries,
		  # Once a day, exact date is meaningless.
		  [$wwwuser, "$sbindir/renewal.pl", "41 02 * * *",
		   "Sends emails for certificates that should be renewed."],
		  );
}


if ($config->val ("_components", "webserver")) {
	push (@cronentries,
		  [$wwwuser, "$sbindir/reload-CRL-in-Apache.pl", "3-59/5 * * * *",
		   "To force the update of the CRL in Apache. This is not necessary",
		   "with a patched mod_ssl that reloads its own CRL."],
		  );
}


# Removes non applicable entries, if test_privs is not set but we got a
# --user.
if ((! $testprivs) && $user) {
	@cronentries = grep {($_->[0] eq $user)} @cronentries;
}

####################
# Change the crontab. Cf. InstallLib::installcron().

# Get the current crontab.
my $crontab = "";
if ($testprivs || $user) {
	# Empty crontab if neither $testprivs nor $user are set.
	local *OLDCRONTAB;
	open (OLDCRONTAB, "-|", "crontab".($user ? " -u $user" : "")." -l") ||
		die "Couldn't get the old crontab: $!";
	my @crontab = <OLDCRONTAB>;
	close (OLDCRONTAB);
	# RedHat specific: Remove cruft at top.
	if ($crontab[0] && $crontab[1] && $crontab[2] &&
		$crontab[0] =~ m/^\# DO NOT EDIT THIS FILE/ &&
		$crontab[1] =~ m/^\# / && $crontab[2] =~ m/^\# /) {
		splice (@crontab, 0, 3);
	}
	$crontab = join ("", @crontab);
}

foreach my $entry (@cronentries) {
	die "Incorrect entry $entry" if (ref ($entry) ne "ARRAY");
	my ($curuser, $cmd, $freq, @comments) = @$entry;
	die "Incorrect entry: no command" unless ($cmd);

	# Canonicalise comments.
	unshift (@comments, "Automatically installed by IDX-PKI.");
	@comments = map {m/^\#/ ? "$_\n" : "# $_\n"} @comments;

	# Separator. Cosmetic for alignment!
	my $tab;
	if ($freq =~ m/^\@[a-z]*$/) {
		# '@blah' format.
		$tab = "\t\t";
	} elsif ($freq =~ m/^[^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+$/) {
		# '* * * * *' format.
		$tab = "\t";
	} else {
		die "Invalid crontab frequence value: '$freq'\n";
	}

	my $cmdtest = quotemeta ($cmd);
	my $freqpat = qr '(@[a-z]+)|([^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+)';

	my $oldpat = qr /(^|\n)
		(?:\#\ Automatically\ installed\ by\ IDX\-PKI.\n(?:\#[^\n]*\n)*)?
		$freqpat\s+$cmdtest\s*\n/sx;

	if (! $remove) {		# Add line.
		my $new = join ("", @comments)."$freq$tab".
			(($testprivs || $user) ? "" : "$user\t")."$cmd\n";
		if ($crontab =~ m/^$freqpat\s+$cmdtest\s*$/m) {
			# Already present, maybe with a different frequency.
			$crontab =~ s/$oldpat/$1$new/sx;
		} else {
			$crontab .= $new;
		}
	} else {				# Remove line.
		if ($crontab !~ m/^$freqpat\s*$cmdtest\s*$/m) {
			# Not present, even with a different frequency: Nothing to do.
		} else {
			$crontab =~ s/$oldpat/$1/sx;
		}
	}
}

if ($stdout) {
	print $crontab;
} else {
	open (NEWCRONTAB, "|-", "crontab".($user ? " -u $user" : "")." -") ||
		die "Couldn't set the new crontab: $!";
	print NEWCRONTAB $crontab;
	close (NEWCRONTAB);
}

exit 0;
