#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<mkhttpd-conf.pl> - Generates an Apache configuration file

=head1 SYNOPSIS

B<mkhttpd-conf.pl>
[B<--standalone>]
[B<--rootdir> I<rootdir>]
[B<--pidfile> I<pidfile>]

=head1 DESCRIPTION

Generates an Apache configuration file and writes it on the standard
output.  If the B<--standalone> parameter is present, generates a complete
configuration file; otherwise generates only what is relevant for IDX-PKI,
so you can include it in your main Apache configuration file.

If the B<--standalone> parameter is present, I<pidfile> and I<rootdir> are
mandatory (respectively the file containing its PID and its root
directory).

=cut "

use strict;

use Getopt::Long;
use Pod::Usage;
use FindBin;

sub usage {
	require Carp;
	Carp::carp @_;
	pod2usage(-verbose=>1,-exitval=>1,-output=>\*STDERR);
}

# Include installation tools.
use vars qw ($topdir);
$topdir = "$FindBin::RealBin/../../";
push (@INC, "$topdir/tools");
require InstallLib;
import  InstallLib ();

# Program global variables.
use vars qw (%CONFIG
	$standalone
	$rootdir
	$pidfile

	$porthttps
	$porthttp
	$user
	$group
	$vardir
	$logsdir

	$htdocsdir
	$securedir
	$perllibdir
	$target_distribution
);
%CONFIG = parseConfigStatus ($topdir);
$standalone = 0;

my @OPTS = (
	'standalone!'     => \$standalone,
	'rootdir=s'       => \$rootdir,
	'pidfile=s'       => \$pidfile,
);

$porthttps   = $CONFIG{'CONFIG_PORT_HTTPS'};
$porthttp    = $CONFIG{'CONFIG_PORT_HTTP'};
$user        = $CONFIG{'CONFIG_PRIVS_USER_WWW'};
$group       = $CONFIG{'CONFIG_PRIVS_GROUP_MAIN'};
$vardir      = $CONFIG{'CONFIG_VARDIR'};
$logsdir     = $CONFIG{'CONFIG_LOGSDIR'};
$htdocsdir   = $CONFIG{'CONFIG_HTDOCSDIR'};
$securedir   = $CONFIG{'CONFIG_SECUREDIR'};
$perllibdir  = $CONFIG{'CONFIG_PERLLIBDIR'};
$target_distribution = $CONFIG{'CONFIG_TARGET_DISTRIBUTION'};

GetOptions (@OPTS) || usage();

usage ("No default value for target-distribution, you must pass it as ".
	   "an argument") unless ($target_distribution);

if ($standalone) {
	foreach my $ref (\$rootdir, \$pidfile) {
		next if ($$ref);
		my %revopts = reverse @OPTS;
		usage("missing option $revopts{$ref} for standalone server");
	}
}

usage("missing dirs") unless ($htdocsdir &&
	$vardir && $securedir && $perllibdir);

my $basepath = $standalone ? "" : "/idxpki";
my $basedir  = ($basepath eq "") ? "/" : $basepath;

my $config = << "CONFIGFILE";
# Configuration file for IDX-PKI Apache web server.
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

CONFIGFILE


# We need them in both cases, but in different places.
my $certfile = "$securedir/WWW/webserver.crt";
my $keyfile  = "$securedir/WWW/webserver.key";
my $cadir    = $CONFIG{'CONFIG_CACERTSDIR'};
my $crldir   = $CONFIG{'CONFIG_CRLDIR'};
my $ssldefinitions = << "SSLDEFINITIONS";
    SSLEngine on
    SSLOptions +StdEnvVars +ExportCertData +StrictRequire

    SSLCertificateFile    $certfile
    SSLCertificateKeyFile $keyfile
    SSLCACertificatePath  $cadir
<IfDefine WITH_CRL>
    # mod_ssl is a plague! It fails miserably to use a CRL directory
    # although it is documented. Fortunately a "crl-bundle.pem" file does
    # the job.
    SSLCARevocationFile   $crldir/crl-bundle.pem
</IfDefine>

    SSLVerifyClient optional
    SSLVerifyDepth  2

    # Restricted directories.
    <LocationMatch ^$basepath/(RA|CA|escrow|config|logs)>
        SSLRequire ( %{SSL_CLIENT_VERIFY} eq "SUCCESS" )
    </LocationMatch>
SSLDEFINITIONS


##############################################################################
# Complete Apache installation only.
##############################################################################
if ($standalone) {
	my $hostname  = $CONFIG{'CONFIG_WEBSERVER_HOSTNAME'} | "localhost";
	my $varrundir = $CONFIG{'CONFIG_VARDIR'}."/run";
	$config .= << "CONFIGFILE";
ServerType standalone
ServerRoot     $rootdir

PidFile        $pidfile
LockFile       $varrundir/idxpki-httpd.lock
ScoreBoardFile $vardir/idxpki-httpd.scoreboard

Timeout 300
KeepAlive On
MaxKeepAliveRequests 100
KeepAliveTimeout 15

MinSpareServers 5
MaxSpareServers 20
StartServers    5

MaxClients 150
MaxRequestsPerChild 100

ServerName  $hostname
ServerAdmin webmaster\@$hostname

DocumentRoot $htdocsdir

CONFIGFILE

	sub findmodule {
		my ($modulename)=@_;

		my ($modtoken)=($modulename =~ m/^(.*)_module$/);
		# Dyslectic logs ?
		$modtoken =~ s/^(.*)_log$/log_$1/;
		# Lazy action(s) ?
		$modtoken =~ s/^action$/actions/;

		my $modulesourcename="mod_$modtoken.c";
		my $modulepath;

		if (($target_distribution =~ m/^mandrake/i) &&
			($modtoken =~ m/^perl/)) {
			# Perl is statically linked to Apache under Mandrake
			return (undef,undef,$modulesourcename);
		}

		if ($target_distribution =~ m/^(redhat|mandrake)/i) {
			if ($modtoken =~ m/^perl|php|ssl/) {
				$modulepath = ($target_distribution =~ m/^mandrake/) ?
					"/usr/lib/apache-extramodules/lib$modtoken.so" :
					"/usr/lib/apache/lib$modtoken.so";
			} else {
				$modulepath = "/usr/lib/apache/mod_$modtoken.so";
			}
		} elsif ($target_distribution =~ m/^debian/i) {
			my $version = `$CONFIG{'CONFIG_WITH_APACHE'} -v`;
			$version =~ s/^.*?([0-9]\.[0-9]+).*/$1/is;
			if ($modtoken =~ m/^php/) {
				$modulepath = "/usr/lib/apache/$version/lib$modtoken.so";
			} else {
				$modulepath = "/usr/lib/apache/$version/mod_$modtoken.so";
			}
		} else {
			die "Unsupported target platform $target_distribution";
		}
		warn "\n!!!! Module $modulename does not appear to be installed on ".
			"your system !!!!\n(Path should be $modulepath)\n\n"
			if (! -f $modulepath);
		return ($modulename, $modulepath, $modulesourcename);
	};

	my @modules = map {findmodule($_)} (qw(autoindex_module env_module
			config_log_module agent_log_module referer_log_module mime_module
			negotiation_module status_module setenvif_module dir_module
			cgi_module action_module alias_module headers_module
			perl_module php4_module ssl_module));

	my @modules_copy = @modules;
	while (my ($name, $path, undef) = splice (@modules_copy, 0, 3)) {
		$config .= sprintf ("LoadModule %-20s %s\n", $name, $path) unless
			(!defined $name);
	}
	$config .= "\nClearModuleList\n";
	@modules_copy = @modules;
	$config .= "AddModule mod_so.c\n"; # Or the rest won't load, of course.
	while (my (undef, undef, $name_c) = splice (@modules_copy, 0, 3)) {
		$config .= "AddModule ${name_c}\n";
	}

	$config .= "\n";

	# There just isn't any way of both using setgroups() and binding to a
	# privileged port (short of Linux capabilities tricks, that is).  So
	# we are forced to use UNIX groups this way around: Any file that
	# needs to be shared between the WWW server and process foo must
	# belong to the server's GID, not foo's GID.
	if ($user && $group) {
		$config .= "User  $user\n";
		$config .= "Group $group\n\n";
	}

	my $mimetypes= ($target_distribution =~ m/^mandrake/i) ?
		"/etc/httpd/conf/apache-mime.types":
		"/etc/mime.types";


	$config .= << "CONFIGFILE";
UseCanonicalName Off

TypesConfig $mimetypes
DefaultType text/plain

HostnameLookups Off

ServerSignature On

AddEncoding x-compress Z
AddEncoding x-gzip gz tgz

AddLanguage en .en
AddLanguage fr .fr
AddLanguage de .de
AddLanguage da .da
AddLanguage el .el
AddLanguage it .it

LanguagePriority fr en de

AddType application/x-tar       .tgz
AddType application/x-httpd-php .php

# Compatibility options with buggy browsers.
BrowserMatch "Mozilla/2"       nokeepalive
BrowserMatch "MSIE 4\\.0b2;"    nokeepalive downgrade-1.0 force-response-1.0
BrowserMatch "RealPlayer 4\\.0" force-response-1.0
BrowserMatch "Java/1\\.0"       force-response-1.0
BrowserMatch "JDK/1\\.0"        force-response-1.0
SetEnvIf User-Agent ".*MSIE.*" nokeepalive ssl-unclean-shutdown

SSLSessionCache shm:$vardir/ssl_gcache_data(524288)

# Logs and error reporting.
ErrorLog $logsdir/httpd_error_log
LogLevel warn

LogFormat "%h %l %u %t \\"%r\\" %>s %b \\"%{Referer}i\\" \\"%{User-Agent}i\\"" combined
LogFormat "%h %l %u %t \\"%r\\" %>s %b" common
LogFormat "%{Referer}i -> %U" referer
LogFormat "%{User-agent}i" agent

# HTTPS server.
Listen $porthttps
<VirtualHost _default_:$porthttps>
    DocumentRoot $htdocsdir
    SSLLogLevel warn
    SSLLog $logsdir/httpd_ssl_log
    CustomLog $logsdir/httpd_access_log combined
    CustomLog $logsdir/httpd_ssl_request_log \\
        "%t %h %{SSL_PROTOCOL}x %{SSL_CIPHER}x \\"%r\\" %b"

    # SSL definitions.
$ssldefinitions
</VirtualHost>

CONFIGFILE

	# Optional configuration of a plain-HTTP-enabled port
	do {
		my $crldir = $CONFIG{'CONFIG_CRLDIR'};
		my $documentroot = $crldir;
		$config .= << "CONFIGFILE";
# HTTP server.
Listen $porthttp
<VirtualHost _default_:$porthttp>
    DocumentRoot $documentroot
    CustomLog $logsdir/httpd_plain_HTTP_access_log combined
</VirtualHost>

ScriptAlias "/cgi-bin/" "$htdocsdir/scep/"
<Directory "$htdocsdir/scep/">
    AddHandler "perl-script" .exe
    Options ExecCGI
</Directory>

CONFIGFILE
	};
} else {		# Not standalone.
	$config .= << "CONFIGFILE";
# This file is meant to be included from your main Apache configuration
# file.  Please note that your web server needs to load the perl, php4 and
# ssl modules.

# You may want to change "_default_" hostname into a specific virtual
# hostname and:
# "Alias $basedir $htdocsdir"
# into:
# "DocumentRoot $htdocsdir".

# HTTPS server.
<VirtualHost _default_:$porthttps>
    Alias $basedir $htdocsdir

    # SSL definitions.
$ssldefinitions
</VirtualHost>

CONFIGFILE

	# Optional configuration of a plain-HTTP-enabled port
	do {
		my $crldir = $CONFIG{'CONFIG_CRLDIR'};
		my $documentroot = $crldir;
		$config .= << "CONFIGFILE";
# HTTP server.
<VirtualHost _default_:$porthttp>
    Alias $basedir $documentroot
</VirtualHost>

CONFIGFILE
	};
}


##############################################################################
# IDX-PKI specific.
##############################################################################
$config .= << "CONFIGFILE";
# PKI related types.
AddType application/x-pkcs7-crl          .crl
AddType application/x-pkcs7-certificates .p7b
AddType application/x-x509-ca-cert       .crt

# Do this early, even before anything Perl.
PerlTaintCheck On

<perl>
    # Funny enough, all global symbols defined in <perl> are turned
    # into configuration options, whether they are in the PerlThing
    # style or not! This means we could reconfigure all of Apache
    # from Perl (see
    # http://modperl.com:9000/book/chapters/ch8.html#Configuring_Apache_with_Perl).
    # Anyway this is not what we want (yet), and some of our modules do
    # export stuff so we change packages.
    package IDX::PKI::__NoWhere__;

    use strict;

    BEGIN {
        # Untaint environment. Those variables come from
        # Apache; even if they didn't, they would come from the root user.
        # No security problems here.

        my \%cleanenv;
        foreach my \$var (qw(GATEWAY_INTERFACE MOD_PERL)) {
           (\$cleanenv{\$var})=(\$ENV{\$var} =~ m/^(.*)\$/g);
        }
        \%ENV=\%cleanenv;
    }

    use lib "$perllibdir";
    use lib "$perllibdir/i386-linux";

    # Those are missing in a RedHat mod_perl, even though some RPM-packaged
    # Perl modules go there.
    use lib "/usr/lib/perl5/vendor_perl/5.6.1";
    use lib "/usr/lib/perl5/vendor_perl/5.6.1/i386-linux";

    # Those are optimization options.  If you enable these use()s, they
    # are loaded at server startup time; this saves time and memory thanks
    # to shared pages between fork()s.
    use IDX::PKI::Config;
    use IDX::PKI::Authentication;
    use IDX::PKI::Error;
    use IDX::PKI::FileManip;
    use IDX::PKI::Log;
    use IDX::PKI::Transport;
    use IDX::PKI::Principal;
    use IDX::PKI::Profile;
    use IDX::PKI::LDAP;
    use IDX::PKI::CA;
    use IDX::PKI::CertDB;
    use IDX::PKI::Registry;
    use IDX::PKI::Crypto::CSR;
    use IDX::PKI::Crypto::CRR;
    use IDX::PKI::Crypto::X509;
    use IDX::PKI::Crypto::PrivateKey;
    use IDX::PKI::Crypto::PKCS7;
    use IDX::PKI::Crypto::Tools;
    use IDX::PKI::Crypto::CRL;
    use IDX::PKI::Crypto::SignedBlob;
    use IDX::PKI::Message::Publish;

    # Perform a few integration tests and fail early in case something
    # goes wrong.
    # Don't use this code here.  We don't want to rely on the fact that
    # the LDAP server has been started _before_ Apache by the init
    # sequence.
    #my \$config = the IDX::PKI::Config ();
    #IDX::PKI::CertDB::CA->new("CA",
    #      (\$config->val ("_components", "ca") ? 1 : 0))->functest();

    # Fix an environment pollution issue here: Everybody (mod_ssl,
    # mod_perl) fiddles with %ENV, with quite nefarious consequences
    # (global per-process environment is altered, and this persists from
    # one request to the next - this can leak confidential data).
    # Dom 20020806: this should be dead code by now, but it doesn't hurt.
    package IDX::PKI::Registry;

    use strict;
    use Apache;

    use vars qw (%ENVORIG);

    %ENVORIG = %ENV;

    sub cleanupHandler {
        %ENV = %ENVORIG;
    }

    1;
</perl>

<Location $basedir>
    DirectoryIndex index.html index.cgi index.php

    AllowOverride None
    Options ExecCGI Indexes
    AddHandler "perl-script" .cgi
    Action "AccessDenied" $basepath/403.cgi
    AddHandler "AccessDenied" .inc

    ErrorDocument 403 $basepath/403.cgi

    PerlHandler IDX::PKI::Registry->handler
    # We need a cleanup handler to fix an environment pollution issue (see
    # <perl> section).
    PerlCleanupHandler IDX::PKI::Registry::cleanupHandler
</Location>

# Activate this if you want to be able to get server informations.
#<Location $basepath/debug>
#    SetHandler perl-script
#    PerlHandler Apache::Status
#</Location>
CONFIGFILE

print $config;
