#!/usr/bin/perl -w
#
#   start-stop-slapd     This script starts/stops an internal LDAP daemon in IDX-PKI.
# chkconfig: 2345 60 60
# description: Script to start/stop an IDX-PKI LDAP server, initializing /
#      refreshing the LDAP skeleton if needed.
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<start-stop-slapd> - IDX-PKI internal LDAP database management tool

=head1 SYNOPSIS

B<start-stop-slapd> I<command>

=head1 DESCRIPTION

This script encapsulates everything useful for an IDX-PKI internal LDAP
database: The configuration file, schema and LDIF skeleton are all bundled
in there.

=head1 COMMANDS

=over 4

=cut "

use strict;

use IDX::PKI::Config;
require IDX::PKI::SysV;

use MIME::Base64;
use Fatal qw(:void open close chown chmod mkdir chdir socket);
use IO::Pipe;
use Socket;
use Errno qw(ENOENT ECONNREFUSED EPERM EACCES);

our $config = the IDX::PKI::Config ();

our $mode;
$mode ||= "repository"; # This may be patched up at "make" time.

our $rootdn = ($mode eq "repository" || $mode eq "escrow") ?
	$config->getval ("dn", "idxpki_ldapbase") : "";

#### Rendez-vous constants
# Must be kept in sync with the build process suite (config.status, Makefiles)

our $dir = $config->getval ("paths", "vardir")."/ldap-$mode";
our $sock="$dir/sock";

our $certfile = $config->getval ("paths", "securedir").
	"/$mode-ldap/$mode-ldap.crt";
our $keyfile = $config->getval ("paths", "securedir").
	"/$mode-ldap/$mode-ldap.key";

our $serverport = $config->getval ("network", "port_ldap_".lc($mode));
$serverport ||= 389;

#### Internal constants
# May be fiddled with without a remorse.

our $pidfile="$dir/$mode-slapd.pid";
our $configfile="$dir/$mode-slapd.conf";
our $datadir="$dir/data";

our $procname; $procname="$mode-slapd on port $serverport";

our ($user,$group,$uid,$gid,$sockgroup,$sockgid);

my $testprivs = $config->val ("_components", "test_privs");
if (! $testprivs) {
	$user = $config->getval ("privileges", "user_ldap_".lc($mode));
	$uid = (($user =~ m/^[0-9]+$/) ? $user : getpwnam($user));
	$group = $config->getval ("privileges", "group_ldap_".lc($mode));
	$gid = (($group =~ m/^[0-9]+$/) ? $group : getgrnam($group));
	$sockgroup = $config->getval ("privileges", "group_main");
	$sockgid = (($sockgroup =~ m/^[0-9]+$/) ? $sockgroup :
				getgrnam($sockgroup));
}


########################## Code ####################################

=pod "

=item I<usage>

Prints a usage diagnostic.

=cut "

sub usage {
	require Pod::Usage;
	Pod::Usage::pod2usage(
		{-message=>"Usage: $0 [start|stop|status|init|refresh]\n",
		 -verbose=>2,
		 -exitval=>2});
}

=pod "

=item I<init>

This function gets called to initialize the contents of the database
(create directories, configuration files, sockets, etc). This must be done
while the LDAP server is offline. Also creates the root of the LDAP
hierarchy with I<_topLDIF()>.

=cut "

sub init {
	# Create directories
	mkdir($dir) || 1;
	chmod(0755,$dir);

	mkdir($datadir) || 1;
	chmod(0700,$datadir);

	# Create config file.
	open(CONFIG,">",$configfile);
	print CONFIG (_config($rootdn));
	print CONFIG ("");
	close(CONFIG);

	if (! -f "$datadir/id2entry.gdbm") {
		# We have to init the root DN using slapd, because
		# the bogus ACL mechanism in OpenLDAP won't let us
		# do that through the socket (short of a rootpw
		# that would open a security hole)
		my $pipe = new IO::Pipe;
		$pipe->writer($config->getval ("paths", "slapadd"), "-f", $configfile);
		$pipe->print(_topLDIF($rootdn));
		$pipe->close();
	}

	# Apply ownership if appropriate.
	chown ($uid, $gid, $dir, $datadir, $certfile, $keyfile,
		   glob ("$datadir/*")) unless ($testprivs);
}

=pod "

=item I<start>

Starts the LDAP server. I<init()> and/or I<refresh> will be called
automatically if appropriate.

=cut "

sub start {
	my ($mustrefresh, $firsttime);

	my @stat = stat ($configfile);
	if (! -f _) {	# Config file does not exist
		$firsttime = 1;
		$mustrefresh = 1;
	} else {
		my $configmtime = $stat[9];
		# Must refresh if CA files or this script itself were updated.
		# CA certificates.
		my $cadir = $config->getval ("paths", "cacertsdir");
		local *DIR;
		opendir (DIR, $cadir) || die "Can't read directory $cadir: $:";
		my @cahashs = map {(m/^([a-z0-9]+.\d+)$/i && -r "$cadir/$_") ?
							   ($1) : ()} readdir (DIR);
		closedir (DIR);
		# CRLs.
		my $crldir = $config->getval ("paths", "crldir");
		opendir (DIR, $crldir) || die "Can't read directory $crldir: $:";
		my @crlhashs = map {(m/^([a-z0-9]+.\d+)$/i && -r "$crldir/$_") ?
								($1) : ()} readdir (DIR);
		closedir (DIR);
		foreach my $file (@cahashs, @crlhashs, $0, $certfile, $keyfile,
						  $IDX::Config::PATH) {
			next unless (defined $file);
			@stat = stat ($file);
			next if (! -f _);
			if ($stat[9] > $configmtime) {
				$mustrefresh = 1;
				last;
			}
		}
	}

	if ($mustrefresh) {
		init() || return 0;
	}

	defined (my $pid = fork()) || die "Can't fork(): $!";
	if (!$pid) {
		# This stupid slapd cannot bind to a UNIX socket except if
		# it is in the current directory...
		chdir ($dir);

		my $logfile = $config->getval ("paths", "logsdir")."/$mode-ldap.log";
		open (STDOUT, ">", $logfile);
		open (STDERR, ">&STDOUT");

		my $ports = "ldapi://sock/";
		if (-f $certfile && -f $keyfile) {
			$ports .= " ldaps://0.0.0.0:$serverport/";
		}

		verboseexec ($config->getval ("paths", "slapd"), "-f",$configfile,
					 $testprivs ? () : ("-u", $user, "-g", $group),
					 "-h",$ports) || do {
						 require POSIX;
						 POSIX::_exit(42);
					 };
	}

	waitstatus(1) || return undef;

	return refresh($firsttime) if ($mustrefresh);

	1;
}

=pod "

=item I<refresh>

This function accepts a parameter. If set to some true value, then the
LDAP skeleton will be assumed to be missing (and therefore B<ldapadd> will
be used instead of B<ldapmodify>). Otherwise B<ldapmodify> is tried first,
then B<ldapadd> in case the LDAP skeleton is only half installed. This is
slow, but at least it works.

=cut "

sub refresh {
	my ($init) = @_;
	printf STDERR ("%s the IDX-PKI LDAP layout...\n",
				   ($init ? "Creating": "Updating"));

	my $status = 1;
	foreach my $contents (_populateLDIF($rootdn)) {
		_launch_ldapmodify ($contents, $init) || do {$status = 0};
	}

	return $status;
}

sub stop {
	my $pid = status();
	return 0 if (! $pid);
	kill INT => $pid;
}

# status checks for up-ness of the daemon and returns the pid
# if the thing is up.
sub status {
	# Test by pid

	my $pid = `cat '$pidfile' 2>/dev/null`;
	chomp $pid;
	return undef unless (($pid =~ m/^[0-9]+$/) && (kill 0 => $pid));
	my $pname = `ps www '$pid' 2>/dev/null`;
	chomp $pname;
	return undef unless ($pname =~ m/slapd/);

	# Test availability of the UNIX domain socket
	local *SOCK;
	$!=0;
	socket(SOCK, PF_UNIX, SOCK_STREAM, 0);
	connect(SOCK,pack_sockaddr_un($sock));
	if (! $!) {
		unless ($testprivs) {
			chown (-1, $sockgid, $sock);
			chmod (0770, $sock);
		}
		close(SOCK);
		return $pid;
	} else {
		close(SOCK);
		warn "Cannot connect to $sock: $!\n";
		return undef;
	}
}

###################### Config file templates ############################

=pod "

=back

=head1 SUPPORT FUNCTIONS

=over 4

=item I<_config ($rootdn)>

Returns an OpenLDAP configuration file.

=cut "

# Returns an LDAP configuration file, in full text. Takes no argument.
sub _config {
	my ($rootdn) = @_;
	my $schemadir = $config->getval ("paths", "schemadir");
	my $idxschema = $config->getval ("paths", "configdir").
		"/schema/idxpki.schema";

	my $conftext = << "CONFIGFILE";
# Configuration file for IDX-PKI $mode LDAP server.
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# See slapd.conf(5) for details on configuration options.

include		$schemadir/core.schema
include		$schemadir/cosine.schema
include		$schemadir/inetorgperson.schema
include		$schemadir/nis.schema
include		$schemadir/misc.schema
include		$schemadir/openldap.schema

include		$idxschema

pidfile		$pidfile
argsfile	$datadir/slapd.args

# LDBM database definitions.
database	ldbm
schemacheck	on
directory	$datadir

suffix		"$rootdn"

# Indices to accelerate search queries
# pres: match by presence
# sub:  match by substring
# eq: match by strict equality
# approx: match by approximation
#
# There seems to be a bug in OpenLDAP's indexation code:
# index   objectClass     pres,eq
index   cn              pres,eq,sub
index   uid             eq,sub

index   idxpkiSubjectID eq
index   idxpkiSerial    eq

CONFIGFILE

$conftext .= << "CONFIGFILE" if (-f $certfile);
# LDAPS configuration.
TLSCertificateFile    $certfile
TLSCertificateKeyFile $keyfile
# This is odd - when we say this, it seems that OpenLDAP sends the
# certificate down the wire instead of using it to authenticate clients!?
# TLSCACertificateFile \$cacert

CONFIGFILE

my $accessstring = "";
$conftext .= << "CONFIGFILE";
#####
# Access rights.
# Write access is restricted to the UNIX-domain socket. This way we do not
# need any passwords anymore.
# Keep this one first so that authorised access may change user passwords!
access to dn=*
	${accessstring}by sockurl.regex=^ldapi write
	by * read
	by anonymous read

access to dn=""
	${accessstring}by sockurl.regex=^ldapi write
	by * read
	by anonymous read

# This one is necessary to allow access to schemas.
access to dn=".*"
	by * read
	by anonymous read

# Everyone may authenticate (necessary if password authentication is in
# use!).
access to attr=userPassword
	by * auth
	by anonymous auth

CONFIGFILE

return $conftext;
}

=pod "

=item I<_launch_ldapmodify ($contents, $add)>

Launches B<ldapmodify> to update the I<$contents> LDIF. If the I<$add>
option is set, uses the B<-a> switch (add) rather than the default modify
action. If the modify fails, tries again with I<$add>.

=cut "

sub _launch_ldapmodify {
	my ($contents, $add) = @_;

	my $pipe = new IO::Pipe;

	defined (my $pid = fork()) || die "Can't fork()";
	if (! $pid) {
		$pipe->reader ();
		open (STDIN, "<&". $pipe->fileno ());
		# I guess ldapmodify shares bugs with slapd... Here we go with
		# chdir again.
		chdir ($dir);

		verboseexec ($config->getval ("paths", "ldapmodify"),
					 ($add ? ("-a") : ()), "-x", "-H", "ldapi://sock/") ||
						 do {
							 require POSIX;
							 POSIX::_exit(42);
						 };
	}
	$pipe->writer ();
	$pipe->print ($contents);
	$pipe->close ();
	waitpid ($pid, 0);
	if ($?) {
		# If $add was set, the error is propagated, otherwise retries with
		# $add.
		if ($add) {
			my $exitcode = $? >> 8;
			warn "ldapmodify failed with code $exitcode (see errors above)\n".
				"Used ldif was:\n$contents\n";
			return 0;
		} else {
			return _launch_ldapmodify ($contents, 1);
		}
	}
	return 1;
}

=pod "

=item I<_topLDIF ($rootdn)>

Returns an LDIF initial file, in full text, with only the top of the
database (the root DN). Takes the root DN as an argument.

=cut "

sub _topLDIF {
	my ($rootdn) = @_;
	return "" if ($rootdn eq "");
	$rootdn =~ m/^o=([^,]+)/i || die "Unexpected rootdn ($rootdn) is ".
		"not o=...,...";
	my $name = $1;
	return <<"LDIF";
dn: $rootdn
objectClass: organization
o: $name

LDIF
}

=pod "

=item I<_recursiveLDIF ($dn, $rootdn)>

Returns a LDIF complete enough to go from I<$rootdn> (excluded) up to
I<$dn> (included). All elements are C<Reserved for IDX-PKI internal use>.

=cut "

sub _recursiveLDIF {
	my ($dn, $rootdn) = @_;
	return () if ($dn eq $rootdn);

	my $patt = $rootdn ? ",$rootdn" : "";
	$dn =~ s/$patt$//i || die "Subdn does not match rootdn: $dn vs $rootdn";

	my @chunks = split (m/\s*[,=]\s*/, $dn);
	die "Odd number of elements ($dn)" if (scalar (@chunks) % 2);
	my @ldifs;
	my $rest = $rootdn ? ",$rootdn" : "";

	require IDX::PKI::DN;

	while (@chunks && (my ($key, $val) = splice (@chunks, -2))) {
		my $long = IDX::PKI::DN::type2class
			(IDX::PKI::DN::short_to_long ($key));
		die "Unknown DN part $key" unless ($long);
		push (@ldifs, << "LDIF");
dn: $key=$val$rest
objectclass: $long
$key: $val
description: Reserved for IDX-PKI internal use

LDIF
		$rest = ",$key=$val".$rest;
	}

	return @ldifs;
}

=pod "

=item I<_encodeCert ()>

Encodes a certificate (in PEM form, passed by filename) in Base64,
indenting all lines, so that the result is OK for putting inside an LDIF
file.

=cut "

sub _encodeCert {
	my ($filename) = @_;
	my $openssl = $config->getval ("paths", "openssl");
	my $text = encode_base64 (`$openssl x509 -outform der -in "$filename"`);
	$text =~ s/^/ /gm;
	chomp ($text);
	return $text;
}

=pod "

=item I<_encodeCRL ()>

Encodes a CRL (in DER form, passed by filename) in Base64, indenting all
lines, so that the result is OK for putting inside an LDIF file.

=cut "

sub _encodeCRL {
	my ($filename) = @_;
	local *FILE;
	open (FILE, "<", $filename) || die "Can't read CRL from $filename: $!";
	my $contents = join ("", <FILE>);
	close (FILE);
	my $text = encode_base64 ($contents);
	$text =~ s/^/ /gm;
	chomp ($text);
	return $text;
}

=pod "

=item I<_populateLDIF ($rootdn)>

Returns an array of LDIF chunks, in full text, with all but the root DN.
Takes the root DN as an argument. CA certs are automatically merged in (in
DER form) at the appropriate location.

=cut "

sub _populateLDIF {
	my ($rootdn) = @_;
	my @ldifs;

	if ($mode eq "repository")
	{
		# Repository only: Signed certificates.
		push (@ldifs, << "LDIFFILE");
dn: ou=certificate,$rootdn
objectclass: organizationalUnit
ou: certificate
description: Top of the CA certificate database

LDIFFILE
	}
	if ($mode eq "repository" || $mode eq "publishing")
	{
		# Repository and publication only: CA certificates and CRLs.
		my @CAs;
		require IDX::PKI::Crypto::X509;
		foreach my $x509 (loaddir IDX::PKI::Crypto::X509
						  ($config->getval ("paths", "cacertsdir"))) {
			push (@CAs, $x509);
		}
		my %crls;
		require IDX::PKI::Crypto::CRL;
		foreach my $crl (loaddir IDX::PKI::Crypto::CRL
						 ($config->getval ("paths", "crldir"))) {
			my $index = $crl->getSignerIndex (@CAs);
			next unless (defined $index);	# CRL from an unknown CA.
			$crls{$CAs[$index]->getSubjectDN ()} = $crl;
		}

		foreach my $x509 (@CAs) {
			my $cacn = $x509->getAttribute ("dn_subject")->getRDNValue ("CN");
			my $cadn = "cn=$cacn".($rootdn ne "" ? ",$rootdn" : "");

			# CA certificate.
			my $cert = encode_base64 ($x509->serialize ("DER"));
			$cert =~ s/^/ /gm;
			chomp ($cert);

			# Associated CRL, if any.
			my $crl;
			if ($crls{$x509->getSubjectDN ()}) {
				$crl = encode_base64 ($crls{$x509->getSubjectDN ()}
									  ->serialize ("DER"));
				$crl =~ s/^/ /gm;
				chomp ($crl);
			}

			# XXX TODO: No distinction is made between
			# authorityRevocationList and certificateRevocationList.

			push (@ldifs, << "LDIFFILE");
dn: $cadn
objectClass: top
objectClass: pkiCA
objectClass: cRLDistributionPoint
cn: $cacn
cACertificate;binary::$cert
${ $crl ? \"certificateRevocationList;binary::$crl" : \""}

LDIFFILE
		}
	}

	return @ldifs;
}

=pod "

=back

=cut "
