#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::Config> - Configuration files for IDEALX projects

=head1 SYNOPSIS

In normal code (e.g. outside unit tests) say:

 use IDX::MyApp::Config; # Not IDX::Config directly - see below.

 my $config = the IDX::MyApp::Config();

 # All calls in Config::IniFiles still work:
 my $singlevalue = $config->val ('Section', 'SingleParameter');
 my @multiplevalues = $config->val ('Section', 'MultiParameter');

 my %confighash;
 tie (%confighash, 'IDX::MyApp::Config');
 print $confighash{'Section'}{'SingleParameter'};

 # Locking is provided by this class:
 IDX::Config->updateLocked (sub {
 	my $config = shift;
 	$config->setval('Section', 'SingleParameter', 'foo');
 	$config->setval('Section', 'MultiParameter', 'bar',
 		'baz', 'quux');
 });

 reload() if ($config->timestamp() > $startuptime);

 # Typed indirection is supported. Imagine a configuration like
 # this:
 # [main]
 # somekey=somevalue
 # somekey=anothervalue
 # indirect=sub           ; with type "SUBSECTION"
 #
 # [SUBSECTION-sub]
 # you=canreachthis

 print $config->getPath('main', [SUBSECTION => 'indirect'],
 	'you');
 # Prints "canreachthis"

 # Or in case you just want the subsection name (here
 # "SUBSECTION-sub"):
 my $subsection = $config->getSectionName('main',
 	[SUBSECTION => 'indirect']);

In tests you can also say:

 IDX::MyApp::Config->set(<<"CONFIG");
 [section]
 key1=value1
 key2=value2
 CONFIG

=head1 DESCRIPTION

B<IDX::Config> is the configuration file abstraction for IDEALX projects,
overloaded atop B<Config::IniFiles>, from CPAN. The file format is
Windows-style F<.INI>. This class performs the following duties:

=over 4

=item *

Configuration file reading and writing, atomicity (but not lost-update
prevention), automatic reloading of configuration as needed;

=item *

Default value management and configuration bootstrap, that is,
incorporating build-time parameters as default values and arranging for
unit tests to see them even though the installation is not done yet. This
part needs customization for every application, through class inheritance.

=back

=head2 Subclassing is mandatory

Due to the application-specificity of bundling a default configuration
with the installed copy of B<IDX::*::Config> (see L</BOOTSTRAP PROCESS>),
B<this class should not be used directly>, except in pure
application-independent library code (typically the other B<IDX::*>
classes). Application-specific code should use an application-specific
subclass of B<IDX::Config>, which said application should provide.

Here is an example of a minimal B<IDX::MyApp::Config>:

 #!/usr/bin/perl -Tw

 package IDX::MyApp::Config;
 use strict;
 use base "IDX::Config";

 sub bootstrap_configpath {
 	my ($class, %opts) = @_;
 	return $opts{"CONFIGDIR"}."/myapp.conf";
 }

 sub bootstrap_config {
 	my ($class, %opts) = @_;

 	# Gets the IDX::Config default values from %opts.
 	my $config = $class->SUPER::bootstrap_config(%opts);

 	# Add just one option to the configuration file
 	$config->setval("network", "port_http", $opts{PORT_HTTP});
 	$config->SetParameterComment("network", "port_http",
 		"The port number the myappd listens on");

 	return $config;
 }

 1;

=head1 REFERENCE MANUAL

=head2 Global variables

=over 4

=item I<$PATH>

The path to the configuration file implicitly read through this module. In
order to modify it, e.g. according to some command-line switch, one should
do something like:

 BEGIN {
 	$IDX::Config::PATH = "/some/where";
 }

preferably in a BEGIN block, before doing any "use" of IDEALX-specific
code (in case some hasty module reads the configuration at init time). The
default value for this variable is set at install time by modifying the
source (see L</bootstrap_module>).

=item I<$APPCLASS>

The name of the base class for B<IDX::Config> objects. This
application-specific constant must be set up at run time by the toplevel
code (e.g. autonomous Perl scripts and mod_perl initialization code).
Simply doing a C<use IDX::MyApp::Config> at top-level is enough, see
L</import()>.

=back

=cut "

package IDX::Config;

use strict;

use base 'Config::IniFiles';

use IDX::PKI::Error;
use IO::Scalar;
use Fcntl ':flock';

our $PATH;
our $APPCLASS;

=pod "

=head2 Methods and Functions

=over 4

=item I<the()>

This constructor returns "the" configuration object for the current
application, as in:

 my $config = the IDX::MyApp::Config();

Returns the application's configuration object, made up of a stack of the
compile-time default configuration variables and the real configuration
file (computed from I<configFile()>). See L<Config::IniFiles> for the
semantics of stacking two configuration objects.

As the name of the constructor implies, this object is a singleton. This
means that calling I<the()> will not generally reload the configuration,
except if its contents is different from the on-disk file. This happens if
it is I<dirty()> (which see), or if the timestamp of the configuration
file changed.

The following form is also valid:

 my $config = the IDX::Config();

The resulting object will not typically be of class B<IDX::Config>, but of
some descendant class (see L</BOOTSTRAP PROCESS>), namely the class
pointed to by I<$IDX::Config::APPCLASS> (see L</Global variables>). This
form is useful for application-independent library code that wants to read
the configuration file (e.g. B<IDX::Log>).

If you want to modify the construction mechanism in the
application-specific subclass, rather overload I<fresh()> below: I<the()>
calls I<fresh()> as needed.

=item I<fresh()>

Exactly the same as I<the()>, except for the singleton behavior: A new
object is created and its contents are loaded from the configuration file
every time this object is used. Use of this method in user code is not
recommended; it is intended for the implementation of I<the()> and
I<updateLocked()> only.

This constructor may be overloaded in subclasses, and the object/singleton
voodoo will still do what you want when user code calls C<the
IDX::Config>.

=cut "

sub fresh {
	my $class=shift;

	if (($class eq "IDX::Config") && $APPCLASS && ($APPCLASS ne $class)) {
		my $fresher=UNIVERSAL::can($APPCLASS,"fresh");
		throw IDX::PKI::Error::Internal("Wrong APPCLASS in Config.pm",
				-details=>"$APPCLASS has no method named \"fresh\"") unless
				  (defined $fresher);
		return &$fresher($APPCLASS,@_);
	};

	my $defaultconfig=$class->_default() if
	  UNIVERSAL::can($class,"_default");

	my $self;

	do {
		$self=$class->Config::IniFiles::new
		  (
		   ( ($PATH && -f $PATH && -s _)? (-file=>$PATH) : ()),
		   ( (defined $defaultconfig) ? (-import=>$defaultconfig) : ()));
	};

	if (!defined $self) {
		throw IDX::PKI::Error::IO("Cannot load config file",-IOfile=>$PATH,
							-details=>\@Config::IniFiles::errors) if
		  (defined $PATH);
		throw IDX::PKI::Error::Config("Cannot initialize default config");
	};

	delete $self->{IDX_dirty}; # Constructor uses write accessor methods,
	# overloaded by us.

	if (defined $defaultconfig) {
		$defaultconfig->{cf}='';
		$defaultconfig->{imported}=[];
	};

	$self->{IDX_defaultconfig}=$defaultconfig if defined $defaultconfig;
	$self->{IDX_loadtime}=time();

	# Some encapsulation violation because of some braindamage in
	# Config::IniFiles : we do not want $defaultconfig to be reloaded
	# ever, and by default $self->{cf} is inherited from $defaultconfig->{cf}
	# at construction time
	$self->{cf} = $PATH;
	$self->{imported}=[];

	return $self;
}

my $singleton;

sub the {
	my $class=shift;

	# Special case in tests: one can set an arbitrary config object.
	if ($singleton && $singleton->{IDX_testing}) {
		return $singleton;
	};

	if (defined $PATH) {
		my @stat=stat $PATH;
		if (
			(defined $singleton) && (! $singleton->{IDX_dirty}) &&
			( (! -f _) || ($stat[9] <= $singleton->{IDX_loadtime}) )
		   ) {
			return $singleton;
		};
	} else {
		# If $PATH is not defined, we are in the not-installed-yet phase,
		# which is kinda special. I did not quite make up my mind yet,
		# let us just do something sensible.
		return $singleton if defined $singleton;
	};

	return ($singleton=$class->fresh(@_));
}

=pod "

=item I<set($text)>

=item I<set($configobject)>

This class method is only useful for tests. It is typically used like
this:

 IDX::Config->set(<<"TESTCONFIG");
 [main]
 somekey=somevalue
 somekey=anothervalue
 TESTCONFIG

In order to protect the innocent, this method will fail horribly if the
top level frame of the call stack is not a test suite, as determined
through I<IDX::Test::inUnitTest ()> (see L<IDX::Test>).

=cut "


sub set {
	require IDX::Test;

	die "Cannot IDX::Config->set() outside a test suite"
		unless (IDX::Test::inUnitTest ());

	my ($class,$text_or_obj)=@_;

	if (UNIVERSAL::isa($text_or_obj,"IDX::Config")) {
		$singleton=$text_or_obj;
	} else {
		# Bug in Config::IniFiles: an empty config file wreaks havoc
		# everywhere.  Still, we need to test translucency of the default
		# config and have an existing config file at the same time.
		$text_or_obj ||= "[_empty]\n";

		# I don't know why the hell perl bugs the "new
		# IDX::Test::Filename()" syntax in the "set()" test when you
		# launch "perl -MIDX::MyApp::Config IDX/Config.pm"... Anyway this
		# one works.

		my $file = IDX::Test::Filename->new ();

		local *FILE;
		open(FILE,">",$file) || die "Cannot open: $!\n";
		print FILE $text_or_obj || die "Cannot print: $!\n";
		close(FILE) || die "Cannot close: $!\n";

		$singleton=($APPCLASS || $class)
		  ->Config::IniFiles::new(-file=>"$file");

		die "Cannot load $file. Errors were:\n".
			join ("\n", @Config::IniFiles::errors) if (! $singleton);
	};

	$singleton->{IDX_testing}=1;
	$singleton->{IDX_loadtime}=1; # a very old config indeed

	return $singleton;
}

=pod "

=item I<import()>

B<IDX::Config> exports nothing, but I<use()>ing an inherited class causes
I<$APPCLASS> to be set to the name of said class. The following idiom is
also a valid way of saying that B<IDX::MyApp::Config> is to be the
configuration class:

 IDX::Config::import("IDX::MyApp::Config");

although I can think of no circumstance where it would be useful, except
in the test suite of this module.

=cut "

sub import {
	my ($class)=@_;
	$APPCLASS||=$class if ($class ne 'IDX::Config');
}

=pod "

=item I<is_bootstrapped()>

This function (not a method or class method) returns true if the
configuration is bootstrapped already.

=cut "

sub is_bootstrapped {
	return ((defined($APPCLASS) && UNIVERSAL::can($APPCLASS,"_default")) ||
			(UNIVERSAL::can ("IDX::Config", "_default")));
}


=pod "

=item I<_default()>

Class method; returns the default configuration, as a B<Config::IniFiles>
object. This method has no implementation in B<IDX::Config>, and should
not be defined in B<IDX::MyApp::Config>; instead, the bootstrap process
writes code for this class method (see L</BOOTSTRAP PROCESS>).

=item I<setval ()>

Overloaded from B<Config::IniFiles> because I<Config::IniFiles::setval()>
doesn't allow seting a new entry (you must use I<newval> instead), which
just uses to make your code bigger and less readable.

=cut "

sub setval {
	my ($self, $section, $entry, @vals) = @_;

	$self->{"IDX_dirty"} ++;

	# Check if the parameter exists.
	my $ret = $self->val ($section, $entry);
	return defined ($ret) ?
		$self->Config::IniFiles::setval ($section, $entry, @vals) :
		$self->newval ($section, $entry, @vals);
}

=pod "

=item I<val ()>

Overloaded from B<Config::IniFiles> because I<Config::IniFiles::val()> in
list context returns C<(undef)> instead of the expected C<()> on
nonexistent entries.

=cut "

sub val
{
	my ($self, @args) = @_;
	return (wantarray &&
			! defined (scalar ($self->Config::IniFiles::val (@args)))) ?
			() : $self->Config::IniFiles::val (@args);
}

=pod "

=item I<getval ($section, $entry)>

Does the same as I<Config::IniFiles::val()> except that it throws an
B<IDX::PKI::Error::Config> error if the entry does not exist.

=cut "

sub getval ($$$)
{
	my ($self, $section, $entry) = @_;
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (defined $section && defined $entry);
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		if (ref ($section) || ref ($entry));
	my $ret = $self->val ($section, $entry);
	# Warning: We may wantarray() so we redo ->val here with proper context.
	defined ($ret) ? return $self->val ($section, $entry) :
		throw IDX::PKI::Error::Config ("Missing configuration option",
									   "-section" => $section,
									   "-parameter" => $entry);
}

=pod "

=item I<< getPath ($topsection, [$type1 => $indirect1], ..., $entry) >>

Given a path with possible indirections, returns a configuration value
performing a typed cross-section walk through the configuration file.
Things here get much clearer with an example (see L</SYNOPSIS>).

Expected arguments are:

=over 4

=item I<$topsection>

The name of a section in the configuration file, from which the search
begins.

=item Middle arguments

A list of anonymous pairs of the form C<[$type, $indirect]>, where
I<$type> is the prefix of a section name, and the suffix is computed as
the value for configuration directive I<$indirect> in the current section.
Prefix and suffix are joined with C<->, and the result is the new current
section name, whence the search continues on next hop.

=item I<$entry>

The final configuration directive name to look up, in the current section
at the end of the search. I<$entry> may be single- or multi-valued, and
the return value will depend on calling context exactly like in
I<Config::IniFiles::val()>.

=back

=cut "

sub getPath {
	throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS") if (scalar (@_) < 3);
	my ($self, $section, @paths) = @_;
	my $entry = pop (@paths);
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (defined $section && $section ne "" && defined $entry);

	foreach my $path (@paths)
	{
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
			unless (ref ($path) eq "ARRAY" && scalar (@$path) == 2 &&
					defined ($path->[0]) && defined ($path->[1]));
		$section = $path->[0] . "-" . $self->getval ($section, $path->[1]);
	}

	return $self->val ($section, $entry);
}


=pod "

=item I<< getSectionName ($topsection, [$type1 => $indirect1], ...) >>

Does the same search as I<getPath()> above, but stops at computing the
last current section name and returns it. Calling context does not matter.

=cut "

sub getSectionName {
	throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS") if (scalar (@_) < 2);
	my ($self, $section, @paths) = @_;
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (defined $section && $section ne "");

	foreach my $path (@paths)
	{
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
			unless (ref ($path) eq "ARRAY" && scalar (@$path) == 2 &&
					defined ($path->[0]) && defined ($path->[1]));
		$section = $path->[0] . "-" . $self->getval ($section, $path->[1]);
	}

	return $section;
}

=pod "

=item I<RewriteConfig ()>

Overloaded from B<Config::IniFiles> in order to nicely throw rather than
returning C<undef> if the current configuration has no associated file.

=cut "

sub RewriteConfig {
	my ($self) = @_;
	throw IDX::PKI::Error::Config ("CONFIG_SAVING", "-details" =>
								   "No defined configuration file")
		unless (defined ($self->{"cf"}) && $self->{"cf"} ne "");
	return $self->WriteConfig ($self->{"cf"});
}

=pod "

=item I<WriteConfig($configfile)>

This method is overloaded from B<Config::IniFiles> in order to ensure
consistency of the configuration file updates: It refuses to overwrite the
original file, except from inside an I<updateLocked()> block, and it
resets the dirty flag. This also has an effect on the I<RewriteConfig()>
method, since it is implemented in terms of I<WriteConfig()>.

To avoid unauthorised people to know private data (such as LDAP
passwords), the configuration file has a 0640 mode.

=cut "

sub WriteConfig {
	my $self=shift;
	my ($file)=@_;
	if ( ($self->{IDX_loadtime} == time()) && !$self->{IDX_testing} ) {
		sleep 1; # Make sure the timestamp *will* change
	};

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") if (!defined $file);

	if ((my @stat=stat($file)) && $self->{cf}) {
		my @statcf=stat($self->{cf});
		if (
			(! $self->{IDX_lockfd}) &&
			($stat[0] eq $statcf[0]) &&
			($stat[1] eq $statcf[1])
		   ) {
			throw IDX::PKI::Error::Config
			  ("Cannot overwrite main config file outside coherent block");
		};
	};

	throw IDX::PKI::Error::IO ("Cannot write", "-IOfile" => $file)
		unless ($self->Config::IniFiles::WriteConfig(@_));

	chmod (0640, $file);

	delete $self->{IDX_dirty};
	$self->{IDX_loadtime}=time();
}

=pod "

=item I<timestamp()>

Returns the last modify date of the configuration file, as given by
I<stat()>. It is guaranteed that the timestamp will change each time the
configuration file needs to be reloaded; storing C<< $config->timestamp()
>> along with persistent data and resources constructed from configuration
options is therefore a correct way of caching them.

=cut "

sub timestamp
{
	my ($self) = @_;
	return (stat ($self->{"cf"}))[9];
}

=pod "

=item I<< All other writer functions in B<Config::IniFiles> >>

Those functions are overloaded from B<Config::IniFiles> in order to ensure
consistency of the configuration file updates: Each time they are called,
they set the internal dirty flag of the object (see L</dirty>). The rest
of their semantics is identical.

B<WARNING:> You can perform updates from outside an I<updateLocked()>
block, but you will not be able to overwrite the main config file (on the
other hand, you can save to another file). This is a precaution for data
consistency (see L</updateLocked>). Using I<rename()>, it is trivial to
overcome it, so don't do that.

=cut "

foreach my $writerfunc (qw(newval delval
				   AddSection DeleteSection
				   DeleteSectionComment DeleteParameterComment
				   SetParameterEOT DeleteParameterEOT
							Delete)) {
	no strict "refs";
	*{"IDX::Config::$writerfunc"}=sub {
		my ($self)=@_;
		$self->{IDX_dirty}++;
		goto &{"Config::IniFiles::$writerfunc"};
	};
};

# We also want some of the functions to throw whenever the original
# returns undef.

sub SetFileName {
	my $self=shift;
	my $result=$self->Config::IniFiles::SetFileName(@_);
	unless (defined $result) {
		my ($filename)=@_;
		throw IDX::PKI::Error::Config
		  ("Cannot access configuration file",-IOfile=>$filename);
	};
	$self->{IDX_dirty}++;
	return $result;
}

# Original SetParameterComment behaves badly if the comment contains
# newlines.

foreach my $writerfunc (qw(SetParameterComment SetSectionComment)) {
	no strict "refs";
	*{"IDX::Config::$writerfunc"} = sub {
		my ($self, @comment) = @_;
		$self->{"IDX_dirty"}++;
		&{"Config::IniFiles::$writerfunc"} ($self,
											map {(split /\n/)} @comment);
	};
}

=pod "

=item I<dirty()>

This method returns true if the object is "dirty" in the cache coherence
sense, that is, there was at least one modification performed onto it
since the last writing to disk. Changing the file name with
L<Config::IniFiles/SetFileName> also counts as making the object dirty.

=cut "

sub dirty { return shift->{IDX_dirty}; }

=pod "

=item I<updateLocked($sub)>

This is a class method. I<$sub> will be executed with one argument, a
I<fresh()> object of the class; after I<$sub> finishes (its return value
is ignored), his I<write()> method will be called. During execution of the
subroutine, an exclusive (writer) lock will be held over the configuration
file, thus preventing concurrent processes from entering the
I<updateLocked()> code.

Nested I<updateLocked()> blocks are supported, which is quite cool for
both data coherence and performance considerations. In this case, only the
most exterior one will lock, write and then unlock the file.

B<WARNING:> These precautions for coherence are moot if user code uses
values in I<$sub> that it fetched from another B<IDX::Config> object (such
as I<the> main one), because said object may be already obsolete by the
time I<updateLocked()> acquires the lock. I<$sub> must re-read everything
it needs. This can be hard to ensure (e.g. many useful local variables
will be computed from the configuration somehow).

=cut "

our $_writeableobj;

sub updateLocked (&) {
	my ($class,$sub)=@_;

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") unless
		(ref($sub) eq "CODE");

	if (defined $_writeableobj) { # Nested transaction
		&$sub($_writeableobj);
		return;
	}

	my $config = (($singleton && $singleton->{IDX_testing}) ?
				  $singleton : $class->fresh());
	my $configfile = $config->{"cf"};
	throw IDX::PKI::Error::Config ("CONFIG_SAVING", "-details" =>
								   "No defined configuration file")
		unless (defined ($configfile) && $configfile ne "");

	# If the file doesn't already exist, create it, but take care of not
	# overwriting it if a concurrent process is also writing it (which is
	# not very likely, but could happen).
	if (! -f $configfile) {
		local *FILE;
		open (FILE, ">>", $configfile);
		close FILE;
	}

	# LOCK
	require IO::File;
	my $fd = new IO::File ($configfile, "r") || throw IDX::PKI::Error::IO
		("CONFIG_SAVING", "-IOfile" => $configfile,
		 "-details" => "Can't create a new IO::File");
	my $locked;
	for (my $i = 0; $i <= 5; $i ++) {
		flock ($fd, LOCK_EX | LOCK_NB) && do {$locked = 1; last};
		sleep 1;
	}
	throw IDX::PKI::Error::IO
		("CONFIG_SAVING", "-IOfile" => $configfile,
		 "-details" => "Config file is already locked") unless ($locked);

	my $errors;

	# Avoids lost updates: Freshs the configuration with respect to the
	# configuration file.
	local $_writeableobj = (($singleton && $singleton->{IDX_testing}) ?
							$singleton : $class->fresh());
	$_writeableobj->{IDX_lockfd}=1;

	try {
		&$sub($_writeableobj);
		$_writeableobj->RewriteConfig();
	} finally {
		# UNLOCK
		flock ($fd, LOCK_UN);
		delete $_writeableobj->{IDX_lockfd};
	};
}

=pod "

=item I<_make_filehandle($thing)>

Overloaded from base class in order to refine its semantics. If I<$thing>
is a reference to a scalar, then create an B<IO::Scalar> over I<$thing>
and use this to load the configuration from. The expected usage is in the
bootstrap, where the configuration will be passed on in full text
(obviously this will only work if I<$self> is blessed in the
B<IDX::Config> class).

=cut "

sub _make_filehandle {
	my ($self,$thing)=@_;
	if ($thing && ref($thing) eq "SCALAR") {
		return new IO::Scalar($thing);
	} else {
		return $self->Config::IniFiles::_make_filehandle($thing);
	};

}

=pod "

=item I<ReadConfig()>

Overloaded from the base class in order to shut off warnings that appear
because B<Config::IniFiles> tries to I<stat()> the B<IO::Scalar>.

=cut "

sub ReadConfig {
	my $self=shift;
	local $SIG{__WARN__}=sub {};
	return $self->Config::IniFiles::ReadConfig(@_);
}


=pod "

=item I<DESTROY ()>

Removes any pending lock, in case something went wrong (this situation
shouldn't happen because B<IDX::Config> objects generally stay a very long
time in scope, but it doesn't hurt).

=cut "

sub DESTROY
{
	my ($self) = @_;

	# Only if the config is supposed to be locked and there is a config
	# file.
	if ($self->{"IDX_lockfd"} && $self->{"cf"}) {
		require IO::File;
		my $fd = new IO::File ($self->{"cf"}, "r");
		flock ($fd, LOCK_UN) if ($fd);
	}
}


################### BOOTSTRAP STUFF #################################

=pod "

=back

=head1 BOOTSTRAP PROCESS

The bootstrap process for this module is quite hackish, but it does
provide a very interesting benefit: Everything can be configured from the
configuration file, yet IDEALX applications provide sensible default
values for most of the options, and so a useful configuration file can be
kept very short.

=over 4

=item 1.

The user launches the F<configure.pl> script, with appropriate
command-line options, and/or answers interactive questions. The result is
a file named F<config.status> at the top of the directory, in Makefile
format. Undefined values are done like in the Linux kernel configuration
file:

 CONFIG_NAME=unquoted value
 # CONFIG_OTHERNAME is not set

=item 2.

The I<bootstrap_module()> is called in the app-specific installation
module. Its task is to "cook" the B<IDX::MyApp::Config> from its source
and the F<config.status>. Here is an appropriate Makefile entry for this
job:

 $(MYAPP_BUILDPERLLIBDIR)/IDX/MyApp/Config.pm: \
 IDX/MyApp/Config.pm
 	install -d $(dir $@)
 	perl -I. -MIDX::MyApp::Config -e \
 		"print(bootstrap_module \
 		IDX::MyApp::Config('../config.status', \
 		'$<'))" > $@

=item 3.

Optionnally, the same code can be used to generate a commented
configuration file template, ready to be edited by the systems
administrator:

 $(MYAPP_CONFIGDIR)/myapp.conf.sample: \
 IDX/MyApp/Config.pm
 	install -d $(dir $@)
 	perl -I. -MIDX::MyApp::Config -e \
 		"print(bootstrap_configfile \
 		IDX::MyApp::Config('../config.status'))" \
 		> $@

The same file could just as well be named F<myapp.conf>, since it contains
exactly the default configuration. This would only incur a small
performance penalty at startup time (e.g. the time spent parsing the
configuration file).

=back

=head2 Bootstrap class methods

=over 4

=item I<bootstrap_readconfigstatus($configstatusfile)>

This class method parses I<$configstatusfile> and returns a hash of C<<
key => value >> pairs (some values may be C<undef>, because the C<#
CONFIG_FOO is not set> form is recognized). The C<CONFIG_> prefixes are
suppressed. In scalar context, returns a hash reference instead, with same
contents.

=cut "

sub bootstrap_readconfigstatus
{
	my ($class,$config_status) = @_;
	my $configopts={};

	local *CONFIG_STATUS;
	open (CONFIG_STATUS, "<",$config_status) ||
		die "Can't open $config_status";
	while (my $line = <CONFIG_STATUS>)
	{
		if ($line =~ m/^\#\s*CONFIG_(.*) is not set$/)
		{
			$configopts->{$1} = undef;
			next;
		} elsif ( ($line =~ m/^\#/) || ($line !~ m/\S/) ) {
			next;
		} elsif ($line =~ m/^\s*CONFIG_(.*?)\s*=\s*(.*)\s*$/) {
			$configopts->{$1}=$2;
		} else {
			die "bogus line in $config_status:\n$line\n";
		}
	}
	close (CONFIG_STATUS);
	return (wantarray ? %$configopts : $configopts);
}

=pod "

=item I<bootstrap_configpath(%configstatus)>

This class method returns the path to the default configuration file. It
has no implementation in B<IDX::Config> and should be defined in
subclasses. I<%configstatus> is an associative array of the configuration
options from F<config.status>, as returned by
I<bootstrap_readconfigstatus()>.

This function will only be used at bootstrap time, and its result will be
turned into a piece of code that sets I<$IDX::Config::PATH>. Applications
wishing to change the configuration file at run-time (for example, because
of a command line switch) should modify this variable in a BEGIN block.

=item I<bootstrap_config(%configstatus)>

This class method returns an B<IDX::MyApp::Config> object with an
appropriate default content, according to the compile-time values in
I<%configstatus> which is an associative array of the configuration
options from F<config.status>, as returned by
I<bootstrap_readconfigstatus()>. This method is very likely to need
overloading in the per-application code.

By default, I<bootstrap_config()> does the following:

=over 4

=item *

Translate all C<WITH_*> (without the C<WITH_> prefix), C<*DIR> and
C<*PREFIX*> options into section C<[paths]>;

=item *

Translate all C<*PORT*> and C<*HOST*> options into section C<[network]>;

=item *

Translate all C<PRIVS_*> (without the C<PRIVS_> prefix) options into
section C<[privileges]>;

=item *

Translate all C<INSTALL_*> (without the C<INSTALL_> prefix), C<USE_*>,
C<TEST*> and C<TARGET_DISTRIBUTION> options into section C<[_components]>
(the leading underscore means that the systems administrator should
refrain from modifying them, for obvious reasons).

=back

=cut "

sub bootstrap_config
{
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS") unless
	  (@_ % 2);
	my ($class,%confighash) = @_;

	# We want to create a clean empty new object.
	my $config = fresh IDX::Config ();
	while (my ($option, $val) = each %confighash)
	{
		# We don't want to let "undef" values because it is our way to
		# recognise inexistant keys.
		$val = "" unless (defined $val);
		if ($option eq "BUILDDIR") {
			# Isolate this one, since it looses its meaning after the
			# unit integration tests are done.
			$config->newval ("_tests", "builddir", $val);
		} elsif (($option =~ m/^WITH_(.*)$/) || ($option =~ m/^(.*DIR)$/) ||
			($option =~ m/^(.*PREFIX.*)$/))
		{
			$config->newval ("paths", lc ($1), $val);
		}
		elsif (($option =~ m/^(.*PORT.*)$/) || ($option =~ m/^(.*HOST.*)$/))
		{
			$config->newval ("network", lc ($1), $val);
		}
		elsif (($option =~ m/^(TEST.*)$/) || ($option =~ m/^(USE_.*)$/) ||
			   ($option =~ m/^INSTALL_(.*)/) ||
			   ($option =~ m/^(TARGET_DISTRIBUTION)$/))
		{
			$config->newval ("_components", lc ($1), $val);
		}
		elsif ($option =~ m/^PRIVS_(.*)$/)
		{
			$config->newval ("privileges", lc ($1), $val);
		}
	}

	return $config;
}

=pod "

=item I<bootstrap_configfile($configstatus)>

This class method returns the contents of the default configuration file.
In the base class, returns the same contents as I<<
$config->OutputConfig() >> would I<print()>, where I<$config> is
calculated with I<bootstrap_config()>. I<$configstatus> is the filename of
the C<config.status> file. This is only used by the Makefile and should
probably not be overloaded, lest the config file sample be different from
what is really in the default configuration.

=cut "

sub bootstrap_configfile
{
	my ($class,$configstatus)=@_;
	my $config = $class->bootstrap_config
	  ($class->bootstrap_readconfigstatus($configstatus));

	my $contents = "";

	# Grrr, OutputConfig() can only write to a filehandle!!
	my $sh = new IO::Scalar (\$contents);
	my $oldfh = select ($sh);
	$config->OutputConfig ();
	close ($sh);
	select ($oldfh);

	return $contents;
}

=pod "

=item I<bootstrap_module ($configstatus, $me)>

Returns a patched version of the source code of the class' implemenation,
with all I<bootstrap_*> methods removed (still TODO), and code added to
initialize I<$IDX::Config::PATH>, I<$IDX::Config::APPCLASS> and a
I<_default()> method (which see). The default configuration is taken from
I<bootstrap_configfile()>.

The base B<IDX::Config> class is an exception: I<$IDX::Config::PATH> and
I<$IDX::Config::APPCLASS> remain unset, and the default configuration set
by I<_default()> it is empty.

=cut "

# For code removal we could use the same kind of idiom than Class:Contract:

# my %no_opt;	# NOT IN PRODUCTION
#
# sub myfunc {
# # NOT IN PRODUCTION...
#
# 	# Some stuff goes there...
#
# # ...NOT IN PRODUCTION
# }

sub bootstrap_module
{
	my ($class,$configstatus, $me) = @_;

	my %confighash = $class->bootstrap_readconfigstatus ($configstatus);

	open (SOURCE, "<", $me) || die "Can't read module source $me: $!";
	my $contents;
	my $done = 0;
	while (defined (my $line = <SOURCE>))
	{
		if ((! $done) && ($line =~ m/^\s*[^\s\#]/))
		{
			# No $IDX::Config::PATH in base IDX::Config.
			my $quotedpath = UNIVERSAL::can ($class, "bootstrap_configpath") ?
				quotemeta ($class->bootstrap_configpath (%confighash)) :
				undef;
			my $pathline = defined ($quotedpath) ?
				"\$IDX::Config::PATH = \"$quotedpath\"" : "1";
			# Emtpy default configuration file in base IDX::Config.
			my $defaultconf = $IDX::Config::APPCLASS ?
				$class->bootstrap_configfile ($configstatus) : "";
			# Don't define $IDX::Config::APPCLASS in base IDX::Config.
			my ($appclass, $appline) = $IDX::Config::APPCLASS ?
				($IDX::Config::APPCLASS,
				 "\$IDX::Config::APPCLASS = $IDX::Config::APPCLASS") :
				 ("IDX::Config", "1");
			$contents .= << "EOSTRING";
#####  STUFF BELOW WAS AUTOMATICALLY ADDED AT BOOTSTRAP TIME  #####


BEGIN
{
	$pathline;
	$appline;
}

sub ${appclass}::_default
{
	my \$contents = << 'ALLYOURCONFAREBELONGTOUS';
$defaultconf
ALLYOURCONFAREBELONGTOUS

	my \$config = IDX::Config->Config::IniFiles::new
		((\$contents =~ m/\\S/) ? ('-file' => \\\$contents) : ())
		|| throw IDX::PKI::Error::Config ("Can't create default config",
			"-details" => join ("\n", @Config::IniFiles::errors));
	return \$config;

}

#####  END OF BOOTSTRAP TIME STUFF  #####

EOSTRING
			$done = 1;
		}
		$contents .= $line;
	}
	close (SOURCE);
	throw IDX::PKI::Error::Internal ("No uncommented line in $me?!")
		unless ($done);

	return $contents;
}

=pod "

=back

=head1 BUGS / TODO

All I<bootstrap_> functions are dead code after installation is done, and
should be removed including in the B<IDX::Config> main class itself.
This just requires being a little smarter in I<bootstrap_module()>,
and adding another special make rule for F<build/perllib/IDX/Config.pm>.

=cut "

################### COMPATIBILITY STUFF #################################
# With the following code, IDX::PKI::Config-compatibility is provided.

sub new { goto &the;};


#################### END COMPATIBILITY STUFF ############################

require IDX::Test::Suite unless caller();
1;
__END__

##############################  TEST CASE  ##############################

use IDX::PKI::Error;

# Cleans up environment pollution because various applications may want to
# use their specific IDX::MyApp::Config when testing their modules
# (including us). And if their module is properly bootstrapped, it sets
# those.

undef $IDX::Config::PATH;
undef $IDX::Config::APPCLASS;
undef *IDX::Config::_default;

%ENV = ("PATH" => "/usr/bin:/bin");

#################################################################### INIT
# Writes the content of the here document section in a temporary file
#   and removes that file upon completion of the script
my $configdata= <<'DATA';
; first digit is the section number,
; second digit is the key number,
; third digit is the value number.
[sect1]
key11=val111
key11=val112
key12=val121
key13=<<MULTILINE
val131
val132
val133
MULTILINE

[sect2]
; empty section

[sect3]
key31=val311

[sect4]
key41=<<MULTILINE
val411
val412
MULTILINE

[sect5]
; Empty value.
key51=

[INDIRECTION1-val121]
keyI1=valI1

[INDIRECTION2-valI1]
keyI2=valI2
DATA

my $origconfigfile = new IDX::Test::Filename();
open(CONFIGFILE, ">",$origconfigfile) ||
  die "Can't open CONFIGFILE $origconfigfile for writing";
print CONFIGFILE $configdata;
close CONFIGFILE;

# This is an ersatz for a bootstrapped subclass:

@MyBogusApp::Config::ISA=qw(IDX::Config);
sub MyBogusApp::Config::_default {
	return ( IDX::Config->Config::IniFiles::new(-file=>"$origconfigfile") or
		   die "Cannot load _default config file");
}

my $configfile = new IDX::Test::Filename();
$IDX::Config::PATH="$configfile";

# Bug in Config::IniFiles: an empty config file wreaks havoc
# everywhere.  Still, we need to test translucency of the default
# config and have an existing config file at the same time.
open(TOUCH,">",$configfile); print TOUCH "[empty]\n"; close(TOUCH);

####################################################################


sub sameobj {
	# Testing for object identity in Perl is a real PITA...
	my ($obj1,$obj2)=@_;
	$obj1->{".touch"}++;
	return (defined($obj2->{".touch"}) &&
			($obj1->{".touch"} == $obj2->{".touch"}));
}


test "import() and \$APPCLASS" => sub {
	TEST(!defined $IDX::Config::APPCLASS) || warn "APPCLASS is already ".
		"defined: $IDX::Config::APPCLASS";
	IDX::Config::import("MyBogusApp::Config");
	TEST($IDX::Config::APPCLASS eq "MyBogusApp::Config") ||
		warn "Unexpected APPCLASS: $IDX::Config::APPCLASS";
	IDX::Config::import("YetAnotherBogusApp::Config");
	TEST($IDX::Config::APPCLASS eq "MyBogusApp::Config") ||
		warn "APPCLASS has changed: $IDX::Config::APPCLASS";
};

my $config;
test "the() from both main and subclass" => sub {
	$config=the MyBogusApp::Config();
	TEST(ref($config) eq "MyBogusApp::Config") ||
	  warn "ref(\$config) = ".ref($config);
	my $config2=the IDX::Config();
	TEST(ref($config2) eq "MyBogusApp::Config") ||
	  	  warn "ref(\$config2) = ".ref($config2);

	TEST(sameobj($config,$config2)) ||
	  warn "Object mismatch";

	TEST(! $config->dirty) ||
	  warn "\$config is dirty after read\n".Data::Dumper::Dumper($config);
	TEST(! $config2->dirty) ||
	  warn "\$config2 is dirty after read\n".Data::Dumper::Dumper($config2);
};

test "the() vs. fresh()" => sub {
	TEST(sameobj($config,the IDX::Config)) ||
	  warn "Object mismatch (1)";

	my $fresh=fresh IDX::Config;
	TEST(! sameobj($config,$fresh)) ||
	  warn "Object clash (1)";

	TEST(! sameobj($fresh,fresh IDX::Config)) ||
	  warn "Object clash (2)";

	TEST(sameobj($config,the IDX::Config)) ||
	  warn "Object mismatch (2)";
};

test "reading from the defaults" => sub {
	TEST($config->val("sect1","key12") eq "val121") ||
	  warn "[sect1]key12=".$config->val("sect1","key12");
	TEST(!defined $config->val("sect1","keybogus"))  ||
	  warn "[sect1]keybogus=".$config->val("sect1","keybogus");
};

test "multi-valued entries (here-doc-style and old-style)"
  => sub {
	  my @val=$config->val("sect1","key11");
	  TEST($val[0] eq "val111");
	  TEST($val[1] eq "val112");

	  @val=$config->val("sect1","key13");
	  TEST($val[0] eq "val131");
	  TEST($val[1] eq "val132");
	  TEST($val[2] eq "val133");
};

test "val() in array context (tests inexistant keys)" => sub
{
	my @vals = $config->val ("sect1", "key11");
	TEST (scalar (@vals) == 2) || warn "Did not get 2 matching entries for ".
		"[sect1]/key11: ".Data::Dumper::Dumper (\@vals);
	@vals = $config->val ("sect5", "key51");
	TEST (scalar (@vals) == 1) || warn "Did not get 1 matching entry for ".
		"[sect5]/key51: ".Data::Dumper::Dumper (\@vals);
	@vals = $config->val ("sect1", "inexistant");
	TEST (scalar (@vals) == 0) || warn "Did not get an empty array on ".
		"inexistant key: ".Data::Dumper::Dumper (\@vals);
};

test "getval() in scalar context" => sub
{
	# Some value.
	my $val    = $config->getval ("sect3", "key31");
	my $refval = $config->val    ("sect3", "key31");
	TEST ($val eq $refval) || warn "Got '$val' instead of '$refval'";
	# Empty but existant value.
	$val    = $config->getval ("sect5", "key51");
	$refval = $config->val    ("sect5", "key51");
	TEST ($val eq $refval) || warn "Got '$val' instead of '$refval'";
	# Inexistant value.
	try
	{
		$val = $config->getval ("sect1", "inexistant");
		warn "Did not throw an error for inexistant key.";
		TESTFAIL;
	}
	catch IDX::PKI::Error::Config with
	{
		TESTOK;
	};
};

test "getval() in array context" => sub
{
	# Some value.
	my @vals = $config->getval ("sect1", "key11");
	TEST (scalar (@vals) == 2) || warn "Didn't get 2 values for [sect1]/key11";
	TEST ($vals[0] eq "val111") ||
		warn "$vals[0] instead of val111 for [sect1]/key11[0]";
	@vals = $config->getval ("sect5", "key51");
	# Empty but existant value.
	TEST (scalar (@vals) == 1) || warn "Didn't get 1 values for [sect1]/key11";
	TEST ($vals[0] eq "") ||
		warn "'$vals[0]' instead of '' for [sect5]/key51[0]";
	# Inexistant value.
	try
	{
		@vals = $config->getval ("sect1", "inexistant");
		warn "Did not throw an error for inexistant key.";
		TESTFAIL;
	}
	catch IDX::PKI::Error::Config with
	{
		TESTOK;
	};
};

test "getPath()" => sub
{
	my $val    = $config->getPath ("sect1", "key12");
	my $refval = $config->val     ("sect1", "key12");
	TEST ($val eq $refval) || warn "[LVL0] Got '$val' instead of '$refval'";
	$val    = $config->getPath ("sect1", ["INDIRECTION1", "key12"], "keyI1");
	$refval = $config->getPath ("INDIRECTION1-val121",              "keyI1");
	TEST ($val eq $refval) || warn "[LVL1] Got '$val' instead of '$refval'";
	$val = $config->getPath ("sect1", ["INDIRECTION1", "key12"],
							 ["INDIRECTION2", "keyI1"], "keyI2");
	$refval = $config->getPath ("INDIRECTION2-valI1",   "keyI2");
	TEST ($val eq $refval) || warn "[LVL2] Got '$val' instead of '$refval'";

	TEST(!defined($config->getPath("BOGUSPATH","boguskey"))); # Should not throw
};

my $writerconfig=fresh IDX::Config(); # We do not use the(), so as to
# test auto-reload later.

test "updating and reading back" => sub {
	TEST(! $writerconfig->dirty());

	$writerconfig->setval("sect1","key12","zoinx");

	TEST($writerconfig->dirty());

	$writerconfig->setval("sect3","key33","zoinx","me","plenty");

	TEST($writerconfig->val("sect1","key12") eq "zoinx") ||
	  warn "subtest 2: ".$writerconfig->val("sect1","key12");
	TEST($writerconfig->val("sect3","key33") eq "zoinx\nme\nplenty") ||
	  warn "subtest 3: ".$writerconfig->val("sect3","key33");

};

test "writing to the main config file (must throw)" => sub {
	try {
		# Try to fool filename-based detections.
		$writerconfig->WriteConfig("/tmp/../../../../$configfile");
		TESTFAIL();
	} catch IDX::PKI::Error::Config with {
		TESTOK();
	};
};


my $newfilename = new IDX::Test::Filename();

# OK we are going to accept the fact that this test fails for
# now. This only means that the config file gets cluttered with
# useless stuff after each save.
test "writing to another config file (should only write the delta)" => sub {
	$writerconfig->WriteConfig("$newfilename");
	TEST(-f $newfilename) || die "File not created";

#	TEST(system("grep", "sect1", $newfilename)) ||
#		#warn "$newfilename should not contain a [sect1]\n".`cat $newfilename`;
#		warn "$newfilename should not contain a [sect1]";
	warn "For now this test is not implemented";
	1;
};

test "automatic reloading" => sub {
	my $oldconfig=$config;
	rename($newfilename,$configfile);
	$config=the IDX::Config();

	TEST($config->val("sect3","key31") eq "val311") ||
	  warn "subtest 1: ".$config->val("sect1","key12");

	TEST($config->val("sect1","key12") eq "zoinx") ||
	  warn "subtest 2: ".$config->val("sect1","key12");

	TEST(!sameobj($oldconfig,$config));
};


test "updateLocked()" => sub {
	IDX::Config->updateLocked (sub {
		my $config = shift;
		$config->AddSection ("Section");
		$config->newval ('Section', 'SingleParameter', 'foo');
		$config->newval ('Section', 'MultiParameter', 'bar', 'baz', 'quux');
	});

	$config = the IDX::Config();
	TEST (! $config->dirty ()) || warn "Config still dirty";
	TEST ($config->val ("Section", "SingleParameter") eq "foo") ||
		warn "Can't find SingleParameter again";
	open (FILE, "<", "$configfile");
	TEST (grep {m/^\[Section\]$/} (<FILE>)) ||
		warn "Config file should contain a [Section]";
	close (FILE);
};

# Persistent lock problems.
test "File is locked by updateLocked" => sub {
	defined (my $pid = fork()) || die "Can't fork()";
	if (!$pid)
	{	# Child process: Do nothing, just wait with with keeping the
		# config locked.
		IDX::Config->updateLocked (sub {1 while (1)});
	}
	else
	{	# Parent process: Try to save the config.
		sleep (1);	# Let the child a bit time to lock our file.
		try {
			IDX::Config->updateLocked (sub {});
			TESTFAIL;
		} catch IDX::PKI::Error::IO with {
			TESTOK;
		};

		kill KILL => $pid || die "Couldn't kill our child with PID $pid";
	}
};

test "File is not locked any more" => sub {
	# No need to do anything useful.
	IDX::Config->updateLocked (sub {});
};

test "nested updateLocked and performance" => sub {
	my $incrs=100;
	IDX::Config->updateLocked
	  (sub {
		my ($config)=@_;
		$config->newval("TESTnested","counter",0);
		for(my $i=0; $i<$incrs; $i++) {
			IDX::Config->updateLocked
			  (sub {
				my ($config)=@_;
				$config->newval("TESTnested","counter",
							   $config->getval("TESTnested","counter")+1);
			   });
		};
	   });
	TEST(IDX::Config->the()->getval("TESTnested","counter") == $incrs) ||
	  warn IDX::Config->the()->getval("TESTnested","counter");
};

test "automatic reloading when the() is dirty" => sub {
	warn "Please write this test";
	1;
};

test "bogus config file throws" => sub {
	warn "Please write this test";
	1;
};

test "Comments with newlines" => sub {
	$config->AddSection ("mysection");
	$config->newval ("mysection", "someparam", "blah");
	my $testcomment = << "EOCOMMENT";
This is a multiline parameter.
# Some lines already have a marker,
while some other don't.
; is also a valid comment marker.
EOCOMMENT
#'#
	$config->SetSectionComment ("mysection", $testcomment,
								"And an other comment.");
	$config->SetParameterComment ("mysection", "someparam", $testcomment,
								  "And an other comment.");
	foreach my $line ($config->GetSectionComment ("mysection", $testcomment))
	{
		next unless ($line);
		TEST ($line =~ m/^\s*[\#;]/s) || warn "Section comment line has no ".
			"marker: $line";
		TEST ($line !~ m/\n/s) || warn "Section comment line contains a ".
			"\\n: $line";
	}
	foreach my $line ($config->GetParameterComment ("mysection", $testcomment))
	{
		next unless ($line);
		TEST ($line =~ m/^\s*[\#;]/s) || warn "Parameter comment line has no ".
			"marker: $line";
		TEST ($line !~ m/\n/s) || warn "Parameter comment line contains a ".
			"\\n: $line";
	}
};

test "set()" => sub {
	IDX::Config->set();
	TEST(! the IDX::Config->val("main","some"));
	IDX::Config->set(<<"CONFIG");
[main]
some = thing
CONFIG
	TEST(the IDX::Config->val("main","some") eq "thing");

	my $config=the IDX::Config;
	$config->setval("main","some","body");
	IDX::Config->set($config);

	TEST(the IDX::Config->val("main","some") eq "body");
};

test "set() forbidden outside test suites" => sub {
	TEST(! system("/usr/bin/perl","-Tw","-I",join(":",@INC),
				"-MIDX::Config","-e",1));
	TEST(system("/usr/bin/perl","-Tw","-I",join(":",@INC),
				"-MIDX::Config","-e","IDX::Config->set('')"));
};

$config = set IDX::Config (<< "EOCONF");
[sect1]
key11=val11
key12=val12

[sect2]
key21=val21
EOCONF
undef ($config->{"cf"});

test "updateLocked() without a config file (must fail)" => sub {
	try {
		IDX::Config->updateLocked (sub {
			my ($config) = @_;
			$config->newval ("sect2", "key22", "val22");
		});
		TESTFAIL;
	} catch IDX::PKI::Error::Config with {
		TESTOK;
	};
};

{
	my $file = new IDX::Test::Filename ();
	$config->{"cf"} = "$file";
	$IDX::Config::PATH = "$file";
	# Test situation.
	test "updateLocked() with a defined but inexistant file [1/2]" => sub {
		TEST (! -f $file) || die "$file already exists";
		IDX::Config->updateLocked (sub {
			my ($config) = @_;
			$config->newval ("sect2", "key22", "val22");
		});
		TEST ($config->val ("sect2", "key22") eq "val22") ||
			die "Can't find back [sect2]/key22: ".
			Data::Dumper::Dumper ($config);
	};

	# Not test situation.
	test "updateLocked() with a defined but inexistant file [2/2]" => sub {
		delete ($config->{"IDX_testing"});
		unlink ($file) || die "Can't unlink $file";
		IDX::Config->updateLocked (sub {
			my ($config) = @_;
			$config->newval ("sect2", "key23", "val23");
		});
		$config = the IDX::Config ();	# So it will reload $PATH
		TEST ($config->val ("sect2", "key23") eq "val23") ||
			die "Can't find back [sect2]/key23: ".
			Data::Dumper::Dumper ($config);
	};
}	# End of scope for $file.


################################################################
# Bootstrap tests.

# Don't want to get default values from previous tests.
undef $IDX::Config::PATH;
undef $IDX::Config::APPCLASS;
undef $config;

my $configstatus = new IDX::Test::Filename ();
open (CONFIGSTATUS, ">", $configstatus) ||
	die "Can't open config.status file $configstatus for writing.";
print CONFIGSTATUS <<EOCONF;
CONFIG_ANOPTION=This is an option
CONFIG_INSTALL_AMODULE=1
CONFIG_INSTALL_ANOTHERMODULE=1
# CONFIG_INSTALL_UNUSED is not set
# CONFIG_NOOPTION is not set
CONFIG_PRIVS_GROUP_ME=somegroup
CONFIG_PRIVS_USER_ME=someuser
CONFIG_PORT_HISPROTO=4321
CONFIG_PORT_MYPROTO=1234
# CONFIG_PORT_UNUSEDPROTO is not set
CONFIG_SOMEOPTION=some value
CONFIG_WITH_BINARY=/usr/bin/binary
CONFIG_WITH_DIR=/usr/local/share
EOCONF
close (CONFIGSTATUS);

my %opts;
test "Reading config.status" => sub {
	%opts = IDX::Config->bootstrap_readconfigstatus ($configstatus);
	TEST ($opts{"ANOPTION"} eq "This is an option") ||
		warn "ANOPTION: ".$opts{"ANOPTION"};
	TEST (exists $opts{"NOOPTION"}) || warn "NOOPTION does not exist";
	TEST (!defined $opts{"NOOPTION"}) || warn "NOOPTION is defined";
};

test "bootstrap_config()" => sub {
	$config = IDX::Config->bootstrap_config (%opts);
	TEST (UNIVERSAL::isa ($config, "IDX::Config")) ||
		die "The returned value is not an IDX::Config object";
	my $val = $config->getval ("paths", "binary");
	TEST ($val eq "/usr/bin/binary") ||
		warn "Wrong [paths]/binary value: '$val'";
};

test "bootstrap_configfile()" => sub {
	$configfile = IDX::Config->bootstrap_configfile ($configstatus);

	# Tests that we can load it.
	my $testfile = new IDX::Test::Filename ();
	open (FILE, ">", $testfile) || die "Can't write to $testfile.";
	print FILE $configfile;
	close (FILE);

	local $IDX::Config::PATH = "$testfile";
	$config = fresh IDX::Config ();
	my $val = $config->getval ("paths", "binary");
	TEST ($val eq "/usr/bin/binary") ||
		warn "Wrong [paths]/binary value: '$val'";
};

my $patchedmod = new IDX::Test::Filename ();

test "bootstrap_module() -- basics" => sub {
	my $testmodule = new IDX::Test::Filename ();
	{
		local *FILE;
		open (FILE, ">", $testmodule) || die "Can't write test module ".
			"$testmodule";
		print FILE << "TESTMODULE";
#!/usr/bin/perl -Tw
package IDX::MyApp::Config;
use strict;
use base "IDX::Config";
1;
TESTMODULE
		close (FILE);
	}
	require $testmodule;

	# Overwrite normal definition.
	{
		no warnings;
		sub IDX::Config::bootstrap_configpath {
			return "/usr/local/etc/mybogusapp/default.conf";
		}
	}

    my @oldtmpfiles=(glob "/tmp/idx-config-*");

	local $IDX::Config::APPCLASS = "IDX::MyApp::Config";
	my $module = IDX::MyApp::Config->bootstrap_module ($configstatus,
													   "$testmodule");
	TEST ($module) || warn "Empty output!";
	TEST ($module =~ m/default\\\.conf/m) ||
		warn "Can't find quoted 'default.conf' in bootstraped module.";
	TEST ($module =~ m/^sub ([A-Za-z0-9]+::)*_default\b/m) ||
		warn "Can't find _default() definition";

	{
		local *FILE;
		open (FILE, ">", $patchedmod) || die "Can't open $patchedmod to ".
			"write the new module.";
		print FILE $module;
		close (FILE);
	}

    my @tmpfiles=glob "/tmp/idx-config-*";
    if (@tmpfiles > @oldtmpfiles) {
		TESTFAIL;
		warn "Temp file leak";
        warn "This subtest may fail if several developpers are testing at once (e.g. you may be seeing someone else's temp file";
    };


	# Test that we don't have suppressed things from original module.
	require IO::Pipe;
	my $pipe = new IO::Pipe;
	$pipe->reader ("diff", "$testmodule", "$patchedmod");
	my $diff = join ("", <$pipe>);
	TEST ($diff !~ m/^</m) || warn "Lost lines:\n$diff";
};

# Useful to find our IDX::* modules.
my $uselib = "use lib qw (".join (" ", @INC).");\n";

# Use the bootstrapped module in a standard script.
test "bootstrap_module() -- _default() [1/2]" => sub {
	$DB::single=1;
	my $testscript = new IDX::Test::Filename ();
	open (FILE, ">", $testscript) || die "Can't open test script $testscript ".
		"for writing.";
	print FILE << "EOSCRIPT";
#!/usr/bin/perl -Tw
use strict;
$uselib;
require "$patchedmod";
my \$config = the IDX::Config ();
my \$val = \$config->val("privileges", "user_me");
die "Wrong [privileges]/user_me value: '\$val'" unless (\$val eq "someuser");
\$val = \$config->val ("paths", "binary");
die "Wrong [paths]/binary value: '\$val'" unless (\$val eq "/usr/bin/binary");
EOSCRIPT
	close (FILE);

	TEST (! system ("perl", "-Tw", $testscript));
};

# Use the bootstrapped module in a subclass.
test "bootstrap_module() -- _default() [2/2]" => sub {
	my $testmod = new IDX::Test::Filename ();
	open (FILE, ">", $testmod) || die "Can't open test script $testmod ".
		"for writing.";
	print FILE << "EOSCRIPT";
#!/usr/bin/perl -Tw
package IDX::TestMod::Config;
use strict;
use vars qw (\@ISA);
BEGIN
{
	$uselib;
	require "$patchedmod";
	push (\@ISA, "IDX::MyApp::Config");
}
1;
EOSCRIPT
	close (FILE);

	my $testscript = new IDX::Test::Filename ();
	open (FILE, ">", $testscript) || die "Can't open test script $testscript ".
		"for writing.";
	print FILE << "EOSCRIPT";
#!/usr/bin/perl -Tw
use strict;
require "$testmod";
my \$config = the IDX::TestMod::Config ();
my \$val = \$config->val("privileges", "user_me");
die "Wrong [privileges]/user_me value: '\$val'" unless (\$val eq "someuser");
\$val = \$config->val ("paths", "binary");
die "Wrong [paths]/binary value: '\$val'" unless (\$val eq "/usr/bin/binary");
EOSCRIPT
	close (FILE);

	TEST (! system ("perl", "-Tw", $testscript));
};

1;
