#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::MagicHash> - Automagically user-filled hashes

=head1 SYNOPSIS

 use IDX::MagicHash;
 my %hash;
 tie (%hash, "IDX::MagicHash", (
 	"name" => "What's your name?",
 	"age"  => "What's your age?",
 ));
 print "Hello ".$hash{"name"}.".You said you are ".
 	$hash{"age"}." years old.";

=head1 DESCRIPTION

Hashes which are I<tie()>d to this package are automatically build when
needed, by asking a question to the user.

This package inheritates from B<Tie::StdHash>. See L<perltie> for more
informations about tying a variable.

=cut "

package IDX::MagicHash;

use strict;

use IDX::PKI::Error;

=pod "

=head1 METHODS

=over 4

=item I<TIEHASH (\%questions, \%defaults)>

This is the constructor. It expects a list of questions to ask the user in
order to fill the tied hash, and an optional list of default values.

=cut "

sub TIEHASH {
	my ($class, $questions, $defaults) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		if ($questions && ref ($questions) ne "HASH");
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		if ($defaults && ref ($defaults) ne "HASH");

	my %questions = $questions ? %$questions : ();
	my %defaults  = $defaults  ? %$defaults  : ();

	my $self = {
		"questions" => \%questions,
		"defaults"  => \%defaults,
		"data"      => {},
	};
	return bless ($self, $class);
}

=pod "

=item I<FETCH ($key)>

Returns the value corresponding to key I<$key>. If I<$key> doesn't
I<exist()> yet, asks the corresponding question to the user.

A C<> answer (empty string) means to use the default, if any, while C<"">
(quotes) means an empty answer.

=cut "

BEGIN { our $term=$ENV{TERM}; }

sub FETCH {
	my ($self, $key) = @_;

	unless (exists ($self->{"data"}{$key})) {
		# Computes the question to ask.
		my $question = $self->{"questions"}{$key};
		$question = $key unless ($question);
		$question =~ s/^\s*//;
		$question =~ s/\s*$//;

		# Default value.
        my $default = defined $self->{"defaults"}{$key} ?
			$self->{"defaults"}{$key} : "";
		$question .= "?" unless (substr ($question, -1) eq "?");
		$question .= " [$default]" if (defined ($default) && $default ne "");

		my $val;
			# User answer ("" means empty).
			print $question.((length ($question) < 60) ? " " : "\n");
			chomp ($val = <STDIN>);

		if ($val eq "") {
			$val = $default;
		} elsif ($val eq '""') {
			$val = "";
		}
		$self->{"data"}{$key} = $val;
	}

	return $self->{"data"}{$key};
}

=pod "

=item I<STORE ($key, $val)>

Stores I<$val> as the value corresponding to key I<$key>.

=cut "

sub STORE {
	my ($self, $key, $val) = @_;
	return $self->{"data"}{$key} = $val;
}

=pod "

=item I<DELETE ($key)>

Implements the I<delete()> function.

=cut "

sub DELETE {
	my ($self, $key) = @_;
	return delete ($self->{"data"}{$key});
}

=pod "

=item I<CLEAR ()>

Clears the whole hash.

=cut "

sub CLEAR {
	my ($self) = @_;
	$self->{"data"} = {};
	return 1;
}

=pod "

=item I<EXISTS ($key)>

Implements the I<exists()> function.

=cut "

sub EXISTS {
	my ($self, $key) = @_;
	return exists ($self->{"data"}{$key});
}

=pod "

=item I<FIRSTKEY ()>

Gets the first key of the hash.

=cut "

sub FIRSTKEY {
	my ($self) = @_;
	(undef) = keys %{$self->{"data"}};	# Reset each() iterator.
	return each (%{$self->{"data"}});
}

=pod "

=item I<NEXTKEY ()>

Gets the next key of the hash.

=cut "

sub NEXTKEY {
	my ($self) = @_;
	return each (%{$self->{"data"}});
}

=pod "

=item I<set_question ($key, $question, [$key, $question, ...])>

Sets I<$question> as the question for the I<$key> key. You may pass more
than one pair of parameters.

=cut "

sub set_question {
	my ($self, @args) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		if (scalar (@args) % 2);

	while (my ($key, $question) = splice (@args, 0, 2)) {
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-key" => $key,
										 "-question" => $question)
			if ((! defined $key) || ($key eq "") ||
				(! defined $question) || ($question eq "") ||
				ref ($key) || ref ($question));
		$self->{"questions"}{$key} = $question;
	}
}

=pod "

=item I<set_default ($key, $default, [$key, $question, ...])>

Sets I<$default> as the default value for the I<$key> key. You may pass
more than one pair of parameters.

=cut "

sub set_default {
	my ($self, @args) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		if (scalar (@args) % 2);

	while (my ($key, $default) = splice (@args, 0, 2)) {
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-key" => $key,
										 "-default" => $default)
			if ((! defined $key) || ($key eq "") || (! defined $default) ||
				ref ($key) || ref ($default));
		$self->{"defaults"}{$key} = $default;
	}
}

=pod "

=item I<get_default ($key)>

Returns the default value for the I<$key> key. May return undef if I<$key>
currently has no defined default value.

=cut "

sub get_default {
	my ($self, $key) = @_;

	return $self->{"defaults"}{$key};
}

=pod "

=back

=cut "

require IDX::Test::Suite unless (caller ());
1;

__END__

####################  Test Suite  ####################

use strict;

my %questions = (
				 "name"  => "What's your name?",
				 "age"   => "What's your age",
				 "city"  => "Where do you live",
				 "phone" => "Phone number",
				 );

my %defaults = (
				"age"   => 21,
				"city"  => "Nowhere",
				"phone" => "012345678",
				);

my %answers = (
			   "name"  => "Alice",
			   "age"   => 25,
			   "city"  => "",			# Should get the default.
			   "phone" => '""',			# Should get "".
			   );

my (%hash, $object);

sub do_answer ($$) {
	my ($hashref, $field) = @_;

	die "I don't know what to answer for $field"
		unless (defined $answers{$field});

	# Saves STDIN & STDOUT.
	local (*OLDIN, *OLDOUT);
	open (OLDIN,  "<&STDIN");
	open (OLDOUT, ">&STDOUT");

	# Redirects STDIN & STDOUT from and to files.
	my $file_w = new IDX::Test::Filename ();
	my $file_r = new IDX::Test::Filename ();
	local *FILE;
	open (FILE, ">", $file_r);
	print FILE $answers{$field}."\n";
	close (FILE);

	open (STDIN,  "<", $file_r) || die "Can't redirct STDIN: $!";
	open (STDOUT, ">", $file_w) || die "Can't redirct STDOUT: $!";

	# Does the magic.
	my $val = $hashref->{$field};

	# Reads result.
	open (STDIN,  "<&OLDIN")  || die "Can't reestablish STDIN: $!";
	open (STDOUT, ">&OLDOUT") || die "Can't reestablish STDOUT: $!";

	open (FILE, "<", $file_w);
	my $question = join ("", <FILE>);
	close (FILE);

	# Checks.
	my $expected = $answers{$field};
	if ((! defined ($expected)) || $expected eq "") {
		$expected = $defaults{$field};
	} elsif ($expected eq '""') {
		$expected = "";
	}

	TEST ($val eq $expected) || warn "Unexpected value: '$val' instead of ".
		"'$expected'";
	TEST ($question =~ m/$questions{$field}/ &&
		  ((! defined $defaults{$field}) ||
		   ($question =~ m/$defaults{$field}/))) ||
		   warn "Unexpected question: '$question' (should contain ".
		   "'$questions{$field}'".($defaults{$field} ?
								 " and '$defaults{$field}'" : "").")";
}

test "Tying a hash" => sub {
	$object = tie (%hash, "IDX::MagicHash", \%questions, \%defaults);
};

test "Accessing keys" => sub {
	do_answer (\%hash, "name");
	do_answer (\%hash, "age");
	do_answer (\%hash, "city");
	do_answer (\%hash, "phone");
};

1;
