#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: Authentication.pm,v 1.21 2003/07/28 15:54:43 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Authentication.pm,v $

=pod "

=head1 NAME

B<IDX::PKI::Authentication> - Tools for authenticating SSL peers

=head1 SYNOPSIS

 use IDX::PKI::Authentication qw(authorize);

 authorize($cgi, "PRIV_CONFIG", $section);

=head1 DESCRIPTION

B<IDX::PKI::Authentication> encapsulates methods for checking the
identity (is our peer really who she pretends to be?) and the
privilege (is our peer on our accept list?) in a variety of
situations.

=head2 Access control lists (ACLs)

ACLs implement a simple way of dealing with privilege attribution.
IDX-PKI's ACLs are based on X509/LDAP DNs: they consist in just
a list of DNs stored in the PKI's configuration. Only those people
able to authenticate using a certificate with one of those DNs, while
respecting all PKIX rules (proof of ownership, certification paths,
validity periods and CRLs etc.), are granted access.

ACLs are further subdivided into 4 privilege classes, that (for now)
seem to capture most cases: C<PRIV_MAINT> (the right to perform routine
maintenance such as reboot a server, do a backup, etc.); C<PRIV_CONFIG>
(the right to change runtime configuration parameters); C<PRIV_USE> (the
right to use the facility on a day-to-day basis, such as accept or
reject a certificate at the CA); C<PRIV_CHPRIV> (the right to add or
remove privileges for somebody else). It is up to the calling
application to determine which class applies to the requested action,
and the ACL is checked in the requested action only (or put it otherwise,
there are four different ACLs among which the calling application gets
to pick and choose).

Here is an example (using the default file-based configuration):

 [ACL-somefunction]
 can-change-privs=CN=Bob Administrator,O=IDEALX
 can-configure=CN=John R. Hacker,O=IDEALX
 can-configure=CN=Somebody E. Lse,O=IDEALX
 ; Comments are introduced in the usual way.
 other-parameters=are OK and ignored


=cut "

package IDX::PKI::Authentication;

use strict;
use Exporter;
use vars qw(@ISA @EXPORT_OK);
@ISA=qw(Exporter);

@EXPORT_OK=qw(authorize is_authorized authenticate %privilegeNames);

use IDX::PKI::Error;
use IDX::PKI::Config;
use IDX::PKI::DN;

use vars qw(%privilegeNames); # See pod below
use vars qw($authOK); # See pod below


=pod "

=head1 VARIABLES

=over 4

=item I<$authOK>

This package variable is set to one every time the I<authorize()> function
succeeds.  This intended as a hook for a handful of tricks to make IDX-PKI
prefer not to work at all rather than work insecurely (see
L<IDX::PKI::CGI> and L<IDX::PKI::Transport>).

=item I<%privilegeNames>

An associative array that maps privilege names to their configuration file
suffix. Should never be written to or a security hazard would occur.

=back

=cut "

%privilegeNames=(PRIV_CHPRIV=>"change-privs",
	PRIV_CONFIG=>"configure",
	PRIV_MAINT=>"maintain",
	PRIV_USE=>"use");


=pod "

=head1 METHODS

=over 4

=item I<getPeerDN()>

This method must be implemented in all packages that encapsulate an SSL
connection: External packages receive an extra piggyback method at the
time B<IDX::PKI::Authentication> is loaded, while packages belonging to
IDX-PKI don't need this trick and provide the method themselves.  The
method checks the validity of the peer's certificate w.r.t. the CA
certificate, and B<only if> it is valid, returns the DN of the peer's
certificate.  In case of any failure, returns C<undef>.  "valid" is to be
understood in the PKIX sense, that is: Correct signature from the CA's
public key; validity dates OK; no unimplemented critical extension fields.

If called in an array context, returns the list of the RDN keys and
values (e.g. C<CN>,C<John R. Hacker>,C<O>,C<IDEALX>,C<C>,C<fr>),
in L<IDX::PKI::DN> order, or the empty list upon failure.  In scalar
context, returns the full DN with whatever separator the application
uses natively (this is only used for debugging).  As per the X509
spec, the DN list is to be treated as identical to its reverse, and
the case of the DN component identifiers (e.g. C<C>, C<OU> etc.) does
not matter.  User code (see L</authorize>) has to take care of not
making abusive distinctions between DNs despite these two rules.

=item I<authorize($object, $operation, $section)>

Checks if the individual who talks to us via I<$object> is allowed to
perform I<$operation>, given the ACL in I<$section> of the configuration.
I<$object> must implement a I<getPeerDN()> method (see L</getPeerDN>).

I<$operation> must be one of the following constants: C<PRIV_CHPRIV>,
C<PRIV_CONFIG>, C<PRIV_MAINT>, and C<PRIV_USE>.  The exact semantics of
each of these priviledges depends on the IDX-PKI module; however, the
following general rules apply: C<PRIV_CHPRIV> means this user can change
other user's priviledges (but not his own), and add or remove priviledged
users; C<PRIV_CONFIG> means this user can change the configuration
parameters of this module (such as the IP address of the external LDAP
server); C<PRIV_MAINT> means this user can trigger maintenance operations
(such as make a backup copy or reboot a server); C<PRIV_USE> means this
user can use this facility at a privileged level for PKI tasks (for
example, accept CSR's in the RA, or trigger a republication).

If the authentication fails, I<authorize()> will throw an
B<IDX::PKI::Error::Security> exception (see L<IDX::PKI::Error>) with an
HTTP error code of 403.

=cut "

sub authorize {
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS") unless
		(scalar(@_) eq 3);

	my ($object, $operation, $section) = @_;

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") unless
		(my $method=UNIVERSAL::can($object,'getPeerDN'));


	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") unless
		(my $name = $privilegeNames{$operation});

	my @DN=&$method($object);

	my $config = the IDX::PKI::Config ();

	throw IDX::PKI::Error::Security
		("SSL_REJECTED", -HTTPCode => 403,
		 -priv => $name, -section => $section,
		 -Authorized => [$config->getPath($section,"can-$name")],
		 -You => 'Unauthenticated',
	) unless (@DN);

	foreach my $authorized
		($config->getPath($section,"can-$name")) {
			do {
				$authOK=1;
				return 1;
			} if (IDX::PKI::DN::compare($authorized, \@DN));
		};

	throw IDX::PKI::Error::Security
		("SSL_REJECTED", -HTTPCode => 403,
		 -priv => $name, -section => $section,
		 -Authorized => [$config->getPath($section,"can-$name")],
		 -You => IDX::PKI::DN::string(@DN)
	);
};

=pod "

=item I<is_authorized($object, $operation, $section)>

Performs the same as I<authorize()> except it returns C<1> when the
person is authorized and C<undef> if not.

=cut "

sub is_authorized {
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS") unless
		(scalar(@_) eq 3);

	my ($object, $operation, $section) = @_;

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") unless
		(my $method=UNIVERSAL::can($object,'getPeerDN'));

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") unless
		(my $name = $privilegeNames{$operation});

	my @DN=&$method($object);

	my $config = the IDX::PKI::Config ();

	return undef unless (@DN);

	foreach my $authorized ($config->getPath($section,"can-$name")) {
		  return 1 if (IDX::PKI::DN::compare($authorized, \@DN));
	};

    return undef;
};

=pod "

=item I<authenticate($object,$expectedDN1,$expectedDN2,...)>

Checks if the individual who talks to us via I<$object> is known to us.
I<$object> must implement a I<getPeerDN()> method (see L</getPeerDN>) and
said method must return a DN that matches one of I<$expectedDN1>, ...

If the authentication fails, I<authenticate()> will throw an
B<IDX::PKI::Error::Security> exception (see L<IDX::PKI::Error>).

NOTE: If you design new services that need authentication, better use
authorize() than authenticate(). This way we can later envision more
sophisticated ways of doing authorization than just configuration-based ACLs
(databases, LDAP, Keynote, etc.). authenticate() is best used by clients
for authenticating servers, not the other way round.

=cut "

sub authenticate {
	my ($object,@expected_DNs) = @_;

	my @DN = $object->getPeerDN();

	if (@DN) {
		foreach my $testdn (@expected_DNs) {
			return 1 if (IDX::PKI::DN::compare($testdn, \@DN));
		};

		throw IDX::PKI::Error::Security("Authentication failed",
			-You=>IDX::PKI::DN::string(@DN),
			-Authorized=>\@expected_DNs);
	} else {
		throw IDX::PKI::Error::Security("Authentication required",
			-You => "NOBODY",
			-Authorized=>\@expected_DNs);
	};
}


=pod "

=back

=cut "


eval(join '',<main::DATA>) || die $@ unless caller;

1;

__END__

#################### TEST SUITE #########################

use IDX::Test;

sub Bogus::SSLSocket::getPeerDN {
	return wantarray ? ("DC","com","dc","example", "CN","John Doe") :
		"CN=John Doe,DC=example,DC=com";
}

my $object = bless {},'Bogus::SSLSocket';

my $config = set IDX::PKI::Config (<< "CONFIG");
[bogusmodule]
can-change-privs=CN=John R. Hacker, DC=example, DC=com
can-change-privs=CN=God Himself, O=too many symbolic links
can-mess-up-with=CN=John Doe, DC=example, DC=com
can-configure=CN=John Doe, DC=example, DC=com

[zonkmodule]
can-change-privs=CN=John Doe/DC=example/DC=com
can-change-privs=CN=John Doeuf/DC=example/DC=com
CONFIG

sub TEST_AUTHFAILS(&) {
	my $sub = shift;
	try {
		&$sub;
		TESTFAIL;
	} catch IDX::PKI::Error::Security with {
		TESTOK;
	};
}

number_of_tests(3);

test "Wrong calling conventions" => sub {
	try {
		authorize();
		warn "Authorize without arguments should throw";
		TESTFAIL;
	} catch IDX::PKI::Error::Internal with {
		TESTOK;
	};

	try {
		authorize("bogusmodule", $object, "PRIV_CONFIG");
		warn "Authorize with inverted arguments should throw";
		TESTFAIL;
	} catch IDX::PKI::Error::Internal with {
		TESTOK;
	};

	try {
		authorize($object, "PRIV_MESS_UP_WITH", "bogusmodule");
		warn "Authorize with bogus PRIV_ should throw";
		TESTFAIL;
	} catch IDX::PKI::Error::Internal with {
		TESTOK;
	};
};


test "Granting" => sub {
	authorize($object, "PRIV_CONFIG", "bogusmodule");
	authorize($object, "PRIV_CHPRIV", "zonkmodule");
	TEST(is_authorized($object, "PRIV_CONFIG", "bogusmodule"));
	TEST(is_authorized($object, "PRIV_CHPRIV", "zonkmodule"));
	authenticate($object, "CN=John Doe/DC=example/DC=com");
	TEST(is_authorized($object, "PRIV_CONFIG", "bogusmodule"));
	TEST(is_authorized($object, "PRIV_CHPRIV", "zonkmodule"));
};


test "Denying" => sub {
	TEST_AUTHFAILS {authorize($object, "PRIV_CHPRIV", "bogusmodule")};
	TEST_AUTHFAILS {authorize($object, "PRIV_CONFIG", "zonkmodule")};
	TEST_AUTHFAILS {authenticate($object, "CN=John Foe/DC=example/DC=com")};
	TEST(!is_authorized($object, "PRIV_CHPRIV", "bogusmodule"));
	TEST(!is_authorized($object, "PRIV_CONFIG", "zonkmodule"));
};
