#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::PKI::CA> - CA specific functions

=head1 SYNOPSIS

 use IDX::PKI::Config;
 use IDX::PKI::CA;
 use IDX::PKI::Principal;
 use IDX::PKI::Transport;

 my $config = the IDX::PKI::Config ();
 my $ca = new IDX::PKI::CA ();

 my $csr_id = 'ABCDE';
 my $csr_transport = new IDX::PKI::Transport::Unspooler
 	('TRANSPORT-IN-'.$config->getPath ('CA', 'csr_transport_in'));
 my $csr = pullCSR ($csr_transport, $csr_id);
 my $x509 = $ca->csr_accept ($csr);
 $csr_transport->expunge ();

 my $crr_id = 'ZYXWV';
 my $crr_transport = new IDX::PKI::Transport::Unspooler
 	('TRANSPORT-IN-'.$config->getPath ('CA', 'crr_transport_in'));
 my $crr = pullCRR ($crr_transport, $crr_id);
 my $crr_serial = $ca->crr_accept ($crr);
 $crr_transport->expunge ();

=head1 DESCRIPTION

This module centralises the functions the CA has to do both in manual and
automatic modes. This includes:

=over 4

=item *

Certificate signature (in various situations);

=item *

Certificate revocation and CRL issuing;

=item *

Certification policy control (checking permissions over who can make
what kind of requests);

=item *

Key generation in centralized mode;

=item *

CA database, both for internal bookkepping needs (e.g. CRL and serial
number for new certificates) and for third-party analysis of the
validity of certificates at any point in time.

=back

Deliverables of a CA are:

=over 4

=item *

X509 certificates;

=item *

X509 CRLs;

=item *

PKCS#12 files (in centralized mode);

=item *

Messages to the publishing entity (responsible for updating external
LDAP directories, sending user-friendly e-mails, etc.) and the
distribution entity (that forwards the PKCS#12 files to the users in a
CP-dependent way);

=item *

Full logging, both informal (for drawing charts) and database-style
(for rebuilding the state of the PKI at any point in time).

=back

This class is an implementation of these requirements. The CA database
(which is mandatory) is an B<IDX::PKI::CertDB::CA> object, created at the
same time as the CA object.

The IDX-PKI CA is designed to be able to work in the most automatic
possible way. Still, there is a need for very detailed specification
of certificate contents, and make the latter available for special
cases (such as the key bootstrap procedure) - hence the abundant API,
comprising both simple and complex functions.

All the cryptographic tasks are implemented using the B<openssl> shell tool
(version 0.9.6. or more recent is mandatory).


=cut "

package IDX::PKI::CA;

use strict;

$IDX::PKI::CA::VERSION = '0.01';

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Crypto::ASN1;
use IDX::PKI::Crypto::CSR;
use IDX::PKI::Crypto::CRL;
use IDX::PKI::Crypto::CRR;
use IDX::PKI::Crypto::X509;
use IDX::PKI::Crypto::SPKAC;
use IDX::PKI::Crypto::PKCS7 qw(pkcs7Decrypt);
use IDX::PKI::Crypto::PKCS10;
use IDX::PKI::Crypto::PKCS12;
use IDX::PKI::Crypto::Tools;
use IDX::PKI::CertDB;
use IDX::PKI::FileManip;
use IDX::PKI::Profile;
use IDX::PKI::Principal; # for sendMimeMail
use IDX::PKI::DN;
use IDX::PKI::LDAP;
use IDX::PKI::Transport;
use IDX::PKI::Message::Publish;
use IDX::PKI::Log;
use MIME::Base64;
use Fcntl qw(:DEFAULT :flock);

=pod "

=head1 METHODS

=over 4

=item I<new ([$log])>

Object constructor. An IDX-PKI CA in working order will be constructed in
memory from the data in section C<[CA]>, which will log into an
B<IDX::PKI::Log> object either passed as an argument or constructed from
section C<[log]>.

=cut "

sub new {
	my ($that, $log) = @_;
	my $class = ref ($that) || $that;

	my $config = the IDX::PKI::Config ();
	$log = new IDX::PKI::Log () unless (defined $log);
	return bless {
		config => $config,
		log => $log,
	}, $class;
}


=pod "

=item I<_handle_centralized ($csr, $x509, $pkcs12)>

Does The Right Thing with a pair (private key, X509 certificate) the
CA just generated (such as bundling it into a PKCS#12 file, and
sending it encrypted by email). This code should be moved into another
PKI entity, and should not be called directly meanwhile.

=cut "

sub _handle_centralized {
	my ($self, $csr, $x509, $pkcs12) = @_;
	my $config = $self->{"config"};

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (UNIVERSAL::isa ($csr, "IDX::PKI::Crypto::CSR") &&
				UNIVERSAL::isa ($x509, "IDX::PKI::Crypto::X509") &&
				UNIVERSAL::isa ($pkcs12, "IDX::PKI::Crypto::PKCS12"));

	my $profilename = $csr->getFormEntry ("profile");
	my $profile = new IDX::PKI::Profile ($profilename);

	my $distribution = $config->getPath ('CA', 'central') || 'profile';
	$distribution    = $profile->getDistribution ()
		if (($distribution eq 'profile') && $profile);
	$distribution  ||= 'user';

	my $caadmin = $config->getPath ('CA', 'admin_email');

	# Infos on the administrator.
	my $admin_cert = parse IDX::PKI::Crypto::X509
		(decode_base64($csr->getFormEntry ("admin")));
	my $admin = $admin_cert->getEmail() || $caadmin;

	my $mail   = $x509->getEmail();
	my $dn     = build IDX::PKI::DN ($csr->getDN ());
	my $cn     = $dn->getUserID ();

	if ($distribution eq 'user')
	{
		# Emails the passwd-encrypted PKCS#12 to the user.
		my $p12 = $pkcs12->serialize ($csr->getFormEntry ("password"));
		my $message = << 'MESSAGE';
[French translation at the end. / Traduction en franais  la fin.]

This email contains a PKCS#12 file that has been created for your usage.
If you don't understand the meaning of this message (and its
implications) or you think you got it by mistake, please contact your
nearest system administrator.

          ----------------------------------------

Ce courrier lectronique contient un fichier PKCS#12 cr pour votre
usage. Si vous ne comprenez pas pourquoi vous recevez ce message (et ses
implications) ou vous pensez l'avoir reu par erreur, contactez votre
administrateur informatique le plus proche.
MESSAGE

		$self->{'log'}->attempt({"-subject" => "P12_SENT",
								 "-details" => ["dist: ".$distribution,
												"DN: ".$x509->getSubjectDN (),
												"dest: ".$mail]},
	sub {
		(my $filename = $cn) =~ s/[^a-z0-9\-]+/_/gi;
		IDX::PKI::Principal::sendMimeMail (
			'-from'        => $admin,
			'-to'          => $mail,
			'-subject'     => "IDX-PKI PKCS#12",
			'-message'     => $message,
			'-attachment'  => $p12,
			'-base64'      => 1,
			'-mimetype'    => "application/x-pkcs12",
			'-filename'    => "${filename}.p12",
		);
	});

	} elsif ($distribution eq 'admin') {
		# Emails the PKCS#12 to the administrator.
		my $message = '
[French translation at the end. / Traduction en franais  la fin.]

This mail contains a PKCS#12 file which has been created for:
'.$cn.(($mail && $mail ne $cn) ? ' ('.$mail.')' : '').'.

          ----------------------------------------

Ce courrier lectronique contient un fichier PKCS#12 cr pour:
'.$cn.(($mail && $mail ne $cn) ? ' ('.$mail.')' : '').'.
';

		$self->{'log'}->attempt({"-subject" => "P12_SENT",
								 "-details" => ["dist: ".$distribution,
												"DN: ".$x509->getSubjectDN (),
												"dest: ".$admin]},
	sub {
		(my $filename = $cn) =~ s/[^a-z0-9\-]+/_/gi;
		IDX::PKI::Principal::sendMimeMail (
			-from        => $admin,
			-to          => $admin,
			-subject     => "PKCS#12",
			-message     => $message,
			-attachment => $pkcs12->serialize(),
			-base64      => 1,
			-mimetype    => "application/x-pkcs12",
			-filename    => "${filename}.p12",
			-cert        => $admin_cert,
		);
});

	} elsif ($distribution eq 'ee') {
		# Sends the passwd-encrypted PKCS#12 back to the EE.
        my $pass = $csr->getFormEntry ("password");
		my $p12 = $pkcs12->serialize ($pass);
		require IDX::PKI::Message::PKCS12Distribution;
		my $message = build IDX::PKI::Message::PKCS12Distribution
			('-data' => $p12, '-x509' => $x509, '-pass' => $pass);
		my $section = $config->getPath ("CA", "p12_transport_out");
		my $transport = new IDX::PKI::Transport::Sender
			("TRANSPORT-OUT-".$section);

		$self->{'log'}->attempt({"-subject" => "P12_SENT",
								 "-details" => ["dist: ".$distribution,
												"DN: ".$x509->getSubjectDN (),
												"dest: EE"]},
	sub {

		$transport->send ($message);
});

	} else {
		# Unknown distribution.
		throw IDX::PKI::Error::Internal ("Unknown distribution method",
			'-distribution' => $distribution);
	}
}

=pod "

=item I<csr_accept ($csr)>

Accepts a pending CSR and returns the new certificate
B<IDX::PKI::Crypto::X509> object. I<$csr> is an B<IDX::PKI::Crypto::CSR>
object, as returned (for example) by I<IDX::PKI::Principal::pullCSR()>, or
an B<IDX::PKI::Message::TrustedRequest> (still unsupported yet).

A I<certificate signing request> (CSR) is a bunch of key-value parameters
about the data that has to go into the certificate, as well as some
tracing information. I<csr_accept()> will check permissions about said
tracing data (this called I<applying the CA policy>), optionaly generate a
private key (see L</_generate_private_key>), and then sign the request
into a certificate (see L</_sign_cert>). An exception of class
B<IDX::PKI::Error::Security> will be raised if an invalid CSR is received.

=cut "

sub csr_accept {
	my ($self, $csr) = @_;
	my $x509;
	$self->{log}->attempt
		({-subject => "CSR_SIGN", -attachment => [$csr]},
		 sub {
			 $x509 = $self->_do_csr_accept ($csr);
			 return {'-attachment' => [$x509]};
		 });
	return $x509;
}

=pod "

=item I<_do_csr_accept ($csr)>

Does the real job of I<csr_accept>, which is simply a wrapper to ensure
proper logging. Don't call it directly.

=cut "

sub _do_csr_accept {
	my ($self, $csr) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless ($csr &&
				(UNIVERSAL::isa ($csr, 'IDX::PKI::Crypto::CSR') ||
				 UNIVERSAL::isa ($csr, "IDX::PKI::Message::TrustedRequest")));

	throw IDX::PKI::Error::Internal ("UNIMPLEMENTED")
		if ($csr && UNIVERSAL::isa ($csr,
									'IDX::PKI::Message::TrustedRequest'));

	my $config = $self->{'config'};
	my $log    = $self->{'log'};

	# Some default values are set according the the profile (if any).
	my $profilename = $csr->getFormEntry ("profile");
	my $profile = new IDX::PKI::Profile ($profilename);
	# No profile? No publishing-specific data, just the cert! Fair enough.
	# The IDX::PKI::Profile class will handle this for us (undef now is
	# a valid value for the profile name)

	####################
	# XXX TODO: Should add here some policy checks about the CSR.

	####################
	# We are about to generate a private key (central mode only) and to
	# sign the certificate.

	# Central-mode certificates: Generate the private key.
	my $type = $csr->typename ();
	my ($key, $req);
	if ($type eq "Central") {
		$log->attempt ({
			"-subject" => "CERT_SIGN",
			"-details" => "Private key generation.",
		}, sub {
			($key, $req) =
				($self->_generate_private_key ($profile->getKeySize (),
											   [$csr->getDN()]));
			throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-key" => $key,
											 "-req" => $req)
				unless ($key && $req);
			# Don't log the private key. ;-)
			return {"-attachment" => $req};
		});
	} else {
		$req = $csr->getCSROpenssl ();
	}
	$type = "PKCS10" if ($type eq "Central");

	# Decrypt what is to be decrypted.
	# XXX TODO: Do it on demand only, no by modifying the whole CSR
	# object.
	$csr = $self->_decrypt_csr ($csr);

	my $transaction = new IDX::PKI::CertDB::CA ("CA", 1);

	# Signs the request.
	my $x509;
	try {
		$log->attempt ({
			"-subject" => "CERT_SIGN",
			"-details" => ["Certificate signature.", "DN: ".$csr->getDN()],
		}, sub {
			$x509 = $self->_sign_cert (
									   "req" => $req,
									   "type" => $type,
									   ($profile->csr2certoptions ($csr)),
									   "serial" => $transaction->nextSerial (),
									   );
			return {"-attachment" => $x509};
		});
	} otherwise {
		my ($E) = @_;
		# At this point, nothing has been done outside memory, so we can
		# rollback.
		$transaction->rollback ();
		$E->throw ();
	};

	# Add the newly created cert into the internal repository.
	$transaction->addCert ($x509);
	$transaction->setCertAttr ($x509, "profile", $profilename)
		if ($profilename);
	$transaction->commit ();

	####################
	# At this point, IDX-PKI knows about the issued certificate. We still
	# have to announce the good news to the whole world. :-)

	# We try to sends messages right now, to avoid unnecessary delays.
	# However if something fails, we keep the informations on the hard
	# disk so that we can retry later.

	if (defined $key) {		# Central mode.
		my @CAs = loaddir
			IDX::PKI::Crypto::X509 ($config->getval ("paths", "cacertsdir"));
		my $p12obj = build IDX::PKI::Crypto::PKCS12 ($x509, $key, @CAs);
		my $error;


		# Distribute the PKCS#12 to its destination.
		try {
			$log->attempt ({
				"-subject" => "CERT_SIGN",
				"-details" => "Sending the PKCS#12 to its recipient.",
			}, sub {
				$self->_handle_centralized ($csr, $x509, $p12obj);
			});
		} catch IDX::PKI::Error with {
			# Sending the message to its recipient failed.
			my ($E) = @_;
			$self->_dump_state ("P12_DISTRIBUTION", 0, $csr, $x509, $p12obj,
								$E);
			$error = 1;
		};
		return $x509 if ($error);
	}

	# Sends the message to the publishing module.
	my $section = $config->val ("CA", "publish_transport_out");

	# Some EPCs may not have a publishing module at all.
	if ($section) {
		my $error;
		try {
			$log->attempt
				(["CERT_SIGN",
				  "-details" => ["Sending the cert to publishing module",
								 "DN: ".IDX::PKI::DN::rfc2253($csr->getDN()),
								 "profile: ".$profile->getType(),
								 "serial: ".$x509->getSerial()],
				  "-attachment" => $x509],
				 sub {
					 # Transport initialisation.
					 my $publish_transport = new IDX::PKI::Transport::Sender
						 ("TRANSPORT-OUT-".$section);

					 # Sends the message.
					 my $message = build IDX::PKI::Message::Publish
						 ("-type" => "certificate",
						  "-object" => $x509,
						 ($profilename ? ("-profile" => $profilename) : ()));
					 $publish_transport->send($message);
				 });
		} catch IDX::PKI::Error with {
			# Sending the message to the publishing module failed.
			# No need to keep the private key here.
			my ($E) = @_;
			$self->_dump_state ("PUBLISH", 0, $csr, $x509, undef, $E);
			$error = 1;
		};
		return $x509 if ($error);
	}

	# XXX TODO: Find a way to move this into the publishing module.

	# Send server certs by email
	if (my $admin = $csr->getFormEntry ("adminEmail")) {
		my $server  = $csr->getFormEntry ("subjectAltName");
		my $message = << "EOF";
[French translation at the end. / Traduction en franais  la fin.]

The certificate for $server has been created.

          ----------------------------------------

Le certificat pour $server a t cr.
EOF
		(my $filename = $server) =~ s/[^a-z0-9\-]+/_/gi;
		IDX::PKI::Principal::sendmailAttached ($admin, $admin,
			"Server Certificate Generated / Certificat serveur genere",
			$message,
			"application/pkix-cert", "${filename}.pem", $x509->serialize (),
			"application/pkix-cert", "${filename}.der", $x509->serialize ('DER'),
			"application/x-pkcs7-certificates", "${filename}.p7b", $x509->serialize ('IE'));
	};

	return $x509;
}

=pod "

=item I<_generate_private_key ([$keysize[, $dn]])>

This method generates a private key and returns it as an
B<IDX::PKI::Crypto::PrivateKey> object. If I<$dn> is provided, it also
generates the associated certificate request and returns it as a PKCS#10
string. In scalar context, only returns the private key.

It is an error to call I<_generate_private_key()> in array context without
supplying a DN (it probably means you wouldn't get what you wanted
anyway).

If I<$keysize> is not provided or null, defaults to the configured value.

I<$dn> can be either a string, an array reference or an B<IDX::PKI::DN>
object.

=cut "

sub _generate_private_key ($;$$)
{
	my ($self, $keysize, $dn) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-keysize" => $keysize)
		if (defined ($keysize) && ($keysize =~ m/\D/));
	$keysize = $self->{"config"}->getval ("CA", "default_keysize")
		unless ($keysize);

	# Sanity check: Array context is for key + req generation only.
	throw IDX::PKI::Error::Internal ("ILLEGAL_CONTEXT")
		if (wantarray () && ! $dn);

	# We could first create a private key in all cases, and then only
	# create the associated request from that private key, but openssl
	# does not help us much with that (we _have_ to write the private key
	# into a file to run openssl req)...

	# No DN or not wantarray? We only need to create a private key.
	if ((! defined $dn) || (! wantarray)) {
		return new IDX::PKI::Crypto::PrivateKey ($keysize);
	}

	# Canonicalise the DN: We want it for openssl.
	if (! defined ($dn)) {
		$dn = "";
	} elsif (UNIVERSAL::isa ($dn, "IDX::PKI::DN")) {
		$dn = $dn->to_openssl ();
	} elsif (ref ($dn) eq "ARRAY") {
		$dn = (build IDX::PKI::DN (@$dn))->to_openssl ();
	} elsif (! ref ($dn)) {
		$dn = (parse IDX::PKI::DN ($dn))->to_openssl ();
	} else {
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-dn" => $dn);
	}

	# Creates our workdir.
	my $cadir = $self->{"config"}->getval ('CA', 'database_dir');
	throw IDX::PKI::Error::Config ("CONFIG_INVALID", "-cadir" => $cadir)
		unless (-d $cadir);
	my $tmpdir = new IDX::PKI::FileManip::TempDirectory ("$cadir/tmp");

	# Request.
	my $opensslconf = << "CONFIG";
[req]
prompt = no
distinguished_name = dn

[dn]
$dn
CONFIG
	string2file ($opensslconf, "$tmpdir/openssl.conf");

	# Create the private key and associated request.
	my ($result, $error) =
		run_openssl ("req", "-new", "-newkey", "rsa:$keysize",
					 "-nodes", "-sha1", "-config", "$tmpdir/openssl.conf",
					 "-out", $tmpdir."/req.pem",
					 "-keyout", $tmpdir."/key.pem");

	throw IDX::PKI::Error::Crypto ("Error generating key",
								   "-result" => $result, "-error" => $error,
								   "-conf" => $opensslconf)
		unless (-s "$tmpdir/key.pem" && -s "$tmpdir/req.pem");

	my $key = load IDX::PKI::Crypto::PrivateKey ("$tmpdir/key.pem");
	my $req = file2string ("$tmpdir/req.pem");

	throw IDX::PKI::Error::Crypto ("Error generating key",
								   "-result" => $result, "-error" => $error,
								   "-conf" => $opensslconf,
								   "-key" => $key, "-req" => $req)
		unless ($key && $req);

	# If we are here, we wantarray().
	return ($key, $req);
}


=pod "

=item I<_getSubjectAltName ("altnames" => \%altnames)>

Returns the proper C<subjectAltName=...> string for the openssl
configuration file.

I<%altnames> is a hash of alternative names. Recognised keys are C<email>,
C<URI> and C<DNS>; values may be plain strings or anonymous arrays.

=cut "

sub _getSubjectAltName ($%) {
	throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS")
		unless (scalar (@_) % 2);
	my ($self, %args) = @_;
	my $altnames = $args{"altnames"};
	$altnames ||= {};
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
									 "-details" => "Incorrect altnames")
		unless (ref ($altnames) eq "HASH");
	my $altstring;

		# SubjectAltName.
		# Format: subjectAltName=type:altname1, type:altname2
		my @altnames;
		foreach my $type (keys %$altnames) {
			push (@altnames, (ref ($altnames->{$type}) eq "ARRAY") ?
				  map {"$type:$_"} @{$altnames->{$type}} :
				  "$type:$altnames->{$type}");
		}
		$altstring = @altnames ? "subjectAltName=".join (", ", @altnames) : "";
	return $altstring;
}

=pod "

=item I<< _sign_cert ("serial" => $serial, "req" => $req, "type" => $type,
"dn" => $dn, "validity" => $validity, "extensions" => \%exts, "altnames"
=> \%altnames, "crldistpoints" => \@crldistpoints) >>

Signs a PKCS#10 or SPKAC request and returns the corresponding new
B<IDX::PKI::Crypto::X509> object.

I<serial> is the serial number to use for signing the certificate.

I<$req> is the request string, either a PKCS#10 (in which case I<$type>
must be C<PKCS10>) or a SPKAC (in which case I<$type> must be C<SPKAC>).
If I<$type> is an B<IDX::PKI::Crypto::PKCS10> or an
B<IDX::PKI::Crypto::SPKAC> object, I<$type> is not necessary.

I<$dn> is the DN, as a B<IDX::PKI::DN> object, a DN string or an anonymous
array. I<$dn> must me provided if the request type is C<SPKAC>. In case of
PKCS#10 requests, the DN is automatically extracted from the request (it
is an error if I<$dn> is provided and doesn't match the request DN,
because B<openssl> does not support setting the DN in a PKCS#10
certificate).

I<$validity> (optional) is the wanted certificate validity in days.
Otherwise takes the default from I<_opensslconf()>.

I<%exts> is the hash of wanted extensions (same format as
I<%IDX::PKI::Crypto::X509::keyUsageExtensions>, just smaller). For a given
key, we may pass C<critical> or C<noncritical> as the first value to set
criticity. By default, I<keyUsage> is considered critical.

I<%altnames> is a hash of alternative names. Recognised keys are C<email>,
C<URI> and C<DNS>; values may be plain strings or anonymous arrays.

I<@crldistpoints> are the CRL distribution points, if any.

=cut "

sub _sign_cert ($%) {
	throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS")
		unless (scalar (@_) % 2);
	my ($self, %args) = @_;

	# Serial number to use.
	my $serial = $args{"serial"};
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-details" =>
									 "Invalid or missing serial",
									 "-serial" => $serial)
		unless ($serial && $serial =~ m/^[0-9a-f]+$/i);

	# The request string and type (PKCS#10 or SPKAC).
	my ($reqobj, $type) = map {$args{$_}} qw (req type);
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
									 "-details" => "No request")
		unless ($reqobj);
	my $reqstring;
	if (ref $reqobj) {
		if (UNIVERSAL::isa ($reqobj, "IDX::PKI::Crypto::PKCS10")) {
			$type = "PKCS10";
			$reqstring = $reqobj->getCSROpenssl ();
		} elsif (UNIVERSAL::isa ($reqobj, "IDX::PKI::Crypto::SPKAC")) {
			$type = "SPKAC";
			$reqstring = $reqobj->getCSROpenssl ();
		} else {
			throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
											 "-details" => "Request object ".
											 "is of a wrong type",
											 "-request" => $reqobj);
		}
	} else {
		if ($type eq "PKCS10") {
			$reqstring = $reqobj;
			$reqobj = parse IDX::PKI::Crypto::PKCS10 ($reqobj);
		} elsif ($type eq "SPKAC") {
			$reqstring = $reqobj;
			$reqobj = parse IDX::PKI::Crypto::SPKAC ($reqobj);
		} else {
			throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
											 "-details" => "Unrecognised type",
											 "-type" => $type);
		}
	}
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
									 "-details" => "No type") unless ($type);

	# DN to use.
	my $dn = $args{"dn"};
	if (! defined ($dn)) {
		if ($type eq "PKCS10") {
			$dn = $reqobj->getDN ();
		} else {
			throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
											 "-details" => "Missing DN");
		}
	} elsif (UNIVERSAL::isa ($dn, "IDX::PKI::DN")) {
		# It's OK, nothing to do.
	} elsif (! ref ($dn)) {
		$dn = parse IDX::PKI::DN ($dn);
	} elsif (ref ($dn) eq "ARRAY") {
		$dn = build IDX::PKI::DN (@$dn);
	} else {
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
										 "-details" => "Invalid DN",
										 "-dn" => $dn);
	}

	# Strips everything except the SPKAC itself.
	if ($type eq "SPKAC") {
		if ($reqstring =~ m/^SPKAC\s*=\s*(.*)$/m) {
			$reqstring = $1;
		}
		$reqstring =~ s/\s//gs;
		$reqstring = "SPKAC=$reqstring\n".$dn->to_openssl ();
	}

	# Openssl DOESN'T know how to change a request DN!
	if ($type eq "PKCS10") {
		my $reqdn = $reqobj->getDN ();
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
										 "-details" => "Request and ".
										 "supplied DN differ.",
										 "-request_dn" => $reqdn->string (),
										 "-args_dn" => $dn->string ())
			unless (IDX::PKI::DN::compare ($dn, $reqdn));
	}

	# Validity.
	my $validity = $args{"validity"};

	# Certificate extensions.
	# Format: category=[critical, ]ext1, ext2
	my $exts = $args{"extensions"};
	$exts ||= {};
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
									 "-details" => "Incorrect extensions")
		unless (ref ($exts) eq "HASH");
	my $extstring = "";
	foreach my $type (keys %$exts) {
		# Key usage are critical.
		my $critical = ($type eq "keyUsage");
		my @exts = (ref ($exts->{$type}) eq "ARRAY") ? @{$exts->{$type}} :
			($exts->{$type});
		# We may pass wanted criticity as the first argument.
		$critical = ((shift @exts) =~ m/^critical$/i)
			if ($exts[0] && $exts[0] =~ m/^(non)?critical$/i);
		unshift (@exts, "critical") if ($critical);
		# We remove the NS prefix from NSemail & al.
		@exts = map {s/^NS//; $_} @exts if ($type eq "nsCertType");
		$extstring .= "$type=".join (", ", @exts)."\n" if (@exts);
	}

	my $altstring = $self->_getSubjectAltName (%args);

	# CRL distribution points.
	# Format: crlDistributionPoints=URI:crl1,URI:crl2
	my $crldists = $args{"crldistpoints"};
	$crldists ||= [];
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
									 "-details" => "Incorrect crldistpoints")
		unless (ref ($crldists) eq "ARRAY");
	my $crlstring = @$crldists ?
		"crlDistributionPoints=".join (",", map {"URI:$_"} @$crldists) : "";

	# Creates our workdir.
	my $cadir = $self->{"config"}->getval ('CA', 'database_dir');
	throw IDX::PKI::Error::Config ("CONFIG_INVALID", "-cadir" => $cadir)
		unless (-d $cadir);
	my $tmpdir = new IDX::PKI::FileManip::TempDirectory ("$cadir/tmp");


	# Does the signing.
	my $opensslconf = $self->_opensslconf ();
	$opensslconf .= << "CONFIG";
database        = $tmpdir/index.txt       # database index file.
serial          = $tmpdir/serial          # The current serial number
x509_extensions = x509_extensions         # The extentions to add to the cert

[ x509_extensions ]
subjectKeyIdentifier=hash
authorityKeyIdentifier=keyid:always
$extstring
$altstring
$crlstring

CONFIG
	string2file ($opensslconf, "$tmpdir/openssl.conf");
	string2file ($reqstring, "$tmpdir/req.pem");	# Cert request.
	string2file ("", "$tmpdir/index.txt");			# Empty index.txt.
	string2file ($serial, "$tmpdir/serial");		# Serial number.

	# Sign the certificate.
	my $in = ($type eq "SPKAC") ? "spkac" : "in";

    my $cakey = load IDX::PKI::Crypto::PrivateKey
      	($self->{"config"}->getval ('CA', 'key'),
      	(-r $cadir."/passwd" ? ($cadir."/passwd") : ()));

	my ($result, $error) =
		run_openssl ("ca", "-preserveDN", "-batch", "-md", "sha1", "-notext",
					 "-config", "$tmpdir/openssl.conf",
					 ($validity ? ("-days", $validity) : ()),
					 "-$in", $tmpdir."/req.pem",
			         "-cert", $self->{"config"}->getval ('CA', 'certificate'),
					 $cakey->getOpensslParams("CA")
					 );

	throw IDX::PKI::Error::Crypto ("Certificate signing failed",
								   "-result" => $result, "-error" => $error,
								   "-config" => $opensslconf)
		unless ($result);

	return parse IDX::PKI::Crypto::X509 ($result);
}

=pod "

=item I<_decrypt_csr ($csr)>

This internal method decrypts encrypted data in the CSR, such as protected
passwords. Of course, they have to be encrypted I<for> the CA.

TODO: Currently I<$csr> can only be a B<IDX::PKI::Crypto::CSR>;
B<IDX::PKI::Message::TrustedRequest> are not implemented.

=cut "

sub _decrypt_csr ($$)
{
	my ($self, $csr) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (UNIVERSAL::isa ($csr, "IDX::PKI::Crypto::CSR") ||
				UNIVERSAL::isa ($csr, "IDX::PKI::Message::TrustedRequest"));

	throw IDX::PKI::Error::Internal ("UNIMPLEMENTED")
		if (UNIVERSAL::isa ($csr, "IDX::PKI::Message::TrustedRequest"));

	# No profile? Nothing to do then.
	my $profilename = $csr->getFormEntry ("profile");
	return $csr unless (defined $profilename);

	my $profile = new IDX::PKI::Profile ($profilename);

	# For each form element, decrypt it if the profile wants it.
	foreach my $elem ($profile->getFormElements ()) {
		my $encrypt  = $profile->elementOptions ($elem, "encrypt");
		my $longname = $profile->elementLongName ($elem);

		if ($encrypt && $encrypt eq "CA") {
			my $certpath = $self->{"config"}->getPath ("CA", "certificate");
			my $keypath  = $self->{"config"}->getPath ("CA", "key");
			# Decrypt the field directly into the CSR object.
			# XXX TODO: Don't mess-up into the csr internals...
			$csr->{'form'}->{$longname} = pkcs7Decrypt
				($csr->{'form'}->{$longname}, $keypath, $certpath);
		}
	}

	return $csr;
}


=pod "

=item I<_pkcs12_distribution ($pkcs12object)>

Sends the PKCS#12 object to its final recipient (may be the user, the
administrator, or a special page on the EE entity).

=cut "

sub _pkcs12_distribution ($$) {
	my ($self, $p12obj) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (UNIVERSAL::isa ($p12obj, "IDX::PKI::Crypto::PKCS12"));

	my $config = $self->{"config"};

	throw IDX::PKI::Error::Internal ("UNIMPLEMENTED");
}

=pod "

=item I<_certificate_publish ()>

=cut "

sub _certificate_publish () {
	throw IDX::PKI::Error::Internal ("UNIMPLEMENTED");
}

=pod "

=item I<_dump_state ($state, $attempts, $csr, $x509, $p12obj, $error)>

Saves the current request state on the hard disk, so we can retry the
sending later. I<$csr> is the currently handled CSR, I<$x509> is the
signed certificate and, if relevant, I<$p12obj> is the PKCS#12 object
containing the associated private key and certificate. I<$attempts> is the
number of previously failed attempts (C<0> means first failed attempt), so
that we can warn the administrator by email but not spam him too often.

I<$error> is the B<IDX::PKI::Error> object of the error which just
occured. It is used for the warning email only.

I<$state> represents the next undone step:

=over 4

=item C<ESCROW>

We have to send the PKCS#12 to the key escrow module (see
L</_pkcs12_escrow()>);

=item C<P12_DISTRIBUTION>

We have to send the PKCS#12 to its final recipient (see
L</_pkcs12_distribution()>);

=item C<PUBLISH>

We have to send the certificate to the publishing module (see
L</_certificate_publish()>).

=back

=cut "

sub _dump_state ($$$$$$) {
	my ($self, $state, $attempts, $csr, $x509, $p12obj, $error) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		if ($state ne "ESCROW" && $state ne "P12_DISTRIBUTION" &&
			$state ne "PUBLISH");
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		if (defined ($attempts) && $attempts =~ m/\D/);
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		if (! UNIVERSAL::isa ($csr, "IDX::PKI::Crypto::CSR"));
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		if (! UNIVERSAL::isa ($x509, "IDX::PKI::Crypto::X509"));
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		if (defined ($p12obj) &&
			! UNIVERSAL::isa ($p12obj, "IDX::PKI::Crypto::PKCS12"));
	# Textifies the error.
	if (! defined ($error)) {
		$error = "";
	} elsif (UNIVERSAL::can ($error, "humanize")) {
		$error = $error->humanize ();
	} elsif (UNIVERSAL::can ($error, "serialize")) {
		$error = $error->serialize ();
	} elsif (ref ($error)) {
		throw IDX::PkI::Error::Internal ("INCORRECT_ARGS");
	}

	my $config = $self->{"config"};

	# Warn the administrator every 10 attempts -- begins with first one.
	$attempts = 0 unless ($attempts);
	if (($attempts % 10) == 0)
	{
		my $admin;
		try {
			my $admin_cert = parse IDX::PKI::Crypto::X509
				(decode_base64 ($csr->getFormEntry ("admin")));
			$admin = $admin_cert->getEmail();
		};
		$admin = $config->getval ("CA", "admin_email") unless ($admin);

		my ($stateen, $statefr);
		if ($state eq "ESCROW")
		{
			$stateen = "contact the key escrow module";
			$statefr = "contacter le squestre de clefs";
		}
		elsif ($state eq "P12_DISTRIBUTION")
		{
			$stateen = "distribute the generated PKCS#12";
			$statefr = "distribuer le PKCS#12 gnr";
		}
		elsif ($state eq "PUBLISH")
		{
			$stateen = "contact the publishing module";
			$statefr = "contacter le module de publication";
		}
		my $serial = $x509->getSerial ();
		my $dn     = $x509->getSubjectDN ();
		my $message = << "MESSAGE";
[French translation at the end. / Traduction en franais  la fin.]

Certificate $serial belonging to $dn:

An error occured when trying to $stateen.
I will retry later but you should check the network.

          ----------------------------------------

Certificat $serial appartenant  $dn:

J'ai rencontr une erreur en tentant de $statefr.
Je ressaierai plus tard mais vous devriez vrifier le rseau.


$error
MESSAGE
#'#"#

		IDX::PKI::Principal::sendmail ($admin, $admin, "Network Error / ".
									   "Erreur reseau", $message);
	}

	my $cacert = load
		IDX::PKI::Crypto::X509 ($config->getval ("CA", "certificate"));

	# CSR is encrypted to us (the CA) because it may contain sensitive
	# data.
	my $encreq = build
		IDX::PKI::Crypto::PKCS7::EnvelopedData ($csr->serialize (), $cacert);

	# PKCS#12 -- encrypted too.
	my $encp12;
	$encp12 = build
		IDX::PKI::Crypto::PKCS7::EnvelopedData ($p12obj->serialize (), $cacert)
		if ($p12obj);

	# Builds the message.
	require IDX::PKI::Message::Simple;
	my $msg = build
		IDX::PKI::Message::Simple ("-objects" => [$x509, $encreq,
												  ($p12obj ? $encp12 : ())
												  ],
								   "-data" => ["state"    => $state,
											   "attempts" => $attempts + 1,
											   ]);

	# Saves the message in the spool directory.
	my $dir = $config->getval ("CA", "waiting_dir");
	return IDX::PKI::Transport::write_spoolx ($msg->serialize(), $dir);
}

=pod "

=item I<crr_accept ($crr)>

=item I<crr_accept ($crr,0)>

=item I<crr_accept ($crr,$transaction)>


Accepts a pending I<Certificate Revocation Request> (CRR), after
checking tracing data against the CA policy if appropriate (like
L</csr_accept> does for CSRs) and updates the CA database accordingly.
I<$crr> is a B<IDX::PKI::Crypto::CSR> object (as returned, for
example, by L<IDX::PKI::Principal/pullCRR>). Unless a second argument
is specified, this function also issues and signs a new CRL by calling
L</issueCrl>. If the second argument is an I<IDX::PKI::CertDB::CA> object,
then it will be used to write down database changes instead of a
locally generated transaction.

=cut "

sub crr_accept {
	my ($self, $crr, $transaction) = @_;

	throw IDX::PKI::Error::Internal ("UNIMPLEMENTED")
		if ($crr && UNIVERSAL::isa ($crr,
						'IDX::PKI::Message::TrustedRequest'));

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
	  unless ($crr && UNIVERSAL::isa ($crr, 'IDX::PKI::Crypto::CRR'));

	# No tests at all. We just trust the protection over the transport layer.
	# See csr_accept() for justification of the encapsulation violation below.

	my $reason = $crr->getFormEntry("reason");
	$self->revoke($crr->{crr}, $reason);

	# mail the certificate's owner
	my %reasons_en = ("unspecified"          => "unspecified",
					  "keyCompromise"        => "key was compromised",
					  "cACompromise"         => "CA was compromised",
					  "affiliationChanged"   => "affiliation changed",
					  "cessationOfOperation" => "cessation of operation",
					  "superseded"           => "superseded",
					  "certificateHold"      => "temporarily withheld",
					  "removeFromCRL"        => "remove from CRL",
					  );
	my %reasons_fr = ("unspecified"          => "non spcifi",
					  "keyCompromise"        => "compromission de la clef",
					  "cACompromise"         => "compromission de l'AC",
					  "affiliationChanged"   => "changement d'affiliation",
					  "cessationOfOperation" => "cessation d'activit",
					  "superseded"           => "certificat remplac",
					  "certificateHold"      => "suspension temporaire",
					  "removeFromCRL"        => "supprimer de la CRL",
					  );
	my $reason_en = $reason ? "The reason was: ".$reasons_en{$reason} : "";
	my $reason_fr = $reason ? "La raison en tait: ".$reasons_fr{$reason} : "";
	my $serial_num = $crr->{crr}->getSerial();
	my $email = $crr->{crr}->getEmail();
	my $from_email = $self->{"config"}->val("CA", "admin_email");
	my $email_subject = "Certificate revocation / Revocation de certificat";
	my $email_message = <<"EOF";
[French translation at the end. / Traduction en franais  la fin.]

Your certificate with serial number $serial_num has been revoked.
$reason_en

          ----------------------------------------

Votre certificat avec le numro de srie $serial_num a t rvoqu.
$reason_fr

EOF

	if ($email) {
		&IDX::PKI::Principal::sendmail($from_email, $email, $email_subject,
									   $email_message);
	}

	if ($#_==1) {
		$self->issueCrl();
	} elsif ($transaction && UNIVERSAL::isa
					 ($transaction, "IDX::PKI::CertDB::CA")) {
		$self->issueCrl(undef, $transaction);
	};

	return 1;
}

=pod "

=item I<revoke($cert)>

=item I<revoke($cert, $reason)>

=item I<revoke($cert, $reason, $transaction)>

Revokes a certificate, by marking it as such in the CA database, but
does not issue a CRL yet (see L</issueCrl>).

The reason parameter is optional specifies the reason of the 

The third call template with three arguments reuses $transaction as its
underlying I<IDX::PKI::CertDB::CA> object, instead of creating a local
transaction. This is handy for example for a batch of revocations that
use the same transaction (and will appear in the database as having
occured at exactly the same time).

=cut "

sub revoke  {
	my ($self, $cert, $reason, $transaction) = @_;
	$reason ||= "unspecified";

	$self->{"log"}->attempt
		({
			"-subject" => "CERT_REVOKE",
			"-details" => ["certificate serial: ".$cert->getSerial(),
						  ($reason ? ("reason : ".$reason) : () )],
		},
		 sub {
			 my $owntransaction = (! defined $transaction);
			 $transaction = new IDX::PKI::CertDB::CA ("CA", 1)
				 if ($owntransaction);
			 if($reason) {
				 $transaction->setCertAttr ($cert, "reason", "$reason");
			 }
			 $transaction->setCertAttr ($cert, "revocation", "R");
			 $transaction->commit() if ($owntransaction);
		 });
}


=pod "

=item I<issueCrl()>

=item I<issueCrl($validity_in_days)>

=item I<issueCrl($validity_in_days,$transaction)>

Issues a CRL using the current list of revoked certificates (from the CA
database) and the CA's private key and certificate. Returns an
B<IDX::PKI::Crypto::CRL> object.

If I<$validity_in_days> is specified, this will be the validity period of
the CRL (default is taken from the configuration). The start time will
always be the current system time.

If I<$transaction> is specified, it will be used instead of a
locally-created transaction. This could speed up the process (taking
advantage of the caching in B<IDX::PKI::CertDB::CA>) if using the same
transaction as the one used to revoke some certificates.

The newly issued CRL is sent to the publishing module, to be updated into
the publication LDAP, if any. If the sending fails, the action is logged
but this does not throw an error.

Additionnally, CRL publication automatically occurs by files and through
the repository LDAP, if the CA is configured this way. The files indicated
in the configuration under I<[CA]/crl> and I<[CA]/crl_der> will be
modified atomically, and encoded using the appropriate format (resp. PEM
and DER). If configuration entry I<[CA]/crl_in_repository> is defined to
some RDN, then the CRL is pushed into the repository LDAP at this RDN just
below the repository root. The node will have I<cRLDistributionPoint> as
an objectClass, and the CRL will be written into the
I<certificateRevocationList> attribute in binary DER form.

=cut "

sub issueCrl  {
	my ($self, $validity, $transaction) = @_;

	my $config = $self->{config};

	my $localtransaction = (! defined $transaction);
	$transaction = new IDX::PKI::CertDB::CA () if ($localtransaction);

	my $cadir = $config->getPath('CA', 'database_dir');
	throw IDX::PKI::Error::Config("CA is not correctly configured ".
		"(database_dir)") unless (defined $cadir);

	my $tmpdir = new IDX::PKI::FileManip::TempDirectory($cadir);

	my $crlpem = $config->getPath('CA', 'crl');
	my $crlder = $config->getPath('CA', 'crl_der');

	$crlpem ||= "$tmpdir/crl.pem";
	if (!defined $crlder) { # Second-guess the DER filename.
		($crlpem =~ m/^(.*)\.pem$/) && ($crlder = "$1.crl");
	}

	my $crlconf=<<"CRLCONF";
database        = $tmpdir/index.txt       # database index file.
crl             = $crlpem                 # The current CRL
CRLCONF

	string2file ($self->_opensslconf().$crlconf,
				 $tmpdir."/temp.conf");

	my @revoked=$transaction->searchCerts("+revocation"=>"R");

	local *INDEX;
	open(INDEX,">","$tmpdir/index.txt") ||
	  throw IDX::PKI::Error::IO("Cannot open for writing",
								-IOfile=>"$tmpdir/index.txt");
	local $\="";
	foreach my $cert (@revoked) {
		my $revocationtime=$transaction->getCertAttr($cert,"revocationTime");

		# Aagh, that's what one gets when one pokes the internals of
		# a bogus program...
		$revocationtime =~ s/^(19|20)//;

		my $serial = $cert->getSerial();
		do {
			printf INDEX
				"R\t020304183900Z\t%s\t%s\tunknown\t/CN=Mind Your Own Business\n",
				$revocationtime, $serial;
		} || throw IDX::PKI::Error::IO("Cannot write",
									   -IOfile=>"$tmpdir/index.txt");
	}; # End of loop over revoked certs.

	$transaction->rollback() if $localtransaction; # Frees lock
	close(INDEX)  || throw IDX::PKI::Error::IO("Cannot write",
								-IOfile=>"$tmpdir/index.txt");
    my $cakey = load IDX::PKI::Crypto::PrivateKey
      	($self->{"config"}->getval ('CA', 'key'),
     	(-r $cadir."/passwd" ? ($cadir."/passwd") : ()));

	my ($result, $error) = run_openssl ("ca", "-gencrl",
			"-config", $tmpdir."/temp.conf", "-md", "sha1",
			"-out", $crlpem.".new",
			($validity ? ("-crldays", $validity) : ()),
            "-cert",  $self->{"config"}->getval ('CA', 'certificate'),
            $cakey->getOpensslParams("CA")
			);

	throw IDX::PKI::Error::Crypto ("Error Generation CRL",-Openssl=>$error)
		if ($error =~ m/error|invalid/i || ! -f $crlpem.".new");

	if (defined $crlder) {
		($result, $error) = run_openssl ("crl", "-in", $crlpem.".new",
					"-out", $crlder.".new", "-outform", "der");
		throw IDX::PKI::Error::Crypto ("Error generating CRL in DER ".
				"format",-Openssl=>$error) unless ($error =~ m/^\s*$/s);
	};

	foreach my $file ($crlpem,$crlder) {
		next if !defined $file;
		rename("$file.new",$file) ||
		  throw IDX::PKI::Error::IO("Can't overwrite",-IOfile=>$file);
	}

	# Put the CRL into the repository LDAP if required to.

	if (my $rdn=$config->getPath("CA","crl_in_repository")) {
		my $dn = "$rdn,".$config->getval ("dn", "idxpki_ldapbase");
		my $repository=new IDX::PKI::LDAP::CertDB
			($config->getSectionName("CA",["LDAP-CLIENT"=>"repository_ldap"]));

		my $result = $repository->search
		  (base => $dn, scope => "base",
		   filter=> "(objectClass=*)");

		my ($cdp) = $result->entries();

		if (defined $cdp) {
			$cdp->changetype("modify");
		} else {
			$cdp=new Net::LDAP::Entry();
			$cdp->changetype("add");
			$cdp->dn($dn);
			$rdn =~ m/=(.*)$/;
			$cdp->add(CN => $1);
		}
		unless (grep {m/crldistributionpoint/i}
				$cdp->get_value("objectClass")) {
			$cdp->add(objectClass=>"cRLDistributionPoint");
		}

		$cdp->replace("certificateRevocationList;binary" =>
					  [file2string($crlder)]);
		$cdp->update($repository);

		$repository->unbind();
	}

	# mod_ssl is a plague! It fails miserably to use a CRL directory
	# although it is documented. Fortunately a "crl-bundle.pem" file does
	# the job.
	my $bundle = $config->getval ("paths", "crldir")."/crl-bundle.pem";
	local *BUNDLE;
	open (BUNDLE, ">", "$bundle-NEW$$") ||
		throw IDX::PKI::Error::IO ("Can't write CRL bundle",
								   "-IOfile" => "$bundle-NEW$$");
	foreach my $file (glob ($config->getval ("paths", "crldir")."/*.pem")) {
		next if ($file =~ m/crl-bundle/);
		local *CRLFILE;
		open (CRLFILE, "<", $file) ||
			throw IDX::PKI::Error::IO ("Can't read CRL file",
									   "-IOfile" => $file);
		print BUNDLE join ("", <CRLFILE>);
		close (CRLFILE);
	}
	close (BUNDLE);
	rename ("$bundle-NEW$$", $bundle);

	my $crlobj = load IDX::PKI::Crypto::CRL ($crlpem);

	# Sends the CRL to the publishing module. If it fails, logs the action
	# but does not stop.
	try {
		$self->{"log"}->attempt ({
			"-subject" => "PUBLISH_CRL_UPDATE",
			"-attachment" => file2string($crlpem),
		}, sub {
			# Transport initialisation.
			my $section = $config->val ("CA", "publish_transport_out");
			return unless ($section);
			my $transport = new
				IDX::PKI::Transport::Sender ("TRANSPORT-OUT-".$section);

			require IDX::PKI::Message::Publish;
			my $msg = build
				IDX::PKI::Message::Publish ("-type" => "crl",
											"-object" => $crlobj);

			$transport->send ($msg);
		});
	} catch IDX::PKI::Error with {
		# Nothing, it's all right.
		1;
	};

	return $crlobj;
}

=pod "

=item I<_opensslconf()>

Returns a snippet of OpenSSL configuration file, which contains the
common parts (file skeleton, policy or lack thereof) and is suitable
for making both certificates and CRLs. The configuration file snippet
is set up in such a way that appending directives just after the
returned string causes them to land right into the main CA
configuration section.

This function is internal to I<IDX::PKI::CA>.

=cut "

sub _opensslconf {
	my ($self)=@_;

	my $config = $self->{"config"};
	my $cadir    = $config->getval ("CA", "database_dir");
	my $certdays = $config->getval ("CA", "default_validity");
	my $crldays  = $config->getval ("CA", "default_crl_validity");
	return << "COMMONCONF";
####################################################################
oid_section = new_oids                  # This allows user-defined OIDs.

[ new_oids ]
msCertTemplate=1.3.6.1.4.1.311.20.2
uniqueIdentifier=2.5.4.45
# Put your client-specific OIDs here.

[ ca ]
default_ca = CA_Default                 # The default ca section, see below

# CA policy: no policy (we do it ourselves thank you).
[ policy_anything ]
countryName             = optional
stateOrProvinceName     = optional
localityName            = optional
organizationName        = optional
organizationalUnitName  = optional
commonName              = optional
uniqueIdentifier        = optional
emailAddress            = optional
domainComponent         = optional

####################################################################
[ CA_Default ]

dir             = $cadir                # Where everything is kept
certs           = \$dir/certs           # Where the issued certs are kept
new_certs_dir   = \$dir/certs           # default place for new certs.

certificate     = \$dir/OpCA.crt        # The CA certificate
private_key     = \$dir/OpCA.key        # The private key

default_md      = sha1
default_days    = $certdays                   # how long to certify for
default_crl_days= $crldays                    # how long before next CRL
default_md      = sha1                  # which md to use.
preserve        = yes                   # keep passed DN ordering

# Policy specification: we do not use OpenSSL's policy management mechanisms,
# since we do that by ourselves. See [policy_anything] above.
policy          = policy_anything

COMMONCONF
}

=pod "

=back

=head1 TALES AND LORE

This paragraph sums up our best knowledge of the meaning that
some ubiquitous PKIX clients assign to certificate attributes.

As a side note, many pieces of software are in a "how could my beloved
CA do anything wrong to me?" mood when it comes to interpreting the
certificate fields, which, besides, are often less than clear in the
standards (to say the least - see also L<IDX::PKI::DN>). This means
that 1) all the robustness of the resulting system is on our shoulders
(yeah, Outlook too...) and 2) we must juggle with subtly different
interpretations of standards among clients if we want everybody to go
crypto happily together - e.g. I<we have to save the world in this
very module>. And we do enjoy this!


=head2 Apache

The I<SSLCACertificatePath> does not work, use I<SSLCACertificateFile>
instead (Apache 1.3.23, RedHat 7.2 and 7.3).

=head2 Netscape navigator / communicator

Netscape relies on the Netscape extensions (surprise, surprise!) to
discriminate certificates. A certificate to be used for WebSSO must
therefore feature the C<client> I<nsCertType> extension if any NS
extension is set, while a certificate used for e-mail must have the
C<email> netscape extension (if it has any).

There is a rumour that Netscape Mail can use different certificates
for signing and encryption, but we don't know how (although this ought
to be more or less the same as in Outlook).

=head2 Internet Explorer

Under Win32, "Internet Explorer" is a word that captures more than a
mere browser (see Judge Jackson's abundant paperwork for more on this
topic :-). The cryptographic libraries and the key store are both part
of what is called "Internet Explorer": the cryptographic strength of
Windows can be seen in IE's About box; upgrading the crypto to
domestic grade (128 bits) is an IE add-on, and the key store is
available through IE configuration (Tools -> Internet options ->
Content -> Certificates).

We only deal about domestic-grade encryption here. The following notes
are known to be true for version 5.50 only.

It is possible to (wrongly) accept foreign certificates in the
C<intermediate authorities> tab, if one uses IDX-PKI's 'poor man's
directory' to fetch other user's certificates. This happened with
certificates that had no C<CA:FALSE> critical extension, and importing
certificates this way is not the appropriate procedure anyway (one
should use Outlook's address book instead - fetching peer certificates
for HTTP/S doesn't make sense). The possible effect on this incorrect
behaviour of setting C<CA:FALSE> is unknown.

Certificates that have an I<extendedKeyUsage> featuring C<clientAuth> may
be used as SSL client (WebSSO) certificates. The tested certificates
had I<keyUsage> flags set to C<nonRepudiation> and C<digitalSignature>,
whose usefulness is unknown.  They definitely need not have
C<keyEncipherment> in the I<keyUsage>, which is handy for discriminating
such certificates (which typically need to be decentralized e.g. for
nonrepudiable form signing) from e-mail encryption certificates (which
need escrowing in some CPs).

It is assumed that Netscape extensions have no effect under IE :-),
especially because the certificate browser cannot parse them.

=head2 Outlook Express

Outlook Express is an SMTP/POP3 mail client and not at all the same as
Outlook, below.

=head2 Outlook

The french version of Outlook needs a registry patch to be able to
encrypt mail (search for a file named F<outlook-enable-crypto.reg> in the
sources / installed version of IDX-PKI). Unpatched versions can still
enroll themselves (since this is done through Internet Explorer) and
decrypt e-mail.

Outlook supports different certificates for signing and encrypting,
and can send both of them in outgoing mails (the format of the
resulting S/MIME message, which appears to discriminate both certs in
some way, has not been studied yet). E-mail certs, whatever their use,
must feature C<emailProtection> in their I<extendedKeyUsage> (if they have
one), while the I<keyUsage> helps discriminating both usages of
certificates in email. Certs that have C<keyEncipherment> are OK for
encrypting mail, while certs that have one or both of C<digitalSignature>
and C<nonRepudiation> - not sure of which exactly, but both is OK - are
good for signing.

=head1 BUGS

This module still needs a lot of work. The to-do list includes:

=over 4

=item *

Add more hooks for passing any and all required certificate parameters
to I<_sign_cert()> (CA nature among others);

=item *

Remove all non-IDX-PKI-generic code (such as sending e-mails), and
create a distribution entity instead (to replace I<_handle_centralized()>
altogether);

=item *

Accept B<IDX::PKI::Message::TrustedRequest>'s in addition to (if not
instead of) the old CSR/CRR format, and implement a CA policy that is
not based only on profiles (target use-case: Multiple RAs);

=item *

Do the logging part, either here or in B<IDX::PKI::CertDB::CA>.

=back

=head1 SEE ALSO

The X509 style guide is the primary reference for what a CA has to do
to comply with the standards/best practices/worst practices/folklore
out there:

 http://www.cs.auckland.ac.nz/~pgut001/pubs/x509guide.txt

=cut "

eval (join '', <main::DATA>) || die $@ unless (caller ());

1;

__END__

####################  TEST SUITE  ####################

use strict;

use IDX::Test;
use IDX::Test::LDAPServer;

# All tests are integration tests.
use IDX::PKI::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}

my $number = 10;

number_of_tests ($number);

######## Test framework #########

# All CA tests are integration tests. I really don't feel like
# instrumenting a fake IDX::PKI::CertDB::CA object, just to find out
# afterwards that IDX::PKI::CA is not interoperable with the LDAP
# directory. Sheesh.

my $dir = new IDX::Test::Directory();


my $org = "TestPKI";
my $basedn = "O=$org";

my $server = new IDX::Test::LDAPServer::Stateful(<<"LDIF", $basedn);
dn: $basedn
objectClass: top
objectClass: organization
organizationName: $org
LDIF
$server->start();

my $sslserver = new IDX::Test::SSLServer ("-server" => $server)->start();
my ($port, $dn) = ($sslserver->getPort(), $sslserver->getDN());

# Keep some values from the "real" config.
my $config = the IDX::PKI::Config ();
my $withopenssl = $config->getPath ("paths", "openssl");

# Config file
$config = set IDX::PKI::Config (<< "CONFIG");
[paths]
openssl=$withopenssl
crldir=$dir

[dn]
idxpki_ldapbase=$basedn

[CA]
database_dir=$dir
repository_ldap=repository
crl=$dir/crl.pem
crl_der=$dir/crl.crl
crl_in_repository=CN=IDX-PKI Operational CA
crlurl=http://some.where.example.com/OpCRL.crl
default_validity=365
default_crl_validity=7
key=$dir/OpCA.key
certificate=$dir/OpCA.crt

[log]
type=syslog
facility=user
logopt=pid

[LDAP-CLIENT-repository]
commtype=UNIX
socket=$server->{"dir"}/sock

[PROFILE-test]
title=en:Bogus test
title=fr:Test bidon
type=user
keysize=1024
validity=1
rdn=CN,commonName,64,name,mandatory,,
;rdn=UID
;rdn=email
rdn=OU,organizationalUnitName,64,name,optional,,
;rdn=OU,organizationalUnitName,64,name,optional,,
rdn=O,organizationName,64,name,mandatory,,
;rdn=L,localityName,64,name,optional,,
;rdn=S,stateName,64,name,optional,,
rdn=C,countryName,2,country,mandatory,,
;form=SN,surName,64,name,optional,,
;form=GN,givenName,64,name,optional,,
form=emailAddress,emailAddress,64,email,mandatory,publish:mail,
;form=emailAddress,emailAddress,64,email,optional,publish:mail,
form=secret,sharedSecret,64,raw,mandatory,password,
; Key usage.
extension=digitalSignature
extension=nonRepudiation
; extension=keyEncipherment
; Extended key usage.
extension=clientAuth
extension=emailProtection
; Netscape.
extension=client
extension=email
can-use=
crldistpoint=http://no.where.example.com/OpCRL.crl


CONFIG

IDX::PKI::CertDB::CA->initdb();
mkdir("$dir/certs");
mkdir("$dir/tmp");

# CA certificate
string2file(<< "CACERT", $dir."/OpCA.crt");
-----BEGIN CERTIFICATE-----
MIIDKjCCAtSgAwIBAgIBADANBgkqhkiG9w0BAQUFADCBkzELMAkGA1UEBhMCRlIx
DjAMBgNVBAcTBVBhcmlzMRUwEwYDVQQKEwxUZXN0IENvbXBhbnkxHDAaBgNVBAsT
E0NvbXB1dGVyIFJlc3NvdXJjZXMxDDAKBgNVBAsTA1BLSTEVMBMGA1UEAxMMVGVz
dCBSb290IENBMRowGAYJKoZIhvcNAQkBFgtjYUB0ZXN0LmNvbTAeFw0wMTA3MDMx
MzM0NTVaFw0wNjA3MDIxMzM0NTVaMIGTMQswCQYDVQQGEwJGUjEOMAwGA1UEBxMF
UGFyaXMxFTATBgNVBAoTDFRlc3QgQ29tcGFueTEcMBoGA1UECxMTQ29tcHV0ZXIg
UmVzc291cmNlczEMMAoGA1UECxMDUEtJMRUwEwYDVQQDEwxUZXN0IFJvb3QgQ0Ex
GjAYBgkqhkiG9w0BCQEWC2NhQHRlc3QuY29tMFwwDQYJKoZIhvcNAQEBBQADSwAw
SAJBAMUnLPgRs9wTc/6dxWwxvFxAIeQeM2b+XMq91sAfVAefeJ+fQrTrPPlYZnQ9
pji8BPFrwIrcWuvfSZ95uUwDl8kCAwEAAaOCAQ8wggELMA8GA1UdEwEB/wQFMAMB
Af8wHQYDVR0OBBYEFDcAFggT1lsrWc0YMG0KviI7wwWwMIHABgNVHSMEgbgwgbWA
FDcAFggT1lsrWc0YMG0KviI7wwWwoYGZpIGWMIGTMQswCQYDVQQGEwJGUjEOMAwG
A1UEBxMFUGFyaXMxFTATBgNVBAoTDFRlc3QgQ29tcGFueTEcMBoGA1UECxMTQ29t
cHV0ZXIgUmVzc291cmNlczEMMAoGA1UECxMDUEtJMRUwEwYDVQQDEwxUZXN0IFJv
b3QgQ0ExGjAYBgkqhkiG9w0BCQEWC2NhQHRlc3QuY29tggEAMBYGA1UdEQQPMA2B
C2NhQHRlc3QuY29tMA0GCSqGSIb3DQEBBQUAA0EAwcdCLYgH2DSwu2qCt/LwKlZQ
mxZSOlzwjmJ6jYT5as+vNaMqpD84u/QP8zfVkuIJbHugA+mXYy2pANSA8HXDYw==
-----END CERTIFICATE-----
CACERT

# CA Key
string2file(<< "CAKEY", $dir."/OpCA.key");
-----BEGIN RSA PRIVATE KEY-----
Proc-Type: 4,ENCRYPTED
DEK-Info: DES-EDE3-CBC,A13647485BD481CA

6ag9rvYTK6v0RGNzgOPVueYJ8JctJlb5ZI/nG/JjpPaabtjmCOxobleoUVLG2w0W
iCNEbx9GTQfVPbpF0yu7VmAil64HtMPFjLPGNHVgYnVAisfIkgmM8lxCwFiU8h2x
SdsINwsWV5G2paiQ/yh3FxAPp/D1eOpEFMpIofjD8saq4+C8yo2Tfv4c7x35lKRZ
gixF6OjTilwwAseXBxVXuuAlwyRGOOFirU2UP1E14TOg6oQ9FvKxDpH3fFCDsLlq
DqEYhvvphaaT7uCoQacxy9faAhZE8lcsOQ9AMK6ccMJAykVLHMfvbDZaOPuSnmv/
k/ez0h5Z/BLMvEtk2or4WM/4Klufh5k4g4kXLUMScAr7kBsf1wyraIGK8gLBmfgO
36hT1P9N1Bk0wCIi36NiLMUqvx8kSlKVTIodTmhqN9Q=
-----END RSA PRIVATE KEY-----
CAKEY

# CA Key Password. In a file; yeah, security is pretty lame, but we
# don't want to let this functionality go untested and suffer feature rot.

string2file("coucou", $dir."/passwd");

############# Tests proper #################

my $ca;


test "new" => sub {
	$ca = new IDX::PKI::CA ();
	TEST(defined $ca);
};


my @dn = (
	Email => 'client@idealx.com',
	CN => "TestClient",
	OU => "IDXPKI",
	O => "IDEALX",
	L => "Paris",
	C => "FR",
);
my @bogusdn=(
	Email => "jxdoe\@someorg.com",
	CN => "John X. Doe",
	OU => "Netscape Compatibility Group",
	O => "Some Organization, Inc.",
	L => "Paris",
	C => "US",
);


my $dnstring=IDX::PKI::DN::string(@dn);

my $p10=<<"PKCS10";
-----BEGIN CERTIFICATE REQUEST-----
MIIBtjCCAR8CAQAwdjELMAkGA1UEBhMCRlIxDjAMBgNVBAcTBVBhcmlzMQ8wDQYD
VQQKEwZJREVBTFgxDzANBgNVBAsTBklEWFBLSTETMBEGA1UEAxMKVGVzdENsaWVu
dDEgMB4GCSqGSIb3DQEJARYRY2xpZW50QGlkZWFseC5jb20wgZ8wDQYJKoZIhvcN
AQEBBQADgY0AMIGJAoGBAMaWunM8nDvL7mlmTxJ60ImC5jUKOm+fOcienv9TDIWM
I3fzlbEoFJmblXg+01ghMm8djbqLzpMQve/IQtbs1+SmyHdWlktiSme80nHHhA1S
trEHH91sITJEqlAvSzQppxivlY2IH6UEqLw6oK4v8of7H8fiIRNTkEdsj295TI6l
AgMBAAGgADANBgkqhkiG9w0BAQQFAAOBgQCv97/mAYeg0hiRkY2MI7bTC9hRaIu9
UCXFUtmScmEkhchayB36SYdliZR2Z2XHP+h2om9DVxT2kQozfDm0TVnEKQojpWRh
c4/oAfAIsZPhlF9gDWpWz+EmtNWqinQabf7N7dA8RC8wYYMsucgwI8JkI4fhp4gx
/CMNiQ48B0dAvA==
-----END CERTIFICATE REQUEST-----
PKCS10

my $nakedspkac = 'MIG0MGAwXDANBgkqhkiG9w0BAQEFAANLADBIAkEAwmk6FMJ4uAVIYbcvIOx5+bDGTfvL8X5gE+R67ccMk6rCSGbVQz2cetyQtnI+VIs0NwdD6wjuSuVtVFbLoHonowIDAQABFgAwDQYJKoZIhvcNAQEEBQADQQBFZDUWFl6BJdomtN1Bi53mwijy1rRgJ4YirF15yBEDM3DjAQkKXHYOIX+qpz4KXKnl6EYxTnGSFL5wWt8X2iyx';

my $spkac = << "SPKAC";
SPKAC=$nakedspkac
C=FR
L=Paris
O=IDEALX
OU=IDXPKI
CN=TestClient
Email=client\@idealx.com
SPKAC

# With broken lines inside... They won't love this, boys.
my $regressionspkac = << 'SPKAC';
MIIBQzCBrTCBnzANBgkqhkiG9w0BAQEFAAOBjQAwgYkCgYEAyeO99lhA2QY1a6YG
7yM3+nOc+lSw2x1Sf0g6xMPxFkO0r20zAl4DxcYJIsciDT/1EwKw/6IZpb0akkZI
unDh4Cw1x9xrvXbvBjCNmZYwAFcr/cx/hZeSH2vjjia5/3dDMrAwsRPsVBft5d+r
iZ1R1dOfIKLyLAJnRPZ8kq0SkVkCAwEAARYJY2hhbGxlbmdlMA0GCSqGSIb3DQEB
BAUAA4GBAL/Icv7uc8rXlbkg9O6UMEzgMKNO0YVSQJUZzlK1nhF+gKOKP/OmMaMC
PaAy4gcqDZladnaLxLODxL28AxP6lYd1u/go9/ipurGMd0BhKBeLvvHHRdWaIEn+
09lbIQLn4IiuvGBQVfF9IeoBv02eQLkfsaMRhHJotmvHCX981p9F
SPKAC

my $csr = new IDX::PKI::Crypto::CSR::PKCS10
	(-form   => {nsComment => "Comme"},
	 -dn     => \@dn,
	 -csr    => $p10);

my $transaction = new IDX::PKI::CertDB::CA ("CA", 1);


test "Sign certificate" => sub {
	my $certificate = $ca->
		_sign_cert (
					"serial"   => $transaction->nextSerial (),
					"req"      => $p10,
					"type"     => "PKCS10",
					"dn"       => \@dn,
					"altnames" => {"email" => 'client@idealx.com'},
					);
	$transaction->addCert ($certificate);
	TEST ($certificate->getSubjectDN() eq $dnstring) ||
		warn "Unexpected DN: ".$certificate->getSubjectDN().
		" instead of $dnstring";
	TEST ($certificate->getEmail() eq 'client@idealx.com') ||
		warn "Unexpected email: ".$certificate->getEmail().
		"instead of client\@idealx.com";
};





test "Multivalued e-mail field" => sub {
	my $cert = $ca->
		_sign_cert (
					"serial"   => $transaction->nextSerial (),
					"req"      => $p10,
					"type"     => "PKCS10",
					"dn"       => \@dn,
					"altnames" => {"email" => [qw (zoinx@idealx.com
												   client@ird.idealx.com)]},
					);
	$transaction->addCert ($cert);
	my @addresses = $cert->getEmail();
	TEST (@addresses == 3) || warn "Unexpected addresses: ".
		Data::Dumper::Dumper (\@addresses);
};

test "SPKAC signature" => sub {
	my $cert = $ca->
		_sign_cert (
					"serial" => $transaction->nextSerial (),
					"req"    => $spkac,
					"type"   => "SPKAC",
					"dn"     => \@bogusdn,
					);
	$transaction->addCert ($cert);
	my %DN = $cert->getSubjectDN();
	TEST ($DN{"Email"} eq 'jxdoe@someorg.com') ||
		warn "Unexpected Email DN chunk: $DN{'Email'} instead of ".
		"jxdoe\@someorg.com";
};

test "Invalid SPKAC" => sub {
	try {
		my $cert = $ca->
			_sign_cert (
						"serial" => $transaction->nextSerial (),
						"req"    => $regressionspkac,
						"type"   => "SPKAC",
						"dn"     => \@bogusdn,
						);
		$transaction->addCert ($cert);
		warn "Invalid SPKAC accepted!";
		TESTFAIL ();
	} catch IDX::PKI::Error with {
		TESTOK ();
	};
};

test "_sign_cert() with objects" => sub {
	my $certificate = $ca->
		_sign_cert (
					"serial" => $transaction->nextSerial (),
					"dn" => \@dn,
					"req" => parse IDX::PKI::Crypto::PKCS10 ($p10),
					"altnames" => {"email" => 'client@idealx.com'},
					);
	$transaction->addCert ($certificate);
	TEST ($certificate->getSubjectDN() eq $dnstring) ||
		warn "Unexpected DN: ".$certificate->getSubjectDN().
		" instead of $dnstring";

	my $cert = $ca->
		_sign_cert (
					"serial" => $transaction->nextSerial (),
					"req"    => parse IDX::PKI::Crypto::SPKAC ($spkac),
					"dn"     => \@bogusdn,
					);
	$transaction->addCert ($cert);
	my %DN = $cert->getSubjectDN();
	TEST ($DN{"Email"} eq 'jxdoe@someorg.com') ||
		warn "Unexpected Email DN chunk: $DN{'Email'} instead of ".
		"jxdoe\@someorg.com";
};

test "Centralized certificate generation" => sub {
	my ($key, $req) = $ca->_generate_private_key (1024, \@dn);
	my $cert = $ca->
		_sign_cert (
					"serial"   => $transaction->nextSerial (),
					"req"      => $req,
					"type"     => "PKCS10",
					"altnames" => {
						"email" => [qw (zoinx@idealx.com
										client@ird.idealx.com)],
						"URI"   => 'http://zoinx.idealx.com/',
						"DNS"   => 'zoinx.idealx.com',
						},
					);
	$transaction->addCert ($cert);
	TEST (UNIVERSAL::isa ($key, "IDX::PKI::Crypto::PrivateKey")) ||
		warn "\$key is not an IDX::PKI::Crypto::PrivateKey object: ".
		ref ($key);
	TEST ($cert->getSubjectDN() eq $dnstring) ||
		warn "Unexpected certificate DN: ".$cert->getSubjectDN().
		" instead of $dnstring";
};

test "Certificate extension flags" => sub {
	my ($key, $req) = $ca->
		_generate_private_key (1024, ["CN" => "Jean Rumain", "O" => "IDEALX",
									  "C" => "FR"]);
	my $cert = $ca->
		_sign_cert (
					"serial"   => $transaction->nextSerial (),
					"req"      => $req,
					"type"     => "PKCS10",
					"altnames" => {"email" => 'testclient@idealx.com'},
					"extensions" => {
						"keyUsage" => [qw(digitalSignature nonRepudiation)],
						"extendedKeyUsage" => [qw(emailProtection clientAuth)],
					},
					);
	$transaction->addCert ($cert);
	my $kus = join (" ", $cert->getExtensions ("keyUsage"));

	TEST ($kus =~ m/nonRepudiation/i) || warn "Didn't find nonRepudiation";
	TEST ($kus !~ m/cipher/i) || warn "Found cipher";

	my $ekus = join (" ", $cert->getExtensions ("extendedKeyUsage",
												"NONCRITICAL"));
	TEST ($ekus =~ m/emailProtection/ && $ekus =~ m/clientAuth/) ||
		warn "extendedKeyUsage: $ekus";

	TEST ($cert->getExtensions ("keyUsage", "CRITICAL")) ||
		warn "keyUsage is not critical";

	TEST (! $cert->getExtensions ("nsCertType")) || warn "Found nsCertType";
};

test "Criticality of extensions" => sub {
	my $cert = $ca->
		_sign_cert (
					"serial"   => $transaction->nextSerial (),
					"req"      => $p10,
					"type"     => "PKCS10",
					"altnames" => {"email" => 'testclient@idealx.com'},
					"extensions" => {
						"keyUsage" => [qw (NONCRITICAL digitalSignature
										   nonRepudiation)],
						"nsCertType" => [qw (CRITICAL NSemail NSobjsign)],
					},
					);
	$transaction->addCert ($cert);

	TEST (! $cert->getExtensions ("keyUsage", "CRITICAL")) ||
					warn "keyUsage is CRITICAL";

	my $ns = join (" ", $cert->getExtensions ("nsCertType", "CRITICAL"));
	TEST ($ns =~ m/NSemail/) || warn "NSemail is not critical";
};

test "Bogus extension" => sub {
	try {
		my $cert = $ca->
			_sign_cert (
						"serial"   => $transaction->nextSerial (),
						"req"      => $p10,
						"type"     => "PKCS10",
						"altnames" => {"email" => 'client@idealx.com'},
						"extensions" => {
							"extendedKeyUsage" => [qw (digitalSignature
													   codeSigning
													   monkeyFarming)],
												   },
						);
		$transaction->addCert ($cert);
		warn "Accepted the bogus extension.";
		TESTFAIL();
	} catch IDX::PKI::Error::Crypto with {
		TESTOK();
	};
};


# warn("You can browse at port ".$server->{port}); sleep(1800);

# This ensures that the server is DESTROY()ed before the temporary files
# (otherwise it won't be killed).
$server = undef;
1;
