#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

package IDX::PKI::CertDB;

use strict;

=pod "

=head1 NAME

B<IDX::PKI::CertDB> - Certificate databases in IDX-PKI (escrow and CA)

=head1 SYNOPSIS

 use IDX::PKI::CertDB;

 my $reader = new IDX::PKI::CertDB::CA ("RA");
 my @mycerts = $reader->searchCerts ('email' => "john*",
 	'valid_at' => "now");

 my $writer = new IDX::PKI::CertDB::CA ("CA");

 my $id = $writer->nextSerial();

 # Insert code here to build a new cert from $id and store it
 # in $newcert.

 $writer->addCert($newcert);
 $writer->setCertAttr ($x509, "revocation", "R");
 my $log = $writer->commit("1/2");
 # First phase in other databases here
 $writer->commit("2/2");
 # Second phase in other databases here

=head1 DESCRIPTION

IDXPKI, just like any other application, needs to store data about its
state somehow. More precisely, the very purpose of a CA in any PKIX
implementation is to be a distributed yet secure database of
certificates and CRLs, thanks to public key cryptography.

There is therefore a generic need in IDXPKI for B<certificate
databases> with the following structural characteristics:

=over 4

=item *

the database schema always features a join with the list of all
certificates emitted by the OpCA;

=item *

there is one privileged writer, which has complete access over the
database, and is typically the PKI entity responsible for generating
the data inside;

=item *

there may be many readers, and they may want to access the database
across the network.

=back

This concept right now has two instantiations, the B<CA database>
which remembers all certificates issued in the past, and their
revocation status; and the B<escrow database> which maps said
certificates to their encrypted private key material (that is, the
encrypted PKCS#12 file). This module provides implementation for both,
in classes B<IDX::PKI::CertDB::CA> and B<IDX::PKI::CertDB::Escrow>;
both are subclasses of the generic B<IDX::PKI::CertDB> which models
the list of properties above.

The added bonus of the encapsulation of all PKI databases in this
module is that it may be decided later to change the implementation of
the PKI databases with very little effort, most notably in order to
scale up IDX-PKI towards the hundreds of thousands of certificates -
see L</IMPLEMENTATION DETAILS>.

=head2 Base class IDX::PKI::CertDB

This class provides the basic functions and methods needed for any
certificate database inside IDX-PKI. It is a virtual class. Use
B<IDX::PKI::CertDB::CA> or B<IDX::PKI::CertDB::Escrow> instead.

B<IDX::PKI::CertDB> boasts transaction support, as one may see in this
interface documentation: in fact, B<IDX::PKI::CertDB> objects are
transactions rather than database handles in the traditional
(e.g. DBI) sense.  An important and very special feature that stems
from this support is that B<all method calls that alter the database's
state in any way, either in the local cache or in the back-end,
respect coherence by rollbacking the transaction upon error>. Any
exception raised by any such method in this class or its descendants
will I<rollback()> the transaction, so that it may not be used anymore
afterwards. This is implemented using some AUTOLOAD trickery in
B<IDX::PKI::CertDB>; to use this feature in your own subclasses, just
rename your I<YourDBClass::yourMethod> to
I<YourDBClass::COHERENT::yourMethod> and this is it!

=cut "

sub AUTOLOAD {
	our $AUTOLOAD;
	return if $AUTOLOAD =~ m/::DESTROY$/;

	no strict "refs";

	my ($pack, $func) = ($AUTOLOAD =~ m/^(.*)::(.*?)$/);

	if (my $funcref = IDX::PKI::CertDB::_mkcoherentwrapper_WalkISA
		($pack, $func)) {
		goto &$funcref;
	} else {
		die "Cannot find $func through package $pack, even through ".
			"COHERENT subclasses";
	}
}

# Recursively walks the ISA tree, trying to find a place whence the
# coherent wrapper can be grafted at (the wrapper is a code snippet
# that does the exception catching and rollbacking).  Returns a
# reference to the newly created function, or undef if it fails.
sub IDX::PKI::CertDB::_mkcoherentwrapper_WalkISA {
	my ($pack, $func) = @_;
	no strict "refs";

	if (my $funcref = UNIVERSAL::can($pack."::COHERENT", $func)) {
		my $coherentwrapper = sub {
			my @return;
			my $wantarray = wantarray();
			eval {
				if ($wantarray) {
					@return = &$funcref(@_);
				} elsif (defined wantarray) {
					$return[0] = &$funcref(@_);
				} else {
					&$funcref(@_);
				}
				1;
			} || do {
				my $SAVEDEXN = $@;
				eval {$_[0]->rollback();};
				die "An error occured in ${pack}::${func} ".$SAVEDEXN;
			};
			return wantarray ? @return : $return[0];
		};
		*{$pack."::$func"} = $coherentwrapper;
		return $coherentwrapper;
	} else {
		SUPERCLASS: foreach my $superclass (@{$pack."::ISA"}) {
			my $ref = _mkcoherentwrapper_WalkISA($superclass, $func);
			return $ref if $ref;
		}
		return undef;
	}
}

=pod "

=over 4

=item I<new ($section, $access, $lockfile)>

Object constructor. I<$section> is the usual configuration section to find
the data from.
I<$access> specifies the level of access requested (C<0> means read
access; C<1> means write access). I<$lockfile> is used to do the
serialization; creating an B<IDX::PKI::CertDB> object locks this file (and
therefore the whole database) with the appropriate kind of lock (reader or
writer). If I<$lockfile> is not set, no file-based locking will be
performed and serialization will rely on the internal mechanisms (if any)
of the underlying database management system.

=cut "

package IDX::PKI::CertDB;

use IDX::PKI::Config;
use IDX::PKI::LDAP qw(quotefilter datefilter matches datenow);
use IDX::PKI::Error;
use IDX::PKI::Debug (0);
use Fcntl qw(:DEFAULT :flock);
use Net::LDAP;
use Net::LDAP::LDIF; # for _toLDIF()
use IO::Scalar; # For _toLDIF() to write into memory, not a file.
use IO::File;

use vars qw(%itf2ldap); # Actually this is a constant; see definition
                        # in IMPLEMENTATION DETAILS below.

sub new {
	my ($class, $section, $access, $lockfile) = @_;

	throw IDX::Error::Internal ("PURE_VIRTUAL")
		if ($class eq "IDX::PKI::CertDB");

	my $config = the IDX::PKI::Config ();
	throw IDX::Error::Internal ("INCORRECT_ARGS")
		unless ($config->SectionExists ($section));

	# Sanity check: Commtype may be UNIX or SSL with netDN.
	my $commtype = $config->getval ($section, "commtype");
	unless (($commtype =~ m/unix/i) ||
			($commtype =~ m/ssl/i && $config->getval ($section, "netDN"))) {
		throw IDX::Error::Config ("CONFIG_INVALID", "-details" =>
									   "commtype may be UNIX or ssl ".
									   "with netDN", "-section" => $section);
	}

	my $self = bless ({
		"access"   => $access,
		"ldap"     => new IDX::PKI::LDAP::CertDB ($section),
		"ldapbase" => $config->getval ("dn", "idxpki_ldapbase"),
	}, $class);

	if (defined $lockfile) {
		$self->{'lockfile'} = $lockfile;
		$self->{'lockfd'} = new IO::File ($lockfile, ($access ? "+<" : "<")) ||
			throw IDX::Error::IO ("Cannot open lock file",
									   "-IOfile" => $lockfile,
									   "-readwrite" => $access);
		# Non-blocking lock.
		flock ($self->{"lockfd"}, LOCK_NB | ($access ? LOCK_EX : LOCK_SH)) ||
			throw IDX::Error::IO ("Cannot lock", "-IOfile" => $lockfile,
									   "-readwrite" => $access);
	}

	return $self;
}

=pod "

=item I<functest()>

Performs a bogus read or read/write access (according to the access
level requested at I<new()> time), and raises appropriate errors upon
failure. This function is *not* a removable unit test, as is it is
meant to be called from the WWW server's initialization sequence (so
that we don't bother making tests or even actually using a PKI that
will not work).

=cut "

sub functest {
	my ($self)=@_;

	my $config = the IDX::PKI::Config ();
	my $base = $config->getval ("dn", "idxpki_ldapbase");
	my $mesg = $self->{ldap}->search
		(
		 "base"   => $base,
		 "scope"  => "base",
		 "filter" => "($base)");
	throw IDX::Error::Network
		("functest() read test failed",
		 "-LDAPbase" => $base,
		 "-LDAPcode" => $mesg->code(),
		 "-LDAPerr"  => $mesg->error(),
		 "-count"    => $mesg->count())
		unless ((! $mesg->code()) && ($mesg->count() == 1));

	if ($self->{access}) {
		my ($root)=$mesg->all_entries();
		$root->add("description","this is a test");
		$root->update($self->{ldap});
		$root->delete("description");
		$root->update($self->{ldap});
	};

}

=pod "

=item I<< _now() >>

This method returns the current date, in "zulu" format
(e.g. "20020404183900Z"). More precisely, it returns the current date
the first time it is called for the current object, and then always
returns the same date. By using this method as a timestamp source,
writer methods ensure that all writes are logged as having occured at
precisely the same time.

=cut "

sub _now {
	my ($self) = @_;
	if ($self->{"now"}) {
		return $self->{"now"};
	} else {
		return ($self->{"now"} = datenow());
	}
}

=pod "

=item I<< searchCerts ('-sizelimit' => 10, $filtername1 => $filterexpr1,
$filtername2 => $filterexpr2, ...) >>

Searches for certificate(s) in the database, according to the supplied
criteria. Returns a list of matching I<IDX::PKI::Crypto::X509>
objects, ordered by serial number, or in scalar context the one with
the highest serial number. One may require a sizelimit with the above
syntax; in this case, at most this many certificates will be returned.

The return value in list context is a list of matching
I<IDX::PKI::Crypto::X509> objects, ordered by serial number, or in
scalar context the one with the highest serial number.

Search filters are specified by pairs of arguments. The first item is
the name of the criterion (it may be preceded by a "!" character to
negate the search on this criterion); the second is the filter to
apply to this criterion (typically the literal value to look for or to
reject, but wildcards are OK for some criteria).

Not all certificates have all criteria inside. For example, there are
certificates without subject DNs, certificates without extensions,
etc. Specifying undef as a search criterion is always valid and only
selects those certificates that have a value in this field, whatever it is.

Available criteria and appropriate filters for each of them are:

=over 4

=item I<serial>

A serial number, in hexadecimal. There is at most one certificate for each
given serial number, and all certificates have a serial number.
Certificate serial number 0 is the OpCA's self-signed certificate, if it
exists (e.g. in CISCO-bugwared IPSEC certification policies).

=item I<DN>

An I<IDX::PKI::DN> object or RFC2253 string. This DN is the one that
has to be present in the certificate (B<not> the DN under which it is
stored in the repository LDAP, that you shouldn't be aware of to begin
with). Not all certificates have a DN: per RFC2459, some have just a
SubjectAltName.

=item I<CN>

The I<CommonName> field of the DN. May contain the "*"
character with its usual wildcard semantics. Again, not all
certificates have a CN - even "user" certificates, even those having a
DN, but one may "reasonably" assume that certificates having a DN and
being "user" certificates have a CN.

=item I<ID>

The "user id" of the certificate.  It may be the DN fields
C<uniqueIdentifier> (nonstandard, OID 2.5.4.45) or C<dnQualifier>
(2.5.4.46), but not necessary (see L<IDX::PKI::Crypto::X509/getUserID>).
I<ID> is B<not> the same as I<serial>: The ID identifies a person while
the serial identifies a certificate.  One person may own several
certificates, either simultaneously or sequentially.

=item I<CA>

B<Not implemented yet>: the DN of the CA that signed this certificate,
in the same format as the I<DN> field above. (right now all
certificates are assumed to belong to the same CA).

=item I<validAt>

An ISO date or date+time (e.g. C<20020521 10:21:00>), or C<now>.
Selecting C<< ('validAt' => "now", '+revocation' => "V") >> selects all
currently active certificates.

=item I<subjectAltName>

The subject alternative name of the certificate, passed as a reference
to a list with two elements. The first is the type of the
subjectAltName field of the matching certificates: "FQDN" (synonymous
to "DNS"), "URI", "UFQDN" (synonymous to "email"), "IPv4", or
"IPv4subnet". The second is the expected content of the subjectAltName
field of the certificates ("*" wildcards are appropriate).

In case of multiple subjectAltNames in one certificate, any of them
matching said name and type will elicit a search hit. As is the case
for single-scalar search criteria, one or both of the name and type
may be specified as undef, meaning a wildcard at this place.

=item I<extension>

An X509v3 keyUsage, extendedKeyUsage or nsCertType extension to be
present in the certificate (see L<IDX::PKI::CA> for a list of
recognized extensions). Certificates will typically have more than one
extension.

As an example, the following search would exclude all server
certificates (which comes in handy for the EE search form):

  $db->searchCerts (..., '!extension' => "NSserver",
 	'!extension' => "serverAuth", ...);

=item I<email>

The email of the certificate, either as a subjectAltName or as a
(nonstandard) component of the DN. "*" wildcards are appropriate.

=item I<+profile>

A profile name. Not all certificates have a profile, even for most
recent versions of IDX-PKI (e.g. admin.p12). The leading "+" in this
criterion name and the following below is meant, and serves to
distinguish attributes that are not in the certificate proper.

=item I<+creationTime>

The date (in ISO format, with resolution to the second, see above) at
which the certificate was created. This field, as well as all date
fields below, may be prefixed by a math comparison operator chosen
among "<", ">", "<=", ">=", "<>" and "=" (see also
L<IDX::PKI::LDAP/datefilter>).

=item I<+modificationTime>

The date (in ISO format, see above) at which the certificate
attributes were last modified.

=item I<+expirationTime>

The expiration date (in ISO format, see above) of the certificate.

=back

B<NOTE:> The astute reader will have noticed that the documentation for
I<searchCerts()> matches L<IDX::PKI::LDAP/searchStrongs> quite closely,
for obvious reasons (except for the ordering of return values, and the
C<+> criteria).  Though, for the sake of encapsulation, the dependency
relationship shall be set up with B<IDX::PKI::CertDB> as the "master" and
B<IDX::PKI::LDAP::CertDB> as the "slave" (so that we do not have to rely
on LDAP forever for the PKI databases).

=item I<< _searchLDAPEntries ($filtername1 => $filterexpr1, $filtername2
=> $filterexpr2, ...) >>

Same function, but returns LDAP entries from the back-end LDAP server
instead.  For internal use only.

=cut "

sub IDX::PKI::CertDB::COHERENT::searchCerts {
	my ($self, @args) = @_;
	my @result = map {IDX::PKI::LDAP::CertDB::entry2cert($_)}
		($self->_searchLDAPEntries(@args));
	@result = sort {
		my ($sa, $sb) = ($a->getSerial(), $b->getSerial());
		$sa =~ s/^0*//;
		$sb =~ s/^0*//;
		(length ($sa) <=> length ($sb)) || ($sa cmp $sb);
	} @result;

	return @result if (wantarray);
	return $result[$#result];
}

sub IDX::PKI::CertDB::COHERENT::_searchLDAPEntries {
	my ($self, @criteria) = @_;
	throw IDX::Error::Internal("WRONG_NUMBER_ARGS") if (@criteria % 2);

	my @LDAPcriteria;
	my @searchargs;

	while (my ($k, $pattern) = splice (@criteria, 0, 2)) {
		unless ($k =~ m/^(\!?)\+(\w+)$/) {
			push (@searchargs, $k, $pattern);
			next;
		}

		my ($neg, $name) = ($1, $2);
		my $ldapfilt;

		if (defined (my $ldapkey = $itf2ldap{$name})) {
			if ($name =~ m/time$/i) {
				$ldapfilt = datefilter ($ldapkey,
										($pattern =~ m/^([<>=!]*)(.*)$/g));
			} else {
				$ldapfilt = sprintf ("(%s=%s)", $ldapkey,
									 quotefilter ($pattern));
			}
		} else {
			throw IDX::Error::Internal("INCORRECT_ARGS");
		}

		push (@LDAPcriteria, ($neg ? "(!$ldapfilt)" : $ldapfilt))
			if ($ldapfilt);

	} # End of loop over function arguments

	my @filterargs = (@LDAPcriteria ?
					  ('-filter' => sprintf ("(&%s)",
											 join ("", @LDAPcriteria))):
					  ());
	DEBUG("LDAP prefilter = ".sprintf("(&%s)",join("",@LDAPcriteria)));

	return $self->{'ldap'}->searchStrongs ('-base' => $self->{'ldapbase'},
										   @filterargs, @searchargs);
}

=pod "

=item I<addCert($certificate)>

=item I<addCert($certificate, $profile)>

This method is only available if write access was selected at new()
time. Schedules addition of a certificate to the database; its initial
state for CA variables is valid ("V") and the specified profile. This
also has the side effect of setting the +creationTime and
+modificationTime certificate attributes (see L</searchCerts>).

All modifications to the database are performed at commit() time (see
L</commit>).

=cut "

sub IDX::PKI::CertDB::COHERENT::addCert {
	my ($self, $cert, $profile) = @_;

	throw IDX::Error::Database ("PERMISSION_DENIED")
		unless ($self->{"access"});
	throw IDX::Error::Database ("WRONG_COMMIT")
		if ($self->{"commitphase"});

	my $id = $cert->getSerial();

	if ((scalar $self->searchCerts("serial" => $id)) ||
		(defined $self->{"cache"}->{$id})) {
		throw IDX::Error::Database("duplicate certificate ID $id");
	}

	my $entry = IDX::PKI::LDAP::CertDB::cert2entry($cert);

	$entry->changetype("add");
	$entry->dn("CN=$id,".$self->{"ldapbase"});

	$entry->replace("idxpkiCertLastModifiedTime", $self->_now());
	if (defined $profile) {
		throw IDX::Error::Internal ("INCORRECT_ARGS",
										 "-profile" => $profile)
			if (ref ($profile) && ref ($profile) ne "ARRAY");
		$entry->replace("idxpkiProfile", $profile);
	} else {
		$entry->delete("idxpkiProfile");
	}

	$self->{"cache"}->{$id} = $entry;
	1;
}

=pod "

=item I<getCertAttr($cert, $attribute)>

=item I<setCertAttr($cert, $attribute, $value)>

Accessors for the C<+> attributes (see L</searchCerts>). I<$cert> is the
B<IDX::PKI::Crypto::X509> certificate to look up. I<$attribute> is the
attribute name, but I<< without the leading C<+> >>.

Setting an attribute is only supported for C<revocation> and C<profile>
right now, and of course only if the B<IDX::PKI::CertDB> object was
created with write access.  Calling I<setCertAttr()> has the side effect
of updating the C<modificationTime> certificate attribute (and also the
C<revocationTime>, if the C<revocation> attribute was accessed - see
L</searchCerts()>).

Transaction isolation level is "serializable".  That is, with
I<getCertAttr()> one gets back the modifications scheduled with
I<setCertAttr()> before they are I<commit()>ed, but doesn't get back any
modifications (even commited) performed by concurrent writers.

If the requested certificate does not exist in the database, an
B<IDX::PKI::Error::Database> is raised.

=cut "

sub IDX::PKI::CertDB::COHERENT::getCertAttr {
	my ($self, $cert, $attribute) = @_;
	my $entry = $self->_getLDAPEntryBySerial ($cert->getSerial());
	throw IDX::Error::Database("NO_MATCH") if (! defined $entry);

	return $entry->get_value($itf2ldap{$attribute});
}

sub IDX::PKI::CertDB::COHERENT::setCertAttr {
	my ($self, $cert, $attribute, $value) = @_;

	throw IDX::Error::Database("PERMISSION_DENIED")
		unless ($self->{"access"});
	throw IDX::Error::Internal ("INCORRECT_ARGS",
								"-details" => "Attribute not allowed",
								"-attribute" => $attribute)
		unless ($attribute =~ m/^(revocation|reason|profile|pkcs12)$/);
	# This is yet another instance of an encapsulation violation for
	# the sake of code simplicity, see %itf2ldap below.

	throw IDX::Error::Database("WRONG_COMMIT") if ($self->{commitphase});

	my $entry = $self->_getLDAPEntryBySerial($cert->getSerial());
	throw IDX::Error::Database("NO_MATCH") if (! defined $entry);

	$entry->replace("idxpkiCertLastModifiedTime", $self->_now());
	$entry->replace("idxpkiCertRevocationTime", $self->_now())
		if ($attribute eq "revocation");
	throw IDX::Error::Internal ("INCORRECT_ARGS", "-value" => $value)
		if (ref ($value) && ref ($value) ne "ARRAY");
	return $entry->replace ($itf2ldap{$attribute}, $value);
}

=pod "

=item I<_getLDAPEntryBySerial($id)>

Fetches an LDAP database entry from the database or from the transaction
cache, with appropriate locking according to the read/write mode set at
I<new()> time, and stores it in the cache.  This is an internal method
used to implement I<getCertAttr()> and I<setCertAttr()> while maintaining
a per-transaction view of the data set: When available, entries from the
cache are used instead of those from the real database.  Returns C<undef>
if the requested ID does not exist.

=cut "

sub IDX::PKI::CertDB::COHERENT::_getLDAPEntryBySerial {
	my ($self, $id) = @_;
	if (exists $self->{"cache"}->{$id}) {
		return $self->{"cache"}->{$id};
	}


	return undef unless (defined
						 ($self->{"cache"}->{$id} =
						  $self->_searchLDAPEntries("serial" => $id)));

	$self->{"cache"}->{$id}->changetype("modify");
	return $self->{"cache"}->{$id};
}

=pod "

=item I<_toLDIF()>

Converts the current state of the transaction to modification LDIF,
decorating it with transaction ordering data. This function will be
used in a future database logging mechanism (not yet designed) and
should not be called directly.

=cut "

sub _toLDIF {
	my ($self)=@_;

	my $str="";
	my $FH=new IO::Scalar(\$str);

	my $LDIFer=new Net::LDAP::LDIF($FH,"a");
	foreach my $entry (values %{$self->{cache}}) {
		$LDIFer->write_cmd($entry);
		print $FH "\n\n";
	};
	return $str;
}

=pod "

=item I<commit()>

=item I<commit("1/2")>

=item I<commit("2/2")>

Performs all modifications registered with I<addCert()> and I<setCertAttr()>.

The calls of I<commit()> with "1/2" and "2/2" perform B<two-phase
commit>. In the first phase, preliminary writes are pushed into the
database and all validity checks are done, so that the database gets
ready to perform either a commit or a rollback and promises success in
both cases. The subsequent call to I<commit("2/2")> actually makes the
changes visible to other processes, while I<rollback()> cancels them
completely. This mechanism is the key to distributed database coherency.

The return value of I<commit()> and I<commit("1/2")> is a piece of
text that precisely describes the modifications made to the database
during this transaction, adequate to be fed to I<replay()>. It
contains full reordering information, so that I<replay()>ing a
shuffled list of logs restores the correct state of the database.

Upon the first call of commit(), all writer methods (such as
L</addCert> and L</setCertAttr>) are disabled. Once the second phase
of commit() is performed, the database handle is rendered unusable and
any subsequent calls to any methods will fail.

=cut "

sub IDX::PKI::CertDB::COHERENT::commit {
	my ($self,$phase)=@_;

	throw IDX::Error::Database("PERMISSION_DENIED") unless
	  ($self->{access});

	if ($phase && $phase eq "1/2") {
		$self->{commitphase}="1/2"; # Write accessors forbidden from now on.
		return $self->functest();
	} elsif ($phase && $phase eq "2/2") {
		throw IDX::Error::Database("WRONG_COMMIT") unless
		  ($self->{commitphase});
	};

	foreach my $entry (values %{$self->{cache}}) {
		$entry->update($self->{ldap});
	};

	close $self->{lockfd} ||
		  throw IDX::Error::IO("Cannot close() the lock file");


	$self->{ldap}->unbind();

	bless $self,"IDX::PKI::CertDB::Commited";

	1;
}

=pod "

=item I<rollback()>

Cancels all pending modifications registered with I<addCert()>,
I<setCertAttr()>, I<setSerial()> and I<nextSerial()>. The database
lock will be released; one can thus use rollback() to properly
relinquish locks in case of woes in of Perl's garbage collection
mechanism, or to ensure equity.

Once rollback() is called, the database handle is rendered unusable and any
subsequent calls to its methods will fail.

=cut "

sub IDX::PKI::CertDB::rollback {
	my ($self)=@_;
	$self->{ldap}->unbind() if (defined $self->{ldap});
	close $self->{serialfd} if (defined $self->{serialfd});

	bless $self,"IDX::PKI::CertDB::Rollbacked";
}

#######################################################################
#######################################################################
#######################################################################

=pod "

=back

=head2 CA database: B<IDX::PKI::CertDB::CA>

All public-key CAs need some amount of on-disk storage in order to do
their job. X509v3 CAs, in particular, need the following:

=over 4

=item *

a serial number to uniquely distinguish certificates from each other
(this is the file named "serial" in openssl). This is implemented using
a text file (which means that network writes are not possible yet);

=item *

the list of certificates already issued; this is "index.txt" in
openssl, and is implemented using the ancestor class
B<IDX::PKI::CertDB>;

=item *

A `join' between the certificates above and some ancillary data, such
as the revocation state, the profile in which they were created, etc
(the list may be enriched for some certification policies).

=back

This class provides support for all of these, both for reading and
writing.  Reading may also occur over the network (typically from a
"repository" LDAP server), while writing may require to
actually be the CA.

=over 4

=item I<new ($base_section, $access)>

=item I<new ()>

Object constructor.  Identical to the I<new> constructor in the parent
class, except that:

=over 4

=item *

The real C<LDAP-CLIENT-*> section is computed from the
I<[$base_section]/repository_ldap> parameter;

=item *

I<$section> and I<$access> may be omitted (I<$section> defaults to C<CA>
and I<$access> to C<0>);

=item *

The lockfile is deduced from the configuration (from entry
C<[CA]/database_dir>).

=back

=cut "

package IDX::PKI::CertDB::CA;

use strict;
use IDX::PKI::Config;
use IDX::PKI::Error;
use IO::File;

our @ISA = qw(IDX::PKI::CertDB);

sub new {
	my ($class, $section, $access) = @_;
	$section ||= "CA";

	my $config = the IDX::PKI::Config ();
	my $dbdir = $config->val ($section, "database_dir");
	my $subsection = $config->getSectionName ($section, ["LDAP-CLIENT" =>
														 "repository_ldap"]);
	throw IDX::Error::Config ("CONFIG_INVALID",
								   "-details" => "Missing parameter",
								   "-section" => $section,
								   "-indirection" => "repository_ldap")
		unless (defined $subsection);

	my $self = IDX::PKI::CertDB::new ($class, $subsection, $access,
									  ($dbdir ? ("$dbdir/serial") : ()));

	$self->{"serialfd"} = $self->{"lockfd"};
	$self->{"ldapbase"} = "OU=certificate,".
		$config->getval ("dn", "idxpki_ldapbase");

	return $self;
}

=pod "

=item I<initdb ($section)>

This is a class method (B<not> a method) that creates the whole database
in case it does not exist yet (as determined by the existence of the lock
file). The configuration section I<$section> must provide appropriate
write access to the database.

=cut "

sub initdb {
	my ($class, $section) = @_;

	$section ||= "CA";
	my $config = the IDX::PKI::Config ();
	my $cadir = $config->getPath ($section, "database_dir");
	throw IDX::Error::Config ("Must be local to initdb()")
		unless (defined $cadir);

	if (-f (my $file = "$cadir/serial")) {
		throw IDX::Error::Database ("PERMISSION_DENIED",
									-IOfile=>$file);
	}

	my $FD = new IO::File ("$cadir/serial", ">");
	throw IDX::Error::IO ("Cannot create",
							   "-IOfile" => "$cadir/serial")
		unless (defined ($FD));

	my $self = $class->new ($section, 1); # Locks file at the same time.

	($FD->print("01\n") && $FD->close()) ||
		throw IDX::Error::IO ("Cannot write",
								   "-IOfile" => "$cadir/serial");

	try {
		$self->{ldap}->delete ("dn" => $self->{ldapbase});
	} otherwise {};

	$self->{ldap}->add ("dn" => $self->{ldapbase},
						"attr" =>["objectClass" => [qw(top
													   organizationalUnit)],
								  "organizationalUnitName" => "certificate",
								  ]);
}

=pod "

=item I<searchCerts()>

The I<searchCerts()> function is overloaded in B<IDX::PKI::CertDB::CA>. It
is exactly similar to the one in B<IDX::PKI::CertDB> above, except that
it accepts the following extra criteria:

=over 4

=item I<+revocation>

Either "V" (Valid) or "R" (Revoked permanently). More letters may be
affected in the future for disabled ("temporarily revoked") certificates.

=item I<+revocationTime>

The date (in ISO format, see above) at which the certificate
was revoked, to appear in the CRL.

=back

=item I<getCertAttr()>

=item I<setCertAttr()>

Those functions are overloaded too, in order to handle the two extra
certificate attributes above.

=cut "


# VOID IMPLEMENTATION - this is in fact done directly in
# IDX::PKI::CertDB::searchCerts(). See %IDX::PKI::CertDB::itf2ldap below
# for an explanation.

=pod "

=item I<addCert()>

This method is the same as I<addCert()> in the superclass, except it
sets the initial revocation state of the certificate to "V".

=cut "

sub IDX::PKI::CertDB::CA::COHERENT::addCert {
	my ($self, $cert, $profile) = @_;

	$self->IDX::PKI::CertDB::addCert ($cert, $profile);

	$self->{"cache"}->{$cert->getSerial()}->replace("idxpkiStatus", "V");

	1;
}


=pod "

=item I<nextSerial()>

Returns an appropriate value for the next certificate ID, and
simultaneously increments it. This is a writer method, which thus
requires the B<IDX::PKI::CertDB::CA> object to have been created with write
access to the database.

=cut "

sub IDX::PKI::CertDB::CA::COHERENT::nextSerial {
	my ($self)=@_;

	throw IDX::Error::Database("PERMISSION_DENIED") unless
	  ($self->{access});
	throw IDX::Error::Database("WRONG_COMMIT") if ($self->{commitphase});

	if (! $self->{serial}) {
		$self->_getSerial();
	} else {
#		$self->{serial} = sprintf ("%X", hex ($self->{serial}) + 1);
		$self->{serial} = sprintf ("%X", hex ($self->{serial}) + rand(100));
		# Openssl wants an even number of chars.
		$self->{"serial"} = "0".$self->{"serial"}
			if (length ($self->{"serial"}) % 2);
	}
	return $self->{serial};
}

=pod "

=item I<lastSerial()>

Returns the last ID deliverd by nextSerial() or set with setSerial(),
any one of which must have been called first lest an exception be
raised. B<This is a writer method>; requesting an ID has no meaning
outside a write lock of the database.

=cut "

sub IDX::PKI::CertDB::CA::COHERENT::lastSerial {
	my ($self)=@_;

	throw IDX::Error::Database("PERMISSION_DENIED") unless
	  ($self->{access});
	throw IDX::Error::Database("WRONG_COMMIT") if ($self->{commitphase});

	throw IDX::Error::Internal("UNKNOWN_SERIAL") unless
	  (defined $self->{serial});
	return $self->{serial};
}

=pod "

=item I<setSerial($serial)>

Sets the current ID w.r.t. lastSerial() (so that the next call to
nextSerial() will return $serial + 1). Be warned that the serial
number is a pure sequence, in SQL style: if you setSerial() wrongly,
you may be returned IDs that already assigned to certificates (this
will still be caught by L</addCert()> though).

=cut "

sub IDX::PKI::CertDB::CA::COHERENT::setSerial {
	my ($self,$serial)=@_;

	throw IDX::Error::Database("PERMISSION_DENIED") unless
	  ($self->{access});
	throw IDX::Error::Database("WRONG_COMMIT") if ($self->{commitphase});

	throw IDX::Error::Internal ("INCORRECT_ARGS")
		unless ($serial =~ m/^[0-9a-f]+$/i);

	$self->_getSerial() unless defined $self->{serial};
	$self->{serial} = uc ($serial);
}

=pod "

=item I<_getSerial()>

Reads the current serial number from disk, but do not return
it. Internal use only.

=cut "

sub IDX::PKI::CertDB::CA::COHERENT::_getSerial {
	my ($self)=@_;
	seek($self->{serialfd},0,0) ||
	  throw IDX::Error::IO("Cannot seek()");
	local $/="\n";
	$self->{serial}=$self->{serialfd}->getline(); chomp $self->{serial};
	throw IDX::Error::Security("No serial in the file?!")
		unless ($self->{serial} =~ m/^[0-9a-f]+$/i);
	$self->{serial} = uc ($self->{serial});
	# Respect OpenSSL's convention: the file holds the *next* value to
	# put in a certificate.
	$self->{origserial}=$self->{serial};
	1;
}


=pod "

=item I<_toLDIF()>

This function is overloaded from the superclass and adds the past and
current serial number as LDIF comments.

=cut "

sub _toLDIF {
	my ($self)=@_;

	my $str=$self->IDX::PKI::CertDB::_toLDIF();
	$str=(<<"SERIALDATA").$str if (defined $self->{serial});
# origserial: $self->{origserial}
# serial: $self->{serial}
SERIALDATA
	return $str;
}

=pod "

=item I<commit()>

This function is overloaded from the superclass and records the
changes to the serial number made with I<setSerial()> and I<nextSerial()>
along with the other changes.

=cut "

sub IDX::PKI::CertDB::CA::COHERENT::commit {
	my ($self,$phase)=@_;
	if ( (!$phase || ($phase eq "2/2" && $self->{commitphase}))
		 && (defined $self->{serial}) ) {
		# Respect OpenSSL's convention.
		$self->{serial} = sprintf ("%X", hex ($self->{serial}) + rand(100));
		# Openssl wants an even number of chars.
		$self->{"serial"} = "0".$self->{"serial"}
			if (length ($self->{"serial"}) % 2);
		my $newserial = sprintf ("%X", hex ($self->{serial})+1);
		# Openssl wants an even number of chars.
		$newserial = "0".$newserial if (length ($newserial) % 2);
		seek($self->{serialfd},0,0) ||
		  throw IDX::Error::IO("Cannot seek()");
		truncate($self->{serialfd},0) ||
		  throw IDX::Error::IO("Cannot truncate()");
		$self->{serialfd}->print("$newserial\n") ||
		  throw IDX::Error::IO("Cannot write() the serial");
	};
	return $self->IDX::PKI::CertDB::commit($phase);
}


# VOID IMPLEMENTATION - this is in fact done directly in
# IDX::PKI::CertDB::getCertAttr(). See %IDX::PKI::CertDB::itf2ldap below
# for an explanation.


#######################################################################
#######################################################################
#######################################################################

=back

=head1 IMPLEMENTATION DETAILS


CertDB is implemented with an LDAP database (for giving read access to
everyone over the network) and a flat file for the serial number (for
the CA only). Additionnally, and for increased recovery abilities, a
log of all write accesses to the database is maintained as a flat text
file (not implemented yet). When using "openssl ca" to sign new
certificates or CRLs, we make copies of the data maintained by the
CertDB::CA (while holding a read lock), invoke openssl, then use the
appropriate CertDB::CA methods to record the changes; the use of
openssl's own database mechanisms is B<forbidden> (e.g. do B<not> use
the serial number file from this module as a serial file in openssl!).

Transactions are faked using an "I'll write down all this stuff later"
strategy, and a file lock. This will be changed in some (remote!)
future in favor of a full-text (LDIF) CA database, with non-LDAP
remote read accessors (such as FTP).

The expected syntax of the configuration file sections is as follows:

 [readaccess]
 ; This is a pointer of type LDAP-CLIENT to another section
 ; (consult IDX::PKI::LDAP manpage)
 repository_ldap=repository-readonly

 [writeaccess]
 ; Directory for the flat-file CertDB (e.g. the lockable serial file)
 database_dir=/var/idxpki/secure/CA/DB/
 repository_ldap=repository-readwrite

The [writeaccess] section is often [CA] or [escrow], since mostly only
them need write access to the databases.

The structure of the LDAP server is a flat list of certificates identified
by their serial number, as follows (the C<O=IDXPKI> root is in fact taken
from the configuration as we MUST ensure coherence):

    O=IDXPKI
       |
       +---- OU=certificate
       |           |
       |           +-- idxpkiSerial=1
       |           |
       |           +-- idxpkiSerial=2
       |           |
       |           +-- idxpkiSerial=3
       |
       +---- OU=pkcs12
                   |
                   +-- idxpkiSerial=1
                   |
                   +-- idxpkiSerial=2
                   |
                   +-- idxpkiSerial=3

(both branches can, but need not, be on the same LDAP server)

The certificates are stored in class idxpkiStrong (see L<IDX::PKI::LDAP>).

=over 4

=item I<%IDX::PKI::CertDB::itf2ldap>

Private hash holding the correspondence between the "+" field names
above and the relevant field names in the IDXPKI schema (see also
sources/src/etc/ldap/idxpki.schema). This hash helps implementing
I<_searchLDAPEntries> as a wrapper around
L<IDX::PKI::LDAP/searchEntries> and is probably useless outside this module.

=cut "

package IDX::PKI::CertDB;

# Actually not all fields are useful in all classes. But being able to
# search a certificate in the escrow database by revocation time
# is just useless, not harmful. So we break the encapsulation here
# for simplicity of the code: contrary to what is stated in the
# POD, all search criteria and all getCertAttr()s would work for all classes
# (although, of course, taking advantage of this in user code is forbidden
# since this would break for other database backends that use a tighter
# schema paradigm, such as SQL).
our %itf2ldap = (
				 "profile" => "idxpkiProfile",
				 "reason" => "idxpkiReason",
				 # Same as publication time (sounds very reasonable but maybe
				 # we will want backwards-dated publication for bootstrapping
				 # someday):
				 "creationTime" => "idxpkiCertPublicationTime",
				 "modificationTime" => "idxpkiCertLastModificationTime",

				 "revocation" => "idxpkiStatus",
				 "revocationTime" => "idxpkiCertRevocationTime",
				 "expirationTime" => "idxpkiCertExpirationTime",

				 # Only for getCertAttr() / setCertAttr()
				 # According to inetorgperson.schema: ";binary" transfer
				 # should NOT be used as syntax is binary.
				 "pkcs12" => "userPKCS12",
				);

=pod "

=back

=head1 BUGS

The following still need to be designed and/or implemented:

=over 4

=item *

You can't open more than one connection to the same database at a time
(even read-only), because of locking.

=item *

Real two-phase commit: Record half-finished transactions to medium, add a
method to list half-finished transactions as newly constructed
B<IDX::PKI::CertDB> objects that one may either I<< ->commit("2/2") >> or
I<< ->rollback() >>.

=item *

Persistent database logging (warning: A database log is not the same thing
as babbling about your life into some random syslog), and accessors to
same.  This in turn requires maintaining the transaction ID.

=item *

Multiple CAs stored in one LDAP directory, while still allowing
cross-CA searches (requires re-engineering some of the API since the
serial number would no longer be unique)

=back

commit("1/2") merely performs functest().  This doesn't capture
non-systemic write failures such as disk space shortage, but is still
far better than nothing. LDAP cannot do much about this anyway.

Writes made in the current transaction but not yet commited are
oblivious to the searchCerts() and searchEntries() functions. This
problem is a pain to solve (requires re-engineering the LDAP filtering
mechanism to make it available on the Perl side as well).

=cut "

#################### TEST SUITE #################################

require IDX::Test::Suite unless caller();
1;

__END__

######## Preparing tests

use strict;

use IDX::PKI::Config;
use IDX::Test;
use IDX::Test::LDAPServer;
use IDX::Error;
use IO::Socket::SSL;
use Fcntl qw(:DEFAULT :flock);

# All tests are integration tests.
use IDX::PKI::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}

my $numberOfTests = 19;       # Public IDX-PKI version.

number_of_tests ($numberOfTests);

use vars qw($server $CAdir $config $lockfile);
$config = the IDX::PKI::Config ();

our $org = "TestPKI";
our $rootdn = "O=$org";
$server = new IDX::Test::LDAPServer::Stateful(
	<<"LDIF", $rootdn)->start();
dn: $rootdn
objectClass: top
objectClass: organization
organizationName: $org
LDIF

my $cadir = new IDX::Test::Directory ();
{
	my ($cacert, $rootcacert) = (IDX::Test::certificates ())[0,6];
	my ($cahash, $rootcahash) = map {IDX::Test::getCertHash ($_)}
		($cacert, $rootcacert);
	local *FILE;
	open (FILE, ">", "$cadir/$cahash.0") ||
		die "Can't write to $cadir/$cahash.0: $!";
	print FILE $cacert;
	close (FILE);
	open (FILE, ">", "$cadir/$rootcahash.0") ||
		die "Can't write to $cadir/$rootcahash.0: $!";
	print FILE $rootcacert;
	close (FILE);
}

our $SSLserver;

# Keep some values from the "real" config.
my $withopenssl = $config->getPath ("paths", "openssl");

sub reconfig {

	$CAdir=new IDX::Test::Directory();

	$SSLserver=new IDX::Test::SSLServer(-server=>$server)->start();
	my $dn=IDX::Test::getCertDN($SSLserver->{mycert});

	$lockfile="$CAdir/serial"; # Let's take this even for pure (non-CA) CertDB
	# and escrow tests.

	$config = set IDX::PKI::Config (<<"CONFIG");
[paths]
openssl=$withopenssl
cacertsdir=$cadir

[dn]
idxpki_ldapbase=$rootdn

[EE]
repository_ldap=repository

[TOURIST]
repository_ldap=tourist

[CA]
repository_ldap=repository
database_dir=$CAdir

[LDAP-CLIENT-repository]
host=localhost
port=$SSLserver->{port}
commtype=ssl
netDN=$dn

[LDAP-CLIENT-tourist]
host=localhost
port=$SSLserver->{port}
commtype=ssl

[escrow]
lockfile=$lockfile
repository_ldap=repository
CONFIG

	if ($server->isa("IDX::Test::LDAPServer::InPerl")) {
		local *TOUCH;
		open(TOUCH,">",$lockfile); close(TOUCH);

	} else {
		IDX::PKI::CertDB::CA->initdb();
	};
}

sub bogusanswer {
        local $_=shift;
        return undef unless m/^SEARCH$/m;
        return undef unless (m/^base: (.*)$/m); my $base=$1;

        return (<<"BOGUSDATA".IDX::Test::LDAPServer::result());
dn: $rootdn
objectClass: top
objectClass: organizationalUnit
organizationalUnitName: $org

BOGUSDATA
}

######## CertDB::CA tests

do {
	local $server = new IDX::Test::LDAPServer::InPerl
		(\&bogusanswer, $rootdn)->start();
	reconfig();

	$DB::single=1;

	test "double initdb() (must fail)" => sub {
		try {
			IDX::PKI::CertDB::CA->initdb();
			TESTFAIL();
		} catch IDX::Error::Database with {
			TESTOK();
		};
	};

	test "new() and functest() on a read-only server" => sub {
		my $db = new IDX::PKI::CertDB::CA("EE");
		$db->functest();
		TESTOK();
	};

	test "new() and functest() read-write on a read-only server" => sub {
		my $db = new IDX::PKI::CertDB::CA("CA", 1);
		try {
			$db->functest();
			TESTFAIL();
		} catch IDX::Error::Network with {
			TESTOK();
		};
	};

}; # end-of-scope for the InPerl LDAP server
reconfig();

test "server authentication is mandatory (OT_TRANS_DATA)" => sub {
	try {
		$DB::single=1;
		my $db = new IDX::PKI::CertDB::CA("TOURIST", 0);
		TESTFAIL();
	} catch IDX::PKI::Error::Config with {
		TESTOK();
	};
};

my $db;
test "new() and functest() for the EE" => sub {
	$db = new IDX::PKI::CertDB::CA ("EE", 0);
	$db->functest();
	TESTOK();
};

test "new() and functest() for an undefined entity with write access" => sub {
	$db = new IDX::PKI::CertDB::CA (undef, 1);
	$db->functest();
	TESTOK();
};

test "serial is locked" => sub {
	local *LOCK;
	open(LOCK,"<",$lockfile);
	use Errno qw(EAGAIN);
	TEST(! flock(LOCK,LOCK_SH|LOCK_NB));
	TEST($! == EAGAIN);
};

test "searchCerts (void)" => sub {
	my @certs = $db->searchCerts ("serial" => undef);
	TEST(! @certs);
};

test "addCert" => sub {
	require IDX::PKI::Crypto::X509;
	my ($cert1, $cert2) = map {scalar (parse IDX::PKI::Crypto::X509 ($_))}
		(IDX::Test::certificates ())[1,3];
	$db->addCert($cert1);
	$db->addCert($cert2);
};

# Could test concurrent transactions here, but this makes little sense with the
# global locking scheme.

test "_toLDIF for adds" => sub {
	TEST($db->_toLDIF() !~ m/changetype.*modify/);
};

test "commit" => sub {
	$db->commit();
	try {
		$db->searchCerts ("serial" => undef);
		TESTFAIL;
	} otherwise {
		TESTOK;
	};
};

my $thecert;
test "searchCerts" => sub {
	$db = new IDX::PKI::CertDB::CA (undef, 1);
	my @certs = $db->searchCerts ("serial" => undef);
	TEST(@certs == 2) || warn "Did not get 2 certs with a serial: ".
		Data::Dumper::Dumper (@certs);

	@certs = $db->searchCerts ("CN" => "Administrator");
	TEST (@certs == 1) || warn "Didn't find 1 cert :".
		Data::Dumper::Dumper (@certs);
	$thecert = $certs[0];
	my $cert = parse
		IDX::PKI::Crypto::X509 ((IDX::Test::certificates ())[1]);
	TEST ($thecert->getSerial() eq $cert->getSerial ()) ||
		warn "Unexpected serial ".$thecert->getSerial();
};

test "searchCerts by DN" => sub {
	my $cert1 = parse IDX::PKI::Crypto::X509 ((IDX::Test::certificates ())[1]);

	use IDX::PKI::DN;
    my @certs = $db->searchCerts (DN=>
								  build IDX::PKI::DN($cert1->getSubjectDN()));
	TEST(@certs == 1) || warn @certs;
    @certs = $db->searchCerts (DN=>scalar($cert1->getSubjectDN()));
	TEST(@certs == 1) || warn @certs;
};



my $revoktime;
my $serial;
test "access to the certificate attributes" => sub {
	TEST($db->getCertAttr($thecert,"revocation") eq "V") ||
		die $db->getCertAttr($thecert,"revocation");
	TEST(!defined $db->getCertAttr($thecert,"revocationTime")) || die
		"already a revocationTime?";
	my @valids = $db->searchCerts ("+revocation" => "V");
	TEST (scalar (@valids) == 2) || warn "Number of valid certs found: ".
		scalar (@valids);
	my @notrevoked = $db->searchCerts ("!+revocation" => "R");
	TEST (scalar (@notrevoked) == 2) || warn "Number of not revoked certs ".
		"found: ".scalar (@notrevoked);

	$db->setCertAttr($thecert,"revocation","R");
	TEST(defined($revoktime=$db->getCertAttr($thecert,"revocationTime")));
	TEST($revoktime =~ m/^\d+Z$/);
	TEST($revoktime gt $db->getCertAttr($thecert,"creationTime")) ||
		die "$revoktime is not gt ".$db->getCertAttr($thecert,"creationTime");
};

test "serial number" => sub {
	my $lastserial = $db->nextSerial();
	my $serial     = $db->nextSerial();
	TEST (hex ($lastserial) < hex ($serial)) ||
		die "nextSerial is not greater than current serial: $serial vs ".
		"$lastserial";
	TEST (hex ($serial) == hex ($db->lastSerial())) ||
		die "Can't find back $serial: ".$db->lastSerial();
};

# This is an OpenSSL requirement for some operations... This test may
# become obsolete some day.
test "Serial numbers have an even number of characters" => sub {
	for (my $i = 0; $i <= 20; $i ++) {
		my $serial = $db->nextSerial ();
		TEST (! (length ($serial) % 2)) || warn "Bad serial $serial";
	}
};

test "cache state" => sub {
	my ($entry)=values %{$db->{cache}};
	TEST($entry->changetype() eq "modify");
};

test "_toLDIF for updates" => sub {
	my $log = $db->_toLDIF();
	TEST($log =~ m/changetype.*modify/) || warn $log;
	TEST($log =~ m/^# serial:/im) || die $log;
};

test "remanence" => sub {
	$db->commit();
	$db = new IDX::PKI::CertDB::CA (undef, 1);
	TEST($revoktime eq $db->getCertAttr($thecert,"revocationTime"));
	TEST("R" eq $db->getCertAttr($thecert,"revocation"));
};

$db->rollback();
unlink($lockfile);


# This ensures that the servers are DESTROY()ed before the temporary files
# (otherwise they won't be killed).
$server = undef;
$SSLserver = undef;
1;
