#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::PKI::Config> - IDX-PKI configuration management tools

=head1 SYNOPSIS

Do this first in mod_perl and every autonomous (non-CGI) Perl script:

 use IDX::PKI::Config;

=head1 DESCRIPTION

This package is a subclass of I<IDX::Config>. As described in
L<IDX::Config/Subclassing is mandatory>, it does exactly the same
thing as I<IDX::Config> application-wise. It only serves as a
repository for the default configuration, which is computed from the
config.status and then inserted into this module as extra code at
install time.

See L<IDX::Config> for all information about how to use this class in
your Perl code.

=cut "

package IDX::PKI::Config;
use strict;

use base 'IDX::Config';
use IDX::PKI::Error;

=pod "

=head1 METHODS

=over 4

=item I<bootstrap_configpath (%CONFIG)>

Class method overloaded from B<IDX::Config> (see
L<IDX::Config/bootstrap_configpath()>).

=cut "

sub bootstrap_configpath {
	my ($class, %CONFIG) = @_;
	return $CONFIG{"CONFIGDIR"}."/idxpki.conf";
}

=pod "

=item I<bootstrap_config (%CONFIG)>

Overloaded from B<IDX::Config> to add IDX-PKI specific configuration
parameters.

=cut "

sub bootstrap_config {
	my ($class, %CONFIG) = @_;
	my $config = $class->SUPER::bootstrap_config (%CONFIG);

	my $bootstraptime = "\t;;;;;;;;;; Should be set at bootstrap time";

	# DNs.
	$config->AddSection ("dn");
	$config->SetSectionComment ("dn", "IDX-PKI reserved DNs");
	foreach my $entity (grep {m/.*_CN/} keys (%CONFIG)) {
		$config->newval ("dn", lc ($entity), $CONFIG{$entity})
			if ($CONFIG{$entity});
	}
	$config->newval ("dn", "idxpki_subdn", $CONFIG{"IDXPKI_SUBDN"});
	$config->newval ("dn", "idxpki_ldapbase", "O=IDX-PKI");
	$config->SetParameterComment ("dn", "idxpki_ldapbase",
								  "Suffix for internal-use LDAPs");
	$config->newval ("dn", "rootdn", $bootstraptime);
	$config->SetParameterComment ("dn", "rootdn", "Bootstrap-configured root ".
								  "DN (for IDX-PKI certificates)");

	# Entities certificates.
	# XXX TODO: Currently only the CA certificate is supposed to be
	# available everywhere. We will have to find a way to get entities
	# certificates which is compatible with multi-CA installations.
	$config->AddSection ("certificates");
	$config->SetSectionComment ("certificates",
								"IDX-PKI entities certificates");
	{
		my $cn = $config->getval ("dn", "ca_cn");
		$cn =~ s/[^a-z0-9]/_/ig;
		$config->newval ("certificates", "CA",
						 $config->getval ("paths", "cacertsdir")."/$cn.pem");
	}

	# Vanilla or full-featured.
	$config->newval ("_components", "full_featured",
					 ($CONFIG{"FULL_FEATURED"} ? 1 : 0));

	# Various configuration options needed by IDX-PKI...
	my $whoami;
	my $localhost;
	if ($config->val ("_components", "test")) {
		$whoami= getpwuid ($>);
		chomp ($localhost = `hostname -f`);
	} else {
		$whoami = "root";
		$localhost = "localhost";
	}

	# Some useful values.
	my $vardir        = $config->getval ("paths", "vardir");
	my $securedir     = $config->getval ("paths", "securedir");
	my $configdir     = $config->getval ("paths", "configdir");
	my $sbindir       = $config->getval ("paths", "sbindir");
	my %components    = map { $_, $config->val ("_components", $_ ); }
	  ( "ee", "ra", "ca", "publish" );
	my $section;

	####################  BASE CONFIGURATION  ####################
	# Logs.
	$section = "log";
	$config->AddSection ($section);
	$config->SetSectionComment ($section, ";;;;;;;;;;;;;;;;;;;;  GLOBAL SETUP".
								"  ;;;;;;;;;;;;;;;;;;;;");
		$config->newval ($section, "type", "syslog");
		$config->newval ($section, "facility", "daemon");
		$config->newval ($section, "logopt", "pid");

	# Web server.
	$section = "webserver";
	$config->AddSection ($section);
	$config->newval ($section, "restart", 1);
	$config->SetParameterComment ($section, "restart", "If true, Apache will ".
								  "be restarted when getting a new CRL.");

	# Root CA.
	$section = "rootCA";
	$config->AddSection ($section);
	$config->SetSectionComment ($section, ";;;;;;;;;;;;;;;;;;;;  PRINCIPALS  ".
								";;;;;;;;;;;;;;;;;;;;");
	$config->newval ($section, "crlurl", $bootstraptime);
	$config->newval ($section, "default_keysize", $CONFIG{"CA_KEYSIZE"});

	# Operational CA.
	$section = "CA";
	$config->AddSection ($section);
	$config->SetSectionComment
		($section, "key, certificate, crl and crlurl must be present even ".
		 "if we are not the CA");
	$config->newval ($section, "key", "$securedir/CA/OpCA.key");
	$config->newval ($section, "certificate", "$securedir/CA/OpCA.crt");
	{
		# my $cn = $config->getval ("dn", "ca_cn");
		my $cn = $CONFIG{"CA_CN"};
		$cn =~ s/[^a-z0-9]/_/ig;
		$config->newval ($section, "crl",
						 $config->getval ("paths", "crldir")."/$cn.pem");
	}
	$config->newval ($section, "crlurl", $bootstraptime);
	$config->newval ($section, "web_acl", "CA");
	$config->SetParameterComment ($section, "web_acl", "Type: ACL");
	$config->newval ($section, "database_dir", "$securedir/CA");
	$config->newval ($section, "default_keysize", $CONFIG{"KEYSIZE"});
	$config->newval ($section, "default_validity", $CONFIG{"VALIDITY"});
	$config->newval ($section, "default_crl_validity",
					 $CONFIG{"CA_CRL_VALIDITY"});

	$config->newval ($section, "repository_ldap", "repository-caaccess");
	$config->SetParameterComment ($section, "repository_ldap",
								  "Type: LDAP-CLIENT");
	$config->newval ($section, "mode", "auto");

	$config->newval ($section, "admin_email", "$whoami\@$localhost");
	$config->newval ($section, "waiting_dir", "$securedir/CA/waiting");
		$config->newval ($section, "central", "user");
		$config->SetParameterComment ($section, "central",
									  "user, admin");
		$config->newval ($section, "escrow", "none");

	# RA.
	$section = "RA";
	$config->AddSection ($section);
	$config->newval ($section, "web_acl", "RA");
	$config->SetParameterComment ($section, "web_acl", "Type: ACL");
	$config->newval ($section, "key", "$securedir/RA/RA.key");
	$config->newval ($section, "certificate", "$securedir/RA/RA.crt");
	$config->newval ($section, "repository_ldap", "repository-readonly");
	$config->SetParameterComment ($section, "repository_ldap",
								  "Type: LDAP-CLIENT");
	$config->newval ($section, "mode", "manual");
	$config->newval ($section, "max_search_count", 20);

	# EE.
	$section = "EE";
	$config->AddSection ($section);
	$config->newval ($section, "key", "$securedir/EE/EE.key");
	$config->newval ($section, "certificate", "$securedir/EE/EE.crt");
	$config->newval ($section, "web_acl", "CA");
	$config->SetParameterComment ($section, "web_acl", "Type: ACL");
	$config->newval ($section, "p12_url", $bootstraptime);
	$config->newval ($section, "repository_ldap", "repository-readonly");
	$config->SetParameterComment ($section, "repository_ldap",
								  "Type: LDAP-CLIENT");
	$config->newval ($section, "admin_email", "$whoami\@$localhost");
	$config->newval ($section, "max_search_count", 20);
	$config->newval ($section, "profiles", "standard", "central", "server");
	$config->SetParameterComment ($section, "profiles", "Type: PROFILE");

	# Publication module.
	$section = "publish";
	$config->AddSection ($section);
	$config->SetSectionComment ($section, "Publication module.");
	$config->newval ($section, "ee_url", $bootstraptime);
	$config->SetParameterComment ($section, "ee_url",
								  "Parameters for generating URLs in emails");


	# WWW-based configuration module.
	$section = "config";
	$config->AddSection ($section);
	$config->SetSectionComment ($section, "WWW-based configuration module.");
	$config->newval ($section, "web_acl", "CA");
	$config->SetParameterComment ($section, "web_acl", "Type: ACL");
	$config->newval ($section, "repository_ldap", "repository-readonly");
	$config->SetParameterComment ($section, "repository_ldap",
								  "Type: LDAP-CLIENT");

	# Renewal module.
	$section = "renewal";
	$config->AddSection ($section);
	$config->newval ($section, "web_acl", "CA");
	$config->SetParameterComment ($section, "web_acl", "Type: ACL");
	$config->newval ($section, "repository_ldap", "repository-readonly");
	$config->SetParameterComment ($section, "repository_ldap",
								  "Type: LDAP-CLIENT");
	{
		my @array = (15, 7, 2);
		my (@days, @active, @textfile);
		for (my $i = 1; $i <= scalar (@array); $i ++) {
			push (@days, $array[$i-1]);
			push (@active, 1);
			push (@textfile, "$configdir/renewal_message-${i}.conf");
		}
		$config->newval ($section, "days", @days);
		$config->newval ($section, "active", @active);
		$config->newval ($section, "textfile", @textfile);
	}

	####################  USERS ACLS  ####################
	$section = "ACL-CA";
	$config->AddSection ($section);
	$config->SetSectionComment ($section, ";;;;;;;;;;;;;;;;;;;;  USERS ACLS".
								"  ;;;;;;;;;;;;;;;;;;;;");
	$config->newval ($section, "can-change-privs", $bootstraptime);
	$config->newval ($section, "can-configure", $bootstraptime);
	$config->newval ($section, "can-maintain", $bootstraptime);
	$config->newval ($section, "can-use", $bootstraptime);

	$section = "ACL-RA";
	$config->AddSection ($section);
	$config->newval ($section, "can-change-privs", $bootstraptime);
	$config->newval ($section, "can-configure", $bootstraptime);
	$config->newval ($section, "can-maintain", $bootstraptime);
	$config->newval ($section, "can-use", $bootstraptime);

	$section = "ACL-EE";
	$config->AddSection ($section);
	$config->SetSectionComment ($section, "The following are not used for ".
								"now, as we all use ACL-CA (see relevant ".
								"web_acl lines).");
	$config->newval ($section, "can-change-privs", $bootstraptime);
	$config->newval ($section, "can-configure", $bootstraptime);
	$config->newval ($section, "can-maintain", $bootstraptime);
	$config->SetParameterComment ($section, "can-maintain",
								  "Everybody can-use.");

	$section = "ACL-renewal";
	$config->AddSection ($section);
	$config->newval ($section, "can-change-privs", $bootstraptime);
	$config->newval ($section, "can-configure", $bootstraptime);
	$config->newval ($section, "can-maintain", $bootstraptime);
	$config->SetParameterComment ($section, "can-maintain", "can-use is ".
								  "irrelevant here (no user interface).");

	$section = "ACL-config";
	$config->AddSection ($section);
	$config->newval ($section, "can-change-privs", $bootstraptime);
	$config->newval ($section, "can-configure", $bootstraptime);
	$config->newval ($section, "can-maintain", $bootstraptime);
	$config->SetParameterComment ($section, "can-maintain", "No can-use ".
								  "here: Refer to the proper can-configure ".
								  "line for each entity.");


	####################  TRANSPORT LAYER SETUP  ####################
	my %transport = map { $_ , $config->getval ("network", "transporttype_$_"); }
	  ( "ee", "ra",	"ca", "publish");
	

	my %transport_list =
	  (
	   "EE" => [ "csr_transport_out", "ee-ra-csr", "out",
				 "crr_transport_out", "ee-ra-crr", "out",
			   ],
	   "RA" => [ "csr_transport_in", "ee-ra-csr", "in",
				 "crr_transport_in", "ee-ra-crr", "in",
				 "csr_transport_out", "ra-ca-csr", "out",
				 "crr_transport_out", "ra-ca-crr", "out",
			   ],
	   "CA" => [ "csr_transport_in", "ra-ca-csr", "in",
				 "crr_transport_in", "ra-ca-crr", "in",
				 "p12_transport_out", "ca-ee-p12", "out",
				 "publish_transport_out", "publish", "out",
			   ],
	   "publish" => [ "publish_transport_in", "publish", "in" ],
	  );

	my ($item, $list);
	while (($item, $list) = each %transport_list) {
		my @list = @{$list};
		my ($k, $v, $c);
		while (($k, $v, $c) = splice @list, 0, 3) {
			$config->newval ($item, $k, $v);
			$config->SetParameterComment ($item, $k, "Type: TRANSPORT-OUT")
			  if($c eq "out");
			$config->SetParameterComment ($item, $k, "Type: TRANSPORT-IN")
			  if($c eq "in");
			$config->SetParameterComment
			  ($item, $k, "Type: TRANSPORT-OUT and TRANSPORT-IN")
			  if($c eq "both");
		}
	}

	my @transport_map =
	  (
	   { name => "csr", out => "ee", in => "ra", dir => "transportRA_CSR", },
	   { name => "crr", out => "ee", in => "ra", dir => "transportRA_CRR", },
	   { name => "csr", out => "ra", in => "ca", dir => "transportCA_CSR", },
	   { name => "crr", out => "ra", in => "ca", dir => "transportCA_CRR", },
	   { name => "publish", out => "ca", in => "publish",
		 section => "publish", dir => "transport_publish", },
	  );

	my %keys = ( ee => "$securedir/EE/net-EE.key",
				 ra => "$securedir/RA/net-RA.key",
				 ca => "$securedir/CA/net-CA.key",
				 use_xml_logs => "$securedir/logs/net-logs.key" );
	my %certs = ( ee => "$securedir/EE/net-EE.crt",
				  ra => "$securedir/RA/net-RA.crt",
				  ca => "$securedir/CA/net-CA.crt",
				  use_xml_logs => "$securedir/logs/net-logs.crt" );

	foreach my $current (@transport_map) {
		my $name = $current->{name};
		my $out = $current->{out};
		my $in = $current->{in};
		my $type = $transport{$in} || "https";
		my $section = $current->{section} ||
		  (($in eq $out) ? "$in-$name" : "$out-$in-$name");
		my $directory = $current->{directory} || "$vardir/transport/".
		  $current->{dir};
		my $sectionout = "TRANSPORT-OUT-".$section;
		my $sectionin = "TRANSPORT-IN-".$section;
		my $post = $current->{post} || "";
		
		$type = "local" if ($components{$out} and $components{$in});
		$type = "https" if ($current->{https});
		
		# OUT part
		$config->AddSection ($sectionout);
		$config->newval ($sectionout, "type", $type);
		
		# IN part
		$config->AddSection ($sectionin);
		$config->newval ($sectionin, "type", $type);
		$config->newval ($sectionin, "directory", $directory);
		if($type =~ m/https/i) {
			my $key = $current->{key} || $keys{$out};
			my $cert = $current->{cert} || $certs{$out};
			# OUT part
			$config->newval ($sectionout, "uri", $bootstraptime);
			$config->newval ($sectionout, "key", $key);
			$config->newval ($sectionout, "cert", $cert);
			$config->newval ($sectionout, "peerDN", $bootstraptime);
			
			# IN part
			$config->newval ($sectionin, "post", $post) if ($post);
			$config->newval ($sectionin, "acl", "transport-".$section);
			$config->AddSection ("ACL-transport-".$section);
			$config->newval ("ACL-transport-".$section, "can-use",
							 $bootstraptime);
		} else {
			# OUT part
			$config->newval ($sectionout, "directory", $directory);
			$config->newval ($sectionin, "post", $post) if ($post);
		}
	}

	####################  LDAP HOOKS  ####################
	$section = "LDAP-CLIENT-repository-caaccess";
	$config->AddSection ($section);
	$config->SetSectionComment ($section, ";;;;;;;;;;;;;;;;;;;;  LDAP HOOKS".
								"  ;;;;;;;;;;;;;;;;;;;;",
								"Internal LDAP repository entry points.");
	# Internal use only, so no need to be configured -- but all IDX-PKI
	# entities MUST agree about it.
	$config->newval ($section, "commtype", "UNIX");
	$config->newval ($section, "socket", "$vardir/ldap-repository/sock");

	$section = "LDAP-CLIENT-repository-readonly";
	$config->AddSection ($section);
	$config->newval ($section, "host", $bootstraptime);
	$config->newval ($section, "port", $bootstraptime);
	$config->newval ($section, "commtype", "ssl");
	$config->newval ($section, "netDN", $bootstraptime);
	$config->SetParameterComment ($section, "netDN",
								  "No credentials required");


	# OK now we are done.
	return $config;
}

=pod "

=back

=cut "

require IDX::Test::Suite unless (caller());

1;

####################### TEST CASE ##############################

__END__

import IDX::PKI::Config;
use strict;
use IDX::PKI::Error;

1;
