#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

package IDX::PKI::Crypto::CRL;

use IDX::PKI::Crypto::Tools;
use IDX::PKI::Crypto::X509;
use IDX::PKI::FileManip;

$IDX::PKI::Crypto::CRL::VERSION = '0.0.1';

use strict;

=pod "

=head1 NAME

IDX::PKI::Crypto::CRL - CRL Management module

=head1 SYNOPSIS

 use IDX::PKI::Crypto::CRL;
 my $crl_object = parse IDX::PKI::Crypto::CRL ($crl);

=head1 DESCRIPTION

This class handles CRL objects (Certificate Revocation List).

=head1 METHODS

=over 4

=item I<new()>

Creates a new instance of the class.  One of C<-data> or C<-crl> argument
must be provided.  C<-format> argument is only useful with C<-crl>
argument.

=over 4

=item *

C<-crl> => The CRL.

=item *

C<-format> => The format of the CRL (B<PEM>|B<DER>). B<PEM> by default.

=item *

C<-data> => The CRL as provided by I<serialize()>.

=back

I<This method is obsolete!  Don't use it; use I<parse()> instead.>

=cut "

sub new
{
	my ($class, %args) = @_;

	my ($data, $format);
	if ($args{'-data'})
	{
		$data = $args{'-data'};
	}
	elsif ($args{'-crl'})
	{
		$data = $args{'-crl'};
		$format = $args{'-format'};
	}
	else
	{
		throw IDX::PKI::Error::Internal ("Neither -data nor -crl arguments ".
										 "provided to new().");
	}

	return parse ($class, $data, $format);
}


=pod "

=item I<parse ($data, [$format])>

Constructs a certificate object from its string representation (C<PEM> or
C<DER>, according to I<$format>, default C<PEM>).

=cut "

sub parse {
	my ($class, $data, $format) = @_;

	throw IDX::PKI::Error::Internal ("No data provided to parse()!")
		unless ($data);

	my $self = bless ({}, $class);

	if (! $format || $format eq 'PEM')
	{
		$self->{'crl'} = $data;
	}
	elsif ($format eq 'DER')
	{
		($self->{'crl'}, my $err) = run_data_pipe_openssl ($data,
			"crl", "-inform", "DER", "-outform", "PEM");
		throw IDX::PKI::Error::Crypto ("Conversion failed",
			'-error' => $err) if ($err);
	}
	else
	{
		throw IDX::PKI::Error::Internal ("Unknown format",
			'-format' => $format);
	}

	$self->parse_crl ();

	return $self;
}


=pod "

=item I<load ($file, [$format])>

Constructs a certificate object from a file in DER or PEM format (see also
L</parse>).

=cut "

sub load {
	my ($class, $file, $format) = @_;

	return parse ($class, file2string ($file), $format);
}


=pod "

=item I<loaddir ($dir)>

Loads CRLs from the directory I<$dir>. See
L<IDX::PKI::Crypto::Tools/loaddir>.

=cut "

sub loaddir {
	return IDX::PKI::Crypto::Tools::loaddir (@_);
}


=pod "

=item I<parse_crl()>

Parses a certificate to extract the attributes. Internal use only.

=cut "

sub parse_crl
  {
    my $self = shift;
    my $keys = { @_ };
    my $pid;

    my ($dump,$err) = run_data_pipe_openssl( $self->{crl},"crl","-text");

    if ( length( $err ) ne 0 )
      {
	throw IDX::PKI::Error::Crypto("Invalid CRL", -OpenSSL => $err);
      }

    unless(defined $dump)
      {
		throw IDX::PKI::Error::Crypto("Empty CRL");
      }

    my %att;
    my $i;
    my $buffer;
    my @table;

    # CRL Version
    ($att{version})       = ($dump =~ m/\n\s+Version:\s*(\d)\s*\(\w*\)\n/);
    # Serial Number
    ($att{sig_algorithm}) = ($dump =~ m/\n\s+Signature Algorithm:\s+(\w+)\n/);
    # DN Issuer
    ($att{dn_issuer})     = ($dump =~ m/\n\s+Issuer:\s+(.+)\n/);
    # Last Update (Date)
    ($att{last_update})   = ($dump =~ m/\n\s+Last Update:\s+(.+)\n/);
    # Next Update (Date)
    ($att{next_update})   = ($dump =~ m/\n\s+Next Update:\s+(.+)\n/);

    # Certificates.
    ($buffer) =
      ($dump =~ m/\n\s*Revoked Certificates:\n(.+)Signature Algorithm:/s);

    if(defined $buffer)
      {
	my %certificates;

	(@table) = ($buffer =~ m/\s+Serial Number: ([A-Za-z0-9]+)\n\s+Revocation Date:\s*(.+)\n/g);

	while (@table)
	  {
	    my $serial = shift @table;
	    my $date   = shift @table;
	    $certificates{$serial} = $date;
	  }
	$att{certificates} = { %certificates };
      }

    ($att{sig_value})     =
      ($dump =~ m/\n\s+Signature Algorithm:\s+\w+\n([\s\na-f0-9:]+)-----/s);

	throw IDX::PKI::Error::Crypto("No signature in CRL")
	  unless (defined $att{sig_value});

	$att{sig_value}=~ s/\s//gs;

    $self->{attr} = {%att};
  }

=pod "

=item I<getAttribute ($attr)>

Returns the value of a certificate attribute.  Following value can be
provided as a value for I<$attr>:

=back

=over 4

=item *

C<version>

=item *

C<sig_algorithm>

=item *

C<dn_issuer>

=item *

C<last_update>

=item *

C<next_update>

=item *

C<certificates>

=item *

C<sig_value>

=back

The C<certificates> value return a reference to a hash table
containing all the certificate, with the serial as key (in hex,
uppercase letters), and the date as value.

=cut "

sub getAttribute {
	my ($self, $attribute) = @_;

	$self->parse_cert() unless (defined $self->{attr});

	return $self->{attr}->{$attribute};
}

=pod "

=item I<serialize([$format])>

Return the CRL, in a form suitable for transport.  Should be given as
C<-data> argument to I<new()> method.  If $format is specified and equals
C<DER>, serializes to DER.

=cut "

#pod ends "
sub serialize
{
	my ($self, $format) = @_;

	if (defined $self->{'crl'})
	{
		if ($format && $format eq 'DER')
		{
			my ($crl, $err) = run_data_pipe_openssl ($self->{'crl'},
				"crl", "-inform", "PEM", "-outform", "DER");
			throw IDX::PKI::Error::Crypto ("Unable to serialize",
				'-error' => $err) if ($err ne '');
			return $crl;
		}
		else
		{
			return $self->{'crl'};
		}
	}
	else
	{
		throw IDX::PKI::Error::Crypto("No CRL found in the object!");
	}
}


=pod "

=item I<deserialize ($data)>

Returns the the object from its I<serialize()>d representation.  See also
L</parse()>.

=cut "

sub deserialize
{
	my ($class, $data) = @_;

	return parse ($class, $data);
}


=pod "

=item I<getSignerIndex($x509_1, $x509_2, ...)>

Returns the index of the IDX::PKI::Crypto::X509 that signed the CRL.

=cut "

sub getSignerIndex {
	my ($self, @x509s) = @_;
	my $index = 0;
	my $dir = new IDX::PKI::FileManip::TempDirectory();
	my $file = "$dir/cert.pem";

	while (@x509s) {
		my $x509 = shift @x509s;
		string2file($x509->serialize("PEM"), $file);
		my ($error, $result) = run_data_pipe_openssl
		  ($self->serialize("PEM"), "crl", "-noout", "-CAfile", $file);
		if($result =~ m/verify OK/) {
			return $index;
		}
		$index++;
	}
	return undef;
}

=pod "

=head1 SEE ALSO

L<IDX::PKI::Crypto::X509>,
L<IDX::Crypto::PKI::CRR>, L<IDX::Crypto::PKI::CSR>

=cut "

1;

eval(join '',<main::DATA>) || die $@ unless caller();

__END__

use strict;

use IDX::Test;
use IDX::PKI::Error;
use IDX::PKI::Crypto::X509;

# All tests are integration tests, because they all want to know where the
# openssl binary is.
use IDX::PKI::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}

number_of_tests (5);

my $pem = << 'CRL_EXAMPLE';
-----BEGIN X509 CRL-----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-----END X509 CRL-----
CRL_EXAMPLE

test "parse()" => sub {
  my $crl = parse IDX::PKI::Crypto::CRL ($pem);

  TEST( $crl->getAttribute ('sig_algorithm') eq 'sha1WithRSAEncryption' );
  TEST( $crl->getAttribute ('certificates')->{ '1FE9' }
	eq 'May 19 17:03:16 1999 GMT' );
  TEST( $crl->getAttribute ('certificates')->{ '12BE' }
	eq 'Apr  6 17:05:03 1999 GMT' );
};

test "parse() in DER format" => sub {
  my ($der, $err) = run_data_pipe_openssl( $pem,
			"crl","-inform","PEM","-outform","DER");

  my $crl = parse IDX::PKI::Crypto::CRL ($der, 'DER');

  TEST( $crl->getAttribute ('dn_issuer')
	eq '/C=US/O=U.S. Government/OU=DoD/OU=PKI/CN=Med CA-1' );
  TEST( $crl->getAttribute ('certificates')->{ '63' }
	eq 'May 29 19:37:11 1998 GMT' );
  TEST( $crl->getAttribute ('certificates')->{ '23DA' }
	eq 'Jun  7 14:39:20 1999 GMT' );

};

test "Serialisation/deserialisation test" => sub {
	my $original = parse IDX::PKI::Crypto::CRL ($pem);

	my $data = $original->serialize();

	my $crl = deserialize IDX::PKI::Crypto::CRL ($data);

	TEST( $crl->getAttribute ('sig_algorithm') eq 'sha1WithRSAEncryption' );

	TEST( $crl->getAttribute ('certificates')->{'1FE9'}
		eq 'May 19 17:03:16 1999 GMT' );

	TEST( $crl->getAttribute ('certificates')->{'12BE'}
		eq 'Apr  6 17:05:03 1999 GMT' );

};

my ($ca, $crl, $other);

test "Getting Test Objects" => sub {
	($ca, $other, undef, undef, undef, $crl) = IDX::Test::certificates();
	TEST($ca = parse IDX::PKI::Crypto::X509($ca));
	TEST($crl = parse IDX::PKI::Crypto::CRL($crl));
	TEST($other = parse IDX::PKI::Crypto::X509($other));
	
};

test "getSignerIndex" => sub {
	my $result = $crl->getSignerIndex($other, $ca);
	TEST(defined $result);
	TEST($result == 1);
};
