#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: CRR.pm,v 1.13 2003/06/17 14:35:49 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Crypto/CRR.pm,v $

package IDX::PKI::Crypto::CRR;

use strict;
use IDX::PKI::Debug( 0 );
use IDX::PKI::Error;
use IDX::PKI::Crypto::X509;
use IDX::PKI::Crypto::Tools;

$IDX::PKI::Crypto::CRR::VERSION = '0.0.1';


=pod "

=head1 NAME

IDX::PKI::Crypto::CRR - Perl extention to handle CRR objects

=head1 SYNOPSIS

use IDX::PKI::Crypto::CRR;

=head1 DESCRIPTION

This class handles CRR (Certificate Revoking Request) objects.

=cut "

#for emacs "

my %vars = (crr    => undef,	# The revocation certificate (an X509 object).
			date   => undef,	# Revocation date.
			form   => undef	# user-defined data.
		   );

=pod "

=head1 METHODS

=over 4

=item I<new()>

Creates a new instance of the class. C<-data> or C<-cert> must be provided. The C<-date>, C<-format> and C<-form> arguments can only be used together with the C<-cert> argument.

=over 4

=item *

C<-cert> => The CRR, in B<PEM> or B<DER> format.

=item *

C<-data> => Serialized CRR as provided by I<serialize()>.

=item *

C<-date> => Revocation date.

=item *

C<-format> => Format (B<PEM>|B<DER>). B<PEM> is used by default.

=item *

C<-form> => User defined data, as a hash table.

=back

=cut "

sub new
{
    my $that = shift;
    my $args = { @_ };
    my $class = ref($that) || $that;
    my $self = {
	%vars,
    };
    bless $self, $class;

    test_args( $args, '-cert', '-format',
	       '-data', '-date', '-form' );

    throw IDX::PKI::Error::Internal("One of '-data' or '-cert' argument is required")
	unless ( defined $args->{-data}
	         or defined $args->{-cert} );

    if ( defined $args->{-data} ) {
		$self->parse_data( -data => $args->{-data} );
    } else {
		$self->{crr} = parse IDX::PKI::Crypto::X509
		  ($args->{-cert},$args->{-format});

		$self->{date} = $args->{-date};
		$self->{form} = $args->{-form};
    }

    return $self;
}

=pod "

=item I<parse_data()>

Parses the serialized data in order to reconstruct the object. Internal use only.

=over 4

=item *

C<-data> => Serialized data containing the CRR.

=back

=cut "

sub parse_data
{
	my $self = shift;
	my $args = { @_ };
	my $form;
	my $data;
	my $cert;

	test_args( $args, '-data' );

	if ( defined $args->{-data} ) {
		$data = $args->{-data};
    } else {
		throw IDX::PKI::Error::Internal("No serialized data to parse");
    }

    ($self->{date},$form, $cert) = ( $data =~
    m/\-\-\-\-\-BEGIN\ CRR\-\-\-\-\-\n
	  date:\ ([0-9]*)\n
      (.*)
      (\-\-\-\-\-BEGIN\ CERTIFICATE\-\-\-\-\-\n
      .+
      \-\-\-\-\-END\ CERTIFICATE\-\-\-\-\-\n)
      \-\-\-\-\-END\ CRR\-\-\-\-\-/sx);

    (%{$self->{form}}) = ($form =~ m/\s*([A-Za-z][A-Za-z0-9_-]*)\s*=\s*(.*)\n/gx);

	$self->{crr} = parse IDX::PKI::Crypto::X509($cert);
  }

=pod "

=item I<getAttribute ($attr)>

Returns a CRR attribute.

=over 4

=item I<$attr>

Attribute to be returned. See L<IDX::PKI::Crypto::X509/getAttribute> for
a list of available attributes.

=back

=cut "

sub getAttribute
{
	my ($self, @args) = @_;

	return $self->{crr}->getAttribute (@args);
}


=pod "

=item I<get_date>

Returns the revocation date.

=cut "

sub get_date
{
    my $self = shift;
    my $args = { @_ };

    return $self->{date};
}

=pod "

=item get_crr_openssl()

Returns the certificate to revoque in a serialized form.

=cut "

sub get_crr_openssl
{
    my $self = shift;
    return $self->{crr}->serialize();
}

=pod "

=item I<serialize_frm()>

Generates content of the form in a format suitable for transport. Internal use only.

=cut "

sub serialize_frm
{
    my $self = shift;
    IDX::PKI::Crypto::Tools::serialize_form( %{$self->{form}} );
}

=pod "

=item I<serialize()>

Return the certificate, in a form suitable for transport. Sould be given as C<-data> argument to the I<new()> method.

=cut "

sub serialize
{
    my $self = shift;
    return "-----BEGIN CRR-----\n" .
	   "date: ".$self->{date}."\n".
	   $self->serialize_frm().
	   $self->{crr}->serialize().
	   "-----END CRR-----\n";
}

=pod "

=item I<get_certificate()>

Return the certificate to revoke.

=cut "
sub get_certificate
{
	my $self = shift;

	return $self->{crr}->{cert};
}

=pod "

=item I<getFormEntry ($entry)>

Returns the value of the form entry I<$entry>.

=cut "

sub getFormEntry
{
	my ($self, $entry) = @_;
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless ((scalar (@_) == 2) && defined $entry);

	return $self->{form}->{$entry};
}

=pod "

=item I<setFormEntry()>

Sets the value of a form entry.

=over 4

=item *

C<key> => key of the entry.

=item *

C<value> => value of the entry.

=back

=cut "

sub setFormEntry {
	my ($self, $key, $value) = @_;

	$self->{'form'}->{$key} = $value;
}



=pod "

=back

=head1 SEE ALSO

L<IDX::PKI::Crypto::X509>,
L<IDX::Crypto::PKI::CRL>, L<IDX::Crypto::PKI::CSR>

=cut "

1;

eval(join '',<main::DATA>) || die $@ unless caller();

__END__

#######################################################"

use strict;

use IDX::Test;
use IDX::PKI::Error;

# All tests are integration tests, because they all want to know where the
# openssl binary is.
use IDX::PKI::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}

number_of_tests(3);

my $x509 = << 'CERTIFICATE_EXAMPLE';
-----BEGIN CERTIFICATE-----
MIIDKjCCAtSgAwIBAgIBADANBgkqhkiG9w0BAQQFADCBkzELMAkGA1UEBhMCRlIx
DjAMBgNVBAcTBVBhcmlzMRUwEwYDVQQKEwxUZXN0IENvbXBhbnkxHDAaBgNVBAsT
E0NvbXB1dGVyIFJlc3NvdXJjZXMxDDAKBgNVBAsTA1BLSTEVMBMGA1UEAxMMVGVz
dCBSb290IENBMRowGAYJKoZIhvcNAQkBFgtjYUB0ZXN0LmNvbTAeFw0wMTA2MjUx
MDI3MDFaFw0wNjA2MjQxMDI3MDFaMIGTMQswCQYDVQQGEwJGUjEOMAwGA1UEBxMF
UGFyaXMxFTATBgNVBAoTDFRlc3QgQ29tcGFueTEcMBoGA1UECxMTQ29tcHV0ZXIg
UmVzc291cmNlczEMMAoGA1UECxMDUEtJMRUwEwYDVQQDEwxUZXN0IFJvb3QgQ0Ex
GjAYBgkqhkiG9w0BCQEWC2NhQHRlc3QuY29tMFwwDQYJKoZIhvcNAQEBBQADSwAw
SAJBAL2hSuDI7dedWERpNrVH7CHEDyharkiCXzg5x64Whhq3Z8Q4WXQZGjoiqBUG
clqwvKZSUXqbwb+0gOxk2KzrlM0CAwEAAaOCAQ8wggELMA8GA1UdEwEB/wQFMAMB
Af8wHQYDVR0OBBYEFAOjfTIpLa12A+7x2VS6aMpLmQiSMIHABgNVHSMEgbgwgbWA
FAOjfTIpLa12A+7x2VS6aMpLmQiSoYGZpIGWMIGTMQswCQYDVQQGEwJGUjEOMAwG
A1UEBxMFUGFyaXMxFTATBgNVBAoTDFRlc3QgQ29tcGFueTEcMBoGA1UECxMTQ29t
cHV0ZXIgUmVzc291cmNlczEMMAoGA1UECxMDUEtJMRUwEwYDVQQDEwxUZXN0IFJv
b3QgQ0ExGjAYBgkqhkiG9w0BCQEWC2NhQHRlc3QuY29tggEAMBYGA1UdEQQPMA2B
C2NhQHRlc3QuY29tMA0GCSqGSIb3DQEBBAUAA0EAsn7aXDdukQ8ygDxpfQSn1PW1
R5eOLlC4A8ntJjVBRHmEIgdwbYiKfhAcA7fhs5MgCG7xLrCQSjfMSEvpvQhpeA==
-----END CERTIFICATE-----
CERTIFICATE_EXAMPLE

test "Contructor test." => sub
{
    my $crr = new IDX::PKI::Crypto::CRR (-form => {tintin => "milou",
												   tom    => "jerry",
												   romeo  => "juliette"},
										 -cert => $x509,
										 -date => 20010704);

	TEST ($crr->get_certificate() eq $x509) || warn "get_vertificate: ".
		$crr->get_certificate();

	TEST ($crr->get_date() eq "20010704") || warn "get_date: ".
		$crr->get_date();

	TEST ($crr->getFormEntry ("tom") eq "jerry") ||
		warn "getFormEntry: ".$crr->getFormEntry ("tom");

	$crr->setFormEntry ("tom", "benny");

	TEST ($crr->getFormEntry ("tom") eq "benny") ||
		warn "getFormEntry: ".$crr->getFormEntry ("tom");

	TEST ($crr->getAttribute ("pk_algorithm") eq "rsaEncryption") ||
		warn "getAttribute pk_algorithm: ".$crr->getAttribute ("pk_algorithm");

	TEST ($crr->getAttribute ("dn_issuer")->string () =~ m/Test Company/) ||
		warn "getAttribute dn_issuer: ".
		$crr->getAttribute ("dn_issuer")->string ();

	TEST ($crr->getAttribute ("extensions")
		  ->{"X509v3 Basic Constraints"}->{'critical'} eq '1' ) ||
		warn "getAttribute extenions: ".
		$crr->getAttribute ("extensions")->{"X509v3 Basic Constraints"}
			->{'critical'};

	TEST ($crr->getAttribute ("extensions")
		  ->{"X509v3 Subject Key Identifier"}->{'value'}
		  eq '03:A3:7D:32:29:2D:AD:76:03:EE:F1:D9:54:BA:68:CA:4B:99:08:92') ||
		warn "getAttribute extenions: ".
		$crr->getAttribute ("extensions")->{"X509v3 Subject Key Identifier"}
			->{'value'};
};

test "Test serialisation." => sub {
	my $original = new IDX::PKI::Crypto::CRR (-form => {tintin => "milou",
														tom    => "jerry",
														romeo  => "juliette"},
											  -cert => $x509,
											  -date => 20010704);

	my $data = $original->serialize();

	my $crr = new IDX::PKI::Crypto::CRR (-data => $data);

	TEST ($crr->get_certificate() eq $x509) || warn "get_vertificate: ".
		$crr->get_certificate();

	TEST ($crr->get_date() eq "20010704") || warn "get_date: ".
		$crr->get_date();

	TEST ($crr->getFormEntry ("tom") eq "jerry") ||
		warn "getFormEntry: ".$crr->getFormEntry ("tom");

	TEST ($crr->getAttribute ("pk_algorithm") eq "rsaEncryption") ||
		warn "getAttribute pk_algorithm: ".$crr->getAttribute ("pk_algorithm");

	TEST ($crr->getAttribute ("dn_issuer")->string () =~ m/Test Company/) ||
		warn "getAttribute dn_issuer: ".
		$crr->getAttribute ("dn_issuer")->string ();

	TEST ($crr->getAttribute ("extensions")
		  ->{"X509v3 Basic Constraints"}->{'critical'} eq '1') ||
		warn "getAttribute extenions: ".
		$crr->getAttribute ("extensions")->{"X509v3 Basic Constraints"}
			->{'critical'};

	TEST ($crr->getAttribute ("extensions")
		  ->{"X509v3 Subject Key Identifier"}->{'value'}
		  eq '03:A3:7D:32:29:2D:AD:76:03:EE:F1:D9:54:BA:68:CA:4B:99:08:92') ||
		warn "getAttribute extenions: ".
		$crr->getAttribute ("extensions")->{"X509v3 Subject Key Identifier"}
			->{'value'};
};

test "Test mauvais argument constructeur." => sub {
	try {
		my $original = new IDX::PKI::Crypto::CRR (-for => {tintin => "milou",
														   tom => "jerry",
														   romeo => "juliette"
														   },
												  -crr => $x509,
												  -date => 20010704);
		TESTFAIL();
	} catch IDX::PKI::Error::Internal with {
		TESTOK();
	} otherwise {
		TESTFAIL();
	};
};
