#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

package IDX::PKI::Crypto::CSR;

$IDX::PKI::Crypto::CSR::VERSION = '0.0.1';

use strict;

use IDX::PKI::Config;
use IDX::PKI::Crypto::Tools;

=pod "

=head1 NAME

B<IDX::PKI::Crypto::CSR> - Perl extension to manage Certificate Signing Requests

=head1 SYNOPSIS

 use IDX::PKI::Crypto::CSR;
 my $iecsr = new IDX::PKI::Crypto::CSR::SPKAC ("-csr" => $spkac,
 	"-dn" => {"CN" => "Some User", "O" => "Idealx"},
 	"-form" => {});

=head1 DESCRIPTION

The CSR format is internal to IDX-PKI:

 ---- BEGIN CSR -----
 DN__CN=...
 DN__OU=...
 DN__O=...
 DN__C=...
 What=ever
 ---- BEGIN SPKAC ----
 [SPKAC ...]
 ---- END SPKAC ----
 ---- END CSR -----

or

 ---- BEGIN CSR -----
 DN__CN=...
 DN__OU=...
 DN__O=...
 DN__C=...
 What=ever
 ---- BEGIN PKCS10 ----
 [PKCS#10 ...]
 ---- END PKCS10 ----
 ---- END CSR -----

Where:

=over 4

=item *

C<[SPKAC]> and C<[PKCS10]> hold the key material, self-signed by the
requestor (public key, modulus, algorithm), as sent. Additionally,
C<[PKCS10]> contains the requested DN. It is unknown as of this writing
whether Internet Explorer would accept any other DN in the issued
certificate - anyway, OpenSSL refuses to modify it, as per the PKCS#10
standard.

=item *

The upper section holds all other data entered by the user's navigator in
the enrollment form, including its DN (which is consequently duplicated in
the PKCS#10 case). Caution: "entered by the user's navigator" is not the
same as "entered by the user" (think hidden CGI fields). In fact, we
consider the immutable-DN feature rather as a nuisance, since a human
operator is going to validate things at the RA level and she will be
unable to correct the DN in the PKCS#10 case. For consistency IDX-PKI will
enforce this restriction in the SPKAC case as well; this means that the DN
components that are part of the form data (C<C=>, C<0.OU=> etc. see
L<IDXPKI-API>) are considered authoritative in lieu of those that Internet
Explorer signed.

=back

=head1 METHODS

=over 4

=item I<< new ("-csr" => $pkcs10_or_spkac, "-dn" => ["CN" => "Some User",
"O" => "Idealx"], "-form" => {"someinfo" => "value"}) >>

Creates a new instance of the class. This constructor may not be called in
the ancestor class, but rather in one of the
B<IDX::PKI::Crypto::CSR::PKCS10> and B<IDX::PKI::Crypto::CSR::SPKAC>
subclasses.

=over 4

=item C<-csr>

Cryptographic raw data (B<SPKAC> or B<PKCS#10> format), as sent by the
browser.

=item C<-dn>

The envisioned Distinguished Name for this certificate, as an array
reference whose even elements contain the keys and the odd elements
contain the values.

=item C<-form>

The form containing user-defined data. This must be an hash reference.

=back

All options are mandatory.

=cut "

sub new {
	my ($that, %args) = @_;
	my $class = ref($that) || $that;

	throw IDX::PKI::Error::Internal("PURE_VIRTUAL")
		if ($class eq 'IDX::PKI::Crypto::CSR');

	my ($csr, $dn, $form) = @args{"-csr", "-dn", "-form"};

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-args" => \%args)
		unless ($csr && ref ($dn) eq "ARRAY" && ref ($form) eq "HASH");

	my $self = {
		csr    => $csr,		# The certificate request, as-is (to keep the
		# signature intact).
		form   => $form,	# Data from user form.
		dn     => $dn,		# The Distinguished Name
	};
	bless ($self, $class);

	# The attributes of the certificate request (key parameters and
	# parasit values such as DN).
	# This is useful to check that $csr is a correct expected CSR (PKCS#10
	# or SPKAC).
	$self->{"attr"} = $self->_parseCSR();

	return $self;
}

=pod "

=item I<deserialize()>

Unpacks a text representation of a CSR (see above). Example of use:

 my $csr = deserialize
 	IDX::PKI::Crypto::CSR::PKCS10 ($blob);

=cut "

sub deserialize {
	my $class = (ref($_[0]) || $_[0]);

	if ((bless {}, $class)->isa('IDX::PKI::Crypto::CSR')) {
		shift;
	} else {
		$class = undef;		# To be computed from the text in $data :-)
	}

	my ($data) = @_;
	throw IDX::PKI::Error::Crypto("Trying to deserialize empty or ".
								  "undefined data") unless($data);

	#### Parsing

	$data =~ s/\r//gs;
	my ($form, $readtype, $csr) =
		($data =~
			m/\-\-\-\-\ BEGIN\ CSR\ \-\-\-\-\n
			(.*)
			\-\-\-\-\ BEGIN\ ([[:alnum:]]*)\ \-\-\-\-\n
			(.+)\n
			\-\-\-\-\ END\ (\2)\ \-\-\-\-\n
			\-\-\-\-\ END\ CSR\ \-\-\-\-/sx);

	#    $self->{csr} =~ s/\s//gs;	# Remove the ending \n.
	#    Because the tailing \n is part of the spkac, we keep it.

	throw IDX::PKI::Error::Crypto("Unable to deserialize", -data => $data)
		unless($readtype and $csr);

	my @fields = map {($_ =~ m/\s*([A-Za-z0-9_.-]*)\s*=\s*(.*)/)}
		split ("\n", $form);

	my (@dn, %form);
	while (my ($k, $v) = splice (@fields, 0, 2)) {
		if ($k =~ m/^DN__(.*)$/) {
			push (@dn, $1, $v);
		} else {
			# Can be a single value or array reference.
			if (exists ($form{$k}) && ref ($form{$k}) eq "ARRAY") {
				# Already an array ref.
				push (@{$form{$k}}, $v);
			} elsif (exists $form{$k}) {
				# Already a text value => must become a hash.
				$form{$k} = [$form{$k}, $v];
			} else {
				# No need to have an array ref.
				$form{$k} = $v;
			}
		}
	}

	#### Blessing

	$class = 'IDX::PKI::Crypto::CSR::'.$readtype
		if ((!defined $class) || ($class eq "IDX::PKI::Crypto::CSR"));

	my $self = $class->new ("-csr" => $csr, "-dn" => \@dn, "-form" => \%form);

	throw IDX::PKI::Error::Internal("Wrong type")
		unless ($readtype eq $self->typename());

	return $self;
}

=pod "

=item I<serialize_DN()>

Generates content of the DN in a format suitable for transport. For
internal use by I<serialize()> only.

=cut "

sub serialize_DN {
	my $self = shift;
	if (defined $self->{dn}) {
		my @DN=@{$self->{dn}};
		my $ret="";
		while (my ($k,$v) = splice @DN,0,2) {
			$ret.="DN__$k=$v\n";
		}
		return $ret;
	}
	return "";
}

=pod "

=item I<getAttribute()>

Returns the value of a CSR attribute (i.e. parameters of the public key).

=cut "

sub getAttribute {
	my ($self, $attr) = @_;

	return $self->{attr}->{$attr};
}


=pod "

=item serialize()

Return the content of the object in a form suitable for transport.

=cut "

sub serialize {
	my $self = shift;
	my $args = { @_ };

	my $typename=$self->typename();

	return "---- BEGIN CSR ----\n".
		$self->serialize_DN().
		IDX::PKI::Crypto::Tools::serialize_form (%{$self->{form}}).
		"---- BEGIN $typename ----\n".
		$self->{csr}."\n".
		"---- END $typename ----\n".
		"---- END CSR ----\n";
}

=pod "

=item I<typename ()>

Returns the CSR type (such as C<PKCS10>, C<SPKAC> or C<Central>). Of
course, this function has no implementation in the base class.

=cut "

sub typename {
	throw IDX::PKI::Error::Internal("PURE_VIRTUAL");
}


=pod "

=item I<getForms ()>

Returns all the form entries as a hash.

=cut "

sub getForms {
	my ($self) = @_;

	return %{$self->{'form'}};
}


=pod "

=item I<getFormEntry ($entry)>

Returns the value of the form entry I<$entry>.

=cut "

sub getFormEntry {
	my ($self, $entry) = @_;
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless ((scalar (@_) == 2) && defined $entry);

	return $self->{'form'}->{$entry};
}

=pod "

=item I<setFormEntry()>

Sets the value of a form entry.

=over 4

=item *

C<key> => key of the entry.

=item *

C<value> => value of the entry.

=back

=cut "

sub setFormEntry {
	my ($self, $key, $value) = @_;

	$self->{'form'}->{$key} = $value;
}

=pod "

=item I<getDN()>

Extracts the DN from a CSR object. In array context, returns an
ordered list whose even elements are keys and whose odd elements are
values. In scalar context, returns a newline-separated DN.

=cut "

sub getDN {
	my ($self) = @_;
	if (defined $self->{"dn"}) {
		my @DN = @{$self->{"dn"}};
		return @DN if (wantarray);
		my $ret;
		while (@DN) {
			my $k = shift (@DN);
			my $v = shift (@DN);
			$ret .= "$k=$v\n";
		}
		return $ret;
	} else {
		return wantarray () ? () : "no DN";
	}
}

=pod "

=item I<getSubject ()>

Returns the "name" of the certificate request owner. By decreasing order
of preference, returns the (first) C<CN> from the DN, the (first)
C<commonName> from the form, or the (first) C<subjectAltName> from the
form.

If none of them can be found, returns C<Unknown>.

=cut "

sub getSubject ($) {
	my ($self) = @_;

	# Try the DN "CN".
	my @dn = $self->getDN ();
	while (my ($key, $val) = splice (@dn, 0, 2)) {
		return $val if ($key =~ m/^cn$/i || $key =~ m/^commonname$/i);
	}

	# Form "commonName" or "subjectAltName".
	foreach my $tag ("commonname", "subjectaltname") {
		foreach my $key (keys %{$self->{"form"}}) {
			next unless ($key =~ m/^$tag$/i);
			my $value = $self->getFormEntry ($key);
			$value = $value->[0] if (ref ($value) eq "ARRAY");
			return $value if ($value);
		}
	}

	return "Unknown";
}

=pod "

=item I<getEmail ()>

Returns the email address associated to this request. By decreasing order
of preference, returns the (first) C<Email> from the DN, the (first)
C<emailAddress> from the form, or the (first) C<*email*> from the form.

If none of them can be found, returns C<No email>.

=cut "

sub getEmail ($) {
	my ($self) = @_;

	# Try the DN "Email".
	my @dn = $self->getDN ();
	while (my ($key, $val) = splice (@dn, 0, 2)) {
		return $val if ($key =~ m/^email$/i);
	}

	# Form "emailAddress" or "*email*".
	foreach my $tag ("emailaddress", "*email*") {
		foreach my $key (keys %{$self->{"form"}}) {
			next unless ($key =~ m/^$tag$/i);
			my $value = $self->getFormEntry ($key);
			$value = $value->[0] if (ref ($value) eq "ARRAY");
			return $value if ($value);
		}
	}

	return "No email";
}

###################################################################
###################################################################
###################################################################

=pod "

=back

=head1 SUBPACKAGES

=head2 B<IDX::PKI::Crypto::CSR::PKCS10>

This package is to be used for a PKCS#10 Certificate Signing Request, such
as one produced by C<openssl req>, by most servers and by Internet
Explorer.

=over 4

=cut "

package IDX::PKI::Crypto::CSR::PKCS10;

use strict;

use IDX::PKI::Crypto::Tools;
use IDX::PKI::DN;
use IDX::PKI::Crypto::PKCS10;

use vars qw(@ISA);
@ISA = qw(IDX::PKI::Crypto::CSR);

sub typename {
	return "PKCS10";
}

=pod "

=item I<new()>

Overloaded from B<IDX::PKI::Crypto::CSR> to ensure that the request string
has the proper header and footer.

=cut "

sub new {
	my ($class, %args) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (defined $args{"-csr"});

	# Normalise trailing backspace.
	$args{"-csr"} =~ s/[\r\n]*$/\n/s;

	# IE forgets to put the standard BEGIN and END header and footer, so we
	# normalise it and thus we are sure of what we get.

	$args{"-csr"} =~ s/^.*-----BEGIN [a-z0-9 \#]+-----\r?\n//si;
	$args{"-csr"} =~ s/-----END [a-z0-9 \#]+-----.*$//si;
	$args{"-csr"} = "-----BEGIN CERTIFICATE REQUEST-----\n".$args{"-csr"}.
		"-----END CERTIFICATE REQUEST-----\n";

	my $self = $class->SUPER::new (%args);

	return $self;
}

=pod "

=item I<_parseCSR()>

Internal use.

Parses a PKCS#10 CSR and returns the attributes (in fact it is merely a
wrapper around I<IDX::PKI::Crypto::PKCS10::parse>). In the current
implementation, this function is quite expensive since it forks an openssl
subprocess.

=cut "

sub _parseCSR {
	my $self = shift;

	my $p10 = parse IDX::PKI::Crypto::PKCS10 ($self->{"csr"});
	my %att;

	foreach my $tag (qw(version dn_subject
					   pk_algorithm pk_bitsize
					   sig_algorithm
					   sig_value)) {
		$att{$tag} = $p10->getAttribute($tag);
	};
	$att{"pk_modulus"} = $p10->getAttribute("pk_modulus_bn");

	return wantarray ? %att : \%att;
}

=pod "

=item I<getCSROpenssl()>

Returns the PKCS#10 CSR.

=cut "

sub getCSROpenssl {
	my $self = shift;
	return $self->{"csr"};
}


###########################################################################
###########################################################################
###########################################################################

=pod "

=back

=head2 B<IDX::PKI::Crypto::CSR::SPKAC>

This package is to be used for a PKCS#10 Certificate Signing Request, such
as one produced by Netscape and some other SSL-aware browsers (but not
Internet Explorer).

=over 4

=cut "

package IDX::PKI::Crypto::CSR::SPKAC;

use strict;

use IDX::PKI::Crypto::Tools;
use IDX::PKI::DN;

use vars qw(@ISA);
@ISA = qw(IDX::PKI::Crypto::CSR);

# Those functions are documented in the IDX::PKI::Crypto::CSR package
# and need not be documented again.
sub typename {
	return "SPKAC";
};

=pod "

=item I<_parseCSR ()>

Internal use.

Parses a SPKAC CSR and returns the attributes.

=cut "

sub _parseCSR {
	my $self = shift;

	my ($dump, $err) = run_data_pipe_openssl ($self->getCSROpenssl (),
											  "spkac","-pubkey");
	if ($err ne '' || (! $dump)) {
		throw IDX::PKI::Error::Crypto ("Unable to parse CSR",
									   "-spkac" => $self->getCSROpenssl ());
	}

	my %att;

	# Public Key Algorithm
	($att{pk_algorithm})  = ($dump =~ m/\n\s+Public Key Algorithm:\s+(\w+)\n/);
	# Public Key Size (in bits)
	($att{pk_bitsize})    = ($dump =~ m/\n\s+\w+\s+Public Key: \((\d+) bit\)/);
	# Signature Algorithm
	($att{sig_algorithm}) = ($dump =~ m/\n\s+Signature Algorithm:\s+(\w+)\n/);
	# Public Key Modulus
	($att{pk_modulus})    = ($dump =~ m/\n\s+Modulus \(\d+ bit\):\n\s+(.+)\n\s+Exponent/s);
	$att{pk_modulus} =~ s/\s//gs;
	# Signature Value
	($att{sig_value})     = ($dump =~ m/\n\s+Signature Algorithm:\s+\w+\n([\s\na-f0-9:]+)/s);
	$att{sig_value}=~ s/\s//gs;

	# One more run of openssl for the pk_modulus_hex...
	my $modulus = run_data_pipe_openssl
	  ($dump,"rsa","-modulus","-pubin","-noout");
	($modulus =~ m/^Modulus=\s*(\S.*)$/mix) ||
	  throw IDX::PKI::Error::Crypto
		("Cannot read modulus in SPKAC", -OpenSSL => $dump);
	$att{pk_modulus_hex} = $1;

	return wantarray ? %att : \%att;
}

=pod "

=item I<getCSROpenssl()>

Returns the completed SPKAC CSR so that B<openssl> can understand it. This
means prepending the DN (see L<IDX::PKI::DN/to_openssl>) and a C<SPKAC = >
prefix to the Netscape request.

=cut "

sub getCSROpenssl {
	my $self = shift;
	my $spkac = $self->{"csr"};
	$spkac =~ s/\s//gs;
	return IDX::PKI::DN::to_openssl($self->getDN())."\nSPKAC = ".$spkac."\n";
}

###########################################################################
###########################################################################
###########################################################################

=pod "

=back

=head2 B<IDX::PKI::Crypto::CSR::Central>

This package is to be used for a central-mode Certificate Signing Request.
In this case, no public key has been sent.

=over 4

=cut "

package IDX::PKI::Crypto::CSR::Central;

use strict;

use IDX::PKI::Crypto::Tools;

use vars qw(@ISA);
@ISA=qw(IDX::PKI::Crypto::CSR);

# Those functions are documented in the IDX::PKI::Crypto::CSR package
# and need not be documented again.
sub typename {
	return "Central";
};

=pod "

=item I<_parseCSR ()>

Internal use.

There are no attributes to be set in a central CSR, so this just returns
an empty hash (or hash ref in scalar context). :-)

=cut "

sub _parseCSR {
	my $self = shift;

	return wantarray ? () : {};
}

=pod "

=item I<getCSROpenssl()>

This method throws, as it means nothing for a central-mode request (there
is no included CSR!).

=cut "

sub getCSROpenssl {
	my $self = shift;
	throw IDX::PKI::Error::Internal
		("REQUEST_INVALID", "-details" =>
		 "A central-mode CSR has no openssl contents");
}

=pod "

=back

=cut "

eval(join '',<main::DATA>) || die $@ unless caller();
1;

__END__


######################### TEST SUITE #########################

use strict;

use IDX::Test;
use IDX::PKI::Error;

# All tests are integration tests, because they all want to know where the
# openssl binary is.
use IDX::PKI::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}

number_of_tests (7);

my $spkac = << 'SPKAC_EXAMPLE';
MIG0MGAwXDANBgkqhkiG9w0BAQEFAANLADBIAkEAwmk6FMJ4uAVIYbcvIOx5+bDGTfvL8X5gE+R67ccMk6rCSGbVQz2cetyQtnI+VIs0NwdD6wjuSuVtVFbLoHonowIDAQABFgAwDQYJKoZIhvcNAQEEBQADQQBFZDUWFl6BJdomtN1Bi53mwijy1rRgJ4YirF15yBEDM3DjAQkKXHYOIX+qpz4KXKnl6EYxTnGSFL5wWt8X2iyx
SPKAC_EXAMPLE

my $pkcs10 = << 'PKCS_EXAMPLE';
-----BEGIN CERTIFICATE REQUEST-----
MIIBdDCB3gIBADA1MQswCQYDVQQGEwJGUjEPMA0GA1UEChMGSWRlYWx4MRUwEwYD
VQQDEwxJRFgtUEtJIFVzZXIwgZ8wDQYJKoZIhvcNAQEBBQADgY0AMIGJAoGBAO0L
HQBDLB1sZhkxA0uS3+DU0ycEdJe4m9WWOJndUtsxlENoUIURdsrzJkuJPHOLiIrH
YTeNL4/BFhcXv+7AXwIFFNZMR41cX7Qhtoj+oayGjS8MVmIrvnWXs1S2npLWAAaE
WfMZxaHgF8falaamIUbpbj6COaRXs+G33mqhlYspAgMBAAGgADANBgkqhkiG9w0B
AQQFAAOBgQCPmYAxkLsxPoN8MtarsYEuKOR9uayJFcE1gJsvzFh4/6znB2XEmOx8
Sylxyuey3KPh6mHI/qGWhOVuU38TYE+yvAmx7TJfAsLxpI6nuhOjK9KmjwT8dJgY
eZUj2Z/XTY6NrhAKvtxdA+S29gmbT/S7FD9VykcK6g8CYELXqG13Dg==
-----END CERTIFICATE REQUEST-----
PKCS_EXAMPLE

my @bogusdn = (
			   Email => "jxdoe\@someorg.com",
			   CN => "John X. Doe",
			   OU => "Netscape Compatibility Group",
			   O => "Some Organization, Inc.",
			   L => "Paris",
			   C => "FR",
			  );

test "SPKAC parsing" => sub {
	my $csr = new IDX::PKI::Crypto::CSR::SPKAC
		(-csr => $spkac,
		 -form => {Laurel => "Hardy",
				   Holmes => "Watson",
				   Kirk => "Spock"},
		 -dn => \@bogusdn);

	TEST ($csr->getDN() =~ m/CN=John/) || warn "getDN: ".$csr->getDN();
	my %dn = $csr->getDN();
	TEST ($dn{CN} eq "John X. Doe");

	TEST ($csr->getAttribute ("pk_algorithm") eq "rsaEncryption");

	TEST ($csr->getFormEntry ('Holmes') eq 'Watson');
};

test "PKCS#10 parsing" => sub {
	my $csr = new IDX::PKI::Crypto::CSR::PKCS10
		(-csr => $pkcs10,
		 -form => {Picard => "Ryker",
				   Sinclair => "Ivanova",
				   Asterix => "Obelix",
				   DN__C => "fr" },
		  -dn=>\@bogusdn);

	TEST ($csr->getAttribute ('pk_modulus') eq
		  "00:ed:0b:1d:00:43:2c:1d:6c:66:19:31:03:4b:92:".
		  "df:e0:d4:d3:27:04:74:97:b8:9b:d5:96:38:99:dd:".
		  "52:db:31:94:43:68:50:85:11:76:ca:f3:26:4b:89:".
		  "3c:73:8b:88:8a:c7:61:37:8d:2f:8f:c1:16:17:17:".
		  "bf:ee:c0:5f:02:05:14:d6:4c:47:8d:5c:5f:b4:21:".
		  "b6:88:fe:a1:ac:86:8d:2f:0c:56:62:2b:be:75:97:".
		  "b3:54:b6:9e:92:d6:00:06:84:59:f3:19:c5:a1:e0:".
		  "17:c7:da:95:a6:a6:21:46:e9:6e:3e:82:39:a4:57:".
		  "b3:e1:b7:de:6a:a1:95:8b:29") ||
		warn "pk_modulus: ".$csr->getAttribute ('pk_modulus');

	TEST ($csr->getFormEntry ('Sinclair') eq 'Ivanova') ||
		warn "Sinclair: ".$csr->getFormEntry ('Sinclair');
};

test "Parsing with no headers/trailers (such as produced by IE)" => sub {
	my $naked = $pkcs10;
	$naked =~ s/^-----BEGIN CERTIFICATE REQUEST-----\n//s;
	$naked =~ s/-----END CERTIFICATE REQUEST-----\n$//s;

	my $csr = new IDX::PKI::Crypto::CSR::PKCS10
		(-csr => $naked,
		 -form => {Picard => "Ryker",
				   Sinclair => "Ivanova",
				   Asterix => "Obelix",
				   DN__C => "fr" },
		 -dn=>\@bogusdn);
};

test "SPKAC serialisation." => sub {
	my $original = new IDX::PKI::Crypto::CSR::SPKAC
		(-csr => $spkac,
		 -form => {Laurel => "Hardy",
				   Holmes => "Watson",
				   Kirk => "Spock"},
		 -dn => \@bogusdn);

	my $csr = deserialize IDX::PKI::Crypto::CSR($original->serialize());

	TEST ($csr->{csr} eq $spkac) || warn "New object doesn't contain the ".
		"original SPKAC: ".$csr->{"csr"};
	TEST ($csr->getFormEntry ('Kirk') eq 'Spock') ||
		warn "Kirk: ".$csr->getFormEntry ('Kirk');
};

test "PKCS#10 serialisation." => sub {
	my $original = new IDX::PKI::Crypto::CSR::PKCS10
		(-csr => $pkcs10,
		 -form => {Picard => "Ryker",
				   Sinclair => "Ivanova",
				   Asterix => "Obelix",
				   digitalSignature => "1",
				   "reserved" => "1"
				   },
		 -dn => \@bogusdn);

	my $csr = deserialize IDX::PKI::Crypto::CSR($original->serialize());

	TEST ($csr->getAttribute ('sig_value') eq
		  "8f:99:80:31:90:bb:31:3e:83:7c:32:d6:ab:b1:81:2e:28:e4:".
		  "7d:b9:ac:89:15:c1:35:80:9b:2f:cc:58:78:ff:ac:e7:07:65:".
		  "c4:98:ec:7c:4b:29:71:ca:e7:b2:dc:a3:e1:ea:61:c8:fe:a1:".
		  "96:84:e5:6e:53:7f:13:60:4f:b2:bc:09:b1:ed:32:5f:02:c2:".
		  "f1:a4:8e:a7:ba:13:a3:2b:d2:a6:8f:04:fc:74:98:18:79:95:".
		  "23:d9:9f:d7:4d:8e:8d:ae:10:0a:be:dc:5d:03:e4:b6:f6:09:".
		  "9b:4f:f4:bb:14:3f:55:ca:47:0a:ea:0f:02:60:42:d7:a8:6d:".
		  "77:0e") ||
		warn "sig_value: ".$csr->getAttribute ('sig_value').
		"\nCSR: ".Data::Dumper::Dumper ($csr);

	TEST ($csr->getFormEntry ('Picard') eq 'Ryker') ||
		warn "Picard: ".$csr->getFormEntry ('Picard');
};

test "Constructor with bad args." => sub {
	try {
		my $original = new IDX::PKI::Crypto::CSR::PKCS10
			(-csr => $pkcs10,
			 -format => {Picard => "Ryker",
						 Sinclair => "Ivanova",
						 Asterix => "Obelix"});
		TESTFAIL();
	} catch IDX::PKI::Error::Internal with {
		TESTOK();
	};
};

test "Deserialisation with empty fields." => sub {
	my $test = << 'EOSTRING';
---- BEGIN CSR ----
DN__CN=The user
DN__OU=
DN__O=Some organisation
emailAddress=the.user@someorg.net
emailAddress=alias@someorg.net
emailAddress=
digitalSignature=1
keyEncipherment=1
clientAuth=1
emailProtection=1
NSclient=1
NSemail=1
---- BEGIN Central ----
Central
---- END Central ----
---- END CSR ----
EOSTRING
	my $csr = deserialize IDX::PKI::Crypto::CSR ($test);

	# DN elements.
	require IDX::PKI::DN;
	my $dn = build IDX::PKI::DN ($csr->getDN ());
	TEST ($dn->getRDNValue ("CN") eq "The user") ||
		warn "Bad CN: ".$dn->getRDNValue ("CN");
	TEST (! $dn->getRDNValue ("OU")) || warn "Bad OU: ".
		$dn->getRDNValue ("OU");
	TEST ($dn->getRDNValue ("O") eq "Some organisation") ||
		warn "Bad O: ".$dn->getRDNValue ("0");

	# Emails.
	my $emails = $csr->getFormEntry ("emailAddress");
	TEST ($emails->[0] eq 'the.user@someorg.net') ||
		warn "Bad email#1: $emails->[0]";
	TEST ($emails->[1] eq 'alias@someorg.net') ||
		warn "Bad email#2: $emails->[1]";
	TEST (! $emails->[2]) || warn "Bad email#3: $emails->[2]";

	# Extensions.
	TEST ($csr->getFormEntry ("digitalSignature")) ||
		warn "Missing digitalSignature";
	TEST ($csr->getFormEntry ("keyEncipherment")) ||
		warn "Missing keyEncipherment";
	TEST ($csr->getFormEntry ("NSemail")) || warn "Missing NSemail";
};
