#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: PKCS10.pm,v 1.22 2003/09/16 14:51:34 laura Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Crypto/PKCS10.pm,v $

package IDX::PKI::Crypto::PKCS10;

$VERSION = '0.0.1';

use strict;

use IDX::PKI::Crypto::Tools;
use IDX::PKI::FileManip;
use IDX::PKI::Error;
use IDX::PKI::DN;

=pod "

=head1 NAME

IDX::PKI::Crypto::PKCS10 - Handling of the PKCS10 file format
(inter-PKI standard for certificate requests).

=head1 SYNOPSIS

 use IDX::PKI::Crypto::PKCS10;

 # Parsing:
 my $p10=parse IDX::PKI::Crypto::PKCS10($whatIjustreadfromafile);
 print $p10->getAttribute ("pk_modulus_hex");
 my @dn=$p10->getDN();

 # Creating:
 my $p10=new IDX::PKI::Crypto::PKCS10($key,@dn);
 print $req->serialize();

=head1 DESCRIPTION

=over 4

=item I<new($key,@DN)>

Class constructor; creates a PKCS#10 data structure from a private key
and a DN. $key is an I<IDX::PKI::Crypto::PrivateKey> object; @DN is
the DN in the same style as in L<IDX::PKI::DN>.

=cut "

sub new {
	my ($class, $key, @dn) = @_;

	# And there we go with the OpenSSL config file nightmare...
	my $tmpdir = new IDX::PKI::FileManip::TempDirectory();
	local *CONFIG;
	my $conf = "$tmpdir/openssl.cnf";
	my $dn = IDX::PKI::DN::to_openssl(@dn);

	open (CONFIG, ">", $conf) ||
		throw IDX::PKI::Error::IO ("-IOfile" => $conf);
	print CONFIG <<"CONFIG" || throw IDX::PKI::Error::IO ("-IOfile" => $conf);
[ req ]
distinguished_name = DN
prompt             = no

[ DN ]
$dn
CONFIG
	close (CONFIG) || throw IDX::PKI::Error::IO ("-IOfile" => $conf);

	my $result = run_openssl ("req", "-new", $key->getOpensslParams("req"),
							  "-config", $conf);
	if ($result =~ m/BEGIN\ CERTIFICATE\ REQUEST[^\n]*\n(.*\n)
		[^\n]*END\ CERTIFICATE\ REQUEST/sx) {
		return parse($class,$result);
	} else {
		throw IDX::PKI::Error::Crypto("Failure in OpenSSL",
									  "-OpenSSL" => $result);
	}
}

=pod "

=item I<parse ()>

=cut "

sub parse {
	my ($class, $data) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS") unless (defined $data);

	my $pkcs10;
	if ($data =~ m/BEGIN\ (?:NEW\ )?CERTIFICATE\ REQUEST[^\n]*\n(.*\n)
		[^\n]*END\ (?:NEW\ )?CERTIFICATE\ REQUEST/sx) {
		$pkcs10 = $1;
	} else {
		throw IDX::PKI::Error::Internal("This is not a PKCS10",
										"-data" => $data);
	}

	my ($dump, $err) = run_data_pipe_openssl ($data, "req", "-text",
											  "-modulus", "-noout");

	throw IDX::PKI::Error::Crypto ("Error Reading CSR", "-error" => $err,
								   "-dump" => $dump, "-input" => $data)
		unless ($dump and ($err !~ m/^error/i));

	my %att;

	# Version
	($att{version})       = ($dump =~ m/\n\s+Version:\s*(\d)\s*\(\w*\)\n/);
	# DN Subject. Better not leave the ill-ordered version in the
	# object lest some poor soul tries to IDX::PKI::DN->parse()
	# it...
	my ($revdn)           = ($dump =~ m/\n\s+Subject:\s+(.+)\n/);
	$att{dn_subject} = build IDX::PKI::DN (map {m/([^=]+)=(.*)/}
		reverse (split (m"(?:,\s*)|/", $revdn)));
	# Public Key Algorithm
	($att{pk_algorithm})  = ($dump =~ m/\n\s+Public Key Algorithm:\s+(\w+)\n/);
	# Public Key Size (in bits)
	($att{pk_bitsize})    = ($dump =~ m/\n\s+\w+\s+Public Key: \((\d+) bit\)/);
	# Signature Algorithm
	($att{sig_algorithm}) = ($dump =~ m/\n\s+Signature Algorithm:\s+(\w+)\n/);
	# Public Key Modulus
	($dump =~ m/\n\s+Modulus\s*\(\d+ bit\):\n\s*(.+)\n\s*Exponent/s) ||
		throw IDX::PKI::Error::Crypto("Unable to read modulus in PKCS#10 CSR",
									  "-OpenSSL" => $dump);
	$att{pk_modulus_bn}    = $1;
	$att{pk_modulus_bn}      =~ s/\s//gs;
	# Signature Value
	($att{sig_value})     = ($dump =~ m/\n\s+Signature Algorithm:\s+\w+\n([\s\na-f0-9:]+)/s);
	$att{sig_value}       =~ s/\s//gs;

	# Modulus (hex style)
	($dump =~ m/^Modulus=\s*(\S.*)$/mix) ||
	  throw IDX::PKI::Error::Crypto("Cannot read modulus in PKCS#10",
									-OpenSSL=>$dump);
	$att{pk_modulus_hex}=$1;

    # for SCEP 
    ($dump =~ m/\n\s+challengePassword\s*:\s*(\w+)\n/);
    $att{attr_challengePassword}=$1;

	return bless({
				  pubkey=>\%att,
				  PKCS10=>$pkcs10
				 },$class);
}

=pod "

=item I<getAttribute ()>

=cut "

sub getAttribute {
	my ($self, $attr) = @_;

	return $self->{pubkey}->{$attr};
}

=pod "

=item I<getDN()>

Returns the DN of the request.  In scalar context, returns an
I<IDX::PKI::DN> object.  In list context, returns the result of the
I<canonical()> method call on same (see L<IDX::PKI::DN>).

=cut "

sub getDN {
	my ($self) = @_;

	my $dn = $self->getAttribute ('dn_subject');

	return wantarray ? $dn->canonical () : $dn;
}

=pod "

=item I<serialize ()>

=cut "

sub serialize {
	my ($self)=@_;

	return "-----BEGIN CERTIFICATE REQUEST-----\n".
		$self->{PKCS10}.
		"-----END CERTIFICATE REQUEST-----\n";
}

=pod "

=item I<getCSROpenssl ()>

Same as I<serialize()>.

=cut "

sub getCSROpenssl { serialize(@_); }

=pod "

=back

=cut "

eval(join '',<main::DATA>) || die $@ unless caller();
1;
__END__


######################### TEST SUITE #########################

use strict;

use IDX::Test;
use IDX::PKI::Error;

# All tests are integration tests, because they all want to know where the
# openssl binary is.
use IDX::PKI::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}

number_of_tests(6);

my $example = << 'PKCS10_EXAMPLE';
-----BEGIN CERTIFICATE REQUEST-----
MIIBRDCB7wIBADCBiTELMAkGA1UEBhMCRlIxDjAMBgNVBAcTBVBhcmlzMRUwEwYD
VQQKEwxUZXN0IENvbXBhbnkxHDAaBgNVBAsTE0NvbXB1dGVyIFJlc3NvdXJjZXMx
EzARBgNVBAMTClRlc3RDbGllbnQxIDAeBgkqhkiG9w0BCQEWEWNsaWVudEBpZGVh
bHguY29tMFwwDQYJKoZIhvcNAQEBBQADSwAwSAJBAKwziC3Ku9K3omZVKJ+mpG8k
C6v/lz8Hn6SCK6c5d/5kCy/ypGGDhsobBFncPdZSDVf2O7oqXedkFXmiRh2zYI0C
AwEAAaAAMA0GCSqGSIb3DQEBBAUAA0EALeWNcRyVfqh0GYDR3+6OnfnyWDO9KWML
foNLofQcM4qMGX3RBQisscUKUXVc3avltZjqn0Tl29pgi92jhR2/kg==
-----END CERTIFICATE REQUEST-----
PKCS10_EXAMPLE

require IDX::PKI::Crypto::PrivateKey;
my $k=new IDX::PKI::Crypto::PrivateKey(512);
$k->setKeyFile(new IDX::Test::Filename());

test "New PKCS10 from existing data" => sub {
	my $p10 = parse IDX::PKI::Crypto::PKCS10($example);

	TEST($p10->getAttribute ("pk_algorithm") eq "rsaEncryption");

};


test "New PKCS10 from thin air (requires IDX::PKI::Crypto::PrivateKey to work)"
 => sub {
  my $p10=new IDX::PKI::Crypto::PKCS10($k,CN=>"IamABogusCSR");
  TEST($p10->{PKCS10} =~ m/...../) || do {
	require Data::Dumper;
	warn Data::Dumper::Dumper($p10);
  };
};

test "parse <-> serialize reflexivity" => sub {
	my $example1=parse IDX::PKI::Crypto::PKCS10($example);
	my $example2=parse IDX::PKI::Crypto::PKCS10($example1->serialize());
	TEST($example1->{PKCS10} eq $example2->{PKCS10}) || do {
		warn $example1->{PKCS10};
		warn $example2->{PKCS10};
	};
};

my $rsa4096=<<'RSA';
-----BEGIN RSA PRIVATE KEY-----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-----END RSA PRIVATE KEY-----
RSA

# Actually this is a test for IDX::PKI::Crypto::Tools, but also
# a regression test in P740.
my $filename=new IDX::Test::Filename();
test "Big key" => sub {
	my $key=parse IDX::PKI::Crypto::PrivateKey($rsa4096);
	$key->setKeyFile($filename);
	TEST(new IDX::PKI::Crypto::PKCS10($key,CN=>"IamABogusCSR"));
};

test "DN order" => sub {
	my $p10=new IDX::PKI::Crypto::PKCS10($k,"CN"=>"IDX-PKI request",
										 "O"=>"IDEALX");
	my $txt=run_data_pipe_openssl($p10->serialize(),
								  "req","-text","-noout");
	TEST($txt =~ m/O=.*CN=/) || warn $txt;
};

test "getDN()" => sub {
	my $regression=<<REG;
-----BEGIN CERTIFICATE REQUEST-----
MIIBfzCB6QIBADBAMQswCQYDVQQGEwJGUjENMAsGA1UEChMEZ291djERMA8GA1UE
CxMIYWMtZGlqb24xDzANBgNVBAMTBnZwbjY0MTCBnzANBgkqhkiG9w0BAQEFAAOB
jQAwgYkCgYEAnSuapSsyG0GyG7BQ3yhV/IEQ9mYmUMU10Eu5r3pUCVN9x5T42CBV
1j9642rx69nymxOrW5HRH8aNDKJFRnyQ9AqhWRbRvXgdeKvjBFt3dKAh2T7QJXen
mUy5Dx/oYzTgqIry97z7r8tFfQSOi2jr1LVYjSe5nKJJbFbfnCcaMRcCAwEAAaAA
MA0GCSqGSIb3DQEBBAUAA4GBAAmvX1mKAFdRVwdvPiqIDuaLsmZBFSZJ0ocxmRlQ
+piQoUVXCIKhVVVPajHK/CwYagFQKkSjuYJMCyzixyq9gJ9HNIsiVsk2q0jZvXKV
k2AXC0/+m6V75xawpWspLjVWr4xzRZL8mV3T2HjZ6sYcUEySAa2c8ma4hMfINg34
/6HY
-----END CERTIFICATE REQUEST-----
REG
	my @DN=IDX::PKI::Crypto::PKCS10->parse($regression)->getDN();
	TEST($DN[$#DN-1] eq "C");
};
