#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: PKCS12.pm,v 1.24 2003/08/19 12:48:23 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Crypto/PKCS12.pm,v $

package IDX::PKI::Crypto::PKCS12;

use strict;
use IDX::PKI::Debug (0);
use IDX::PKI::Error;
use IDX::PKI::Crypto::Tools;
use IDX::PKI::Crypto::X509;
use IDX::PKI::Crypto::PrivateKey;
use MIME::Base64;

$IDX::PKI::Crypto::PKCS12::VERSION = '0.9';

=pod "

=head1 NAME

IDX::PKI::Crypto::PKCS12

=head1 SYNOPSIS

 use IDX::PKI::Crypto::PKCS12;

 my $p12 = parse
 	IDX::PKI::Crypto::PKCS12 ($data, "myverysecretpassword");
 my $cert = $p12->getCert();
 my $key = $p12->getKey();
 my @CAs = $p12->getCAs();

 my $anotherp12 = build
 	IDX::PKI::Crypto::PKCS12 ($cert, $key, @CAs);
 print $anotherp12->serialize("IthinkIpreferthisotherpassword");

=head1 DESCRIPTION

This class handles PKCS12 objects (for reading, writing and building).

=head1 METHODS

=over 4

=item I<parse($data, [$pass])>

Creates a new instance of the class. C<$data> is a string
representing an existing PKCS#12 data structure. $pass is the password
protecting the key (if not specified, decryption with the blank password
will be attempted).

=cut "

sub parse {
	my ($class, $p12, $pass)=@_;

	my $self=bless {},$class;

	my $passref=[ ($pass ? ($pass) : () ) ];
	my $passspec=($pass ? "%" : "pass:");

	my $text = run_data_and_passwords_pipe_openssl
	  ( $p12,$passref, "pkcs12","-clcerts","-nodes", "-passin", $passspec );

	DEBUG($text);
	unless ($text =~ m/
					   ^(-+BEGIN.*KEY-+$
					   (?:.|\n)*
					   ^-+END.*KEY-+)$
					   /mx) {
		throw IDX::PKI::Error::Crypto("OpenSSL failed on PKCS#12 key parsing",
									  -OpenSSL=>$text);
	};

	$self->{key} = parse IDX::PKI::Crypto::PrivateKey($1);

	unless ($text =~ m/^Bag\ Attributes((?:.|\n)*)?
					   ^(-+BEGIN.*CERTIFICATE-+$
					   (?:.|\n)*?
					   ^-+END.*CERTIFICATE-+)$
					   /mx) {
		throw IDX::PKI::Error::Crypto("OpenSSL failed on PKCS#12 cert parsing",
									  -OpenSSL=>$text);
	};
	my ($bag,$x509txt)=($1,$2);
	my $x509=parse IDX::PKI::Crypto::X509($x509txt);
	my ($fname)=_parseBag($bag);
	$self->{x509}=[$x509,$fname];

	# One more run for the CA hierarchy...
	$text=run_data_and_passwords_pipe_openssl
	  ($p12,$passref,"pkcs12", "-cacerts", "-nokeys", "-passin", $passspec );

	pos($text)=0;
	$self->{CAs}=[ map {
		parse IDX::PKI::Crypto::X509($_)
	} ($text =~ m/(-+BEGIN.*CERTIFICATE-+$
					   (?:.|\n)*?
					   ^-+END.*CERTIFICATE-+)$/gmx) ];
	return $self;
}

=pod "

=item I<< _parseBag($bag) >>

Internal use only.

Returns a list of interesting things in OpenSSL's output regarding the
"bag attributes" of one item in the PKCS#12 file. Right now only
the friendlyName is returned in position 0. This function doesn't make
sense in scalar context.

=cut "

sub _parseBag {
	throw IDX::PKI::Error::Internal("ILLEGAL_CONTEXT") unless wantarray;
	my ($bag)=@_;
	if ($bag =~ m/^\s*friendlyName:\s(.*)$/m) {
		my $msg=$1; chomp $msg; return ($msg);
	} else {
		return (undef);
	};
}

=pod "

=item I<< build($cert, $key, @CAs) >>

Creates a new instance of the class, with pieces specified. C<$key> is
an C<IDX::PKI::Crypto::PrivateKey> object; C<$cert> and C<@CAs> are all
C<IDX::PKI::Crypto::X509> objects. C<@CAs> should be ordered with the
root CA or highest CA first (anyway the specified ordering will be respected
in the generated PKCS#12).

Everywhere an C<IDX::PKI::Crypto::X509> object parameter is accepted,
one may also provide an anonymous pair with the
C<IDX::PKI::Crypto::X509> object and a text instead. The text is then
interpreted as a friendly name.

=back

=cut "

sub build {
	my ($class, $cert, $key, @CAs) = @_;

	# Sanity check.
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-key" => $key)
		unless (UNIVERSAL::isa ($key, "IDX::PKI::Crypto::PrivateKey"));

	my $self = bless({key => $key}, $class);

	if (ref($cert) eq "ARRAY") {
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-cert" => $cert)
			unless (UNIVERSAL::isa ($cert->[0], "IDX::PKI::Crypto::X509"));
		$self->{x509} = $cert;
	} else {
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-cert" => $cert)
			unless (UNIVERSAL::isa ($cert, "IDX::PKI::Crypto::X509"));
		$self->{x509} = [$cert, undef];
	}

	foreach my $ca (@CAs) {
		if (ref($ca) eq "ARRAY") {
			throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-ca" => $ca)
				unless (UNIVERSAL::isa ($ca->[0], "IDX::PKI::Crypto::X509"));
			push @{$self->{CAs}}, $ca;
		} else {
			throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-ca" => $ca)
				unless (UNIVERSAL::isa ($ca, "IDX::PKI::Crypto::X509"));
			push @{$self->{CAs}}, [$ca, undef];
		}
	}
	return $self;
}

=pod "

=item I<serialize()>

Returns the PKCS#12 structure, in a form suitable for transport or
for sending to another PKI entity. If a parameter is specified, it
is interpreted as the password to use to protect the PKCS#12 contents.

=cut "

sub serialize
{
	my ($self, $password) = @_;

	my $msg = $self->getKey()->serialize()."\n";

	my @friendlyargs;
	my $notfirst;
	foreach my $c ($self->{'x509'}, @{$self->{'CAs'}}) {
		my ($cert, $friendlyname);
		if (ref($c) eq "ARRAY") {
			($cert, $friendlyname) = @$c;
		} else {
			$cert = $c;
		}

		$msg .= $cert->serialize()."\n";
		if ($notfirst++) {
			next if (!defined $friendlyname);
			push (@friendlyargs, "-caname", $friendlyname);
		} else {
			next if (!defined $friendlyname);
			push(@friendlyargs, "-name", $friendlyname);
		}
	}

	DEBUG($msg); DEBUG(join(" ",@friendlyargs));
	my ($text, $err) = run_data_and_passwords_pipe_openssl (
		$msg, [($password ? ($password) : ())],
		"pkcs12", "-export",
		"-passout", ($password ? ("%") : ("pass:")),
		@friendlyargs
	);
	throw IDX::PKI::Error::Crypto("PKCS#12 creation failed",
		'-Openssl' => $err, '-cert' => $self->{'x509'},
		'-key' => $self->getKey ()) if ($err =~ m/error|usage/i);
	return $text;
}

=pod "

=item I<getCert ()>

Returns the corresponding certificate.

=cut "

sub getCert {
	return shift->{x509}->[0];
}

=pod "

=item I<getCertFriendlyName()>

Returns the certificate's friendly name, if any, or undef.

=cut "

sub getCertFriendlyName {
	return shift->{x509}->[1];
}

=pod "

=item I<getKey ()>

Returns the corresponding private key.

=cut "

sub getKey {
	my ($self) = @_;
	return $self->{key};
}

=pod "

=item I<getCAs ()>

Returns the list of CA certificates (in the same order as inside the
PKCS#12 file).

=cut "

sub getCAs {
	my ($self)=@_;
	my @CAs=@{$self->{CAs}};

	return map {( (ref($_) eq "ARRAY") ? $_->[0] : $_)} @CAs;
}

=pod "

=back

=head1 BUGS

Support for friendly names is incomplete. Namely, they are only parsed
for the user's certificate and not for the CAs (but they should be
output OK, even though this is not tested).

=head1 SEE ALSO

L<IDX::PKI::Crypto::X509>,
L<IDX::PKI::Crypto::CRL>, L<IDX::PKI::Crypto::CSR>

=cut "

1;

eval(join '',<main::DATA>) || die $@ unless caller();

__END__

#######################################################"

use strict;

use IDX::Test;
use IDX::PKI::Error;
use MIME::Base64;

# All tests are integration tests, because they all want to know where the
# openssl binary is.
use IDX::PKI::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}

number_of_tests (9);

# This is a bogus  admin.p12 

my $p12mime=<<'PKCS12';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=
PKCS12

my $p12=MIME::Base64::decode_base64($p12mime);
my ($p,$key,$cert);my @CAs;
test "parse" => sub {
	$p=parse IDX::PKI::Crypto::PKCS12($p12);
};

test "getCert()" => sub {
	TEST(UNIVERSAL::isa($cert=$p->getCert(),"IDX::PKI::Crypto::X509"));
	TEST(!$p->getCertFriendlyName());
};

test "getKey()"=> sub {
	TEST(UNIVERSAL::isa($key=$p->getKey(),"IDX::PKI::Crypto::PrivateKey"));
};

test "getCAs()" => sub {
	@CAs=$p->getCAs();
	TEST(scalar(@CAs) eq 2);
};

my $string;
test "build() and serialize()" => sub {
	my $newp=build IDX::PKI::Crypto::PKCS12([$cert,"FriendlyName"],
											$key,@CAs);
	TEST(ref($newp) eq "IDX::PKI::Crypto::PKCS12");
	TEST($newp->getCertFriendlyName() eq "FriendlyName");
	$string=$newp->serialize("boguspwd");
};

test "Decrypting without a password fails" => sub {
	try {
		parse IDX::PKI::Crypto::PKCS12($string);
		TESTFAIL();
	} catch IDX::PKI::Error::Crypto with {
		TESTOK()
	};
};

test "Check passwd with run_data_pipe_openssl" => sub {
	my ($text, $err) = run_data_pipe_openssl ($string, "pkcs12",
											  "-passin", "pass:boguspwd",
											  "-nodes");
	TEST ($err =~ /MAC verified OK/) || warn "Openssl error: $err";
};

test "Check passwd with run_data_and_passwords_pipe_openssl" => sub {
	my ($text, $err) = run_data_and_passwords_pipe_openssl ($string,
		["boguspwd"], "pkcs12", "-passin", "%", "-nodes");
	TEST ($err =~ /MAC verified OK/) || warn "Openssl error: $err";
};

test "parse again and examine differences" => sub {
	my $p2=parse IDX::PKI::Crypto::PKCS12($string,"boguspwd");

	TEST($p->getCert()->serialize() eq $p2->getCert()->serialize());
	my @CAs2=$p2->getCAs();
	for my $i (0..$#CAs) {
		TEST($CAs[$i]->serialize() eq $CAs2[$i]->serialize());
	};
	TEST($p2->getCertFriendlyName() eq "FriendlyName");
};
