#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

package IDX::PKI::Crypto::PKCS7;
use strict;

=pod "

=head1 NAME

B<IDX::PKI::Crypto::PKCS7> - Handling of RFC2315 cryptographically enhanced data structures

=head1 SYNOPSIS

 use IDX::PKI::Crypto::PKCS7;

 my $signedstruct=build IDX::PKI::Crypto::PKCS7::SignedData($data,
    -signer=>[$cert1,$key1],-signer=>[$cert2,$key2]);

 my $encryptedstruct=build IDX::PKI::Crypto::PKCS7::EnvelopedData(
	$signedstruct,$cert2,$cert3,$cert4);

 print $encryptedstruct->serialize();
    # Encryption is performed, of course.

 # This special form means that the enveloped data is not known
 # except by its type, and does not work yet:
 # my $signedstruct=blindBuild
 #   IDX::PKI::Crypto::PKCS7::EnvelopedData($opaquestring,"SignedData");

 # CRL and certificates bundle for incorporation into IE
 my $p7bundle=build IDX::PKI::Crypto::PKCS7::SignedData(undef,
    -cert=>$cacert,-cert=>$usercert,-CRL=>$crl,...)->serialize("DER");

 # Parsing is not "magical" yet, but would eventually work like this:
 # my $pkcs7=parse IDX::PKI::Crypto::PKCS7($data, $key1, $key2, ...);

 # Right now it is possible to do the following instead:
 my $pkcs7=parse IDX::PKI::Crypto::PKCS7::SignedData($data);

 my $pkcs7=parse IDX::PKI::Crypto::PKCS7::EnvelopedData($data,"PEM",
		$key1,$key2,...);
 # Or likewise:
 my $pkcs7=parse IDX::PKI::Crypto::PKCS7::EnvelopedData($data);
 $pkcs7->unlock($key1,$key2);

 # Syntactic sugar if you "just want to encrypt": you may specify
 # certificates and keys by filenames instead of objects.
 my $ciphertext=pkcs7Encrypt("secret",$cert);
 my $cleartext=pkcs7Decrypt($ciphertext,$key,$cert);

=head1 DESCRIPTION

The subclasses in this package implement the PKCS#7 (RFC2315)
cryptographically-enhanced nested data structures (but see
L</BUGS>). The general principle is to build a hierarchy of
B<IDX::PKI::Crypto::PKCS7::*> objects containing one another, and then
call serialize() on the topmost one. The appropriate cryptographic
operations will be performed and a PEM-encoded string will be returned
(you can choose another encoding instead if you prefer, see below).

=cut "

package IDX::PKI::Crypto::PKCS7::EnvelopedData;

use strict;

use IDX::PKI::Error;
use IDX::PKI::Crypto::Tools qw(run_openssl run_data_pipe_openssl string2file);
use IDX::PKI::FileManip;
use MIME::Base64;
use IDX::PKI::Debug (0);
use IDX::PKI::Crypto::PrivateKey;

=pod "

=head2 IDX::PKI::Crypto::PKCS7::EnvelopedData

This class can parse and produce encrypted messages, decorated with
the type of the inner message (among the six in RFC2315) and the list
of intended recipients.

=over 4

=item I<build($data,$cert1,$cert2,...)>

Object constructor, builds a PKCS#7 EnvelopedData object with data
$data and intended recipients $cert1, $cert2... as
B<IDX::PKI::Crypto::X509> objects. Returns an
B<IDX::PKI::Crypto::PKCS7::EnvelopedData> object.

=cut "


sub build {
	my ($class, $data, @certs) = @_;

	foreach my $i (0..$#certs) {
		# Automagic support begins.
		unless (ref($certs[$i])) {
			if ($certs[$i] =~ m/BEGIN.*CERT/) {
				$certs[$i] = parse IDX::PKI::Crypto::X509($certs[$i]);
			} elsif (-f $certs[$i]) {
				$certs[$i] = load IDX::PKI::Crypto::X509($certs[$i]);
			} else {
				throw IDX::PKI::Error::Internal("INCORRECT_ARGS",
												"-cert" => $certs[$i]);
			}
		}
		# Automagic support ends.
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
										 "-cert" => $certs[$i])
			unless (UNIVERSAL::isa($certs[$i],"IDX::PKI::Crypto::X509"));
	}
	my $self = bless {"cleartext" => $data, "dest" => \@certs}, $class;
	return $self;
}

=pod "

=item I<serialize()>

=item I<serialize($format)>

Serializes the object, performing encryption at the same time. I<$format>
may be either C<PEM>, C<DER>, C<Base64> or C<SMIME>, C<PEM> being the
default.

=cut "

sub serialize {
	my ($self,$mode)=@_;

	$mode ||= "PEM";

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") unless
	  ($mode =~ m/^(DER|PEM|SMIME|Base64)/);$mode = $1;

	if (my $alreadyknown = $self->{"envelopedtext_$mode"}) {
		return $alreadyknown;
	} elsif ( (exists $self->{cleartext}) && ($mode ne "Base64") ) {
		my $dir=new IDX::PKI::FileManip::TempDirectory();

		my @certfiles;
		for(my $i=0; $i <= $#{$self->{dest}}; $i++) {
			my $filename="$dir/cert-$i";
			string2file($self->{dest}->[$i]->serialize("PEM"),
						$filename);
			push(@certfiles,$filename);
		};

		my ($txt,$err)=run_data_pipe_openssl($self->{cleartext},
				"smime","-encrypt","-des3","-binary",
					"-outform",$mode,@certfiles);
		if($err =~ m/error|usage/i) {
			throw IDX::PKI::Error::Crypto("$mode encryption failed",
									  -Openssl => $err);
		}
		return $txt;
	} elsif ($mode eq "DER") { # something-to-DER conversion.
		foreach my $b64type (qw(PEM SMIME Base64)) {
			next if (!defined (my $b64 = $self->{"envelopedtext_$b64type"}));
			# Weed all lines that are not pure Base64 (not foolproof I admit
			# but still OK given that "-" and ":" are mandatory in extra
			# lines of PEM and SMIME)
			$b64 =~ s|^.*[^A-Za-z0-9+/=](?<=\S).*$||gm;
			$self->{"envelopedtext_DER"} = decode_base64($b64);
			return $self->{"envelopedtext_DER"};
		};
		throw IDX::PKI::Error::Internal("Empty PKCS#7 structure ?!");
	} else { # conversion through DER.
		my $der = $self->serialize("DER");


		if ($mode eq "PEM") {
			my $base64 = encode_base64($der);
			return ($self->{envelopedtext_PEM}=<<"PEM");
-----BEGIN PKCS7-----
$base64-----END PKCS7-----
PEM
		} elsif ($mode eq "SMIME") {
			throw IDX::PKI::Error::Internal("UNIMPLEMENTED");
			# I'm too lazy to make up the headers / separators.
		} elsif ($mode eq "Base64") {
			my $base64 = encode_base64($der, '');
			return ($self->{envelopedtext_Base64} = $base64);
		} else {
			throw IDX::PKI::Error::Internal("UNIMPLEMENTED");
		};
	}
}


=pod "

=item I<parse($data)>

=item I<parse($data, $encoding)>

=item I<parse($data, $encoding, $key1, $key2, ...)>

=item I<parse($data, $encoding, [$key1, $cert1], [$key2, $cert2], ...)>

Object constructor, parses a PKCS#7 EnvelopedData under encoding
I<$encoding> (C<PEM>, C<DER>, C<SMIME> or C<Base64>, default is C<PEM>).
Returns an B<IDX::PKI::Crypto::PKCS7::EnvelopedData> object.  If keys are
specified, I<unlock()> is called with them and this constructor will throw
an exception if the operation doesn't succeed.

=cut "

sub parse {
	my ($class, $data, $encoding, @keys) = @_;
	$encoding ||= "PEM";

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") unless
		($encoding =~ m/^(DER|PEM|SMIME|Base64)/);
	$encoding = $1;

	my $self = bless {"envelopedtext_$encoding" => $data}, $class;
	if (@keys) {
		$self->unlock(@keys);
	}

	return $self;
}

=pod "

=item I<unlock ($key1, $key2,...)>

=item I<unlock ([$key1, $cert1], [$key2, $cert2], ...)>

Applies each key in turn to the encrypted data inside the object and sees
if it can open the envelope using it.  Returns the string or the
B<IDX::PKI::Crypto::PKCS7::*> sub-object that was hidden inside, or throws
an exception if the keys didn't match.  I<$key1>, I<$key2>, ... must be
B<IDX::PKI::Crypto::PrivateKey> objects or strings; in the latter case,
I<load()>ing occurs automagically if those strings are filenames.

The second form allows for specifying matching certificates for all keys.
This is theoretically not necessary but unfortunately this is the only
option for now (the other style of invocation doesn't work due to a
limitation in OpenSSL).  Automagic load()ing of certificates specified as
filenames is also supported.

=cut "

sub unlock {
	my ($self, @keys) = @_;

	my ($text, $mode);
	foreach my $trymode (qw(SMIME PEM DER Base64)) {
		if (exists $self->{"envelopedtext_$trymode"}) {
			$mode = $trymode;
			$text = $self->{"envelopedtext_$trymode"};
			last;
		}
	}

	if ($mode eq 'Base64') {
		$mode = 'DER';
		$text = decode_base64($text);
	}

	throw IDX::PKI::Error::Internal ("Can't determine what to decrypt",
									 "-mode" => $mode, "-encrypted" => $text)
		if ((!defined $mode) || (!defined $text));

	my $dir = new IDX::PKI::FileManip::TempDirectory();

	my $lasterr;
	foreach my $key (@keys) {
		throw IDX::PKI::Error::Internal("UNIMPLEMENTED") unless
			(ref($key) && ref($key) eq "ARRAY");
		my ($key, $cert) = @$key;

		# Automagic support begins.
		unless (ref($cert)) {
			if ($cert =~ m/BEGIN.*CERT/) {
				$cert = parse IDX::PKI::Crypto::X509($cert);
			} elsif (-f $cert) {
				$cert = load IDX::PKI::Crypto::X509($cert);
			} else {
				throw IDX::PKI::Error::Internal("INCORRECT_ARGS");
			}
		}

		unless (ref($key)) {
			if (-f $key) {
				$key = load IDX::PKI::Crypto::PrivateKey($key);
			} else {
				throw IDX::PKI::Error::Internal("UNIMPLEMENTED");
				# We don't want to write keys to disk automatically, and
				# we would have to under the current implementation of
				# IDX::PKI::Crypto::PKCS7 (see this module)
			}
		}

		# Automagic support ends.

		unless (UNIVERSAL::isa($cert,"IDX::PKI::Crypto::X509") &&
				UNIVERSAL::isa($key,"IDX::PKI::Crypto::PrivateKey")) {
			throw IDX::PKI::Error::Internal("INCORRECT_ARGS");
		}

		# Yes, keys have to be on disk and not password-protected. Go
		# ask the OpenSSL folks why.
		if (defined $key->getPassword()) {
			throw IDX::PKI::Error::Internal("UNIMPLEMENTED");
		}
		unless (defined $key->getKeyFile()) {
			throw IDX::PKI::Error::Internal("UNIMPLEMENTED");
		}

		string2file($cert->serialize(), "$dir/cert");

		my ($txt, $err) = run_data_pipe_openssl ($text, "smime","-decrypt",
			"-inform", $mode, "-recip", "$dir/cert",
			$key->getOpensslParams("smime"));

		if ($err =~ m/error|usage/i) {
			DEBUG("Error Unlocking (mode: $mode): \n$err\n");
			$lasterr = $err;
			next;
		}

		$self->{"cleartext"} = $txt;
		return $txt;
	}
	throw IDX::PKI::Error::Crypto ("No matching key found", "-mode" => $mode,
								   "-encrypted" => $text,
								   '-lasterror' => $lasterr);
}

=pod "

=item I<getRecipients()>

NOT IMPLEMENTED

=item I<getData()>

Returns the unencrypted contents. If I<unlock()> or the extended form of
the I<parse()> constructor was not called beforehand, an exception will be
raised.

=cut "

sub getData {
	my ($self)=@_;
	throw IDX::PKI::Error::Crypto("Data was not decrypted") unless
	  (defined $self->{cleartext});
	return $self->{cleartext};
}

=back

=cut "

############################################################################

# From RFC2315, in justification of the API above:
#
# EnvelopedData ::= SEQUENCE {
#     version Version,                              # Always 1
#     recipientInfos RecipientInfos,                # Handled
#     encryptedContentInfo EncryptedContentInfo }
#
#   RecipientInfos ::= SET OF RecipientInfo
#
# RecipientInfo ::= SEQUENCE {
#     version Version,
#     issuerAndSerialNumber IssuerAndSerialNumber,
#     keyEncryptionAlgorithm
#       KeyEncryptionAlgorithmIdentifier,           # May be implemented in
#                                     # some future version using a new method
#                                     # or extra parameters to serialize()
#     encryptedKey EncryptedKey }
#
#   EncryptedContentInfo ::= SEQUENCE {
#     contentType ContentType,                # Handled
#     contentEncryptionAlgorithm
#       ContentEncryptionAlgorithmIdentifier, # See above
#     encryptedContent
#       [0] IMPLICIT EncryptedContent OPTIONAL }
#
#   EncryptedContent ::= OCTET STRING
#


package IDX::PKI::Crypto::PKCS7::SignedData;

use strict;
use IDX::PKI::Error;
use IDX::PKI::Crypto::Tools qw(run_openssl run_data_pipe_openssl string2file);
use IDX::PKI::FileManip;

=pod "

=head2 B<IDX::PKI::Crypto::PKCS7::SignedData>

This class can parse and produce signed messages, decorated with
the type of the inner message (among the six in RFC2315) and the list
of intended recipients.

=over 4

=item I<< build ($data, "-signer" => [$cert1, $key1],..., "-cert" =>
$cacert, ..., "-CRL" => $crl) >>

Object constructor, builds a PKCS#7 SignedData object with data
$data (which may be undef). Named parameters are as follows:

=over 2

=item C<< "-signer" => [$cert, $key] >>

Signs the PKCS#12 file with this certificate and this key. $cert must
be an B<IDX::PKI::Crypto::X509> object and $key must be an
B<IDX::PKI::Crypto::PrivateKey> object, and they must match.

=item C<< "-cert" => $cacert >>

Adds this certificate in the additional verification fields of the
PKCS#7 object (typically used to send untrusted CAs up the
verification path). $cacert must be an B<IDX::PKI::Crypto::X509> object.

=item C<< "-CRL" => $crl >>

Adds this CRL to the additional verification fields of the PKCS#7 object.
$crl must be an B<IDX::PKI::Crypto::CRL> object.

Due to extensive breakage in OpenSSL's CRL management features, only
one CRL may be specified and therefore only the last one will be
honored.

=back

=cut "

sub build {
	my ($class, $data, @namedargs) = @_;

	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS") if (@namedargs % 2);

	my $self = bless {"data" => $data,
					  "signers" => [],
					  "CAcerts" => [],
					  "CRLs" => []}, $class;

	while (my ($k, $v) = splice (@namedargs, 0, 2)) {
		if ($k eq "-signer") {
			throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
											 "-details" => "Wrong signer",
											 "-signer" => $v)
				unless ((ref $v) && (ref($v) eq "ARRAY") &&
						(scalar(@$v) eq 2));
			push (@{$self->{"signers"}}, $v);
			next;
		} elsif ($k eq "-cert") {
			throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
											 "-details" => "Wrong cert",
											 "-cert" => $v)
				unless (UNIVERSAL::isa ($v, "IDX::PKI::Crypto::X509"));
			push (@{$self->{"CAcerts"}}, $v);
			next;
		} elsif ($k eq "-CRL") {
			throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
											 "-details" => "Wrong CRL",
											 "-CRL" => $v)
				unless (UNIVERSAL::isa ($v, "IDX::PKI::Crypto::CRL"));
			push (@{$self->{"CRLs"}}, $v); # OpenSSL bugware
			next;
		} else {
			throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
											 "-details" => "Unknown param",
											 "-param" => $k);
		}
	}

	throw IDX::PKI::Error::Internal("UNIMPLEMENTED") if
		(exists $self->{"CRLs"} && defined $self->{"data"});

	return $self;
}


=pod "

=item I<serialize()>

=item I<serialize($mode)>

Serializes the object, performing signature at the same time. $format may
be either C<PEM> or C<DER>, C<PEM> being the default.

BUG: the private keys need to be written to disk (using $key->setKeyFile())
and cannot be encrypted on-disk. This is a limitation in OpenSSL.

=cut "

sub serialize {
	my ($self, $mode)=@_;
	if ($mode && $mode eq "DER") {
		$mode="der";
	} else {
		$mode="pem";
	};

	if ($self->{data}) {
		throw IDX::PKI::Error::Internal("UNIMPLEMENTED");

		my ($txt,$err)=run_data_pipe_openssl($self->{data},"smime","-sign",
				map {"XXX"} @{$self->{signers}},
				map {"XXX"} @{$self->{CAcerts}},
				"-outform",$mode);
		throw IDX::PKI::Error::Crypto("Cannot sign",-Openssl=>$err) if $err;

		return $txt;
	} else {
		my $dir=new IDX::PKI::FileManip::TempDirectory();
		my @crls=@{$self->{CRLs}};
		my $crltxt=(@crls ? $crls[$#crls]->serialize("PEM") : "");

		my $cabundle=join("",map {$_->serialize()."\n"} @{$self->{CAcerts}});
		string2file($cabundle,"$dir/cabundle");

		my ($txt,$err)=run_data_pipe_openssl($crltxt,
				"crl2pkcs7", (! @crls ? ("-nocrl") : ()),
				"-outform",$mode,
				"-certfile","$dir/cabundle");

		throw IDX::PKI::Error::Crypto
		  ("Could not transform to PKCS7", -OpenSSL => $err)
			unless ( length( $err ) == 0 );

		# IE kludge, maybe it should be turned into an option instead ?
		if ($mode eq "pem") {
			$txt =~ s/-----BEGIN PKCS7-----/-----BEGIN CERTIFICATE-----/;
			$txt =~ s/-----END PKCS7-----/-----END CERTIFICATE-----/;
		};

		return $txt;
	};
}

=pod "

=item I<parse($data)>

=item I<parse($data, $encoding)>

Object constructor, parses a PKCS#7 SignedData under encoding $encoding
(C<PEM> or C<DER>, default is C<PEM>). Returns an
B<IDX::PKI::Crypto::PKCS7::SignedData> object. It is a fatal error (which
elicits an exception) for the $data to be badly signed.

=cut "




=pod "

=item I<getSigners()>

Returns a list of the signers' certificates of this datagram, in the
same order as OpenSSL sees them.

=cut "




=pod "

=item I<getData()>

Returns the signed contents.

=cut "




=pod "



=back

=cut "

#######################################################################
#
# From RFC2315, in justification of the API above:
#
# SignedData ::= SEQUENCE {
#     version Version,               # Always 1
#     digestAlgorithms DigestAlgorithmIdentifiers, # Will be handled
#                                     # with extra methods or extra args to
#                                     # serialize()
#     contentInfo ContentInfo,                     # Recursion into
#				     # another PKCS7:: class is not yet implemented but
#                    # will be at some point
#     certificates								   # Through build()
#        [0] IMPLICIT ExtendedCertificatesAndCertificates
#          OPTIONAL,
#     crls										   # Through build()
#       [1] IMPLICIT CertificateRevocationLists OPTIONAL,
#     signerInfos SignerInfos }                    # Through build()
#
#  SignerInfos ::= SET OF SignerInfo
#
#   SignerInfo ::= SEQUENCE {
#     version Version,                               # Always 1
#     issuerAndSerialNumber IssuerAndSerialNumber,   # The Key ID from cert.
#     digestAlgorithm DigestAlgorithmIdentifier,     # From cert.
#     authenticatedAttributes
#       [0] IMPLICIT Attributes OPTIONAL,            # Not managed
#     digestEncryptionAlgorithm                      # Will be handled with
#       DigestEncryptionAlgorithmIdentifier,         # extra methods
#     encryptedDigest EncryptedDigest,               # Computed from message
#     unauthenticatedAttributes                      # Not managed
#       [1] IMPLICIT Attributes OPTIONAL }



=pod "

=head2 IDX::PKI::Crypto::PKCS7::SignedAndEnvelopedData

This package is unimplemented, use a combination of I<SignedData> and
I<EnvelopedData> instead (EnvelopedData being outermost, as shown in
the synopsis). The encoding routines in this package will Do The Right Thing.

=head2 IDX::PKI::Crypto::PKCS7::EncryptedData

This package is unimplemented.

=head2 IDX::PKI::Crypto::PKCS7::DigestedData

This package is unimplemented.

=head2 IDX::PKI::Crypto::PKCS7::Data

There is no such class. Instead, all I<build()> methods called with
data that is either not a reference or a reference to something that
can("serialize") but is not in any of the other
B<IDX::PKI::Crypto::PKCS7::*> classes, is interpreted as data in the
PKCS#7 sense. Just gimme your strings, I'll encode them.

=head2 Syntactic sugar

The two functions below don't do object-oriented stuff and are
provided for when you "just want to crypt" and don't care about the
real format. The format used will be "Base64", meaning the character
set is that of Base64 ([^A-Za-z0-9+/=]) and all data fits on one line.

=over 4

=item I<pkcs7Encrypt($secret,$cert)>

=item I<pkcs7Decrypt($ciphertext,$key,$cert)>

$key and $cert may be passed either as objects or filenames.

=back

These functions are not imported into user packages by default;
if you want them directly in your namespace, do

  use IDX::PKI::Crypto::PKCS7 qw(pkcs7Encrypt pkcs7Decrypt);

instead of

  use IDX::PKI::Crypto::PKCS7;

=cut "

package IDX::PKI::Crypto::PKCS7;

use Exporter;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw(pkcs7Encrypt pkcs7Decrypt);

sub pkcs7Encrypt {
  my ($secret, $cert)=@_;

  return build IDX::PKI::Crypto::PKCS7::EnvelopedData
	($secret,$cert)->serialize("Base64");
}

sub pkcs7Decrypt {
	my ($secret, $key, $cert)=@_;
	return parse IDX::PKI::Crypto::PKCS7::EnvelopedData
	  ($secret, "Base64", [$key, $cert])->getData();
}

=pod "

=head1 BUGS

Actually, this module can only handle as much PKCS#7 as OpenSSL can,
which is not much. That's why there are restrictive subclasses of
B<IDX::PKI::Crypto::PKCS7::> inside this module and restrictive
constructors thereof (namely, PKCS#7 recursion is not implemented at
all, and only 4 classes out of the 6 in the RFC are dealt
with). If/when we get a decent ASN.1 parser/generator with matching
crypto capabilities, all UNIMPLEMENTED exceptions will disappear.

=cut "

eval join('',<main::DATA>) || die $@ unless caller();

__END__

###############################################################

use strict;

use IDX::Test;
use IDX::PKI::Error;
use IDX::PKI::Crypto::Tools qw(run_openssl run_data_pipe_openssl);
use IDX::PKI::Crypto::X509;
use IDX::PKI::Crypto::PrivateKey;

# All tests are integration tests, because they all want to know where the
# openssl binary is.
use IDX::PKI::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}

number_of_tests(11);

my ($cacertfile,$certfile,$keyfile)=IDX::Test::certFiles();
my $cacert = scalar (load IDX::PKI::Crypto::X509 ($cacertfile));
my $cert   = scalar (load IDX::PKI::Crypto::X509 ($certfile));
my $key    = scalar (load IDX::PKI::Crypto::PrivateKey ($keyfile));

my $data="hello \0\n\t";
#my $data=`cat /boot/vmlinuz`;

my ($p7obj,$p7string);

# The following tests are OK, but the matching code is not implemented yet.

# test "Simple signed object" => sub {
# 	$p7obj=build IDX::PKI::Crypto::PKCS7::SignedData($data,-signer=>[$cert,$key]);
# 	$p7string=$p7obj->serialize();
# 	TEST($p7string =~ m/BEGIN PKCS/);
# };
#
# test "parse signed object" => sub {
# 	$p7obj=parse IDX::PKI::Crypto::PKCS7::SignedData($p7string);
# 	my @signers=$p7obj->getSigners();
# 	TEST(scalar @signers eq 1);
# 	TEST(UNIVERSAL::isa($signers[0],'IDX::PKI::Crypto::X509'));
# 	TEST($p7obj->getData() eq $data);
# };
#
# test "invalid signature" => sub {
# 	$p7string =~ s|^([A-Za-z0-9+/=])|($1 eq "Z" ? "Y" : "Z")|e;
# 	try {
# 		parse IDX::PKI::Crypto::PKCS7::SignedData($p7string);
# 		TESTFAIL();
# 	} catch IDX::PKI::Error::Crypto with {
# 		TESTOK();
# 	};
# };

test "IE-style empty PKCS#7" => sub {
	$p7obj=build IDX::PKI::Crypto::PKCS7::SignedData(undef,
		-cert=>$cacert,-cert=>$cert);
	my ($txt,$err)=run_data_pipe_openssl($p7obj->serialize("DER"),
				"pkcs7","-print_certs","-inform","der");
	my @certheads=($txt =~ m/BEGIN CERT/g);
	TEST(scalar(@certheads) == 2);
};

test "Encryption" => sub {
	$p7obj=build IDX::PKI::Crypto::PKCS7::EnvelopedData($data,$cert);
	$p7string=$p7obj->serialize();
	TEST($p7string =~ m/BEGIN/);
	$p7string=$p7obj->serialize("DER");
	TEST($p7string !~ m/BEGIN/);
	TEST($p7string !~ m/\Q$data\E/);
};

test "Decryption (style 1)" => sub {
	$p7obj=parse IDX::PKI::Crypto::PKCS7::EnvelopedData($p7string,"DER");
	TEST(defined $p7obj->unlock([$key,$cert]));
	TEST(scalar($p7obj->getData()) eq $data) ||
	  printf STDERR ('"%s" "%s"',$p7obj->getData(),$data);
};

test "Decryption (style 2)" => sub {
	$p7obj=build IDX::PKI::Crypto::PKCS7::EnvelopedData($data,$cert);
	$p7string=$p7obj->serialize();
	$p7obj=parse IDX::PKI::Crypto::PKCS7::EnvelopedData($p7string,"PEM",
														[$key,$cert]);
	TEST($p7obj->getData() eq $data);
};

test "Decryption (using filenames)" => sub {
	$p7obj=build IDX::PKI::Crypto::PKCS7::EnvelopedData($data,$cert);
	$p7string=$p7obj->serialize();
	$p7obj=parse IDX::PKI::Crypto::PKCS7::EnvelopedData($p7string,"PEM",
														[$keyfile,$certfile]);
	TEST($p7obj->getData() eq $data);
};

test "S/MIME encryption / decryption" => sub {
	$p7obj=build IDX::PKI::Crypto::PKCS7::EnvelopedData($data,$cert);
	$p7string=$p7obj->serialize("SMIME");
	TEST($p7string =~ m/Content-/);
	$p7obj=parse IDX::PKI::Crypto::PKCS7::EnvelopedData($p7string,"SMIME",
														[$key,$cert]);
	TEST($p7obj->getData() eq $data);
};

my %encoded;
test "Conversion between formats, without the key" => sub {
	my @formats=qw(SMIME PEM DER Base64);
	my $p7=parse IDX::PKI::Crypto::PKCS7::EnvelopedData($p7string,"SMIME");
	foreach my $format (@formats) {
		$encoded{$format}=$p7->serialize($format);
	}
	foreach my $from (@formats) { 	foreach my $to (@formats) {
		next if ($to eq "SMIME"); # FIXME in the source (grep upwards for "lazy")
		try {
			TEST(parse IDX::PKI::Crypto::PKCS7::EnvelopedData
				 ($encoded{$from},$from)->serialize($to) eq
				 $encoded{$to}) ||
				   warn "Conversion mismatch from $from to $to
$encoded{$from}
$encoded{$to}
";
		} otherwise {
			my $E=shift;
			warn "Conversion exception from $from to $to";
			throw $E;
		};
	}}
};

test "Unlocking in all formats" => sub {
#	use Data::Dumper;
#	print Data::Dumper::Dumper(\%encoded);

	while(my ($format,$ciphertext) = each %encoded) {
		my $p7obj = parse IDX::PKI::Crypto::PKCS7::EnvelopedData($ciphertext,$format);
		try {
			if (TEST(defined $p7obj->unlock([$key,$cert]))) {
				TEST(scalar($p7obj->getData()) eq $data) ||
				  printf STDERR ('Wrong deciphering %s
"%s" "%s"',$format,$p7obj->getData(),$data);
			} else {
				warn "Deciphering $format failed";
			};
		} otherwise {
			warn "Deciphering $format raised an exception";
			my $E=shift; throw $E;
		};
	};
};

test "Unauthorized decryption attempt" => sub {
	$p7obj=build IDX::PKI::Crypto::PKCS7::EnvelopedData($data,$cacert);

	$p7obj=parse IDX::PKI::Crypto::PKCS7::EnvelopedData($p7string,"PEM");
	try {
		$p7obj->unlock([$key,$cert]);
		TESTFAIL();
	} catch IDX::PKI::Error::Crypto with {
		TESTOK();
	};
	try {
		$p7obj=parse IDX::PKI::Crypto::PKCS7::EnvelopedData($p7string,"PEM",
															[$key,$cert]);
		TESTFAIL();
	} catch IDX::PKI::Error::Crypto with {
		TESTOK();
	};
};

test "pkcs7Encrypt / pkcs7Decrypt" => sub {
	my $ciphertext = pkcs7Encrypt($data,$certfile);
	TEST($ciphertext =~ m|^[A-Za-z0-9+/=]*$|) || warn "Doesn't look like Base64 to me...\n";
	TEST(pkcs7Decrypt($ciphertext, $keyfile, $certfile) eq $data) ||
	  warn "Decrypted data isn't the same as original data.";
};

test "pkcs7Decrypt with wrong key" => sub {
	my $ciphertext = pkcs7Encrypt($data,$cacertfile);
	TEST($ciphertext =~ m|^[A-Za-z0-9+/=]*$|) ||
	  warn "Doesn't look like Base64 to me...\n";
	try {
		pkcs7Decrypt($ciphertext, $keyfile, $certfile);
		TESTFAIL;
	} catch IDX::PKI::Error::Crypto with {
		TESTOK();
	};
};
