#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: PrivateKey.pm,v 1.17 2003/08/01 13:30:10 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Crypto/PrivateKey.pm,v $

package IDX::PKI::Crypto::PrivateKey;

$IDX::PKI::Crypto::PrivateKey::VERSION = '0.0.1';

use strict;

use IDX::PKI::Error;
use IDX::PKI::Crypto::Tools;

=pod "

=head1 NAME

IDX::PKI::Crypto::PrivateKey - handling of private keys in IDX-PKI

=head1 METHODS

=over 4

=item I<new($keylength)>

Constructs an RSA private key "ex nihilo" with length $keylength. It is
not stored in a file by default (use setKeyFile() if needed).

=cut "

sub new
  {
    my ($that,$keysize,$key_params) = @_;
    my %params = decode_params ($key_params) if defined($key_params);
    my $result = "";

        $result=run_openssl("genrsa",$keysize);

	unless ($result =~ m/BEGIN/) {
		throw IDX::PKI::Error::Crypto("Error when generating a private key",
									  -OpenSSL=>$result);
	};
	return bless {key=>$result,keysize=>$keysize,params=>$key_params},$that;
}

=pod "

=item I<< parse($string, $pass) >>

Loads the private key from its text representation in $string,
optionally using $pass as an encrypting password (the latter feature
is not implemented).

=cut "

sub parse {
	my ($self, $key) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
		'-details' => "No key") unless ($key);

	return bless ({key => $key}, $self);
}

=pod "

=item I<< load($file, $pass) >>

Loads the private key from $file, optionally using $pass as an encrypting
password (the latter feature is not implemented).

=cut "

sub load {
	my ($class,$file,$passfile)=@_;
	my $self = parse($class, file2string($file));
	$self->{keyfile} = $file;
	$self->{passfile} = $passfile if (defined $passfile);
	

    my $config_file = $file;
    $config_file =~ s/\.key$/.cnf/;
	$config_file = $config_file.'.cnf' if ("$config_file" eq "$file");
    if ( -f $config_file)   {
        my $params = file2string($config_file);
        $self->{params} = $params;
    }

	return $self;
}


=pod "

=item I<openssl_passargs($in_or_out)>

Returns the list arguments suitable for passing the pass phrase
arguments to the openssl shell tool. At the moment, passing the pass
phrase over a pipe is not supported so it systematically gets written
to the disk. $in_or_out should be "passin" or "passout" depending on the
required direction (see L<openssl>).

=item I<openssl_params()>

Same as I<openssl_passargs()> but also prepends the parameters to find
the private key (this of course means that it needs to be saved to
disk first). I<$self> needs to have been created with I<load()> (or
I<setKeyFile()> must already have been called). The required initial
C<-key>, C<-inkey>, C<-signkey> or such is not returned, so the typical
use of this method goes like this:

      my $result=run_openssl("x509","-signkey",$thekey->openssl_params(),...)

=cut "

sub openssl_passargs {
	my ($self,$in_or_out)=@_;
	my $passfile=$self->getPasswordFile();

	throw IDX::PKI::Error::Internal("WRONG_CONTEXT") unless wantarray();
	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") unless
	  ($in_or_out =~ m/^pass(in|out)$/);

	if ($passfile) {
		return ("-$in_or_out",$passfile);
	} else {
		return ("-$in_or_out","pass:"); # Prevents OpenSSL from stupidly
		# asking for the passphrase on the terminal
	};
}




=pod "

=item I<getKey()>

Returns the C<key> of the object (possibly encrypted, unless if the
-forcedecrypt parameter is specified). THIS FUNCTION IS OBSOLETE,
use C<serialize()> instead.

=cut "

sub getKey
  {
    my ($self, %args) = @_;

	return $self->serialize();
}

=pod "

=item I<serialize($password)>

Serializes the key, using the provided password (optional). The result is
a string in PEM format ("BEGIN PRIVATE KEY" etc.).

=cut "

sub serialize {
	my ($self)=@_;
	if (defined $self->{key})
	  {
			return $self->{key};
	  }
	elsif (defined $self->{keyfile})
	  {
			return file2string($self->{keyfile});
	  }
}

=pod "

=item I<serialize_params($params)>

Serializes the key parametrs (engine or vendor specified).

=cut "


sub serialize_params {
    my ($self)=@_;
    my $ret = "";

    if(defined $self->{params})   {
        my %params = decode_params($self->{params});

        while (my ($key,$value) = each %params)   {
            $value =~ s/,/\n/g;
            $ret = $ret."$key $value\n";
        }
    }

    return $ret;
}


=pod "

=item I<getKeyFile()>

Returns the name of a file containing C<key> (possibly crypted). $file
must be the name of a file in which to save the key if it was not
saved yet (for example, because we created it). If the key has no file
yet (or cannot have one - think hardware devices), undef is
returned. There is no automatic creation of files, to prevent unwanted
copies of higly security-sensitive key material; on the other hand,
if the object was created with load(), the file it was loaded from
is returned.

=cut "


sub getKeyFile { my $self=shift; "$self->{keyfile}"; }

=pod "

=item I<setKeyFile($file)>

Sets the filename under which to save the key. May raise an exception
if the key is located in a hardware token (support for such tokens is
not implemented yet). Returns I<$file>.

=cut "

sub setKeyFile {
	my ($self,$name)=@_;
	string2file($self->{key}, $name);
	$self->{keyfile}=$name;

	if (defined($self->{params})) {
	    my $config_file = $name;
	    $config_file =~ s/\.key$/.cnf/;
		$config_file = $config_file.'.cnf' if ("$config_file" eq "$name");
		string2file($self->serialize_params(),$config_file);
		$self->{keyconfigfile}=$config_file;
	}
	
	return $name;
}


=pod "

=item I<getOpensslParams($application)>

Returns a list of extra parameters to pass to /usr/bin/openssl (actually,
the binary pointed to by option I<openssl> in configuration section
I<paths>) in order to actuate this private key (calling this function in
scalar context is an error). Example of possible return values :

  ("-key", "/tmp/some/where", "-passin", "%")

for a password-protected private key (see also
L<IDX::PKI::Crypto::Tools/run_data_and_passwords_pipe_openssl()>).

I<$application> is a usage qualifier (one of C<rsa> (for key details),
C<genrsa>, C<req>, C<CA>, C<sign>, C<decrypt>, C<smime_sign> or
C<smime_decrypt>) that allows this method to return different values for
different openssl subcommands.

Throws an B<IDX::Error::Crypto> error if calling OpenSSL with this method
is not supported (e.g. C<genrsa> for hardware tokens often requires using
a vendor-provided support program). This also applies if the private key
is not linked to a file (use I<setKeyFile()> if needed).

=cut "


sub getOpensslParams {
	throw IDX::Error::Internal("ILLEGAL_CONTEXT") unless wantarray();

	my ($self, $application) = @_;

	my $key_file = $self->{keyfile};
	throw IDX::Error::Crypto ("The private key must be linked to a file")
		unless (defined $key_file);

	throw IDX::Error::Internal("WRONG_NUMBER_ARGS")
		unless (defined $application);

	my @ret; # Stop laughing please.
	if ($application =~ m/^smime/) {
		push (@ret, "-inkey");
	} elsif ($application eq "sign") {
		push (@ret, "-signkey");
	} elsif ($application eq "CA") {
		push (@ret, "-keyfile");
	} elsif ($application eq "rsa") {
		push (@ret, "-in");
	} else {
		push (@ret, "-key");
	}
	push (@ret, "$key_file");

	my $in_or_out = ($application eq "genrsa" ? "passout" : "passin");

	if (my $passfile = $self->getPasswordFile()) {
		push (@ret, "-$in_or_out", "file:$passfile");
	} else {
		push (@ret, "-$in_or_out", "pass:"); # Prevents OpenSSL from stupidly
		# asking for the passphrase on the terminal
	}


	return @ret;
}



=pod "

=item I<getPassword()>

Returns the C<password> of the object, which is needed to decrypt the key.
WARNING: there is no support for password-protected private keys in
IDX-PKI yet, and this function always returns undef.

=cut "

#pod ends "
sub getPassword
  {
    my $self = shift;

	return undef unless ((defined $self->{password}) or
						 (defined $self->{passfile}));

    if (defined $self->{password})
	  {
		return $self->{password};
	  }
	else
	  {
		return file2string($self->{passfile});
	  }
  }

=item I<getPasswordFile([$file])>

Returns the C<password> of the object, which is needed to decrypt the key.

=cut "

#pod ends "
sub getPasswordFile
  {
    my $self = shift;

	return undef unless ((defined $self->{password}) or 
						 (defined $self->{passfile}));

    if (defined $self->{passfile})
	  {
		return $self->{passfile};
	  }
	else
	  {
        my $name = shift;
        if(defined $name)
		  {
			string2file($self->{password}, $name);
			return $name;
		  }
		else
		  {
            throw IDX::PKI::Error::Internal("INCORRECT_ARGS");
		  }
	  }
  }

=pod "

=item I<getAttribute ($attr)>

Example of use: $pkey->getAttribute ("pk_modulus_hex");

Returns the value of a certificate attribute.  Recognized values for
I<$attr>:

=over 4

=item *

C<pk_modulus_hex>

The modulus of the (public) key, in hex, caps for hex letters. This
format is suitable for finding the certificate(s) associated with this
key (see L<pk_modulus_hex/IDX::PKI::Crypto::X509>).

=back

=item I<get_attributes()>

Returns a reference to the hash table containing all the attributes.

=cut "

# This undocumented function :-) builds the attribute cache.
sub parse_private_key {
	my ($self)=@_;

	return if (exists $self->{attr});

	my $parsed=run_data_pipe_openssl($self->getKey(),
			"rsa","-noout","-modulus",
			$self->getOpensslParams("rsa"));

	unless ($parsed =~ m/Modulus=([0-9A-F]+)$/i) {
		throw IDX::PKI::Error::Crypto("Cannot parse private key",
									  -OpenSSL=>$parsed);
	};

	$self->{attrs}->{pk_modulus_hex}=$1;
}

sub get_attributes {
	my ($self) = @_;
	$self->parse_private_key();

	return $self->{attrs};
}

sub getAttribute {
	my ($self, $attr) = @_;
	$self->parse_private_key();

	return $self->{attrs}->{$attr};
}

sub decode_params {
    my ($args) = @_;
    my %ret;
	
    my @chapter = split(';',$args);
    foreach my $item (@chapter)   {
        my @opts = split (' ',$item,2);
        if (scalar(@opts)>1)   {
            $ret{$opts[0]} = $opts[1];
        }
		else   {
			$ret{$opts[0]} = "";
		}
    }

    return %ret;
}

=pod "

=back

=cut "


eval(join '',<main::DATA>) || die $@ unless caller();

1;

__END__

#############################################

package IDX::PKI::Crypto::PrivateKey;

use strict;

use IDX::PKI::Debug (1);
use IDX::Test;

# All tests are integration tests, because they all want to know where the
# openssl binary is.
use IDX::PKI::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}


number_of_tests(6);

my $keyfile=new IDX::Test::Filename();
my $cnffile=$keyfile;
$cnffile =~ s/\.key$/.cnf/;
$cnffile = $cnffile.'.cnf' if ("$cnffile" eq "$keyfile");
my $key;
my $keystring;

test "key generation" => sub {
	$key=new IDX::PKI::Crypto::PrivateKey(512,join(" ",@ARGV));
	$key->setKeyFile($keyfile);
	TEST("$keyfile" eq $key->getKeyFile());
	TEST(-f $cnffile && -s _) if (scalar(@ARGV));
	TEST(run_openssl("rsa","-passin", "pass:", "-text", "-in", $keyfile, @ARGV))
	  =~ m/prime/i;
};

test "serialization" => sub {
	$keystring=$key->serialize();
};

test "test with direct data" => sub
{
	my $key = parse IDX::PKI::Crypto::PrivateKey($key);
	TEST($key->getKey());

	TEST(!defined $key->getPassword());
};

test "test with files" => sub
{
	my $dir = new IDX::Test::Directory();
	my $keyfile = "$dir/keyfile";

	string2file( $keystring, $keyfile );

	my $key = load IDX::PKI::Crypto::PrivateKey($keyfile);

};


test "getOpensslParams(\"rsa\")" => sub {
	my @params=$key->getOpensslParams("rsa");
	TEST("$params[0]" eq "-in");
	TEST("$params[1]" eq "$keyfile");
	TEST($params[2] eq '-passin');
	TEST($params[3] eq 'pass:');
	if (scalar(@ARGV))   {
		TEST($params[4] eq  $ARGV[0]);
		TEST($params[5] eq  $ARGV[1]);
	}
};

test "getAttribute" => sub {
	TEST($key->getAttribute ("pk_modulus_hex") =~ m/^[0-9A-F]+$/);
};
