#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

package IDX::PKI::Crypto::SignedBlob;

$IDX::PKI::Crypto::SignedBlob::VERSION = '0.0.1';

use strict;

use MIME::Base64;
use IDX::PKI::Error;
use IDX::PKI::FileManip;
use IDX::PKI::Crypto::Tools;
use IDX::PKI::Crypto::PrivateKey;
use IDX::PKI::Crypto::X509;

=pod "

=head1 NAME

B<IDX::PKI::Crypto::SignedBlob> - Module to handle signed data

=cut "

my %vars = (
			signed => undef,	# Signed BLOB.
			blob   => undef,	# Base BLOB (to sign).
			type   => undef		# User defined data which allows to identify the BLOB.
			);

=pod "

=head1 METHODS

=over 4

=item I<new()>

Creates a new instance of the class.

=over 4

=item C<-data>

Data containing the typed signed data (type of the object, followed by the
object, signed);

=item C<-blob>

Serialization of the object;

=item C<-type>

User-defined data for the BLOB type. Can only be constituted by digits and
letters;

=item C<-object>

Replaces B<-blob> and B<-type> with a "live" version of the object;

=back

C<-object> can not be provided together with C<-data>. C<-type> can only
be provided together with C<-object>.

=cut "

sub new
{
	my ($that, %args) = @_;
	my $class = ref($that) || $that;
	my $self = {
		%vars,
	};

	bless $self, $class;

	test_args (\%args, "-data", "-object", "-blob", "-type");

	if (defined $args{-data}) {
		($self->{type}, $self->{signed}) =
			$args{-data} =~ m/(\S*)\n(.*)/s;
	}

	if (defined $args{-object}) {
		$self->{blob} = encode_base64($args{-object}->serialize());
		$self->{type} = ref($args{-object});
	}

	if (defined $args{-blob}) {
		$self->{blob} =  encode_base64($args{-blob});
	}

	if (defined $args{-type}) {
		$self->{type} = $args{-type};
	}
	return $self;
}

=pod "

=item I<sign()>

Sign the object, with provided private key. If an error occurs, returns a
message about this error.

=over 4

=item C<key>

B<IDX::PKI::Crypto::PrivateKey>;

=item C<cert>

B<IDX::PKI::Crypto::X509>.

=back

=cut "

sub sign
{
	my ($self, %args) = @_;

	test_args (\%args, '-key', '-cert');

	throw IDX::PKI::Error::Crypto("Nothing to Sign")
		unless (defined $self->{blob});

	my $key = $args{-key};
	my $cert = $args{-cert};

	throw IDX::PKI::Error::Crypto("Missing Argument")
		unless ((defined $key) and (defined $cert));

	my $dir = new IDX::PKI::FileManip::TempDirectory();

	string2file($cert->serialize(), $dir."/cert.pem");
	string2file($self->{blob}, $dir."/infile");

	my ($signed, $error) = run_openssl
		("smime", "-sign", "-signer", $dir."/cert.pem", "-in", $dir."/infile",
		 "-inkey", $key->setKeyFile($dir."/key.pem"));

	throw IDX::PKI::Error::Crypto("Couldn't sign: $error")
		unless (($error eq "") or ($error =~ m/unable to write 'random state'/));

	$self->{signed} = $signed;
	return $error;
}

=pod "

=item I<verify()>

Verifies if the ths signature is valid. If a error occurs, throws an
B<IDX::PKI::Error>.

=over 4

=item C<-chain>

Path to the file containing the certification chain;

=item C<-cadir>

Directory containing the CA certificates;

=item C<-signer>

Path to a file where to store the certificate (*).

=back

One of C<-chain> or C<-cadir> si mandatory.

(*) optional parameters

=cut "

sub verify
{
	my ($self, %args) = @_;

	my $chain = $args{"-chain"};
	my $cadir = $args{"-cadir"};
	my $signer = $args{"-signer"};

	throw IDX::PKI::Error::Internal ("ASSERTION_FAILED", "-details" =>
									 "No signed data available")
		unless ($self->{"signed"});

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless ($chain || $cadir);

	my ($blob, $error) =
		run_data_pipe_openssl ($self->{"signed"}, "smime", "-verify",
							   ($signer ? ("-signer", $signer) : ()),
							   ($chain ? ("-CAfile", $chain) : ()),
							   ($cadir ? ("-CApath", $cadir) : ()),
							   );

	throw IDX::PKI::Error::Crypto
		("Couldn't verify digital signature", '-details' => $error,
		 '-cafile' => $chain, "cadir" => $cadir,
		 '-signeddata' => $self->{'signed'},
		 "-signer" => $signer)
		unless ($error =~ /Verification Successful/is );

	$self->{"blob"} = $blob;
	return $error;
}

=pod "

=item I<get_type()>

Returns the user-defined type.

=cut "

sub get_type()
{
	my ($self) = @_;
	return $self->{type};
}

=pod "

=item I<get_object()>

Returns the object (unsigned).

=cut "

sub get_object
{
	my ($self) = @_;
	return decode_base64($self->{blob});
}

=pod "

=item I<serialize()>

Returns the content of the object in a form suitable for transport.

=cut "

sub serialize {
	my ($self) = @_;

	throw IDX::PKI::Error::Internal("INCORRECT_CALL_SEQUENCE")
		unless (defined $self->{signed});

	return $self->{type}."\n".$self->{signed};
}

=pod "

=back

=cut "

eval (join '', <main::DATA>) || die $@ unless caller();
1;

__END__

#############################################

package IDX::PKI::Crypto::SignedBlob;
use strict;

use IDX::PKI::Debug(1);
use IDX::Test;
use IDX::PKI::Crypto::X509;
use IDX::PKI::Crypto::PrivateKey;

# All tests are integration tests, because they all want to know where the
# openssl binary is.
use IDX::PKI::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}

number_of_tests(6);

my $cadir = new IDX::Test::Directory ();
{
	my ($cacert, $rootcacert) = (IDX::Test::certificates ())[0,6];
	my ($cahash, $rootcahash) = map {IDX::Test::getCertHash ($_)}
		($cacert, $rootcacert);
	local *FILE;
	open (FILE, ">", "$cadir/$cahash.0") ||
		die "Can't write to $cadir/$cahash.0: $!";
	print FILE $cacert;
	close (FILE);
	open (FILE, ">", "$cadir/$rootcahash.0") ||
		die "Can't write to $cadir/$rootcahash.0: $!";
	print FILE $rootcacert;
	close (FILE);
}

my (undef, $data, $keydata) = IDX::Test::certificates();
my $x509 = scalar (parse IDX::PKI::Crypto::X509 ($data));
my $key = scalar (parse IDX::PKI::Crypto::PrivateKey ($keydata));

my $type = "TestBlob";

my $blob;

test "new" => sub
{
	$blob = new IDX::PKI::Crypto::SignedBlob
		(-blob => $data, -type => $type);
	TEST(defined $blob);
};

test "sign" => sub
{
	$blob->sign (-key => $key, -cert => $x509);
	TESTOK;
};

my $smime;

test "serialize" => sub
{
	$smime = $blob->serialize();
	TEST(defined $smime);
};

my $sblob;

test "unserialize" => sub
{
	$sblob = new IDX::PKI::Crypto::SignedBlob (-data => $smime);
	TEST(defined $sblob);
};

test "verify" => sub {
	$sblob->verify ("-cadir" => $cadir);
	TESTOK;
};

test "verify type" => sub
{
	TEST ($sblob->get_type() eq $type);
};
