#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

package IDX::PKI::Crypto::Tools;

use strict;

# IPC::Open3 does not do the job because it confuses STDOUT with
# descriptor 0 and STDERR with descriptor 1 - which just isn't the
# case under mod_perl. So we do the pipework ourselves.
use Fcntl;
use IO::Pipe;

use IDX::PKI::Error;
use IDX::PKI::FileManip;
use IDX::PKI::Config;
use IDX::PKI::Debug (0);

use Exporter;
use vars qw(@ISA @EXPORT @EXPORT_OK);
@ISA = qw( Exporter );

@EXPORT = qw(file2string string2file
			 run_openssl run_data_pipe_openssl
			 run_data_and_passwords_pipe_openssl run_data_program
			 serialize_form test_args);

@EXPORT_OK = @EXPORT;


=pod "

=head1 NAME

IDX::PKI::Crypto::Tools

=head1 FUNCTIONS

=over 4

=item I<test_args()>

Tests if arguments are allowed.  The first argument is the hashtable
containing the argument list, followed by the list of the allowed argument
key.

=cut "

sub test_args {
	my ($hash, @rest) = @_;
	my %allowed = ();
	foreach my $allowed (@rest) {
		$allowed{$allowed} = 1;
	}

	foreach my $arg (keys (%$hash)) {
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", -Arg => $arg)
			unless ($allowed{$arg});
	}

	return 1;
}

=pod "

=item I<file2string()>

Opens the file given as an argument and returns its content.

=cut "

sub file2string
{
	my ($file) = @_;

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") unless ($file);

	open (FILE, $file) || throw IDX::PKI::Error::IO ("-IOfile" => $file);
	my $contents = join ('', <FILE>);
	close(FILE);

	return $contents;
}

=pod "

=item I<string2file($string, $file)>

Saves the content of $string into a new file named $file.

=cut "

sub string2file
{
	my ($text, $file) = @_;
	unless ((defined $text) && (defined $file))
	{
		throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS");
	}

	open(FILE, ">$file") || throw IDX::PKI::Error::IO( -IOfile => $file);
	if ($text =~ m/./) {
		# Bug in Perl 5.6: print '' returns false
		print FILE $text  || throw IDX::PKI::Error::IO( -IOfile => $file);
	}
	close(FILE) || throw IDX::PKI::Error::IO( -IOfile => $file);
}


=pod "

=item I<loaddir ($class, $dir)>

Loads cryptographic objects (such as certificates or CRLs) from the
directory I<$dir>. Only hash-style filenames are considered (C<<
<hash>.<num> >>); the files must be in PEM format. Returns an array of
I<$class> objects.

=cut "

sub loaddir {
	my ($class, $dir, @args) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless ($class && UNIVERSAL::can ($class, "load") && $dir);

	local *DIR;
	opendir (DIR, $dir) || throw
		IDX::PKI::Error::IO ("Can't read directory", "-IOfile" => $dir);
	my @hashs = map {(m/^([a-z0-9]+.\d+)$/i && -r "$dir/$_") ? ($1) : ()}
		readdir (DIR);
	closedir (DIR);

	return map {$class->load ("$dir/$_")} @hashs;
}


=pod "

=item I<serialize_form ()>

Generates a string based on the given form.

=cut "

sub serialize_form {
	my %hash = @_;
	my $result = "";
	foreach my $key (keys (%hash)) {
		my $val = $hash{$key};
		if (ref ($val) eq "ARRAY") {
			foreach my $trueval (@$val) {
				$result .= $key."=".$trueval."\n";
			}
		} else {
			$result .= $key."=".$val."\n";
		}
	}
	return $result;
}

=pod "

=item run_openssl($arg1,...)

Runs openssl in a controlled way (not a subshell) so that one can pass
arbitrary filenames and options. In list context, returns a pair of
strings consisting of the output of STDOUT and the output of STDERR;
in scalar context, both are merged. $? is also set in the usual way.

=item run_data_pipe_openssl($data,$arg1,...)

Runs openssl in a controlled way similar to run_openssl() above, except that
$data is fed to the standard input of openssl.

=item run_data_and_passwords_pipe_openssl($data,\@passwords,$arg1,...)

As above, but also feeds one or more passwords to Openssl. All
occurences of a lone "%" in the argument list occuring in the
right places (that is, just after an option starting with "-pass"),
are substituted with the appropriate pipework so that the
corresponding password is fed to openssl in a secure way using pipes.

To be a little bit clearer, here is an example of the correct invocation to
transcode a private key from one password ("foo") to another ("bar"):

  my ($newkey,$error)=run_data_and_passwords_pipe_openssl($key,
     [ "foo", "bar" ], "rsa", "-des3", "-passin", "%", "-passout", "%");

=cut "

sub run_openssl {
	run_data_pipe_openssl(undef,@_);
}

sub run_data_pipe_openssl {
	my ($data,@args)=@_;
	run_data_and_passwords_pipe_openssl($data,[],@args);
}

sub run_data_and_passwords_pipe_openssl {
	my ($data,$passref,@cmdline)=@_;

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS")
		unless (ref($passref) && ref($passref) eq "ARRAY");
	my @passwords=@$passref;

	my $config = the IDX::PKI::Config ();
	my $binary = $config->getval ("paths", "openssl");
 	($binary && -x $binary) ||
		throw IDX::PKI::Error::Install("Missing openssl binary ($binary)");

	# One cannot be too careful with locales when parsing output of
	# shell programs... This also prevents people from monkeying with
	# env:-style passwords.
	local %ENV=('PATH' => '');

	my $opensslout=new IO::Pipe();
	my $opensslerr; $opensslerr=new IO::Pipe() if wantarray;
	defined(my $pid=fork()) ||
		throw IDX::PKI::Error::Memory("Cannot fork()");

	if ($pid) { # Parent reads, child skips this block
		$opensslout->reader();
		$opensslerr->reader() if wantarray;

		# Kludged together from "perldoc -f select":
		my ($return,$err)=("","");
		my @selected=([$opensslout => \$return]);
		push(@selected,[$opensslerr => \$err]) if wantarray;

		my $selected="";
		foreach my $s (@selected) {
			my ($file,undef)=@$s;
			vec($selected,fileno($file),1) = 1;
		};

		while (scalar grep {scalar(@$_)} @selected) {
			DEBUG($selected);
			select((my $rout=$selected),undef,undef,undef);
			foreach my $s (@selected) {
				my ($file,$variable)=@$s;
				next if (!defined $file);
				next unless vec($rout,fileno($file),1);

				my $ret=sysread($file,$$variable,4096,length($$variable));
				if (! defined $ret) {
					throw IDX::PKI::Error::IO ("Cannot communicate with ".
											   "OpenSSL");
				} elsif (! $ret) {
					DEBUG("closing ".fileno($file));
					vec($selected,fileno($file),1)=0;
					@$s=();
					close($file);
				} else {
					DEBUG($$variable);
				};
			};
		};

		waitpid($pid,0);
		if (($? >> 8) == 250) {
			throw IDX::PKI::Error::Install("Cannot exec() $binary");
		}
		return wantarray ? ($return, $err) : $return;

	} else { # Child exec's openssl, parent skips this block
		require POSIX; # Ordinary version of exit() does global destruction,
 		# which is not what we want (even from a forked process this may
    	# have unwanted consequences such as saying goodbye on network
		# sockets, destroying temporary files, etc.)

		$opensslerr->writer() if wantarray;
		$opensslout->writer();
		fcntl($opensslout,F_SETFD,0); # Prevents close-on-exec
		POSIX::dup2(fileno($opensslout),1);
		POSIX::dup2((wantarray ? fileno($opensslerr) : fileno($opensslout)),
					 2);

		# Pipes must be in-scope until exec at the end of the function.
		my @passpipes_openssl;
		# Set up stdin and password pipes, either from /dev/null or
		# from a forked Perl:
		if ((defined $data) || (@passwords)) {
			# You get a forked Perl as stdin either way in this case
			# (using /dev/null would save us nothing since we have to
			# pay the fork price anyway)
			my $opensslin=new IO::Pipe;

			# We cannot use IO::Pipes here (because we cannot get
			# their fileno's before having called reader() or writer()
			# on them).
			my @passpipes_perl;
			foreach my $dummy (@passwords) {
				my ($read,$write)=((new IO::Handle),(new IO::Handle));
				pipe($read,$write) || POSIX::_exit(250);
				push(@passpipes_perl,$write);
				push(@passpipes_openssl,$read);
			};

			defined(my $pid=fork()) || POSIX::_exit(250);
			if (! $pid) {
				# child skips this block, grandchild hangs around writing
				# data and passwords and then terminates silently.
				foreach my $pipe (@passpipes_openssl) {
					close $pipe;
				};
				foreach my $pipe (@passpipes_perl) {
					# Hopefully the passwords are less than 4k...
					# So we use the OS's pipe buffers and we don't
					# have to set up yet another select().
					my $pass=shift @passwords;
					# BUGWARE BIG ALERT: OpenSSL seems to require a
					# newline below. Tests will fail if this behaviour
					# change.
					print $pipe "$pass\n";
					close $pipe;
				};
				$opensslin->writer();
				print $opensslin $data if (defined $data);
				close $opensslin;
				close STDERR;
				POSIX::_exit(0); # Nobody cares about this process's exit code.
			};

			# Child prepares pipes.
			$opensslin->reader();
#			foraeach my $pipe (@passpipes_perl) { close $pipe; };
			foreach my $pipe ($opensslin,@passpipes_openssl) {
				fcntl($pipe,F_SETFD,0); # Prevents close-on-exec
			};
			POSIX::dup2(fileno($opensslin),0);
		} else { # You just get /dev/null
			local *DEVNULL;
			open(DEVNULL,"</dev/null");
			fcntl(DEVNULL,F_SETFD,0);
			POSIX::dup2(fileno(DEVNULL),0);
		}

		# Child finally execs Openssl, using a special exit code
		# to signal parent that exec failed.
		# We have to hack @cmdline before exec, using the fileno's
		# of the passpipes we have...
		my @passfds=map {fileno($_)} @passpipes_openssl;
		foreach my $i (1..$#cmdline-1) {
			if (($cmdline[$i] =~ m/^-pass/) && ($cmdline[$i+1] eq "%")) {
				$cmdline[$i+1] = "fd:".(shift @passfds);
				last if (! @passfds);
			}
		}
		DEBUG(join(" ",@cmdline));
		exec($binary,@cmdline) || POSIX::_exit(250);

	}
}

sub run_data_program {
	my ($data, @cmdline)=@_;

	my $config = the IDX::PKI::Config ();
	my $binary = $config->getval ("paths", "openssl");
 	($binary && -x $binary) ||
		throw IDX::PKI::Error::Install("Missing openssl binary ($binary)");

	# One cannot be too careful with locales when parsing output of
	# shell programs... This also prevents people from monkeying with
	# env:-style passwords.
	local %ENV=('PATH' => '');

	my $outpipe=new IO::Pipe();
	my $errpipe; $errpipe=new IO::Pipe() if wantarray;
	defined(my $pid=fork()) ||
		throw IDX::PKI::Error::Memory("Cannot fork()");

	if ($pid) { # Parent reads, child skips this block
		$outpipe->reader();
		$errpipe->reader() if wantarray;

		# Kludged together from "perldoc -f select":
		my ($return,$err)=("","");
		my @selected=([$outpipe => \$return]);
		push(@selected,[$errpipe => \$err]) if wantarray;

		my $selected="";
		foreach my $s (@selected) {
			my ($file,undef)=@$s;
			vec($selected,fileno($file),1) = 1;
		};

		while (scalar grep {scalar(@$_)} @selected) {
			select((my $rout=$selected),undef,undef,undef);
			foreach my $s (@selected) {
				my ($file,$variable)=@$s;
				next if (!defined $file);
				next unless vec($rout,fileno($file),1);

				my $ret=sysread($file,$$variable,4096,length($$variable));
				if (! defined $ret) {
					throw IDX::PKI::Error::IO ("Cannot communicate with ".
											   "OpenSSL");
				} elsif (! $ret) {
					DEBUG("closing ".fileno($file));
					vec($selected,fileno($file),1)=0;
					@$s=();
					close($file);
				} else {
					DEBUG($$variable);
				};
			};
		};

		waitpid($pid,0);
		if (($? >> 8) == 250) {
			throw IDX::PKI::Error::Install("Cannot exec() $binary");
		}
		return wantarray ? ($return, $err) : $return;

	} else { # Child exec's openssl, parent skips this block
		require POSIX; # Ordinary version of exit() does global destruction,
 		# which is not what we want (even from a forked process this may
    	# have unwanted consequences such as saying goodbye on network
		# sockets, destroying temporary files, etc.)

		$errpipe->writer() if wantarray;
		$outpipe->writer();
		fcntl($outpipe,F_SETFD,0); # Prevents close-on-exec
		POSIX::dup2(fileno($outpipe),1);
		POSIX::dup2((wantarray ? fileno($errpipe) : fileno($outpipe)), 2);

		if (defined $data) {
			# You get a forked Perl as stdin either way in this case
			# (using /dev/null would save us nothing since we have to
			# pay the fork price anyway)
			my $inpipe=new IO::Pipe;

			defined(my $pid = fork()) || POSIX::_exit(250);
			if (! $pid) {
				$inpipe->writer();
				print $inpipe $data if (defined $data);
				close $inpipe;
				close STDERR;
				POSIX::_exit(0); # Nobody cares about this process's exit code.
			};

			# Child prepares pipes.
			$inpipe->reader();
			POSIX::dup2(fileno($inpipe),0);
		} else { # You just get /dev/null
			local *DEVNULL;
			open(DEVNULL,"</dev/null");
			fcntl(DEVNULL,F_SETFD,0);
			POSIX::dup2(fileno(DEVNULL),0);
		}

		exec(@cmdline) || POSIX::_exit(250);
	}
}

=back

=cut "

1;

eval(join '',<main::DATA>) || die $@ unless caller();

__END__

#######################################################"

use strict;

use IDX::Test;
use IDX::PKI::Error;
use IDX::PKI::FileManip;

# All tests are integration tests, because they all want to know where the
# openssl binary is.
use IDX::PKI::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}

number_of_tests (9);

	my $data = << 'DATA';
MIIDKjCCAtSgAwIBAgIBADANBgkqhkiG9w0BAQQFADCBkzELMAkGA1UEBhMCRlIx
DjAMBgNVBAcTBVBhcmlzMRUwEwYDVQQKEwxUZXN0IENvbXBhbnkxHDAaBgNVBAsT
E0NvbXB1dGVyIFJlc3NvdXJjZXMxDDAKBgNVBAsTA1BLSTEVMBMGA1UEAxMMVGVz
dCBSb290IENBMRowGAYJKoZIhvcNAQkBFgtjYUB0ZXN0LmNvbTAeFw0wMTA2MjUx
MDI3MDFaFw0wNjA2MjQxMDI3MDFaMIGTMQswCQYDVQQGEwJGUjEOMAwGA1UEBxMF
UGFyaXMxFTATBgNVBAoTDFRlc3QgQ29tcGFueTEcMBoGA1UECxMTQ29tcHV0ZXIg
UmVzc291cmNlczEMMAoGA1UECxMDUEtJMRUwEwYDVQQDEwxUZXN0IFJvb3QgQ0Ex
GjAYBgkqhkiG9w0BCQEWC2NhQHRlc3QuY29tMFwwDQYJKoZIhvcNAQEBBQADSwAw
SAJBAL2hSuDI7dedWERpNrVH7CHEDyharkiCXzg5x64Whhq3Z8Q4WXQZGjoiqBUG
clqwvKZSUXqbwb+0gOxk2KzrlM0CAwEAAaOCAQ8wggELMA8GA1UdEwEB/wQFMAMB
Af8wHQYDVR0OBBYEFAOjfTIpLa12A+7x2VS6aMpLmQiSMIHABgNVHSMEgbgwgbWA
FAOjfTIpLa12A+7x2VS6aMpLmQiSoYGZpIGWMIGTMQswCQYDVQQGEwJGUjEOMAwG
A1UEBxMFUGFyaXMxFTATBgNVBAoTDFRlc3QgQ29tcGFueTEcMBoGA1UECxMTQ29t
cHV0ZXIgUmVzc291cmNlczEMMAoGA1UECxMDUEtJMRUwEwYDVQQDEwxUZXN0IFJv
b3QgQ0ExGjAYBgkqhkiG9w0BCQEWC2NhQHRlc3QuY29tggEAMBYGA1UdEQQPMA2B
C2NhQHRlc3QuY29tMA0GCSqGSIb3DQEBBAUAA0EAsn7aXDdukQ8ygDxpfQSn1PW1
R5eOLlC4A8ntJjVBRHmEIgdwbYiKfhAcA7fhs5MgCG7xLrCQSjfMSEvpvQhpeA==
DATA

test "run_openssl (1/2)" => sub {
	my $openssl=run_openssl("version");
	TEST($openssl =~ m/^openssl/i) || do {
		chomp $openssl; warn "$openssl";
	};
	TEST($openssl !~ m/^uninitialized/i) || do {
		chomp $openssl; warn "$openssl";
	};

	my ($cert,undef)=IDX::Test::certificates();
	$openssl=run_data_pipe_openssl($cert,"x509","-text","-noout");
	TEST($openssl =~ m/Serial Number/) || do {
		chomp $openssl; warn "$openssl";
	};
};

test "run_openssl (2/2)" => sub {
	my ($out,$err)=run_openssl("--bogusoption");
	TEST($out eq "");
	TEST($err);

	($out,$err)=run_openssl("version");
	TEST($err eq ""); # Yes, this is OpenSSL. Welcome onboard.
};

test "run_openssl: real use of passwords" => sub {
	my (undef,undef,$key,undef)=IDX::Test::certificates();
	TEST($key !~ m/ENCRYPTED/);
	my $encryptedkey=run_data_and_passwords_pipe_openssl($key,["secret"],
					"rsa","-des3","-passout","%");
	TEST($encryptedkey =~ m/ENCRYPTED/);
	my $decryptedkey=run_data_and_passwords_pipe_openssl($key,["secret"],
					"rsa","-passin","%");
	TEST($decryptedkey =~ m/\Q$key\E/) || die $decryptedkey;
};

test "run_openssl: trying to disturb it with passwords" => sub {
	my ($cert,undef)=IDX::Test::certificates();
	my $openssl=run_data_and_passwords_pipe_openssl($cert,["foo","bar"],
					"x509","-text","-noout");
	TEST($openssl =~ m/Serial Number/) || do {
		chomp $openssl; warn "$openssl";
	};

};

test "string2file -> file2string" => sub
{
	my $dir = new IDX::Test::Directory();
	my $file = "$dir/data";

	string2file( $data, $file );

	my $result = file2string( $file );

#	$result .= "toto";

	TEST( $result eq $data );
};


test "test_args with correct args" => sub
{
	my $args = { dupont=>'dupond', toto=>'titi' };

	try {
		test_args( $args, 'dupont', 'toto' );
		TESTOK;
	} otherwise {
		TESTFAIL;
	}

};

test "test_args with bad args" => sub
{
	my $args = {dupont=>'dupond', toto=>'titi'};

	try {
		test_args ($args, 'dupot', 'toto');
		TESTFAIL;
	} catch IDX::PKI::Error::Internal with {
		TESTOK;
	} otherwise {
		TESTFAIL;
	};
};

my $ciphertext;
test "non-regression in OpenSSL's handling of pipe passwords" => sub {
	my $plaintext=<<REGRESSION;
Hello, I send you this ciphertext in order to have your advice.
REGRESSION
	my ($ciphertext,$text,$err);
  	($ciphertext, $err) = run_data_pipe_openssl ($plaintext,
	"des","-e","-pass","pass:boguspwd");
	($text,$err)=run_data_and_passwords_pipe_openssl ($ciphertext,
		["boguspwd"],"des","-d","-pass","%");
	TEST($text eq $plaintext);
};

test "run_data_program" => sub {
	my $in = $data;
	my ($out, $err) = run_data_program($in, "/bin/cat");

	TEST($in eq $out);
}
