#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

package IDX::PKI::Crypto::X509;

$IDX::PKI::Crypto::X509::VERSION = '0.0.1';

use strict;

use IDX::PKI::Error;
use IDX::PKI::DN;
use IDX::PKI::FileManip;
use IDX::PKI::Crypto::Tools qw(file2string string2file test_args run_openssl run_data_pipe_openssl);

=pod "

=head1 NAME

IDX::PKI::Crypto::X509 - Perl extension for basic handling x509 Certificates

=head1 SYNOPSIS

use IDX::PKI::Crypto::X509;

=head1 CONSTANTS

=over 4

=item I<%keyUsageExtensions>

A multivalued hash (hash of array references) whose keys are X509v3
key usage names ("keyUsage", "extendedKeyUsage" and "nsCertType") and
whose values are symbolic constants for appropriate values of same.

=cut "

our %keyUsageExtensions;
%keyUsageExtensions =
    ("keyUsage" => ["digitalSignature",
					"nonRepudiation",
					"keyEncipherment",
					"dataEncipherment",
					"keyAgreement",
					"keyCertSign",
					"cRLSign",
					"encipherOnly",
					"decipherOnly"],
	 "extendedKeyUsage" => ["emailProtection",
							"clientAuth",
							"serverAuth",
							"codeSigning",
							"timeStamping",
							"OCSPSigning"],
	 "nsCertType" => ["NSclient",
					  "NSserver",
					  "NSemail",
					  "NSobjsign",
					  "NSreserved",
					  "NSsslCA",
					  "NSemailCA",
					  "NSobjCA"],
	 );

=pod "

=back

=head1 METHODS

=over 4

=item I<parse($string, [$type])>

Constructs a certificate object from its string representation (C<PEM> or
C<DER>, according to I<$type>, default C<PEM>).  In an array context,
returns an array of certificates if the input string contains several:
this only works in I<PEM>, and is handy for parsing CA certificate
bundles.

=cut "

sub parse {
	my ($class, $string, $mode) = @_;
	my $der = ($mode && $mode =~ m/DER/i);

	if ($mode && $mode =~ m/IE/i) {
		$string =~ s/-----(BEGIN|END) [A-Z0-9]+-----/-----$1 PKCS7-----/;
		my ($cert, $error) = run_data_pipe_openssl
		  ($string, "pkcs7", "-print_certs");
		$string = $cert;
		throw IDX::PKI::Error::Crypto
		  ("Could parse from PKCS7", -Openssl => $error)
			unless ( length( $error ) == 0 );
	}

	if (!$der && wantarray && $string =~ m/BEGIN.*CERTIFICATE/) {
		my @certs = map {scalar(parse($class,$_))}
		  ( $string =~ m/(-+BEGIN.*CERTIFICATE-+$
						  (?:.|\n)*?
						  ^-+END.*CERTIFICATE-+)$/gmx);
		throw IDX::PKI::Error::Crypto("MALFORMED_MESSAGE") unless
		  @certs;
		return @certs;
	}
	;

	my $self = bless {( $der ? "DERcert" : "cert" ) => $string},$class;

	my ($dump, $err) = run_data_pipe_openssl
	  ($string, "x509", ($der ? ("-inform", "der") : () ), "-text",
	   "-modulus","-issuer","-subject",
	   "-nameopt","sep_multiline","-nameopt","dn_rev");

	if ((length($err) != 0) or (not defined $dump) or (length($dump) == 0)) {
		throw IDX::PKI::Error::Crypto
		  ("Error parsing certificate", -Openssl => $err,
		   ((defined $dump) ? (-dump => $dump) : ()),
		   ((!$der) ? (-cert => $string) : ()));
	}

	my %att;
	my $i;
	my $buffer;
	my @table;

	# X509 Version
	($att{x509version})   = ($dump =~ m/\n\s+Version:\s*(\d)\s*\(\w*\)\n/);
	# Serial Number
	my ($serial)          = ($dump =~ m/\n\s+Serial Number:\s+([0-9]+)(.+)\n/);
	# We present the serial in hex, like NS and IE do.
	$att{serial} = sprintf("%X",$serial);
	# Openssl wants an even number of chars.
	$att{"serial"} = "0".$att{"serial"} if (length ($att{"serial"}) % 2);
	# Signature Algorithm
	($att{sig_algorithm}) = ($dump =~ m/\n\s+Signature Algorithm:\s+(\w+)\n/);
	# DN Issuer is read "better" below.
	# Not Before (Date)
	($att{not_before})    = ($dump =~ m/\n\s+Not Before:\s+(.+)\n/);
	$att{not_before_zulu}=_textDate2ISOTimeGMT($att{not_before});
	# Not After (Date)
	($att{not_after})     = ($dump =~ m/\n\s+Not After :\s+(.+)\n/);
	$att{not_after_zulu}=_textDate2ISOTimeGMT($att{not_after});

	# DN Subject is read "better" below.
	# Public Key Algorithm
	($att{pk_algorithm})  = ($dump =~ m/\n\s+Public Key Algorithm:\s+(\w+)\n/);
	(defined $att{pk_algorithm}) ||
	  throw IDX::PKI::Error::Crypto("Cannot parse public key algorithm",
									-Openssl=>$dump);
	# Public Key Size (in bits)
	($att{pk_bitsize})    = ($dump =~ m/\n\s+\w+\s+Public Key: \((\d+) bit\)/);
	# Public Key Modulus (both in hex and bignum form)
	($att{pk_modulus_bn})    = ($dump =~ m/\n\s+Modulus \(\d+ bit\):\n\s+(.+)\n\s+Exponent/s);
	throw IDX::PKI::Error::Crypto("No modulus in X509 Certificate")
	  unless (defined $att{pk_modulus_bn});
	$att{pk_modulus_bn} =~ s/\s//gs;

	($dump =~ m/^Modulus=\s*(\S.*)$/mix) ||
	  throw IDX::PKI::Error::Crypto("Cannot parse hex modulus",
									-Openssl=>$dump);
	$att{pk_modulus_hex}=$1;

	# Signature (verification of the sig by OpenSSL is implicit)
	($att{sig_value})     =
	  ($dump =~ m/\n\s+Signature Algorithm:\s+\w+\n(\s[\s\na-f0-9:]+)/s);

	throw IDX::PKI::Error::Crypto("No signature in X509 Certificate")
	  unless (defined $att{sig_value});

	$att{sig_value}=~ s/\s//gs;

	# DNs of issuer and subject (list form)
	($dump =~ m/^issuer=\s*$ \n
	 ( (?: ^ \s+ .* = .* $ \n)+ )
	 ^subject=\s*$ \n
	 ( (?: ^ \s+ .* = .* $ \n)+ )
	 /mx) ||
	   throw IDX::PKI::Error::Crypto("Multiline DNs not found",
									 -Openssl=>$dump);
	my @DNs = ($1, $2);
	($att{dn_issuer}, $att{dn_subject}) = map {
		s/^\s+(.*)$ \n/$1,/mgx;
		s/,$//;
		scalar (parse IDX::PKI::DN($_));
	} @DNs;
	# The RFC2253 DN is no longer parsed from OpenSSL because
	# it returns a bogus value for "Email" RDNs anyway. We use
	# IDX::PKI::DN instead and save a fork.

	# X509v3 Extensions
	($buffer)    = ($dump =~ m/\n\s+X509v3 extensions:\n(.+)Signature Algorithm:/s);
	if (defined $buffer) {
		my %extensions;
		(@table) = ($buffer =~ m/\s+([A-Za-z0-9\ ]+):\s*(critical)?\s*\n\s+(.+)\n/g);
		while (@table) {
			my $extension = shift @table;
			my $critical  = shift @table;
			my $value     = shift @table;
			$extensions{$extension} = { critical => (defined $critical ? 1:0),
										value => $value };
		}
		$att{extensions} = { %extensions };
	}


	$self->{attr} = \%att;
	return $self;
}


=pod "

=item I<load($file, [$format])>

Constructs a certificate object from a file in DER or PEM format (see
also L</parse>). The extended form of I<parse()> in array context for
the PEM format is also supported in I<load()>.

=cut "

sub load {
	my ($class, $file, @args) = @_;
	return parse ($class, file2string ($file), @args);
}


=pod

=item I<loaddir ($dir)>

Loads certificate from the directory I<$dir>. See
L<IDX::PKI::Crypto::Tools/loaddir>. Certificates are sorted so that
self-signed ones (root CAs), if any, come first.

=cut

sub loaddir {
	my @certs = IDX::PKI::Crypto::Tools::loaddir (@_);
	return ((grep {$_->getSubjectDN () eq $_->getIssuerDN ()} @certs),
			(grep {$_->getSubjectDN () ne $_->getIssuerDN ()} @certs));
}


=pod "

=item I<parse_cert()>

Parses a certificate to extract the attributes. Obsolete, Internal use only.

=cut "

# This function was first used to perform a bogus parse-on-demand thing.
# Let's continue this way (bogus, that is).
sub parse_cert {
	my ($self)=@_;
	my $selftoo=$self->parse($self->{cert});
	while (my ($k,$v)=each %$selftoo) {
		$self->{$k}=$v;
	};
}

=pod "

=item I<getAttribute ($attr)>

=over 4

Returns the value of a certificate attribute.  Following value can be
provided as a value for I<$attr> argument:

=over 4

=item C<x509version>

=item C<serial>

=item C<sig_algorithm>

=item C<dn_issuer>

Returns a B<IDX::PKI::DN> object.

=item C<not_before>

=item C<not_after>

Return the appropriate dates in a human readable format

=item C<not_before_zulu>

=item C<not_after_zulu>

Return the date in "Zulu" format (e.g. "200207101010Z").

=item C<dn_subject>

Returns a B<IDX::PKI::DN> object.

=item C<pk_algorithm>

=item C<pk_bitsize>

=item C<pk_modulus_hex>

The modulus of the public key, in hex form (hex letters in caps). For
RSA, this is the product of p and q and therefore a good public key
identifier (that can be matched with the corresponding attribute in
L</IDX::PKI::Crypto::PrivateKey>).

=item C<pk_modulus_bn>

The modulus in bignum form (hex values from 00 to ff, hex letters in
lowercase, separated by colons)

=item C<extensions>

=item C<sig_value>

=back

The C<extensions> value return a reference to a hash table. The keys of this hash table is the extension label, and the value a reference to a hash table. For each extension, the corresponding hash table have for keys C<critical> and C<value>, with, as value, those corresponding to the concerned extension.

=cut

#pod ends "

sub getAttribute
{
	my ($self,$attr) = @_;
	return $self->{attr}->{$attr};
}


=pod "

=item I<get_attributes()>

Return a reference to the hash table containing all the attributes.  This
function is for compatibility only.

=cut

#pod ends "

sub get_attributes
{
	my ($self) = @_;

	$self->parse_cert() unless (defined $self->{attr});

	return $self->{attr};
}

=pod "

=item I<getExtensions($category,$criticity)>

Returns the list of all valid boolean X509v3 usage extensions found in
the certificate (in scalar context, the number of such
extensions). $category is optional and may be either "keyUsage",
"extendedKeyUsage" or "nsCertType" (in which case the search is
restricted to the relevant certificates); $criticity is optional and
may be either "CRITICAL" or "NONCRITICAL" (in which case the search is
also restricted in the requested fashion).

=cut "

sub getExtensions {
	my %opensslcats=
	  (
	    keyUsage => "X509v3 Key Usage",
        extendedKeyUsage => "X509v3 Extended Key Usage",
	    nsCertType => "Netscape Cert Type"
	  );
	my %opensslnames=
	  (
	   # The source for this cruft is in the sources of openssl
	   # (crypto/x509v3/v3_bitst.c)

	   "Digital Signature", "digitalSignature",
       "Non Repudiation", "nonRepudiation",
	   "Key Encipherment", "keyEncipherment",
	   "Data Encipherment", "dataEncipherment",
	   "Key Agreement", "keyAgreement",
	   "Certificate Sign", "keyCertSign",
	   "CRL Sign", "cRLSign",
	   "Encipher Only", "encipherOnly",
	   "Decipher Only", "decipherOnly",

	   "SSL Client", "NSclient",
	   "SSL Server", "NSserver",
	   "S/MIME", "NSemail",
	   "Object Signing", "NSobjsign",
	   "Unused", "NSreserved",
	   "SSL CA", "NSsslCA",
	   "S/MIME CA", "NSemailCA",
	   "Object Signing CA", "NSobjCA",

	   # This one is in crypto/objects/obj_dat.h instead - go figure.
	   "TLS Web Server Authentication","serverAuth",
	   "TLS Web Client Authentication","clientAuth",
	   "Code Signing","codeSigning",
	   "E-mail Protection","emailProtection",
	   "Time Stamping","timeStamping",
	   "OCSP Signing", "OCSPSigning",
	  );

	my ($self,$category,$criticity)=@_;

	my @answers;
	foreach my $cat ($category ? ($category) : (keys %opensslcats)) {
		my $stash=$self->getAttribute("extensions")->{$opensslcats{$cat}};
		next unless (exists $stash->{value} && defined $stash->{value});
		next if ($criticity &&
				 ($criticity eq "CRITICAL" xor $stash->{critical}));
		push @answers, map {$opensslnames{$_} ||
							  throw IDX::PKI::Error::Internal("UNIMPLEMENTED",
								-details=>"Unknown cert extension ``$_''");
						} (split m/, /,$stash->{value});
	};
	return @answers;
}

=pod "

=item I<getIssuerDN()>

Returns the issuer DN. In scalar context, returns an RFC2253-like DN (see
L<IDX::PKI::DN/string ()>).  In list context, returns a list of strings of
the same form as in L<IDX::PKI::DN>.

=cut "

sub getIssuerDN {
    my $self = shift;
	$self->parse_cert() unless (defined $self->{'attr'});
	my $dn = $self->{'attr'}->{'dn_issuer'};

	return wantarray ? $dn->canonical () : $dn->string ();
}

=pod "

=item I<getSubjectDN()>

Return the subject DN (same format as above).

=cut

#pod ends "

sub getSubjectDN {
    my $self = shift;
	$self->parse_cert() unless (defined $self->{'attr'});
	my $dn = $self->{'attr'}->{'dn_subject'};

	return wantarray ? $dn->canonical () : $dn->string ();
}

=pod "

=item I<getSerial()>

Returns the serial number of the certificate, as an hexadecimal string.

=cut "

sub getSerial {
	my ($self)=@_;
	return $self->{attr}{serial};
}

=pod "

=item I<getIntegerSerial()>

Returns the serial number of the certificate, as an integer (decimal).

=cut "

sub getIntegerSerial {
	my ($self)=@_;
	return hex($self->getSerial());
}

=pod "

=item I<serialize($mode)>

Returns the certificate, in a form suitable for transport. Possible
formats are "DER", "PEM" and "IE" (a Microsoft-specific PKCS#7
kludge). Default is "PEM".

=cut "

sub serialize {
	my ($self,$mode) = @_;
	  $mode = uc($mode) || "PEM";
	($mode =~ m/^(PEM|DER|IE)$/i) ||
	  throw IDX::PKI::Error::Internal ('INCORRECT_ARGS');

	if (defined $self->{cert}) {
		if($mode eq "PEM") {
			return $self->{cert};
		} elsif ($mode eq "DER") {
			if (!defined $self->{DERcert}) {
				$self->{DERcert}=run_data_pipe_openssl
				  ($self->{cert}, "x509","-inform","pem","-outform","der");
			}
			return $self->{DERcert};
		} elsif ($mode eq "IE") {
			if (!defined $self->{IEcert}) {
				my $dir = new IDX::PKI::FileManip::TempDirectory();
				string2file($self->{cert}, "$dir/cert.pem");
				# Passing on stdin is not supported :-(
				my ($pkcs7, $err) = run_openssl
				  ("crl2pkcs7","-certfile","$dir/cert.pem","-nocrl");
				
				throw IDX::PKI::Error::Crypto
				  ("Could not transform to PKCS7", -Openssl => $err)
					unless ( length( $err ) == 0 );
				$pkcs7 =~ s/-----(BEGIN|END) PKCS7-----/-----$1 CERTIFICATE-----/;
				$self->{IEcert} = $pkcs7;
			}
			return $self->{IEcert}
		}
	} elsif (defined $self->{DERcert}) {
		$self->{cert} = run_data_pipe_openssl
		  ($self->{DERcert}, "x509","-inform","der","-outform","pem");
		return $self->serialize($mode);
	} else {
		throw IDX::PKI::Error::Internal("No data for this certificate ?!");
	}
}

=pod "

=item I<getEmail()>

Returns the email address(es) found in the certificate. It
searches first in the subjectAltname, then in the DN but only returns
one occurence of each address. In Array context returns an array of
all addresses found, in scalar context only returns the first one, or undef.

=cut "

sub getEmail {
	my $self = shift;

    my @emails;

    my $email_altname = $self->getAttribute('extensions')->
	  {"X509v3 Subject Alternative Name"}->{value};
	if ($email_altname) {
		@emails= ($email_altname =~
				 m/email:([a-zA-Z0-9@\-+_.]*)/g);
	};

	my @dn = $self->getSubjectDN();
	while(my ($k,$v)=splice(@dn,0,2)) {
		push (@emails, $v) if ($k =~ m/email/i);
	}

	if (wantarray) {
		my %emails=map { ($_,1) } @emails; # Uniquification
		return keys %emails;
	} else {
		return $emails[0];
	}
}


=pod "

=item I<getUserID ()>

Returns a "reasonably unique" ID for the certificate.  By decreasing
order of preference, returns the UID, the CN (both from the DN), the
email address (see L</getEmail ()>), and the I<string ()> form of the
subject DN (see L</getSubjectDN ()>).

=cut "

sub getUserID {
	my ($self) = @_;
	my %dn = $self->getSubjectDN ();

	# Try DN's UID and CN.
	foreach my $key (qw (uniqueIdentifier CN)) {
		return $dn{$key} if (exists $dn{$key});
	}

	# Emails.
	my $email = $self->getEmail ();
	return $email if (defined ($email));

	# If found nothing, returns the full DN as a string.
	return scalar ($self->getSubjectDN ());
}

=pod "

=item I<getAltName()>

Returns the SubjectAltName of the certificate, or undef.

=cut "


sub getAltName {
  my ($self, $x509) = @_;

  my $extensions      = $x509->getAttribute ('extensions');
  my $mailext         = $extensions->{"X509v3 Subject Alternative Name"}->
	{value};
  # For now it's restricted to DNS alternative names
  my (undef, $servername)  = ($mailext =~ m/(DNS|URI):(.*)$/);
  return $servername;
}



=pod "

=item I<deserialize ($string)>

Returns the the object from its I<serialize()>d representation.  See also
L</parse()>.

=cut "

sub deserialize {
	my ($class, $string) = @_;

	return parse ($class, $string);
}


=pod "

=item I<verify ($cert, $cachain)>
=item I<verify ($cert, $cadir)>

Verifies the certificate. I<$cert> may be an object, a file or a string.
The second argument may be either a file (supposed to contain the full CA
chain) or a directory (containing hash-format files of CA certificates).

=cut "

sub verify
{
	my ($cert, $cachain) = @_;

	throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS")
		unless ($cert && $cachain);

	my $self;
	if (UNIVERSAL::isa ($cert, "IDX::PKI::Crypto::X509")) {
		$self = $cert;
	} elsif ($cert !~ /\n/ && -f $cert) {
		$self = load IDX::PKI::Crypto::X509 ($cert);
	} elsif ($cert =~ m/BEGIN.*CERT/) {
		$self = parse IDX::PKI::Crypto::X509 ($cert);
	} else {
		$self = parse IDX::PKI::Crypto::X509 ($cert, "DER");
	}

	my $result = run_data_pipe_openssl
		($self->serialize (), "verify",
		 (-d $cachain ? "-CApath" : "-CAfile"), $cachain);

	throw IDX::PKI::Error::Crypto ("Invalid certificate",
		"-details" => $result, "-certificate" => $cert) if ($result !~ m/OK/);

	return $result;
}


=pod "

=back

=head1 CONVENIENCE FUNCTIONS

=item I<_textDate2ISOTimeGMT()>

Returns the numerical ISO time GMT from the given text date as found in
openssl's output. Example: "Jul  9 10:01:00 2001 GMT" becomes "200107091001Z".
In array context, returns a list similar to that returned by gmtime(),
except that $wday and $yday slots will be undef.

=cut "

sub _textDate2ISOTimeGMT {
	my ($textDate) = @_;

	my ($monthAsText,$day,$hour,$min,$sec,$year)=
		$textDate =~ m/(\S+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s+(\d+).*/;

	my @months=qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);

	my $month;
	for my $i (0..11) { $month=$i+1 if ($months[$i] eq $monthAsText)};
	throw IDX::PKI::Error::Internal("Invalid date",
									-date => $textDate) if (! $month);

	if (wantarray) {
		return ($sec,$min,$hour,$day,$month,$year-1900);
	} else {
		return sprintf("%04d%02d%02d%02d%02d%02dZ",
		  $year,$month,$day,$hour,$min,$sec);
	};
}


=pod "

=head1 SEE ALSO

=back

=cut


eval(join '',<main::DATA>) || die $@ unless caller();

1;

__END__


  ###########################################################"
package IDX::PKI::Crypto::X509;
use strict;

$INC{"IDX/PKI/Crypto/X509.pm"}=1; # Shuts "function redefined" warnings

use IDX::Test;
use IDX::PKI::Debug(1);

# All tests are integration tests, because they all want to know where the
# openssl binary is.
use IDX::PKI::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}

number_of_tests (12);

my $x509 = << "CERTIFICATE_EXAMPLE";
-----BEGIN CERTIFICATE-----
MIIDRTCCAu+gAwIBAgIBADANBgkqhkiG9w0BAQQFADCBkzELMAkGA1UEBhMCRlIx
DjAMBgNVBAcTBVBhcmlzMRUwEwYDVQQKEwxUZXN0IENvbXBhbnkxHDAaBgNVBAsT
E0NvbXB1dGVyIFJlc3NvdXJjZXMxDDAKBgNVBAsTA1BLSTEVMBMGA1UEAxMMVGVz
dCBSb290IENBMRowGAYJKoZIhvcNAQkBFgtjYUB0ZXN0LmNvbTAeFw0wMTA3MDQw
NzA4NTJaFw0wMjA3MDQwNzA4NTJaMIGJMQswCQYDVQQGEwJGUjEOMAwGA1UEBxMF
UGFyaXMxFTATBgNVBAoTDFRlc3QgQ29tcGFueTEcMBoGA1UECxMTQ29tcHV0ZXIg
UmVzc291cmNlczETMBEGA1UEAxMKVGVzdENsaWVudDEgMB4GCSqGSIb3DQEJARYR
Y2xpZW50QGlkZWFseC5jb20wXDANBgkqhkiG9w0BAQEFAANLADBIAkEArDINNjS9
QcEr77DbQ7+M+s1z5l8gLqvHiaSk6mzWqHU+kgPfyFDPqXwBGjXp7f1ZutX6eFnj
kjeH9yoV8TH1RQIDAQABo4IBNDCCATAwCQYDVR0TBAIwADAdBgNVHQ4EFgQUQECZ
bsO/lDCeBXALb6WX4r2iN4owgcAGA1UdIwSBuDCBtYAUBO2qADyE7qqeOmqyRSCi
ztd3DcShgZmkgZYwgZMxCzAJBgNVBAYTAkZSMQ4wDAYDVQQHEwVQYXJpczEVMBMG
A1UEChMMVGVzdCBDb21wYW55MRwwGgYDVQQLExNDb21wdXRlciBSZXNzb3VyY2Vz
MQwwCgYDVQQLEwNQS0kxFTATBgNVBAMTDFRlc3QgUm9vdCBDQTEaMBgGCSqGSIb3
DQEJARYLY2FAdGVzdC5jb22CAQAwHAYDVR0RBBUwE4ERY2xpZW50QGlkZWFseC5j
b20wFgYDVR0SBA8wDYELY2FAdGVzdC5jb20wCwYDVR0PBAQDAgXgMA0GCSqGSIb3
DQEBBAUAA0EAiYn3CHxKPHn3touS9gf4fZ8ZJ5OLYpB22jSCb8cKpOAuuoYypoII
50CALzW+lGrJD/fKk6t3jg7iDyK0xZ6lmg==
-----END CERTIFICATE-----
CERTIFICATE_EXAMPLE

test "_textDate2ISOTimeGMT()" => sub {
	TEST(_textDate2ISOTimeGMT("Jul  9 10:01:00 2001 GMT") eq "20010709100100Z");
	TEST(_textDate2ISOTimeGMT("Jan  01 00:00:00 2000 GMT") eq "20000101000000Z");
};

test "Constructor and accessors" => sub {
	my $cert = parse IDX::PKI::Crypto::X509($x509);

	TEST( $cert->getAttribute ("pk_algorithm") eq "rsaEncryption" );
	TEST ($cert->getIssuerDN () eq 'Email=ca@test.com,CN=Test Root CA,'.
		  'OU=PKI,OU=Computer Ressources,O=Test Company,L=Paris,C=FR' ) ||
		  warn $cert->getIssuerDN ();
	TEST( $cert->getAttribute ("extensions")
		  ->{"X509v3 Basic Constraints"}->{critical} eq '0' );
	TEST( $cert->getAttribute ("extensions")
		  ->{"X509v3 Subject Key Identifier"}->{value}
		  eq '40:40:99:6E:C3:BF:94:30:9E:05:70:0B:6F:A5:97:E2:BD:A2:37:8A' );

	TEST($cert->getAttribute("not_before") eq "Jul  4 07:08:52 2001 GMT");
	TEST($cert->getAttribute("not_before_zulu") eq "20010704070852Z") ||
	  die $cert->getAttribute("not_before_zulu");
	TEST($cert->getUserID() eq "TestClient") ||
	  die $cert->getUserID();

	my $keyUsages=join(" ",$cert->getExtensions("keyUsage"));
	TEST($keyUsages =~ m/nonRepudiation/) ||
	  warn "keyUsages = $keyUsages\n";

	TEST($cert->getExtensions("keyUsage","NONCRITICAL") == 3) ||
	  warn "keyUsages criticity misdetected\n";

	TEST(! $cert->getExtensions("keyUsage","CRITICAL"));
	TEST(! $cert->getExtensions("extendedKeyUsage"));
};

test "DER->PEM conversion on bad format." => sub {

	try {
		my $cert = parse IDX::PKI::Crypto::X509($x509, "DER" );
		TESTFAIL();
	} catch IDX::PKI::Error::Crypto with {
		TESTOK();
	} otherwise {
		TESTFAIL();
	};
};

test "DER <-> PEM conversion" => sub {
	my $cert = parse IDX::PKI::Crypto::X509($x509);
	my $der=$cert->serialize("DER");
	$cert=parse IDX::PKI::Crypto::X509($der,"DER");

	TEST( $cert->getAttribute ("pk_algorithm") eq "rsaEncryption" );
	TEST ($cert->getIssuerDN () eq 'Email=ca@test.com,CN=Test Root CA,'.
		  'OU=PKI,OU=Computer Ressources,O=Test Company,L=Paris,C=FR' ) ||
		  warn $cert->getIssuerDN ();
	TEST( $cert->getAttribute ("extensions")
		  ->{"X509v3 Basic Constraints"}->{critical} eq '0' );
	TEST( $cert->getAttribute ("extensions")
		  ->{"X509v3 Subject Key Identifier"}->{value}
		  eq '40:40:99:6E:C3:BF:94:30:9E:05:70:0B:6F:A5:97:E2:BD:A2:37:8A' );
};

test "Parsing bundles" => sub {
	my $bundle=$x509 x 3;
	my @certs=parse IDX::PKI::Crypto::X509($bundle);
	TEST(scalar(@certs) == 3);
	foreach my $cert (@certs) {
		TEST( $cert->getAttribute ("pk_algorithm") eq "rsaEncryption" );
	};
};

test "Serializations" => sub {
	my $certin = parse IDX::PKI::Crypto::X509($x509);
	foreach my $source (qw/PEM DER/) {
		my $sin =$certin->serialize($source);
		my $cert = parse IDX::PKI::Crypto::X509($sin, $source);
#		print "Source = $source ".length($sin)."\n";
		foreach my $mode (qw/PEM DER IE/) {
			my $ser = $cert->serialize($mode);
#			print "Source = $source, Mode = $mode ".length($ser)."\n";
			TEST(length($ser) > 0);
			my $des = parse IDX::PKI::Crypto::X509($ser, $mode);
			TEST(length($des) > 0);
		}
	}
};

my $cert=parse IDX::PKI::Crypto::X509 ($x509);

test "DN as a list" => sub {
	my @DN=$cert->getSubjectDN();
	TEST($DN[$#DN-1]eq 'C') || warn(join("-",@DN));
	my %DN=@DN;
	TEST($DN{Email} =~ m/\@/);
};

test "Moduli" => sub {
	TEST($cert->getAttribute ("pk_modulus_bn") =~ m/:/);
	TEST($cert->getAttribute ("pk_modulus_bn") =~ m/^[:0-9a-f]+$/);
	TEST($cert->getAttribute ("pk_modulus_hex") =~ m/^[0-9A-F]+$/);
};

test "getEmail" => sub {
	my @emails = $cert->getEmail();
	TEST (scalar @emails) || warn "Emails :".Data::Dumper::Dumper (\@emails);
	TEST ($cert->getEmail() eq 'client@idealx.com') ||
		warn "Previous cert email: ",$cert->getEmail();

	$cert = scalar (parse IDX::PKI::Crypto::X509
					((IDX::Test::certificates())[1]));
	TEST (! $cert->getEmail()) || warn "Client cert email: ",$cert->getEmail();
};

my ($ca, undef, undef, undef, undef, undef, $rootca) =
	IDX::Test::certificates ();
my (undef, $cert1) = IDX::Test::certFiles ();
my $cadir = new IDX::Test::Directory ();

# Writes things in the directory.
foreach my $cert ($ca, $rootca) {
	my ($result, $error) =
		run_data_pipe_openssl ($cert, "x509", "-noout", "-hash");
	die "Couldn't get the certificate hash:\n$error"
		if ($error || (! $result));
	die "Unexpected hash $result" unless ($result =~ m/^([a-z0-9]+)$/i);
	$result = $1;

	local *FILE;
	open (FILE, ">", "$cadir/$result.0") || die "Can't write to ".
		"$cadir/$result.0: $!";
	print FILE $cert;
	close (FILE);
}

test "Verify certificates" => sub {
	TEST (IDX::PKI::Crypto::X509::verify ($cert1, $cadir)) ||
		warn "Can't verify a file";
	$cert1 = load IDX::PKI::Crypto::X509 ($cert1);
	TEST (IDX::PKI::Crypto::X509::verify ($cert1, $cadir)) ||
		warn "Can't verify an object";
	TEST (IDX::PKI::Crypto::X509::verify ($cert1->serialize (), $cadir)) ||
		warn "Can't verify a PEM cert";
	TEST (IDX::PKI::Crypto::X509::verify ($cert1->serialize ("DER"),
										  $cadir)) ||
		warn "Can't verify a DER cert";
};

test "Verify an invalid certificate (must fail)" => sub {
	my $evilcert = IDX::Test::evilCerts ();
	try {
		IDX::PKI::Crypto::X509::verify ($evilcert, $cadir);
		TESTFAIL ();
	} catch IDX::PKI::Error::Crypto with {
		TESTOK ();
	}
};

test "loaddir ()" => sub {
	my @certs = loaddir IDX::PKI::Crypto::X509 ($cadir);

	TEST (scalar (@certs) == 2) || warn "CA certs: ".
		Data::Dumper::Dumper ([map {scalar ($_->getSubjectDN ())} @certs]);
	TEST (UNIVERSAL::isa ($certs[0], "IDX::PKI::Crypto::X509")) ||
		warn "Cert #0 is ".ref ($certs[0]);
	TEST ($certs[0]->getSubjectDN () eq IDX::Test::getCertDN ($rootca)) ||
		warn "Cert #0 DN:".$certs[0]->getSubjectDN ();
	TEST ($certs[1]->getSubjectDN () eq IDX::Test::getCertDN ($ca)) ||
		warn "Cert #1 DN:".$certs[1]->getSubjectDN ();
};
