#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::PKI::DN> - Support for DNs in IDX-PKI

I<"They can't be read, written, assigned, or routed. Other than that,
they're perfect"> (Marshall Rose -- see L</CCITT CURSING>)

=head1 SYNOPSIS

 use IDX::PKI::DN;

 # Object-style:

 my $dn=build IDX::PKI::DN("CommonName"=>"IDX-PKI OpCA", "OU" => "IRD",
      "OU" => "IDEALX", "C" => "FR");
 my @dn=$dn->canonical(); # "CommonName" becomes "CN" and
		# "FR" is downcased.
 print $dn->rfc2253();
 print $dn->string(); # Would differ if the DN contained an Email
 print $dn->to_openssl();

 print $dn->longClass(); # Examples are "strongAuthenticationUser"
						# and "organization"
 $dn->chop();

 # Plain function style:
 my @dn=("CommonName"=>"IDX-PKI OpCA", "OU" => "IRD",
      "OU" => "IDEALX", "C" => "FR");
 print IDX::PKI::DN::rfc2253(@dn);
 print IDX::PKI::DN::to_openssl(@dn);
 @dn=IDX::PKI::DN::canonical(@dn);
 # Etc. (see above). "chop" is not available in plain style
 # (perldoc -f splice).

 # Comparison between DNs:
 IDX::PKI::DN::compare ($dn, \@dn);
 IDX::PKI::DN::compare ("C=FR/OU=IDEALX/OU=IRD/CN=IDX-PKI OpCA",
 		\@dn);

=head1 DESCRIPTION

In IDX-PKI, a DN is a even-numbered list of values that resembles an
associative array (except there B<is> an order and there may be
multiple values for any given key). Multiple alternative AVAs are not
handled by IDX-PKI and will never be.

DNs are ordered from the "smallest" class to the "biggest" class
throughout IDX-PKI. Attribute types (the keys in the association) can
either be specified by short name (such as "CN"), by long name (such
as "commonName") or Here is a correct example:

  my @DN=( "CN" , "IDX-PKI OpCA",
           "OU", "IRD",
           "OU", "IDEALX",
           "C", "FR");

Here is an incorrect example that will trigger an exception in the
I<IDX::PKI::Error::Internal> class upon nearly every call in this
module:

  my @DN=( "C" , "FR",
           "OU", "IDEALX",
           "CN", "Something");

IDX-PKI DNs may either be objects (there is the build() constructor for
this purpose) or plain old lists (all methods in this package except
build() can also be used as functions).

=head1 METHODS

=over 4

=cut "

package IDX::PKI::DN;
use strict;

use vars qw($YesIWantBrokenDNs);
# Comment the following line if you care more about following
# standards than having a useful PKI
$YesIWantBrokenDNs = 1;

use IDX::PKI::Error;

###################### Constants ##################################

# On my left, the fields that will be presented at the interface
# outputs of the class. On my right, fields that are accepted as
# substitutes at interface input, and also are known IDs in OpenSSL
# and also in LDAP if at all possible. Pfew.

my %fields =
	(
	 "C"  => "countryName",
	 "O"  => "organizationName",
	 "OU" => "organizationalUnitName",
	 "ST" => "stateOrProvinceName",
	 "L"  => "localityName",
	 "CN" => "commonName",
	 "DC" => "domainComponent",

     "unstructuredName" => "unstructuredName",
     "SN" => "serialNumber", # openssl 0.7.0: "serialNumber" => "serialNumber"
     "G" => "givenName",     # openssl 0.7.0: "GN" => "givenName"
     "S" => "surname",       # openssl 0.7.0: "SN" => "surname"

	 "uniqueIdentifier" => "uniqueIdentifier",
	 "UID" => "userID",  # This thingy does not seem to have a meaning in LDAP
	 "Email" => "emailAddress",
);

my @rfc2253_kosher=qw(countryName organizationName organizationalUnitName
					  stateOrProvinceName localityName commonName givenName
					  surname);

# Is domainComponent something valid for IDX-PKI ?
# push @rfc2253_kosher, "domainComponent";

# From RFC2459. %OIDs need to contain at least those entries that are in
# %fields, but not in @rfc2253_kosher. But you can add entries beyond
# this requirement; this will buy equivalence between the numeric and
# string forms of the added OIDs.
my %OIDs=(
		  "emailAddress"=>"1.2.840.113549.1.9.1",
		  "domainComponent" => "0.9.2342.19200300.100.1.25",
		  "unstructuredName" => "1.2.840.113549.1.9.2",
          "serialNumber" => "2.5.4.5",
          "surname" => "2.5.4.4",
          "givenName" => "2.5.4.42",
		  );

=pod "

=item I<build(@DN)>

Returns a new object in the I<IDX::PKI::DN> class.

=cut "


sub build {
	my ($class,@DN)=@_;
	return bless [canonical(@DN)],$class;
}

=pod "

=item I<parse ($string)>

Constructs a list DN from a string DN. In scalar context, returns an
object.  In list context, returns the parsed DN. I<parse()> tries to
be smart in the handling of various DN forms; right now, RFC2253 and
the bogus OpenSSL encoding (see L</like_openssl()>) are supported.

=cut "

sub parse {
	my ($class, $dn) = @_;

	throw IDX::PKI::Error::Internal ('INCORRECT_ARGS')
	    unless ($class && $dn);

	my @dn = split m{[,/\|]\s*(?=$|\w+\s*=)}, $dn;
	if ($dn =~ m|^/|) {	# OpenSSL style
		@dn = reverse @dn;
	}
	@dn = map {m/^(\w+)\s*=(.*)$/} @dn;

	return wantarray ? canonical(@dn) : (bless [canonical(@dn)],$class);
}


=pod "

=item I<chop()>

Suppresses the leftmost (smallest, extent-wise: CN is smaller than
country) element of the DN and returns it.  In scalar context, returns
an I<IDX::PKI::DN> object; in list context, returns the DN list.

=cut "

sub chop {
	if (UNIVERSAL::isa ($_[0], "IDX::PKI::DN")) {
		my $dn = $_[0];
		splice (@$dn, 0, 2);
		return wantarray () ? $dn->canonical () : $dn;
	} else {
		my @dn = @_;
		splice (@dn, 0, 2);
		return wantarray () ? canonical (@dn) : build IDX::PKI::DN (@dn);
	};
}


=pod "

=item I<type2class ($type)>

Returns the LDAP name of a DN key ("type"). I<$type> may be either a short
or a long name.

=cut "

sub type2class {
	my ($longtype) = @_;
	my %classes =
		("organizationalUnitName" => "organizationalUnit",
		 "organizationName"       => "organization",
		 "countryName"            =>"country",
		 "domainComponent"        => "dcObject");

	my $class = $classes{$longtype};
	# Try again with considering $longtype was a short name.
	$class = $classes{short_to_long($longtype)} unless ($class);

	throw IDX::PKI::Error::Internal("UNIMPLEMENTED") unless ($class);
	return $class;
}


=pod "

=item I<getLeftRDNShortType()>

=item I<getLeftRDNLongType()>

=item I<getLeftRDNClass()>

=item I<getLeftRDNValue()>

Accessors for the leftmost (see L</chop>) component of a DN. For
example, if @dn=("O","IDEALX"), getLeftRDNShortType(@dn) is "O",
getLeftRDNLongType(@dn) is "OrganizationName", getLeftRDNClass(@dn)
yields "Organization" and getLeftRDNValue(@dn) returns "IDEALX".

=cut "

sub getLeftRDNShortType {
	my @DN = _theDN (@_);
	return $DN[0];
}

sub getLeftRDNLongType {
	my @DN = _theDN (@_);
	return $fields{$DN[0]};
}

sub getLeftRDNValue {
	my @DN = _theDN (@_);
	return $DN[1];
}

sub getLeftRDNClass {
	return type2class(getLeftRDNLongType(@_));
}


=pod "

=item I<getRightRDNShortType()>

=item I<getRightRDNLongType()>

=item I<getRightRDNValue()>

=item I<getRightRDNClass()>

Same functions, but for the rightmost (see L</chop>) component of a DN.

=cut "

sub getRightRDNShortType {
	my @DN = _theDN (@_);
	return $DN[-2];
}

sub getRightRDNLongType {
	my @DN = _theDN (@_);
	return $fields{$DN[0-2]};
}

sub getRightRDNValue {
	my @DN = _theDN (@_);
	return $DN[-1];
}

sub getRightRDNClass {
	return type2class(getRightRDNLongType(@_));
}


=pod "

=item I<getRDNValue ($element)>

Extract a RDN from the DN.  In scalar context, returns the first (or only)
occurence of I<$element>, in B<IDX::PKI::DN> list order.  In list value,
returns all occurences, in B<IDX::PKI::DN> list order.  This method is
only available in object style.

=cut "

sub getRDNValue
{
	my ($self, $element) = @_;

	throw IDX::PKI::Error::Internal ('INCORRECT_ARGS')
		unless (defined $self && UNIVERSAL::isa ($self, "IDX::PKI::DN"));

	throw IDX::PKI::Error::Internal ('INCORRECT_ARGS')
		unless (defined $element && $element ne '');

	my @dn = $self->canonical ();

	my @result;
	while (my ($key, $val) = splice (@dn, 0, 2)) {
		push (@result, $val) if (uc ($key) eq uc ($element));
	}

	return wantarray () ? @result : $result[0];
}

=pod "

=item I<_theDN(@args)>

Support function for handling the dual calling convention of all methods.
Please do not call it directly.

=cut "

sub _theDN {
	if (UNIVERSAL::isa($_[0],"IDX::PKI::DN")) {
		return @{$_[0]};
	} else {
		return canonical(@_);
	};
}

=pod "

=item I<canonical()>

Makes the DN canonical, in the sense that all attribute types are
coreced to the official syntax (e.g. "C","O","OU", "CN" etc.) and
those entries for which the syntax is known (such as "C") are checked
and corrected (e.g. 2 letters max. uppercase). Additionally, order is
checked for in the following sense: if there is a CN in the DN, it
must come before any "O","OU","ST" or "L". Failure to comply with the
Rules causes an exception of class IDX::PKI::Error::Internal to be raised.

canonical() is automatically called upon construction of an
I<IDX::PKI::DN> object and upon entry of all functions in plain
mode. Therefore, this module only ever handles canonicalized DNs.

=cut "

my %revOIDs = reverse %OIDs;

sub canonical {
	my (@args) = @_;
	return @{$args[0]} if (UNIVERSAL::isa ($args[0], "IDX::PKI::DN"));
	return canonical (@{$args[0]}) if (ref ($args[0]));

	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS") if (@args % 2);

	my @return;
	my $seenTypeAboveCN;
	my %fieldsrev;
	my %ucfields;
	while (my ($k, $v) = each %fields) {
		$fieldsrev{uc($v)} = $k;
		$ucfields{uc($k)} = $k;
	}
	my @chunks = @args;
	FIELD: while (my ($k, $v) = splice (@chunks, 0, 2)) {
		throw IDX::PKI::Error::Internal("INCORRECT_ARGS") if
			((!defined $k) || (!defined $v));
		if ($k =~ m/^[0-9.]*$/) { # literal OID in DN - may be frowned
			# upon in a future version of IDX-PKI
			if (exists $revOIDs{$k}) {
				$k = $revOIDs{$k};
			} else {
				push (@return, "$k", "$v");
				next FIELD;
			}
		}

		# Key canonicalization
		if (exists $fieldsrev{uc($k)}) {
			$k = $fieldsrev{uc($k)};
		} elsif (exists $ucfields{uc($k)}) {
			$k = $ucfields{uc($k)};
		} else {
			throw IDX::PKI::Error::Internal("INCORRECT_ARGS", "-details" =>
											"Unknown DN component $k");
		}

		# Value canonicalization and sanity checks
		if ($k =~ m/^(O|OU|ST|C|L)$/) {
			$seenTypeAboveCN++;
		} elsif ($k eq "CN") {
			throw IDX::PKI::Error::Internal("Wrong DN order", "-dn" => \@args)
				if ($seenTypeAboveCN && ! $YesIWantBrokenDNs);
		}

		if ($k eq "C") {
			throw IDX::PKI::Error::Internal("INCORRECT_ARGS", "-details" =>
											"bogus country $v") unless
											($v =~ m/^[A-Z][A-Z]$/i);
			$v = uc($v);
		}
		push (@return, $k, "$v");
	}

	return @return;
}

=pod "

=item I<to_openssl()>

Returns a string adequate for putting inside an OpenSSL configuration
file or SPKAC in order to create requests or certificates. Do not
forget the C<-preserveDN> option to C<openssl ca> though... Multiple
identical RDNs (such as OU's) are handled properly.

=cut "

sub to_openssl {
	my @dn=_theDN(@_);
	my $dntext="";my $i=0;
	while(@dn) {
		my ($k,$v)=splice @dn,-2,2;
		$k=$fields{$k} if ($fields{$k});
		$dntext.="$i.$k=$v\n";
		$i++;
	};
	return $dntext;
}


=pod "

=item I<like_openssl()>

Returns a bogus string DN encoding like OpenSSL seems to love emitting them
(reversed, slash-separated, with a leading slash). You guessed it,
I<like_openssl()> has nothing to do with I<to_openssl()>. Talk about
the robustness principle...

=cut "

sub like_openssl {
	my @dn=_theDN(@_);
	my @ret;
	while(my ($k,$v)=splice @dn,0,2) {
		unshift @ret,"$k=$v";
	};

	return join("/","",@ret);
}

=pod "

=item I<string()>

Outputs the DN in string form (reversed, comma-separated,
no spurious spaces).

=cut "

sub string {
	my @dn=_theDN(@_);
	my @prettyDN;
	while(my ($k,$v)=splice @dn,0,2) {
		push @prettyDN,"$k=$v";
	};
	return join(",",@prettyDN);
}

=pod "

=item I<rfc2253()>

Outputs the DN in RFC2253 form. This is almost like string() except
that some fields are forbidden in LDAP, and RFC2253 mandates that they
are rendered in their OID form: this is the case for "emailAddress".

=cut "


sub rfc2253 {
	my @dn=_theDN(@_);
	my @prettyDN;
	# OK this is an extremely rough approximation of RFC2253.
	while(my ($k,$v)=splice @dn,0,2) {
		if (! grep {$_ eq $k ||
					  ( (exists $fields{$k}) && ($_ eq $fields{$k}) )
				   } (@rfc2253_kosher)) {
			if (exists $OIDs{$k}) {
				$k=$OIDs{$k};
			} elsif (exists $OIDs{$fields{$k}}) {
				$k=$OIDs{$fields{$k}};
			} else {
				throw IDX::PKI::Error::Internal(
        "Unknown OID (please add appropriate constants to IDX::PKI::DN)",
												-missing=>$k);
			# Modify @rfc2253_kosher and/or %OIDs above.
			};
		};
		push @prettyDN,"$k=$v";
	};
	return join(",",@prettyDN);
}

=pod "

=item I<longClass()>

Returns the long class name (in LDAP parlance) of a DN. Or more
precisely, I<one> appropriate class name (and if this has a sense, the
most appropriate one). Indeed, "O=IDEALX,C=fr" is an Organization, no
doubt, but what exactly is "CN=John Doo,O=IDEALX,C=fr"? (our answer
is: a strongAuthenticationUser, but this is arbitrary).

=cut "

sub longClass {
	my @DN=_theDN(@_);
	throw IDX::PKI::Error::Internal("UNIMPLEMENTED");
}



=pod "

=item I<getUserID()>

Returns a "reasonably unique" ID from a DN. By decreasing order of
preference, returns the UID, the CN, the email address, and the
rfc2253 form of the DN.

=cut "

sub getUserID {
	my %DN=_theDN(@_);

	foreach my $key (qw(uniqueIdentifier CN Email)) {
		return $DN{$key} if (exists $DN{$key});
	}

	return rfc2253(_theDN(@_));
}

=pod "

=item I<compare (\@DN1, \@DN2)>

=item I<compare ($dnref, $dnobj)>

Boolean comparison of two DNs passed as array references or strings.  This
function tries to display some adaptability if some of its args is a
string (that is, some DNs will be deemed equal even though they don't
I<parse()>), and a strict comparison in the list case.

If one of the DNs (or DN components) is invalid, I<compare()> will
systematically return a false value.  This is a feature, not a bug, so
that bogus user-entered DNs do not trigger exceptions.  We assume that
comparing DNs for equality is the _only_ security-critical operation
performed on them.

=cut "

sub compare {
	my ($dn1, $dn2, $_norecurse) = @_;

	# Some recursion trickery to write less code...
	return compare ($dn2, $dn1) if (ref($dn1) && !ref($dn2));

	if (! ref($dn1)) {
		local $YesIWantBrokenDNs = 1;
		return compare ([parse IDX::PKI::DN ($dn1)], $dn2);
		# If $dn2 isn't a reference yet, the next step will exchange them
		# and then we will take this
	}

	# Now both DNs are references on lists.

	return undef unless ((scalar @$dn1) == (scalar @$dn2));
	return undef if (0 == scalar(@$dn1));
	return undef if (scalar(@$dn1) % 2);

	my @dn1 = @$dn1;
	my @dn2 = @$dn2;

	# As an extra measure to protect the innocent, two DNs which are
	# reverse of each other are equal.
	my @dn1rev;
	while (my ($k, $v) = splice (@dn1, 0, 2)) {
		unshift (@dn1rev, $k, $v);
	}
	return 1 if ((! $_norecurse) && (compare (\@dn1rev, $dn2, 1)));
	@dn1 = @$dn1;

	while (my ($kdn1, $vdn1) = splice (@dn1, 0, 2)) {
		my ($kdn2, $vdn2) = splice (@dn2, 0, 2);
		return undef unless (uc($kdn1) eq uc($kdn2));
		return undef unless (($kdn1 =~ m/^c$/i) ? (uc($vdn1) eq uc($vdn2)) :
							 ($vdn1 eq $vdn2));
	}
	return 1;
}

=pod "

=item I<short_to_long ($short)>

This is a convenience function which returns the long name (such as
C<commonName>) associated to a given short name (such as C<CN>).

Returns undef rather than throwing an error if the short name isn't known.
Shoult it be otherwise?

=cut "

sub short_to_long {
	my ($short) = @_;
	return $fields{$short};
}

=pod "

=item I<long_to_short ($long)>

This is a convenience function which returns the short name (such as
C<CN>) associated to a given long name (such as C<commonName>).

Returns undef rather than throwing an error if the short name isn't known.
Shoult it be otherwise?

=cut "

sub long_to_short {
	my ($long) = @_;
	my %fieldsrev;
	foreach my $short (keys %fields) {
		$fieldsrev{$fields{$short}} = $short;
	}
	return $fieldsrev{$long};
}

=pod "

=back

=head1 GLOBAL VARIABLE

=over 4

=item $IDX::PKI::DN::YesIWantBrokenDNs

=back

Suppresses the test in L</canonical> about reversed DNs being
parsed. Setting this global variable to 1 outside of this module is
possible, although strongly discouraged.

=head1 CCITT CURSING

DNs are an incredibly bogus, ill-designed shenanigan of the X.* specs
from the telecom industry worldwide, which themselves are just a
hodgepodge of proprietary trade secrets ("standards" is a word they
can't even spell). Those guys have been laughing at the Internet for
more than 20 years, stating that it wouldn't scale, that giving up
guarantee of service at the network stack level was a design fault,
and that OSI is better than TCP/IP. The truth is that the Internet
has conquered the world, works very well thank you, and that the X.*
protocol suite nowadays is a bunch of failed attempts to get
money-greedy telecom megacorporations to cooperate in any useful
manner. Can you imagine that an ASN.1 client is supposed to *negociate
data encoding protocols* with an ASN.1 server before beginning to
exchange stuff?  Can you imagine that X500 doesn't even specify
whether the country should come first or last in a DN? The X400/X500
implementations are just a bunch of balkanized, over-priced, bloated
dinosaurs that were more or less unable to interoperate with SMTP or
LDAP (let alone with each other) and were slowly dying a painful,
well-deserved death. Until PKIX came around.

Unfortunately enough, a portion of the boatloads of FUD produced at
CCITT managed to filter through the IETF walls, probably because some
smart-ass at Netscape or VeriSign with a telecom background decided
that he knew better about this. So we are now left with 15 different
incompatible types of string formats in certificates, two date formats
(one of them Y2K-buggy), megabytes-long valid X509 certificates with a
video of their owner's cat as part of the DN (authentic), RFCs that
dictate that the world SHOULD fall apart on Dec 31, 2003. Documents
about X509 are obscure, underspecified and contradictory enough, and
"best" (?) current practices are, hm, "flexible" enough, that software
design turns into a task of archeology and exegesis to which Qumran
manuscripts are in comparison like comics you bring to the toilets for
casual reading. That's the environment into which IDX-PKI tries to
bring the software quality so dear to the Internet world - to show
"them" once for all what systems designing really is about (bragging
with beautiful OSI diagrams being way off-topic), and what free
software can do (that they couldn't manage with all their money and
power).

Yeah, right.

More on this, perhaps a little less virulent but much funnier, at:

 http://www.cs.auckland.ac.nz/~pgut001/pubs/x509guide.txt

(my favorite one:

  When the X.500 revolution comes, your name will be lined up against the
  wall and shot
  -- John Gilmore
)


The bottom line:

Do B<not> use DNs for anything serious. Do B<not> use them to identify
certificates: there may be two certificates with the same DN (one for
encryption and one for signing, or one in Unicode and the other in
TeletextString, UniversalString, PrintableString, AsciiString,
BMPString or what-have-you). Do B<not> use them to identify servers:
the existing literature taken together implies there can be at most
one server program per physical machine when doing this (finding out
the reason why is left as an exercise to the reader). Do B<not> use
them to store computer data such as e-mail, UIDs or the like; there
are B<standard> X509v3 extensions much more suited for this
purpose. You may use them for authentication purposes on a
discriminatory basis: "this guy is not the real McCoy because he has
the wrong DN". Doing the converse is strongly discouraged.

The current use of DNs in IDX-PKI is merely as a convoluted login name,
and as something bizarre the customer wants us to put in
certificates. RFC2253-style DNs are listed in ACLs, and if the bogus
comparison algorithm matches, access is granted (if your name is
"France" and you live in Virginia, and somebody is in the opposite
situation, then this is bad luck). DNs are produced according to the
desiderata of IDEALX's customers, purposefully refraining from making
assumptions about any relationship between the DNs of CAs, users,
servers and computer PKI entities. Whenever possible, use the B<full
certificate> to identify some network partner, and refresh it
periodically from the repository LDAP to work around certificate rot.

=cut "

eval join("",<main::DATA>) || die $@ unless caller();

1;

__END__
##############################################################

package IDX::PKI::DN;
use IDX::Test;
use IDX::PKI::Error;
use IDX::PKI::Crypto::Tools;

number_of_tests (23);

my @DNOK=( "CN" , "IDX-PKI OpCA",
           "OrganizationalUNITName", "IRD",
           "OU", "IDEALX",
           "C", "fr");

test "build() and canonical()" => sub {
	my @DN=IDX::PKI::DN::canonical(@DNOK);
	TEST($DN[2] eq "OU");
	TEST($DN[7] eq "FR");
	@DN=build IDX::PKI::DN(@DNOK)->canonical();
	TEST($DN[2] eq "OU");
	TEST($DN[7] eq "FR");
};

test "call by array reference (undocumented but supported)" => sub {
	my @DN=IDX::PKI::DN::canonical(\@DNOK);
	TEST($DN[2] eq "OU");
	TEST($DN[7] eq "FR");
};

# Deprecated
#test "Catching reversed DNs" => sub {
#    my @DNWRONG=( "C" , "fr",
#	  		  "OU", "IDEALX",
#			  "CN", "Something");
#
#	try {
#		build IDX::PKI::DN(@DNWRONG);
#		TESTFAIL;
#	} catch IDX::PKI::Error::Internal with {
#		my $E=shift;
#		TEST($E->{-text} =~ m/order/);
#	};
#};

test "parse ()" => sub {
	my @dn_copy = @DNOK;	# For the splice ().
	my $string = "";
	while (my ($key, $val) = splice (@dn_copy, 0, 2)) {
		$string .= ($string ne '' ? ',' : '') . "$key=$val";
	}
	my @dn = parse IDX::PKI::DN ($string)->canonical ();
	TEST ($dn[2] eq "OU");
	TEST ($dn[7] eq "FR");

	# REGRESSION: DNs with commas inside.
	TEST(parse IDX::PKI::DN('Email=jxdoe@someorg.com,CN=John X. Doe,OU=Netscape Compatibility Group,O=Some Organization, Inc.,L=Paris,C=US'));

	@dn=parse IDX::PKI::DN("/C=fr/O=IDEALX/CN=some bogus OpenSSL DN");
	TEST($dn[5] =~ m/fr/i) || warn join(" ",@dn);
};

test "chop () -- object style, array context" => sub {
	my @dn = (build IDX::PKI::DN (@DNOK))->chop ();
	TEST ($dn[0] eq "OU");
	TEST ($dn[5] eq "FR");
};

test "chop () -- object style, scalar context" => sub {
	my $dn = (build IDX::PKI::DN (@DNOK))->chop ();
	my @dn = $dn->canonical ();
	TEST ($dn[0] eq "OU");
	TEST ($dn[5] eq "FR");
};

test "chop () -- plain function style, array context" => sub {
	my @dn = IDX::PKI::DN::chop (@DNOK);
	TEST ($dn[0] eq "OU");
	TEST ($dn[5] eq "FR");
};

test "chop () -- plain function style, scalar context" => sub {
	my $dn = IDX::PKI::DN::chop (@DNOK);
	my @dn = $dn->canonical ();
	TEST ($dn[0] eq "OU");
	TEST ($dn[5] eq "FR");
};

my $dn = build IDX::PKI::DN (@DNOK);

test "_theDN ()" => sub {
	my @byobj = $dn->_theDN ();
	my @byarr = _theDN (@DNOK);
	TEST (scalar (@byobj) == scalar (@byarr));
	while (my $byobj = shift (@byobj)) {
		my $byarr = shift (@byarr);
		TEST ($byobj eq $byarr);
	}
};

test "DC in DN" => sub {
	my $dn=parse IDX::PKI::DN("CN=John Doo,DC=idealx,DC=com");
	TEST($dn->string() =~ m/DC/);
	TEST($dn->rfc2253() !~ m/DC/);
};


test "getLeftRDNShortType ()" => sub {
	TEST ($dn->getLeftRDNShortType () eq 'CN') ||
		warn $dn->getLeftRDNShortType ();
};

test "getLeftRDNLongType ()" => sub {
	TEST ($dn->getLeftRDNLongType () eq 'commonName') ||
		warn $dn->getLeftRDNLongType ();
};

test "getLeftRDNValue ()" => sub {
	TEST ($dn->getLeftRDNValue () eq 'IDX-PKI OpCA') ||
		warn $dn->getLeftRDNValue ();
};

test "getRightRDNShortType ()" => sub {
	TEST ($dn->getRightRDNShortType () eq 'C') ||
		warn $dn->getRightRDNShortType ();
};

test "getRightRDNLongType ()" => sub {
	TEST ($dn->getRightRDNLongType () eq 'countryName') ||
		warn $dn->getRightRDNLongType ();
};

test "getRightRDNValue ()" => sub {
	TEST ($dn->getRightRDNValue () eq 'FR') ||
		warn $dn->getRightRDNValue ();
};

test "get*RDNClass ()" => sub {
	TEST ($dn->getRightRDNClass () eq 'country') ||
		warn $dn->getRightRDNClass ();
	my $seconddn=IDX::PKI::DN::chop($dn->canonical());
	TEST ($seconddn->getLeftRDNClass () eq 'organizationalUnit') ||
		warn $dn->getLeftRDNClass ();
};

test "getRDNValue (\$element), scalar context" => sub {
	TEST ($dn->getRDNValue ('OU') eq 'IRD') || warn $dn->getRDNValue ("OU");
	TEST ($dn->getRDNValue ('ou') eq 'IRD') || warn $dn->getRDNValue ("ou");
};

test "getRDNValue (\$element), list context" => sub {
	my @rdns = $dn->getRDNValue ('OU');
	TEST (scalar (@rdns) == 2);
	TEST ($rdns[0] eq 'IRD') || warn $rdns[0];
	TEST ($rdns[1] eq 'IDEALX') || warn $rdns[0];
};

test "to_openssl and like_openssl" => sub {
	my @DN=("O"=>"world","O"=>"IDEALX");
	TEST(IDX::PKI::DN::to_openssl(@DN) eq <<EXPECTED);
0.organizationName=IDEALX
1.organizationName=world
EXPECTED
	TEST(IDX::PKI::DN::like_openssl(@DN) eq
		 "/O=IDEALX/O=world") ||
		   warn IDX::PKI::DN::like_openssl(@DN);

};

test "string() and rfc2253()" => sub {
	foreach my $test ([ ["O"=>"IDEALX","c"=>"fr"] =>"O=IDEALX,C=FR" =>"O=IDEALX,C=FR"],
					  [ ["Email"=>'johndoo@world.com',
						 "O"=>"world","O"=>"IDEALX"] =>'Email=johndoo@world.com,O=world,O=IDEALX' => '1.2.840.113549.1.9.1=johndoo@world.com,O=world,O=IDEALX'],
					 ) {
		my ($paramsref,$stringresult,$rfc2253result)=@$test;
	TEST(IDX::PKI::DN::rfc2253(@$paramsref) eq $rfc2253result) ||
		   warn IDX::PKI::DN::rfc2253(@$paramsref);
	TEST(IDX::PKI::DN::string(@$paramsref) eq $stringresult) ||
		   warn IDX::PKI::DN::string(@$paramsref);
	 };
};

test "compare()" => sub {
	TEST(compare(["C"=>"fr","o"=>"IDEALX"],["O"=>"IDEALX","C"=>"fr"])) ||
	  warn "Comparison 1 failed";
	TEST(compare("O=IDEALX,C=fr",["O"=>"IDEALX","C"=>"fr"])) ||
	  warn "Comparison 2 failed";
	TEST(compare(["O"=>"IDEALX","C"=>"fr"],"O=IDEALX, C=fr")) ||
	  warn "Comparison 3 failed";
	TEST(compare("O=IDEALX/C=fr","O=IDEALX|C=fr")) ||
	  warn "Comparison 4 failed";
	TEST(compare("O=acme/OU=IDX-PKI/CN=EE", "cn=EE,OU=IDX-PKI,O=acme")) ||
	  warn "Comparison 5 failed";

	TEST(!compare(["O"=>"IDEALX","C"=>"fr"],["O"=>"idealx","C"=>"fr"])) ||
	  warn "Comparison 6 failed";
	TEST(compare(["OU"=>"ird","1.2.3.4"=>"IDEALX","C"=>"fr"],
				 ["C"=>"fr","1.2.3.4"=>"IDEALX","OU"=>"ird"])) ||
	  warn "Comparison 7 failed";
	TEST(!compare("OU=ird,O=IDEALX/C=fr","O=idealx|C=fr")) ||
	  warn "Comparison 8 failed";
	TEST(!compare("O=idealx/C=fr","O=IDEALX|C=fr")) ||
	  warn "Comparison 9 failed";
	TEST(compare("CN=Administrator,OU=IDX-PKI,OU=intranet,OU=department,OU=sub,O=corp,C=fr",
					 "C=FR/O=corp/OU=sub/OU=department/OU=intranet/OU=IDX-PKI/CN=Administrator")) ||
	  warn "Comparison 10 failed";
};

foreach my $DN (["CN"=>"test self-signed cert",
							  "O"=>"Secu","OU"=>"IRD","O"=>"IDEALX",
							  "C"=>"fr"],
			   ["Email"=>'johndoo@example.com',
				"OU"=>"paperwork division",
				"OU"=>"example",
				"O"=>"example"]) {
	my $dn=build IDX::PKI::DN(@$DN);

	integrationtest "integration: name check of a generated cert." => sub {
		$INC{"IDX/PKI/DN.pm"}=1; # Shuts bogus warning (X509 in turn
		# requires DN, which causes a redefinition of all methods)
		require IDX::PKI::Crypto::X509;
		my $config=new IDX::Test::Filename();
		my $newcert=new IDX::Test::Filename();
		string2file(sprintf(<<CONFIG,$dn->to_openssl()),$config);
[req]
prompt=no
distinguished_name=req_distinguished_name

[req_distinguished_name]
%s
CONFIG
		warn run_openssl("req","-x509","-new","-newkey","rsa:384",
					"-keyout","/dev/null","-nodes",
					"-config","$config","-out",$newcert);
		my $x509=load IDX::PKI::Crypto::X509($newcert);
		TEST($x509->getSubjectDN() eq $dn->string()) ||
		  warn scalar($x509->getSubjectDN());

		my @dn1=$x509->getSubjectDN();
		my @dn2=$dn->canonical();
		TEST($#dn1 == $#dn2);
		foreach my $i (0..$#dn1) {
			TEST(lc($dn1[$i]) eq lc($dn2[$i])) ||
			  warn join(" ",@dn1);
		};
	};
};


1;
