#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::PKI::Error> - OO exception hierarchy for the IDX-PKI project

=head1 SYNOPSIS

 use IDX::PKI::Error;

 open(STUFF,'</etc/passwd') ||
 	throw IDX::PKI::Error::IO (CONFIG_INIT,
 		"-IOfile" => "/etc/passwd");

 s/parse/things/g || throw IDX::PKI::Error::Format($_);

 get_config_item() =~ m/some_pattern/ ||
 	throw IDX::PKI::Error::Config( THIS_ITEM );

 etc.

=head1 DESCRIPTION

B<IDX::PKI::Error> is a bunch of classes for handling errors that happen
in the PKI, in an object-oriented fashion.  There are different constructors
for different types of error (see below); all of them accept the following
syntaxes:

 throw IDX::PKI::Error::IO("Impossible to open()");
 throw IDX::PKI::Error::IO("FILE_NOT_FOUND");
 throw IDX::PKI::Error::IO(-IOfile=>"/tmp/foobar",
 	-value=>4021);
 throw IDX::PKI::Error::IO("Impossible to open()",
 	-IOfile=>"/tmp/foobar", -value=>4021);

There can be a symbolic tag instead of a text message; the corresponding
text is kept in an array inside F<IDX/PKI/Error.pm>.  This should be done
whenever an error condition is generic enough, so as to help future
localization of the PKI and to allow for automatic determination of an
error value.

The B<IDX::PKI::Error> objects are organized internally as hash references
with keys such as C<-file>, C<-line>, C<-text>, C<-value>, C<-object>
(these are the fields inherited from B<Error(3)>), and class-specific
informations (summarized below). The C<-details> field can be used to any
purpose that pleases the programmer, provided no confidential information
is leaked.

The list of all error classes in IDX-PKI is described in L</IDX-PKI
SPECIFIC ERROR CLASSES>.

=head1 ERROR MANAGEMENT POLICY IN IDX-PKI

In any program, it is extremely important that normal control flow be
interrupted when a statement fails to perform as expected (e.g. "produces
an error", see below for a definition).  Failure to enforce this rule
results in "byzantine" errors that show up as a bizarre crash an arbitrary
number of lines of code later, possibly after having triggered an
unexpected, or even nefarious behaviour (e.g. deleting the configuration
file because I<GetTempFileExtension()> returned C<undef> :-/ ...).

So the error must stop the program as soon as it happens, be there an
error correction mechanism at work or not: this is a critical
requirement for robustness of the code and therefore for its security.
The solution adopted by L<Fatal> (checking if an C<undef> return value
is ignored and die()ing if it is the case) is clever, but it imposes a
special semantics to C<undef> that we may not like (e.g. we want to
return C<undef> in a search that yields no result); furthermore, the
error messages are strings stored in I<$@> (see L<perlvar>) which is
quite tedious to catch and match and act upon properly in error
recovery code (10-feet-long, locale-dependent regular expressions to
match error texts? No thanks).

Here comes B<Error>, from CPAN. This is an object- and exception-oriented
way of dealing with errors, just like Java or Objective CAML has. Errors
are exceptions, always thrown with I<die()> and caught with I<eval{}> (but
there is plenty of helpful syntactic sugar - see below).  They can be
arbitrary objects, conveying a rich information set (file and line where
the error happened, human comment, stack trace, HTTP error code, etc.)
which allows for a very sophisticated, even object-oriented (with error
classes and inheritance and whatnot) error creation and management code.
Furthermore, catching errors is never an obligation: provided errors are
properly thrown, user code will perform reliably (even if not
persistently) and byzantine failures will be out of the question.  So the
goal of this package is to adapt the extremely generic and useful B<Error>
framework to the particular case of IDX-PKI.  There we go.

"Errors" are defined as circumstances not foreseen by natural writing of
the control flow in the programs.  That is, the following circumstances
shall I<NOT> be treated like errors, but will instead elicit an C<undef>
return value:

=over 4

=item *

Returning false in a boolean context, no matter how critical the tested
parameter is;

=item *

Failing to find a key in a hash-like environment (for example a
field in an X.509 certificate);

=item *

Failing a system call in a way specifically accounted for by the
API (such as failing to receive data immediately in a function
such as I<IDX::PKI::Transport::get_nonblock()>).

=back

On the other hand, the following circumstances are indeed errors:

=over 4

=item *

Falling short of any kind of resources (from core memory to network sockets
to memory slots on a smart card);

=item *

Failing to open a file that should be there (according to configuration
for example);

=item *

Failing to establish some network connection inside the PKI;

=item *

Finding that the signature over a certificate is incorrect.

=back

Discrimination between the two categories shall be made according to the
following criteria: Human parseability (exceptional error handling shall
not clutter useful code - this roughly translates into "throw early, throw
often, catch later"); common sense (according to the guidelines set in the
above enumeration); performance (exception raising in Perl is more
expensive than normal control flow so raising an exception should remain,
well, exceptional).

All system calls and non-IDX-PKI library functions shall be tested for
errors, which must be dealt with according to the policy stated above
(that is, raising exceptions whenever they return C<undef> or the local
equivalent).  System calls and foreign library calls are to be considered
as the lower border of the domain of our error handling policy; that's
where the error "comes in" and is turned into an exception.

Use of I<$SIG{__DIE__}> is forbidden, as it breaks B<Error>!!! Use of
I<croak()> is allowed only in the toplevel scripts, and then only if
barfing an error message at the user is correct behaviour (this is
definitely not the case in CGI scripts).  This is because I<croak()> is a
much narrower channel for transmitting data about the errors. This being
said, some standard Perl modules I<croak()> upon error, and this can be
caught with B<IDX::PKI::Error> (but see above about 10-feet-long regexps).

Assertions can be done like this:

 throw IDX::PKI::Error::Internal("ASSERTION_FAILED")
 	if (<...>);

(I shall just remember the reader about the Robustness Principle
(RFC791,  3.2), that basically states that mandatory assertion
checking in interface code is a bad idea...  But this is more of a
matter of programming style and no policy statement is made in IDX-PKI
on this topic).

=head1 TRANSITION FROM "ORDINARY" ERROR MANAGEMENT

This nano-tutorial on OO-based error handling is heavily borrowed from
the Perl RFC 88 (F<http://dev.perl.org/rfc/88.html#CONVERSION>).  For
more information on the specifics of object-oriented error management
in Perl, see also L<Error>.

=over 4

=item 1

Wherever you previously would have returned C<undef> or some other special
return code (or a pass-by-reference value), to indicate the failure of a
subroutine or function, instead use:

 throw IDX::PKI::Error::Something()

=item 2

Wherever you previously would have written:

 $x = foo();
 defined $x or return undef;

to propagate an error from a callee back to your caller, you can just
write C<$x = foo();> because unhandled exceptions automatically propagate.

=item 3

Wherever you previously would have written the equivalent of the following
to do something about an error, and then ignore it:

 $x = foo();
 unless (defined $x) {
 	# do something about error here
 };

you can now write:

 try {
 	$x = foo();
 } catch IDX::PKI::Error::IO with {
 	my ($E) = @_;
 	# Do something to correct the error here.
 	$E->throw() if (still_wrong());
 };	# Don't forget the semicolon!

Be careful to B<ALWAYS> put a final semicolon after any I<try>
construct: Since I<try>, I<catch> etc. aren't really keywords, but
clever syntactical tricks, forgetting the semicolon would cause the
next expression in your program to be somehow incorporated into the
exception handling framework (e.g. passed as an extra parameter to
I<IDX::PKI::Error::try()>).

=item 4

Exception from non-IDX-PKI modules (that generally use I<die()> or
I<croak()>), that would previously be caught like this:

 eval {$x=foo(); 1;} || do {
 	$@ =~ m/no such file/ && do {something};
 };

can now alternatively be caught as B<Error::Simple> exceptions, like this:

 try {$x = foo();}
 catch Error::Simple with {
 	my ($E)=@_;
 	$E =~ m/no such file/ && do {something};
 };

=item 5

Wherever you previously would have ignored an error in order to allow you
to restore invariants or enforce postconditions, and then used return
C<undef> to propagate the error, like this:

 open F, ...;
 $x = foo();
 close F;
 defined $x or return undef;

you can now write:

 open F, ...;
 try {
 	$x = foo();
 } finally {
 	close F;
 };

because unhandled exceptions automatically propagate.  The I<finally>
clause will be executed, no matter whether I<foo()> throws or terminates
normally; it does not handle the exception at all.

=back

=cut "

######################## STUB IMPLEMENTATION ###############################

=pod "

=head1 PACKAGE B<IDX::PKI::Error>

This package is the father of all derived error classes, and contains
all useful methods.

=cut "

package IDX::PKI::Error;

use strict;

use Error;
use Exporter;

use vars qw(@ISA $AUTOLOAD @EXPORT @EXPORT_OK %EXPORT_TAGS);
@ISA = qw(Error Exporter);

=pod "

=head2 Constants

They are all exported by default.

=head3 Configuration-Related Errors

=over 4

=item CONFIG_INIT

The configuration file could not be opened.

=item CONFIG_INVALID

An expected configuration parameter is missing, or its value is invalid.

=item CONFIG_SAVING

Error saving the configuration file.

=item CONFIG_PROFILE

A profile already exists with the same name.

=back

=head3 Transport-Related Errors

=over 4

=item TRANSPORT_MSG

The received message could not be read, or properly deserialised.

=item TRANSPORT_NOTFOUND

The message can't be found in queue.

=item TRANSPORT_SENDING

Error sending a message. This always corresponds to an
B<IDX::PKI::Error::Network> exception.

=back

=head3 Requests-Related Errors

Requests can be Certificate Signing Requests, Certificate Revocation
Requests or Key Recovery Requests.

=over 4

=item REQUEST_CREATE

The error occured when creating a request.

=item REQUEST_INVALID

The request is invalid.

=back

=head3 Repository-Related Errors

These errors apply to the CA internal repository, or the escrow LDAP.

=over 4

=item REPOSITORY_CONNECT

The error occured while contacting the repository server. This always
corresponds to an B<IDX::PKI::Error::IO> exception.

=item REPOSITORY_PUBLISH

The error occured while publishing something into the repository.

=item REPOSITORY_NOENTRY

No corresponding entry was found. (This may not always need throwing an
exception.)

=item REPOSITORY_TOOMANY

Some request into the repository returned too many entries.

=back

=head3 CGI Input-Related Errors

These errors generally correspond to B<IDX::PKI::Error::User> exceptions.

=over 4

=item INPUT_HIDDEN

A (normally hidden an automatically transmitted parameter) is missing or
invalid. This generally means calling manually a CGI controller or a PHP
view when you shouldn't.

=item INPUT_MISSING

A mandatory field (which should have been provided by the user) is missing
or empty.

=item INPUT_EMPTY

At least one in a series of parameters should be provided: Here all are
missing or empty.

=item INPUT_FORMAT

An input parameter has an invalid format.

=item INPUT_VERIFY

Two input values don't match when the user is supposed to confirm a
parameter (for example, a password).

=item INPUT_TOOLONG

The input parameter is too long.

=item INPUT_TOOSHORT

The input parameter is too short.

=item INPUT_NOREQ

This may happen for decentralised certificate requests: The request was
transmitted without a CSR.

=item INPUT_KEYSIZE

This may happen for decentralised certificate requests: The request
corresponds to a key which is too small.

=back

=head3 Some Other Errors

=over 4

=item MISC_BROWSER

The user's browser is unsupported.

=item MISC_EMAIL

An operation needs some email to send the result to, but can't determine
one (for example, key recovery).

=back

=cut "

#my %constants =
#	(
#	CONFIG_INIT        => -100,
#	CONFIG_INVALID     => -101,
#	CONFIG_SAVING      => -102,
#	CONFIG_PROFILE     => -103,
#
#	TRANSPORT_MSG      => -200,
#	TRANSPORT_NOTFOUND => -201,
#	TRANSPORT_SENDING  => -202,
#
#	REQUEST_CREATE     => -300,
#	REQUEST_INVALID    => -301,
#
#	REPOSITORY_CONNECT => -400,
#	REPOSITORY_PUBLISH => -401,
#	REPOSITORY_NOENTRY => -402,
#	REPOSITORY_TOOMANY => -403,
#
#	INPUT_HIDDEN       => -500,
#	INPUT_MISSING      => -501,
#	INPUT_EMPTY        => -502,
#	INPUT_FORMAT       => -503,
#	INPUT_VERIFY       => -504,
#	INPUT_TOOLONG      => -505,
#	INPUT_TOOSHORT     => -506,
#	INPUT_NOREQ        => -507,
#	INPUT_KEYSIZE      => -508,
#
#	MISC_BROWSER       => -600,
#	MISC_EMAIL         => -601,
#	);

my %constants =
	(
	CONFIG_INIT        => -10,
	CONFIG_INVALID     => -11,
	CONFIG_SAVING      => -12,
	CONFIG_PROFILE     => -14,

	TRANSPORT_MSG      => -22,
	TRANSPORT_NOTFOUND => -22,
	TRANSPORT_SENDING  => -20,

	REQUEST_CREATE     => -30,
	REQUEST_INVALID    => -43,

	REPOSITORY_CONNECT => -50,
	REPOSITORY_PUBLISH => -51,
	REPOSITORY_NOENTRY => -52,
	REPOSITORY_TOOMANY => -54,

	INPUT_HIDDEN       => -60,
	INPUT_MISSING      => -61,
	INPUT_EMPTY        => -65,
	INPUT_FORMAT       => -62,
	INPUT_VERIFY       => -67,
	INPUT_TOOLONG      => -63,
	INPUT_TOOSHORT     => -62,
	INPUT_NOREQ        => -64,
	INPUT_KEYSIZE      => -31,

	MISC_BROWSER       => -66,
	MISC_EMAIL         => -70,
	);

# From perldoc AutoLoader.
sub AUTOLOAD {
	no strict "refs";
	my $sub = $AUTOLOAD;
	(my $wanted = $sub) =~ s/.*:://;
	if (! exists $constants{$wanted}) {
		$AutoLoader::AUTOLOAD = $sub;
		goto &AutoLoader::AUTOLOAD;
	}
	else
	{
		my $val = $constants{$wanted};
		*$sub = sub {$val};
		goto &$sub;
	}
}

sub DESTROY {
	# Nothing to do but exist to please AUTOLOAD.
}

@EXPORT = (keys %constants);
%EXPORT_TAGS = ("constants" => [keys %constants]);


=pod "

=head2 Functions and Methods

=over 4

=item I<import()>

The import routine for B<IDX::PKI::Error> is a copy of the one from
B<Error(3)>, so that the mandatory C<use Error qw(:try);> always happens
in the calling package (if this isn't done, the package appears to work
but it is then impossible to catch exceptions, which is really annoying).

=cut "

sub import {
	my $pkg = shift;
	local $Exporter::ExportLevel = $Exporter::ExportLevel + 1;
	Exporter::import ($pkg, @_);
	Error::subs->import(qw(:try));
}

=pod "

=item I<< new('some text', -option1 => 'value1', -option2 => 'value2'...) >>

Creates a new error object, which is not I<throw()>n immediately.  Rarely
used in this form: Error objects will rather be thrown as soon as created,
as in:

 throw (new IDX::PKI::Error("something"))

or as F<Error.pm> permits, just:

 throw IDX::PKI::Error("something")

=cut "

sub new {
	splice(@_,1,0,"-text") if ($#_ % 2);
	# "goto"s are in use so that we don't clutter the call stack (see
	# perlfunc): we want Error::new to find out the correct line numbers
	# et al.
	goto &Error::new;
}

=pod "

=item I<safe_dumper ($thing_to_dump)>

Almost like I<Data::Dumper::Dumper>, except that it doesn't fail if the
object doesn't have a I<FETCH> method. The drawback is that you may not be
able to <eval()> the returned string to get your object back.

It can be called either as a method or as a function.

=cut "

sub safe_dumper
{
	my ($self, $thing_to_dump);
	if (scalar (@_) == 2) {
		($self, $thing_to_dump) = @_;
	} elsif (scalar (@_) == 1) {
		($thing_to_dump) = @_;
	} else {
		throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS");
	}

	# If Data::Dumper::Dumper does the job for us, fine.

	my $ret;
	my $copy = $@;	# We are about to overwrite the value of $@.
	eval {
		require Data::Dumper;
		local $Data::Dumper::Indent = $Data::Dumper::Indent = 1;
		local $Data::Dumper::Terse = $Data::Dumper::Terse = 1;
		$ret = Data::Dumper::Dumper ($thing_to_dump);
	};
	my $code = $@;	# Our own eval's code.
	$@ = $copy;		# Restores the real $@.
	return $ret unless ($code);

	# Data::Dumper::Dumper failed. This happens sometimes, like with
	# IO::Socket::SSL objects. Seems that even in that case, Dumpperl
	# doesn't fail.

	# Dumpperl could be a nice fallback if it didn't just forget "tricky"
	# variables which Data::Dumper::Dumper doesn't like.
	# return Data::Dumper->Dumpperl ([$thing_to_dump]);

	if (! defined ($thing_to_dump)) {
		return "undef";
	} elsif (! ref ($thing_to_dump)) {	# Plain string.
		return "\"".quotemeta ($thing_to_dump)."\"";
	} elsif (UNIVERSAL::isa ($thing_to_dump, "ARRAY")) {
		$ret = "[\n";
		foreach my $line (@$thing_to_dump) {
			my $linestr = $self ? $self->safe_dumper ($line) :
				safe_dumper ($line);
			chomp ($linestr);
			$linestr =~ s/\n/\n  /gs;
			$ret .= "  $linestr,\n",
		}
		$ret .= "]\n";
		return $ret;
	} elsif (UNIVERSAL::isa ($thing_to_dump, "HASH")) {
		$ret = "{\n";
		foreach my $key (keys %$thing_to_dump) {
			my $val = $thing_to_dump->{$key};
			my $keystr = $self ? $self->safe_dumper ($key) :
				safe_dumper ($key);
			chomp ($keystr);
			my $valstr = $self ? $self->safe_dumper ($val) :
				safe_dumper ($val);
			chomp ($valstr);
			$valstr =~ s/\n/\n  /gs;
			$ret .= "  $keystr => $valstr,\n";
		}
		$ret .= "}\n";
		return $ret;
	} else {
		return "\\*".ref ($thing_to_dump);
	}
}

=pod "

=item I<stringify()>

Overloaded to publish useful information about the specific errors
of IDX-PKI.  See L<Error>.

=cut "

sub stringify {
	my ($self) = @_;

	# Could be smarter. Sure.
	return $self->safe_dumper ($self);
}

=pod "

=item I<humanize ()>

Displays the error in a human-friendly way.

=cut "

sub humanize {
	my ($self) = @_;
	my %c;
	foreach my $k (keys(%{$self})) {
		$c{$k} = $self->{$k};
	}
	my $text = delete $c{-text};
	$text = "??" unless (defined $text);
	my $file = delete $c{-file};
	$file = "??" unless (defined $file);
	my $line = delete $c{-line};
	$line = "??" unless (defined $line);
	my $package = delete $c{-package};
	$package = "??" unless (defined $package);
	my $rest = "";

	require IDX::PKI::Log;
	foreach my $k (keys(%c)) {
		$rest .= "$k => " . IDX::PKI::Log::encode ($c{$k}) . "\n";
	}

	my $ret = << "ERROR";
An error occured in at line $line in file: $file
Description: $text
Package: $package
Additional information:
<<
$rest
>>
ERROR

	return $ret;
}

# There is a bug in Error... See test "Nested deaths".
{
	no warnings;
	my $savedsub = \&Error::subs::run_clauses;
	*Error::subs::run_clauses = sub ($$$\@) {
		local $Error::THROWN = undef;
		return &$savedsub (@_);
	};
}

=pod "

=back

=cut "

#############################################################################

=pod "

=head1 IDX-PKI SPECIFIC ERROR CLASSES

=head2 B<IDX::PKI::Error::Internal>

An B<IDX::PKI::Error::Internal> is thrown when a programming error occurs
inside the PKI, due to a miscoordination between different parts of
the source code.  You can just name the cause of the error, like this:

 throw IDX::PKI::Error::Internal("wrong number of arguments");

or preferably, you can use one of the predefined error messages, which
are all caps and will be localized according to
I<%IDX::PKI::Error::Internal::stderrors>:

 throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS");

The following errors are predefined:

=over 4

=item WRONG_NUMBER_ARGS

Happens when a function is called with the wrong number of arguments
(this includes the case where a hash is expected literally, but
the parity is incorrect -e.g. for named parameters);

=item INCORRECT_ARGS

An argument was incorrect according to what is expected (undefined when it
shouldn't, of the wrong type, etc.);

=item UNIMPLEMENTED

This function is unimplemented (yet);

=item UNDOCUMENTED

Undocumented functions are just plain useless, you know;

=item PURE_VIRTUAL

Happens when a virtual method of an abstract class is defined;

=item ILLEGAL_CONTEXT

This function is called in scalar context but only makes sense in list
context, or vice versa;

=item TRANSPORT_DIRECTION

Wrong way of transport;

=item ASSERTION_FAILED

Assertion check failed;

=item SHOULD_NOT_HAPPEN

Assertion check failed (too);

=item WRONG_CALLING_ORDER

Wrong calling order;

=item LIBRARY_ERROR

Library call failed (not due to lack of resources).

=back

B<IDX::PKI::Error::Internal> carry a stack trace unless
I<$IDX::PKI::Error::Internal::inhibitTrace> is defined.  This is to
circumvent a bug in Perl 5.6.0 (see above).  Use this if needed (e.g. you
get messages about a bizarre copy of ARRAY):

 use IDX::PKI::Error;
 local $IDX::PKI::Error::Internal::inhibitTrace=1;

=cut "

package IDX::PKI::Error::Internal;

use strict;
use vars qw(@ISA %localize $inhibitTrace);
@ISA = qw(IDX::PKI::Error);

our %stderrors = (
	"WRONG_NUMBER_ARGS"   => "Wrong number of arguments in function call",
	"INCORRECT_ARGS"      => "Incorrect arguments in function call",
	"UNIMPLEMENTED"       => "Feature not implemented",
	"UNDOCUMENTED"        => "Not to be used unless properly documented",
	"PURE_VIRTUAL"        => "Pure virtual method called",
	"ILLEGAL_CONTEXT"     => "Function called in wrong context",
	"TRANSPORT_DIRECTION" => "Wrong way of transport",
	"ASSERTION_FAILED"    => "Assertion check failed",
	"SHOULD_NOT_HAPPEN"   => "Assertion check failed", # Too :-)
	"WRONG_CALLING_ORDER" => "Wrong calling order",
	"LIBRARY_ERROR"       => "Library call failed (not due to lack of ".
		"resources)",
);

sub new {
	local $Error::Depth = $Error::Depth + 1;
	local $Error::Debug = (! $inhibitTrace);
	my $self = IDX::PKI::Error::new(@_);
	defined (my $text = $self->{"-text"}) || return $self;
	$self->{"-text"} = $stderrors{$text} if (exists $stderrors{$text});
	return $self;
}


=pod "

=head2 B<IDX::PKI::Error::Config>

Improper configuration of the PKI (unparseable string, missing CA
certificate etc.)

=cut "

package IDX::PKI::Error::Config;

use strict;
use vars qw(@ISA);
@ISA = qw(IDX::PKI::Error);


=pod "

=head2 B<IDX::PKI::Error::IO>

I/O errors, such as problems accessing files that should be available on
the local machine for the PKI to function correctly.  Special field:
C<-IOfile> holds the name of the file that failed.

=cut "

package IDX::PKI::Error::IO;

use strict;
use vars qw(@ISA);
@ISA = qw(IDX::PKI::Error);

sub new {
	local $Error::Depth = $Error::Depth + 1;
	my $self = IDX::PKI::Error::new(@_);
	$self->{"-IOError"} ||= $!;
	return $self;
}


=pod "

=head2 B<IDX::PKI::Error::Memory>

Memory-bound resource problems (fork failed, etc.)  Warning, it is quite
impossible to trap "regular" out-of-memory errors in Perl.

=cut "

package IDX::PKI::Error::Memory;

use strict;
use vars qw(@ISA);
@ISA = qw(IDX::PKI::Error);


=pod "

=head2 B<IDX::PKI::Error::Network>

Network errors.

=cut "

package IDX::PKI::Error::Network;

use strict;
use vars qw(@ISA);
@ISA = qw(IDX::PKI::Error);


=pod "

=head2 B<IDX::PKI::Error::Crypto>

Mismatches in signature, invalid X.509 format in message, etc.  May
indicate an ongoing attack, or just plain braindamage in smartcard
equipment and the like.

=cut "

package IDX::PKI::Error::Crypto;

use strict;
use vars qw(@ISA);
@ISA = qw(IDX::PKI::Error);


=pod "

=head2 B<IDX::PKI::Error::User>

Errors caused by the user not reading the manual, or something.

=cut "

package IDX::PKI::Error::User;

use strict;
use vars qw(@ISA);
@ISA = qw(IDX::PKI::Error);


=pod "

=head2 B<IDX::PKI::Error::Security>

High-level attempts to circumvent security policies inside the PKI
(violating certification practices, attempting access to administrative
pages without proper credentials).

This class of errors always sets the C<-HTTPCode> parameter to C<403>,
except if it is told otherwise in the constructor.

=cut "

package IDX::PKI::Error::Security;

use strict;
use vars qw(@ISA);
 @ISA = qw(IDX::PKI::Error);

sub new {
	local $Error::Depth = $Error::Depth + 1;
	my $self = IDX::PKI::Error::new(@_);
	$self->{"-HTTPCode"} ||= 403;
	return $self;
}

# Dissimulate sensitive data
sub stringify {
	my ($self)=@_;

	my %copy = %$self;
	delete $copy{"-Authorized"};
	my $obj = bless \%copy, ref($self);

	return $self->safe_dumper ($obj);
}


=pod "

=head2 B<IDX::PKI::Error::Install>

Improper installation of the PKI or any dependency thereof (missing
binary, etc.)

=cut "

package IDX::PKI::Error::Install;

use strict;
use vars qw(@ISA);
@ISA = qw(IDX::PKI::Error);


=pod "

=head2 B<IDX::PKI::Error::Transport>

Transport protocol error (abnormal session termination, etc.)

=cut "

package IDX::PKI::Error::Transport;

use strict;
use vars qw(@ISA);
@ISA = qw(IDX::PKI::Error);

=pod "

=head2 B<IDX::PKI::Error::Database>

A high-level database error due to miscoordination (two lockers
stomped on each other's feet) or an attempt to break an invariant
(such as trying to publish two certificates with the same serial
number).

=cut "

package IDX::PKI::Error::Database;

use strict;
use vars qw(@ISA);
@ISA = qw(IDX::PKI::Error);

=pod "

=head1 BUGS

This implementation of an object-oriented error policy shares bugs with
B<Error> (see L<Error>) and the Perl core, but (AFAICT) doesn't have
specific bugs.

Stack traces in B<IDX::PKI::Error::Internal> trigger a Perl 5.6.0 bug (see
F<http://ftp.lip6.fr/pub/perl/CPAN/modules/by-module/Devel/Devel-StackTrace-0.85.readme>)
that may be quite harmful, up to causing a segmentation fault under
certain implementations of I<IDX::PKI::Error::Internal::new>.  Hence
the I<$IDX::PKI::Error::Internal::inhibitTrace> trick, that should
probably be set to zero during tests that trigger a several
B<IDX::PKI::Error::Internal> exceptions.  In the production
environment, those errors are typically not caught; since the bug is
triggered for every other call to B<Carp::longmess> (go figure...), we
should be safe.  Yeah, I know, this is pretty lame, but alas this bug
still exists in Perl 5.6.1.

Labels and exceptions are two rather incompatible ways of dealing with
non-local control flow.  The bottom line is: Never use a I<next>, I<last>
or I<redo> statement to exit a I<finally> block (you would break the
semantics of the exception propagation - finally blocks should just free
resources, not re-run a loop); and in the other cases, use a label like
this if you must:

 THING: while($something) {
 	try {
 		dosomethingwith($something);
 		next THING if (...);
 	} catch IDX::PKI::Error::IO with {
 		next THING;
 	};
 }

If you used a plain I<next>, it would jump to the next iteration of some
random loop in the innards of B<Error>, which is not what you want.
However, even though this idiom will produce the expected results, it will
elicit a warning in Perl 5.6.0 (subject to change in future Perl
versions).  You have been warned (and you will be again, by Perl :-).

=cut "

1;

############################ TEST SUITE ###############################

eval join('',<main::DATA>) || die "$@" unless caller();

__END__

package IDX::PKI::Error;
use Error qw(:try); # Since we cannot use ourselves, this is hard
# to avoid - but promised, you won't need it in your scripts :-).

use IDX::Test;

number_of_tests (19);

test "\"throw\" interrupts flow" => sub {
	try {
		TESTOK;
		throw IDX::PKI::Error::Security();
		TESTFAIL;
	} otherwise {
		TESTOK;
	};
};

test "\"otherwise\" not called when not needed" => sub {
	try {
		TESTOK;
	} otherwise {
		TESTFAIL;
	};
};


test "simple try-catch" => sub {
	try {
		throw IDX::PKI::Error::Security();
	} catch IDX::PKI::Error::Security with {
		TESTOK;
	};
	0; # Default is to fail test
};

test "error messages are useful" => sub {
	eval {
		throw IDX::PKI::Error::Internal("bad error");
	};
	TEST($@ =~ m/bad error/) || warn "\$\@: $@";
};

test "\"finally\" works OK" => sub {
	my $critical=0;
	$critical++;
	try {
		try {
			throw IDX::PKI::Error::Security();
		} finally {
			$critical--;
		};
	} otherwise { }; # Catch the error

	TEST($critical == 0);
};

test "\"otherwise\" works OK" => sub {
	try {
		try {
			throw IDX::PKI::Error::Security();
		} catch IDX::PKI::Error with {
			TESTOK();
		};
	} otherwise {
		TESTFAIL();
	};
	0; # Default is to fail test
};

test "Double \"otherwise\" works OK" => sub {
	try {
		try {
			throw IDX::PKI::Error::Security();
		} otherwise {
			TESTOK();
		};
	} otherwise {
		TESTFAIL();
	};
	0; # Default is to fail test
};

sub bonk { bink(); }

sub bink { throw IDX::PKI::Error::Internal("ouch"); }

for my $i (1..4) {
	test "repeated invocations of an IDX::PKI::Error::Internal exception ($i/4)" => sub {
		try {
			bonk();
		} catch IDX::PKI::Error::Internal with {
			TESTOK;
		};
		0;
	};
};

$IDX::PKI::Error::Internal::inhibitTrace=undef;
test "stack traces for IDX::PKI::Error::Internal" => sub {
	try {
		bonk();
	} catch IDX::PKI::Error::Internal with {
		my ($e)=@_;
		TEST($e =~ m/bink/);
		TEST($e =~ m/bonk/);
		TEST($e =~ m/ouch/);
	};
	0;
};

test "eval blocks throw (immediate)" => sub {
	eval {throw IDX::PKI::Error::Config("please provide [BEER]");};
	TESTOK();
};

test "eval blocks throw (in subroutine)" => sub {
	eval {bonk();};
	TESTOK();
};

test "loops and exceptions" => sub {
	my $i;
	LABEL:for($i=1; $i<10; $i++) {
		try {
			throw IDX::PKI::Error::User("Just the facts, ma'am");
		} catch IDX::PKI::Error::User with {
			last LABEL;
		};
	};
	TEST($i == 1);
};

test "stringification" => sub {
	eval {throw IDX::PKI::Error::Security("through obscurity")};
	TEST ("$@" =~ m/obscurity/) || warn "Should be an error: $@";
	eval {1};
	TEST ("$@" !~ m/obscurity/) || warn "Should be no error: $@";
};

# This test captures what really is a test in Error: Dying in an otherwise
# gives you the last thrown value outside your current try block, instead
# of the locally died value.

test "Nested deaths" => sub {
	eval {
		throw IDX::PKI::Error ("Reference throwed error");
	};
	eval {
		die "Reference died error";
	};

	eval {
		try {
			die "Error 1";
			warn "Throw #1 didn't stop execution of the block!";
			TESTFAIL ();
		} otherwise {
			die "Error 2";
			warn "Throw #2 didn't stop execution of the block!";
			TESTFAIL ();
		};
		warn "Still there?!";
		TESTFAIL ();
	};
	TEST ($@ =~ m/Error 2/) || warn "Got an unexpected error: $@";
};

# The test below reproduces what happens when you throw an error
# containing an IO::Socket::SSL object.

test "Errors are properly handled even when Data::Dumper fails" => sub {
	local $Error::THROWN = undef;	# See "nested deaths" test above.
	# If you don't reset $Error::THROWN and the test fails, the error
	# message will not be appropriate.

	{
		package SOMETHING;
		sub TIEHANDLE
		{
			my ($class, $var) = @_;
			return bless (\$var, $class);
		}
	}

	local *NOTHING;
	my $trickyvar = \*NOTHING;
	tie (*{$trickyvar}, "SOMETHING", $trickyvar);
	try {
		throw IDX::PKI::Error ("Argh", "-attachment" => $trickyvar);
		warn "Throw didn't stop execution of the block!";
		TESTFAIL;
	} otherwise {
		my $E = shift;
		my $string = $E->humanize ();
		TEST ($string =~ m/Argh/ && $string =~ m/-attachment/) ||
			warn "Humanisation lead to an unexpected result: $string";
		$string = $E->stringify ();
		TEST ($string =~ m/Argh/ && $string =~ m/-attachment/) ||
			warn "Stringification lead to an unexpected result: $string";
		$string = "$E";
		TEST ($string =~ m/Argh/ && $string =~ m/-attachment/) ||
			warn "Perl internal stringification lead to an unexpected ".
				"result: $string";
	};
};

test "Errors and Data::Dumper in a separate module." => sub {
	my $script = new IDX::Test::Filename ();
	local *FILE;
	open (FILE, ">", $script) || die "Can't open test script $script ".
		"for writing.";
	my $uselib = "use lib qw (".join (" ", @INC).");\n";
	print FILE << 'EOSCRIPT';
#!/usr/bin/perl -Tw
use strict;
EOSCRIPT
	print FILE $uselib;
	print FILE << 'EOSCRIPT';
use IDX::PKI::Error;
local $Error::THROWN = undef;	# See "nested deaths" test above.
# If you don't reset $Error::THROWN and the test fails, the error
# message will not be appropriate.
{
	package SOMETHING;
	sub TIEHANDLE
	{
		my ($class, $var) = @_;
		return bless (\$var, $class);
	}
}
local *NOTHING;
my $trickyvar = \*NOTHING;
tie (*{$trickyvar}, "SOMETHING", $trickyvar);
try {
	throw IDX::PKI::Error ("Argh", "-attachment" => $trickyvar);
	warn "Throw didn't stop execution of the block!";
} otherwise {
	my $E = shift;
	my $string;
	$string = $E->humanize ();
	print "Humanisation NOK: $string"
		unless ($string =~ m/Argh/ && $string =~ m/-attachment/);
	$string = $E->stringify ();
	print "Stringification NOK: $string"
		unless ($string =~ m/Argh/ && $string =~ m/-attachment/);
	$string = "$E";
	print "Perl internal stringification NOK: $string"
		unless ($string =~ m/Argh/ && $string =~ m/-attachment/);
};
1;
EOSCRIPT
	close (FILE);

	my $output = `perl -Tw "$script" 2>&1`;
	TEST ($output eq "" ) || warn "Got an output: $output";
};
