#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: FileManip.pm,v 1.10 2003/01/07 17:06:37 laura Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/FileManip.pm,v $

=pod "

=head1 NAME

IDX::PKI::FileManip - File manipulation utilities

=head1 NOTES

This is not an object class.

Note: This module heavily relies on the Unix conventions for filesystems:
C</> as a separator, C</> as the root, C<.> and C<..> having special
meanings, no C<NULL> characters inside a path, and blanks being extremely
annoying most of the time.

=cut "

# Note: some code is inspired from IDX-P213, module "myFile".

package IDX::PKI::FileManip;
use strict;

use IDX::PKI::Debug (0);
use IDX::PKI::Error;


=pod "

=head1 FUNCTIONS

=over 4

=item I<unique()>

Builds a unique string that can be used as a filename.  WARNING: This
is not I<mktemp()> and offers much less security warranties.  This is
B<FileManip>, not POSIX, after all.

=cut "

use vars qw($unique_counter);
$unique_counter = 0;
sub unique {
	my $printf = (shift or "unique%d-%d");
	$printf .= "%d-%d" if ($printf !~ m/\%/);
	return sprintf($printf,$$,$unique_counter++);
}


=pod "

=item I<tmp_extension()>

Extension in the private name space we reserve for ourselves.  This privacy
plus the alledged unicity means we will never test that we are not erasing
something when writing.

Should be used preceded with a dot.

The prefix is explicit and should allow in recovering after a crash, to
search for such files and either delete them, or read them and decide
about what to do.

=cut "

sub tmp_extension {
	return "IDEALX_internal_use.tmp.".&unique();
}


=pod "

=item I<is_absolute_path($path)>

Returns a true value if the path is absolute in a Unix way.

=cut "

sub is_absolute_path {
	my ($path) = @_;
	throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
		unless (defined $path);
	return substr($path,0,1) eq "/";
}


=pod "

=item I<is_path_safe($path)>

Returns a true value if the path includes no blanks or C<NULL>
characters and involves no C<..> parent directory.

=cut "

sub is_path_safe {
	my ($path) = @_;
	throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
		unless (defined $path);
	return 0 if $path =~ m{[\s\0]}s;   # no embedded blanks or NULLs
	return 0 if $path =~ m{^\.\.\/}s;  # no heading "../"
	return 0 if $path =~ m{\/\.\.$}s;  # no trailing "/.."
	return 0 if $path =~ m{\/\.\.\/}s; # no embedded "/../"
	return 1;
}

=pod "

=item I<clean_path($path)>

Cleans the file path in a Unix way: removes multiple slashes, single
C<.>'s and heading C</..>'s (but not other single C<..>'s).

Returns the cleaned path, and the root directory if the remaining path is
empty.

=cut "

sub clean_path {
	my ($path) = @_;
	throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
		unless (defined $path);

	my $res = $path;
	$res =~ s{/+}{/}gs;								# no multiple slashes
	1 while $res =~ s{/\./}{/}gso;					# no embedded .'s
	1 while $res =~ s{/\.$}{}gso;					# no trailing .'s
	1 while $res =~ s{^\./(.)}{$1}gso;				# no heading .'s
	1 while $res =~ s{^/\.\.?/}{/}gso;				# no heading /. or /..'s
	$res = "/" if ($res eq ""); # returns the root path for empty paths

	return $res;
}


=pod "

=item I<base_name($path)>

Returns the last component of a Unix path (path or file).

=cut "

sub base_name {
	my ($path) = @_;
	throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
		unless (defined $path);
	$path =~ s{(.)/+$}{$1}s;  # remove final slashes
	$path =~ s/^.*\///s;
	return $path;
}


=pod "

=item I<base_path($path)>

Returns the whole path without its last component, and the path itself if
it only has one component.

=cut "

sub base_path {
	my ($path) = @_;
	throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
		unless (defined $path);
	$path =~ s{(.)/+$}{$1}s;
	return $1 if $path =~ /^(.+)\/[^\/]+$/s;
	return $path;
}


=pod "

=item I<path_components($path)>

Returns in an array the base_dir and the base_name of the path passed
along.  In case it could not match a filesystem separator, returns
nothing for the base_dir or the base_dir and the base_name.

=cut "

sub path_components {
	my ($path) = @_;
	throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
		unless (defined $path);
	if ($path =~ m{^(.+?)/+([^/]+)/*$}s) {
		return ($1, $2);
	} elsif ($path =~ m{^/*([^/]+)/*$}s) {
		return ("", $1);
	} else {
		return ("", "");
	}
}


=pod "

=item I<path_append($path1, $path2, ...)>

Appends two or more paths.  If one is absolute, forgets the path built so
far.

=cut "

sub path_append {
	my @args = @_;
	throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
		unless ($#args >= 1);
	my $result = "";
	foreach my $path (@args) {
		$result = (&is_absolute_path ($path) ? &clean_path ($path) :
			&clean_path ($result . '/' . $path));
	}
	return $result;
}


=pod "

=item I<file_read($file)>

Returns the contents of the file:

=over 4

=item *

As a string (scalar) if called in a scalar context;

=item *

As an array of lines if called in a list context.

=back

=cut "

# XXX (TODO: test it) this may not work if the filename has imbedded
#   blanks
sub file_read {
	my ($filename) = @_;
	throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
		unless (defined $filename);

	local *FILE; # No side effects!
	open (FILE, "<", $filename) || 
		throw IDX::PKI::Error::IO("Impossible to open() for reading: $!",
			-IOfile => $filename);

	if (wantarray()) {
		local $/ = "\010"; # default value: line mode. Hence no side effects!
		my @res = <FILE>;
		close FILE; # Can this fail? What then?
		return @res;
	} else {
		local $/ = undef; # slurp mode. No side effects!
		my $res = <FILE>;
		close FILE; # Can this fail? What then?
		return $res;
	}
}


=pod "

=item I<file_write($filename, $contents, %args)>

Writes the contents passed to the destination file.  The contents may be a
scalar or a reference to an array, in which case each line of which must
end with a C<\n> character and have no other C<\n> character embedded.

Returns a true value if things worked out fine.

Optional args C<-append> means to append and not overwrite.

=cut "

sub file_write {
	my ($filename, $contents, %args) = @_;
	throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
		unless (defined $filename && defined $contents);

	my $text;
	if (ref $contents eq "") {
		$text = $contents;
	} elsif (ref $contents eq "ARRAY") {
		$text = "";
		foreach my $line (@{$contents}) {
			# If array ref, lines must have just one final \n and none
			# before, or else.
			throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
				unless ($line =~ m/^[^\n]*\n$/s);
			$text .= $line;
		}
	} else {
		throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
			unless ((ref $contents eq "ARRAY") || (ref $contents eq ""));
	}

	local *FILE; # no side effects!

	my ($mode, $detail) = ($args{"-append"} ? (">>", "appending") :
		(">", "writing"));

	open (FILE, $mode, $filename) || 
		throw IDX::PKI::Error::IO("Unable to open() for ".$detail.": $!",
		-IOfile => $filename);

	#TODO test the return value of the print calls?
	print FILE $text;
	close (FILE);
	return 1;
}


=pod "

=item I<file_append($filename, $contents)>

Appends the contents at the end of the file if it does exist.  Creates the
file if it does not exist.  Same remarks and conditions as the I<file_write()>
sub.

=cut "

sub file_append {
	my ($destination_file, $contents) = @_;
	throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
		unless (defined $destination_file && defined $contents);
	return &file_write($destination_file, $contents, -append => 1);
}


=pod "

=item I<file_link($source, $destination, %args)>

Links the source file to the destination file (which must be on the same
filesystem, of course, if a hard link [default] is used).  This can be used
for backups prior to updating (through a I<file_move()>!) because it is an
atomic operation.

The optional argument C<-symbolic> means to use a symbolic link.

=cut "

sub file_link {
	my ($source_file, $destination_file, %args) = @_;
	throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
		unless (defined $source_file && defined $destination_file);

	if (defined $args{"-symbolic"}) {
		symlink ($source_file, $destination_file) ||
			throw IDX::PKI::Error::IO("Problem symlinking '$source_file' ".
				"to '$destination_file': $!");
	} else {
		link ($source_file, $destination_file) ||
			throw IDX::PKI::Error::IO("Problem linking '$source_file' ".
				"to '$destination_file': $!");
	}
}


=pod "

=item I<file_move($source, $destination)>

Moves the source file to the destination file (which must be on the same
filesystem, of course, for the operation to be atomic).

=cut "

sub file_move {
	my ($source_file, $destination_file) = @_;
	throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
		unless (defined $source_file && defined $destination_file);

	throw IDX::PKI::Error::IO("Can't rename() to",-IOfile=>$destination_file)
		unless (rename ($source_file, $destination_file));
}


=pod "

=item I<file_delete()>

Deletes the file.

=cut "

sub file_delete {
	my $filename = shift;
	throw IDX::PKI::Error::Internal("INCORRECT_ARGUMENTS")
		unless (defined $filename);

	throw IDX::PKI::Error::IO("Can't unlink()",-IOfile=>$filename)
		unless (unlink ($filename));
}


=pod "

=back

=head1 B<IDX::PKI::FileManip::TempDirectory>

This module is an object class. Typical usage:

 my $directory = new IDX::PKI::FileManip::TempDirectory($dir);

Returns the name of a newly created subdirectory of I<$dir> (C<.> by
default).  You can do whatever you want inside.  It will be removed with
its contents when I<$directory> falls out of scope.

=cut "

package IDX::PKI::FileManip::TempDirectory;

use overload '""' => \&getname;

sub new {
	my ($class,$maindir)=@_;
	if (!defined $maindir) {
		require IDX::PKI::Config;
		my $config = the IDX::PKI::Config;
		$maindir = $config->getval ("paths", "vardir")."/tmp";
	}

	my $name;
	TRY: for(1..10) {
		my $tryname=$maindir."/".IDX::PKI::FileManip::unique();
		mkdir($tryname) && do { $name=$tryname; last TRY; }
	};
	throw IDX::PKI::Error::IO("Can't create temp dir.",
				  -IOfile => $maindir)
		unless (defined $name);

	return bless({name=>$name},$class);
};

sub getname { my $self=shift; return $self->{name}; }

sub DESTROY {
	my ($self)=@_;
	system("/bin/rm","-rf",$self->{name});
};


1;

eval(join '',<main::DATA>) || die $@ unless caller();

__END__

#########################################################################

package IDX::PKI::FileManip;
use IDX::Test;

number_of_tests(23);

test "unique" => sub {
	my $u = IDX::PKI::FileManip::unique("ahaha");
	TEST($u =~ m/ahaha/);
	my $pid = $$;
	TEST($u =~ m/$pid/);
	TESTFAIL if ($u eq IDX::PKI::FileManip::unique("ahaha"));
	# Admittedly pretty lame. But how can one test uniqueness reliably ?
};


test "is_absolute_path with an absolute path" => sub {
	TEST (is_absolute_path ("/"));
	TEST (is_absolute_path ("/an/absolute/path"));
};


test "is_absolute_path with a non absolute path" => sub {
	TEST (!is_absolute_path ("not/an/absolute/path"));
};


test "is_path_safe with safe paths" => sub {
	TEST (is_path_safe ("/"));
	TEST (is_path_safe ("/an/absolute/path"));
	TEST (is_path_safe ("not/an/absolute/path"));
	TEST (is_path_safe ("yet/an/other/safe/path/"));
};


test "is_path_safe with paths containing spaces or NULLs" => sub {
	TEST (!is_path_safe ("this/is/not good"));
	TEST (!is_path_safe ("this/ei\0ther"));
};


test "is_path_safe with paths containing ..'s" => sub {
	TEST (!is_path_safe ("../where/am/i"));
	TEST (!is_path_safe ("here/../you/are"));
	TEST (!is_path_safe ("/../the/root/dir"));
	TEST (!is_path_safe ("/i/am/lost/.."));
};


test "is_path_safe with paths containing unharming ..'s or single .'s" => sub {
	TEST (is_path_safe ("this..one/is/ok"));
	TEST (is_path_safe ("/this/.one/./too"));
};


test "clean_path removes multiple /'s" => sub {
	TEST (clean_path ("many////slashes") eq "many/slashes");
	TEST (clean_path ("//trailing/slashes//") eq "/trailing/slashes/");
};


test "clean_path removes single .'s" => sub {
	TEST (clean_path ("./a/./test/path/.") eq "a/test/path");
	TEST (clean_path ("/./yet/another/./") eq "/yet/another/");
	TEST (clean_path ("leave/this./alone") eq "leave/this./alone");
};


test "clean_path doesn't remove single ..'s except ^/.." => sub {
	TEST (clean_path ("../a/../test/path/..") eq "../a/../test/path/..");
	TEST (clean_path ("/../yet/another/../") eq "/yet/another/../");
};


test "base_name" => sub {
	TEST (base_name ("/path/to/a/file") eq "file");
	TEST (base_name ("path/to/a/dir/") eq "dir");
	TEST (base_name ("single") eq "single");
	TEST (base_name ("/single") eq "single");
};


test "base_path" => sub {
	TEST (base_path ("/path/to/a/file") eq "/path/to/a");
	TEST (base_path ("path/to/a/dir/") eq "path/to/a");
	TEST (base_path ("single") eq "single");
	TEST (base_path ("/single") eq "/single");
};


test "path_components" => sub {
	my ($path, $name);
	($path, $name) = path_components ("/path/to/a/file");
	TEST ($path eq "/path/to/a" && $name eq "file");
	($path, $name) = path_components ("path/to/a/dir/");
	TEST ($path eq "path/to/a" && $name eq "dir");
	($path, $name) = path_components ("single");
	TEST ($path eq "" && $name eq "single");
	($path, $name) = path_components ("/single");
	TEST ($path eq "" && $name eq "single");
	($path, $name) = path_components ("single/");
	TEST ($path eq "" && $name eq "single");
};


test "path_append" => sub {
	TEST (path_append ("/an/example", "path") eq "/an/example/path");
	TEST (path_append ("/whith/an absolute", "/path") eq "/path");
};


my $file = new IDX::Test::Filename;
my $content = <<EOF;
This is a test file.
On several lines.
EOF
local *FILE;
open (FILE, ">", $file);
print FILE $content;
close (FILE);

test "file_read in scalar context" => sub {
	my $read_string = file_read ($file);
	TEST ($read_string eq $content);
};


test "file_read in array context" => sub {
	my @read_array = file_read ($file);
	TEST (join ("", @read_array) eq $content);
};


my $new_content = <<EOF;
This is a new content.
To erase the old one.
EOF

test "file_write in normal mode" => sub {
	file_write ($file, $new_content);
	TEST (file_read ($file) eq $new_content);
};


test "file_write in append mode" => sub {
	file_write ($file, $content, '-append' => 1);
	TEST (file_read ($file) eq $new_content.$content);
};


test "file_append" => sub {
	file_append ($file, $new_content);
	TEST (file_read ($file) eq $new_content.$content.$new_content);
};


test "Creating a directory out of nowhere (shall fail)" => sub {
	try {
		new IDX::PKI::FileManip::TempDirectory("/no/where");
	} catch IDX::PKI::Error::IO with {
		TESTOK;
	};
};


my $tmpdirname;
do {
	my $tmpdir;
	integrationtest "IDX::PKI::FileManip::TempDirectory creates a directory" => sub {
		$tmpdir = new IDX::PKI::FileManip::TempDirectory();
		$tmpdirname = $tmpdir."";
		TEST (-d $tmpdirname);

		local *TESTFD;
		open(TESTFD,">$tmpdir/.zoinxrc") || TESTFAIL;
		print TESTFD "[zoinx]\n";
		close(TESTFD);
		1;
	};

	integrationtest "Dodging existing filenames" => sub {
		# Five times in a row, try to predict the filenames and
		# create files where IDX::PKI::FileManip::TempDirectory::new() will
		# find them.
		for my $i (1..9) {
			my $unique = IDX::PKI::FileManip::unique();
			for(1..$i) {
				open(TESTFD,">$tmpdir/".$unique);
				close(TESTFD);
				$unique =~ s/^(.*\D)(\d+)$/$1.(1+$2)/e;
			};
			TEST(my $inner = new IDX::PKI::FileManip::TempDirectory($tmpdir));
			# Means that it saw our directories, but
			# successfully dodged them.
			TEST($inner =~ m/$unique/);
		};
	};
}; # End of scope for $tmpdir - deletion should kick in.


test "out-of-scope deletion works" => sub {
	(defined $tmpdirname && -d $tmpdirname) ? TESTFAIL : TESTOK;
};
