#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: ChangePrivs.pm,v 1.17 2003/07/23 13:20:45 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Interface/ChangePrivs.pm,v $

package IDX::PKI::Interface::ChangePrivs;

use strict;

=pod "

=head1 NAME

B<IDX::PKI::Interface::ChangePrivs> - Code factorisation for GUI-enabled privilege management (Access Control Lists) in IDX-PKI

=head1 SYNOPSIS

Use the following snippet to make a CGI that can change the existing ACL
of an IDX-PKI application:

 use strict;

 use IDX::PKI::Config;
 use IDX::PKI::Error;
 use IDX::PKI::Registry;

 use IDX::PKI::Interface::ChangePrivs;
 use IDX::PKI::Authentication qw (%privilegeNames);

 $cgi->setupViews ("mypage.php");
 $cgi->authorize ("PRIV_CHPRIV", $config->getSectionName
 	('RA', ["ACL", 'web_acl']));
 $log->addAuthorizer ($cgi->getPeerCertificate ());

 my $section = "somesection";
 my @privs = keys (%privilegeNames);
 IDX::PKI::Interface::ChangePrivs::runChange
 	($cgi, $config, $log, $section, @privs);

=head1 DESCRIPTION

This module is merely used for code factorisation, because operations are
so much the same whenever you want to change the access privileges of an
arbitrary number of administrators.

B<< Warning: It is meant to work only with B<IDX::PKI::Registry>, and not
with obsolete B<IDX::PKI::CGI>. >>

In the current implementation, ACLs are stored in the configuration (see
L<IDX::PKI::Authentication> for more details). The fields
C<can-configure>, C<can-change-privs>, C<can-maintain> and C<can-use> from
the config file's section are updated by this module; others are left
unchanged.

CGI parameters are expected to have a certain format: There can be an
arbitrary number of administrators (at least one!) as long as they are
numbered in a growing order, with no holes. Only modified administrators
need to be provided. The DN for administrator #I<< <nb> >> should be
passed as parameter I<< MODIFY_DN_<nb> >>, while its "use" privilege is
defined as I<< SET_PRIV_USE_<nb> >>, etc (refer to
L<IDX::PKI::Authentication/%privilegeNames> for all available privileges).
For example:

 MODIFY_DN_0="CN=whomever, O=here, C=fr"
 SET_PRIV_CONFIG_0="y"
 SET_PRIV_MAINT_0="y"
 SET_PRIV_CHPRIV_0="y"
 MODIFY_DN_1="CN=someoneelse, O=here, C=fr"
 MODIFY_DN_2="CN=anotherguy, O=here, C=fr"
 SET_PRIV_USE_2="y"

Here are the performed operations:

=head2 CGI Input Validation

Checks that I<MODIFY_DN_*> parameters have a correct content (according to
I<IDX::PKI::ValidateInput::dn>). I<SET_PRIV_*> are supposed to be booleans
(according to I<IDX::PKI::ValidateInput::boolean>). Also checks for
duplicate entries. An error terminates the CGI (see
L<IDX::PKI::Registry/error()>).

=head2 Configuration Writing

Configuration is updated and, if no error occured, written on the hard
disk (see L<IDX::PKI::Config/updateLocked()>). Whatever happens, the
action is loggued (see L<IDX::PKI::Log/attempt()>).

It is noteworthy that an administrator may raise or lower any privilege,
except their own. Such attempt raises an B<IDX::PKI::Error::Security>.
However, this restriction doesn't prevent switching PKI security officers.
The procedure is simple and sound: The former security officer has to
authorize the newcomer, who in turn must revoke his predecessor.

If an error occured during configuration modification attempt, the proper
error is returned to the view.

=cut "

use IDX::PKI::Error;
use IDX::PKI::DN;
use IDX::PKI::ValidateInput;
use IDX::PKI::Config;
use IDX::PKI::Authentication qw (%privilegeNames);

=pod "

=head1 FUNCTIONS

=over 4

=item I<runChange ($cgi, $config, $log, $section, @privs)>

For a detail of all operations performed by I<runChange()>, please refer
to L</DESCRIPTION>. Expected paramters are:

=over 4

=item I<$cgi>

The B<IDX::PKI::Registry::CGIobject> object which has been set by a
previous call of C<use IDX::PKI::Registry>;

=item I<$config>

The B<IDX::PKI::Config> object which has been set by a previous call of
C<use IDX::PKI::Registry>;

=item I<$log>

The B<IDX::PKI::Log> object which has been set by a previous call of C<use
IDX::PKI::Registry>;

=item I<$section>

The section to which the given C<can-*> lines should be updated (for
example, C<ACL-CA>, or C<PROFILE-standard>);

=item I<@privs>

A list of significant C<PRIV_*> privileges for this run (may or may not be
the whole content of a C<keys (%IDX::PKI::Authentication::privilegeNames)>
call).

=back

The return value has no special meaning, as this function automatically
terminates the CGI run upon error by calling the appropriate C<<
$cgi->error () >> or C<< $cgi->failure () >>.

=cut "

sub runChange {
	my ($cgi, $config, $log, $section, @privs) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (UNIVERSAL::isa ($cgi,    "IDX::PKI::Registry::CGIobject") &&
				UNIVERSAL::isa ($config, "IDX::PKI::Config") &&
				UNIVERSAL::isa ($log,    "IDX::PKI::Log") &&
				$config->SectionExists ($section) &&
				scalar (@privs));

	# Check that the supplied DNs are valid.
	my %newadmins;
	my $selfindex;	# View-friendly.
	LINES: for (my $i = 0; 1; $i++) {	# Will exit if MODIFY_DN_$i is undef.
		my $dn = $cgi->validate ("MODIFY_DN_$i", \&IDX::PKI::ValidateInput::dn,
								 "-default" => undef);
		if ($cgi->validateFailures () &&
			$cgi->internalparam ("ERROR_MODIFY_DN_$i")) {
			$cgi->internalparam ("ERROR", "dn");
			$cgi->error (INPUT_FORMAT);
		}
		last LINES unless (defined $dn);	# Just empty: Last line.

		# I'm there! Keep in mind my index for the view.
		$selfindex = $i if (IDX::PKI::DN::compare ($dn, [$cgi->getPeerDN ()]));

		# Duplicate entry
		if (exists $newadmins{$dn}) {
			$cgi->internalparam ("ERROR_MODIFY_DN_$i", "DUPLICATE");
			$cgi->error (INPUT_HIDDEN);
		}

		foreach my $priv (@privs) {
			my $value = $cgi->validate
				("SET_${priv}_$i", \&IDX::PKI::ValidateInput::boolean,
				 "-default" => undef);
			# Can a boolean be invalid??
			$newadmins{$dn}{$priv} = $value;
		}
	}

	# We want at least one entry!
	unless (%newadmins) {
		$cgi->internalparam ("ERROR", "dn");
		$cgi->error (INPUT_MISSING);
	}

	try {
		$log->attempt
			(["CONFIG", "-details" => ["section $section",
									   "admin ".scalar ($cgi->getPeerDN ()),
									   scalar (keys (%newadmins)).
									   " modified administrator(s)",
									   ],
			  "-attachment" => \%newadmins],
			 sub {
				 IDX::PKI::Config->updateLocked (sub {
					 my ($config) = @_;
					 foreach my $priv (@privs) {
						 my $subsect = $privilegeNames{$priv};
						 # Currently allowed admins for this privilege.
						 # Must not throw if there are none!
						 my @curadmins = $config->getPath ($section,
														   "can-$subsect");

						 # You may not modify your own privileges. We
						 # check it here because we _may_ get the peer DN
						 # here if the user does a global change, but we
						 # just don't want it to be changed from the
						 # current config. Beware, $dn may not be
						 # canonical!

						 my $dn = [$cgi->getPeerDN ()];
						 my $curright = (grep {IDX::PKI::DN::compare ($dn, $_)}
										 @curadmins) ? 1 : 0;
						 my $newright = (grep {IDX::PKI::DN::compare ($dn, $_)
												   && $newadmins{$_}{$priv}}
										 keys (%newadmins)) ? 1 : 0;

						 # $selfindex is set only if I sent infos about my
						 # own privs.

						 if (defined ($selfindex) &&
							 ($curright != $newright)) {
							 $cgi->internalparam ("ERROR_MODIFY_DN_$selfindex",
												  "PERMISSION_DENIED");
							 throw IDX::PKI::Error::Security
								 ("INPUT_HIDDEN", "-details" => "Attempt to ".
								  "change one's own privileges (priv $priv, ".
								  "current value: $curright, new value: ".
								  "$newright, index: $selfindex)",
								  "-admin" => $dn);
						 }

						 # Keep untouched administrators unchanged.
						 my @old = grep {! exists ($newadmins{$_})} @curadmins;
						 my @allowed = grep {$newadmins{$_}{$priv}} keys
							 (%newadmins);
						 $config->setval ($section, "can-$subsect",
										  @old, @allowed);
					 }
				 });
			 });
	} catch IDX::PKI::Error with {
		$cgi->failure (CONFIG_SAVING);
	};
}


=pod "

=back

=cut "

require IDX::Test::Suite unless (caller());

1;

__END__

##################### Test suite ##########################

use strict;

use IDX::PKI::Config;
use IDX::PKI::Interface::ChangePrivs;
use IDX::PKI::Error;
use IDX::PKI::DN;
use IDX::PKI::Authentication qw (%privilegeNames);

my ($thegood, $thebad, $theugly, $thegooddn, $thebaddn, $theuglydn);
BEGIN {
	# Just for fun - unused :-)
	$thegood = ["CN"=>"John R. Hacker",  "DC"=>"example", "DC"=>"net"];
	$thebad  = ["CN"=>"John R. Luser",   "DC"=>"example", "DC"=>"net"];
	$theugly = ["CN"=>"John R. Cracker", "DC"=>"example", "DC"=>"net"];
	($thegooddn, $thebaddn, $theuglydn) = map {
		IDX::PKI::DN::string (@$_)
		} ($thegood, $thebad, $theugly);

	# Prepare a config file that will be read and written to.
	my $config = set IDX::PKI::Config (<< "PRIVS");
[log]
type=syslog
facility=daemon
logopt=pid

[bogusmodule]
can-change-privs=$thegooddn
can-change-privs=CN=God Himself, O=too many symbolic links
can-configure=$theuglydn
can-maintain=$thegooddn
can-use=$theuglydn
can-use=$thebaddn

PRIVS
}

# Only _after_ having changed $config.
use IDX::PKI::Registry;

# Will contain the CGI return code.
our $cgireturn;
{
	no warnings;
	sub IDX::PKI::Registry::CGIobject::_choose_redirect_method {
		my (undef, undef, $code) = @_;
		$cgireturn = $code;
		die "End of CGI: $code";
	}
	sub IDX::PKI::Registry::CGIobject::getPeerDN {
		return wantarray ? @$thegood : $thegooddn;
	}
}

sub cgiparams {
	my (%params) = @_;
	$cgi = IDX::PKI::Registry::CGIobject->_new ($log, \%params);
	$cgi->setupViews ("somepage.php");
	$cgi->authorize ("__NONE__");
}

our $section = "bogusmodule";
our @privs   = keys (%privilegeNames);
our @subsect = values (%privilegeNames);

test "runChange (valid changes)" => sub {
	&cgiparams (
				"MODIFY_DN_0"       => $theuglydn,
				"MODIFY_DN_1"       => $thebaddn,
				"SET_PRIV_MAINT_1"  => 1,
				"SET_PRIV_CONFIG_1" => 1,
				);
	IDX::PKI::Interface::ChangePrivs::runChange ($cgi, $config, $log,
												 $section, @privs);

	my %privs;
	foreach my $subsect (@subsect) {
		foreach my $admin ($config->getPath ($section, "can-$subsect")) {
			$privs{$admin}{$subsect} = 1;
		}
	}

	# Good: Unchanged, still change privs and maintain.
	TEST (! $privs{$thegooddn}{"use"}) || warn "The good can use";
	TEST (! $privs{$thegooddn}{"configure"}) || warn "The good can configure";
	TEST ($privs{$thegooddn}{"change-privs"}) ||
		warn "The good can't change privs";
	TEST ($privs{$thegooddn}{"maintain"}) ||
		warn "The good can't maintain";

	# Bad: Configure and maintain.
	TEST (! $privs{$thebaddn}{"use"}) || warn "The bad can use";
	TEST ($privs{$thebaddn}{"configure"}) || warn "The bad can't configure";
	TEST (! $privs{$thebaddn}{"change-privs"}) ||
		warn "The bad can change privs";
	TEST ($privs{$thebaddn}{"maintain"}) || warn "The bad can't maintain";

	# Ugly: Nothing.
	TEST (! $privs{$theuglydn}{"use"}) || warn "The ugly can use";
	TEST (! $privs{$theuglydn}{"configure"}) || warn "The ugly can configure";
	TEST (! $privs{$theuglydn}{"change-privs"}) ||
		warn "The ugly can change privs";
	TEST (! $privs{$theuglydn}{"maintain"}) || warn "The ugly can maintain";

	warn Data::Dumper::Dumper (\%privs) if (TESTFAILED);
};


test "runChange (no change)" => sub {
	# The arguments must not be used, because count starts at 0.
	&cgiparams (
				"MODIFY_DN_1"       => $theuglydn,
				"SET_PRIV_CHPRIV_1" => 1,
				);
	try {
		IDX::PKI::Interface::ChangePrivs::runChange ($cgi, $config, $log,
													 $section, @privs);
		warn "runChange went to its normal end";
		TESTFAIL;
	} otherwise {
		TESTOK;
	};

	my %privs;
	foreach my $subsect (@subsect) {
		foreach my $admin ($config->getPath ($section, "can-$subsect")) {
			$privs{$admin}{$subsect} = 1;
		}
	}

	# Good: Still change privs and maintain.
	TEST (! $privs{$thegooddn}{"use"}) || warn "The good can use";
	TEST (! $privs{$thegooddn}{"configure"}) || warn "The good can configure";
	TEST ($privs{$thegooddn}{"change-privs"}) ||
		warn "The good can't change privs";
	TEST ($privs{$thegooddn}{"maintain"}) ||
		warn "The good can't maintain";

	# Bad: Still configure and maintain.
	TEST (! $privs{$thebaddn}{"use"}) || warn "The bad can use";
	TEST ($privs{$thebaddn}{"configure"}) || warn "The bad can't configure";
	TEST (! $privs{$thebaddn}{"change-privs"}) ||
		warn "The bad can change privs";
	TEST ($privs{$thebaddn}{"maintain"}) || warn "The bad can't maintain";

	# Ugly: Still nothing.
	TEST (! $privs{$theuglydn}{"use"}) || warn "The ugly can use";
	TEST (! $privs{$theuglydn}{"configure"}) || warn "The ugly can configure";
	TEST (! $privs{$theuglydn}{"change-privs"}) ||
		warn "The ugly can change privs";
	TEST (! $privs{$theuglydn}{"maintain"}) || warn "The ugly can maintain";

	warn Data::Dumper::Dumper (\%privs) if (TESTFAILED);
};


test "runChange (duplicate DN)" => sub {
	&cgiparams (
				"MODIFY_DN_0"       => $thebaddn,
				"SET_PRIV_CHPRIV_0" => 1,
				"MODIFY_DN_1"       => $thebaddn,
				"SET_PRIV_CHPRIV_1" => 1,
				);
	try {
		IDX::PKI::Interface::ChangePrivs::runChange ($cgi, $config, $log,
													 $section, @privs);
		warn "runChange went to its normal end";
		TESTFAIL;
	} otherwise {
		TESTOK;
	};

	TEST ($cgireturn == INPUT_HIDDEN) ||
		warn "Didn't get a ".INPUT_HIDDEN." error ($cgireturn)";
	TEST ($cgi->{"subreq"}{"ERROR_MODIFY_DN_1"} eq "DUPLICATE") ||
		warn "Bad ERROR_MODIFY_DN_1: ".$cgi->{"subreq"}{"ERROR_MODIFY_DN_1"};

	# Must not have changed.
	my %privs;
	foreach my $subsect (@subsect) {
		foreach my $admin ($config->getPath ($section, "can-$subsect")) {
			$privs{$admin}{$subsect} = 1;
		}
	}

	# Bad: Still configure and maintain.
	TEST (! $privs{$thebaddn}{"use"}) || warn "The bad can use";
	TEST ($privs{$thebaddn}{"configure"}) || warn "The bad can't configure";
	TEST (! $privs{$thebaddn}{"change-privs"}) ||
		warn "The bad can change privs";
	TEST ($privs{$thebaddn}{"maintain"}) || warn "The bad can't maintain";

	# Ugly: Still nothing.
	TEST (! $privs{$theuglydn}{"use"}) || warn "The ugly can use";
	TEST (! $privs{$theuglydn}{"configure"}) || warn "The ugly can configure";
	TEST (! $privs{$theuglydn}{"change-privs"}) ||
		warn "The ugly can change privs";
	TEST (! $privs{$theuglydn}{"maintain"}) || warn "The ugly can maintain";

	warn Data::Dumper::Dumper (\%privs) if (TESTFAILED);
};


test "runChange (can't change one's own privs)" => sub {
	&cgiparams (
				"MODIFY_DN_0"       => $thegooddn,
				"SET_PRIV_MAINT_0"  => 1,
				"SET_PRIV_USE_0"    => 1,
				"SET_PRIV_CONFIG_0" => 1,
				"SET_PRIV_CHPRIV_0" => 1,
				);
	try {
		IDX::PKI::Interface::ChangePrivs::runChange ($cgi, $config, $log,
													 $section, @privs);
		warn "runChange went to its normal end";
		TESTFAIL;
	} otherwise {
		TESTOK;
	};

	TEST ($cgireturn == CONFIG_SAVING) ||
		warn "Didn't get a ".CONFIG_SAVING." error ($cgireturn)";
	TEST ($cgi->{"subreq"}{"ERROR_MODIFY_DN_0"} eq "PERMISSION_DENIED") ||
		warn "Bad ERROR_MODIFY_DN_0: ".$cgi->{"subreq"}{"ERROR_MODIFY_DN_0"};

	# Must not have changed.
	my %privs;
	foreach my $subsect (@subsect) {
		foreach my $admin ($config->getPath ($section, "can-$subsect")) {
			$privs{$admin}{$subsect} = 1;
		}
	}

	# Good: Still change privs and maintain.
	TEST (! $privs{$thegooddn}{"use"}) || warn "The good can use";
	TEST (! $privs{$thegooddn}{"configure"}) || warn "The good can configure";
	TEST ($privs{$thegooddn}{"change-privs"}) ||
		warn "The good can't change privs";
	TEST ($privs{$thegooddn}{"maintain"}) ||
		warn "The good can't maintain";

	# Ugly: Still nothing.
	TEST (! $privs{$theuglydn}{"use"}) || warn "The ugly can use";
	TEST (! $privs{$theuglydn}{"configure"}) || warn "The ugly can configure";
	TEST (! $privs{$theuglydn}{"change-privs"}) ||
		warn "The ugly can change privs";
	TEST (! $privs{$theuglydn}{"maintain"}) || warn "The ugly can maintain";

	warn Data::Dumper::Dumper (\%privs) if (TESTFAILED);
};


# Actually this is a regression test.  When one accreditated a bogus DN,
# it would never be deleted during subsequent upddates.
test "runChange (bogus DN)" => sub {
	# First, accreditate a DN.
	&cgiparams (
				"MODIFY_DN_0"       => "bogusDN",
				"SET_PRIV_MAINT_0"  => "y",
				"SET_PRIV_CONFIG_0" => "y",
				"SET_PRIV_USE_0"    => "y",
				"SET_PRIV_CHPRIV_0" => "y",
				);
	IDX::PKI::Interface::ChangePrivs::runChange ($cgi, $config, $log,
												 $section, @privs);

	# Then clear it.
	&cgiparams (
				"MODIFY_DN_0"       => "bogusDN",
				);
	IDX::PKI::Interface::ChangePrivs::runChange ($cgi, $config, $log,
												 $section, @privs);

	# Should not be there anymore.
	my %privs;
	foreach my $subsect (@subsect) {
		foreach my $admin ($config->getPath ($section, "can-$subsect")) {
			$privs{$admin}{$subsect} = 1;
		}
	}

	TEST (! exists ($privs{"bodugDN"})) ||
		warn Data::Dumper::Dumper (\%privs);
};

1;
