#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::PKI::LDAP> - Head IDX-PKI module for LDAP operations

=head1 SYNOPSIS

 use IDX::PKI::LDAP;

 my $repository = new IDX::PKI::LDAP($section);
 # One may perform standard Net::LDAP calls afterwards.

 my $repository = new IDX::PKI::LDAP::CertDB($section);

=head1 DESCRIPTION

B<IDX::PKI::LDAP> is a wrapper class that inherits from B<Net::LDAP>
(see L<Net::LDAP>).  It is a standard LDAP client socket that offers
several authentication methods, and can set up itself up from a
section in an B<IDX::PKI::Config> object. Additionnally,
I<IDX::PKI::LDAP> supports LDAP over UNIX domain sockets (see
L</Net::LDAPI>).

Said configuration section looks like this for a local (UNIX domain)
connection to the LDAP server:

 [CA]
 repository_ldap=repository-caaccess

 [LDAP-CLIENT-repository-caaccess]
 commtype=unix      ; or "plain" or "ssl"
 bindpw=secret

Or for an SSL connection:

 [publish]
 repository_ldap=publishing

 [LDAP-CLIENT-publishing]
 host=snail.mycompany.com
 port=389
 binddn=cn=CA,ou=IDX-PKI,o=mycompany
 comm=ssl
 ;; One may (try to) authenticate oneself with a certificate:
 cert=/etc/idxpki/certs/net-CA-cert.pem
 key=/var/idxpki/secure/CA/net-CA-key.pem
 ;; One may also wish to authenticate the peer:
 netDN=CN=some.server,OU=IDXPKI,O=somewhere


The parameters are as follows:

=over 4

=item C<commtype=type>

The "wire format" that will be spoken by the client, among C<plain>,
C<SSL> or C<UNIX> (case insensitive).  Defaults to plain.

=item C<host=some.where.com>

=item C<port=123>

Standard TCP/IP service location.

=item C<socket=/where/is/it>

If commtype="UNIX", this is where to find the socket.

=item C<netDN=O=someorg,CN=someone>

If you want to authenticate the LDAPS server we are connecting to, use
this option.  The CA parameters will be fetched from section [CA].

=item C<binddn=cn=someone,o=somewhere>

We will try to convince the LDAP server that we are this guy.

=item C<authtype=type>

Available types are C<password> (synonym C<passwd>), C<SASL> (don't know
exactly what it is yet, but sounds cool) and C<SSL> (certificate-based
authentication, but see remark below).  Case is insensitive.  Of course,
C<SSL> authentication is only available over C<SSL> commtype.

I<SAD NOTE:> The OpenLDAP version present in RedHat 7.1 does not support
X.509 SSL authentication.  We have to resort to passwords (but still over
an encrypted channel - honor is safe).

=item C<cert=/a/file>

=item C<key=/another/file>

The key material for authenticating ourselves to the peer, in case we want
to (and can) do SSL authentication.

=item C<netDN=CN=someCN,O=someOrg>

The DN that the peer will need to exhibit in his SSL certificate (this
option is therefore only valid if commtype=ssl). If we fail to
authenticate the peer, an I<IDX::PKI::Error::Security> exception will
be raised. The peer must be signed by IDX-PKI's CA (whose certs
are fetched from the C<[paths]/cacertsdir> entry of the configuration).

=cut "

package IDX::PKI::LDAP;

use strict;

use Net::LDAP 0.26 qw(LDAP_SIZELIMIT_EXCEEDED LDAP_NO_SUCH_OBJECT);
  # 0.23 shipped with RedHat has a synchronism bug that prevents it
  # from reporting errors correctly over SSL.
use Net::LDAPS;
use Net::LDAP::Util qw(ldap_error_name ldap_error_text);

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Debug (0);
use IDX::PKI::Authentication qw(authenticate);
use IDX::PKI::DN;
use IDX::SSLSocket qw(SSLscope);
use IDX::Error;

use Exporter; our @ISA=qw(Exporter);
our @EXPORT_OK=qw(quotefilter matches datefilter datenow);

=pod "

=head1 METHODS

=over 4

=item I<new ($section)>

Object constructor. I<$section> is the name of the section in which to
find the LDAP connection information. The LDAP connection will be
automatically constructed (using L<Net::LDAP/new>) and bound (with
L<Net::LDAP/bind>).

=cut "

sub new {
	my ($classname, $section) = @_;

	throw IDX::Error::Internal("INCORRECT_ARGS") unless ($section);
	my $config = the IDX::PKI::Config ();

	my $self = bless ({'config' => $config,
					   'section' => $section},
					  $classname);

	my ($host, $port) = map {$self->getConfig($_)} (qw(host port));

	# Open network session.

	my $onerror = sub {
		my $mesg = shift;
		return $mesg if ($mesg->isa("Net::LDAP::Search") &&
						 ($mesg->code() == LDAP_SIZELIMIT_EXCEEDED ||
						  $mesg->code() == LDAP_NO_SUCH_OBJECT));
		local $IDX::PKI::Error::Internal::inhibitTrace = 1;
		throw IDX::Error::Network
			("Error while communicating with LDAP server",
			 -details => ldap_error_text($mesg->code),
			 -LDAPmsg => $mesg,
			 -host => $host,
			 -port => $port);
	};

	my $type = $self->getConfig("commtype");

	if ((!$type) || ($type =~ m/^plain$/i)) {		# Plain-text
		throw IDX::Error::Config("Cannot get LDAP server location",
									  '-section' => $section)
			unless ($host && $port);
		$self->{"cnx"} = new Net::LDAP ($host, "port" => $port,
										"onerror" => $onerror);

	} elsif ($type =~ m/^unix$/i) {	# LDAPI
		my $sock = $self->getConfig("socket");
		throw IDX::Error::Config("Must specify socket parameter for ".
									  "UNIX LDAP clients",
									  -section => $section)
			unless (defined $sock);
		$self->{"cnx"} = new Net::LDAPI ($sock, "onerror" => $onerror);

	} elsif ($type =~ m/^ssl$/i) {	# LDAPS
		throw IDX::Error::Config("Cannot get LDAP server location",
									  '-section' => $section)
			unless ($host && $port);

		my @newopts = ($host, "port" => $port, "onerror" => $onerror);

		# SSL authentication for two, for me...
		my ($cert, $key) = map {$self->getConfig($_)} (qw(cert key));
		push (@newopts, "clientcert" => $cert, "clientkey" => $key)
			if ((defined $cert) && (defined $key));

		# ... and you.
		if (defined $self->getConfig("netDN")) {
			my $cacertsdir = $self->{"config"}->getval ("paths", "cacertsdir");
			$self->{"SSL_cert_checked"} = 1;
			push (@newopts, "verify" => 'require', "capath" => $cacertsdir);
		}

		SSLscope {
			$self->{"cnx"} = new Net::LDAPS (@newopts);
		};

	    # Check peer cert just after setting up the socket.
		if (defined(my $dn = $self->getConfig("netDN"))) {
			authenticate($self, $dn);
		}
	} else {
		throw IDX::Error::Config("Unknown commtype $type",
									  -section=>$section);
	}

 	defined($self->{cnx}) || throw IDX::Error::Network
		("Can't connect to LDAP server", "-type" => $type, "-error" => "$@",
		 (($type && $type =~ m/^unix$/i) ?
		  ("-sock" => $self->getConfig("socket")) :
		  ("-host" => $host, "-port" => $port)));

	###### Bind (if required)

	my @bindopts;

	my $authtype = $self->getConfig("authtype");

	if ((! $authtype) || ($authtype =~ m/^passw(or)?d$/i)) {
		my ($dn, $pass) = ($self->getConfig("binddn"),
						   $self->getConfig("bindpw"));

		if (!defined $pass) {
			my $passfile = $self->getConfig("bindpwfile");
			if (defined $passfile) {
				local *READPASS;
				open (READPASS, "<$passfile") ||
					throw IDX::Error::IO ("Can't read password file",
											   "-IOfile" => $passfile);
				$pass = <READPASS>;
				chomp $pass;
				close(READPASS);
			}
		}

		push (@bindopts, $dn, "password" => (defined $pass ? $pass : ""))
			if ($dn);
	} elsif ($authtype =~ m/^ssl$/i) {
		# Nothing to do - we presented our cert above, and if we did
		# not then we will be locked out anyway.
	} elsif ($authtype =~ m/^sasl$/i) {
		throw IDX::Error::Internal("UNIMPLEMENTED");
	} else {
		throw IDX::Error::Config("Unknown authtype $authtype");
	}

 	my $ret = $self->bind(@bindopts);

	throw IDX::Error::Network("Cannot bind to the LDAP server",
								   "-details" => ldap_error_text($ret->code))
		if ($ret->code);

	return $self;
}


=pod "

=item I<getPeerDN(@path)>

See L<IDX::PKI::Authentication/getPeerDN>.

=cut "

sub getPeerDN {
	my ($self) = @_;
	return undef unless $self->{'SSL_cert_checked'};
	my $cert = $self->certificate();
	return undef if (! $cert);

	my $name = $cert->subject_name();
	throw IDX::Error::Internal("SHOULDNT_HAPPEN",-DN=>$name) unless
		($name =~ m|^/(.*)$|);

	return wantarray ? parse IDX::PKI::DN($name) : $name;
}


=pod "

=item I<getConfig(@path)>

Convenience function for doing calls to
L<IDX::PKI::Config/getPath>. @path should be the same as in said
function, only without the leading section name which is implicit
here.

=cut "

sub getConfig {
	my ($self, @path) = @_;
	return $self->{'config'}->getPath ($self->{'section'},@path);
}

=pod "

=item I<AUTOLOAD()>

This a crude trick to get some sort of classless inheritance w.r.t. the
underlying LDAP object (I<< $self->{cnx} >>) whose type we do not know at
compiletime (can be B<Net::LDAP> or B<Net::LDAPS>).  This is dubbed
"object delegation" in perlbot.

Upon calling of any method that is not defined in I<Net::LDAP>, this
function is called. Three cases may arise:

=over 2

=item *

the called method is an LDAP write method (one of I<add()>, I<modify()>,
I<delete()> and I<moddn()>): the appropriate call is forwarded to
I<< $self->{cnx} >> and the return value is checked for errors (following
the policy set up in L<IDX::PKI::Error>);

=item *

the called method is an LDAP read method (i.e. any other method): the
appropriate call is forwarded, leaving the catching of errors to
L<Net::LDAP/onerror>;

=item *

the called method does not exist in the I<< $self->{cnx} >> object either: an
error is raised.


=back

=cut "

use vars qw($AUTOLOAD);
sub AUTOLOAD {
	my ($self, @args) = @_;
	return if $AUTOLOAD =~ /::DESTROY$/;

	my ($pack, $func) = ($AUTOLOAD =~ m/^(.*)::(.*?)$/);

	DEBUG("Calling cnx's $func");

	throw IDX::Error::Internal ("Cannot find $func through package $pack",
									 "-method" => $AUTOLOAD,
									 "-args" => \@args)
		unless ((ref $self) && (ref($self->{cnx})));

	die ("Cannot find $func through delegated object ".ref($self->{cnx}).
		 " (original object ".ref($self).")")
		unless (my $funcref = $self->{cnx}->can($func));

	return &$funcref($self->{cnx},@args);
}


=pod "

=item I<disconnect()>

Calls I<unbind()>.  For compatibility's sake only.

=cut "

sub disconnect {
	my ($self) = @_;
	$self->unbind();
}

=pod "

=back

=head2 Utility functions

=over 4

=item I<quotefilter($value,$wildcardsOK)>

This utility function (not a method) transforms $value by quoting the
characters that RFC2254 reserve for special purposes in LDAP filter
strings (except "*" if $wildcardsOK is true). The undef value for $value
is turned into "*" by this function.

=cut "

sub quotefilter {
	my ($value,$wildcardsOK)=@_;
	if (!defined $value) {
		return "*";
	};

	$value =~ s/\\/\\5c/g;
	$value =~ s/\(/\\28/g;
	$value =~ s/\)/\\29/g;
	$value =~ s/\000/\\00/g;

	unless ($wildcardsOK) {
		$value =~ s/\*/\\2a/g;
	};
	return $value;
}

=pod "

=item I<matches($value,$pattern)>

Returns true iff $value matches $pattern, taking into account the possible
wildcards in $pattern. An undef value for $pattern always matches.

=cut "

sub matches {
	my ($value,$pattern)=@_;
	return 1 if (!defined $pattern);
	$pattern = quotemeta($pattern);
	$pattern =~ s/\\\*/.*/g;
	return (scalar($value =~ $pattern));
}

=pod "

=item I<datefilter($fieldname,$operator,$date)>

Returns an LDAP pattern appropriate for matching the contents of
$fieldname with $date (GMT, in ISO notation) using operator $operator
(one of "<=", ">=", "<>" (identical to "!="), or "=" (identical to
"==")).  Resolution of dates is assumed to be seconds. $date may be
mangled with any intervening non-digit non-word characters, so that
"2002-04-04 18:39:00" is a valid ISO date as regards this function.

=cut "

sub datefilter {
	my ($fieldname,$operator,$date)=@_;

	# Strict comparison is not first-class in LDAP.
	if ($operator eq "<" || $operator eq ">") {
		return sprintf("(&%s%s)",
					   datefilter($fieldname,"!=",$date),
					   datefilter($fieldname,$operator."=",$date));
	};

	my $digits=join("",($date =~ m/\d/g));
	throw IDX::Error::Internal("INCORRECT_ARGS") unless
	  (length($digits)>= 8 && length($digits) <= 14);
	$digits.="0"x(14-length($digits));

	if (($operator eq "<=") || ($operator eq ">=")) {
		return "($fieldname$operator${digits}Z)";
	} elsif (($operator eq "==") || ($operator eq "=")) {
		return "($fieldname=${digits}Z)";
	} elsif (($operator eq "<>") || ($operator eq "!=")) {
		return "(!($fieldname=${digits}Z))";
	} else {
		throw IDX::Error::Internal("INCORRECT_ARGS");
	};
}

=pod "

=item I<datenow()>

Returns the current (GMT) date, in classical ISO/LDAP form
(e.g. "20020404183900Z").
datenow can too be used with a number of day in parameter.
datenow(+1) return the tomorrow's date ; datenow(-1) of yesterday.

=cut "

sub datenow {
	$_[0]=0 unless defined($_[0]);
	my ($sec,$min,$hour,$day,$mon,$year)=gmtime(time()+86400*$_[0]);
	return sprintf("%04d%02d%02d%02d%02d%02dZ",
					 $year+1900,$mon+1,$day,$hour,$min,$sec);
}

###################################################################

=pod "

=back

=head1 B<IDX::PKI::LDAP::CertDB> CLASS

Subclass of B<IDX::PKI::LDAP> for handling those internal databases in
IDXPKI that are implemented using LDAP servers. Said approach solves a lot
of database-oriented problems like indexation, publication over a network
and access control. (Right now the CA database and the escrow database are
implemented like this - see L<IDX::PKI::CertDB>).

In the current implementation of B<IDX::PKI::CertDB>, the certificates are
stored into the LDAP database as a flat hierarchy one level under the base
DN which is itself at the top of the DIT, using an I<idxpkiStrong>
nonstandard node type (see F<sources/src/etc/ldap/idxpki.schema>) which
contains as many fields as necessary to duplicate the internal data of the
certificates and allow filtering on them. The B<IDX::PKI::LDAP::CertDB>
class encapsulates encoding of certificates to this form, as well as
searching by certificate fields.

=cut "

package IDX::PKI::LDAP::CertDB;

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Debug (0);
use Net::LDAP::Entry;
use IDX::PKI::Crypto::X509;

use strict;

our @ISA=qw(Exporter IDX::PKI::LDAP);
our @EXPORT_OK=qw(cert2entry entry2cert);

import IDX::PKI::LDAP qw(quotefilter matches datefilter datenow);


=pod "

=over 4

=item I<< searchStrongs (-base => "O=top", -scope => "one", -filter =>
"(description=nothing)", -sizelimit => 10, $filtername1 => $filterexpr1,
$filtername2 => $filterexpr2, ...) >>

Performs a special search over certificates in an IDXPKI LDAP database,
which are of LDAP type I<idxpkiStrong>. The arguments with a leading C<->
are those to be fed to I<Net::LDAP::search()>, after removing the C<-> of
course; I<-base> defaults to C<< $config->getval ("dn", "idxpki_ldapbase")
>>; if a I<-filter> is present, it will be ANDed with the other search
criteria. The other arguments will be converted into the appropriate LDAP
filters (with additional client-side Perl pipework if need be).

The return value in list context is a list of matching B<Net::LDAP::Entry>
objects, in no particular order, or in scalar context one arbitrary
B<Net::LDAP::Entry> among those that match the search.

Search filters are specified by pairs of arguments. The first item is the
name of the criterion (it may be preceded by a C<!> character to negate
the search on this criterion); the second is the filter to apply to this
criterion (typically the literal value to look for or to reject, but
wildcards are OK for some criteria).

Not all certificates have all criteria inside. For example, there are
certificates without subject DNs, certificates without extensions,
etc. Specifying undef as a search criterion is always valid and only
selects those certificates that have a value in this field, whatever it is.

Available criteria and appropriate filters for each of them are:

=over 2

=item I<serial>

A serial number. There is at most one certificate for each given
serial number, and all certificates have a serial number. Certificate
serial number 0 is the OpCA's self-signed certificate, if it exists
(in CISCO-bugwared IPSEC certification policies).

=item I<DN>

An I<IDX::PKI::DN> object or RFC2253 string. This DN is the one that
has to be present in the certificate (B<not> the DN under which it is
stored in the repository LDAP, that you shouldn't be aware of to begin
with). Not all certificates have a DN: per RFC2459, some have just a
SubjectAltName.

=item I<CN>

The I<CommonName> field of the DN. May contain the "*"
character with its usual wildcard semantics. Again, not all
certificates have a CN - even "user" certificates, even those having a
DN, but one may "reasonably" assume that certificates having a DN and
being "user" certificates have a CN.

=item I<ID>

The DN fields I<uniqueIdentifier> (nonstandard, OID 2.5.4.45) or
I<dnQualifier> (2.5.4.46). In some (most ?) certification policies,
no certificate owner will ever have an ID.  I<ID> is B<not> the same
as I<serial>: one person (identified by one user ID) may own several
certificates, either simultaneously or sequentially.

=item I<validAt>

An ISO date or date+time (e.g. "20020521 10:21:00"), or a form like
"now+4" (where "4" is in days). Selecting (validAt=>"now",
revocation=>"V") selects all currently active certificates.

=item I<subjectAltName>

The subject alternative name of the certificate, passed as a reference
to a list with two elements. The first is the type of the
subjectAltName field of the matching certificates: "FQDN" (synonymous
to "DNS"), "URI", "UFQDN" (synonymous to "email"), "IPv4", or
"IPv4subnet". The second is the expected content of the subjectAltName
field of the certificates ("*" wildcards are appropriate).

In case of multiple subjectAltNames in one certificate, any of them
matching said name and type will elicit a search hit. As is the case
for single-scalar search criteria, one or both of the name and type
may be specified as undef, meaning a wildcard at this place.

=item I<extension>

An X509v3 keyUsage, extendedKeyUsage or nsCertType extension to be
present in the certificate (see L<IDX::PKI::X509/%keyUsageExtensions>
for a list of recognized keywords as values for this
criterion). Certificates will typically have more than one extension.

=item I<email>

The email of the certificate, either as a subjectAltName or as a
(nonstandard) component of the DN. "*" wildcards are appropriate.

=back

=cut "

sub searchStrongs {
	my ($self,@criteria)=@_;

	throw IDX::Error::Internal("WRONG_NUMBER_ARGS") if
	  (@criteria % 2);

	# Handle "legacy" options first.
	my %criteria=@criteria;
	my $legacyfilter=$criteria{-filter};
	my $config = the IDX::PKI::Config ();
	$criteria{-base} ||= $config->getval ("dn", "idxpki_ldapbase");
	my @legacy;
	foreach my $legacykey (grep {m/^-/} (keys %criteria)) {
		next if ($legacykey eq "-filter");
		$legacykey =~ m/^-(.*)$/; push(@legacy,$1,$criteria{$legacykey});
	};


	my (@LDAPcriteria, @perlyescriteria, @perlnocriteria);
	while (my ($k, $pattern) = splice (@criteria, 0, 2)) {
		next if ($k =~ m/^-/);
		throw IDX::Error::Internal("INCORRECT_ARGS") unless
		  ($k =~ m/^(\!?)(\w+)$/);
		my ($neg,$name) = ($1,$2);

		my ($ldapfilt,$perlfilt);
		if ($name eq "serial") {
			next if (!defined $pattern); # All certs have a serial number
			$ldapfilt = sprintf ("(idxpkiSerial=%s)",
								 quotefilter($pattern, 1));
		} elsif ($name eq "DN") {
			$ldapfilt = sprintf("(idxpkiSubject=%s)",
								(UNIVERSAL::isa($pattern, "IDX::PKI::DN") ?
								 quotefilter($pattern->string(),0) :
								 quotefilter($pattern,1)));
		} elsif ($name eq "CN") {
			my $filt = quotefilter ($pattern, 1);
			$ldapfilt = sprintf ("(idxpkiSubject=*CN=%s%s)",
								 $filt,
								 ($filt =~ m/\*$/ ? "" : "*"));
			if (defined $pattern) {
				$perlfilt = sub {
					my ($entry) = @_;
					my $cert = entry2cert ($entry);
					my $dn = build IDX::PKI::DN ($cert->getSubjectDN());
					my $cn = $dn->getRDNValue ("cn");
					return ($cn && matches ($cn, $pattern));
				};
			}
		} elsif ($name eq "ID") {
			$ldapfilt = sprintf ("(idxpkiSubjectID=%s)",
								 quotefilter ($pattern, 1));
		} elsif ($name eq "validAt") {
			next if (!defined $pattern); # All certs have a validity period
			if ($pattern eq "now") {
				$pattern=datenow();
			}
			$ldapfilt = sprintf
				("(&%s%s)",
				 datefilter("idxpkiCertPublicationTime", "<=", $pattern),
				 datefilter("idxpkiCertExpirationTime", ">=", $pattern));

		} elsif ($name eq "subjectAltName") {
			throw IDX::Error::Internal("UNIMPLEMENTED");
		} elsif ($name eq "extension") {
			$ldapfilt = sprintf ("(idxpkiExtension=%s)",
								 quotefilter($pattern));
		} elsif ($name eq "email") {
			$ldapfilt = sprintf ("(idxpkiSubjectEmail=%s)",
								 quotefilter($pattern,1));
		} else {
			throw IDX::Error::Internal("INCORRECT_ARGS",
									   '-criterion' => $name);
		}

		if ($neg) {
			push (@LDAPcriteria, "(!$ldapfilt)") if ($ldapfilt);
			push (@perlnocriteria, $perlfilt) if ($perlfilt);
		} else {
			push (@LDAPcriteria, $ldapfilt) if ($ldapfilt);
			push (@perlyescriteria, $perlfilt) if ($perlfilt);
		}

	} # End of loop over function arguments

	my $filter = sprintf ('(&%s)',
						  join("",
							   "(objectClass=idxpkiStrong)",
							   ($legacyfilter ? ($legacyfilter) : ()),
							   @LDAPcriteria));
	DEBUG("LDAP filter = $filter");

	my @hits = $self->search (@legacy, "filter" => $filter)->all_entries();

	my @result;
	foreach my $hit (@hits) {
		next if (grep {! &$_($hit)} @perlyescriteria);
		next if (grep {&$_($hit)} @perlnocriteria);
		push (@result, $hit);
	}

	# Special case here: if the Perl criteria remove some certificates
	# from the answer, and there was a capping on the size of the
	# search, then we may have suppressed more certs than really
	# intended. We should then restart the query, doubling the
	# sizelimit.
	if ($criteria{"-sizelimit"} && ($#result < $#hits) &&
		($#hits == $criteria{"-sizelimit"})) {
		# The last condition of the test is the key to the termination
		# of the algorithm: if the search led to even less certs than
		# requested, then in fact the search was exhaustive (not
		# capped) and our client-side filtering is therefore perfectly
		# correct w.r.t. the capping.
		my @bigger=searchStrongs($self,@criteria,
							 -sizelimit=>2*$criteria{-sizelimit});
		$#bigger = $criteria{-sizelimit}-1 if
		  ($#bigger > $criteria{-sizelimit}-1);
		return wantarray ? @bigger: $bigger[$#bigger];
	};

	return wantarray ? @result: $result[$#result];
}

=pod "

=item I<< cert2entry($certificate) >>

This function (not a method) returns a B<Net::LDAP::Entry> object from
I<$certificate>, an B<IDX::PKI::Crypto::X509> object. This entry will have
the LDAP type I<idxpkiStrong> and will feature all fields that will make
it suitable for successful searches using I<searchCerts()>. The entry has
no DN (setting it is up to the caller by calling the appropriate C<<
$entry->dn (...) >>).

=cut "

sub cert2entry {
	my ($x509) = @_;

	# Checking that the given argument is not undefined
	throw IDX::Error::Internal ("INCORRECT_ARGS",
		'-details' => 'No x509 certificate.') unless ($x509);

	# Checking that the given $x509 argument is really an X509 object
	throw IDX::Error::Internal ("INCORRECT_ARGS",
		'-details' => 'The given argument is not an X509 object')
		unless (UNIVERSAL::isa($x509, "IDX::PKI::Crypto::X509"));

	##### fetch certificate details

	my $certSerialNumber = $x509->getSerial();

	my @dn=$x509->getSubjectDN();
	my $subject;
	if (@dn) { # Not all certificates have a DN, but all CA certs do.
		$subject=IDX::PKI::DN::string(@dn);
	};

	my $certPubTime = $x509->getAttribute ('not_before_zulu');
	my $certExpTime = $x509->getAttribute ('not_after_zulu');

	my ($uid,$emailAddress,$servername);
	try {
		$uid = $x509->getUserID();
	} otherwise {};
	try {
		$emailAddress = $x509->getEmail();
	} otherwise {};
	try {
		$servername = $x509->getAltName();
	} otherwise {};

	my @extensions=$x509->getExtensions();

	##### Now cast to LDAP

	my $entry = new Net::LDAP::Entry();

	$entry->add(
				'userCertificate;binary' => $x509->serialize("DER"),

				'objectclass' => 'idxpkiStrong',
				($uid ?          ('idxpkiSubjectID' => $uid) : ()),
				($servername ?   ('idxpkiSubjectID' => $servername) : ()),
				($emailAddress ? ('idxpkiSubjectEmail' => $emailAddress) : ()),
				'idxpkiCertPublicationTime' => $certPubTime,
				'idxpkiCertExpirationTime' => $certExpTime,

				($subject ?      ('idxpkiSubject' => $subject) : ()),
				'idxpkiSerial' => $certSerialNumber,
				(@extensions ?   ('idxpkiExtension' => \@extensions) : ()),
				);

	return $entry;
}

=pod "

=item I<entry2cert($entry)>

This function (not a method) is the converse of the above. It reads the
appropriate field (namely I<userCertificate>) from $entry, which is
a I<Net::LDAP::Entry>, and casts it to an I<IDX::PKI::Crypto::X509> object.

=cut "

sub entry2cert {
	my ($entry)=@_;
	return scalar parse IDX::PKI::Crypto::X509
					  ($entry->get_value("usercertificate;binary"),"DER");
}

=pod "

=back

=head1 Net::LDAPI CLASS

This class is a descendant of L<Net::LDAP>, from CPAN, that handles
UNIX domain sockets.

=over 4

=item I<new()>

The object constructor, copied/modified from I<Net::LDAP>. It accepts
the same arguments as L<Net::LDAP/new> except that the first argument
is the path of the UNIX domain socket instead.

=cut "

package Net::LDAPI;

use strict;
use IO::Socket::UNIX ();
use Net::LDAP ();

# Zoinx me if I understand why the canonical "use vars qw(@ISA);
# @ISA=qw()" fails all tests after the one for Net::LDAPI !!!!
@Net::LDAPI::ISA=qw(Net::LDAP);

sub new {
  my $self = shift;
  my $type = ref($self) || $self;
  my $sockpath = shift if @_ % 2;
  my $arg  = &Net::LDAP::_options(@_);
  my $obj  = bless {}, $type;

  my $sock = IO::Socket::UNIX->new($sockpath) or return;

  $sock->autoflush(1);

  $obj->{net_ldap_socket}  = $sock;
  $obj->{net_ldap_host}    = "localhost";
  $obj->{net_ldap_resp}    = {};
  $obj->{net_ldap_version} = $arg->{version} || $Net::LDAP::LDAP_VERSION;
  $obj->{net_ldap_async}   = $arg->{async} ? 1 : 0;

  my %onerror =
	('die'   => sub {
		 require Carp;
		 Carp::croak(_err_msg(@_))
	 },
	 'warn'  => sub { require Carp; Carp::carp(_err_msg(@_)); $_[0] },
	 'undef' => sub { require Carp; Carp::carp(_err_msg(@_)) if $^W; undef },
	);

  if (defined(my $onerr = $arg->{onerror})) {
    $onerr = $onerror{$onerr} if exists $onerror{$onerr};
    $obj->{net_ldap_onerror} = $onerr;
  }

  $obj->debug($arg->{debug} || 0 );

  $obj;
}

=pod "

=back

=head1 BUGS

=over 4

=item *

SSL authentication just doesn't work with OpenLDAP 2.0.7, and won't
anytime soon (it is expected for version 2.2 in the roadmap).

=back

=head1 SEE ALSO

=over 2

=item RFC2251

Lightweight Directory Access Protocol (v3)

=item RFC2253

UTF-8 String Representation of Distinguished Names

=item RFC2254

The String Representation of LDAP Search Filters

=item F<sources/src/etc/ldap/idxpki.schema>

The LDAP schema for IDXPKI-specific data types

=cut "


eval join("", <main::DATA>) || die $@ unless caller();
1;

__END__

####################### TEST SUITE ###############################

package IDX::PKI::LDAP;

use strict;

use IDX::PKI::Config;
use IDX::Test;
use IDX::Test::LDAPServer;
use IDX::PKI::Error;

import IDX::PKI::LDAP::CertDB qw(cert2entry entry2cert);

# All tests are integration tests, because they use
# IDX::Test::LDAPServer::Stateful servers. Maybe we should rather use
# InPerl servers for tests which don't rely on a real server.
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (1);

	test "No unit test!" => sub {
		TESTOK();
	};

	exit(0);
}

number_of_tests(34);

#  Certificate:
#      Data:
#          Version: 3 (0x2)
#          Serial Number: 3 (0x3)
#          Signature Algorithm: md5WithRSAEncryption
#          Issuer: C=FR, L=Paris, O=Test Company, OU=Computer Ressources, OU=PKI, CN=Test Root CA/Email=ca@test.com
#          Validity
#              Not Before: Jul 10 12:50:58 2001 GMT
#              Not After : Jul 10 12:50:58 2002 GMT
#          Subject: C=FR, O=Test Company, OU=Computer Ressources, CN=C0000042/Email=Waza@idealx.com
#          Subject Public Key Info:
#              Public Key Algorithm: rsaEncryption
#              RSA Public Key: (512 bit)
#                  Modulus (512 bit):
#                      00:a9:7d:[...]:58:e3
#                  Exponent: 65537 (0x10001)
#          X509v3 extensions:
#              X509v3 Basic Constraints:
#                  CA:FALSE
#              X509v3 Subject Key Identifier:
#                  47:C9:C1:62:D3:77:39:0F:A9:C5:0A:CD:EB:77:3E:19:ED:1C:9F:F1
#              X509v3 Authority Key Identifier:
#                  keyid:A4:4C:B5:11:BF:63:2D:D9:5B:EE:AB:E7:43:94:40:9F:85:78:19:D8
#                  DirName:/C=FR/L=Paris/O=Test Company/OU=Computer Ressources/OU=PKI/CN=Test Root CA/Email=ca@test.com
#                  serial:00
#              X509v3 Subject Alternative Name:
#                  email:Waza@idealx.com
#              X509v3 Issuer Alternative Name:
#                  email:ca@test.com
#              X509v3 Key Usage:
#                  Digital Signature, Non Repudiation, Key Encipherment
#      Signature Algorithm: md5WithRSAEncryption
#          5a:53:[...]:4f:4e
my $serializedX509_1 = << 'EOT';
-----BEGIN CERTIFICATE-----
MIIDLjCCAtigAwIBAgIBAzANBgkqhkiG9w0BAQQFADCBkzELMAkGA1UEBhMCRlIx
DjAMBgNVBAcTBVBhcmlzMRUwEwYDVQQKEwxUZXN0IENvbXBhbnkxHDAaBgNVBAsT
E0NvbXB1dGVyIFJlc3NvdXJjZXMxDDAKBgNVBAsTA1BLSTEVMBMGA1UEAxMMVGVz
dCBSb290IENBMRowGAYJKoZIhvcNAQkBFgtjYUB0ZXN0LmNvbTAeFw0wMTA3MTAx
MjUwNThaFw0wMjA3MTAxMjUwNThaMHUxCzAJBgNVBAYTAkZSMRUwEwYDVQQKEwxU
ZXN0IENvbXBhbnkxHDAaBgNVBAsTE0NvbXB1dGVyIFJlc3NvdXJjZXMxETAPBgNV
BAMTCEMwMDAwMDQyMR4wHAYJKoZIhvcNAQkBFg9XYXphQGlkZWFseC5jb20wXDAN
BgkqhkiG9w0BAQEFAANLADBIAkEAqX1DaF3BEqacy3W/ixIJEY2IyyBGu81TydaP
PxqMSiHZBNiz6THUwCDvQKt0NxaNvw7UuEGEJKaD1flI0jZY4wIDAQABo4IBMjCC
AS4wCQYDVR0TBAIwADAdBgNVHQ4EFgQUR8nBYtN3OQ+pxQrN63c+Ge0cn/EwgcAG
A1UdIwSBuDCBtYAUpEy1Eb9jLdlb7qvnQ5RAn4V4GdihgZmkgZYwgZMxCzAJBgNV
BAYTAkZSMQ4wDAYDVQQHEwVQYXJpczEVMBMGA1UEChMMVGVzdCBDb21wYW55MRww
GgYDVQQLExNDb21wdXRlciBSZXNzb3VyY2VzMQwwCgYDVQQLEwNQS0kxFTATBgNV
BAMTDFRlc3QgUm9vdCBDQTEaMBgGCSqGSIb3DQEJARYLY2FAdGVzdC5jb22CAQAw
GgYDVR0RBBMwEYEPV2F6YUBpZGVhbHguY29tMBYGA1UdEgQPMA2BC2NhQHRlc3Qu
Y29tMAsGA1UdDwQEAwIF4DANBgkqhkiG9w0BAQQFAANBAFpTGnYGApyGxC/0yC2T
IUElG8ogPLAnD0wX64N30vfNx9ITrZw8FVv9Tf7EOe7q2pNotrmifDoReewBLOcK
T04=
-----END CERTIFICATE-----
EOT

#  Certificate:
#      Data:
#          Version: 3 (0x2)
#          Serial Number: 4 (0x4)
#          Signature Algorithm: md5WithRSAEncryption
#          Issuer: C=FR, L=Paris, O=Test Company, OU=Computer Ressources, OU=PKI, CN=Test Root CA/Email=ca@test.com
#          Validity
#              Not Before: Jul 10 12:50:58 2001 GMT
#              Not After : Jul 10 12:50:58 2002 GMT
#          Subject: C=FR, O=Test Company, OU=Computer Ressources, CN=C0345123/Email=Wazup@idealx.com
#          Subject Public Key Info:
#              Public Key Algorithm: rsaEncryption
#              RSA Public Key: (512 bit)
#                  Modulus (512 bit):
#                      00:a9:[...]:e3
#                  Exponent: 65537 (0x10001)
#          X509v3 extensions:
#              X509v3 Basic Constraints:
#                  CA:FALSE
#              X509v3 Subject Key Identifier:
#                  47:C9:C1:62:D3:77:39:0F:A9:C5:0A:CD:EB:77:3E:19:ED:1C:9F:F1
#              X509v3 Authority Key Identifier:
#                  keyid:A4:4C:B5:11:BF:63:2D:D9:5B:EE:AB:E7:43:94:40:9F:85:78:19:D8
#                  DirName:/C=FR/L=Paris/O=Test Company/OU=Computer Ressources/OU=PKI/CN=Test Root CA/Email=ca@test.com
#                  serial:00
#              X509v3 Subject Alternative Name:
#                  email:Wazup@idealx.com
#              X509v3 Issuer Alternative Name:
#                  email:ca@test.com
#              X509v3 Key Usage:
#                  Digital Signature, Non Repudiation, Key Encipherment
#      Signature Algorithm: md5WithRSAEncryption
#          53:[...]:5e
my $serializedX509_2 = << 'EOT';
-----BEGIN CERTIFICATE-----
MIIDMDCCAtqgAwIBAgIBBDANBgkqhkiG9w0BAQQFADCBkzELMAkGA1UEBhMCRlIx
DjAMBgNVBAcTBVBhcmlzMRUwEwYDVQQKEwxUZXN0IENvbXBhbnkxHDAaBgNVBAsT
E0NvbXB1dGVyIFJlc3NvdXJjZXMxDDAKBgNVBAsTA1BLSTEVMBMGA1UEAxMMVGVz
dCBSb290IENBMRowGAYJKoZIhvcNAQkBFgtjYUB0ZXN0LmNvbTAeFw0wMTA3MTAx
MjUwNThaFw0wMjA3MTAxMjUwNThaMHYxCzAJBgNVBAYTAkZSMRUwEwYDVQQKEwxU
ZXN0IENvbXBhbnkxHDAaBgNVBAsTE0NvbXB1dGVyIFJlc3NvdXJjZXMxETAPBgNV
BAMTCEMwMzQ1MTIzMR8wHQYJKoZIhvcNAQkBFhBXYXp1cEBpZGVhbHguY29tMFww
DQYJKoZIhvcNAQEBBQADSwAwSAJBAKl9Q2hdwRKmnMt1v4sSCRGNiMsgRrvNU8nW
jz8ajEoh2QTYs+kx1MAg70CrdDcWjb8O1LhBhCSmg9X5SNI2WOMCAwEAAaOCATMw
ggEvMAkGA1UdEwQCMAAwHQYDVR0OBBYEFEfJwWLTdzkPqcUKzet3PhntHJ/xMIHA
BgNVHSMEgbgwgbWAFKRMtRG/Yy3ZW+6r50OUQJ+FeBnYoYGZpIGWMIGTMQswCQYD
VQQGEwJGUjEOMAwGA1UEBxMFUGFyaXMxFTATBgNVBAoTDFRlc3QgQ29tcGFueTEc
MBoGA1UECxMTQ29tcHV0ZXIgUmVzc291cmNlczEMMAoGA1UECxMDUEtJMRUwEwYD
VQQDEwxUZXN0IFJvb3QgQ0ExGjAYBgkqhkiG9w0BCQEWC2NhQHRlc3QuY29tggEA
MBsGA1UdEQQUMBKBEFdhenVwQGlkZWFseC5jb20wFgYDVR0SBA8wDYELY2FAdGVz
dC5jb20wCwYDVR0PBAQDAgXgMA0GCSqGSIb3DQEBBAUAA0EAU5rampE25TXti5Yv
Wo9CgnK+hmo81RYaSLk5a04IRP4mRC98I9Kj3By+cD0TrhQfMLhwqWgx2bCL2PqJ
QG8hXg==
-----END CERTIFICATE-----
EOT

#  Certificate:
#      Data:
#          Version: 3 (0x2)
#          Serial Number: 5 (0x5)
#          Signature Algorithm: md5WithRSAEncryption
#          Issuer: C=FR, L=Paris, O=Test Company, OU=Computer Ressources, OU=PKI, CN=Test Root CA/Email=ca@test.com
#          Validity
#              Not Before: Jul 10 12:50:58 2001 GMT
#              Not After : Jul 10 12:50:58 2002 GMT
#          Subject: C=FR, O=Test Company, OU=Computer Ressources, CN=C0345124/Email=Zorglub@idealx.com
#          Subject Public Key Info:
#              Public Key Algorithm: rsaEncryption
#              RSA Public Key: (512 bit)
#                  Modulus (512 bit):
#                      00:[...]:e3
#                  Exponent: 65537 (0x10001)
#          X509v3 extensions:
#              X509v3 Basic Constraints:
#                  CA:FALSE
#              X509v3 Subject Key Identifier:
#                  47:C9:C1:62:D3:77:39:0F:A9:C5:0A:CD:EB:77:3E:19:ED:1C:9F:F1
#              X509v3 Authority Key Identifier:
#                  keyid:A4:4C:B5:11:BF:63:2D:D9:5B:EE:AB:E7:43:94:40:9F:85:78:19:D8
#                  DirName:/C=FR/L=Paris/O=Test Company/OU=Computer Ressources/OU=PKI/CN=Test Root CA/Email=ca@test.com
#                  serial:00
#              X509v3 Subject Alternative Name:
#                  email:Zorglub@idealx.com
#              X509v3 Issuer Alternative Name:
#                  email:ca@test.com
#              X509v3 Key Usage:
#                  Digital Signature, Non Repudiation, Key Encipherment
#      Signature Algorithm: md5WithRSAEncryption
#          38:[...]:6d
my $serializedX509_3 = << 'EOT';
-----BEGIN CERTIFICATE-----
MIIDNDCCAt6gAwIBAgIBBTANBgkqhkiG9w0BAQQFADCBkzELMAkGA1UEBhMCRlIx
DjAMBgNVBAcTBVBhcmlzMRUwEwYDVQQKEwxUZXN0IENvbXBhbnkxHDAaBgNVBAsT
E0NvbXB1dGVyIFJlc3NvdXJjZXMxDDAKBgNVBAsTA1BLSTEVMBMGA1UEAxMMVGVz
dCBSb290IENBMRowGAYJKoZIhvcNAQkBFgtjYUB0ZXN0LmNvbTAeFw0wMTA3MTAx
MjUwNThaFw0wMjA3MTAxMjUwNThaMHgxCzAJBgNVBAYTAkZSMRUwEwYDVQQKEwxU
ZXN0IENvbXBhbnkxHDAaBgNVBAsTE0NvbXB1dGVyIFJlc3NvdXJjZXMxETAPBgNV
BAMTCEMwMzQ1MTI0MSEwHwYJKoZIhvcNAQkBFhJab3JnbHViQGlkZWFseC5jb20w
XDANBgkqhkiG9w0BAQEFAANLADBIAkEAqX1DaF3BEqacy3W/ixIJEY2IyyBGu81T
ydaPPxqMSiHZBNiz6THUwCDvQKt0NxaNvw7UuEGEJKaD1flI0jZY4wIDAQABo4IB
NTCCATEwCQYDVR0TBAIwADAdBgNVHQ4EFgQUR8nBYtN3OQ+pxQrN63c+Ge0cn/Ew
gcAGA1UdIwSBuDCBtYAUpEy1Eb9jLdlb7qvnQ5RAn4V4GdihgZmkgZYwgZMxCzAJ
BgNVBAYTAkZSMQ4wDAYDVQQHEwVQYXJpczEVMBMGA1UEChMMVGVzdCBDb21wYW55
MRwwGgYDVQQLExNDb21wdXRlciBSZXNzb3VyY2VzMQwwCgYDVQQLEwNQS0kxFTAT
BgNVBAMTDFRlc3QgUm9vdCBDQTEaMBgGCSqGSIb3DQEJARYLY2FAdGVzdC5jb22C
AQAwHQYDVR0RBBYwFIESWm9yZ2x1YkBpZGVhbHguY29tMBYGA1UdEgQPMA2BC2Nh
QHRlc3QuY29tMAsGA1UdDwQEAwIF4DANBgkqhkiG9w0BAQQFAANBADiOpbztyBW1
mqnJjWrbzh8PJxAyeyrIkjZwD9Xdlr8uAc6LAEBmcv0ulykDS09HgQTS97eERuGI
yfW2vVtGVW0=
-----END CERTIFICATE-----
EOT

#  Certificate:
#      Data:
#          Version: 3 (0x2)
#          Serial Number: 2 (0x2)
#          Signature Algorithm: md5WithRSAEncryption
#          Issuer: C=FR, L=Paris, O=Test Company, OU=Computer Ressources, OU=PKI, CN=Test Root CA/Email=ca@test.com
#          Validity
#              Not Before: Jul 10 12:50:57 2001 GMT
#              Not After : Jul 10 12:50:57 2002 GMT
#          Subject: C=FR, O=Test Company, OU=Computer Ressources, CN=C0345125/Email=client@idealx.com
#          Subject Public Key Info:
#              Public Key Algorithm: rsaEncryption
#              RSA Public Key: (512 bit)
#                  Modulus (512 bit):
#                      00:[...]:e3
#                  Exponent: 65537 (0x10001)
#          X509v3 extensions:
#              X509v3 Basic Constraints:
#                  CA:FALSE
#              X509v3 Subject Key Identifier:
#                  47:C9:C1:62:D3:77:39:0F:A9:C5:0A:CD:EB:77:3E:19:ED:1C:9F:F1
#              X509v3 Authority Key Identifier:
#                  keyid:A4:4C:B5:11:BF:63:2D:D9:5B:EE:AB:E7:43:94:40:9F:85:78:19:D8
#                  DirName:/C=FR/L=Paris/O=Test Company/OU=Computer Ressources/OU=PKI/CN=Test Root CA/Email=ca@test.com
#                  serial:00
#              X509v3 Subject Alternative Name:
#                  email:client@idealx.com
#              X509v3 Issuer Alternative Name:
#                  email:ca@test.com
#              X509v3 Key Usage:
#                  Digital Signature, Non Repudiation, Key Encipherment
#      Signature Algorithm: md5WithRSAEncryption
#          06:[...]:d8
my $serializedX509_4 = <<'EOT';
-----BEGIN CERTIFICATE-----
MIIDMjCCAtygAwIBAgIBAjANBgkqhkiG9w0BAQQFADCBkzELMAkGA1UEBhMCRlIx
DjAMBgNVBAcTBVBhcmlzMRUwEwYDVQQKEwxUZXN0IENvbXBhbnkxHDAaBgNVBAsT
E0NvbXB1dGVyIFJlc3NvdXJjZXMxDDAKBgNVBAsTA1BLSTEVMBMGA1UEAxMMVGVz
dCBSb290IENBMRowGAYJKoZIhvcNAQkBFgtjYUB0ZXN0LmNvbTAeFw0wMTA3MTAx
MjUwNTdaFw0wMjA3MTAxMjUwNTdaMHcxCzAJBgNVBAYTAkZSMRUwEwYDVQQKEwxU
ZXN0IENvbXBhbnkxHDAaBgNVBAsTE0NvbXB1dGVyIFJlc3NvdXJjZXMxETAPBgNV
BAMTCEMwMzQ1MTI1MSAwHgYJKoZIhvcNAQkBFhFjbGllbnRAaWRlYWx4LmNvbTBc
MA0GCSqGSIb3DQEBAQUAA0sAMEgCQQCpfUNoXcESppzLdb+LEgkRjYjLIEa7zVPJ
1o8/GoxKIdkE2LPpMdTAIO9Aq3Q3Fo2/DtS4QYQkpoPV+UjSNljjAgMBAAGjggE0
MIIBMDAJBgNVHRMEAjAAMB0GA1UdDgQWBBRHycFi03c5D6nFCs3rdz4Z7Ryf8TCB
wAYDVR0jBIG4MIG1gBSkTLURv2Mt2Vvuq+dDlECfhXgZ2KGBmaSBljCBkzELMAkG
A1UEBhMCRlIxDjAMBgNVBAcTBVBhcmlzMRUwEwYDVQQKEwxUZXN0IENvbXBhbnkx
HDAaBgNVBAsTE0NvbXB1dGVyIFJlc3NvdXJjZXMxDDAKBgNVBAsTA1BLSTEVMBMG
A1UEAxMMVGVzdCBSb290IENBMRowGAYJKoZIhvcNAQkBFgtjYUB0ZXN0LmNvbYIB
ADAcBgNVHREEFTATgRFjbGllbnRAaWRlYWx4LmNvbTAWBgNVHRIEDzANgQtjYUB0
ZXN0LmNvbTALBgNVHQ8EBAMCBeAwDQYJKoZIhvcNAQEEBQADQQAGmWZpscZkBw7C
EJkaBu4hAJNy3o6NHbwgZLRhTWsH/tb5Bq8osP2C4cV9OyTfFz4rJSfAeOAeLqJd
zp653urY
-----END CERTIFICATE-----
EOT

my $basedn = "O=TestPKI";

# This will create a LDAP and a LDAPS server.
my $ldif=<<"LDIF";
dn: $basedn
objectClass: organization
organizationName: IDXPKI

dn: OU=bogusdb,$basedn
objectClass: top
objectClass: organizationalUnit
organizationalUnitName: bogusdb

dn: CN=12345,OU=bogusdb,$basedn
objectClass: top
objectClass: inetOrgPerson
commonName: 12345
surname: this is not a certificate and should not show up as one.

LDIF

my $server = new IDX::Test::LDAPServer::Stateful($ldif, $basedn)->start ();

die "Can't create non-SSL LDAP" unless ($server);
my $port = $server->getPort();
my $sock = $server->getSock();
my ($rootDN, $binddn, $bindpw) = map {$server->{$_}}
	qw(suffix rootdn rootpw);

my $server_s = new IDX::Test::SSLServer (-server => $server)->start();
my $port_s = $server_s->getPort();

my $serverauthstring = &IDX::Test::getCertDN ($server_s->{'mycert'});
die "no serverauthstring" if (!defined $serverauthstring);

# Keep some values from the "real" config.
my $config = the IDX::PKI::Config ();
my $configdir      = $config->getval ("paths", "configdir");
my $withslapd      = $config->getval ("paths", "slapd");
my $withldapmodify = $config->getval ("paths", "ldapmodify");
my $withschemadir  = $config->getval ("paths", "schemadir");
my $withopenssl    = $config->getval ("paths", "openssl");
my $portldap       = $config->getval ("network", "port_ldap_repository");

# Overrite the "real" config with ours.
$config = set IDX::PKI::Config (<< "CONFIG");
[paths]
configdir=$configdir
slapd=$withslapd
ldapmodify=$withldapmodify
schemadir=$withschemadir
openssl=$withopenssl
cacertsdir=$server_s->{'cadir'}

[dn]
idxpki_ldapbase=$basedn

[log]
type=syslog
facility=daemon
logopt=pid

; LDAP client sections
[LDAP-CLIENT-plainldap-readonly]
host=localhost
port=$port
commtype=plain

[LDAP-CLIENT-plainldap-fullaccess]
host=localhost
port=$port
commtype=plain
authtype=password
binddn=$binddn
bindpw=$bindpw

; LDAPS client sections
[LDAP-CLIENT-ldaps-readonly]
host=localhost
port=$port_s
commtype=ssl

[LDAP-CLIENT-ldaps-fullaccess]
host=localhost
port=$port_s
commtype=ssl
authtype=password
binddn=$binddn
bindpw=$bindpw

[LDAP-CLIENT-ldaps-fullaccess-passinfile]
host=localhost
port=$port_s
commtype=ssl
authtype=password
binddn=$binddn
; bindpwfile= dollar bindpwfile

[LDAP-CLIENT-ldapi]
socket=$sock
commtype=UNIX

[LDAP-CLIENT-auth-readonly]
commtype=ssl
host=localhost
port=$port_s
netDN=$serverauthstring

[LDAP-CLIENT-auth-evilserver]
commtype=ssl
host=localhost
port=$port_s
netDN=CN=unexpectedServer,O=IDEALX,C=fr

CONFIG

my $ldap;

test "quotefilter()" => sub {
	TEST(length(quotefilter('()*')) eq 9);
	TEST(quotefilter('()*') eq '\\28\\29\\2a');
};

test "matches()" => sub {
	TEST(matches("portnawak",undef));
	TEST(matches("portnawak","port*"));
	TEST(matches("portna(wak","*(wak"));
};

test "datefilter()" => sub {
	TEST(datefilter("zoinx","=","20020621000001") eq
		 "(zoinx=20020621000001Z)");

	TEST(datefilter("zoinx","<=","20020621") eq
		 "(zoinx<=20020621000000Z)");

	my @a=(datefilter("zoinx","<","20020621") =~ m/zoinx/g);
	TEST(@a == 2);

	my @b=(datefilter("zoinx","<","20020621") =~ m/\(/g);
	my @c=(datefilter("zoinx","<","20020621") =~ m/\)/g);

	TEST(@b == @c);
};

test "datenow()" => sub {
	TEST(datenow() =~ m/^20\d+Z$/);
};
test "datenow(+4)" => sub {
	TEST(datenow(+4) =~ m/^20\d+Z$/);
};
test "datenow(-4)" => sub {
	TEST(datenow(-4) =~ m/^20\d+Z$/);
};

# Three functions to test the established LDAP connections.
# TESTread tests that a known node is available.
sub TESTread {
	my ($ldap)=@_;
	my $msg=$ldap->search(base=>"OU=bogusdb,".$config->getval
						  ("dn", "idxpki_ldapbase"),
						  filter=>"(objectClass=inetOrgPerson)");
	TEST($msg->count()==1) || die "Cannot read from LDAP";
	return $msg;
}

# TESTwrite tests that we can write to the same test node.
sub TESTwrite {
	my ($ldap)=@_;
	my $entry=TESTread($ldap)->entry(0);
	$entry->changetype("modify");
	my $uid=int(rand(1000));
	$entry->replace("uid",$uid);
	my $write=$entry->update($ldap);
	TEST(! $write->code()) || do {
		warn "write failed: ".$write->error();
		return 0;
	};
	$entry=TESTread($ldap)->entry(0);
	TEST($entry->get_value("uid") == $uid);
}

# TESTwritefails tests that we cannot write to yet the same test
# node, that we must nonetheless be able to read. Thus TESTwritefails
# typically only tests ACLs.
sub TESTwritefails {
	my ($ldap)=@_;
	my $entry=TESTread($ldap)->entry(0);
	$entry->changetype("modify");
	my $uid=int(rand(1000));
	$entry->replace("uid",$uid);
	try {
		my $write=$entry->update($ldap);
		warn "write should have thrown an exception !";
		return TESTFAIL();
	} otherwise {
		return TESTOK();
	};
}

test "LDAP: New, read-only" => sub {
	$ldap = new IDX::PKI::LDAP ('LDAP-CLIENT-plainldap-readonly');
	TESTread($ldap);
	$ldap->unbind();
};

test "New, full access" => sub {
	$ldap = new IDX::PKI::LDAP ('LDAP-CLIENT-plainldap-fullaccess');
	TESTwrite($ldap);
	$ldap->unbind();
};

## LDAPS
die unless (defined $server_s);

test "LDAPS: New, read-only" => sub {
	$ldap = new IDX::PKI::LDAP('LDAP-CLIENT-ldaps-readonly');
	TESTread($ldap);
	$ldap->unbind();
};


test "LDAPS: New, full access" => sub {
	$ldap = new IDX::PKI::LDAP('LDAP-CLIENT-ldaps-fullaccess');
	TESTwrite($ldap);
	$ldap->unbind();
};

# Support for this feature is momentarily out.
# test "New, password in a file" => sub {
# 	$ldap = new IDX::PKI::LDAP('LDAP-CLIENT-ldaps-fullaccess-passinfile');
# 	$ldap->unbind();
# };


test "getPeerDN" => sub {
	TEST (!defined $ldap->getPeerDN());
};

## Authentication tests

test "new (with server verification)" => sub {
	$ldap = new IDX::PKI::LDAP("LDAP-CLIENT-auth-readonly");
	my $expected = ($serverauthstring =~ m/^cn=([^,]+)/);
	TEST (scalar $ldap->getPeerDN() =~ m/$expected/) ||
		warn "Unexpected peerDN: ".$ldap->getPeerDN().
		" (looked for $expected)";
};

test "subtly evil server" => sub {
	try {
		$ldap = new IDX::PKI::LDAP("LDAP-CLIENT-auth-evilserver");
		TESTFAIL;
	} catch IDX::PKI::Error::Security with {
		TESTOK;
	};
};


### UNIX domain socket tests

test "Net::LDAPI" => sub {
	my $ldap=new Net::LDAPI($sock);
	TESTwrite($ldap);

# I believe there are more gotchas with Net::LDAPI which is but a hack..
	my $otherserver=new IDX::Test::LDAPServer::Stateful($ldif, $basedn,
														<<"ACL");
access to dn=*
	by * read
	by anonymous read
ACL
	$otherserver->start();

	$ldap=new Net::LDAPI($otherserver->getSock(),
						   onerror=>"die");
	die if (!defined $ldap);
	TESTwritefails($ldap);
};

test "INTEGRATION: OpenLDAP as installed in the deployed CertDB setup" => sub {
# Keep the ACL below in sync with src/etc/start-stop-slapd.pl
	my $server=new IDX::Test::LDAPServer::Stateful($ldif, $basedn, <<"ACL");
access to attr=userPassword
	by * auth

access to dn=*
	by sockurl.regex=^ldapi write
	by * read
	by anonymous read

# This one is necessary to allow access to schemas.
access to dn=".*"
	by * read
	by anonymous read

ACL
	$server->start();

	my $ldap=new Net::LDAPI($server->getSock(),
						   onerror=>"die");
	die if (!defined $ldap);
	TESTwrite($ldap);
 	$ldap=new Net::LDAP("localhost",port=>$server->getPort(),
					   onerror=>"die");
 	die if (!defined $ldap);
 	TESTread($ldap);
	TESTwritefails($ldap);
};

test "new IDX::PKI::LDAP (through UNIX socket)" => sub {
	my $ldap=new IDX::PKI::LDAP("LDAP-CLIENT-ldapi");
	TESTwrite($ldap);

	# REGRESSION: "onerror" handling was forgotten for LDAPI-backed
	# IDX::PKI::LDAP objects.
	try {
		my $msg=$ldap->add("CN=zoinx,OU=nosuchdb,".
						   $config->getval ("dn", "idxpki_ldapbase"),
						   attr => [ 'cn'   => "zoinx",
                                sn    => 'Jensen',
                                mail => 'b.jensen@umich.edu',
                                objectclass => ['top', 'person' ]]);
		warn "Adding a bogus DN should have thrown";
		TESTFAIL();
	} catch IDX::Error::Network with {
		TESTOK();
	};
};


######## CertDB TESTS ##################

my @certs=map {scalar parse IDX::PKI::Crypto::X509($_)}
  ($serializedX509_1, $serializedX509_2, $serializedX509_3, $serializedX509_4);
my @entries;

test "cert2entry" => sub {
	@entries = map {
		my $entry = IDX::PKI::LDAP::CertDB::cert2entry ($_);
		$entry->dn ("CN=".$_->getSerial().",OU=bogusdb,".
					$config->getval ("dn", "idxpki_ldapbase"));
		$entry;
	} @certs;

	TEST ($entries[0]->get_value("idxpkiSubjectEmail") eq
		  'Waza@idealx.com') ||
		  warn "#0 email ".$entries[0]->get_value("idxpkiSubjectEmail");
	TEST ($entries[0]->get_value("idxpkiCertPublicationTime") eq
		  '20010710125058Z') ||
		  warn "#0 pubtime ".
		  $entries[0]->get_value("idxpkiCertPublicationTime");
	TEST ($entries[0]->get_value("idxpkiCertExpirationTime") eq
		  '20020710125058Z')  ||
		  warn "#0 exptime ".
		  $entries[0]->get_value("idxpkiCertPublicationTime");;
	TEST (hex ($entries[0]->get_value("idxpkiSerial")) == 3) ||
		warn "#0 serial (hex)".hex ($entries[0]->get_value("idxpkiSerial"));
	TEST ($entries[0]->get_value("idxpkiSubject") eq
		  'Email=Waza@idealx.com,CN=C0000042,OU=Computer Ressources,'.
		  'O=Test Company,C=FR' ||
		  warn "#0 subject".$entries[0]->get_value("idxpkiSubject"));

	TEST (hex ($entries[1]->get_value("idxpkiSerial")) == 4) ||
		warn "#1 serial (hex)".hex ($entries[1]->get_value("idxpkiSerial"));
	TEST ($entries[1]->get_value("userCertificate;binary")) ||
		warn "#1 no certificate";
	TEST ($entries[1]->get_value("userCertificate;binary") !~ m/BEGIN/) ||
		warn "#1 certificate ".
		$entries[1]->get_value("userCertificate;binary");

	# All certs included in the LDAP unit test suite have the same exts...
	my @allexts = $entries[1]->get_value("idxpkiExtension");
	TEST (@allexts == 3) || warn "#1 ext ".Data::Dumper::Dumper (\@allexts);

	require IDX::PKI::Crypto::X509;
	my $ca = scalar (parse IDX::PKI::Crypto::X509
					 ((IDX::Test::certificates ())[0]));
	my $caentry = IDX::PKI::LDAP::CertDB::cert2entry($ca);
	TEST ($caentry->get_value("idxpkiSubject") eq
		  IDX::Test::getCertDN ($ca)) ||
		  warn "Cacert subject ".$caentry->get_value("idxpkiSubject");

	# But those in Test have a different set:
	my %exts = map {($_ => 1)} $caentry->get_value("idxpkiExtension");
	TEST ($exts{"keyCertSign"} && $exts{"cRLSign"}) ||
		warn "Cacert ext ".
		Data::Dumper::Dumper ([$caentry->get_value("idxpkiExtension")]);
};

test "New IDX::PKI::LDAP::CertDB, plain LDAP" => sub {
    $ldap = new IDX::PKI::LDAP::CertDB('LDAP-CLIENT-plainldap-fullaccess');
};

test "Adding IDXPKI certs" => sub {
	foreach my $entry (@entries) {
		$ldap->add($entry);
	};
	1;
};

test "searchStrongs" => sub {
    my @entries = $ldap->searchStrongs();
	TEST (@entries == 4) || warn "Didn't find 4 entries: ".scalar (@entries);
	TEST (UNIVERSAL::isa ($entries[0], "Net::LDAP::Entry")) ||
		warn "First entry is not a Net::LDAP::Entry object: ".
		ref ($entries[0]);

	my $entry = $ldap->searchStrongs();
	TEST (UNIVERSAL::isa ($entry, "Net::LDAP::Entry")) ||
		warn "Entry in scalar context is not a Net::LDAP::Entry object: ".
		ref ($entry);
};


test "entry2cert" => sub {
    my @certs = $ldap->searchStrongs();
	TEST(UNIVERSAL::isa(entry2cert($certs[0]),"IDX::PKI::Crypto::X509"));
};

test "searchStrongs by serial" => sub {
	my $cert = parse IDX::PKI::Crypto::X509 ($serializedX509_2);
	my $serial = $cert->getSerial ();
    my @certs = map {entry2cert($_)} $ldap->searchStrongs (serial => $serial);
	TEST (@certs == 1) || warn "Didn't find 1 matching cert for serial = ".
		$serial;
	TEST (hex ($certs[0]->getSerial()) == hex ($serial));
};

test "searchStrongs by email" => sub {
    my @certs = $ldap->searchStrongs(email=>'Waza@idealx.com');
	TEST (@certs == 1);
	TEST(hex (entry2cert($certs[0])->getSerial()) == 3);

 	@certs = $ldap->searchStrongs(email=>'Waza@*');
	TEST (@certs == 1) || die @certs;
	TEST(hex (entry2cert($certs[0])->getSerial()) == 3);
};

test "searchStrongs by DN" => sub {
    my @certs = $ldap->searchStrongs(DN=>'Email=Waza@idealx.com,CN=C0000042,OU=Computer Ressources,O=Test Company,C=FR');
	TEST(@certs == 1) || die @certs;
	TEST(hex (entry2cert($certs[0])->getSerial()) == 3);

    @certs = $ldap->searchStrongs(DN=>'Email=Waza@idealx.com*C=FR');
	TEST(@certs == 1) || die @certs;
	TEST(hex (entry2cert($certs[0])->getSerial()) == 3);
};


test "searchStrongs by CN" => sub {
    my @certs = $ldap->searchStrongs(CN=>'C*42');
	TEST(@certs == 1) || die @certs;
	TEST(hex (entry2cert($certs[0])->getSerial()) == 3);
};

test "searchStrongs by extension" => sub {
    my @certs = $ldap->searchStrongs(extension=>'keyEncipherment');
	TEST(@certs == 4) || die "keyEncipherment:",@certs;

    @certs = $ldap->searchStrongs(extension=>'!keyEncipherment');
	TEST(@certs == 0) || die "!keyEncipherment:",@certs;
};

test "searchStrongs now!" => sub {
	$ldap->searchStrongs(validAt=>'now');
	TESTOK();
};

test "negated search" => sub {
    my @certs = $ldap->searchStrongs("!email"=>'Waza@idealx.com');
	TEST (@certs == 3);
	TEST(! grep {hex (entry2cert($_)->getSerial()) == 3} @certs);
};

test "searchStrongs by validity date" => sub {
    my @certs = $ldap->searchStrongs(validAt=>"20010710125057Z");
	TEST(@certs==1) ||
	  die @certs;
	TEST(hex (entry2cert($certs[0])->getSerial()) == 2);

 	@certs=$ldap->searchStrongs(validAt=>"20020710125058Z");
 	TEST(@certs==3) ||
 	  die @certs;
 	TEST(! grep {hex (entry2cert($_)->getSerial()) == 2} @certs);
};

test "Multiple criteria" => sub {
	my @certs = $ldap->searchStrongs("!validAt"=>"20010710125057Z",
										 "CN"=>"C0345124");
	TEST(@certs==1) ||
	  die @certs;
	TEST(hex (entry2cert($certs[0])->getSerial()) == 5);
};

test "add a CA certificate" => sub {
	require IDX::PKI::Crypto::X509;
	my $ca = scalar (parse IDX::PKI::Crypto::X509
					 ((IDX::Test::certificates ())[0]));
	my $entry = IDX::PKI::LDAP::CertDB::cert2entry ($ca);
	$entry->dn ("CN=".$ca->getSerial().",OU=bogusdb,".
				$config->getval ("dn", "idxpki_ldapbase"));
	$ldap->add ($entry);

    my @certs = $ldap->searchStrongs();
	TEST (@certs == 5);
};

test "searching by existence of a field" => sub {
	require IDX::PKI::Crypto::X509;
	my $cert = parse IDX::PKI::Crypto::X509(<<'ONEMORECERT');
-----BEGIN CERTIFICATE-----
MIIDUDCCArmgAwIBAgIBczANBgkqhkiG9w0BAQQFADCBgzELMAkGA1UEBhMCRlIx
DjAMBgNVBAcTBVBhcmlzMRUwEwYDVQQKEwxUZXN0IElEWC1QS0kxGjAYBgNVBAsT
EUlEWC1QS0kgSW50ZXJuYWxzMRUwEwYDVQQDEwxUZXN0IFJvb3QgQ0ExGjAYBgkq
hkiG9w0BCQEWC2NhQHRlc3QubmV0MB4XDTAyMDYyNTE1MTMyOVoXDTIyMDYyMDE1
MTMyOVowRTELMAkGA1UEBhMCQVUxEzARBgNVBAgTClNvbWUtU3RhdGUxITAfBgNV
BAoTGEludGVybmV0IFdpZGdpdHMgUHR5IEx0ZDCBnzANBgkqhkiG9w0BAQEFAAOB
jQAwgYkCgYEAlnAiLChR++AcJejwcOlZVNaaaRlljj3wiwnjQxXiazR8HOWNkSNR
NPCHDZMz5NTHx5vGLQ6HtS36H/yEwsYyznQt6YMcnscUihlOIT881Adc1C3OgiWT
VIiKphsbWO7ON3fYgrDBiJd+kd4pkP2i+Lil1ji+ElDzcJJryOeBA7kCAwEAAaOC
AQ8wggELMAkGA1UdEwQCMAAwLAYJYIZIAYb4QgENBB8WHU9wZW5TU0wgR2VuZXJh
dGVkIENlcnRpZmljYXRlMB0GA1UdDgQWBBTd+hyWzOf2K03cmyr1JtKemH+kpjCB
sAYDVR0jBIGoMIGlgBRrk6LF4/u4Rse6qKA5YFv12MuT6KGBiaSBhjCBgzELMAkG
A1UEBhMCRlIxDjAMBgNVBAcTBVBhcmlzMRUwEwYDVQQKEwxUZXN0IElEWC1QS0kx
GjAYBgNVBAsTEUlEWC1QS0kgSW50ZXJuYWxzMRUwEwYDVQQDEwxUZXN0IFJvb3Qg
Q0ExGjAYBgkqhkiG9w0BCQEWC2NhQHRlc3QubmV0ggEAMA0GCSqGSIb3DQEBBAUA
A4GBAECEmXuS2Im+szmapgWopGxmoxx6wIEDj8cwZn+BzhQ+Tcadu2wXwlK/ptG4
+8vZrT98EmX0gWGukVlH6wHEkoVS1B4PDSRkjpPuimOKuZFRfpZFyGAdqz9NY9Qh
PxzWU6F+EsBEbNGtUllep26L6oEdWhr6hrgh82I5w2yEnb8+
-----END CERTIFICATE-----
ONEMORECERT
	my $entry = IDX::PKI::LDAP::CertDB::cert2entry ($cert);
	$entry->dn ("CN=".$cert->getSerial().",OU=bogusdb,".
				$config->getval ("dn", "idxpki_ldapbase"));

	TEST(! grep {m/email/i} $entry->attributes()) ||
		warn "Found email: ".Data::Dumper::Dumper ([$entry->attributes()]);
	$ldap->add($entry);

    my @certs = $ldap->searchStrongs(email=>undef);
	TEST (@certs == 4) || warn "Certs with email: ".
		Data::Dumper::Dumper([map {$_->get_value("idxpkiSubject")} @certs]);

    @certs = $ldap->searchStrongs("!email"=>undef);
	TEST (@certs == 2) || warn "Cert without email: ".
		Data::Dumper::Dumper([map {$_->get_value("idxpkiSubject")} @certs]);
	@certs = grep {$_->get_value("idxpkiSubject") !~ m/operational/i} @certs;
	TEST (@certs == 1) || warn "Non-CA cert without email: ".
		Data::Dumper::Dumper([map {$_->get_value("idxpkiSubject")} @certs]);
	TEST (hex (entry2cert($certs[0])->getSerial()) == 0x73) ||
		warn "Wrong serial: ".hex (entry2cert($certs[0])->getSerial());
};

test "correct interaction between client-side filtering and -sizelimit" => sub {
	my @certs = $ldap->searchStrongs(CN=>"C03*");
	TEST (@certs == 3) || warn "There were ".scalar(@certs)." certs to begin with";
	@certs = $ldap->searchStrongs(CN=>"C03*",-sizelimit=>2);
	TEST (@certs == 2) ||
	  warn "There were ".scalar(@certs)." after filtering";
	# Unfortunately this test may succeed by random.
};

test "searching for nonexistent data (shall not throw)" => sub {
	my $result=$ldap->search(base=>"OU=nowhere,".
							 $config->getval ("dn", "idxpki_ldapbase"),
							 filter=>"(objectClass=*)");
	TEST($result->count()==0) || warn Data::Dumper::Dumper($result);
};

# This ensures that the server is DESTROY()ed before the temporary files
# (otherwise it won't be killed).
$server = undef;
1;
