#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::PKI::Log> - Wrapper to call the right true logging mechanism

=head1 SYNOPSIS

 use IDX::PKI::Log;

 my $log = new IDX::PKI::Log ();

 # This will be called automatically by IDX::Registry:
 $log->setSource("somescript.cgi");

 # You may enrich this description from your own script:
 $log->setSource(-role=>"RA",-name=>"first RA");

 $log->addAuthorizer($x509);

 $log->addTracking('12345@myworkflow');

 # Logging debug information:
 $log->debug("Info:", -details => [ $what, $ever ]);
 $log->debug("Big data", "-attachment" => \%datastructure);

 $log->debug("done the work", -attachment => [
   $object,
   [ -data => $der_cert, -mime => "application/x-x509-cert"],
   [ $der_cert, -mime => "application/x-x509-cert", -encoding => "8bit" ],
   [ ]
])

 # Logging alerts:
 $log->alert("No more disk space");

 # Logging an action, for success or failure: watch the subject string,
 # it has to be neutral w.r.t. the outcome (e.g. same message for success or
 # failure)
 $log->success("doing the hard work");
 $log->success({-subject => "doing the hard work",
                -details => ["wanna get the details?"]});
 $log->failure("doing the hard work", -details => "Too bad, better luck next time?");

 # More robust form:
 $log->attempt ("doing the hard work", sub {
 	...
 });

 $log->attempt ([ "doing the hard work", -details => ["my details"]], sub {
 		my $toto;
 		...

		return {-details => ["hello"], -attachment => [$toto]};
 	});

 $log->attempt ({ -subject =>  "doing the hard work",
                 -details => ["my details"] } , sub {
 		my $toto;
 		...

		return {-details => ["hello"], -attachment => [$toto]};
 	});

=cut "

package IDX::PKI::Log;
use strict;

use IDX::PKI::Error;
use IDX::PKI::Config;

use Data::Dumper;

our %stdsubjects =
	(
	 'CERT_ACCEPT'        => 'Certificate accept',
	 'CERT_REVOKE'        => 'Certificate revoke',
	 'CERT_SIGN'          => 'Certificate sign',
	 'CONFIG'             => 'Change(s) in configuration',
	 'CONFIG_PROFILE'     => 'Change(s) in Profiles configuration',
	 'CONFIG_RENEWAL'     => 'Change(s) in Renewal configuration',
	 'CRL_ISSUE'          => 'CRL issue',
	 'CRR_ACCEPT'         => 'CRR accept',
	 'CRR_CREATE'         => 'CRR creation',
	 'CRR_REJECT'         => 'CRR reject',
	 'CSR'                => 'CSR content',
	 'CSR_CREATE'         => 'CSR Create',
	 'CSR_REJECT'         => 'CSR reject',
	 'CSR_SENT'           => 'CSR sent',
	 'CSR_SIGN'           => 'CSR sign',
	 'CSR_VALIDATE'       => 'CSR validate',
	 'ERROR'              => 'Error',
	 'ESCROW_P12'         => 'PKCS#12 recover',
	 'KRR_REJECT'         => 'Key recovery request reject',
	 'KRR_ACCEPT'         => 'Key recovery request accept',
	 'KRR_SENT'           => 'Key recovery request sent',
	 'TRANSPORT_LIST'     => 'Message List Construction',
	 'TRANSPORT_MSG'      => 'Message Retrieval',
	 'MESSAGE_RECEIVE'    => 'Message receive',
	 'P12_RECEIVE'        => 'PKCS#12 receive',
	 'P12_RETRIEVE'       => 'PKCS#12 retrieve',
	 'P12_SENT'           => 'PKCS#12 sent',
	 'PUBLISH_CA_INSERT'  => 'Root CA certificate publish',
	 'PUBLISH_CA_UPDATE'  => 'Root CA certificate update',
	 'PUBLISH_CERT'       => 'Certificate publish in the repository LDAP',
	 'PUBLISH_CRL_INSERT' => 'CRL insert in publishing repository',
	 'PUBLISH_CRL_UPDATE' => 'CRL update in publishing repository',
	 "REPOSITORY_CONNECT" => "LDAP connection",
	 );

=pod "

=head1 METHODS

=over 4

=item I<new ([$section])>

=cut "

sub new {
	my ($class, $section) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS") if (ref $section);

	# Just in case we might want to use different a specific log mechanism
	# for a given entity.
	$section = ($section ? $section : "log");
	my $config = the IDX::PKI::Config ();
	my $type = $config->getPath ($section, "type") || "syslog";

	if ($type eq "syslog") {
		require IDX::PKI::Log::Syslog;
		return new IDX::PKI::Log::Syslog ($section);
	} elsif ($type eq "mailbox") {
		require IDX::PKI::Log::Mailbox;
		return new IDX::PKI::Log::Mailbox ($section);
	} elsif ($type eq "mail") {
		require IDX::PKI::Log::Mail;
		return new IDX::PKI::Log::Mail ($section);
	} elsif ($type eq "xml") {
		require IDX::PKI::Log::Xml;
		return new IDX::PKI::Log::Xml ($section);
	} else {
		throw IDX::PKI::Error::Internal ("Unknown log type '$type'");
	}
}

=pod "

=item I<setSource ()>
=item I<addAuthorizer ($cert_or_obj, [$cert_or_obj, ...])>
=item I<addTracking ($string, [$string, ...])>
=item I<clearTracking ()>

These functions are only stubs in the main implementation. They may do
things in subclasses.

=cut "

sub setSource {}
sub addAuthorizer {}
sub addTracking {}
sub clearTracking {}

=pod "

=item I<encode ($message)>

Encodes the message for logging.

=cut "

sub encode {
	my ($msg) = @_;

	if (! defined ($msg)) {
		return "";
	} elsif (UNIVERSAL::can($msg, 'serialize')) {
		return $msg->serialize();
 	} elsif (ref($msg) eq 'ARRAY') {
		my @tmp = map {encode ($_)} @{$msg};
		return wantarray ? @tmp : join ("\n", @tmp);
 	} elsif (! ref ($msg)) {	# Plain string.
		return $msg;
 	} else {
		local $Data::Dumper::Indent = 1;
		local $Data::Dumper::Terse = 1;
		try {
			return Data::Dumper::Dumper ($msg);
		} otherwise {
			return "$msg";
		};
	}
}

=pod "

=item I<attempt($args, $sub)>

This method executes the subroutine specified by I<$sub>: If this
subroutine throws an exception, a failure is logged then the exception is
propagated, otherwise a success is logged.

If needed, the subroutine can return an anonymous hash with the following
fields:

=over 2

=item *

I<-details>, details concerning the log message (a single string or an
anonymous array of strings);

=item *

I<-attachment>, misc data (a single object or an anonymous array of
objects).

=back

Of course, the return value of the subroutine can be used only in case of
success. In case of success, the details and/or data returned are added to
the details and/or data specified in the attempt call.

I<$args> is, in its simpliest form, a string representig the subject of
the message or an anonymous hash with the following fields:

=over 2

=item *

I<-subject>, the subject of the log message (cf the source of
B<IDX::PKI::Log> for a list of predefined subjects);

=item *

I<-details>, details concerning the log message (a single string or an
anonymous array of strings);

=item *

I<-attachment>, misc data (a single object or an anonymous array of
objects).

=back

In both forms, the subject is mandatory.

=cut "

sub attempt {
	my ($self, $args, $sub) = @_;
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS")
		unless ($args && $sub);
	throw IDX::PKI::Error::Internal("INVALID_ARGS")
		unless (ref ($sub) eq "CODE");
	my %args;

	if (ref($args) eq 'ARRAY') {
		if ((scalar @{$args}) % 2) {
			my $subject = shift (@{$args});
			%args = (@$args);
			$args{"-subject"} = $subject if ($subject);
		}
	} elsif (ref($args) eq 'HASH') {
		%args =  %{$args};
	} else {
		%args = ("-subject" => $args);
	}

	$args{'-details'} = [$args{'-details'}]
		if (ref($args{'-details'}) ne 'ARRAY');
	$args{'-attachment'} = [$args{'-attachment'}]
		if (ref($args{'-attachment'}) ne 'ARRAY');

	try {
		my $ret = &$sub;
		if (ref($ret) eq 'HASH') {
			if (ref ($ret->{'-details'}) eq 'ARRAY') {
				push (@{$args{'-details'}}, @{$ret->{'-details'}});
			}
			if (ref ($ret->{'-attachment'}) eq 'ARRAY')  {
				push (@{$args{'-attachment'}}, @{$ret->{'-attachment'}});
			}
		}
		$self->success(%args);
	} otherwise {
		my ($E) = @_;
		push (@{$args{'-attachment'}}, $E);
		push (@{$args{'-details'}}, $@) if ($@);
		$self->failure(%args);
		$E->throw ();
	};
}

=pod "

=item I<success ($subject, %args)> or I<success ("-subject" => $subject,
%args)>

Logs a successful action. The subject may be passed either as the first
argument, or in a hash manner with the C<-subject> key.

=cut "

sub success {
	my ($self, @args) = @_;
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS")
	  unless (scalar @args);

	if ((scalar @args) % 2) {
		my $subject = shift (@args);
		push (@args, "-subject" => $subject) if ($subject);
	}

	$self->doLog ('action', (@args, "-status" => 'success'));
}

=pod "

=item I<failure (@arge)>

Logs an unsuccessful action. Takes the same arguments as I<success()>.

=cut "

sub failure {
	my ($self, @args) = @_;
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS")
		unless (scalar @args);

	if ((scalar @args) % 2) {
		my $subject = shift (@args);
		push (@args, "-subject" => $subject) if ($subject);
	}

	$self->doLog ('action', (@args, "-status" => 'failure'));
}

=pod "

=item I<debug (@arge)>

Logs a pure informational message, mainly aimed at debugging. Takes the
same arguments as I<success()>.

=cut "

sub debug {
	my ($self, @args) = @_;
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS")
		unless (scalar @args);

	if ((scalar @args) % 2) {
		my $subject = shift (@args);
		push (@args, "-subject" => $subject) if ($subject);
	}

	$self->doLog ('debug', @args);
}

=pod "

=item I<alert (@arge)>

Logs a critical problem. Takes the same arguments as I<success()>.

=cut "

sub alert {
	my ($self, @args) = @_;
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS")
		unless (scalar @args);

	if ((scalar @args) % 2) {
		my $subject = shift (@args);
		push (@args, "-subject" => $subject) if ($subject);
	}
	$self->doLog ('alert', @args);
}

eval join("", <main::DATA>) || die $@ unless caller();

1;
__END__

########## Test Suite ##########

package IDX::PKI::Log;

use IDX::PKI::Config;
use IDX::Test;
#use IDX::PKI::Error;
#use IO::File;

use strict;

my $testdir = new IDX::Test::Directory();

my $configtxt1 = set IDX::PKI::Config (<< "EOF");
[log]
type=syslog
facility=user
logopt=pid
EOF


my @subjects = ("it's gonna crash", "help!!!",
                "a subject with <><> inside", "a subject with many ''''''''''''");

number_of_tests (4);	# Public IDX-PKI version.

my $log;
my $config = set IDX::PKI::Config ($configtxt1);
test "New logging objects" => sub {
	$log = new IDX::PKI::Log();
};

test "Logging (effectiveness of logging not checked)" => sub {
	$log->debug("Problem there youpi");
	$log->failure("A worse problem.");
	$log->success("hi", -details => "good morning") ||
		warn "Can't do info with simple details";
	$log->success("much details", -details => ["hello", "world"]) ||
		warn "Can't do info with much details";
	$log->debug("a very good logger", -attachment => $log);
};

test "attempt() propagates and loggs errors" => sub {
	our @logcontents;
	local *IDX::PKI::Log::Syslog::doLog = sub {
		my ($self, $lvl, @args) = @_;
		@logcontents = @args;
	};
	try {
		$log->attempt ({
			"-subject" => "A sub that throws",
		}, sub {
			throw IDX::PKI::Error::Internal ("This is an error");
		});
		warn "Should have thrown";
		TESTFAIL ();
	} catch IDX::PKI::Error::Internal with {
		my ($E) = @_;
		TEST ($E->{"-text"} eq "This is an error") ||
			warn "Unexpected error: ".$E->humanize ();
		my %args = @logcontents;
		TEST ($args{"-status"} eq "failure") ||
			warn "Wrong status: $args{'status'}";
		TEST ($args{"-subject"} eq "A sub that throws") ||
			warn "Wrong subject: $args{'subject'}";
		TEST (ref ($args{"-attachment"}) eq "ARRAY") ||
			warn "Attachment is not an array: ".
			Data::Dumper::Dumper ($args{"-attachment"});
		my @attach = grep {defined ($_)} @{$args{"-attachment"}};
		TEST (scalar (@attach) == 1) || warn "Wrong attachments: ".
			Data::Dumper::Dumper (\@attach);
		TEST (UNIVERSAL::isa ($attach[0], "IDX::PKI::Error::Internal") &&
			  ($attach[0]->{"-text"} eq "This is an error")) ||
			warn "Wrong log error: ".Data::Dumper::Dumper ($attach[0]);
	};
};

test "Closing logs" => sub {
	$log->end();
};

