#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: PKCS12Distribution.pm,v 1.7 2003/08/19 12:48:24 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Message/PKCS12Distribution.pm,v $

package IDX::PKI::Message::PKCS12Distribution;

use strict;

$IDX::PKI::Message::PKCS12Distribution::VERSION = '0.0.1';

=pod "

=head1 NAME

IDX::PKI::Message::PKCS12Distribution - Messages aimed at distribuing a PKCS#12 file

=head1 SYNOPSIS

 use IDX::PKI::Message::PKCS12Distribution;
 my $message = build IDX::PKI::Message::PKCS12Distribution
 	('-data' => $p12, '-x509' => $x509);
 $message->serialize ();

=head1 DESCRIPTION

Class for messages aimed ad distribuing a PKCS#12 file.  We need to keep a
clear copy of the certificate because we may not be able to "open" the
PKCS#12 file: If it is password-encrypted, it will be no more to us than
mere raw data...

=cut "

use IDX::PKI::Error;
use IDX::PKI::Debug (0);

# For object deserialisations...
use IDX::PKI::Crypto::X509;
use MIME::Base64;

=pod "

=head1 METHODS

=over 4

=item I<< build ('-data' => $p12, '-x509' => $x509[, "-pass" => $password]) >>

Constructs the object from logical components.  I<$p12> is the PKCS#12
contents; it is just considered raw data, because it may be encrypted.
The C<-x509> argument is the certificate which corresponds to the PKCS#12.

If I<$p12> is a B<IDX::PKI::Crypto::PKCS12> object, it will be
I<serialize()>d with the specified I<$password>, otherwise I<$password> is
unused.

For convenience of the user, if the I<$x509> argument is a string,
appropriate magic is performed (loading files, parsing string
representations).  Otherwise it must be must be an
B<IDX::PKI::Crypto::X509> object.

=cut "

sub build {
	my ($class, %args) = @_;

	my $self = {};

	# PKCS#12.
	$self->{'p12'} = delete ($args{'-data'}) ||
		throw IDX::PKI::Error::Internal ('INCORRECT_ARGS',
			'-details' => "Missing or empty -data argument");
	if (UNIVERSAL::isa ($self->{"p12"}, "IDX::PKI::Crypto::PKCS12")) {
		my $pass = $args{'-pass'};
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
										 "-details" => "Missing password")
			unless ($pass);
		$self->{"p12"} = $self->{"p12"}->serialize ($pass);
		$self->{"pass"} = crypt ($pass, 'IDX-PKI');
	} else {
		$self->{"pass"} = $args{'-pass'};
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
										 "-details" => "Missing password")
			unless ($self->{"pass"});
	}
	# X509 certificate object.
	my $x509 = delete ($args{'-x509'}) ||
		throw IDX::PKI::Error::Internal ('INCORRECT_ARGS',
			'-details' => "Missing or empty -x509 argument");
	if (ref ($x509)) {
		throw IDX::PKI::Error::Internal ("The -x509 argument is a ref but ".
			"not an IDX::PKI::Crypto::X509 object.", '-ref' => ref ($x509))
			unless (UNIVERSAL::isa ($x509, "IDX::PKI::Crypto::X509"));
		$self->{'x509'} = $x509;
	} else {
		if ($x509 =~ m/BEGIN.*CERT/) {
			$self->{'x509'} = parse IDX::PKI::Crypto::X509 ($x509);
		} elsif (-f $x509) {
			$self->{'x509'} = load IDX::PKI::Crypto::X509 ($x509);
		} else {
			throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
				'-details' => "Unrecognised -x509 argument",
				'-x509' => $x509);
		}
	}

	DEBUG ($self);

	return bless ($self, $class);
}


=pod "

=item I<parse ($data, [$format])>

From data.  The only recognized format (for now) is C<serialized> (default
if I<$format> is not provided).

=cut "

sub parse {
	my ($class, $data, $format) = @_;

	throw IDX::PKI::Error::Internal ("No data supplied.") unless ($data);

	$format ||= 'serialized';

	DEBUG ("Message:\n".("<"x50)."\n$data".(">"x50)."\n");

	if ($format eq 'serialized') {
		$data =~ m/
			\-\-\-\-\-BEGIN\ DISTRIBUTION\ MESSAGE\-\-\-\-\-\n
			(.+)\n
			\-\-\-\-\-BEGIN\ DATA\-\-\-\-\-\n
			(.+)\n
			\-\-\-\-\-END\ DATA\-\-\-\-\-\n
			(.+)\n
			\-\-\-\-\-END\ DISTRIBUTION\ MESSAGE\-\-\-\-\-
			/sx ||
			throw IDX::PKI::Error::Internal ("Can't parse message.",
				'-message' => $data);

		my ($pass, $data, $cert) = ($1, $2, $3);
		DEBUG ("Data: '$data', cert: '$cert'\n");

		# Call build to avoid code duplication.
		return build ($class, '-data' => decode_base64 ($data),
					  '-x509' => $cert, '-pass' => $pass);
	} else {
		throw IDX::PKI::Error::Internal ("Unknown format '$format'.");
	}
}


=pod "

=item I<serialize ()>

Returns the object as a string, in a form suitable for transport.

=cut "

sub serialize {
	my ($self) = @_;

	my $data = encode_base64 ($self->getP12 ());
	my $cert = $self->getX509 ()->serialize ();
	my $pass = $self->{pass};

	return << "SERIALIZED";
-----BEGIN DISTRIBUTION MESSAGE-----
$pass
-----BEGIN DATA-----
$data
-----END DATA-----
$cert
-----END DISTRIBUTION MESSAGE-----
SERIALIZED
}

=pod "

=item I<deserialize ($data)>

I<Idem> as I<parse()>, with a fixed type of C<serialized>.

=cut "

sub deserialize {
	my ($class, $data) = @_;

	return parse ($class, $data, 'serialized');
}

=pod "

=item I<getP12 ()>

Returns the PKCS#12 contents as raw data.

=cut "

sub getP12 {
	my ($self) = @_;

	return $self->{'p12'};
}


=pod "

=item I<getX509 ()>

Returns the associated X509 object.

=cut "

sub getX509 {
	my ($self) = @_;

	return $self->{'x509'};
}

=pod "

=item I<testPassword ()>

Test if the password is correct

=cut "

sub testPassword {
	my ($self, $pass) = @_;

	return ($self->{"pass"} eq crypt($pass, 'IDX-PKI')) ? 1 : undef;
}

=pod "

=back

=cut "

eval (join '', <main::DATA>) || die $@ unless (caller ());
1;

__END__

####################  TEST SUITE  ####################

package IDX::PKI::Message::PKCS12Distribution;
use strict;
use IDX::Test;
use IDX::PKI::Crypto::X509;

# All tests are integration tests, because they want to know where the
# openssl binary is.
use IDX::PKI::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}

number_of_tests (7);

my (undef, $cert, $key) = IDX::Test::certificates ();
my $x509 = parse IDX::PKI::Crypto::X509 ($cert);
my $data = "This should be a PKCS#12 string.\nAnyway this is just raw data.";
my $msg;

test "Constructor with an X509 object." => sub {
	$msg = build IDX::PKI::Message::PKCS12Distribution ('-data' => $data,
		'-x509' => $x509, '-pass' => "123");
};

test "Serialisation of getX509() gives \$cert." => sub {
	TEST ($msg->getX509 ()->serialize () eq $cert);
};

test "Constructor with X509 data." => sub {
	$msg = build IDX::PKI::Message::PKCS12Distribution ('-data' => $data,
		'-x509' => $cert, '-pass' => "123");
};

test "Constructor with a PKCS#12 object." => sub {
	require IDX::PKI::Crypto::PKCS12;
	require IDX::PKI::Crypto::X509;
	require IDX::PKI::Crypto::PrivateKey;
	my $p12 = build IDX::PKI::Crypto::PKCS12
		(parse IDX::PKI::Crypto::X509 ($cert),
		 parse IDX::PKI::Crypto::PrivateKey ($key));
	my $pass = "somepassword";
	try {
		my $msg = build
			IDX::PKI::Message::PKCS12Distribution ("-data" => $p12,
												   "-x509" => $cert);
		warn "Should have thrown.";
		TESTFAIL;
	} catch IDX::PKI::Error with {
		TESTOK;
	};
	TEST (build IDX::PKI::Message::PKCS12Distribution ("-data" => $p12,
													   "-x509" => $cert,
													   "-pass" => $pass));
};

test "Result of getX509() isa() IDX::PKI::Crypto::X509." => sub {
	TEST (UNIVERSAL::isa ($msg->getX509 (), "IDX::PKI::Crypto::X509"));
};

my $serialised;
test "Serialize." => sub {
	$serialised = $msg->serialize ();
};

test "Deserialize (parse) gives the same P12 and X509." => sub {
	my $msg2 = deserialize ('IDX::PKI::Message::PKCS12Distribution',
							$serialised);
	TEST ($msg->getP12 () eq $msg2->getP12 ()) ||
		warn "Orig: '".$msg->getP12 ()."'; new: '".$msg2->getP12 ()."'\n";
	TEST (UNIVERSAL::isa ($msg2->getX509, "IDX::PKI::Crypto::X509")) ||
		warn "New object class is '".ref ($msg2->getObject ())."'\n";
	TEST ($msg->getX509 ()->serialize () eq
		  $msg2->getX509 ()-> serialize ()) ||
		warn "Orig: '".$msg->getX509 ()->serialize ().
		"'; new: '".$msg2->getX509 ()->serialize ()."'\n";
};

1;
