#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: Publish.pm,v 1.25 2003/06/17 14:35:50 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Message/Publish.pm,v $

package IDX::PKI::Message::Publish;

use strict;

$IDX::PKI::Message::Publish::VERSION = '0.0.1';

=pod "

=head1 NAME

IDX::PKI::Message::Publish - Messages to and from the publication module

=head1 SYNOPSIS

 use IDX::PKI::Message::Publish;
 my $message = build IDX::PKI::Message::Publish (
 	'-type' => "certificate", '-object' => $x509,
 	'-profile' => $profile, '-form' => $hashref);
 $message->serialize ();

=head1 DESCRIPTION

Class for messages sent to the publication module.

=cut "

use IDX::PKI::Error;
use IDX::PKI::Debug (0);

# For object deserialisations...
use IDX::PKI::Crypto::X509;
use IDX::PKI::Crypto::CRL;

=pod "

=head1 METHODS

=over 4

=item I<< build ('-object' => $object, '-type' => $type, ['-profile' =>
$name, ['-form' => $hashref]]) >>

Constructs the object from logical components.  The C<-object> must be an
object containing the data we want to publish, and must accept the
I<serialize()> and I<deserialize()> methods.  Recognised C<-type> argument
are:

=over 4

=item C<certificate>

If we want to publish an B<IDX::PKI::Crypto::X509> object.

=item C<crl>

If we want to publish an B<IDX::PKI::Crypto::CRL> object.

=item C<ca-certificate>

If we want to publish an B<IDX::PKI::Crypto::X509> object belonging
to a CA.

=back

I<$profile> is the name of the profile under which the certificate has
been created.  I<$hashref> is an anonymous hash containing keys and values
issued from the request which has led to this publication; keys are
profile form elements.

=cut "

sub build {
	my ($class, %args) = @_;

	my $self = {};

	# Main data we want to publish.
	$self->{'object'} = delete ($args{'-object'}) ||
		throw IDX::PKI::Error::Internal ('INCORRECT_ARGS',
			'-details' => "Missing or empty -object argument");
	# Type of the publication.
	$self->{'type'} = delete ($args{'-type'}) ||
		throw IDX::PKI::Error::Internal ('INCORRECT_ARGS',
			'-details' => "Missing or empty -type argument");
	# Profile and request informations.
	$self->{'profile'} = delete ($args{'-profile'});
	$self->{'form'} = delete ($args{'-form'}) || {};
	throw IDX::PKI::Error::Internal ('INCORRECT_ARGS',
		'-details' => "-form argument is not an anonymous hash")
		unless (ref ($self->{'form'}) eq "HASH");
	throw IDX::PKI::Error::Internal ('INCORRECT_ARGS',
		'-details' => "Form elements supplied without a profile")
		if (scalar (%{$self->{'form'}}) && !$self->{'profile'});

	# Sanity checks.
	foreach my $method (qw (serialize deserialize)) {
		throw IDX::PKI::Error::Internal ("The given ".
			ref($self->{'object'})." object can't $method()")
			unless (UNIVERSAL::can ($self->{'object'}, $method));
	}

	DEBUG ($self);

	return bless ($self, $class);
}


=pod "

=item I<parse ($data, [$format])>

From data.  The only recognized format (for now) is C<serialized> (default
if I<$format> is not provided).

=cut "

sub parse {
	my ($class, $data, $format) = @_;

	throw IDX::PKI::Error::Internal ("No data supplied.") unless ($data);

	$format ||= 'serialized';

	DEBUG ("Message:\n".("<"x50)."\n$data".(">"x50)."\n");

	if ($format eq 'serialized') {
		$data =~ m/
			\-\-\-\-\-BEGIN\ PUBLISH\ MESSAGE\-\-\-\-\-\n
			type:\ ([^\n]+)\n
			\-\-\-\-\-BEGIN\ DATA\-\-\-\-\-\n
			class:\ ([^\n]+)\n
			(.+)\n
			\-\-\-\-\-END\ DATA\-\-\-\-\-\n
			profile:\ ([^\n]*)\n
			(.*)
			\-\-\-\-\-END\ PUBLISH\ MESSAGE\-\-\-\-\-
			/sx ||
			throw IDX::PKI::Error::Internal ("Can't parse message.",
				'-message' => $data);

		my ($type, $objclass, $objdata, $profile, $form) =
			($1, $2, $3, $4, $5);
		DEBUG ("Type: '$type', object class: '$objclass', ".
			"object data: '$objdata', profile: '$profile'\n");

		# Support for more than one value in a field.
		my %form = ();
		foreach my $line (split (/\n+/, $form)) {
			next if ($line !~ /\S/);
			throw IDX::PKI::Error::Internal ("Malformed message",
											 '-message' => $data)
				unless ($line =~ /^\s*([^:]+): (.*)$/);
			my ($key, $val) = ($1, $2);
			if (exists $form{$key} && ref ($form{$key}) eq "ARRAY") {
				push (@{$form{$key}}, $val);
			} elsif (exists $form{$key}) {
				$form{$key} = [$form{$key}, $val];
			} else {
				$form{$key} = $val;
			}
		}

		# Deserialise the object.
		my $deserialize = UNIVERSAL::can (bless ({}, $objclass),
			'deserialize') || throw IDX::PKI::Error::Internal (
				"The $objclass data class can't deserialize()");
		my $object = &$deserialize ($objclass, $objdata);

		# Call build to avoid code duplication.
		return build ($class, '-object' => $object, '-type' => $type,
			'-profile' => $profile, '-form' => \%form);
	} else {
		throw IDX::PKI::Error::Internal ("Unknown format '$format'.");
	}
}


=pod "

=item I<serialize ()>

Returns the object as a string, in a form suitable for transport.

=cut "

sub serialize {
	my ($self) = @_;

	my $type    = $self->getType ();
	my $object  = $self->getObject ();
	my $class   = ref ($object);
	my $data    = $object->serialize ();
	my $profile = $self->getProfile () || "";
	my $form    = "";
	my %form    = $self->getForm ();
	foreach my $field (keys %form) {
		# Support for multi-valued fields.
		if (ref ($form{$field}) eq "ARRAY") {
			foreach my $elem (@{$form{$field}}) {
				$form .= "$field: $elem\n";
			}
		} else {
			$form .= "$field: $form{$field}\n";
		}
	}

	return << "SERIALIZED";
-----BEGIN PUBLISH MESSAGE-----
type: $type
-----BEGIN DATA-----
class: $class
$data
-----END DATA-----
profile: $profile
$form
-----END PUBLISH MESSAGE-----
SERIALIZED
}


=pod "

=item I<deserialize ($data)>

I<Idem> as I<parse()>, with a fixed type of C<serialized>.

=cut "

sub deserialize {
	my ($class, $data) = @_;

	return parse ($class, $data, 'serialized');
}

=pod "

=item I<getType ()>

=cut "

sub getType {
	my ($self) = @_;

	return $self->{'type'};
}


=pod "

=item I<getObject ()>

=cut "

sub getObject {
	my ($self) = @_;

	return $self->{'object'};
}


=pod "

=item I<getProfile ()>

=cut "

sub getProfile {
	my ($self) = @_;

	return $self->{'profile'};
}


=pod "

=item I<getForm ()>

=cut "

sub getForm {
	my ($self) = @_;

	return (wantarray ? %{$self->{'form'}} : $self->{'form'});
}


=pod "

=back

=cut "

eval (join '', <main::DATA>) || die $@ unless (caller ());
1;

__END__

####################  TEST SUITE  ####################

package Bogus::Package;
sub new {
	return bless ({}, 'Bogus::Package');
}
sub serialize {
	return "This is serialised data.";
}
sub deserialize {
	return bless ({}, 'Bogus::Package');
}

package IDX::PKI::Message::Publish;
use strict;
use IDX::Test;
use Data::Dumper;

number_of_tests (10);

my $cert = bless ({}, "Bogus::Package");
my $crl  = bless ({}, "Bogus::Package");

my ($certmsg, $crlmsg, $s_certmsg);

test "Constructor, certificate publication." => sub {
	$certmsg = build IDX::PKI::Message::Publish ('-object' => $cert,
		'-type' => "certificate", '-profile' => 'myprofile',
		'-form' => {'blah' => 'bleh', 'blih' => ['bloh', 'bluh'],
				'empty' => "\t"});
};

test "getType(), certificate publication." => sub {
	TEST ($certmsg->getType () eq 'certificate');
};

test "Constructor, CRL publication." => sub {
	$crlmsg = build IDX::PKI::Message::Publish ('-object' => $crl,
		'-type' => "crl");
};

test "getType(), CRL publication." => sub {
	TEST ($crlmsg->getType () eq 'crl');
};

test "getObject()" => sub {
	TEST (UNIVERSAL::isa ($certmsg->getObject, 'Bogus::Package'));
};

test "getProfile()" => sub {
	TEST ($certmsg->getProfile () eq 'myprofile');
};

test "getForm()" => sub {
	my %hash = $certmsg->getForm ();
	TEST ($hash{'blah'} eq 'bleh') || warn "blah => $hash{'blah'}";
	TEST (ref ($hash{'blih'}) eq "ARRAY" && $hash{'blih'}[0] eq 'bloh') ||
		warn "blih => ".Data::Dumper::Dumper ($hash{'blih'});
	TEST ($hash{'empty'} eq "\t") || warn "empty => $hash{'empty'}";
};

test "Serialize." => sub {
	$s_certmsg = $certmsg->serialize ();
};

test "Deserialize (parse)." => sub {
	my $certmsg2 = deserialize ('IDX::PKI::Message::Publish', $s_certmsg);
	TEST ($certmsg->getType () eq $certmsg2->getType ()) ||
		warn "Orig: '".$certmsg->getType ()."'; new: '".
			$certmsg2->getType ()."'\n";
	TEST (UNIVERSAL::isa ($certmsg2->getObject, 'Bogus::Package')) ||
		warn "New object class is '".ref ($certmsg2->getObject ())."'\n";
	# Empty or undef is the same thing.
	TEST ((! $certmsg->getProfile () && ! $certmsg2->getProfile ()) ||
		($certmsg->getProfile () eq $certmsg2->getProfile ())) ||
		warn "Orig: '".$certmsg->getProfile ()."'; new: '".
			$certmsg2->getProfile ()."'\n";
	my %hash = $certmsg2->getForm ();
	TEST ($hash{'blah'} eq 'bleh') || warn "blah => $hash{'blah'}";
	TEST (ref ($hash{'blih'}) eq "ARRAY" && $hash{'blih'}[0] eq 'bloh') ||
		warn "blih => ".Data::Dumper::Dumper ($hash{'blih'});
};

test "Deserialize for CRL (no profile)." => sub {
	my $s_crlmsg = $crlmsg->serialize ();
	my $crlmsg2 = deserialize ('IDX::PKI::Message::Publish', $s_crlmsg);
	TEST ($crlmsg->getType () eq $crlmsg2->getType ()) ||
		warn "Orig: '".$crlmsg->getType ()."'; new: '".
			$crlmsg2->getType ()."'\n";
	TEST (UNIVERSAL::isa ($crlmsg2->getObject, 'Bogus::Package')) ||
		warn "New object class is '".ref ($crlmsg2->getObject ())."'\n";
	# Empty or undef is the same thing.
	TEST ((! $crlmsg->getProfile () && ! $crlmsg2->getProfile ()) ||
		($crlmsg->getProfile () eq $crlmsg2->getProfile ())) ||
		warn "Orig: '".$crlmsg->getProfile ()."'; new: '".
			$crlmsg2->getProfile ()."'\n";
};

1;
