#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::PKI::Message::Simple> - Generic message class

=head1 SYNOPSIS

 use IDX::PKI::Message::Simple;
 my $message = build IDX::PKI::Message::Simple (
 	"-objects" => $x509,
 	"-data" => ["pkcs12" => $encrypted_p12,
 		"profile" => $profilename],
 );
 my $string = $message->serialize ();

=head1 DESCRIPTION

This class builds generic messages, and knows how to serialise /
deserialise them. You probably won't want to use it directly, because it
has no knowledge of the meaning of its contents, but rather use a
descendant class.

=cut "

package IDX::PKI::Message::Simple;

use strict;

use IDX::PKI::Error;
use MIME::Base64 ();

=pod "

=head1 METHODS

=over 4

=item I<< build ("-objects" => [$object, [...]], "-data" => ["type" =>
$blob, [...]]) >>

Builds a new B<IDX::PKI::Message::Simple> object from its contents.

C<-objects> may be passed as a single object rather than an anonymous
array of objects, if there is only one. Such objects must have a
I<serialize()> and a I<deserialize()> (or I<parse()>) methods.

C<-data> is an array of key => values pairs (it's an array rather than a
hash because there may be multiple keys). Keys are here for your
convenience, but they have no particular meaning for
B<IDX::PKI::Message::Simple>, and values must be plain scalars, such as
strings or numbers.

=cut "

sub build ($%) {
	my ($class, %args) = @_;

	my $self = {};
	bless ($self, $class);

	my (@objects, @data);

	# Objects which can take care of themseleves.
	if ($args{"-objects"}) {
		if ((ref ($args{"-objects"}) eq "ARRAY") &&
			(! UNIVERSAL::can ($args{"-objects"}, "serialize"))) {
			# Seems to be an array of objects.
			foreach my $object (@{$args{"-objects"}}) {
				push (@objects, $self->_check_object ($object));
			}
		} else {
			# Single object.
			@objects = ($self->_check_object ($args{"-objects"}));
		}
	}

	# Raw data.
	if ($args{"-data"}) {
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
			unless (ref ($args{"-data"}) eq "ARRAY" ||
					ref ($args{"-data"}) eq "HASH");
		my @array = (ref ($args{"-data"}) eq "ARRAY") ? @{$args{"-data"}} :
			%{$args{"-data"}};
		while (my ($key, $val) = splice (@array, 0, 2)) {
			throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
											 "-key" => $key,
											 "-value" => $val)
				if (ref ($key) || ref ($val));
			push (@data, $key => $val);
		}
	}

	$self->{"objects"} = \@objects;
	$self->{"data"} = \@data;

	return $self;
}

=pod "

=item I<parse ($string)>

Same as I<deserialize()>.

=cut "

sub parse {
	goto &deserialize;
}

=pod "

=item I<load ($file)>

Same as I<parse()> or I<deserialize()>, except that it takes the message
contents from the file I<$file> rather than from a string.

=cut "

sub load {
	my ($class, $file) = @_;
	local *FILE;
	open (FILE, "<", $file) ||
		throw IDX::PKI::Error::IO ("TRANSPORT_MSG", "-IOfile" => $file);
	my $contents = join ("", <FILE>);
	close FILE;
	return $class->parse ($contents);
}

=pod "

=item I<_check_object ($object)>

This internal method checks that I<$object> is correct (mainly, that it
knows how to I<serialize()> and I<deserialize()> (or I<parse()>)). If it
is, returns it, otherwise throws an B<IDX::PKI::Error::Internal> error.

=cut "

sub _check_object ($$) {
	my ($self, $object) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
									 "-details" => "The object can't ".
									 "serialize().",
									 "-object" => $object)
		unless (UNIVERSAL::can ($object, "serialize"));

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
									 "-details" => "The object can neither ".
									 "deserialize() nor parse().",
									 "-object" => $object)
		unless (UNIVERSAL::can ($object, "deserialize") ||
				UNIVERSAL::can ($object, "parse"));

	return $object;
}

=pod "

=item I<getObjects ([$class])>

Returns the message object(s) belonging to the given I<$class>. If
I<$class> is not supplied, returns all objects. In scalar context, only
returns the first matching object.

=cut "

sub getObjects ($;$) {
	my ($self, $wantedclass) = @_;
	my @objects;
	if ($wantedclass) {
		foreach my $obj (@{$self->{"objects"}}) {
			push (@objects, $obj) if (UNIVERSAL::isa ($obj, $wantedclass));
		}
	} else {
		@objects = @{$self->{"objects"}};
	}

	return wantarray ? @objects : $objects[0];
}

=pod "

=item I<getData ([$type])>

Returns the message data elements corresponding to the given I<$type>. If
I<$type> is not supplied, returns all objects, as an array of key => value
pairs. In scalar context, returns the first corresponding value.

=cut "

sub getData ($;$) {
	my ($self, $type) = @_;
	my @data;
	if ($type) {
		my @list = @{$self->{"data"}};
		while (my ($key, $val) = splice (@list, 0, 2)) {
			push (@data, $key => $val) if ($key eq $type);
		}
	} else {
		@data = @{$self->{"data"}};
	}

	# Warning, $data[0] is the type, $data[1] the value.
	return wantarray ? @data : $data[1];
}

=pod "

=item I<serialize ()>

Returns the message in a form suitable for transport, i.e. as a string.
The result is not very compact, but is made to be robust.

=cut "

sub serialize ($) {
	my ($self) = @_;

	my $string = "-----BEGIN ".ref ($self)." MESSAGE-----\n";

	foreach my $obj (@{$self->{"objects"}}) {
		$string .= "-----BEGIN ".ref ($obj)." OBJECT-----\n";
		$string .= $obj->serialize ()."\n";
		$string .= "-----END ".ref ($obj)." OBJECT-----\n";
	}

	my @data = @{$self->{"data"}};
	while (my ($key, $val) = splice (@data, 0, 2)) {
		my $b64 = MIME::Base64::encode_base64 ($val);
		$string .= "-----BEGIN $key DATA-----\n$b64\n".
			"-----END $key DATA-----\n";
	}

	$string .= "-----END ".ref ($self)." MESSAGE-----\n";

	return $string;
}

=pod "

=item I<deserialize ($string)>

Parses a I<serialize()>d B<IDX::PKI::Message::Simple> string and returns
the corresponding object.

=cut "

sub deserialize ($$) {
	my (undef, $string) = @_;

	my $class;

	my @lines = split (/[\r\n]+/, $string);
	if ($lines[0] =~ m/^-----BEGIN ([^ ]+) MESSAGE-----$/) {
		$class = $1;
		shift (@lines);
	} else {
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
										 "-details" => "No global BEGIN ".
										 "marker.",
										 "-line" => $lines[0],
										 "-string" => $string);
	}

	my (@objects, @data);
	CHUNK: while (defined (my $line = shift @lines)) {
		next CHUNK if ($line eq "");
		last CHUNK if ($line eq "-----END $class MESSAGE-----");

		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
										 "-details" => "No BEGIN marker.",
										 "-line" => $line,
										 "-string" => $string)
			unless ($line =~ m/^-----BEGIN (.+?) ([A-Z]+)-----$/);

		# Type is either OBJECT or DATA, and $key is the OBJECT class or
		# the DATA identifier.
		my ($key, $type) = ($1, $2);
		my $contents = "";

		my $end_seen;
		while (@lines && ! $end_seen) {
			my $chunk = shift (@lines);
			if ($chunk eq "-----END $key $type-----") {
				$end_seen = 1;
			} else {
				$contents .= $chunk."\n";
			}
		}
		chomp ($contents);

		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
										 "-details" => "No END marker.",
										 "-class" => $key, "-type" => $type,
										 "-string" => $string)
			unless ($end_seen);

		if ($type eq "OBJECT") {
			my @chunks = split (/::/, $key);
			while (@chunks) {
				last if (eval "use ".join ("::", @chunks)."; 1;");
				pop (@chunks);
			}
			throw IDX::PKI::Error::Install ("TRANSPORT_MSG", "-details" =>
											"Couldn't find a package to ".
											"load", "-type" => $key)
				unless (@chunks);
			my $obj;
			try {
				$obj = $key->deserialize ($contents);
			} otherwise {
				$obj = $key->parse ($contents);
			};
			push (@objects, $obj);
		} elsif ($type eq "DATA") {
			my $data = MIME::Base64::decode_base64 ($contents);
			push (@data, $key => $data);
		} else {
			throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
											 "-details" => "Unrecognised ".
											 "type.",
											 "-class" => $key,
											 "-type" => $type,
											 "-string" => $string);
		}
	}

	return $class->build ("-objects" => \@objects, "-data" => \@data);
}

=pod "

=back

=cut "

1;
require IDX::Test::Suite unless caller();

__END__

####################  TEST CASE  ####################

use IDX::PKI::Crypto::X509;
use IDX::PKI::Crypto::PrivateKey;
my (undef, $certstring, $keystring) = IDX::Test::certificates ();
my $x509   = parse IDX::PKI::Crypto::X509 ($certstring);
my $key    = parse IDX::PKI::Crypto::PrivateKey ($keystring);
my $blob   = "This is raw data.\nIt even contains newlines.";
my $string = "42";

my $msg;
test "New object" => sub {
	$msg = build IDX::PKI::Message::Simple
		("-objects" => [$x509, $key],
		 "-data" => ["blob" => $blob, "string" => $string]);
};

test "New object, with hash instead of array" => sub {
	$msg = build IDX::PKI::Message::Simple
		("-objects" => [$x509, $key],
		 "-data" => {"blob" => $blob, "string" => $string});
};

test "getObjects()" => sub {
	my $obj = $msg->getObjects ("IDX::PKI::Crypto::X509");
	TEST (UNIVERSAL::isa ($obj, "IDX::PKI::Crypto::X509")) ||
		warn "X509 is not an X509 object:\n".Data::Dumper::Dumper ($obj);
	$obj = $msg->getObjects ("IDX::PKI::Crypto::PrivateKey");
	TEST (UNIVERSAL::isa ($obj, "IDX::PKI::Crypto::PrivateKey")) ||
		warn "PrivateKey is not a PrivateKey object:\n".
		Data::Dumper::Dumper ($obj);
	my @objs = $msg->getObjects ();
	TEST (scalar (@objs) == 2) || warn "Did not get 2 objects back.\n".
		Data::Dumper::Dumper (\@objs);
	TEST (UNIVERSAL::isa ($objs[0], "IDX::PKI::Crypto::X509")) ||
		warn "1st obj is not an X509:\n".Data::Dumper::Dumper ($objs[0]);
	TEST (UNIVERSAL::isa ($objs[1], "IDX::PKI::Crypto::PrivateKey")) ||
		warn "2nd obj is not a PrivateKey:\n".Data::Dumper::Dumper ($objs[1]);
};

test "getData()" => sub {
	my $val = $msg->getData ("blob");
	TEST ($val eq $blob) || warn "Did not get \$blob back: $val";
	$val = $msg->getData ("string");
	TEST ($val eq $string) || warn "Did not get \$string back: $val";
	my @data = $msg->getData ();
	TEST (scalar (@data) == 4) || warn "Did not get 4 elements.\n".
		Data::Dumper::Dumper (\@data);
	my %data = @data;
	TEST ($data{"blob"} eq $blob && $data{"string"} eq $string) ||
		warn "Unexpected data:\n".Data::Dumper::Dumper (\@data);
};

my $serialised;
test "serialize()" => sub {
	$serialised = $msg->serialize ();
	TEST ($serialised =~ m/^-----BEGIN IDX::PKI::Message::Simple\b/m &&
		  $serialised =~ m/^-----BEGIN IDX::PKI::Crypto::X509\b/m &&
		  $serialised =~ m/^-----BEGIN IDX::PKI::Crypto::PrivateKey\b/m &&
		  $serialised =~ m/^-----BEGIN blob\b/m &&
		  $serialised =~ m/^-----BEGIN string\b/m);
	warn $serialised if (TESTFAILED);
};

test "deserialize()" => sub {
	$msg = deserialize IDX::PKI::Message::Simple ($serialised);
};

1;
