#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::PKI::Principal.pm> - Various utilities shared by some IDX-PKI CGI scripts

=head1 SYNOPSIS

 use IDX::PKI::Principal;

=cut "

package IDX::PKI::Principal;

use strict;

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Crypto::CSR;
use IDX::PKI::Crypto::CRR;
use IDX::PKI::Crypto::SignedBlob;
use IDX::PKI::LDAP;
use IDX::PKI::Crypto::PKCS7 qw(pkcs7Encrypt);
use IDX::PKI::Log;
use IDX::PKI::Debug(0);
use IDX::PKI::DN; # For access control to profiles
#use IO::Pipe; # For launching sendmail

use Exporter;
use vars qw(@ISA @EXPORT @EXPORT_OK);
@ISA = qw (Exporter);

@EXPORT = qw ( readCSR pullCSR readCRR pullCRR pullPKCS12
			   readSignedBlob pullSignedBlob printStatus zulu_datenow );

@EXPORT_OK = qw (
				 set_profile_parameter transmit_profile
				 sendmail sendmailAttached sendMimeMail
				 );

###### THIS IS A COMPATIBILITY MEASURE, TO BE REMOVED ANY TIME ######

use IDX::PKI::ValidateInput qw(%wf_functions);
push(@EXPORT,qw(%wf_functions));

use vars qw/%wf_functions/;


=pod "

=head1 FUNCTIONS

=over 4

=item I<readCSR($transport, $id)>

Reads a given CSR from the transport layer.  It is just a wrapper.

=cut "

sub readCSR {
	my ($transport, $id) = @_;

	return $transport->readByID
	  ($id, 'IDX::PKI::Crypto::CSR' =>
	   sub { deserialize IDX::PKI::Crypto::CSR($_[0]); });
}


=pod "

=item I<pullCSR($transport, $id)>

Pulls a given CSR from the transport layer.  It is just a wrapper.

=cut "

sub pullCSR {
	my ($transport, $id) = @_;

	return $transport->pullByID
	  ($id, 'IDX::PKI::Crypto::CSR' =>
	   sub { deserialize IDX::PKI::Crypto::CSR($_[0]); });
}

=pod "

=item I<readCRR($transport, $id)>

Reads a given CRR from the transport layer.  It is just a wrapper.

=cut "

sub readCRR {
	my ($transport, $id) = @_;

	return $transport->readByID ($id, 'IDX::PKI::Crypto::CRR' => sub {
		new IDX::PKI::Crypto::CRR(-data => $_[0]);
	});
}


=pod "

=item I<pullCRR($transport, $id)>

Pulls a given CRR from the transport layer.  It is just a wrapper.

=cut "

sub pullCRR {
	my ($transport, $id) = @_;

	return $transport->pullByID ($id, 'IDX::PKI::Crypto::CRR' => sub {
		new IDX::PKI::Crypto::CRR(-data => $_[0]);
	});
}


=pod "

=item I<readSignedBlob($transport, $id)>

Reads a given signed blob from the transport layer.  It is just a wrapper.

=cut "

sub readSignedBlob {
	my ($transport, $id) = @_;

	return $transport->readByID ($id, 'IDX::PKI::Crypto::SignedBlob' => sub {
		new IDX::PKI::Crypto::SignedBlob (-data => $_[0]);
	});
}


=pod "

=item I<pullSignedBlob($transport, $id)>

Pulls a given signed blob from the transport layer.  It is just a wrapper.

=cut "

sub pullSignedBlob {
	my ($transport, $id) = @_;

	return $transport->pullByID ($id, 'IDX::PKI::Crypto::SignedBlob' => sub {
		new IDX::PKI::Crypto::SignedBlob (-data => $_[0]);
	});
}


=pod "

=item I<expired($expiration_date)>

Takes a date with format C<YYYYMMDDHHMMSSZ> and returns 0 if it is greater
than the current date, 1 otherwise.

Warning, it always assumes GMT timezone.

=cut "

sub expired {
	my ($expire) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", -arg => $expire)
		unless ($expire =~ /^(\d{12}\d*)\w?$/);
	$expire = $1.("0"x(14-length($1)));

	my @date = reverse(gmtime());
	@date = splice(@date, 3, 5);
	$date[0] += 1900;
	$date[1] += 1;
	foreach my $item (@date) {
		if (length($item) == 1) {
			$item = "0$item";
		}
	}
	my $date_str = join('', @date);

	return ($expire gt $date_str) ? 0 : 1;
}

=pod "

=item I<printStatus($ldap, $serial)>

Returns the status of the certificate I<$cerial> in the LDAP server
I<$ldap>.  It just calls I<$ldap->getStatus()> and tests the result.

=cut "

sub printStatus {
	my ($ldap, $serial) = @_;
	my $result = $ldap->getStatus(-serial=>$serial);

	if ($result == &VALID_PUBLISHED || $result == &VALID_WAITING) {
		return "valid";
	} elsif ($result == &REVOKED_PUBLISHED || $result == &REVOKED_WAITING) {
		return "revoked";
	} elsif ($result == &EXPIRED) {
		return "expired";
	} else {
		return "unknown";
	}
}


=pod "

=item I<zulu_datenow ()>

Returns the current date in "zulu" format (C<YYYYMMDDhhmmssZ>).

=cut "

sub zulu_datenow {
	my ($sec, $min, $hour, $day, $mon, $year) = gmtime (time());
	return sprintf("%04d%02d%02d%02d%02d%02dZ",
				   $year + 1900, $mon + 1, $day, $hour, $min, $sec);
}


=pod "

=item I<transmit_profile ($cgi, $profile, \@rdns, \@forms)>

Sets a I<bigparam()> (see L<IDX::PKI::Registry>) named C<IDXPKI_PROFILE>,
which contains the profile informations. I<$profile> may be either an
B<IDX::PKI::Profile> object or a profile name string.

The C<IDXPKI_PROFILE> structure is a hash which elements are:

=over 4

=item I<profile>

The profile name;

=item I<title>

The hash of internationalised profile titles (see
L<IDX::PKI::Profile/getTitlesI18N()>);

=item I<type>

The profile type (see L<IDX::PKI::Profile/getType()>);

=item I<rdns>

The profile (to be shown) RDNs (see L<IDX::PKI::Profile/getDNElements()>),
as given in the I<@rdns> parameter;

=item I<forms>

The profile (to be shown) form elements (see
L<IDX::PKI::Profile/getFormElements()>), as given in the I<@forms>
parameter.

=back

=cut "

sub transmit_profile {
	my ($cgi, $profile, $rdns, $forms) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (defined ($profile) &&
				UNIVERSAL::isa ($cgi, "IDX::PKI::Registry::CGIobject") &&
				(ref ($rdns) eq "ARRAY") && (ref ($forms) eq "ARRAY"));

	$profile = new IDX::PKI::Profile ($profile) unless (ref ($profile));

	my %ret = ('profile' => $profile->getName (),
			   'title'   => {$profile->getTitlesI18N ()},
			   'type'    => $profile->getType (),
			   'rdns'    => $rdns,
			   'form'    => $forms);

	$cgi->bigparam ("IDXPKI_PROFILE", \%ret);
	return;
}


=pod "

=item I<set_profile_parameter ($cgi, $profile, $element, $param, $value)>

Factorisation function. Sets the I<$param> parameter to I<$value>, unless
option I<verify> is set for I<$element> (I<$profile> RDN ou form element),
in which case both I<$param."a"> and I<$param."b"> are set.

=cut "

sub set_profile_parameter {
	my ($cgi, $profile, $elem, $param, $value) = @_;
	return unless (defined $value);

	if ($profile->elementOptions ($elem, "verify")) {
		$cgi->param ($param."a", $value);
		$cgi->param ($param."b", $value);
	} else {
		$cgi->param ($param, $value);
	}
}


=pod "

=item I<sendmail($from, $to, $subject, $message)>

Sends an email to $to appearing to come $from with $subject as subject
and $message as body.

=cut "

sub sendmail {
	my ($from, $to, $subject, $message) = @_;

	_do_sendmail(<<"EOF");
From: $from
To: $to
Subject: $subject
MIME-Version: 1.0
Content-Type: text/plain; charset=iso-8859-1
Content-Transfer-Encoding: 8bit

$message
EOF
}

=pod "

=item I<sendmailAttached($from, $to, $subject, $message, $type,
$filename, $attachment)>

Sends an email to $to appearing to come $from with $subject as subject and
$message as body.  Also the message will have an attachment with type
$type, with content $attachment, and name $filename.

=cut "

sub sendmailAttached {
	my ($from, $to, $subject, $message, @files) = @_;
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
	  unless((scalar(@files) % 3) == 0);

	my $mail = <<"EOF";
From: $from
To: $to
Subject: $subject
MIME-Version: 1.0
Content-Type: multipart/mixed; boundary="=-ZgyFp7wCmeHi7zZ0R7VLTHISCANTHAPPEN"
Content-Transfer-Encoding: 8bit

--=-ZgyFp7wCmeHi7zZ0R7VLTHISCANTHAPPEN
Content-Type: text/plain; charset=iso-8859-1

$message

EOF
	my ($type, $filename, $attachment);
	while (($type, $filename, $attachment) = splice (@files, 0, 3)) {
         $mail .= "--=-ZgyFp7wCmeHi7zZ0R7VLTHISCANTHAPPEN\n".
          "Content-Transfer-Encoding: base64\n".
      (($filename) ? "Content-Disposition: attachment; ".
               "filename=\"$filename\"\n" : '').
      "Content-Type: ".(($type) ?$type : "application/octet-stream")."\n".
      "\n".MIME::Base64::encode($attachment)."\n";
	 }

	$mail .= "--=-ZgyFp7wCmeHi7zZ0R7VLTHISCANTHAPPEN--\n";

	_do_sendmail($mail);
}

=pod "

=item I<_do_sendmail($text)>

This is not a method, but a plain function. Actually launches
I<sendmail>, or whatever method is in use on the local system for
sending e-mail. $text must be an email in the usual RFC823 format
(headers, blank line, body).

=cut "

sub _do_sendmail {
	my ($text)=@_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS",
									 '-details' => "Incorrect mail headers")
		unless ($text && $text =~ m/^To: /m && $text =~ m/^Subject: /m);

	# XXX The IO::Pipe version DOES NOT WORK under Apache, although it
	# works when the scripts are called on the command-line (and thus the
	# unit tests work).  It may be a side effect of the 'PerlTaintCheck
	# On' option, which appears to be more strict than the -T perl switch.
	# The visible effect of that "non working" is an email which is
	# effectively sent, but without any content.  No other error is
	# visible.  I really don't know how to test this for non-regression...

#	my $pipe=new IO::Pipe;
#	throw IDX::PKI::Error::Memory ("Cannot pipe()") unless $pipe;
#
#	throw IDX::PKI::Error::IO ("Cannot send mail.")
#		unless ($pipe->writer("/usr/sbin/sendmail","-t"));
#
#	local $SIG{PIPE}="IGNORE";
#	throw IDX::PKI::Error::IO ("Cannot communicate with sendmail.")
#		unless ($pipe->print($text) && $pipe->close());

	# Untaint the relevant variables. Yes we are ashamed and we ought
	# to fix our code instead.
	local $ENV{PATH}="";
	local $ENV{BASH_ENV}="";
	local *PIPE;
	open (PIPE, "|-", "/usr/sbin/sendmail -t") ||
		throw IDX::PKI::Error::IO ("Cannot send mail.");
	print PIPE $text;
	close PIPE;
	1;
}

=pod "

=item I<sendMimeMail (-from => $from, -to => $to, -subject => $subject,
-message => $message, -attachment => $attachment, -mimetype =>
$mimetype, -base64 => $base64, -filename => $filename, -crypto => $ca,
-cert => $cert)>

Sends a S/MIME encrypted email to $to appearing to come $from with
$subject as subject and $message as body.  Also the message will have an
attachment with type $type, with content $attachment, and name
$filename.

This function does not belong here at all; a little bit of redesign is needed.

=cut "

sub sendMimeMail {
	my %args = @_;

	my $from        = $args{-from};
	my $to          = $args{-to};
	my $subject     = $args{-subject};
	my $message     = $args{-message};
	my $attachment = $args{-attachment};
	my $mimetype    = $args{-mimetype};
	my $base64      = $args{-base64};
	my $filename    = $args{-filename};
	my $cert        = $args{-cert};
	my $mail;

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS")
	  unless ((defined $from) and (defined $to) and
			  (defined $subject) and (defined $message));

	if (defined $base64) {
		throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		  unless(defined $attachment);
		$attachment = MIME::Base64::encode($attachment);
	}

	if (defined $attachment) {
		$message = 'Content-Type: multipart/mixed; boundary="=-ZgyFp7wCmeHi7zZ0R7VLTHISCANTHAPPEN"
Content-Transfer-Encoding: 8bit

--=-ZgyFp7wCmeHi7zZ0R7VLTHISCANTHAPPEN
Content-Type: text/plain; charset=iso-8859-1

'.$message.'

--=-ZgyFp7wCmeHi7zZ0R7VLTHISCANTHAPPEN
'.((defined $base64) ? 'Content-Transfer-Encoding: base64'."\n" : '').
  ((defined $filename) ? 'Content-Disposition: attachment; filename="'.$filename.'"'."\n" : '').
	'Content-Type: '.((defined $mimetype) ? $mimetype : "application/octet-stream" ).'

'.$attachment.'

--=-ZgyFp7wCmeHi7zZ0R7VLTHISCANTHAPPEN--
';
	} else {
		$message = "Content-Type: text/plain; charset=iso-8859-1\n\n".$message;
	}

	if ($cert) {
		my $pkcs7 = build IDX::PKI::Crypto::PKCS7::EnvelopedData
		  ($message, $cert);
		my $encrypted = $pkcs7->serialize("SMIME");

		$mail = 'From: '.$from.'
To: '.$to.'
Subject: '.$subject."\n".$encrypted;
	} else {
		$mail = 'From: '.$from.'
To: '.$to.'
Subject: '.$subject.'
MIME-Version: 1.0
Content-Transfer-Encoding: 8bit
'.$message;
	}

	  _do_sendmail($mail);
}



=pod "

=head2 Profile Related Functions

=over 4

=item I<ParseProfiledResponse ()>

=cut "

sub ParseProfiledResponse {
	my ($cgi, $profile, $forms, $values, $entity) = @_;
	my %counts = ();
	$forms->{profile} = $profile->getName ();
	foreach my $form ($profile-> getFormElements()) {
		my $isEE = ($entity =~ m/^ee$/i);
		next if ($profile->elementOptions($form, "raonly")
						 and (not $isEE));
		my $long  = $profile->elementLongName ($form);
		my $short = $profile->elementShortName ($form);
		my $type  = $profile->elementType ($form);
		my $checked;
		$counts{$short} ++;

		unless ($profile->elementOptions ($form, "edit")
				and (not $isEE)) {
			# In this case we read the value from the CGI
			my $value = $cgi->param ($long.$counts{$short});

			unless($value) {
				next unless ($profile->elementImportance($form) eq "mandatory");
				# Mandatory value not supplied.
				$cgi->internalparam ("ERROR", $long);
				$cgi->return (INPUT_MISSING);
			}

			# If the user must type the value twice for confirmation.
			if ($profile->elementOptions ($form, "verify")) {
				my $value_a = $cgi->param ($long.$counts{$short}."a");
				my $value_b = $cgi->param ($long.$counts{$short}."b");
				if ($value_a eq $value_b) {
					# The two values match.
					$value = $value_a;
				} else {
					# The two values don't match.
					$cgi->internalparam ("ERROR", $long);
					$cgi->return (INPUT_VERIFY);
				}
			}

			# Now validate the value
			if ($type eq "choice") {
				$checked = $wf_functions{'choice'}->
					($value, $profile->elementChoices ($form));
			} elsif (exists $wf_functions{$type}) {
				$checked = $wf_functions{$type}->
					($value, $profile->elementMaximumLength($form));
			} else {
				throw IDX::PKI::Error::Internal
					("Unknown validation type '$type'");
			}
		} else {
			# Transmit the  existing value as-is (already verified).
			$checked = $values->{$long.$counts{$short}};
		}

		unless ($checked) {	# Invalid value.
			$cgi->internalparam ("ERROR", $long);
			$cgi->return (INPUT_FORMAT);
		}

		my $entity = $profile->elementOptions($form, "encrypt");
		if ($entity and not $values->{$long.$counts{$short}}) {
			my $config = the IDX::PKI::Config ();
			$checked = pkcs7Encrypt
				($checked, $config->getPath ($entity, "certificate"));
		}

		# Push values into form
		if (exists ($forms->{$short}) && ref ($forms->{$short}) eq "ARRAY") {
			push (@{$forms->{$short}}, $checked);
		} elsif (exists $forms->{$short}) {
			$forms->{$short} = [$forms->{$short}, $checked];
		} else {
			$forms->{$short} = $checked;
		}
	}
}

######################## TEST CASE ###################################

eval join('',<main::DATA>) || die $@ unless caller();

1;

__END__

package IDX::PKI::Principal;
use strict;
use IDX::Test;
use IO::Pipe;

number_of_tests(4);


test "expired: yes" => sub {
	my $date = "000001010000";
	TEST (expired ($date));
};


test "expired: no" => sub {
	my $date = "999912312359";
	TEST (!expired ($date));
};


my $config = the IDX::PKI::Config (<< "EOF");
[syslog]
facility=user
logopt=pid
EOF

sub Bogus::Package::AUTOLOAD {1;}
my $log=bless {}, "Bogus::Package";

test "Someone is about to receive a mail..." => sub {
	my $username = scalar getpwuid($<);
	IDX::PKI::Principal::sendmail($username,
								  $username,
								  "Just a test",<<MESSAGE);
This message just shows that the IDX::PKI::Principal test suite
was successful.
MESSAGE
};

test "Someone is about to receive a mail (with more stuff)..." => sub {
	my $username = scalar getpwuid($<);
	sendmailAttached($username, $username, "Just another test",
					 "This message just shows that the IDX::PKI::Principal \n".
					 "test suite was successful. Look for attachement",
					 "text/plain", "text.txt", "Blah\n",
					 "", "binary.dat", "\x00\x01");
}
