#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: Profile.pm.public,v 1.10 2004/01/27 11:19:52 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/Profile.pm.public,v $

=pod "

=head1 NAME

IDX::PKI::Profile - IDX-PKI profiles gestion module

=head1 SYNOPSIS

 use IDX::PKI::Profile;

 my $profile = new IDX::PKI::Profile ("myprofile");

=head1 DESCRIPTION

This module aims to manage IDX-PKI profiles.

=cut "

package IDX::PKI::Profile;

use strict;

$IDX::PKI::Profile::VERSION = '0.0.1';

use IDX::PKI::Error;
use IDX::PKI::Config;

=pod "

=head1 METHODS

=head2 Public methods

=over 4

=item I<new ($profilename)>

Object constructor. I<$profilename> is the name of the desired profile. If
the profile name is C<undef>, this profile object will have no defined
values but will still be a valid object.

=cut "

sub new	{
	my ($that, $name) = @_;
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (defined $that);
	my $class = ref ($that) || $that;

	my $config = the IDX::PKI::Config ();

	my $self = {
		'config' => $config,
		'name'   => $name,
	};
	bless ($self, $class);
	return $self;
}

=pod "

=item I<getName ()>

Returns the name of the profile.

=cut "

sub getName {
	my ($self) = @_;

	return $self->{'name'};
}


=pod "

=item I<getTitle ($lang)>

Returns the title in the desired language, if it exists.  Otherwise
returns the default title.

=cut "

my %titles =
  (
   "standard" => { "en" => "User Decentralised Mode",
				   "fr" => "Mode dcentralis utilisateur" },
   "central" => { "en" => "User Centralised Mode",
				  "fr" => "Mode centralis utilisateur" },
   "server" => { "en" => "Server Request",
				 "fr" => "Requte serveur" }
  );

sub getTitle {
	my ($self, $lang) = @_;
	$lang ||= "en";

	my $title = $titles{$self->{name}}{lc ($lang)} ||
	  $titles{$self->{name}}{"en"};
	return $title;
}

=pod "

=item I<getTitlesI18N ()>

Returns the title in all known languages, including the default version.

=cut "

sub getTitlesI18N {
	my ($self) = @_;

	return wantarray ? %{$titles{$self->{'name'}}} : $titles{$self->{'name'}};
}


=pod "

=item I<getType ()>

Returns the type of the profile.

=cut "

sub getType {
	my ($self) = @_;

	my %mapping =
	  (
	   "standard" => "user",
	   "central" => "central",
	   "server" => "server"
	  );

	return $mapping{$self->{name}};
}

=pod "

=item I<getKeySize ()>

Returns the size of keys to be generated.
NB: the value may not be relevant to all profiles.

=cut "

sub getKeySize {
	my ($self) = @_;

	return 1024;
}

=pod "

=item I<getValidity ()>

Returns the length of validity period for certificates?

=cut "

sub getValidity {
	my ($self) = @_;

	return 365;
}

=pod "

=item I<getDistribution ()>

Returns the distribution method for the generated PKCS#12 files (default
is C<user>), if this profile uses centralised mode.  Otherwise throws an
B<IDX::PKI:;Error::Internal> exception.

=cut "

sub getDistribution {
	my ($self) = @_;

	throw IDX::PKI::Error::Internal ("getDistribution has no meaning unless ".
									 "in centralised mode",
									 '-type' => $self->getType ())
		unless ($self->getType () eq 'central');

	return "user";
}

=pod "

=item I<getCRLDistPoints ()>

Returns the list of CRL distribution points for certificates generated
in this profile.

=cut "

sub getCRLDistPoints {
	my ($self)=@_;
	return $self->{config}->getPath("CA","crlurl");
}

=pod "

=item I<getDNElements ()>

Returns the DN elements defined in the profile.  Returns them as an array
or an array reference.

=cut "

sub getDNElements {
	my ($self) = @_;
	my @rdns;

	unless($self->{name} eq 'server') {
		my @rdn =
		  (
			[ "CN", "commonName",             "64", "name",    "mandatory"],
            [ "OU", "organizationalUnitName", "64", "name",    "optional" ],
            [ "O",  "organizationName",       "64", "name",    "mandatory"],
		    [ "C",  "countryName",            "2",  "country", "mandatory"],
		  );

		my $element;
		foreach $element(@rdn) {
			push @rdns,	{ short     => @{$element}[0],
						  long      => @{$element}[1],
						  maxlength => @{$element}[2],
						  type      => @{$element}[3],
						  req       => @{$element}[4],
						  options   => {},
						  defaut    => ""
						};
		}
	}
	return wantarray () ? @rdns : \@rdns;
}


=pod "

=item I<getFormElements ()>

Returns the form elements defined in the profile.  Returns them as an
array or an array reference.

=cut "

sub getFormElements {
	my ($self) = @_;

	my @form;

	my $email  = ["emailAddress", "emailAddress", "64", "email", "mandatory"];
	my $shared = ["sharedSecret", "sharedSecret", "64", "password",
				  "mandatory", {"password" => 1}];
	my $passwd = [ "password", "password", "64", "password", "mandatory",
				   {"password" => 1, "encrypt" => "CA", "verify" => 1}];
	my $pkcs10 = ["pkcsten", "pkcsten", "", "pem", "mandatory"];
	my $admin  = ["adminEmail", "emailAddress", "64", "email", "mandatory"];
	my $altnametype = ["altNameType", "altNameType", "", "choice",
					   "mandatory", "", "URI", {}, ["URI", "DNS"]];
	my $altname = ["subjectAltName", "subjectAltName", "64", "uri",
				   "mandatory"];

	if ($self->{'name'} eq "standard") {
		@form =
			(
			 $email,
			 $shared,
			 );
	} elsif ($self->{'name'} eq "central"){
		@form =
			(
			 $email,
			 $passwd,
			 $shared,
		  );
	} elsif ($self->{'name'} eq "server") {
		@form =
			(
			 $pkcs10,
			 $admin,
			 $altnametype,
			 $altname,
			 $shared,
			 );
	}

	my @forms;
	foreach my $element (@form) {
		push @forms, { short     => @{$element}[0],
					   long      => @{$element}[1],
					   maxlength => @{$element}[2],
					   type      => @{$element}[3],
					   req       => @{$element}[4],
					   options   => defined (@{$element}[5]) ?
						   @{$element}[5] : {},
					   defaut    => defined (@{$element}[6]) ?
						   @{$element}[6] : "",
					   choices   => @{$element}[8],
					 };
	}

	return wantarray () ? @forms : \@forms;
}


=pod "

=item I<getMatchingFormElements ($name)>

Returns the form elements defined in the profile if their long name or
short name matches I<$name>.  In scalar context, returns the first (or
only) one.

=cut "

sub getMatchingFormElements {
	my ($self, $name) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (defined $name && $name ne '');

	my @result;

	foreach my $form ($self->getFormElements) {
		push (@result, $form)
			if ($self->elementShortName ($form) eq $name ||
				$self->elementLongName ($form) eq $name);
	}

	return wantarray () ? @result : $result[0];
}


=pod "

=item I<getExtensions ()>

Returns an array of the the extensions defined in the profile.

=cut "

sub getExtensions {
	my ($self) = @_;

	my %mapping =
	  (
	   "standard" => [ "digitalSignature", "keyEncipherment", "clientAuth", "emailProtection" ],
	   "central" =>  [ "digitalSignature", "keyEncipherment", "clientAuth", "emailProtection" ],
	   "server" =>   [ "digitalSignature", "keyEncipherment", "serverAuth" ]
	  );

	return wantarray () ? @{$mapping{$self->{name}}} : $mapping{$self->{name}};
}


=pod "

=item I<getExtensionsAsHash ()>

Returns the extensions defined in the profile.  Returns them as a hash or
a hash reference.

=cut "

sub getExtensionsAsHash {
	my ($self) = @_;

	my %exts;
	foreach my $ext ($self->getExtensions()) {
		$exts{$ext} = 1;
	}

	return wantarray () ? %exts : \%exts;
}


=pod "

=item I<elementShortName ($element)>

Returns the short name of an element (DN element or form element, see
L</getDNElements> and L</getFormElements>).

=cut "

sub elementShortName {
	my ($self, $element) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (ref ($element) eq "HASH");

	return $element->{'short'};
}


=pod "

=item I<elementLongName ($element)>

Returns the long name of an element (DN element or form element).

=cut "

sub elementLongName {
	my ($self, $element) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (ref ($element) eq "HASH");

	return $element->{'long'};
}


=pod "

=item I<elementMaximumLength ($element)>

Returns the maximum length of an element (DN element or form element).

=cut "

sub elementMaximumLength {
	my ($self, $element) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (ref ($element) eq "HASH");

	my $maxlength = $element->{'maxlength'};
	my ($width, $height);
	if ($maxlength !~ /\D/) {	# Only digits.
		($width, $height) = ($maxlength, 1);
	} elsif ($maxlength =~ /^(\d+)x(\d+)$/) {	# nnxmm format.
		($width, $height) = ($1, $2);
	} else {
		throw IDX::PKI::Error::Config ("Unrecognised max length: $maxlength",
			'-profile' => $self->getName ());
	}

	return wantarray () ? ($width, $height) : $maxlength;
}


=pod "

=item I<elementType ($element)>

Returns the type of an element (DN element or form element).

=cut "

sub elementType {
	my ($self, $element) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (ref ($element) eq "HASH");

	return $element->{'type'};
}


=pod "

=item I<elementImportance ($element)>

Returns the importance of an element: optional, mandatory or fixed (DN
element or form element).

=cut "

sub elementImportance {
	my ($self, $element) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (ref ($element) eq "HASH");

	return $element->{'req'};
}


=pod "

=item I<elementDefaultValue ($element)>

Returns the default value of an element (DN element or form element).

=cut "

sub elementDefaultValue {
	my ($self, $element) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (ref ($element) eq "HASH");

	return $element->{'default'};
}


=pod "

=item I<elementOptions ($element, [$option])>

Without further argument, returns the list of defined options for the
element (DN element or form element).  If I<$option> is supplied, returns
the value corresponding to the option: Either an argument to the option,
or 1 if the option is set, 0 otherwise.

=cut "

sub elementOptions {
	my ($self, $element, $option) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (ref ($element) eq "HASH");

	if (defined $option) {
		# Return the desired option.
		return undef unless (ref $element->{'options'} eq "HASH");
		return $element->{'options'}{$option};
	} else {
		# List all set options.
		my %options = (ref $element->{'options'} eq "HASH") ?
			%{$element->{'options'}} : ();
		my @optlist = keys (%options);
		return wantarray () ? @optlist : \@optlist;
	}
}

=pod "

=item I<elementChoices ($element)>

Returns the available choices for an element, if any (DN element or form
element).

=cut "

sub elementChoices {
	my ($self, $element) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (ref ($element) eq "HASH");

	my @choices;
	if ($element->{'choices'}) {
		@choices = @{$element->{'choices'}};
	}

	return wantarray () ? @choices : \@choices;
}

=pod "

=item I<csr2certoptions ($csr)>

Returns a hash of the options which must be put in the certificate, in a
form suitable for I<IDX::PKI::CA::_sign_cert>, including the DN (C<dn> =>
anonymous array of the DN), the certificate validity (C<validity> =>
number of days), X509 extensions (C<extensions> => anonymous hash),
subject alternative names (C<altnames> => anonymous hash) and CRL
distribution points (C<crldistpoints> => anonymous arrays). I<$csr> must
be a B<IDX::PKI::Crypto::CSR> object.

=cut "

sub csr2certoptions {
	my ($self, $csr) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (UNIVERSAL::isa ($csr, "IDX::PKI::Crypto::CSR"));

	# Prepares certificate extensions.
	my %exts;
	require IDX::PKI::Crypto::X509;
	while (my ($type, $exts) = each
		   %IDX::PKI::Crypto::X509::keyUsageExtensions) {
		my @extensions = grep {$csr->getFormEntry($_)} @$exts;
		$exts{$type} = \@extensions if (@extensions);
	}

	# Prepares subject alternative names.
	my %altnames;
	# Forms emails.
	if (defined (my $email = $csr->getFormEntry ("emailAddress"))) {
		my @emails = (ref ($email) eq "ARRAY") ? @$email :
			split (m/[,]\s*/, $email);
		$altnames{"email"} ? (push (@{$altnames{"email"}}, @emails)) :
			($altnames{"email"} = \@emails);
	}
	# RDNs emails.
	if (my @emails = build IDX::PKI::DN ($csr->getDN ())->
		getRDNValue ("Email")) {
		$altnames{"email"} ? (push (@{$altnames{"email"}}, @emails)) :
			($altnames{"email"} = \@emails);
	}
	# SubjectAltName.
	if (defined (my $altname = $csr->getFormEntry ("subjectAltName"))) {
		my $alttype = $csr->getFormEntry ("altNameType");
		if (! $alttype) {
			# We have to guess the subjectAltName type.
			if ($altname =~ m|^[a-z]+://|i) {
				# Looks like an URI.
				$altnames{"URI"} ? (push (@{$altnames{"URI"}}, $altname)) :
					($altnames{"URI"} = [$altname]);
			} elsif ($altname =~ m|@|) {
				# Looks like an email.
				$altnames{"email"} ? (push (@{$altnames{"email"}}, $altname)) :
					($altnames{"email"} = [$altname]);
			} else {
				# Assumed to be a DNS name.
				$altnames{"DNS"} ? (push (@{$altnames{"DNS"}}, $altname)) :
					($altnames{"DNS"} = [$altname]);
			}
		} else {
			# We know the subjectAltName type.
			$altnames{$alttype} ? (push (@{$altnames{$alttype}}, $altname)) :
				($altnames{$alttype} = [$altname]);
		}
	}

	# Filters out empty alt names.
	foreach my $key (keys %altnames) {
		@{$altnames{$key}} = grep {defined ($_) && ($_ ne "")}
			@{$altnames{$key}};
	}

	my $dn = build IDX::PKI::DN ($csr->getDN());
	my %result = (
				  "dn"            => $dn,
				  "validity"      => $self->getValidity (),
				  "extensions"    => \%exts,
				  "altnames"      => \%altnames,
				  "crldistpoints" => [$self->getCRLDistPoints ()],
				  );

	return wantarray ? %result : \%result;
}

=pod "

=back

=cut "

eval (join '',<main::DATA>) || die $@ unless (caller());
1;

__END__

####################  Test suite  ####################

package IDX::PKI::Profile;
use strict;
use IDX::PKI::Config;
use IDX::Test;

number_of_tests (6);

my ($ca, $servcert, $servkey, $usercert, $userkey, $crl) =
	IDX::Test::certificates ();
my $userdn = IDX::Test::getCertDN ($usercert);

my $profile;

test "Object constructor" => sub {
	$profile = new IDX::PKI::Profile ("standard");
};

test "getTitle with a language" => sub {
	TEST ($profile->getTitle ("fr") eq "Mode dcentralis utilisateur");
};

test "getTitle without a language" => sub {
	TEST ($profile->getTitle () eq "User Decentralised Mode");
};

test "getTitle with an inexistant language" => sub {
	TEST ($profile->getTitle ("es") eq $profile->getTitle ());
};

my @extensions = $profile->getExtensions ();
test "getExtensions gets the right count" => sub {
	TEST (scalar (@extensions)) || warn "Don't get any extension";
};

test "getExtensions contains the desired extension" => sub {
	my %extensions = map {return ($_, 1)} @extensions;
	TEST ($extensions{'emailProtection'});
};


1;
