#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

use strict;

=pod "

=head1 NAME

B<IDX::PKI::Registry> - The all-in-one toolchain for IDX-PKI dynamic WWW pages

=head1 SYNOPSIS

This is an example for a search form. Whatever the outcome of the
search, we use searchform.php to display both the results, the errors
(if any) and a form that can be used again to make another search:

 use IDX::PKI::Registry;
 # No need to use IDX::PKI::ValidateInput, IDX::PKI::Registry
 # already does it for your convenience.
 use IDX::PKI::Other::Modules::If::You::Like;
 # But do not 'use IDX::PKI::CGI;' nor 'use CGI;'.

 # No need to predeclare or construct $cgi, $log and $config:
 # they are automatically set up for you.
 $cgi->setupViews("searchform.php");

 $cgi->authorize("PRIV_USE","searchACL");

 my $filter=$cgi->validate("FILTER", qr/^([a-z]+=[a-z0-9]+)$/i);

 $cgi->error(-32) if $cgi->validateFailures();

 # Do some searching with $filter, put results in @results

 $cgi->success(\@results);

This is an example for an update form, which, upon success, does
redirects to the search form above to display the updated data:

 use IDX::PKI::Registry;
 $cgi->setupViews(-success => "searchform.cgi?filter=%s",
 				  -error => "updateform.php");

 $cgi->authorize("PRIV_USE","updateACL");

 my $id = $cgi->validate("id",qr/^([0-9]*)$/);
 my $name = $cgi->validate("NAME",qr/^([a-z ]*)$/i);
 my $details = $cgi->validate("DETAILS",qr/^([a-z ]*)$/i);

 $cgi->error(-32) if ($cgi->validateFailures());

 # Do updates on back-end database here

 $cgi->success("id=$id"); # Performs an external redirect

=head1 DESCRIPTION

This is I<IDX::PKI::Registry>, the main module to invoke from dynamic
WWW pages (also known as CGI scripts but IDX::PKI::Registry is quite
different - read on for more details).

This class supersedes I<IDX::PKI::CGI>. It aims at implementing the following
features, by decreasing order of priority:

=over 4

=item *

MVC like I<IDX::PKI::CGI> did (see below for a presentation of MVC and
the way it is done in IDX-PKI), with an extra 'commit and redirect'
feature. Whenever a page that has side effects is successfully invoked
by the user's browser, the resulting page is not served directly and a
redirect is fed to the browser instead. This way hitting the 'Reload'
button never has any nasty effects;

=item *

suppress boilerplate code from scripts: provide standard I<$cgi>, I<$log>
and I<$config> objects, trap exceptions;

=item *

enforce security: mandatory user authentication with a flexible,
centralized access control policy; parsing of CGI arguments with a
simplified API, featuring Perl tainting;

=item *

support transparent session management and forwarding from page to page
as hidden CGI parameters;

=item *

alleviate dependency over the passing of variables through the Apache
environment (which could be a security risk).

=item *

compatibility with the standalone CGI use-case (so that scripts
using B<IDX::PKI::Registry> are still valid, autonomous CGI programs
that can run outside mod_perl).

=back

=head1 DESIGN PATTERNS IN IDX::PKI::Registry

=head2 Model-View-Controller (MVC)

B<IDX::PKI::Registry> is a framework intended for writing CGI scripts
that only implements the motor (or "controller"), the graphic shell
being delegated to a distinct output generator (or "view") which
may not even be written in Perl. The ins and outs of this design pattern
are described in the IDX-PKI developper documentation; to make it short:

=over 2

=item *

the B<model> represents the `business logic' - a big buzzword, which
means: stuff that matters for your app, that you store in your
database, etc regardless of how, when and whether you display
it. It is materialized in IDX-PKI by all I<IDX::PKI::*> objects and
classes. They reside in C<sources/src/perllib> in the sources. They
are I<passive>, in that they don't perform any action on their own
behalf but merely exist as function libraries that are ready to be
invoked by the controllers;

=item *

the B<controllers> are pieces of code that handle user input, then
call methods over the model in order to read stuff or do stuff. In
IDX-PKI they are the so-called "CGI" (in fact, registry) scripts,
which reside in C<sources/src/perlscript> and have a .cgi
extension. They are very short, and follow a very strict coding style
(described below) in order to enforce security. Since they are the
code that handles user input, they are indeed the most
security-critical;

=item *

the B<views> are the code that display the results of user
actions. Two kinds of views are implemented in IDX-PKI, more may
follow: I<programmatic views> that display the results in pure text;
and I<PHP views> that trigger an internal redirect to a PHP page. The
PHP is sandboxed (through careful writing of php.ini) in such a way
that B<the views have no power except render the parameters passed to
them by the controllers>. Tasks such as setting up HTML responses,
which can be complex and tedious, is therefore performed in a
low-privilege environment.

Having two kinds of views allows for the straightforward creation of
an IDX-PKI API. The programmatic views are automatically invoked
instead of the PHP views when the server detects that the client is
not a normal browser, but a piece of an IDX-PKI automatic client (the
I<User-Agent> request header does the trick). Therefore we see that
both bonuses of MVC, security and modularity, are simultaneously taken
advantage of in IDX-PKI.

=back

In some special cases (for example, when returning the CA certificate
in a browser-specific encoding), it is desirable for the Perl script
to do its outputting by itself. For this purpose, a non-MVC fallback mode
is supported.


=head2 Tainting

=head2 Low-watermark mandatory access control (LOMAC)

=head2 Sandboxing

=head1 IDX-PKI PAGE PROGRAMMER TUTORIAL

=head2 The CGI variable

Saying "use IDX::PKI::Registry;" has the side effect of importing a
global variable $cgi into the caller's namespace. This variable is
automatically initialized by the support code before the code in the
page is executed.

In an I<IDX::PKI::Registry> script, the $cgi variable is the only contact
of the script with the Hostile Outside World. Through this object, the
script may do authentication, read parameters from the request (in a
controlled way), write parameters for the view, and finally invoke the
view.

In order to enforce the programming guidelines described herein, B<the
order in which $cgi's methods are called is mandatory>. Namely:

=over 2

=item *

$cgi->setupViews() must be the very first method called;

=item *

$cgi->authorize() must be called before $cgi->error(),
$cgi->failure() or $cgi->success().

=back

Failure to comply with these rules will trigger an exception. Hence,
the typical flow of control in a CGI page is as follows:

=over 2

=item *

prepare the HTML output mechanism ahead of time with I<$cgi->setupViews()>;

=item *

authenticate and access-control the user with I<$cgi->authorize()>;

=item *

fetch parameters from the CGI environment with I<$cgi->validate()>. In
case the user made a mistake in the cgi parameters, call
I<$cgi->error()>, maybe like this:

    $cgi->error(-32) if $cgi->validateFailures();

=item *

do any appropriate back-end processing with other IDX-PKI object classes;

=item *

invoke the view with I<$cgi->success()> or I<$cgi->failure()>, depending
on the outcome of the back-end processing above.

=back


=head2 The configuration and log variables

FIXME

=head2 Interface between the controller and the view

The controller does all its job, and then sends the results to the
view. The view is written in PHP that runs in a sandboxed environment;
it has no power whatsoever to query databases, launch scripts, fire up
Perl classes, etc. The only data it gets to act upon are passed
through the Apache I<subprocess_env> array and the request string, so
that they appear as I<$HTTP_SERVER_VARS[]> and I<$HTTP_GET_VARS[]> in
PHP:

=over 2

=item *

The return value passed by the controller to I<success()>, I<failure()>
and I<error()> is encoded into a variable named
I<$HTTP_SERVER_VARS[REDIRECT_DATA]>.  If a complex data structure (that
is, a reference) is passed to I<success()>, then it is transcoded into a
string that PHP may I<eval()> to rebuild the very same structure (with the
same arrays, hashes and nestings thereof):

 $cgi->success([41,42,"zap",{}]);

will put C<array(41, 42, 'zap', array())> in the I<REDIRECT_DATA> server
variable -- ready to I<eval()> from PHP (see code snippets below).

=item *

Those CGI parameters that are I<validate()>d or set with I<param()> are
passed to the request string, which is overwritten, so that they are
visible to the PHP view in array I<$HTTP_GET_VARS[]>. For example, if
C<validate("PARAM", ...)> has been called in the model, the old value
(read from the client request) remains in I<ORIG_PARAM>, the new value is
set in I<TRUSTED_PARAM> and the reason of the error (if any) is set in
I<ERROR_PARAM>. I<PARAM> always keeps the "best known" value for the said
parameter:

=over 4

=item *

If I<PARAM> was not validated, the value is unchanged;

=item *

If I<PARAM> failed the validation, the value is the same as I<ORIG_PARAM>
(I<TRUSTED_PARAM> is C<undef>);

=item *

If I<PARAM> failed the validation, the value is the same as
I<TRUSTED_PARAM>.

=back

=item *

The model may set any number of I<$HTTP_GET_VARS[]> using
I<internalparam()>.

=back

=head2 Examples

Here are an example of an I<IDX::PKI::Registry> script that asks for a
phone number (french - 10 digits) and just displays it (e.g. passes it
to the view):

    $cgi->setupViews(-success=>"test.php");
    $cgi->authorize("__NONE__");

    my $phonenum=$cgi->validate("TELEPHONE",qr/^((?:[0-9]\s*){10})$/);
    $cgi->error(-32) if ($cgi->validateFailures());

    $cgi->success("All subsystems operational");

Here is a PHP view that will display the results of the above script
(assuming the snippet above is named "test.cgi", and resides in the
same directory of the WWW server as the PHP code below, named
"test.php"):

 <!DOCTYPE html PUBLIC "-//W3C//DTD HTML 4.0//EN"
 	"http://www.w3.org/TR/REC-html40/strict.dtd">
 <html>
   <head>
     <title>Sample form for IDX::PKI::Registry</title>
   </head>
   <body>
     <h1>Sample form for IDX::PKI::Registry</h1>


 <?php
  $data = $HTTP_SERVER_VARS["REDIRECT_DATA"];

  if (isset ($data)) {
 	/* We don't error out if $data is empty, so that
 	   pointing the browser to the "bare-bones" PHP page
 	   gives an empty form.
 	 */
    	if (get_magic_quotes_gpc ()) {
    		$data = stripslashes ($data);
    	}
    	eval ("\$data = $data;");
  }
  // Or equivalent to the above block:

  // include "commonvars.inc";
  // $data=IDXControllerData();

  $oldvalue="value=\"".$HTTP_GET_VARS["TELEPHONE"].
 		"\"";

 	if ($data == -32) {
 		$errormsg="<p><strong>We are not satisfied: ".
 			$HTTP_GET_VARS["ERROR_TELEPHONE"].
 			"</strong></p>"; /* In a real-life multi-field form, we should
                                test for all ERROR_* variables separately,
                                e.g. to display error messages as text in
                                red, next to the incorrect input fields. */
 	} elseif (isset($data)) {
 		$tel=$HTTP_GET_VARS["TRUSTED_TELEPHONE"];
 		print "
     <p>
       <srong>Thank you!!</strong><br>
       For your phone number, which is $tel.  For
       information, the result of your query was: <pre> $data</pre>.<br>
       Bye!
     </p>
 		";
 		exit; // Don't display the form again.
 	};

 print "
     <form action=\"test.cgi\">
       <p>$errormsg</p>
       <p>Phone number: <input type=\"text\"
         name=\"TELEPHONE\" $oldvalue></p>
       <input type=\"submit\">
     </form>
 ";

 ?>

   </body>
 </html>

=head2 Testing IDX::PKI::Registry scripts

Just like standard CGI programs written with L<CGI>, one may launch a
standalone IDX::PKI::Registry from the command line like this:

 perl -T -I$PERL5LIB myscript.cgi arg1=val1 arg2=val2...

Mind the -T -I$PERL5LIB, which enable the (mandatory) taint checks
inside the Perl interpreter.

You may also pass parameters to be interpreted as subprocess_env
values by prefixing them with a plus sign, which is useful for
simulating the SSL authentication:

 perl -T -I$PERL5LIB myscript.cgi +SSL_CLIENT_VERIFY=SUCCESS \
 	"+SSL_CLIENT_S_DN=CN=John Doo, O=IDEALX,C=fr"

Please note that I<every script will require a working configuration
file>, just in order to set up the $log object. Tests over
IDX::PKI::Registry scripts are therefore not unit, but integration
tests.

=head2 Porting old IDX::PKI::CGI scripts to IDX::PKI::Registry

This step would typically occur only if some new feature needs to be
added to an I<IDX::PKI::CGI> script: the I<IDX::PKI::Registry> handler
is capable of handling "legacy" scripts without modification.

=over 4

=item *

Convert IDXPKI_DATA to REDIRECT_DATA in the PHP script;

=item *

Check that all PHP variables are accessed through HTTP_SERVER_VARS
(and mind the REDIRECT_ prefix) or HTTP_GET_VARS only (HTTP_ENV_VARS
should not be used anymore);

=item *

Suppress the boilerplate for handling exceptions and creating the
I<$cgi>, I<$config> and I<$log> variables;

=item *

Run through all calls to $cgi and correct them:

   my $cgi=new IDX::PKI::CGI(-PHPpage=>$page);


becomes

   $cgi->setupViews($page);

=back

=head2 The programmatic view

The programmatic views give text/plain documents presented
like this:

 key1=value1
 key2=value2
 key3=value3

 returnvalue

The fields are those set with L</param>, L</internalparam> and
L</validate>.  The C<returnvalue> is the parameter passed to L</error>,
L</failure> or L</success>, except if there should have been an
external redirect according to the rules in L</success> - in which case
C<returnvalue> is just "SUCCESS".


=head2 A Little Note about File Uploads

File uploads are a bit special, as I<two> values are associated with a
single parameter: The original filename and the file contents.  Thus, two
parameters are available: I<paramname> contains the transmitted data,
while the filename is stored as I<< <paramname>.filename >>.  The filename
which may, or may not, contain an absolute or relative path from the
client computer, so you really should strip down any directory
indications.

You can then access these parameters just like any others.


=head1 METHODS AND FUNCTIONS

=head2 Methods

=over 4

=cut "

package IDX::PKI::Registry::CGIobject;

use strict;

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Authentication;
import IDX::PKI::Registry qw($CRLF); use vars qw($CRLF);
use Data::Dumper; # For the programmatic views
use Taint qw (taint tainted);

BEGIN {
	my $tainted = "";
	taint ($tainted);
	my $taint_checking = tainted ($tainted);
	if (caller(2) && ! $taint_checking) {
		die <<MESSAGE if IDX::PKI::Registry::inModPerl();
Tainting must be enabled in all IDX-PKI Registry scripts now. Add this
to the <Perl> startup section of idxpki-httpd.conf:

    PerlTaintCheck on
MESSAGE

		die <<MESSAGE;
Tainting must be enabled in all IDX-PKI
Registry scripts. Relaunch the Perl interpreter like this:

    perl -T -I\$PERL5LIB $0 ...
MESSAGE
	};
}


=pod "

=item I<_new($log)>

=item I<< _new($log, {'key1' => $value1, 'key2' => $value2, ...})
>>

There is no public constructor for B<IDX::PKI::Registry>, which is a
somewhat unusual situtation: the simple fact of saying C<use
IDX::PKI::Registry> creates one for you and puts it in the package-global
variable I<$cgi>, and you need not (and shouldn't) call this constructor
from user code.

This method recognises three calling environments:

=over 4

=item *

The CGI runs under mod_perl (standard mode);

=item *

The CGI is run from the command line (see L</Testing IDX::PKI::Registry
scripts>);

=item *

Pseudo-request parameters are passed directly to this function as an
anonymous hash (for testing purpose too).

=back

Input parameters (as they were submitted by the browser) are automatically
saved into I<< <paramname> >> (which may change later) and I<<
ORIG_<paramname> >> (which remains the same).

=cut "

sub _new {
	my ($class, $log, $args_for_tests) = @_;

	my $config = the IDX::PKI::Config ();
	my $self = bless {
		config => $config,
		log    => $log,
		subreq => {},
	}, $class;

	my %params;
	if (IDX::PKI::Registry::inModPerl()) {
		my $r = Apache->request();
		$self->{'r'} = $r;

		# When we run under mod_perl, we may have three ways to get our
		# parameters: GET args, POST args, and 'multipart/form-data' args
		# (which is used for file uploads).
		my @get_args  = $r->args();
		my @post_args = $r->content();
		my @data_args;

		# This handles multipart/form-data requests.
		my $content_type = $r->header_in ("Content-type");

		# XXX Maybe we should allow this only when a special parameter has
		# been passed to 'use IDX::PKI::Registry'.

		if ($content_type && $content_type =~ m|multipart/form-data|i) {
			my ($boundary) = ($content_type =~ m|boundary=([^;]+)|);
			unless (defined ($boundary)) {
				throw IDX::PKI::Error::User ("Invalid request",
											 '-details' => "No boundary found",
											 '-content-type' => $content_type);
			}
			$boundary = qr"\Q$boundary\E";

			# Gets request contents.
			my $size = $r->header_in ("Content-Length") ||
				throw IDX::PKI::Error::User ("Invalid request", '-details' =>
											 "No content-length");
			my $contents;
			$r->read ($contents, $size);
			$contents =~ s/-*$boundary--.*/$boundary/s;

			# Splits various MIME parts.
			my @parts = split (m/-*$boundary/m, $contents);
			local $CRLF = qr/\r?\n/;
			PART: foreach my $elem (@parts) {
				# A part is composed of headers, then two $CRLF, then
				# contents.
				my ($attr, $val) =
					($elem =~ m/^
					 (?:$CRLF)*
					 (?:[^\r\n]+$CRLF)*
					 Content-Disposition:\ form-data;\ ([^\r\n]*)$CRLF
					 (?:[^\r\n]+$CRLF)*
					 $CRLF
					 (.*?)
					 (?:$CRLF)?
					 $/xs);

				next PART unless (defined ($attr));
				$val = "" unless (defined ($val));

				# Part "headers".
				my %attrs;
				PARTHEAD: foreach my $part (split (m/; /, $attr)) {
					my ($k, $v) = ($part =~ m/^([a-z]+)="(.*)"$/i);
					throw IDX::PKI::Error::User
						("Invalid request",
						 '-details' => "Can't parse Content-Disposition",
						 '-content-disposition' => $part)
						unless (defined ($k) && defined ($v));
					$attrs{lc($k)} = $v;
				}

				# Put the right things into @data_args (arg => contents),
				# and adds arg.filename => filename if present.
				push (@data_args, $attrs{'name'}, $val);
				push (@data_args, "$attrs{'name'}.filename",
					  $attrs{'filename'}) if (exists ($attrs{'filename'}));
			}
		}

		%params = (@get_args, @post_args, @data_args);
	} elsif (ref($args_for_tests)) {

		# There seems to be a very strange bug in Taint::taint 0.09. If an
		# argument here is numeric and we try to taint()ing it, then
		# afterwards all further attempt to taint() values will lead them
		# to be set to 0 with a "Argument <the param value> isn't numeric
		# in bitwise and (&)" message.

		%params = map {"$_"} %{$args_for_tests};
	} elsif (scalar IDX::PKI::Registry::inIntegrationTest()) {
		my @args = IDX::PKI::Registry::inIntegrationTest();
		while (my ($k, $v) = splice(@args, 0, 2)) {
			if ($k =~ m/^\+(.*)$/) {
				# Untaint it, as would be the case in Apache.
				$k = "$1";
				$v =~ m/^(.*)$/;
				$self->{'subprocess_env'}->{$k} = $1;
			} else {
				$params{$k} = $v;
			}
		}
	} else {
		throw IDX::PKI::Error::Internal("UNIMPLEMENTED", '-details' =>
										"Unrecognised calling method.");
	}

	foreach my $val (values %params) {
		taint ($val) if (defined $val);
	}
	$self->{'params_in'} = \%params;

	# Automatically save parameters for reedition by the user.
	foreach my $key (keys (%{$self->{'params_in'}})) {
		# This is a way of doing an internalparam () without it to
		# complain about wrong calling order.
		$self->_internalparam ("ORIG_$key", $self->{'params_in'}{$key});
		$self->_internalparam ("$key", $self->{'params_in'}{$key});
	}

	return $self;
}

=pod "

=item I<setupViews ("__NONE__")>

=item I<setupViews ($onepagefitsall)>

=item I<< setupViews ("-error" => $errorpage, "-success" => $successpage,
"-failure" => $failurepage) >>

Set up the Model-View-Controller parameters of this script.  The three
standard named parameters are the pages to be served to the user's
navigator for each of the four possible outcomes of the script (see
L</failure()>, L</success()> and L</error()>).

I<-success> is mandatory; C<< $cgi->setupViews("somepage") >> is a
short-hand for C<< $cgi->setupViews(-success=>"somepage") >>.  If one and
only one of I<-error> and I<-failure> is provided, then the second one
uses the same value.  If none of them is provided, they default to the
value of I<-success>.

Other specific non-standard views may be defined as well.  You can then
trigger them with I<terminate()>.

All options accept plain URLs as well as I<sprintf()>-encoded URL
patterns.  This has an influence on the outcome of the I<success()> (or
other) call (see L</success> for example).

This method must be the very first one called against I<$cgi> (see
L</IDX-PKI PAGE PROGRAMMER TUTORIAL>).

=cut "

sub setupViews {
	my ($self, @args) = @_;
	throw IDX::PKI::Error::Internal("WRONG_CALLING_ORDER") if
		($self->{did_setupViews});

	if (scalar(@args) == 1) {
		if ($args[0] eq "__NONE__") {
			$self->{'printing_allowed'} = 1;
			$self->{"view_none"} = 1;
			$self->{'did_setupViews'} = 1;
			return 1;
		} else {
			@args = (-success => $args[0]);
		}
	} elsif (scalar(@args) % 2) {
		throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS");
	}

	my %args = @args;

	foreach my $page (keys (%args)) {
		# Options must begin with -.
		$page =~ s/^-// || throw IDX::PKI::Error::Internal ("INCORRECT_ARGS");
		$self->{"pages"}{"$page"} = $args{"-$page"};
	}

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") unless
		(defined $self->{"pages"}{"success"});

	# Error page defaults to failure page if it is defined, success page
	# otherwise.
	if (!defined $self->{"pages"}{"error"}) {
		$self->{"pages"}{"error"} = $self->{"pages"}{"failure"} ?
			$self->{"pages"}{"failure"} : $self->{"pages"}{"success"};
	}
	# Failure page defaults to error page if it is defined, success page
	# otherwise.
	if (!defined $self->{"pages"}{"failure"}) {
		$self->{"pages"}{"failure"} = $self->{"pages"}{"error"} ?
			$self->{"pages"}{"error"} : $self->{"pages"}{"success"};
	}

	$self->{did_setupViews} = 1;
}

=pod "

=item I<authorize ($operation, $section)>

Performs I<IDX::PKI::Authentication::authorize> on the client that invoked
this page. A operation name C<__NONE__> will allow any peer, even if no
authentication mechanism is available (e.g. over plain HTTP).

=cut "

sub authorize {
	my ($self, $operation, $section) = @_;
	throw IDX::PKI::Error::Internal("WRONG_CALLING_ORDER") unless
		($self->{did_setupViews});

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") unless
		($operation && $operation =~ m/^(PRIV|__NONE__)/);

	unless ($operation eq "__NONE__") {
		IDX::PKI::Authentication::authorize($self, $operation, $section);
	}

	$self->{did_authorize} = 1;
}

=pod "

=item I<getPeerDN()>

Returns the DN of the client's certificate, if available. See
L<IDX::PKI::Authentication/getPeerDN>.

=cut "

sub getPeerDN {
	my ($self)=@_;
	my $verify=$self->_subprocessenv("SSL_CLIENT_VERIFY");
	return (wantarray ? () : undef) unless ($verify && $verify eq "SUCCESS");
	my $name=$self->_subprocessenv("SSL_CLIENT_S_DN");

	return wantarray ? parse IDX::PKI::DN($name) : $name;
}

=pod "

=item I<getPeerCertificate()>

Returns the peer's certificate, as set by C<mod_ssl> or similar, in
X509 PEM format. Returns undef if the peer did not successfully
authenticate.

=cut "

sub getPeerCertificate {
	my ($self)=@_;
	my $verify=$self->_subprocessenv("SSL_CLIENT_VERIFY");
	return undef unless ($verify && $verify eq "SUCCESS");
	return $self->_subprocessenv("SSL_CLIENT_CERT");
}

=pod "

=item I<_subprocessenv($variable)>

Returns the value associated with $variable in Apache's "subprocess
environment" table, or the local equivalent under the current WWW
server. This table is the home for standard CGI and less standard SSL
variables.

=cut "

sub _subprocessenv {
	my ($self,$variable)=@_;
	if (IDX::PKI::Registry::inModPerl()) {
		return $self->{r}->subprocess_env($variable);
	} else {
		return $self->{subprocess_env}->{$variable};
	}
}

=pod "

=item I<param ($param[, $value])>

Sets or resets a parameter in CGI parameters (I<$HTTP_GET_VARS[]> in PHP
parlance) that will be passed to the view, emulating what I<validate()>
would do: The old value as it was in the CGI parameters received from the
user's browser remains available as I<$HTTP_GET_VARS["ORIG_paramname"]>,
while I<$value> will end up in I<$HTTP_GET_VARS["TRUSTED_paramname"]>.
I<$HTTP_GET_VARS["paramname"]> will contain I<$value>. If I<$value> is
C<undef>, I<TRUSTED_paramname> and I<paramname> are erased
(I<ORIG_paramname> stays as-is).

If I<$value> is absent, returns the I<validated> value of I<$param>.  It
is an error to try accessing a parameter before having validated it (see
L</validate()>).

=cut "

sub param {
	my ($self, $key, $value) = @_;
	throw IDX::PKI::Error::Internal("WRONG_CALLING_ORDER") unless
		($self->{did_setupViews});

	if (@_ == 3) {
		# Setting a parameter automatically validate()s it.
		$self->_internalparam ($key, $value);
		$self->{'validated_params'}{$key} = 1;
		return $self->_internalparam ("TRUSTED_$key", $value);
	} elsif (@_ == 2) {
		throw IDX::PKI::Error::Internal ("WRONG_CALLING_ORDER",
			'-details' => "You must validate() your parameters before you ".
			"can access their contents with param()")
			unless ($self->{'validated_params'}{$key});
		return $self->_internalparam ("TRUSTED_$key");
	} else {
		throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS");
	}
}

=pod "

=item I<internalparam ($param[, $value])>

Directly sets any value in I<$HTTP_GET_VARS[]>. This function is an
adequate way of passing miscellaneous sidekick variables that for some
reason one doesn't want to put into the main return value through
L</error>, L</success> and L</failure>.

If I<$value> is supplied as C<undef>, the parameter is erased.  If
I<$value> is not supplied at all, returns the current value of the
internal parameter.

=cut "

sub internalparam {
	# Don't use $value, because _internalparam ($key, undef) and
	# _internalparam ($key) don't have the same meaning.
	my ($self, $key, @args) = @_;
	throw IDX::PKI::Error::Internal("WRONG_CALLING_ORDER") unless
		($self->{did_setupViews});

	# Forbids direct access to a parameter...
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-details" =>
									 "Use param() to access CGI parameters")
		if ($key =~ m/^ORIG_/ || exists ($self->{"subreq"}->{"ORIG_".$key}));

	return $self->_internalparam ($key, @args);
}

=pod "

=item I<_internalparam ($param[, $value])>

Almost the same as I<internalparam()> except that it accepts to set any
parameter, without a regard for the C<ORIG_> and C<TRUSTED_> stuff
whatsoever. Internal use only, never use it directly.

This method is the underlying implementation of all other
I<$HTTP_GET_VARS[]> write accessors.

=cut "

sub _internalparam {
	my ($self, $key, $value) = @_;

	if (@_ == 3) {
		if (defined $value) {
			$self->{"subreq"}->{$key} = $value;
		} else {
			delete $self->{"subreq"}{$key};
		}
	} elsif (@_ == 2) {
		return $self->{'subreq'}{$key};
	} else {
		throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS");
	}
}

=pod "

=item I<bigparam($param,$valuestruct)>

Just like I<internalparam()> except that passing occurs through
$HTTP_SERVER_VARS[], and a mandatory "REDIRECT_" prefix will be
prepended to $param from the PHP point of view. This function may be
used to pass a big amount of data to the PHP; it is not compatible
with the external redirect mechanism of L</success>.

$valuestruct undergoes the same PHPification of structures as the
return value of I<success()>, and thus may be a Perl structure.

=cut "

sub bigparam {
	my ($self, $param, $message) = @_;
	if (IDX::PKI::Registry::inModPerl()) {
		$self->{r}->subprocess_env
			($param, IDX::PKI::Registry::phpencode($message));
	} else {
		$self->{subprocess_env}->{$param} =
			IDX::PKI::Registry::phpencode($message);
	}
}


=pod "

=item I<propagate($param)>

B<This method is obsolete and does nothing: Parameters are automatically
propagated for you.>

=cut "

sub propagate {
	return 1;
}

=pod "

=item I<< validate ($paramname, $method, $arg1 => $val1, ...) >>

Validates an inbound CGI parameter, by substituting untrusted user input
(which is still available into what the PHP view sees as
I<$HTTP_GET_VARS["ORIG_paramname"]>) with an untainted, possibly
canonicalized value (termed "corrected value" below) which will end up in
I<$HTTP_GET_VARS["TRUSTED_paramname"]>. I<$HTTP_GET_VARS["paramname"]> is
the same as I<$HTTP_GET_VARS["TRUSTED_paramname"]> if the validation
succeeded, the same as I<$HTTP_GET_VARS["ORIG_paramname"]> otherwise (see
L</param()>).

The return value of I<validate()> is the corrected value. Here are a few
usage examples to make this lengthy documentation entry clearer:

 my $phonenum = $cgi->validate ("admin_phonenum", qr/[0-9 ]+/,
 	"-else" => -16);

 my $gender = $cgi->validate ("admin_sex",
 	sub {m/^(m|f)/i ? $1 : "m"},
 	"-INVALID" => "Sex must be either (M)ale or (F)emale");

 my $email = $cgi->validate ("admin_email",
 	\&IDX::PKI::ValidateInput::email,
 	"-extraargs" => 20,	# Max length.
 	"-default" => 'root@example.com',
 	"-else" => sub {$errors++});

 my $email = $cgi->validate ("user_email",
 	\&IDX::PKI::ValidateInput::email,
 	"-MISSING" => "An email address for the user is mandatory",
 	"-else" => sub {$errors++});

 # Multi-parameter form validation
 my $date = $cgi->validate ("date",
     [\&IDX::PKI::ValidateInput::MultiParam::date, "year",
 		"month", "day"]);

The first argument to this method is the name of the CGI parameter to
validate; the second is the method to use to do the validation. If any
errors occur in the validation, an error message will be set in
I<$HTTP_GET_VARS["ERROR_paramname"]>.

If the CGI parameter asked for does not exist, and C<-default> is
specified, it will be used as the corrected value without any further
computation; the following two paragraphs thus do not apply (setting
C<< -default => undef >> is therefore an appropriate way of saying
that you don't mind for some parameter to be absent).  If C<-default>
is not specified, C<ERROR_$paramname> will be set to the value of the
I<-MISSING> named parameter, or C<MISSING> by default. The validation
error counter will be incremented (see L</validateFailures>) and no
further computation takes place either.

If there is indeed a value provided by the user (as opposed to by the
I<-default> call parameter), I<$method> is used to validate it. I<$method>
can be a subroutine or a regexp containing at least one pattern
extraction (with parentheses). This routine or regexp will be run with
I<$_> (and also the first argument, in the function case) set to the
untrusted value; it should return (or match) the "best guess" of what
the return value should actually be. It is an error to return a
tainted value (see L</perlsec>) so please be careful about this issue
when writing I<$method>'s. If no amount of second-guessing can coerce the
user input into something valid, then I<$method> should return C<undef> (or
match C<undef> in I<$1>). In this case, C<ERROR_$paramname> will be set to
either the value of the C<-INVALID> parameter, or C<INVALID> by
default. Furthermore, the validation failure counter will be
incremented (see L</validateFailures>).

If the C<-extraargs> argument is provided, it is transmitted to I<$method>
(in the function case) as further arguments.  It may be a scalar or an
array reference; in the latter case it is expanded to its contents.

The return value is the corrected value, or undef if no corrected
value was found.

In any case of failure (either by incorrect or missing parameter), the
C<-else> named call parameter, if present, will be considered: If
C<-else> is associated with a subroutine reference, it will be called
without arguments; if C<-else> is a negative number, I<< $cgi->error() >>
will be called with it as an argument. The latter is considered bad
practice since it prevents the form from being further validated; thus
only one typing mistake will be shown to the user, even though she may
have done several in a row along the multi-field form. Better do
something like this when you are done parsing arguments, and do not
use the C<-else> feature for error reporting:

    $cgi->error(-32) if $cgi->validateFailures();

The astute reader will have noticed that I<validate()> is the only way
of getting CGI parameter information out of
B<IDX::PKI::Registry>. This is, of course, on purpose. A library of
adequate validation functions can be found in L<IDX::PKI::ValidateInput>.

=head3 Multi-parameter form validation

As an added bonus, B<IDX::Registry> supports validating forms that
provide a single logical value using multiple HTML input zones. Please
do not try to understand this section until you have a good grasp at
what the "normal" (single-parameter) I<validate()> does.

Consider the following form:

 <FORM>
   Day:   <INPUT TYPE="text" SIZE="2" NAME="date.day"/>
   Month:   <INPUT TYPE="text" SIZE="2" NAME="date.month"/>
   Year:   <INPUT TYPE="text" SIZE="2" NAME="date.year"/>
 </FORM>

By using a call to I<validate()> with an array reference as its second
argument (instead of the customary function reference), one is able to
validate the three parameters at once (here, maybe we would get a UNIX
timestamp). The semantics of I<validate()> described above will change
as follows:

=over 2

=item *

The first value in the array reference should be a function reference,
and the others should be scalars. Here is an example:

 my $date = $cgi->validate ("date",
 	[\&IDX::PKI::ValidateInput::MultiParam::date,
 		"year", "month", "day"]);

=item *

Said function reference will be called with as many (tainted)
arguments as there are extra scalars in the array (in the above
example, that's three). Those arguments will be the form parameters
that were passed to this CGI (just like in the normal I<validate()>
case), whose name is computed by joining the first of argument to
I<validate()> (here C<date>) and the extra scalars (here C<year>,
C<month> and C<day>) with a dot. Here the following call will be made:

 IDX::PKI::ValidateInput::MultiParam::date (
 	thevalue ("date.year"), thevalue ("date.month"),
 	thevalue ("date.day"));

except that of course the function I<thevalue()> does not actually
exist.  Thus, the above sample HTML form will be properly handled when
submitted by the user;

=item *

The function reference should accept both scalar and array context. In
array context, it should return untainted values corresponding to the
input parameters;

=item *

If a value in the anonymous list is undef, the corresponding value
passed to the function is undef too. This is to pad the arguments,
which is most useful in conjunction with C<-extraargs>. For example,

 my $date = $cgi->validate ("date",
     [\&IDX::PKI::ValidateInput::MultiParam::date,
       "year","month","day",undef,undef,undef],
     -extraargs=>"GMT"));

will result in a call to

 IDX::PKI::ValidateInput::MultiParam::date (
 	thevalue ("date.year"), thevalue ("date.month"),
 	thevalue ("date.day"), undef, undef, undef,
         "GMT");

=item *

If the validation fails (the call to the argument function returns undef),
I<ERROR_date> will be set, instead of the individual I<ERROR_date.day> or
such. The I<ORIG_> and I<TRUSTED_> parameters are treated as usual;

=item *

The handling of the return value of this call w.r.t. tainting et al
will be the same as in the normal I<validate()>.

=back

=cut "

sub validate {
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS") unless
		((@_ >= 3) && (@_ % 2));
	my ($self, $paramname, $function, %options) = @_;

	throw IDX::PKI::Error::Internal("WRONG_CALLING_ORDER") unless
		($self->{did_authorize});

	my $oldparam = $self->_internalparam ($paramname);
	if (ref ($function) eq "ARRAY")
	{
		# Multi-parameter form validation: We concatenate all input values.
		my (undef, @exts) = @$function;
		$oldparam = join ("", map {
			defined ($_) &&
				defined ($self->{"params_in"}->{$paramname.".".$_}) ?
				$self->{"params_in"}->{$paramname.".".$_} : ""
			} @exts);
	}
	else
	{
		$oldparam = $self->{"params_in"}->{$paramname};
	}
	my $newparam;

	if ((!defined $oldparam) || ($oldparam eq "")) {
		# No value - try the default value.
		if (exists $options{-default}) {
			$self->param ($paramname, $options{-default});
			$self->{'validated_params'}{$paramname} = 1;
			return $options{-default};
		} else {
			$self->_internalparam("ERROR_$paramname",
								 ($options{-MISSING} ?
								  $options{-MISSING} :
								  "MISSING"));
			return $self->_fail_validate($options{-else});
		}
	}

	# Just for $function as an ARRAY.
	my @newparams;

	# Value found - test $method against it
	if (!ref($function) || (ref($function) eq 'Regexp')) {
		local $1;
		"" =~ m/^$/; # Resets $1
		$oldparam =~ m/$function/;
		$newparam=$1;
	} elsif (ref($function) eq 'CODE') {
		my @extraparams;
		if (defined ($options{'-extraargs'}) &&
			ref ($options{'-extraargs'}) eq "ARRAY") {
			@extraparams = @{$options{'-extraargs'}};
		} elsif (defined ($options{'-extraargs'})) {
			@extraparams = ($options{'-extraargs'});
		}
		local $_=$oldparam;
		$newparam = &$function ($oldparam, @extraparams);
	} elsif (ref($function) eq 'ARRAY') {	# Multi-parameter form validation
		my ($code, @exts) = @$function;
		my @extraparams;
		if (defined ($options{'-extraargs'}) &&
			ref ($options{'-extraargs'}) eq "ARRAY") {
			@extraparams = @{$options{'-extraargs'}};
		} elsif (defined ($options{'-extraargs'})) {
			@extraparams = ($options{'-extraargs'});
		}
		# First: Scalar context (single value).
		$newparam = &$code ((map {
			defined $_ ? $self->{"params_in"}->{$paramname.".".$_} : undef;
		} @exts), @extraparams);
		# Second: Array context (should return the untained elements).
		@newparams = &$code ((map {
			defined $_ ? $self->{"params_in"}->{$paramname.".".$_} : undef;
		} @exts), @extraparams);
	} else {
		throw IDX::PKI::Error::Internal("INCORRECT_ARGS");
	}

	# Are we happy yet?
	if (defined $newparam) {
		# Yes.
		throw IDX::PKI::Error::Internal("TAINTED",
			-details=>"Attempt to use CGI parameter $paramname".
										" without untainting it") if
										  (tainted($newparam));

		$self->param($paramname,$newparam);
		$self->{'validated_params'}{$paramname} = 1;
		# For $function as an ARRAY: Keeps each individual parameter too.
		if (ref ($function) eq "ARRAY") {
			my (undef, @exts) = @$function;
			for (my $i = 0; $i <= $#exts; $i ++)
			{
				next unless (defined $exts[$i]);
				$self->param ($paramname.".".$exts[$i], $newparams[$i]);
				$self->{'validated_params'}{$paramname.".".$exts[$i]} = 1;
			}
		}
		return (wantarray && ref ($function) eq "ARRAY") ? @newparams :
			$newparam;
	}

	# No, and $function went bang. Signal the error.
	$self->_internalparam("ERROR_$paramname",
								 ($options{-INVALID} ?
								  $options{-INVALID} :
								  "INVALID"));
	return $self->_fail_validate($options{-else});
}

=pod "

=item I<_fail_validate($else)>

Forcibly fails a validation, incrementing the failure count (see
L<validateFailures>) and calling $else if useful according to the
semantics of the I<-else> parameter in L</validate>. Only useful
inside said function I<validate()>. Always return undef.

=cut "

sub _fail_validate {
	my ($self,$else)=@_;
	$self->{validate_failures}++;

	if (! defined $else) {
		# nothing
	} elsif (ref($else) eq 'CODE') {
		&{$else}();
	} elsif ($else =~ m/^-[0-9]*$/) {
		$self->error($else);
	} else {
		throw IDX::PKI::Error::Internal("INCORRECT_ARGS");
	};
	return undef;
}

=pod "

=item I<validateFailures()>

Returns the number of times a call to L</validate()> failed since the
creation of the object.

=cut "

sub validateFailures {
	my ($self)=@_;
	return $self->{validate_failures};
}

=pod "

=item I<header_out($header,$value)>

=item I<content_type($value)>

=item I<content_length ($value)>

=item I<status($value)>

=item I<attachment ($filename)>

=item I<print()>

Those functions are only available if the model does not to have an
external view (see L</setupViews>), or from inside B<IDX::PKI::Registry>
itself.  Their semantics are similar to the likewise-named functions in
B<Apache> or B<CGI>.

I<header_out()> prepares for an HTTP header to be set in the output.

I<status()> sets the HTTP status (C<200> is the default).

I<content_type($value)> is a shorthand for C<header_out("Content-Type",
$value)>.

I<content_length($value)> is a shorthand for
C<header_out("Content-Length", $value)>.

I<attachment ($filename)> is a shorthand for C<header_out
("Content-Disposition", "attachment; filename=\"$filename\"")>.

I<print()> prints raw data to the user, first sending any previously set
I<header_out()>s and I<status()> if this is the first time I<print()> is
called.

=cut "

sub print {
	my $self=shift;
	throw IDX::PKI::Error::Internal("ILLEGAL_CONTEXT") unless
		($self->{printing_allowed});

	if (IDX::PKI::Registry::inModPerl()) {
		if (! $self->{sent_headers}) {
			$self->{r}->send_http_header();
			$self->{sent_headers}=1;
		}
		$self->{r}->print(@_);
	} else {
		$self->{"status"} = 200 unless (defined $self->{"status"});
		print "HTTP/1.0 $self->{status}$CRLF";

		if (! $self->{sent_headers}) {
			while(my ($k,$v)=each %{$self->{headers}}) {
				map { print "$k: $_$CRLF" } @$v;
			}
		}
		print @_;
	}
}

sub content_type {
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS") if
	  (@_ ne 2);
	my ($self,$type)=@_;
	throw IDX::PKI::Error::Internal("ILLEGAL_CONTEXT") unless
		($self->{printing_allowed});

	if (IDX::PKI::Registry::inModPerl()) {
		return $self->{r}->content_type($type);
	} else {
		$self->header_out("Content-Type",$type);
	}
}

sub content_length {
	throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS") if (@_ ne 2);
	my ($self, $length) = @_;
	throw IDX::PKI::Error::Internal ("ILLEGAL_CONTEXT")
		unless ($self->{'printing_allowed'});

	$self->header_out ("Content-Length", $length);
}

sub header_out {
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS") if
		(@_ ne 3);
	my ($self,$header,$value)=@_;
	throw IDX::PKI::Error::Internal("ILLEGAL_CONTEXT") unless
		($self->{printing_allowed});

	if (IDX::PKI::Registry::inModPerl()) {
		return $self->{r}->header_out($header,$value);
	} else {
		push @{$self->{headers}->{$header}},$value;
	}
}

sub status {
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS") if
		(@_ ne 2);
	my ($self,$status)=@_;
	throw IDX::PKI::Error::Internal("ILLEGAL_CONTEXT") unless
		($self->{printing_allowed});

	if (IDX::PKI::Registry::inModPerl()) {
		return $self->{'r'}->status($status);
	} else {
		$self->{status}=$status;
	}
}

sub attachment {
	throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS") if (@_ ne 2);
	my ($self, $filename) = @_;
	throw IDX::PKI::Error::Internal ("ILLEGAL_CONTEXT")
		unless ($self->{'printing_allowed'});

	$self->header_out ("Content-Disposition",
					   "attachment; filename=\"$filename\"");
}


=pod "

=item I<success($scalar)>

=item I<success($printfval1, $printfval2, ...)>

=item I<failure($scalar)>

=item I<error($scalar)>

They terminate the script and call the view in order to print the WWW
page.  I<success()> is to be called if the script completed successfully,
while I<failure()> and I<error()> are more or less aliases and mean that
something went wrong.  I<failure()> corresponds to an internal error
(configuration error, transport error, ...) and I<error()> an user error
(wrong format input, empty mandatory field, ...).

One of the two available views (programmatic and PHP) will be selected
according to the following algorithm: if I<< $self->setupViews("__NONE__")
>> was called beforehand (see L</setupViews>), or the C<User-Agent>
request header is absent, or contains the substring C<IDXPKI-API>, the
programmatic view will be selected; otherwise the PHP one is used.
The parameter of the function call above is serialized and passed to
the view, which decides what to do with them (see L</Interface between
the controller and the view> for details).

There are two exceptions to this latter rule:

=over 4

=item *

When I<setupViews()> was called with a I<sprintf()>-style argument to
I<-success>, I<-failure> or I<-error>, an B<external redirect> will be
sent to the browser using the parameter(s) in the second form of the call
above, and no view will be triggered at all (right now - of course the
expected effect of the redirect is still to invoke a page that does output
HTML so the view will in fact follow immediately).  This is the preferred
way of expressing success in scripts that perform updates in any of
IDX-PKI's internal databases.  Redirecting the user's browser creates a
new entry in their history, I<thereby rendering the reload button
idempotent>: Pressing C<reload> merely fetches the redirect target again,
and does not resubmit the form.

=item *

The external redirect rule also applies when B<zero parameters> are
specified to I<success()>, I<failure()> or I<error()>.

=back

Thus the following snippets both perform an external redirect:

 $cgi->setupViews('-success' => "somewhere.html");
 $cgi->success();

 $cgi->setupViews('-success' =>
 	"view-certificate.cgi?id=%d");
 $cgi->success(45);

Those functions shall be the only channel used in an B<IDX::PKI::Registry>
script to communicate with the HTTP client.  Exactly one of them must be
called exactly once (except in I<setupViews("__NONE__")> mode).  These
functions never return; from the point of view of the model script, they
terminate execution of the page.

I<< B<Warning:> In some cases it is not possible to call an other CGI
script as a view, because of an obscure bug in the Apache handler.  This
typically happens when the initial script was called with GET parameters.
In this case, you B<must> call it as an external redirect by one of the
two methods described above.  >>

=cut "

sub error {
	my ($self, @args) = @_;
	$self->terminate ("error", @args);
}

sub failure {
	my ($self, @args) = @_;
	$self->terminate ("failure", @args);
}

sub success {
	my ($self, @args) = @_;
	$self->terminate ("success", @args);
}


=pod "

=item I<terminate ($view, @args)>

This is the same as I<success()>, I<failure()> and I<error()> except that
it allows to call an arbitrary view.  Of course, the view must have been
defined with I<setupViews()>.  I<success>, I<failure> and I<error> are
always defined (see L<setupViews()>).

=cut "

sub terminate {
	my ($self, $view, @args) = @_;
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless ($self->{"view_none"} || defined ($self->{"pages"}{$view}));
	$self->_choose_redirect_method ($self->{"pages"}->{$view}, @args);
}


=pod "

=item I<_choose_redirect_method ($page, @args)>

This method chooses between an external redirect (see L</success()>,
L</failure()> or L</error()>) or a call to I<_callView()>.

=cut "
sub _choose_redirect_method {
	my ($self, $page, @args) = @_;

	if ((defined $page) && ((@args == 0) || ($page =~ m/%[^%]/))) {
		# External redirect
		warn "Doing external redirect";
		my $uri = sprintf ($page, @args);

		# XXX If the URI was relative, we should convert it to an absolute
		# one.  But we need a way to know the canonical URL -- for now
		# just leave it alone and hope for the best.

		#unless ($uri =~ m|^[a-z]+://|) {
		#	if (IDX::PKI::Registry::inModPerl()) {
		#		my $cur_uri = $self->{'r'}->uri ();
		#		$cur_uri =~ s|/[^/]+$||;
		#		$uri = $cur_uri."/".$uri;
		#	}
		#}

		$self->{'printing_allowed'} = 1;

		$self->status (302);
		$self->header_out ("location", $uri);

		IDX::PKI::Registry::exit(0);
	} else {
		# View.
		return $self->_callView($page,@args);
	}
}


=pod "

=item I<_callView ($page, $scalar)>

The private methods implementing I<success()>, I<failure()> and
I<error()>.  Takes a URI in I<$page>, and sends it I<$scalar> through the
interface explained in L</Interface between the controller and the view>.
The external redirect case is not handled here (see
L</_choose_redirect_method()>), but the choice between programmatic and
PHP view is.

=cut "
sub _callView {
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS") if (scalar(@_) != 3);
	my ($self, $page, $arg) = @_;

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS")
		unless (UNIVERSAL::isa ($self, "IDX::PKI::Registry::CGIobject"));
	throw IDX::PKI::Error::Internal("INCORRECT_ARGS") if (ref($page));

	my $reqstring = IDX::PKI::Registry::requestString(%{$self->{subreq}});
	$self->bigparam ("DATA", $arg);

	if ((defined $page) && (! $self->_isAPI())) { ### PHP view
		if (IDX::PKI::Registry::inModPerl() && ($page !~ m/^http(s?):/)) {
			# Do internal redirect. Canonicalize URI first by adding
			# current dir.

			my $destURI;
			if ($page =~ m|^/|) {
				$destURI = $page;
			} else {
				$destURI = $self->{r}->uri();
				$destURI =~ s|/+[^/]*$|/$page|;
			}

			# Apache sub-requests don't work for PHP (the header is
			# not output). We use an internal redirect instead. This
			# solution is far from satisfactory, but it works.

			$self->{r}->method("GET");

			$self->{r}->internal_redirect("$destURI?$reqstring");
		} elsif (IDX::PKI::Registry::inIntegrationTest()) {
			print "CALL VIEW $page$CRLF$CRLF";
			print "HTTP_SERVER_VARS:$CRLF";
			while (my ($k, $v) = each %{$self->{subprocess_env}}) {
				print " REDIRECT_$k=$v$CRLF";
			}
			print "HTTP_ENV_VARS:$CRLF";
			while (my ($k,$v) = each %{$self->{subreq}}) {
				print " $k=$v$CRLF";
			}
			print $CRLF.IDX::PKI::Registry::phpencode ($arg).$CRLF;
		} else {
			# Perform an explicit client-GET-and-copy here. This is
			# used to keep IDX::PKI::Registry's MVC paradigm usable
			# outside Apache, but could be useful even from mod_perl -
			# e.g. to fetch a page from another virtual host.

			throw IDX::PKI::Error::Internal("UNIMPLEMENTED",
				"no proxy redirects possible yet");

		};
	} else { #### API (text) view
		$self->{printing_allowed}=1;
		$self->content_type("text/plain");
		$self->print($CRLF, $self->_programmaticViewOutput($arg), $CRLF);
	}

	IDX::PKI::Registry::exit(0);
}

=pod "

=item I<_isAPI()>

returns a boolean indicating whether we are called in API mode (and we
should output text) or in PHP mode (and we should forward data to a
PHP page so that it does the outputting by itself). Calling this from
user code is almost certainly a bad idea since it allows user code to
discriminate between the two situations and act differently in both
cases, which it shouldn't.

=cut "

sub _isAPI {
	my ($self)=@_;

	my $agent;
	if (IDX::PKI::Registry::inModPerl()) {
		$agent=$self->{r}->header_in("User-Agent");
	} elsif (IDX::PKI::Registry::inIntegrationTest()) {
		return undef;
	} else {
		throw IDX::PKI::Error::Internal("UNIMPLEMENTED",
			-details => "IDX::PKI::Registry does not support operating ".
			"outside mod_perl yet.");
	}

	return 1 if (!defined $agent);
	return ($agent =~ m/IDXPKI-API/);
}


=pod "

=item I<_programmaticViewOutput($arg)>

Returns the text that would be appropriate for returning as the
programmatic view (see L</Interface between the controller and the
view>), with $CRLF as a line separator but without the appropriate
Content-Type header. $arg is the argument that was (or would be)
passed to either L</success>, L</failure> or L</error>; I<undef> means
that the "redirecting success" is in action (see L</success>).

=cut "

# YES, I KNOW: there is only one place in the code where this function
# is used. But DO NOT SNAP IT IN PLACE! We call _programmaticViewOutput
# directly from tests (lest we must fork(), or fire up an Apache+mod_perl
# to do the tests, or whatever).
sub _programmaticViewOutput {
	my ($self, $arg) = @_;
	local $Data::Dumper::Terse = $Data::Dumper::Terse*0+1; # Shuts warning

	my $out = "";

	my $subprocess_env = IDX::PKI::Registry::inModPerl () ?
		$self->{"r"}->subprocess_env () : $self->{"subprocess_env"};
	while (my ($key, $value) = each %$subprocess_env) {
		if (ref ($value) eq "ARRAY") {
			foreach my $element (@$value) {
				$out .= "REDIRECT_$key=$element$CRLF";
			}
		} else {
			$out .= "REDIRECT_$key=$value$CRLF";
		}
	}
	$out .= $CRLF;

	while (my ($key,$value) = each %{$self->{subreq}}) {
		if (ref ($value) eq "ARRAY") {
			foreach my $element (@$value) {
				$out .= "$key=$element$CRLF";
			}
		} else {
			$out .= "$key=$value$CRLF";
		}
	}
	$out .= $CRLF;

	if (!defined $arg) {
		$out .= "SUCCESS$CRLF";
	} elsif (ref($arg)) {
		$out .= Data::Dumper::Dumper($arg);
	} else {
		$out .= $arg;
	}

	return $out;
}

=pod "

=back

=head2 Support Functions

The functions below are defined in the I<IDX::PKI::Registry> package.
They are not methods, and must be called as functions if at all.

=over 4

=cut "


package IDX::PKI::Registry;

use strict;

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Log;
use IDX::PKI::ValidateInput;

use vars qw($cgi $config $log @ISA @EXPORT $CRLF);
BEGIN {
	# Use a BEGIN block since IDX::PKI::Registry::CGIobject wants to import
	# those lexically above here, but *after* this gets run.

	@ISA=qw(Exporter); @EXPORT=qw($cgi $config $log $CRLF);
}



=pod "

=item I<inModPerl()>

Returns true if we are running under mod_perl, and false if running as
either a standalone script or in a test suite.

=cut "

sub inModPerl {
	return (UNIVERSAL::can("Apache","request") ? 1 : 0);
}

=pod "

=item I<inIntegrationTest()>

In scalar context, returns a true value if we are running a standalone
registry script from the command line, in order to unit-test it (see
L</Testing IDX::PKI::Registry scripts>). In array context,
returns an associative array (odd entries being keys and even entries
being values) of the values the developper specified on the command
line.

=cut "

sub inIntegrationTest {
	return undef if (inModPerl());
	return undef if ($ENV{GATEWAY_INTERFACE} &&
					 $ENV{GATEWAY_INTERFACE} =~ m/^CGI/);

	return 1 if ! wantarray;

	return map {m/^([^=]*)=(.*)$/s ? ($1, $2) : ()} @ARGV;
}


=pod "

=item I<phpencode()>

Usage example:

 my $string = phpencode($struct);

Encodes I<$struct> as a string that can be eval()ed from PHP to
reconstruct the data structure in I<$struct>.  Similar to B<Data::Dumper>,
but for PHP.  This function is for use by I<_callView()>, and not really
meant to be called from the B<IDX::PKI::Registry> script.

=cut "

sub phpencode {
	my ($struct)=@_;

	if (!defined $struct) {
		return 'NULL';
	} elsif (! ref($struct)) {
		return $struct
			if (($struct =~ m/^\d*$/) && (($struct + 0) eq $struct));
		$struct =~ s/\\/\\\\/g;
		$struct =~ s/'/\\'/g;
		return qq{'$struct'};
	} elsif ("ARRAY" eq ref($struct)) {
		return "array(".
			(join(",", map { phpencode($_)} @$struct)).
			")";
	} elsif ("HASH" eq ref($struct)) {
		return "array(".
			(join(",", map {"'$_'"."=>".(phpencode($struct->{$_}))}
				(keys %$struct))).
			")";
	} else {
		throw IDX::PKI::Error::Internal("Can't map data to PHP");
	};
}



=pod "

=item I<exit()>

Terminates processing of the controller script. Like in B<Apache>, it
does not necessarily mean that the process is going to be shut
down. This function is exported by default, but should not be called
except in scripts that do the output by themselves. The correct way of
exitting an MVC registry script is through L</success()>, L</error()>
or L</failure()>, lest no data be sent to the browser at all.

=cut "

sub exit {
	if (IDX::PKI::Registry::inModPerl()) {
		Apache::exit(@_);
	} else {
		CORE::exit(@_);
	};
}

=pod "

=item I<CGIescape($data)>

CGI-escapes data (appropriate for both keys and values) for passing
them into a query string. Code and semantics borrowed from L<CGI/escape>.

=cut "

sub CGIescape {
	shift() if ref($_[0]);
  my $toencode = shift;
  return undef unless defined($toencode);
	my $EBCDIC=("\t" ne "\011");
    if ($EBCDIC) {
		throw IDX::PKI::Error::Internal("UNIMPLEMENTED"); # Need a translation
		# map
    } else {
      $toencode=~s/([^a-zA-Z0-9_.-])/uc sprintf("%%%02x",ord($1))/eg;
    }
  return $toencode;

}

=pod "

=item I<requestString(@array)>

Returns the text that would be appropriate for passing as the request
string (the $reqstring in
"http://somewhere/some/path/somepage.php?$reqstring") for an internal
or external redirect, according to parameters set up in @array which
shall be a cast of a hash (odd cells of @array are keys and even cells
are values). This is nothing more than an iteration over L</CGIescape>.

=cut "

sub requestString {
	my @ret;
	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS") if
	  (@_ % 2);
	while(my ($k,$v)=splice @_,0,2) {
		push(@ret,(CGIescape($k)."=".CGIescape($v)));
	};
	return join("&",@ret);
}

=pod "

=item I<$CRLF>

The proper formula for terminating the line (identical to I<$CGI::CRLF>).
This variable is exported by default in the caller's namespace.

=cut "

if ($^O eq 'VMS') {
  $CRLF = "\n";
} elsif ("\t" ne "\011") { # EBCDIC
  $CRLF= "\r\n";
} else {
  $CRLF = "\015\012";
}


=pod "

=back

=head1 APPENDIX: APACHE HANDLER LOGIC

When used in mod_perl, I<IDX::PKI::Registry> becomes a subclass of
I<Apache::RegistryNG> so that it is possible to say, from the Apache
configuration file:

  PerlRequire IDX::PKI::Registry
  PerlHandler IDX::PKI::Registry->handler
  # And do not forget this (mandatory for IDX::PKI::Registry):
  PerlTaintCheck On

This will install I<IDX::PKI::Registry::handler> as the code
responsible for all documents that have an Apache handler set to
"perl-script" (either through C<AddHandler>, C<SetHandler> or
similar). The consequences of this setup are:

=over 2

=item *

controller scripts are compiled and evaluated on-demand, based on
their file timestamp - exactly like I<Apache::Registry> does;

=item *

the three imported global variables I<$cgi>, I<$log> and I<$config> are
reinitialized automatically upon each request;

=item *

any uncaught exception raised by the controller script is displayed using
IDX-PKI's legendary Green Screen Of Death;

=item *

upon compilation, I<IDX::PKI::Registry> does some extremely intrusive
and picky tests to check that the programmer did not invoke both
I<IDX::PKI::Registry> and some other CGI module such as "Apache::*" or
"CGI::*". On the other hand, if I<IDX::PKI::Registry> is not detected
as an imported Perl module (through "use", "require" or "import"),
then I<IDX::PKI::Registry> behaves identically to
I<Apache::RegistryNG>, meaning that the IDX-PKI supports both the old
I<IDX::PKI::CGI> API (or even I<Apache::Registry> or I<CGI>) and the
new I<IDX::PKI::Registry>, but not a mix of both.

=back

=head2 The Apache::PerlRun and Apache::RegistryNG API

The magic of I<IDX::PKI::Registry::handler> is done through
inheritance and method overloading on top of I<Apache::RegistryNG>,
which unfortunately has no documentation anywhere in the universe
(well, except perhaps in the mod_perl cookbook, which one must
purchase). The following insight was gathered through RTFS and RTFML.

L<Apache::PerlRun> says that this module is a wrapper to ``run CGI
scripts unaltered under mod_perl'', and then lists all the features it
misses w.r.t. L<Apache::Registry>. In fact it is much more than this;
I<Apache::PerlRun> is an object-oriented framework for dealing with
the problem of serving WWW pages made up from Perl scripts that reside
on-disk within the Apache file hierarchy (hereafter called "registry
scripts"). There is one registry script per URL (minus the request
parameters), and the normal Apache URI-to-file rules apply exactly as
usual (C<< <Directory> >> section, C<Alias>, C<mod_rewrite> and
whatnot); registry scripts often have a special extension such as
.cgi, but this is only a matter of Apache configuration.

I<Apache::PerlRun> is actually more powerful than I<Apache::Registry>;
the proof of this assertion is the module I<Apache::RegistryNG>, which
provides all features from I<Apache::Registry> by subclassing only two
methods from I<Apache::PerlRun>. Here are the description of the
interfaces of both classes.

=over 4

=item I<handler($r)>

Both a plain function and a class method; defined in
I<Apache::PerlRun>, and overloaded in I<Apache::RegistryNG> and nearly
any CPANian subclass thereof that we know of. This is the main
function, the very topmost one that is called when the control flow
reaches mod_perl in the content-handler phase of Apache's request
handling pipeline (see also the diagram at the I<run()> entry
below). If the Apache configuration file reads

   PerlHandler Some::Module::handler

then I<Some::Module::Handler($r)> will be called, with $r the request
object (see I<new> below); on the other hand, with

   PerlHandler Some::Module->handler

I<< Some::Module->handler($r) >> is called instead, which means that
the I<handler()> function must prepare for both calling conventions
(e.g. with or without a class name as its $_[0]).

In both cases, I<handler()> returns an HTTP error code, for which
L<Apache::Constants> provides symbolic equivalents. It hopefully has
side effects, such as exchanging HTTP data with the client (using
methods from I<$r> - see L<Apache>) and logging any errors.

I<handler()> does all the grunt work of calling the methods below in the
right order: instantiate an object with I<new()>, do the I<chdir_file()>s
(which are more of a CGI-like habit than a functional requirement, but
are so widely accepted that both I<Apache::PerlRun>
I<Apache::RegistryNG> does them without asking) manage the compilation
cache (as determined by I<should_compile()>, below), actually run the
scripts (by calling the I<run()> method), etc.

=item I<new($r)>

Class constructor, defined in I<Apache::PerlRun>. $r is the
C<request_record> object, the one that CApache->request()>
returns. This call creates and returns a new blessed object that
contains $r. It has no side effect.

=item I<chdir_file()>

=item I<chdir_file($dir)>

Method, defined in I<Apache::PerlRun>. Performs a chdir() of the
running server process. Without an argument, chdir()s to the directory
in which the script file resides (as determined from the $r object
that was passed at I<new()> time. With an argument $dir, changes
directory to $dir.


B<Note>: this method is merely a call to the undocumented method
I<< $r->chdir_file() >> from L<Apache> which has exactly the same
semantics.

=item I<namespace_from()>

Method, defined in I<Apache::PerlRun> and overloaded in
I<Apache::RegistryNG>. Should return a unique identifier appropriate
for making up a unique package name (see below); bizarre characters
are OK, since I<namespace()> below will escape them.

=item I<namespace()>

Method, defined in I<Apache::PerlRun>. Returns an appropriate package
name for running the registry script into, usually made up by escaping
the bad characters from the return value of I<namespace_from()>. This
method should not be overloaded in subclasses (it maintains a cache in
the internal data of the object); qubclass I<namespace_from()>
instead. On the other hand, calling this method from overloaded user
code is the preferred way of getting at said package name (for
example, to fill in the gap of the default values in I<sub_wrap()>
below).

=item I<can_compile()>

Method, defined in I<Apache::PerlRun>. Takes no argument and
determines whether the basic file checks allow compilation of this
request's script (is it a file? Is it executable? Does the parent
directory enjoy I<Option +ExecCGI> in Apache's configuration? etc.)
Returns an HTTP error code (see above), and logs failures
appropriately.

=item I<sub_wrap()>

=item I<sub_wrap($code,$package)>

Method, defined in I<Apache::PerlRun>. Given $code (the contents of
the registry script) and $package (), returns an appropriate text that
contains a I<handler()> function and stores it as a B<scalar
reference> (not a plain scalar) into $self->{sub} (which is a
behaviour one must reproduce in subclasses). This handler function
will be compiled, cached, and run in order to actually serve the
request; it will have the same interface as I<handler()> above
(although of course it will no longer be the topmost function - it
will be wrapped in I<IDX::PKI::RegistryNG>, which does the compiling
and caching). The return value of this method is ignored.

Both arguments are optional. $code resides in $self->{code} as a
B<scalar reference> (not a plain scalar), while $package can be
recovered by calling I<< $self->namespace() >>.

I<Apache::PerlRun::sub_wrap()> says "package $package;", imports
I<exit()> from L<Apache> and wraps up everything into a "sub handler
{...}". This is quite enough for plain Perl registry scripts (and for
I<IDX::PKI::Registry>), but overloading this method would come in
handy for hybrid languages such as EmbPerl or Mason that have a text
preprocessing stage that generates the registry scripts from the
source files.

=item I<compile()>

=item I<compile($subref)>

Method, defined in I<Apache::PerlRun>. Actually runs the formatted
script, passed as a scalar B<reference> ($subref or $self->{sub}) in
an "eval" block; returns an HTTP error code upon failure.

=item I<run()>

Method, defined in I<Apache::PerlRun>. Eventually runs the "sub
handler" whose source code was produced by I<sub_wrap()> above,
trapping any exceptions and turning them into "internal server
errors". Returns an HTTP error code. From a stacktrace point of view,
this function is the upper boundary of mod_perl-generic code when
executing registry scripts:

     Your code (as a "sub handler" function made up for you)
             |
   Apache::PerlRun::run
             |
    <various functions that do parsing, compiling, caching>
             |
   Apache::PerlRun::handler (in Perl)
             |
          mod_perl (written in C)
             |
  Apache content handler manager
             |
    Apache request pipeline
             |
      Apache child_main (network loop)
             |
           Apache

Of course Apache::PerlRun can be replaced by any of its subclasses in
the above diagram.

=item I<< $self->{r} >>

Object field that holds the request record (passed at I<new()> time).

=item I<< $self->{code} >>

The verbatim contents of the registry script, as a B<scalar reference>
(and not a mere scalar). This field may not be assumed present at all
times; it only exists during compilation.

=item I<< $self->{sub} >>

The formatted Perl text, deduced from $self->{code} through
I<sub_wrap()>, as a B<scalar reference> (and not a scalar).  Ditto as
regards the duration of the field.

=item I<< $Apache::Registry->{} >>

The eponymous meta-object (see L<perltootc/The Eponymous Meta-Object>)
that holds one entry per registry script ever seen by the mod_perl
registry framework. This is I<Apache::Registry>, not
I<Apache::PerlRun>, for historical reasons (think of it as an adoption
rather than an object filiation if you like). The keys of this hash
are the names of the registry objects (as returned by
$self->namespace()), and the values are anonymous hashes that in turn
contain one key, I<mtime>, and a value which is (you guessed it) the
last modification time of the underlying file. This meta-object
therefore implements the state table of the compilation cache.

=back

There is more to I<Apache::PerlRun> and I<Apache::Registry> than what
is explained above. However, this is sufficient for our purpose: all
methods overloaded or called by I<IDX::PKI::Registry> code are
documented.

=head2 IDX::PKI::Registry Apache handler implementation

When running under mod_perl, I<IDX::PKI::Registry> becomes a subclass
of I<Apache::RegistryNG> that overloads the following methods:

=over 4

=item I<run()>

Prior to calling I<Apache::RegistryNG::run> that does the actual
launching of the script, this method sets up the package-global
variables I<$cgi>, I<$config> and I<$log>, only if the current script
is indeed an I<IDX::PKI::Registry> script (as determined by I<<
$Apache::Registry->{}->{IDX::isRegistry} >>, see below). It also traps
exceptions, and displays either the Green Screen of Death or the
Burning Skull of Forbidden Access (and sets the HTTP error code
accordingly).

=cut "

package IDX::PKI::Registry;

sub run {
	my ($self,@runargs)=@_;

	my $retval=0; # HTTP OK - we hope.

	try {
		newreq() if ($Apache::Registry->{$self->namespace()}->
					 {"IDX::isRegistry"});

		my $coderef = UNIVERSAL::can($self->namespace(), "handler");

		throw IDX::PKI::Error::Internal ("Can't get handler sub",
										 '-code' => $self->{'code'},
										 '-sub'  => $self->{'sub'})
			unless (ref ($coderef) eq "CODE");

		&$coderef($self->{r},@runargs);
		# Scripts should not return: they should call ->success(), ->failure()
		# or ->error().

		$retval = $self->gsod (500, # internal server error
							   "Unterminated IDX::PKI::Registry script");

	} otherwise {
		my ($E) = @_;

		return if (!$E); # Apache::exit is really an exception thrower,
		                 # and we don't want to block nor GSOD this sort
		                 # of exceptions. Just let the control fall off
		                 # run() (we could rethrow, but Error.pm doesn't
                         # quite like this kind of ``false exceptions'').

		my $httpcode = $E->{-HTTPCode} || 503;  # service unavailable

		$log ||= new IDX::PKI::Log ();
		$log->alert ("ERROR", "-attachment" => $E);

		if ($httpcode == 403) {	# Permission denied.
			if (IDX::PKI::Registry::inModPerl()) {
				# I don't remember why this bugware is for, but I seem to
				# recall it fixes something w.r.t. the request error code
				# (for logging).
				$self->{r}->internal_redirect("/403.cgi");
			} else {
				die "UNIMPLEMENTED";
			}
		} else {
			$retval = $self->gsod ($httpcode, "Uncaught Exception",
								   IDX::PKI::Log::encode ($E));
		}
	};
	return $retval;
}

=pod "

=item I<sub_wrap()>

This function is overloaded to detect whether the script is an
I<IDX::PKI::Registry> script or not (by syntactic analysis -
rationale: parsing the unformatted source file will be easier now than
at I<compile()> time), and set I<<
$Apache::Registry->{}->{"IDX::isRegistry"} >> accordingly (see I<<
$Apache::Registry->{} >> above). It also performs picky source
analysis and may set I<< $self->{"IDX::pickyError"} >> to pass on to
I<compile()> below.

=cut "

sub sub_wrap {
    my $self=shift;

	if (${$self->{code}} =~ m/^\s*use\s*IDX::PKI::Registry/m) {
		$Apache::Registry->{$self->namespace()}->{"IDX::isRegistry"}=1;
		$Apache::Registry->{$self->namespace()}->{"IDX::isRegistry"}=1;
		   # Shuts warning

		if (${$self->{code}} =~ m/^\s*use\s*(\S.*?CGI)/m) {
			$self->{"IDX::pickyError"}="Module $1 is incompatible with IDX::PKI::Registry";
		};
	};

	return $self->SUPER::sub_wrap(@_);
}

=pod "

=item I<compile()>

This function is overloaded to raise the $self->{"IDX::pickyError"},
if any, set up by I<sub_wrap()> (according to L</The Apache::PerlRun
and Apache::RegistryNG API>, I<sub_wrap()> cannot signal errors by
itself).

=cut "

sub compile {
	my $self = shift;
	return $self->gsod (503, "Registry script compilation failed",
						$self->{"IDX::pickyError"}) if
							($self->{"IDX::pickyError"});
	return $self->SUPER::compile(@_);
}

=pod "

=back

In addition, the following custom functions are added (do NOT call
them from user code):

=over 4

=item I<newreq()>

Initializes I<$IDX::PKI::Registry::cgi>,
I<$IDX::PKI::Registry::config> and I<$IDX::PKI::Registry::log> with
fresh values. Called by run() under mod_perl, and at BEGIN time in
standalone mode.

=cut "

sub newreq {
	$config = the IDX::PKI::Config ();
	$log = new IDX::PKI::Log ();
	$cgi = _new IDX::PKI::Registry::CGIobject ($log);
}

BEGIN {
	if (IDX::PKI::Registry::inModPerl()) {
		require Apache::RegistryNG;
		our @ISA;
		push(@ISA,qw(Apache::RegistryNG));
	} elsif (caller(2)) {
		newreq();
	}; # Third case is for the test suite; do nothing.
}

=pod "

=item I<gsod($httpcode, $title, $display)>

Dumps an exception or other fatal error onto the user's screen: This is
known as the Green Screen Of Death (hence the method name). Returns
I<$httpcode>, for the following idiom (in some registry function):

 return $self->gsod(...);

Returning I<$httpcode> is mandatory, by the way: This is because I<gsod()>
is implemented on top if L<Apache/custom_response>, and the error code
still needs to make it up to the handler's return value. If said return
value is not the same as I<$httpcode>, this will result in a C<document
contains no data> browser message.

I<gsod()> should I<not> be called directly from inside a registry
script, since it will not do what you want. Better raise an exception.

The GSOD also has an API-friendly version (see L<_isAPI()>).

=cut "

sub gsod {
	my ($self, $httpcode, $title, $display) = @_;
	$display = "" unless (defined $display);
	my @jokes=("<i><p>Murphy's Law: If anything can go wrong, it will.</p>".
			   "<p>Murphy's Corollary: Left to themselves, things tend to".
               " go from bad to worse.</p>".
			   "<p>Murphy's Corollary: It is impossible to make anything".
               " foolproof because fools are so ingenious.</p></i>",
			   "<i>Ginsberg's Restatement of the Three Laws of Thermodynamics:".
			   " You can't win. You can't break even. You can't quit.</i>",
			   "<i>Lowery's Law: If it jams - force it. If it breaks, it needed".
			   " replacing anyway.</i>",
			   "<i><p>Given humanity's track record with languages, you wonder".
			   " why we bother with standards committees</p>".
			   "<p>-- Marcus Leech</p></i>",
               "<i><p>No standard or clause in a standard has a divine right".
			   " of existence</p>".
			   "<p>-- A Microsoft PKI architect explaining Microsoft's".
			   " position on standards compliance.</p></i>",
			   "<i><p>When the X.500 revolution comes, your name will be lined".
			   " up against the wall and shot</p><p>-- John Gilmore</p></i>",
			   "<i><p>[X.500 Names] They can't be read, written, assigned, ".
			   "or routed.  Other than that, they're perfect</p>".
			   "<p>-- Marshall Rose</p></i>",
			   "<i>Anyone who has had to work with X.509 has probably ".
			   "experienced what can best be described as ISO water torture.".
			   "</i></p><p>-- Peter Gutmann</p>",
			  );
	# Huh, mod_perl has bogus random?
	srand(time ^ ($$ + ($$<< 15)));
	my $joke=@jokes[int(rand(@jokes))];

	my ($message, $conttype);
	if ($ENV{"HTTP_USER_AGENT"} && $ENV{"HTTP_USER_AGENT"} !~ m/IDXPKI-API/) {
		$conttype = "text/html";
		$message = <<"GREEN_SCREEN_OF_DEATH";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
	"http://www.w3.org/TR/1999/REC-html401-19991224/loose.dtd">
<html>
  <head>
    <title>IDX-PKI: $title</title>
  </head>
  <body bgcolor="#00ff00">
    <h1><font color="#ffff00">$title</font></h1>
    <pre>$display</pre>
    $joke
  </body>
</html>
GREEN_SCREEN_OF_DEATH
	} else {
		$conttype = "text/plain";
		$message = << "API_FRIENDLY";
$title

$display

-9999
API_FRIENDLY
	}
	$message =~ s/\n/$CRLF/gs;

	if (IDX::PKI::Registry::inModPerl()) {
		$self->{"r"}->custom_response ($httpcode, $message);
		$self->{"r"}->content_type ($conttype);	# Does not work, Apache still
		# says text/html :-(
	} else {
		die "UNIMPLEMENTED";
	}

	return $httpcode;
};


=pod "

=back

=head1 SEE ALSO

On the Model/View/Controller paradigm:

http://java.sun.com/blueprints/patterns/j2ee_patterns/model_view_controller/

http://c2.com/cgi/wiki?ModelViewController

=cut "

1;

eval(join '',<main::DATA>) || die $@ unless caller();

__END__
############################### TEST CASE ################################

package IDX::PKI::Registry;

use IDX::Test;
use IDX::PKI::Error;
use Taint qw (taint tainted);

my $cgi;

number_of_tests (13);

test "PHP encoding of data structures" => sub {
	TEST( (phpencode([41,42,"zap",{},undef])."\n") eq <<'VERBATIM');
array(41,42,'zap',array(),NULL)
VERBATIM

	my $ref=[1,2,{},{"three"=>3},"03","ldkfjd","O'Connell"];
	TEST((phpencode($ref)."\n") eq <<'VERBATIM');
array(1,2,array(),array('three'=>3),'03','ldkfjd','O\'Connell')
VERBATIM
};

test "Mandatory ordering on methods -- PHP page view" => sub {
	$cgi = _new IDX::PKI::Registry::CGIobject (undef, {});

	try {
		$cgi->authorize("__NONE__");
		warn "Should have died (1)";
		TESTFAIL();
	} catch IDX::PKI::Error::Internal with {
		TESTOK();
	};

	$cgi = _new IDX::PKI::Registry::CGIobject (undef, {});
	$cgi->setupViews("somewhere.php");

	try {
		$cgi->validate("someparam",qr/^(.*)$/);
		warn "Should have died (2)";
		TESTFAIL();
	} catch IDX::PKI::Error::Internal with {
		TESTOK();
	};

	$cgi->authorize ("__NONE__");
	$cgi->validate ("someparam", qr/^(.*)$/);
};

test "Mandatory ordering on methods -- API view" => sub {
	$cgi = _new IDX::PKI::Registry::CGIobject (undef, {});

	try {
		$cgi->authorize ("__NONE__");
		warn "Should have died (1)";
		TESTFAIL();
	} catch IDX::PKI::Error::Internal with {
		TESTOK();
	};

	$cgi = _new IDX::PKI::Registry::CGIobject (undef, {});
	$cgi->setupViews ("__NONE__");

	try {
		$cgi->validate ("someparam", qr/^(.*)$/);
		warn "Should have died (2)";
		TESTFAIL();
	} catch IDX::PKI::Error::Internal with {
		TESTOK();
	};

	$cgi->authorize ("__NONE__");
	$cgi->validate ("someparam", qr/^(.*)$/);
};

test "tainting" => sub {
	my $tainted = "";
	taint ($tainted);
	my $taint_checking = tainted ($tainted);
	if (! $taint_checking) {
		warn "Taint test skipped - rerun with perl -T -I \$PERL5LIB\n";
		TESTOK();
	} else {
		$cgi = _new IDX::PKI::Registry::CGIobject (undef, {
			'TELEPHONE'=>'01 23 45 67 89',
			'AGE'=>'twenty-three',
			'EVILPARAM' => 'zoinxzoinxzoinxzoinxzoinxzoinx/bin/sh'});
		$cgi->setupViews("somewhere.php");
		$cgi->authorize("__NONE__");

		try {
			$cgi->validate("TELEPHONE",sub {return shift});
			warn "Validate accepted to return a tainted value";
			TESTFAIL();
		} catch IDX::PKI::Error::Internal with {
			my $E = shift;
			TEST ($E =~ m/taint/i) || warn "$E";
		};
	};
};

# There is a really strange bug if a parameter contents is numeric... This
# test may make all others fail with a "Argument <the param value> isn't
# numeric in bitwise and (&)" message from Taint. A numeric parameter make
# all ulterior validations replace the value with "0".

test "tainting in test mode (regression test)" => sub {
	my $tainted = "";
	taint ($tainted);
	my $taint_checking = tainted ($tainted);
	if (! $taint_checking) {
		warn "Taint test skipped - rerun with perl -T -I \$PERL5LIB\n";
		TESTOK();
	} else {
		$cgi = _new IDX::PKI::Registry::CGIobject (undef, {
			"string" => "blah",
			"numeric" => 1,
			"otherstring" => "zoinx",
			"greaternum" => 25,
		});
		$cgi->setupViews("somewhere.php");
		$cgi->authorize("__NONE__");

		my $val = $cgi->validate ("string", sub {
			return ($_[0] =~ m/^([a-z]+)$/ ? $1 : undef);
		});
		TEST ($val eq "blah") || warn "Lost string: '$val'!";
		$val = $cgi->validate ("numeric", sub {
			return ($_[0] =~ m/^([0-9]+)$/ ? $1 : undef);
		});
		TEST ($val eq "1") || warn "Lost numeric: '$val'!";
		$val = $cgi->validate ("otherstring", sub {
			return ($_[0] =~ m/^([a-z]+)$/ ? $1 : undef);
		});
		TEST ($val eq "zoinx") || warn "Lost otherstring: '$val'!";
		$val = $cgi->validate ("greaternum", sub {
			return ($_[0] =~ m/^([0-9]+)$/ ? $1 : undef);
		});
		TEST ($val eq "25") || warn "Lost greaternum: '$val'!";
		print Data::Dumper::Dumper ($cgi->{"subreq"}) if (TESTFAILED);
	};
};

test "basic CGI.pm functionality" => sub {
	$cgi = _new IDX::PKI::Registry::CGIobject (undef, {
		'TELEPHONE'=>'01 23 45 67 89',
		'AGE'=>'twenty-three',
		'EVILPARAM' => 'zoinxzoinxzoinxzoinxzoinxzoinx/bin/sh'});
	$cgi->setupViews("somewhere.php");
	$cgi->authorize("__NONE__");

	TEST($cgi->validate("TELEPHONE",sub {
							join("",m/([0-9])/g)
						}) eq
		 '0123456789') ||
		   warn "Incorrect validation on TELEPHONE";
	TEST(!defined $cgi->validate("AGE",qr/^([0-9]*)$/)) ||
	  		   warn "Incorrect validation on AGE";
	TEST(!defined $cgi->validate("SEX",qr/^(m|f)/i,-MISSING=>"no sex?!"))  ||
	  		   warn "Incorrect validation on SEX";

	TEST($cgi->validateFailures() == 2) ||
		warn "Incorrect validation failure count (part 1)";

	TEST("5 feet 6 inches" eq $cgi->validate("SIZE",qr/^((\w| )+)/i,
								 -default=>"5 feet 6 inches")) ||
								   warn "Incorrect validation on SIZE";
	TEST(!defined $cgi->validate("PREFERRED_BEVERAGE",qr/^((\w| )+)/i,
								 -default=>undef))  ||
		warn "Incorrect validation on PREFERRED_BEVERAGE";

	TEST ($cgi->validate ("EMPTYPARAM", qr/./, '-default' => "blah") eq
		  'blah') || warn "Can't validate an inexistant param with -default";

	TEST($cgi->validateFailures() == 2) ||
		warn "Incorrect validation failure count (part 2)";

	$cgi->param("NOSUCHPARAMETER",
				"in query, but I decide there should be one anyway.");
	$cgi->internalparam("HELLO_VIEW","how's the PHP today?");
};

test "Status" => sub {
	my $othercgi = _new IDX::PKI::Registry::CGIobject (undef, {});
	$othercgi->setupViews("somewhere.php");
	$othercgi->authorize("__NONE__");
	$othercgi->{'printing_allowed'} = 1;
	TEST ($othercgi->status (302)) || warn "status() doesn't seem to work";
};

use IO::Pipe;
use POSIX;

test "CGI output correct and complete" => sub {
	local $_=$cgi->_programmaticViewOutput("AZERTY");

	TEST(m/AZERTY/) || warn "Failed subtest 1";
	TEST(m/TRUSTED_TELEPHONE.*0123456789/) || warn "Failed subtest 2";
	TEST(m/ORIG_TELEPHONE.*01 23 45 67 89/) || warn "Failed subtest 3";
	TEST(m/^HELLO_VIEW/m) || warn "Failed subtest 4";
	TEST(! m/TRUSTED_AGE/) || warn "Failed subtest 5";
	TEST(m/ORIG_AGE/) || warn "Failed subtest 5 bis";
	TEST(m/ERROR_AGE=INVALID/) || warn "Failed subtest 6";
	TEST(m/ERROR_SEX=no sex/) || warn "Failed subtest 7";
	my @evillines = ($_ =~ m/^(.*EVIL.*)$/mg);
	@evillines = grep {m/^TRUSTED_/} @evillines;
	TEST(! @evillines) || warn "We have TRUSTED_ evil lines";
	@evillines = ($_ =~ m|^(.*/bin/sh.*)$|mg);
	@evillines = grep {m/^TRUSTED_/} @evillines;
	TEST(! @evillines) || warn "Failed subtest 9";
	TEST(m/^TRUSTED_NOSUCHPARAMETER/m) || warn "Failed subtest 10";
	TEST(m/^TRUSTED_SIZE/m) || warn "Failed subtest 11";
	TEST (m/TRUSTED_EMPTYPARAM.*blah/) ||
		warn "validate didn't set TRUSTED_EMPTYPARAM";
};

test "validate" => sub {
	$cgi = _new IDX::PKI::Registry::CGIobject(undef, {
		'TELEPHONE'=>'01 23 45 67 89',
		'AGE'=>'23 year old'});
	$cgi->setupViews("nowhere.php");
	$cgi->authorize("__NONE__");

	# Normal calls: args exist, checks are OK.
	TEST($cgi->validate('AGE',qr(([0-9]*))) == 23);
	TEST($cgi->validate('TELEPHONE',sub {
		local $_=shift;
		s/\D//g;
		m/^(\d+)$/ && return $1;}) eq "0123456789");
	TEST(! $cgi->validateFailures());
	TEST(! $cgi->{subreq}->{"ERROR_AGE"});
	TEST(! $cgi->{subreq}->{"ERROR_TELEPHONE"});

	# Validation of a nonexistent parameter
	$cgi = _new IDX::PKI::Registry::CGIobject(undef, {
		'TELEPHONE'=>'01 23 45 67 89',
		'AGE'=>'23 year old'});
	$cgi->setupViews("nowhere.php");
	$cgi->authorize("__NONE__");

	TEST(!defined $cgi->validate('NOSUCHPARAM',qr{(.)}));
	TEST($cgi->{subreq}->{"ERROR_NOSUCHPARAM"} eq "MISSING");
 	TEST($cgi->validateFailures() == 1);

	# Validation of an invalid parameter
	TEST(!defined $cgi->validate('TELEPHONE',qr/^([0-9]+)$/));
	TEST($cgi->{subreq}->{"ERROR_TELEPHONE"} eq "INVALID");
 	TEST($cgi->validateFailures() == 2);


	# When specifying a -else tag, $cgi->validateFailures() acts
	# as usual. Calling of $cgi->return() is also tested.
	$cgi = _new IDX::PKI::Registry::CGIobject (undef, {
		'TELEPHONE'=>'01 23 45 67 89',
		'AGE'=>'23 year old'});
	$cgi->setupViews("nowhere.php");
	$cgi->authorize("__NONE__");

	try {
		my $returned;
		local *IDX::PKI::Registry::CGIobject::error=sub {
			throw IDX::PKI::Error::Internal(-TESTval=>$_[1])
		};
		$cgi->validate('TELEPHONE',qr/^([0-9]+)$/,-else=>-1);
		TESTFAIL;
	} catch IDX::PKI::Error::Internal with {
		my $E=shift;
		TEST($E->{-TESTval} == -1) || warn "$E";
	};

	TEST($cgi->{subreq}->{"ERROR_TELEPHONE"} eq "INVALID");
	TEST($cgi->validateFailures());

	# Notice that the regexp does not match the default.
	TEST($cgi->validate('NAME',qr(([a-z]*)),-default=>"Anne-Laure") eq "Anne-Laure");


	# -else code should also be called for nonexistent parameters
	try {
		$cgi->validate('ZOINX',qr(([0-9]*)),-else=>sub {die "PSYCHE"});
		warn "didn't die";
		TESTFAIL;
	} otherwise {
		TEST(shift =~ m/PSYCHE/); # Error adds a \n at the end, go figure
	};


};

test "Validate with a sub and extra params" => sub {
	$cgi = _new IDX::PKI::Registry::CGIobject (undef, {
		'TELEPHONE'=>'01 23 45 67 89',
		'AGE'=>'23 year old'});
	$cgi->setupViews("nowhere.php");
	$cgi->authorize("__NONE__");

	my $result = $cgi->validate ('TELEPHONE', sub {my ($val, @args) = @_;
												   return join ('', @args);},
								 '-extraargs' => ["0", "1", "2", "3"]);
	TEST ($result eq "0123") || warn "Result: $result";
	TEST (! $cgi->validateFailures());
	TEST (! $cgi->internalparam ("ERROR_AGE"));
	TEST (! $cgi->internalparam ("ERROR_TELEPHONE"));
};

test "print() forbidden in the general case" => sub {
	try {
		$cgi->print("blah");
	} catch IDX::PKI::Error::Internal with {
		TESTOK();
	};
	0;
};

test "HTML-generating functions are unavailable"=> sub {
	try {
		TEST(!defined ($cgi->h1("zap")));
	} otherwise { TESTOK(); };
	0;
};

test "inIntegrationTest" => sub {
	local @ARGV=qw(arg1=val1 arg1=val2 arg2=val3);

	TEST(scalar inIntegrationTest());
	my %map=inIntegrationTest();
	TEST($map{arg1} eq 'val2');

	my $cgi = _new IDX::PKI::Registry::CGIobject();
	$cgi->setupViews("nowhere.php");
	$cgi->authorize("__NONE__");
	TEST("val3" eq $cgi->validate("arg2" => qr/^(.*)$/));
};

1;
