#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: SysV.pm,v 1.8 2003/10/28 14:42:55 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/PKI/SysV.pm,v $

=pod "

=head1 NAME

IDX::PKI::SysV - System V start/stop scripts in IDX-PKI

=head1 SYNOPSIS

 require IDX::PKI::SysV; # NOT "use IDX::PKI::SysV".

 sub start {}

 sub stop {}

 sub status {}

=head1 DESCRIPTION

B<IDX::PKI::SysV> is a package to build SysV-style init scripts in
Perl (because it is so much easier to get portable Perl code than
shell). The resulting script can be launched with arguments such
as B<start>, B<stop> and B<status>.

=head1 EXPECTED INTERFACE

Contrary to what happens in other modules, B<you>, the programmer, get
to follow the spec! You must define functions I<start()>, I<stop()>
and I<status()> respecting the following standards:

=over 4

=item I<start()>

Starts the daemon or service. Takes no argument; the return value is a
boolean indicating success (except that undef means "I do not know", in
which case status() will be used instead).

=item I<stop()>

Stops the daemon or service. API is the same as start().

=item I<status()>

Checks for up-ness of the daemon or service. Must return a boolean value.

=item I<$procname>

The name of the process, daemon or service (for informational messages)

=back

I<$procname> can be omitted and defaults to C<service>. The I<start()>,
I<stop()> and I<status()> functions are mandatory.

=cut "

our $procname="service";

=pod "

=head1 SUPPORT FUNCTIONS

The following functions will be imported into your namespace to help
writing the required functions above:

=over 4

=item I<verbosesystem()>

Same as Perl's I<system()>, except that if the C<DEBUG> environment
variable is set the command line will be warn()ed beforehand.

=item I<verboseexec()>

Ditto w.r.t. Perl's I<exec()>.

=cut "

%ENV = ("PATH" => "/bin:/usr/bin",
		($ENV{"DEBUG"} ? ("DEBUG" => 1) : ()));

sub verbosesystem {
	warn join(" ", (map {m/\s/ ? qq'"$_"' : $_ } @_),"\n") if
	  ($ENV{DEBUG});
	return system(@_);
}

sub verboseexec {
	warn join (" ", (map {m/\s/ ? qq'"$_"' : $_} @_), "\n")
		if ($ENV{'DEBUG'});
	exec (@_);
}

=pod "

=item I<waitstatus($expected)>

Waits for the service up-ness to become I<$expected>, which is a boolean,
by calling I<status()> several times. It returns true if the service goes
up (or down), or abandons (returning undef) after some time.

=cut "

sub waitstatus {
	my ($expected)=@_;

	sleep 1;

	for(my $i=0; $i<8; $i++) {
		return 1 if (status() xor (!$expected));
		sleep 1;
	};
	return undef;
}

=pod "

=back

=head1 INTERNAL FUNCTIONS IN PACKAGE IDX::PKI::SysV

None of the functions below should be called directly.

=over 4

=item I<usage()>

Prints a terse usage message and exits.

=cut "

sub IDX::PKI::SysV::usage {
	warn <<"USAGE";
Usage: $0 [start | stop | restart | status]
USAGE
	exit(2);
}

=pod "

=back

The module also contains top-level code (not in a function) that does
the command line parsing and launches the relevant functions.

=cut "


END {
	our $WE_ARE_IN_THE_TEST_SUITE;
	return 1 if ($WE_ARE_IN_THE_TEST_SUITE);

my $failures;

unless ($ARGV[0] && $ARGV[0] =~ m/^(start|stop|restart|status)$/) {
	if (my $meth=UNIVERSAL::can(__PACKAGE__,$ARGV[0])) {
		exit(! &$meth());
	} elsif (my $usage=UNIVERSAL::can(__PACKAGE__,"usage")) {
		exit(&$usage());
	} else {
		IDX::PKI::SysV::usage();
	};
};

if ( ($ARGV[0] eq "restart" || $ARGV[0] eq "stop") ) {
	print "Stopping $procname... ";

	if (status() && stop() && waitstatus(0)) {
		print "\033[60G[  \033[1;32mOK\033[0;39m  ]\n";
	} else {
		print "\033[60G[\033[1;33mPASSED\033[0;39m]\n";
		$failures++;
	};
};

if ($ARGV[0] eq "status") {
	my $name = `basename $0`;
	chomp($name);
	if (status()) {
		print "$name is up\n";
		exit 0;
	} else {
		print "$name is down\n";
		exit 1;
	};
};

if ( $ARGV[0] =~ m/start$/ ) {
	$failures=0; # Restart cancels failed stop.
	print "Starting $procname... ";
	if ((!status()) && start() && waitstatus(1)) {
		print "\033[60G[  \033[1;32mOK\033[0;39m  ]\n";
	} else {
		print "\033[60G[\033[1;33mFAILED\033[0;39m]\n";
		$failures++;
	};
}

exit ($failures ? 1 : 0);

}

########## Yes, even this module has a test suite!

eval join('',<main::DATA>) || die $@ unless caller;

1;

__END__

use IDX::Test;

package main;

my $file=new IDX::Test::Filename();

sub start {
	verbosesystem("sleep 3 && touch $file &");
	return ( (-f $file) ? 0 : 1);
}

sub stop {
	verbosesystem("sleep 3 && rm $file &");
	return ( (-f $file) ? 1 : 0);
}

sub status {
	-f $file;
}

number_of_tests(3);

test "waitstatus() after start()" => sub {
	TEST(start()) || warn "start failed";
	TEST(waitstatus(1)) || warn "waitstatus failed";
};

test "waitstatus() after stop()" => sub {
	TEST(stop()) || warn "stop failed";
	TEST(waitstatus(0)) || warn "waitstatus failed";
};

test "full-fledged script" => sub {
	my $script=new IDX::Test::Filename();
	open(SCRIPT,">",$script);
	my $inc = "qw (".join (" ", @INC).")";
	print SCRIPT <<"SCRIPT";
#!/usr/bin/perl -Tw

use strict;
use lib $inc;
require IDX::PKI::SysV;

sub start {
	verbosesystem("sleep 3 && touch $file &");
	return ( (-f "$file") ? 0 : 1);
}

sub stop {
	verbosesystem("sleep 3 && rm $file &");
	return ( (-f "$file") ? 1 : 0);
}

sub status {
	(-f "$file");
}

sub test {
     1;
}

sub fail {
     0;
}

SCRIPT
    close(SCRIPT); chmod(0755 => $script);

# We shall trust caller's PERL5LIB to find us, lest the test fails in
# the harness.

    local $ENV{DEBUG}="1";
    unlink($file);
    TEST(! system $script,"start") || warn "start: failed";
    TEST(! system $script,"stop") || warn "stop: failed";
    TEST(! system $script,"restart") || warn "restart: failed";
    TEST(system $script,"zoinx") || warn "zoinx: unexpected success";
    TEST(system $script,"fail") || warn "fail: unexpected success";
    TEST(! system $script,"test") || warn "test: failed";
};

our $WE_ARE_IN_THE_TEST_SUITE=1;

1;
