#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::PKI::Transport> - IDX-PKI's multiprotocol transport layer.

=head1 SYNOPSIS

 # Receive messages from a spool queue.

 my $transport_in = new
 	IDX::PKI::Transport::Unspooler("ThisSection");

 foreach my $id ($transport_in->list()) {
 	push @objs, $transport_in->readByID($id,
 		'Some::Class', 'Other::Class' =>
 		sub {myconstructor(shift)});
 }

 # Receive with intent of removing from queue
 my $obj = $transport_in->pullByID($id, 'Some::Class',
 	'Other::Class');

 # Commit all pulls
 $transport_in->expunge();

=cut "


#########################
## IDX::PKI::Transport ##
#########################
package IDX::PKI::Transport;
use IDX::PKI::Error;
use MIME::Base64;
use strict;

=pod "

=head1 FUNCTIONS AND METHODS

=head2 Support functions

All functions below are in package I<IDX::PKI::Transport>. They are
not object-oriented.

=over 4

=item I<unique()>

unique() returns a pseudo-random string. The important feature is that it
changes often; since it is only used for transient filenames and IDs, it has
no cryptographic implication (thus bogus random provided by rand() is
enough).

The resulting string matches the regexp /^[A-Za-z0-9+]+$/, which is
vaguely a subset of Base64.

=cut "

use vars qw($unique_counter);
$unique_counter = 0;
sub unique {
	my $p = sprintf("%d%d%d%d%d%d", rand(100),$$,$unique_counter++,
			rand(100),$$,$unique_counter++);
	my $hash = substr(crypt($p, $p), 2);

	$hash =~ s/[^A-Za-z0-9]/x/g; # We loose a bit of entropy but
	# we don't care.
	return $hash;
}


=pod "

=item I<encode_message($obj)>

Computes a string representation of $obj, which must be a blessed
reference in a class that has a serialize() method. $obj->serialize()
is called, and the resulting string is encoded in - well, in a way
that nobody should ever need to know except developpers of this
module. (to give a small hint: there is the word "Base" in it :-)

=cut "

sub encode_message {
	no strict 'refs';
	my (undef, $msg) = @_;

	throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS")
		unless(defined($msg));

	throw IDX::PKI::Error::Internal
		("Trying to send a non-serializable message")
			unless (UNIVERSAL::can($msg, 'serialize'));

	# Serialize the object. ref($msg) is the textual representation of
	# its type.
	my $version = ${ref($msg)."::VERSION"};
	my $message = "--BEGIN IDXPKI MESSAGE--\n".
	  "Type: ".ref($msg).(defined $version ? " $version":"")."\n".
			MIME::Base64::encode($msg->serialize()).
					"--END IDXPKI MESSAGE--\n";
}

=pod "

=item I<decode_message(undef,$message,@classes)>

Decodes a message encoded with L</encode_message>, by calling
constructors or user-specified functions upon $message (which is a
string). Returns a reference to an object of the appropriate type
(hopefully).

@classes is a list of class names, or class names followed by one
reference to a function that takes one argument. The first class
mentioned in @classes that is a superclass of the class advertised in
I<$message> is considered: if it is followed by a function reference,
then this function is called with $message as the argument; else the
C<deserialize()> constructor is called in the class advertised in
$message.  In both cases, the result is the return value of
C<decode_message()>.

To make things maybe a little bit clearer, here are some
examples. Suppose we have the following class diagram:

        Ancestor::Class (defines deserialize())
             |
             |
        Descendant::Class (inherits deserialize())
             |
             |
        Grandchild::Class (overloads deserialize())

and $ancestormessage, $descendantmessage, $grandchildmessage are the
results of calling L</encode_message> on appropriate class members. Then:

=over 4

=item *

Both following expressions will call Ancestor::Class::serialize() upon
$ancestormessage:

    my $obj=$transport->decode_message($ancestormessage,
         "Ancestor::Class");

    my $obj=$transport->decode_message($ancestormessage,
         "Some::Irrelevant::Class","Ancestor::Class");

=item *

The following expression will call Grandchild::Class::deserialize() upon
$grandchildmessage:

    my $obj=$transport->decode_message($grandchildmessage,
         "Ancestor::Class");

=item *

The following expression will raise an exception:

    my $obj=$transport->decode_message($ancestormessage,
         "Descendant::Class","Grandchild::Class");

=item *

The following expression will return "foo":

    my $should_be_an_obj_but_isnt=
        $transport->decode_message($grandchildmessage,
         "Descendant::Class"=> sub {"foo"},
         "Grandchild::Class"=> sub {"bar"});

=item *

The following expression has no meaning:

    my $obj=$transport->decode_message($ancestormessage,
         "Descendant::Class",sub {"foo"}, sub {"bar"});

=back

=cut "

sub decode_message {
	my (undef, $message, @classes) = @_;
	$message =~ m/
		^ --BEGIN\ IDXPKI\ MESSAGE-- $ .*?  # There we are...
		^ Type:\s+? (\S*?)(?:\ (\S*?))? $   # The type and version number
		(.*?)                          # Some text in between, but this one is
					                   # of interest to us.
		^ --END\ IDXPKI\ MESSAGE-- $   # Done!
		/xsm ||
			throw IDX::PKI::Error::Network
			  ("MALFORMED_MESSAGE", -message => $message);

	my ($type,$version,$blobencoded)=($1,$2,$3);
	my @oclasses = @classes;
	my $blob = MIME::Base64::decode($blobencoded);

	{
		no strict "refs";
		throw IDX::PKI::Error::Network("MALFORMED_MESSAGE",
									   -details=>"Unknown class in message (maybe not loaded?)",
									   -class=>$type) unless
				(keys %{$type."::"});
	};

	my $emptyobj=bless {},$type;

	CLASS: while (@classes) {
		my $class = shift @classes;
		# Here there be crude Perl hacks!

		next CLASS unless UNIVERSAL::isa($emptyobj,$class);
		my $nextarg = $classes[0];
		if ((ref $nextarg) && ((ref $nextarg) eq 'CODE')) {
		  shift @classes;
		  return &$nextarg($blob);
		} else {
		  my $constr = UNIVERSAL::can($emptyobj,"deserialize");
		  next CLASS if (! defined $constr);
		  return &$constr($type,$blob);
		};
	};
	throw IDX::PKI::Error::Network ("MALFORMED_MESSAGE",
		'-type' => $type, '-classes' => \@oclasses, '-message' => $blob);
}

=pod "

=item I<write_spoolx($message,$dir)>

Write text $message to spool directory $dir, with focus on UNIX
semantics (e.g. don't terminate, but throw an exception, if anything
went wrong). Files are created with mode 0000, and chmoded to 0700
when they are done writing. This way, a concurrent reader will not
attempt to read half-transfered messages.

=cut "

sub write_spoolx {
	my ($message, $directory) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS")
		unless (defined ($message) && $directory);
	$message = encode_message (undef, $message) if (ref $message);

	use IO::Handle;
	use File::Sync;
	use Fcntl;
	use Errno qw(EEXIST);

	local *FILE;
	my $name;
	while () {
		$name = unique();
		sysopen (FILE, "$directory/$name",
				 O_WRONLY|O_CREAT|O_EXCL|O_SYNC, 0000) && last;
		throw IDX::PKI::Error::IO
			("Error opening", -IOFile => "$directory/$name")
				unless ($! == EEXIST);
	}

	my $io = new IO::Handle;
	($io and
	 $io->fdopen(fileno(FILE),"w") and
	 print FILE ($message) and
	 $io->flush() and
	 $io->fsync() and
	 close(FILE)) ||
		throw IDX::PKI::Error::IO
		("Error writing", -IOFile => "$directory/$name");

	chmod 0700, "$directory/$name" ||
		throw IDX::PKI::Error::IO
		("Error chmoding", -IOFile => "$directory/$name");
	return $name;
}

1;

=pod "

=back

=head2 B<IDX::PKI::Transport::Sender> class

This is a multiplexer class, in that it only contains a constructor that
returns an object in a subclass, determined at runtime (to make things
clear: just like DBI does).

=head3 Methods

=over 4

=item I<new ([$section])>

Creates a new transport object, according to the configuration section
I<$section>. The object I<does not> belong to the
B<IDX::PKI::Transport::Sender> class, but rather to one of
B<IDX::PKI::Transport::*::Sender>.

=cut "

package IDX::PKI::Transport::Sender;
use IDX::PKI::Config;
use strict;
use vars qw(@ISA);

@ISA = qw(IDX::PKI::Transport);

sub new {
	my ($class, $section) = @_;
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS") unless ($section);
	my $config = the IDX::PKI::Config ();
	my $type = $config->getval ($section, "type");

	if ($type =~ m/local/i) {
		require IDX::PKI::Transport::Local;
		return new IDX::PKI::Transport::Local::Sender($section);
	} else {
		throw IDX::PKI::Error::Internal ("Unknown transport type '$type'");
	}
}

1;


=pod "

=back

=head2 Transport receiver classes

The receiver part, on the other hand, is generic. There are three main
ways of receiving messages:

=over 4

=item *

Have them stack up in a spool directory, one file per message, and
treat them either by batch or upon user interaction. This is the
purpose of the B<IDX::PKI::Transport::Unspooler> class, that
provides methods for fetching files from a spool;

=item *

Listen to a socket or some other network server mechanism (procmail
maybe?), and perform an action upon receiving a message. There is
very limited support for this in B<IDX::PKI::Transport::Receiver>
(what it provides is unlikely to be enough for your needs, this class
is basically here so that you may inherit from it);

=item *

Listen to a socket and spool messages for an I<Unspooler>; this is the
purpose of B<IDX::PKI::Transport::Receiver::Spooler>.

=back

=head3 B<IDX::PKI::Transport::Unspooler> class

=over 4

=cut "

package IDX::PKI::Transport::Unspooler;
use strict;
use vars qw(@ISA);

@ISA=qw(IDX::PKI::Transport);

use IDX::PKI::Config;
use IDX::PKI::Error;
use Fcntl qw(:DEFAULT :flock); # For pullByID
use POSIX qw(:errno_h); # For checking return values of syscalls to ensure
                        # atomicity
use IDX::PKI::Debug (0);
use IO::File;

=pod "

=item I<new($section)>

Object constructor. The usual I<$section> designate a section in the
configuration file, where the transport receiver parameters are (see
I<new()> below in class B<IDX::PKI::Transport::Receiver::Spooler> for a
syntax example).

=cut "

sub new {
	my ($class, $section)=@_;
	my $config = the IDX::PKI::Config ();
	my $directory = $config->getPath($section,'directory');

	throw IDX::PKI::Error::Config
		("Unknown directory", -section => $section)
			unless (defined $directory);

	my $self = {
							directory => $directory,
							pulled    => [],
						 };

	return bless($self,$class);
}

=pod "

=item I<list()>

Usage example:

 $trans->list();

Returns a list of available message IDs in a transport queue. Message
IDs are strings whose only property you may rely upon is that they are
different from each other.

=cut "

sub list {
	my ($self) = @_;

	return $self->list_dir ($self->{"directory"});
}

=pod "

=item I<list_dir ($directory)>

This method implements I<list()> in a way which allows it to be used
directly by external scripts.

=cut "

sub list_dir {
	my ($self, $dir) = @_;

	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS") unless ($dir);
	throw IDX::PKI::Error::Internal ("ILLEGAL_CONTEXT") unless (wantarray);

	local *DIR;
	opendir (DIR, $dir) || throw IDX::PKI::Error::IO ("-IOfile" => $dir);

	# Weed out non-executable files, which indicate unfinished
	# transmission. Untaint return values, too.
	return (map {(m/^([A-Za-z0-9+]+)$/ && (-x "$dir/$1")) ? ($1) : ()}
			(readdir DIR));
}


=pod "

=item I<_id2file($id)>

Returns the filename corresponding to the message I<$id>.  This method is
not meant to be called directly.

=cut "

sub _id2file {
	my ($self,$id)=@_;
	throw IDX::PKI::Error::Internal ("No ID supplied to _id2file")
		unless (defined $id && $id ne "");
	$id =~ s|[\\/.]||g; # Avoid nasty attacks. We normally do not let the
	# sender choose message IDs, but better safe than sorry.
	return "$self->{directory}/$id";
}

=pod "

=item I<readByID()>

Usage example:

 $trans->readByID($id,"Some::Type","Other::Type"
 	[ => \&constr ],...);

Returns message identified by I<$id>, provided it belongs to one of the
classes passed as parameters or any subclasses thereof (in I<@ISA> sense).
If I<$id> isn't available, returns C<undef>.  If message I<$id> is not of
a supported type (according to the type tag transmitted - see L<send()>
above), throws I<IDX::PKI::Error::Network("MALFORMED_MESSAGE")>.

If a CODE ref is passed as a parameter just after a type, it is passed the
binary blob in order to decode it; by default, the I<deserialize()>
constructor of the class indicated by the datagram is called.  Here is an
example for both:

 sub Some::Type::deserialize {
 	my ($class,$blob) = @_;
 	my $self = bless({}, $class);
 	# Initialize fields from $blob in $self here...
 	return $self;
 }

 sub constr {
 	my ($blob) = @_;
 	return new DataType(-blob => $blob, ...)
 }

Warning, I<< $trans->readByID(shift ($transport->list()),...) >> may
return C<undef>, because this sequence is not atomic.  I<list()> is
closely followed by I<readByID()>, but not exactly simultaneous: A delete
sequence from a concurrent process could slip in between.

=cut "

sub readByID {
	my ($self,$id,@classes)=@_;
	local *FILE;
	my $filename=$self->_id2file($id);

	open(FILE,"<$filename") || do {
		return undef if ($! == ENOENT);
		throw IDX::PKI::Error::IO(-IOfile => $filename);
	};

	my $ret = $self->readFromFD(\*FILE,$filename,@classes);
	close(FILE) || throw IDX::PKI::Error::IO(-IOfile=>$filename);
	return $ret;
}

=pod "

=item I<readFromFD($fd,$filename,@classes)>

Reads a message from an open file descriptor; $filename is
just used to make up the exception in case of an error. *Please do not
use this function directly*. May return undef if the file is not
complete (because it is being created right now); the convention is that
transport endpoints should set the executable bits on completed files.

=cut "

sub readFromFD {
	my ($self,$fd,$filename,@classes)=@_;

	# For Postfix, non-executable files indicate unfinished transmission.
	# We keep this convention.
	( -x $fd) || return undef;

	$!=0;
	my $message=join('',<$fd>);
	throw IDX::PKI::Error::IO(-IOfile=>$filename) if $!;

	return $self->decode_message($message, @classes);
}

=pod "

=item I<pullByID()>

Usage example:

 $trans->pullByID($id,"Some::Type","Other::Type",...);

Same as I<readByID()>, but asserts intent of removing the message from the
queue (see L<expunge()>).  Any given message cannot be in the
pulled-and-not-yet-expunged state in more than one
B<IDX::PKI::Transport::Sender> object (this is guaranteed by
I<flock()>ing the file).

=cut "

sub pullByID {
	my ($self,$id,@classes)=@_;

	# We lock the file, so that it doesn't get pulled twice by two
	# concurrent processes. In a strong integrity context, we should
	# record this locking to the filesystem before allowing ourselves
	# to proceed (two-phase commit).

	my $lock = new IO::File;
	my $filename = $self->_id2file($id);

	sysopen($lock,$filename,O_RDWR) || (($! == ENOENT) ? return undef :
		throw IDX::PKI::Error::IO(-IOfile=>$filename));

	return undef unless (flock($lock,LOCK_EX|LOCK_NB));
	# File opened but lock failed - we have a concurrent faster
	# than we are, give up.

	my $obj = $self->readFromFD($lock,$filename,@classes);

	push (@{$self->{pulled}},[$id,$lock]) if $obj;

	return $obj;
}

=pod "

=item I<expunge()>

Usage example:

 $trans->expunge();

Erases all messages that where I<pull()>ed since the creation of the
object or the last I<expunge()>.

Warning: Since the delete intent is a mere I<flock()> and is not written
to disk, a crash during execution of the I<expunge()> method will
eventually cause messages to be duplicated.  This is a bug in
B<IDX::PKI::Transport::Unspooler> that could be fixed (e.g. use hard
links to mark up the intent of deleting): This is called I<two-phase
commit> and requires a little bit of care when writing the crash recovery
code (the advantage of the current, simplistic approach is that no
recovery is ever necessary).

=cut "

sub expunge {
	my ($self)=@_;
	foreach my $pairref (@{$self->{pulled}}) {
		my ($id,$lock)=@$pairref;
		unlink($self->_id2file($id)) ||
			throw IDX::PKI::Error::Security("LOCK_VIOLATED");
		close($lock);
	};
	$self->{pulled}=[];
}

=pod "

=item I<release()>

Releases all held write locks, without performing any destructions.

=cut "

sub release {
	my ($self)=@_;

	foreach my $pairref (@{$self->{pulled}}) {
		my ($id,$lock)=@$pairref;
		close($lock);
	};
}

=pod "

=back

=head3 B<IDX::PKI::Transport::Receiver::Spooler> class

This class takes care of receiving messages, and is of use for making
transport endpoint daemons (as CGI scripts, procmail entries,
etc.). Access to transport endpoints is ACL-controlled; this class
takes care of this.

=over 4

=cut "

package IDX::PKI::Transport::Receiver::Spooler;
use strict;
use IDX::PKI::Config;
use IDX::PKI::Authentication;
use IDX::PKI::Error;
use IDX::PKI::Log;

=pod "

=item I<new($section)>

Object constructor. The usual I<$section> designate a section in the
configuration file, where the transport receiver parameters are. Said
section is of class C<TRANSPORT-IN> and looks like this:

 [TRANSPORT-IN-ee-p12]
 directory=/some/spool/directory
 acl=someACL
 post=/some/script to be launched whenever a message is received

 [ACL-someACL]
 ; See IDX::PKI::Authentication for info on the format here. The only useful
 ; privilege for Transports is PRIV_USE.

=cut "

sub new {
	my ($class, $section) = @_;

	my $config = the IDX::PKI::Config ();
	my $directory = $config->getPath($section, "directory");
	my $post = $config->getPath($section, "post");
	my $acl = $config->getPath($section, 'acl');

	throw IDX::PKI::Error::Internal ("Incomplete configuration: missing ".
									 "'directory' in section $section.")
		unless (defined $directory);
	my $self = {"config" => $config,
				"section" => $section,
				"directory" => $directory,
				"post" => $post,
				"acl" => $acl,
};

	return bless($self, $class);
}


=pod "

=over 4

=item I<receive ($message, $object)>

Takes a text message in $message and calls
I<IDX::PKI::Transport::write_spoolx()> to record it to disk. Sender
(represented by $object) must have a I<getPeerDN()> method and be in the
configuration ACL (see L<IDX::PKI::Authentication>) or an exception
will be thrown.

=back

=cut "


sub receive {
	my ($self, $message, $obj) = @_;

	throw IDX::PKI::Error::Internal("INCORRECT_ARGS")
		unless ($self && $message && $obj);

	throw IDX::PKI::Error::Internal("Configuration Error, ACL not defined")
		unless ($self->{acl});

	IDX::PKI::Authentication::authorize ($obj, "PRIV_USE",
										 "ACL-".$self->{acl});

	my $id = IDX::PKI::Transport::write_spoolx($message, $self->{directory});

	if ($self->{post})
	{
		my $log = new IDX::PKI::Log ();

		(system ($self->{post}, $self->{directory}."/".$id) == 0) &&
			$log->success ("MESSAGE_RECEIVE",
						   "-details" => "Successfully executed ".
						   "$self->{'post'} 'post' script for $id") ||
			$log->failure ("MESSAGE_RECEIVE",
						   "-details" => "Couln't launch $self->{'post'} ".
						   "'post' script on $self->{'directory'}/$id");
	}

	return $id;
}

=pod "

=back

=cut "

eval(join '',<main::DATA>) || die $@ unless caller();

1;

__END__

################### Class hierarchy testbed #####################

package Ancestor::Class;

use strict;

our $VERSION="0.0.0.0.0.1";

sub new {
	my ($class)=@_;
	return bless ["Zoinx"],$class;
}

our $text=<<ZOINX;
--BEGIN ZOINX--
Zoinx
--END ZOINX--
ZOINX

sub serialize {
	return $text;
}

sub deserialize {
	my ($class,$ser)=@_;
	throw IDX::PKI::Error::Network("MALFORMED_MESSAGE",-message=>$ser)
	  unless ($ser eq $text);
	return new($class);
}

package Descendant::Class;

use strict;

our @ISA=qw(Ancestor::Class);

sub deserialize { "foo"; }

package Bogus::Class;

use strict;

sub deserialize { die "Shouldn't have come here" };

package IDX::PKI::Transport;

use strict;

use IDX::PKI::Config;
use IDX::Test;
use IDX::PKI::Crypto::Tools;

number_of_tests(22);

test "unique" => sub {
	my $oldunique = unique();
	my $newunique;
	foreach my $i (1..20) {
		$newunique = unique();
		TEST ($newunique ne $oldunique) ||
			warn "Two successive occurences gave same result (current ".
			"count $i): '$newunique'";
		TEST ($newunique =~ m/^[A-Za-z0-9+]+$/) ||
			warn "unique() returned random characters: '$newunique'. ".
			"WARNING: This is a random failure, but a serious failure ".
			"nevertheless";
		$oldunique = $newunique;
	}
};

my $encoded;
test "encode_message" => sub {
	$encoded=IDX::PKI::Transport::encode_message(undef,new Ancestor::Class());
	TEST($encoded =~ m/\Q$Ancestor::Class::VERSION\E/);
	TEST($encoded =~ m/Ancestor::Class/);
};

test "decode_message" => sub {
	my $obj=decode_message(undef,$encoded,"Bogus::Class","Ancestor::Class");
	TEST($obj->isa("Ancestor::Class"));
};

test "encode_message and inheritance" => sub {
	$encoded=IDX::PKI::Transport::encode_message(undef,new Descendant::Class());
	TEST($encoded =~ m/Descendant::Class/);
};

test "decode_message and inheritance" => sub {
	TEST(decode_message(undef,$encoded,"Bogus::Class","Ancestor::Class")
		 eq "foo");

	$encoded=IDX::PKI::Transport::encode_message(undef,new Ancestor::Class());
	try {
		decode_message(undef,$encoded,"Descendant::Class");
		TESTFAIL();
	} catch IDX::PKI::Error::Network with {
		TESTOK();
	};
};

test "decode_message and subs" => sub {
	TEST(decode_message(undef,$encoded,"Bogus::Class"=>sub {"bar"},
						"Descendant::Class" => sub { "booz" },
						"Ancestor::Class"=> sub {"quux"})
		 eq "quux");
};

test "decode_message provides appropriate warnings" => sub {
	my $nosuchmessage=<<'MESSAGE';
--BEGIN IDXPKI MESSAGE--
Type: No::Such::Class 0.0.1
aWRlZml4LmlkZWFseC5jb20K
--END IDXPKI MESSAGE--
MESSAGE
	try {
		decode_message(undef,$nosuchmessage,"No::Such::Class");
		TESTFAIL();
	} catch IDX::PKI::Error::Network with {
		TEST(shift->{-details} =~ m/unknown/i);
	};
};

# Tests on spool operations

sub Bogus::Sender::getPeerDN { return qw(CN JohnDoo O IDEALX C fr); }
sub Evil::Sender::getPeerDN { return qw(CN JohnRandomHacker O IDEALX C fr); }

my $bogussender=bless {},"Bogus::Sender";
my $evilsender=bless {},"Evil::Sender";

my $spool=new IDX::Test::Directory();

my $config = set IDX::PKI::Config (<< "CONFIG");
[TRANSPORT-IN-test]
directory=$spool
acl=someACL

[ACL-someACL]
can-use=CN=JohnDoo,O=IDEALX,C=fr

CONFIG

my $spooler;

test "spooling" =>sub {
	$spooler = new IDX::PKI::Transport::Receiver::Spooler("TRANSPORT-IN-test");
	$spooler->receive
	  (IDX::PKI::Transport::encode_message
	   (undef,new Ancestor::Class()),$bogussender);

	$spooler->receive
	  (IDX::PKI::Transport::encode_message
	   (undef,new Descendant::Class()),$bogussender);

	my @files=glob("$spool/*");
	TEST(@files==2);
};

test "evil spooler" => sub {
	try {
		$spooler->receive
		  (IDX::PKI::Transport::encode_message
		   (undef,new Ancestor::Class()),$evilsender);
		TESTFAIL();
	} catch IDX::PKI::Error::Security with {
		TESTOK();
	};
};

my $transportin;
test "unspooler: constructor" => sub {
	$transportin=new IDX::PKI::Transport::Unspooler ("TRANSPORT-IN-test");
};

my @list;
test "listing" => sub {
	local *TOUCH;

	open(TOUCH,">".$transportin->{directory}."/AAAAAAAA.hammer");
	# U can touch this :-) without disturbing ->list().
	close(TOUCH);

	@list=$transportin->list();
	TEST(scalar(@list) == 2) || warn join(" ",@list);
};

my $obj;

my @searchparams=($list[0],'Ancestor::Class','Descendant::Class');

test "reading" => sub {
	TEST(($obj)=($transportin->readByID(@searchparams)));
	TEST(ref($obj) =~ m/^(Ancestor|Descendant)::Class$/);
	@searchparams=($list[0],ref($obj)); # Who says tests
	# are not reproducible? :->
};


test "reading via anonymous constructor" => sub {
	$transportin->readByID(@searchparams,sub {
							   TEST($_[0] =~ m/zoinx/i) || warn $_[0]});
	0;
};

test "pulling the thing read" => sub {
	my $obj2=$transportin->pullByID(@searchparams);
	TEST( (defined $obj2) &&
		(Data::Dumper::Dumper($obj) eq Data::Dumper::Dumper($obj2)) );
};

test "reading the thing pulled" => sub {
	my $obj2=$transportin->readByID(@searchparams);
	TEST( (defined $obj2) &&
		(Data::Dumper::Dumper($obj) eq Data::Dumper::Dumper($obj2)) );
};

test "pulling again the thing pulled (shall not work)" => sub {
	my $result=$transportin->pullByID(@searchparams);
	TEST(!defined $result);
};

test "pulling from parallel process (shall fail also)" => sub {
	defined(my $pid=fork()) || die;

	if ($pid) {
		waitpid($pid,0);
		TEST(($? >> 8) == 0);
	} else {
		my $othertransport = new
			IDX::PKI::Transport::Unspooler("TRANSPORT-IN-test");
		POSIX::_exit((defined ($transportin->pullByID(@searchparams))) ?
			1 : 0 );
	};
};

test "reading from parallel process (shall succeed)" => sub {
	defined(my $pid=fork()) || die;

	if ($pid) {
		waitpid($pid,0);
		TEST(($? >> 8) == 0);
	} else {
		my $othertransport = new
			IDX::PKI::Transport::Unspooler("TRANSPORT-IN-test");
		POSIX::_exit( (defined ($transportin->readByID(@searchparams))) ?
			0 : 1 );
	};
};

test "expunging" => sub {
	$transportin->expunge();
	my @l=$transportin->list();
	TEST(1==scalar(@l));
};

test "reading after expunge (shall fail)" => sub {
	TEST(!defined ($transportin->readByID(@searchparams)));
};

@searchparams= ($list[1],
				($searchparams[1] eq "Ancestor::Class") ?
					"Descendant::Class" : "Ancestor::Class");

test "automatic release of lock" => sub {
	do {
		my $transportinbis = new IDX::PKI::Transport::Unspooler
			("TRANSPORT-IN-test");
		TEST($transportinbis->pullByID(@searchparams)) ||
		  warn "failed subtest 1";
		TEST(! $transportin->pullByID(@searchparams)) ||
		  warn "failed subtest 2";
	};
	TEST($transportin->pullByID(@searchparams))  ||
		  warn "failed subtest 3";
	$transportin->release();
};

test "automatic release of lock in catastrophic cases" => sub {
	  try {
		  my $transportinbis = new IDX::PKI::Transport::Unspooler
			  ("testtransport");
		  TEST($transportinbis->pullByID(@searchparams));
		  die "if you see this message, try use()ing IDX::PKI::Error";
	  } otherwise {
		  TEST($transportin->pullByID(@searchparams));
	  };
	$transportin->release();
};

1;
