#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::PKI::ValidateInput> - A bunch of functions for validating user input, either in I<IDX::PKI::Registry> scripts or command-line utilities

=head1 SYNOPSIS

 # All functions in this module are available to export.
 # This is an example for a command-line script:
 use IDX::PKI::ValidateInput qw(LDAP_wildcard name);
 my $name=name($ARGV[0]); defined($name) || usage();

 # Or in an IDX::PKI::Registry script:
 use IDX::PKI::Registry;
 use IDX::PKI::ValidateInput qw(LDAP_wildcard name);
 my $filter=$cgi->validate("filter", \&LDAP_wildcard);

 # Functions are also available through a naming convention:
 use IDX::PKI::ValidateInput;
 $filterfunc=$wf_functions{"LDAP_wildcard"};

 # Multi-parameter validation functions.
 use IDX::PKI::ValidateInput;
 my $date = $cgi->validate ("date",
 	[\&IDX::PKI::ValidateInput::MultiParam::Date, "year",
 		"month", "day", undef, undef, undef],
 	"-extraargs" => "GMT");

=head1 DESCRIPTION

All functions in this module share the same semantics: they take a
unique argument, and return it untainted and canonicalized (spurious
characters removed etc.). If no amount of effort on their behalf will
coerce the user input into meaning anything, they return undef. There
is no OO stuff here. Some functions take extra arguments to fine-tune
their validation behavior.

Additionnally, the %wf_functions array (exported by default) contains
all functions of this module under the same key as their name. This is
to accomodate the pieces of code in IDX-PKI that need to choose a
validating function at run time (e.g. profiles).

This package B<obsoletes> the corresponding functionality in
L<IDX::PKI::Principal>.


=over 4

=cut "

package IDX::PKI::ValidateInput;
use Exporter;
use IDX::PKI::Debug (0);

our @ISA=qw(Exporter);
our @EXPORT=qw(%wf_functions);
our @EXPORT_OK=qw(%wf_functions);
our %wf_functions;

{ no strict "refs";
  foreach my $symbol (keys %IDX::PKI::ValidateInput::) {
	  my $funcref=UNIVERSAL::can("IDX::PKI::ValidateInput",$symbol);
	  next unless $funcref;
	  push @EXPORT_OK,$symbol;
	  $wf_functions{$symbol}=$funcref;
  };
}

=pod "

=item I<boolean($string)>

Returns 1 or 0, according to whether $string is true or false perl-wise.
Never fails (even on undef, which can be construed as a bug).

=cut "

sub boolean {
	return ( (scalar shift) ? 1 : 0);
}

=pod "

=item I<messageID($id)>

Verifies if the given I<$id> is a valid transport message ID
(upper-case letters, numbers and the "+" sign).  If it is the case,
returns it untainted, otherwise returns C<undef>.

=cut "

sub messageID {
	my ($id) = @_;
	return ($id =~ m/^([a-z0-9+]+)$/i) ? $1 : undef;
}


=pod "

=item I<email($string)>

=item I<email($string, $length)>

Takes a string and returns it untainted if it ressembles an email address,
otherwise returns C<undef>. (Default maximum length: 255.)

=cut "

sub email {
	my ($email, $length) = @_;
	$length ||= 255;

	defined ($email) ||
		throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS");

	&DEBUG ("[debug] email argument = $email\n");

	return undef if (length ($email) > $length);
	return ($email =~ m/^((?:[a-z0-9_\-+.]+\@(?:[a-z0-9_\-]+\.)*[a-z0-9\-]+)(?:[,]\s*(?:[a-z0-9_\-+.]+\@(?:[a-z0-9_\-]+\.)*[a-z0-9\-]+)){0,2})$/i) ?
	  $1 : undef;
}


=pod "

=item I<dn($string)>

=item I<dn($string,$length)>

Takes a string and returns it untainted if it ressembles a DN (letters,
numbers, C<,>, whitespace, C<=>, C</>, C<@>, C<.>, C<-> or C<_>, and not
only whitespaces), otherwise returns C<undef>.

=cut "

sub dn {
	my ($dn, $length) = @_;

	defined($dn) ||
		throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS");

	&DEBUG("[debug]: DN argument = $dn\n");

	return undef if ($length && length ($dn) > $length);
	return undef if ($dn !~ /\S/);
	return ($dn =~ m|^([a-z0-9,/ =@.\-_]+)$|i) ? $1 : undef;
};


=pod "

=item I<name($string)>

=item I<name($string,$length)>


Takes a string an returns it untainted if it contains only
I<PrintableString> characters (letters, numbers, C<'>, C<">, C<(>,
C<)>, C<+>, C<->, C<.>, C<:>, C<?> and whitespace, and not only
whitespaces), otherwise returns C<undef>.  (Default maximum length:
60, specify 0 to disable length checking altogether)

Normally C<,> and C</> should be available, but they collide with
openssl's definition of a DN.

=cut "'

sub name {
	my ($name, $length) = @_;
	$length = 60 unless (defined $length);

	defined ($name) ||
		throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS");

	&DEBUG ("[debug]: name argument = $name\n");

	return undef if ($length && length ($name) > $length);
	return undef if ($name !~ /\S/);
	return ($name =~ m|^([a-z0-9\'\"()+\-.:? ]+)$|i) ? $1 : undef;
};


=pod "

=item I<country($string)>

Takes a string an returns it untainted if it looks like a 2 letter country
code, otherwise returns C<undef>.

=cut "

sub country {
	my ($country) = @_;

	defined ($country) ||
		throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS");

	&DEBUG ("[debug]: country argument = $country\n");

	return ($country =~ m/^([a-z][a-z])$/i) ? $1 : undef;
};


=pod "

=item I<text($string)>

=item I<text($string, $length)>

Takes a string an returns it untainted if it doesn't contain special
characters, otherwise returns C<undef>.  (Default maximum length: 30,
specify 0 to disable length checks altogether)

Please note it's probably all right for iso-latin 1 but not for other
encodings...

=cut "

sub text {
	my ($generic, $length) = @_;
	$length = 30 unless (defined ($length));

	defined ($generic) ||
		throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS");

	&DEBUG ("[debug]: generic argument = $generic\n");

	return undef if ($length && length ($generic) > $length);
	# Allowed chars = printable chars minus forbidden chars.
	# It's OK for iso-latin 1, but not other encodings...
	my $allowed_chars = join ("", map {sprintf ("%s", chr())}
		(32..126, 160..255));
	my $forbidden_chars = qr'[#+="\\/$@%&|()\[\]{}<>]';
	$allowed_chars =~ s/$forbidden_chars//g;
	$allowed_chars = qr"[$allowed_chars]";
	return ($generic =~ m/^($allowed_chars+)$/) ? $1 : undef;
};


=pod "

=item I<multilinetext($string)>

=item I<multilinetext($string, $length)>

Same as I<text()>, except that the string may contain carriage returns and
newlines.  The default maximum length is 255 (specify 0 to disable length
checks altogether).

=cut "

sub multilinetext {
	my ($generic, $length) = @_;
	$length = 255 unless (defined ($length));

	defined ($generic) ||
		throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS");

	&DEBUG ("[debug]: multi-line text argument = $generic\n");

	return undef if ($length && length ($generic) > $length);
	# Allowed chars = printable chars minus forbidden chars.
	# It's OK for iso-latin 1, but not other encodings...
	my $allowed_chars = join ("", map {sprintf ("%s", chr())}
		(32..126, 160..255));
	$allowed_chars .= "\r\n";
	my $forbidden_chars = qr'[#+="\\/$@%&|()\[\]{}<>]';
	$allowed_chars =~ s/$forbidden_chars//g;
	$allowed_chars = qr"[$allowed_chars]";
	return ($generic =~ m/^($allowed_chars+)$/) ? $1 : undef;
};


=pod "

=item I<password($string, $maxlength, $minlength)>

A valid string for a password (Latin1 characters accepted). Acceptable
characters are in ASCII range 32-126, 160-255. If $minlength and/or
$maxlength are specified and numeric, they are constraints over the
length of the password.

=cut "

sub password {
	my ($string, $max, $min) = @_;
	return undef if (!defined $string);
	return undef if ($min && $min > length($string));
	return undef if ($max && $max < length($string));
	no utf8;
	return ($string =~ m/^([\x20-\x7E\xA0-\xFF]*)$/ ? $1 : undef);
}

=pod "

=item I<uri($string)>

Takes a string an returns it untainted if it looks like an URI, otherwise
returns C<undef>.

=cut "

sub uri {
	my ($uri, $length) = @_;

	defined ($uri) ||
		throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS");

	&DEBUG("[debug]: URI argument = $uri\n");

	return undef if ($length && length ($uri) > $length);
	# not very elegant r.e. by provides portability (localization) at low cost
	return ($uri =~ m|^([a-z0-9\.\-:/%,&?@]+)$|i) ? $1 : undef;
};


=pod "

=item I<LDAP_wildcard($string$length)>

=item I<LDAP_wildcard($string, $length)>

Validates a string that can be used to match entries in an LDAP.  C<*> and
C<?> are valid wildcard characters.

=cut "

sub LDAP_wildcard {
	my ($wildcard, $length) = @_;

	defined($wildcard) ||
		throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS");

	&DEBUG("[debug]: LDAP wildcard argument = $wildcard\n");

	return undef if ($length && length ($wildcard) > $length);
	return ($wildcard =~ m/^([a-z0-9 .\-*?@]+)$/i) ? $1 : undef;
};


=pod "

=item I<pem($string)>

Validates a string that looks like a PEM-encoded file.  Strips any leading
or trailing whitespaces.

=cut "

sub pem {
	my ($pem) = @_;

	defined ($pem) || throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS");

	&DEBUG("[debug]: PEM file = $pem\n");

	return ($pem =~ m|^\s*(
		\-\-\-\-\-BEGIN[a-z0-9 \#]+\-\-\-\-\-
		[A-Za-z0-9+/= \t\r\n]+
		\-\-\-\-\-END[a-z0-9 \#]+\-\-\-\-\-
		)\s*$|sxi) ? $1 : undef;
};


=pod "

=item I<nakedpem ($string)>

Like I<pem()> except that it is not supposed to have the C<-----BEGIN ...
-----> and C<-----END ...-----> markers. This is how Internet Explorer
sends its certificate requests.

=cut "

sub nakedpem {
	my ($pem) = @_;

	defined ($pem) || throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS");

	return ($pem =~ m|^\s*([A-Za-z0-9+/= \t\r\n]+)\s*$|si) ? $1 : undef;
};

=pod "

=item I<pem_message ($string)>

Like I<pem()> except that it is compatible for use with Transport
messages.

=cut "

sub pem_message {
	my ($pem) = @_;

	defined ($pem) || throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS");

	return ($pem =~ m|^\s*([A-Za-z0-9-+/.:= \t\r\n]+)\s*$|si) ? $1 : undef;
};


=pod "

=item I<spkac ($string)>

=item I<spkac ($string, $length)>

Almost like I<pem()>, except that the string isn't supposed to have
the trailer and footer.

=cut "

sub spkac {
	my ($string, $length) = @_;

	defined ($string) || throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS");

	return undef if ($length && length ($string) > $length);
	return ($string =~ m|^\s*([a-z0-9+/= \t\r\n]+)\s*$|sxi) ? $1 : undef;
};


=pod "

=item I<choice($string, @choices)>

Validates a string as an element of a fixed choices list.

=cut "

sub choice {
	my ($string, @list) = @_;

	defined ($string) ||
		throw IDX::PKI::Error::Internal("WRONG_NUMBER_ARGS");

	&DEBUG("[debug]: string argument = $string\n");

	if (@list == 1 && ref ($list[0]) eq "ARRAY") {
		@list = @{$list[0]};
	}

	foreach my $choice (@list) {
		if ($string =~ m/^($choice)$/) {
			return $1;
		}
	}

	# No match found.
	return undef;
};


=pod "

=item I<serial($string)>

Takes a string an returns it untainted if it looks like a certificate
serial number (hexadecimal number), otherwise returns C<undef>.

=cut "

sub serial {
	my ($serial) = @_;

	defined ($serial) ||
		throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS");

	&DEBUG ("[debug]: serial argument = $serial\n");

	return ($serial =~ m/^([0-9a-f]+)$/i) ? $1 : undef;
};


=pod "

=item I<phone ($string)>

Takes a string an returns it untainted if it looks like a phone number
(digits, maybe separated by spaces, dots or dashes), otherwise returns
C<undef>. The phone number is canonicalised and only the digits are
returned.

=cut "

sub phone {
	my ($string) = @_;

	throw IDX::PKI::Error::Internal ("WRONG_NUMBER_ARGS") unless ($string);

	return undef unless ($string =~ m/^(\d+[0-9 \.\-]+\d+)$/i);
	$string = $1;
	$string =~ s/[^0-9]//g;
	return $string;
};


=pod "

=back

=head1 B<IDX::PKI::ValidateInput::MultiParam> SUBMODULE

Functions in this module allow to validate more than one parameter at a
time. They may return a scalar of an array, depending of the calling
context.

These functions are not exported, to avoid conflicts with
B<IDX::PKI::ValidateInput> ones.

=cut "

package IDX::PKI::ValidateInput::MultiParam;

use strict;

use IDX::PKI::Error;

=pod "

=over 4

=item I<date ($year, $month, $day, $hour, $min, $sec, $timezone)>

Validates a splitted date/time. In scalar context, returns the number of
elapsed seconds since the Epoch (cacultated thanks to B<Time::Local>) or
C<undef> on validation failure. In array context, just returns the values
untainted (or an empty array on validation failure).

Expected formats are the same as returned by the Perl I<gmtime()>
function; in particular:

=over 4

=item I<$year>

Number of years since 1900;

=item I<$month>

Months go from C<0> (for January) to C<11> (for December).

I<$hour>, I<$min> and I<$sec> are optional (but if you specify one of
them, you must specify all).

=back

If I<$timezone> is not specified, the local time is assumed.

=cut "

sub date
{
	my ($year, $month, $day, $hour, $min, $sec, $timezone) = @_;

	foreach my $ref (\$hour,\$min,\$sec) {
		$$ref=0 if (!defined $$ref);
	};

	return wantarray ? () : undef unless ($year =~ m/^(\d+)$/);
	$year = $1;
	return wantarray ? () : undef unless ($month =~ m/^(\d+)$/);
	$month = $1;
	return wantarray ? () : undef unless ($day =~ m/^(\d+)$/);
	$day = $1;
	return wantarray ? () : undef unless ($hour =~ m/^(\d+)$/);
	$hour = $1;
	return wantarray ? () : undef unless ($min =~ m/^(\d+)$/);
	$min = $1;
	return wantarray ? () : undef unless ($sec =~ m/^(\d+)$/);
	$sec = $1;

	require Time::Local;
	unless (eval {require Time::Zone; 1;}) {
		*Time::Zone::tz_offset = sub {return 0};
		*Time::Zone::tz_local_offset = sub {return 0};
		*Time::Zone::tz_name = sub {return "gmt"};
	}

	if ($timezone)
	{
		# Checks that the timezone is valid; if yes untaints it.
		my $offset = Time::Zone::tz_offset ($timezone);
		return wantarray ? () : undef unless (defined ($offset));
		$timezone =~ m/^(.*)$/;

		return wantarray ?
			($year, $month, $day, $hour, $min, $sec, $1) :
			Time::Local::timegm ($sec, $min, $hour, $day, $month,
								 $year) + $offset;
	}
	else
	{
		return wantarray ?
			($year, $month, $day, $hour, $min, $sec,
			 uc (Time::Zone::tz_name (Time::Zone::tz_local_offset()))) :
			Time::Local::timelocal ($sec, $min, $hour, $day, $month, $year);
	}
}

=pod "

=item I<confirm ($string, $confirm, $validatesub, @extras)>

Validates I<$string> and I<$confirm> according to the anonymous subroutine
I<$validatesub>, and then checkes that the two values are the same.
I<@extras> are passed as is to I<$validatesub>.

Has no meaning in array context.

=cut "

sub confirm {
	my ($string, $confirm, $sub, @extras) = @_;

	my $vstring  = &$sub ($string, @extras);
	my $vconfirm = &$sub ($confirm, @extras);

	return undef unless ((defined $vstring) && (defined $vconfirm));
	return ($vstring eq $vconfirm) ? $vstring : undef;
}

=pod "

=back

=cut "

eval join('',<main::DATA>) || die $@ unless caller();

__END__

####################### TEST SUITE ##############################

use strict;

package IDX::PKI::ValidateInput;

use IDX::Test;
use Taint qw (taint tainted);
use Carp;

number_of_tests (28);

sub TESTEQ {
	my ($a,$b)=@_;
	TEST($a eq $b) || carp "Failed equality: $a vs. $b";
	TEST(!tainted($a)) || carp "first arg to TESTEQ was tainted";
	TEST(!tainted($b)) || carp "second arg to TESTEQ was tainted";;
}

sub tainted_null {
	my $var = "";
	taint ($var);
	return $var;
}

test "wf_functions construction" => sub {
	TEST(keys %wf_functions) ||
	  warn "wf_functions empty";
	foreach my $func (@EXPORT_OK) {
		next unless ($func =~ m/^[a-z]$/);
		TEST(exists $wf_functions{$func}) ||
		  warn "Missing EXPORT_OK function $func in \%wf_functions";
	};
	TEST(scalar(@EXPORT_OK) == 1 + scalar(keys %wf_functions)) ||
	  warn sprintf("symbol export table mismatch (%d vs. %d)",
				   scalar(@EXPORT_OK), scalar(keys %wf_functions));

	# Proves quite nothing but I can't seem to grok the way one can
	# temporarily override a function, like in the (non-working)
#	local *IDX::PKI::ValidateInput::messageID=sub {"ZOINX"};

	TESTEQ("ZOINX",$wf_functions{"messageID"}("ZOINX"));
};

test "tainted_null (tests the test framework)" => sub {
	TEST (tainted (tainted_null()."")) || warn "tainted_null failed to ".
		"taint our string";
};

test "messageID" => sub {
	my $out = messageID(tainted_null()."A1234B5678");
	TESTEQ($out,"A1234B5678");

	$out = messageID(tainted_null()."");
	TEST(!defined($out));

	$out = messageID(tainted_null().'$_#@~]}');
	TEST(!defined($out));
};

# Tests about $wf_functions{'email'}.
test "\$wf_functions{'email'}: correct email address" => sub {
	my $out = $wf_functions{'email'}->(tainted_null().'I.am@the.best');
	TEST ($out eq 'I.am@the.best');
};

test "\$wf_functions{'email'}: 2 correct email addresses" => sub {
	my $out = $wf_functions{'email'}->(tainted_null().'I.am@the.best, I.am@good.too');
	TEST ($out eq 'I.am@the.best, I.am@good.too');
};

test "\$wf_functions{'email'}: incorrect email address without @" => sub {
	my $out = $wf_functions{'email'}->(tainted_null().'foo.bar');
	TEST (!defined $out);
};

test "\$wf_functions{'email'}: incorrect email address with special chars" =>
	sub {
	my $out = $wf_functions{'email'}->(tainted_null()."f.br\@blah.org\n");
	TEST (!defined $out);
};


# Tests about $wf_functions{'dn'}.
test "\$wf_functions{'dn'}: correct dn with both ,'s and /'s", sub {
	my $dn = 'CN=Administrator,OU=IDX-PKI/O=acme';
	my $out = $wf_functions{'dn'}->($dn);
	TEST ($out eq $dn);
};

test "\$wf_functions{'dn'}: correct dn with spaces", sub {
	my $dn = 'CN=Administrator, OU=IDX-PKI, O=acme';
	my $out = $wf_functions{'dn'}->($dn);
	TEST ($out eq $dn);
};

test "\$wf_functions{'dn'}: incorrect dn with returns", sub {
	my $out = $wf_functions{'dn'}->(tainted_null().'C=fr, O=acme, OU=PKI, '."\n".
		'CN=root/Email=nobody@acme.com');
	TEST (! defined ($out));
};


# Tests about $wf_functions{'name'}.
test "\$wf_functions{'name'}: word with nothing special", sub {
	my $out = $wf_functions{'name'}->(tainted_null()."hello");
	TESTEQ($out, "hello");
};

test "\$wf_functions{'name'}: word with 8bits chars", sub {
	my $out = $wf_functions{'name'}->(tainted_null()."hllo");
	TEST(! $out);		# Accented chars are not admitted inside a certificate
};

test "\$wf_functions{'name'}: word with spaces", sub {
	my $out = $wf_functions{'name'}->(tainted_null()."hello world");
	TESTEQ($out, "hello world");
};


test "\$wf_functions{'name'}: incorrect word with special chars", sub {
	my $out = $wf_functions{'name'}->(tainted_null().'$#\/}');
	TEST(!defined($out));
};


test "\$wf_functions{'name'}: incorrect blank word", sub {
	my $out = $wf_functions{'name'}->(tainted_null()."");
	TEST(!defined($out));
};


# Tests about $wf_functions{'text'}.
test "\$wf_functions{'text'}: admitted characters, including latin1", sub {
	my $out = $wf_functions{'text'}->(tainted_null()."a' ~-*");
	TESTEQ($out, "a' ~-*");
};

test "\$wf_functions{'text'}: newline", sub {
	my $out = $wf_functions{'text'}->(tainted_null()."\n");
	TEST(!defined $out);
};

test "\$wf_functions{'text'}: non-admitted chars", sub {
	my $out = $wf_functions{'text'}->(tainted_null()."/");
	TEST(!defined $out) || warn "Got $out";
	$out = $wf_functions{'text'}->(tainted_null()."\@");
	TEST(!defined $out) || warn "Got $out";
	$out = $wf_functions{'text'}->(tainted_null()."\$");
	TEST(!defined $out) || warn "Got $out";
};


# Tests about $wf_functions{'multilinetext'}.
test "\$wf_functions{'multilinetext'}: multi-line text OK", sub {
	my $string = "a'\n \r~-*\r\n";
	my $out = $wf_functions{'multilinetext'}->(tainted_null().$string);
	TESTEQ ($out, $string);
};

test "\$wf_functions{'multilinetext'}: non-admitted chars", sub {
	my $out = $wf_functions{'multilinetext'}->(tainted_null()."/");
	TEST(!defined $out) || warn "Got $out";
	$out = $wf_functions{'multilinetext'}->(tainted_null()."\@");
	TEST(!defined $out) || warn "Got $out";
	$out = $wf_functions{'multilinetext'}->(tainted_null()."\$");
	TEST(!defined $out) || warn "Got $out";
};


# Tests about $wf_functions{'LDAP_wildcard'}.
test "\$wf_functions{'LDAP_wildcard'}: admitted characters without ".
	"wildcards", sub {
	my $out = $wf_functions{'LDAP_wildcard'}->(tainted_null()."hello");
	TESTEQ($out, "hello");
};

test "\$wf_functions{'LDAP_wildcard'}: wildcards", sub {
	my $out = $wf_functions{'LDAP_wildcard'}->(tainted_null()."h*l?");
	TESTEQ($out, "h*l?");
};

test "\$wf_functions{'LDAP_wildcard'}: non-admitted chars", sub {
	my $out = $wf_functions{'LDAP_wildcard'}->(tainted_null().'" @');
	TEST(!defined $out);
};

test "password()" => sub {
	my $out = $wf_functions{'password'}->(tainted_null()."h*l?");
	TESTEQ($out,"h*l?");
	$out=$wf_functions{'password'}->(tainted_null()."h*l?\e");
	TEST(!defined $out);
};

test "MultiParam::date (scalar context)" => sub {
	my $time = time ();
	my ($sec, $min, $hour, $day, $month, $year) = gmtime ($time);
	my $result = IDX::PKI::ValidateInput::MultiParam::date
		($year, $month, $day, $hour, $min, $sec, "UTC");
	TEST ($result == $time) || warn "Error with UTC (time: $time / ".
		"result: $result)";

	($sec, $min, $hour, $day, $month, $year) = localtime ($time);
	$result = IDX::PKI::ValidateInput::MultiParam::date
		($year, $month, $day, $hour, $min, $sec);
	TEST ($result == $time) || warn "Error with localtime (time: $time / ".
		"result: $result)";

	$result = IDX::PKI::ValidateInput::MultiParam::date (102, 11, 28);
	my ($s, $m, $h, $d, $mo, $y) = localtime ($result);
	TEST ($y == 102) || warn "Bad year $y";
	TEST ($mo == 11) || warn "Bad month $mo";
	TEST ($d == 28) || warn "Bad day $d";
	TEST ($h == 0) || warn "Bad hour $h";
	TEST ($m == 0) || warn "Bad min $m";
	TEST ($s == 0) || warn "Bad sec $s";
};

test "MultiParam::date (incomplete arguments)" => sub {
	my $result = IDX::PKI::ValidateInput::MultiParam::date
		("102", # 2002
		 "11",   # Dec (12)
		 "10",
		 "23", "17");
	no locale;
	TESTEQ(scalar(localtime($result)),"Tue Dec 10 23:17:00 2002");
};


test "MultiParam::date (array context)" => sub {
	my $time = time ();
	my ($sec, $min, $hour, $day, $month, $year) = gmtime ($time);

	my ($y, $mo, $d, $h, $m, $s, $z) =
		IDX::PKI::ValidateInput::MultiParam::date
		($year, $month, $day, $hour, $min, $sec, "UTC");
	TEST ("$year$month$day$hour$min${sec}" eq "$y$mo$d$h$m$s") ||
		warn "Error with UTC (time: $year$month$day$hour$min${sec} / ".
		"result: $y$mo$d$h$m$s)";
	TEST (defined ($z)) || warn "No timezone with UTC";

	($sec, $min, $hour, $day, $month, $year) = localtime ($time);
	($y, $mo, $d, $h, $m, $s, $z) = IDX::PKI::ValidateInput::MultiParam::date
		($year, $month, $day, $hour, $min, $sec);
	TEST ("$year$month$day$hour$min$sec" eq "$y$mo$d$h$m$s") ||
		warn "Error with localtime (time: $year$month$day$hour$min$sec / ".
		"result: $y$mo$d$h$m$s)";
	TEST (defined ($z)) || warn "No timezone with localtime";
};

test "Confirm (with password)" => sub {
	my $passok  = tainted_null()."h*l?";
	my $passneq = tainted_null()."h*l?a";
	my $passnok = tainted_null()."h*l?\e";

	TEST (IDX::PKI::ValidateInput::MultiParam::confirm
		  ($passok, $passok, \&IDX::PKI::ValidateInput::password)
		  eq $passok) || warn "Error validating \$passok twice";

	TEST (IDX::PKI::ValidateInput::MultiParam::confirm
		  ($passneq, $passneq, \&IDX::PKI::ValidateInput::password)
		  eq $passneq) || warn "Error validating \$passneq twice";

	TEST (! defined IDX::PKI::ValidateInput::MultiParam::confirm
		  ($passnok, $passnok, \&IDX::PKI::ValidateInput::password)) ||
		  warn "\$passnok gave a result\!";

	TEST (! defined IDX::PKI::ValidateInput::MultiParam::confirm
		  ($passok, $passneq, \&IDX::PKI::ValidateInput::password)) ||
		  warn "Unmatched values gave a result\!";
};
