#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

IDX::SSLSocket - Same as IO::Socket::SSL, except it respects the
IDEALX software exception semantics, and does extra efforts to make
third-party modules such as LWP::Protocol::https GROK the setting of
SSL parameters such as your key, your cert, etc.

=head1 SYNOPSIS

 use IDX::SSLSocket qw(SSLscope setSSLcontext);

 IDX::SSLSocket->debug(1);

 my $ssl = new IDX::SSLSocket("mailhost", Port => 465,
 	SSL_verify_mode => 0);

 $ssl->send("HELO you");
 $ssl->send("TEA FOR: two"); # Expected response is "208 are you you are?"

 # This module also streamlines the incredibly bogus Net::SSLeay
 # module so that it becomes usable inside third-party classes such as
 # LWP (for HTTP/S URLs):
 SSLscope {
    setSSLcontext({SSL_use_cert=>1,
			SSL_cert_file=>$certfile,
			SSL_key_file=>$keyfile,
			SSL_verify_mode=>0,
		  });
    # Do LWP stuff here and rest assured that your setSSLcontext()
    # will be honored.
 }; # Don't forget the semicolon please

 # Other packages do not accept the setSSLcontext form but use
 # "normal" arguments to the connect() call or equivalent. In this
 # case, a mere SSLscope will scare the global-state bugs out of
 # Net::SSLeay and you'll be fine:

 SSLscope {
    my $req=new Net::HTTPS("Host" => "localhost",
                           "PeerPort" => $sslserver->getPort(),
                           SSL_verify_mode=>0);
 };

=cut "

package IDX::SSLSocket;

use strict;
use Net::SSLeay;
use IO::Socket::SSL 0.92;
use IDX::Error;

use vars qw(@ISA @EXPORT_OK);
@ISA = qw(Exporter IO::Socket::SSL);
@EXPORT_OK=qw(SSLscope setSSLcontext);

# Make sure LWP::Protocol::https uses us, so that we don't have silly surprises
# (like setting an SSL context without effects, because LWP uses the
# wrong SSL module)

require Net::SSLeay;
Net::SSLeay::load_error_strings();
require LWP::Protocol::https;
$LWP::Protocol::https::SSL_CLASS="IO::Socket::SSL";
$LWP::Protocol::https::SSL_CLASS="IO::Socket::SSL"; # Shuts warning

Net::SSLeay::ERR_clear_error();

=pod "

=head1 GLOBAL VARIABLES

=item I<$implicitsockopts>

The global variable holding the context. It is made local by I<sslscope>,
set by I<setSSLcontext()> and read by various kludges into the sanitized
modules. Do not use it, except to sanitize more modules.

=cut "

our $implicitsockopts;

sub LWP::Protocol::https::_extra_sock_opts {
	return %$implicitsockopts;
}

=pod "

=head1 UTILITY FUNCTIONS

=over 4

=item I<SSLscope($coderef)>

=item I<SSLscope BLOCK>

This is a function, but its use case is more like a reserved word (see
the SYNOPSIS above, but don't forget the trailing
semicolon). Evaluates $coderef or BLOCK while making local the global
SSL state variables (that hold your key, your certificate, the CA's,
and the verification details), and making a few other cleanups. The
main use is that SSLscope {} blocks limit the scope of calls to
L</setSSLcontext>, so that one process can perform several SSL
connections with several different certificates without them stomping
on each other.

The return value of the call to SSLscope is the same as the return
value of $coderef or BLOCK (by default, the last executed command in
the block).

=cut "

sub SSLscope(&) {
	throw IDX::Error::Internal
		("WRONG_NUMBER_ARGS",
		 "-details" => "Hint: forgot the trailing semicolon at the ".
		 "end of SSLscope block?") unless (@_ == 1);

	my ($coderef) = @_;
	# We kludge the various packages into working normally and with
	# re-entrance...
	Net::SSLeay::ERR_clear_error();

	local $implicitsockopts;

	return &$coderef;
}

=pod "

=item I<setSSLcontext(-arg1=>val1,...)>

Identical to I<IO::Socket::SSL::context_init()>, except it raises
exceptions upon failure and it sets all missing fields to C<undef> so
that the caller is not forced to pass any and all parameters.
Through a few kludges into L<LWP::Protocol::https>, these
settings will also be passed by any B<LWP> connection that opens while
the I<setSSLcontext()> call is in scope.

You can't specify both C<SSL_ca_file> and C<SSL_ca_path>.

=cut "

sub setSSLcontext {
	my $argsref = (@_ == 1 ? $_[0] :
				   (! (@_ % 2) ? { @_ } : 
					throw IDX::Error::Internal("WRONG_NUMBER_ARGS")));

	if (! exists $argsref->{"SSL_verify_mode"}) {
		$argsref->{"SSL_verify_mode"} =
			((defined($argsref->{"SSL_ca_file"}) ||
			  defined($argsref->{"SSL_ca_path"})) ? 3 : 0);
	}

	throw IDX::Error::Internal ("INCORRECT_ARGS", "-details" =>
									 "You can't specify both SSL_ca_file ".
									 "and SSL_ca_path.")
		if ($argsref->{"SSL_ca_file"} && $argsref->{"SSL_ca_path"});

	throw IDX::Error::Internal ("INCORRECT_ARGS", "-details" =>
									 "Must specify both a key and a cert, or none")
		if ($argsref->{"SSL_cert_file"} xor $argsref->{"SSL_key_file"});

	foreach my $tag (qw(SSL_ca_file SSL_ca_path SSL_cert_file SSL_key_file)) {
		$argsref->{$tag} = undef if (! exists $argsref->{$tag});
	}

	my $ret;
	try {
		$ret = IO::Socket::SSL::context_init ($argsref);
	} catch IDX::Error with {
		my $E = shift;
		$E->{"-arguments"} = $argsref;
		$E->throw ();
	};
	our $implicitsockopts = $argsref;
	throw IDX::Error::Internal
		("call to IO::Socket::SSL::context_init failed",
		 "-SSLeayErrors" => [_getSSLeayErrors()]) unless (defined $ret);
	return $ret;
}

=pod "

=item _getSSLeayErrors()

Grovels through the error stack of openssl and returns a list of all
errors (or only the outermost one in scalar context). Useful only for
constructing exceptions from inside I<IDX::SSLSocket>.

=cut "

sub _getSSLeayErrors {
	my @errors;
	while(my $err=Net::SSLeay::ERR_get_error()) {
		push @errors,Net::SSLeay::ERR_error_string($err);
	};
	return wantarray ? @errors: $errors[0];
}

=pod "

=item IO::Socket::SSL::_myerror()

This function is rewritten to turn the errors inside IO::Socket::SSL
into IDEALX exceptions. This way more errors will make their way up to
the call stack, especially using LWP (which swallows errors a bit).

=cut "

no warnings;
sub IO::Socket::SSL::_myerror {
	my ($self, @args) = @_;
	unshift (@args, $self) unless (ref $self);

	my @SSLerrors = _getSSLeayErrors();
	Net::SSLeay::ERR_clear_error();
	my $i = 0;
	local $Error::Depth = $Error::Depth + 1;
	local $Error::Debug = 1;
	throw IDX::Error::Network ("Internal IO::Socket::SSL error",
									"-SSLeayErrors" => \@SSLerrors,
									map {"-IOSocketSSL_".($i++) => $_} @args);
}
use warnings;


=pod "

=back

=head1 METHODS

IDX::SSLSocket inherits from B<IO::Socket::SSL> and provides the same
methods.  The following overloadings are made:

=over 4

=item I<new()>

Usage example:

 my $smtp = new IDX::SSLSocket("localhost", PeerPort => 33);

 my $smtp = new IDX::SSLSocket("localhost",
 	Config => $config,
 	Section => "TRANSPORTIN-$myprincipalname");

Object constructor.  Takes the same arguments as B<IO::Socket::SSL::new>,
with the following differences:

=over 4

=item *

The hostname is always first, and there is no hash reference
(see example);

=item *

C<PeerPort> may also be specified as C<Port>;

=item *

C<PeerDN>, if present, is the DN the peer must have lest the
connection be closed immediately (to prevent man-in-the-middle
attacks). If L<IDX::PKI::DN> is loaded, comparison is done using this
module; otherwise stringwise equality is required.

=item *

C<Proto> has to be TCP, and C<SSL_cipher_list> defaults to ALL;

=item *

C<SSL_verify_mode> is set according to the existence of an C<SSL_ca_file>
or C<SSL_ca_path> (C<3> if set, C<0> if unset); the latter two options
having a semantics similar to the one they use in B<IO::Socket::SSL>.
Warning: No matter which of those options is set, OpenSSL must be able to
retreive the full certification chain (up to the root CA), lest the
verification fail and a very weird error message appear in the server's
(yes, the server's - go figure) log.

=item *

C<SSL_use_cert> is set automatically from the presence of
both C<SSL_cert_file> and C<SSL_key_file>;

=item *

B<IDX::SSLSocket> can find the values of its key file, certificate file
and CA file from I<IDX::Config::readSSLParams()> (see
L<IDX::Config/readSSLParams>) if passed both a C<Config> and a
C<Section> argument.

=back

You can't specify both C<SSL_ca_file> and C<SSL_ca_path>.

=cut "

sub new {
	throw IDX::Error::Internal("WRONG_NUMBER_ARGS") unless
		((@_ % 2) == 0);
	my ($package, $host, %arg) = @_;

	# Feature: read configuration from IDX::Config
	if ($arg{"Config"} && $arg{"Section"}) {
		require IDX::Config;
		($arg{"SSL_ca_file"}, $arg{"SSL_cert_file"}, $arg{"SSL_key_file"}) =
			IDX::Config::readSSLParams ($arg{"Config"}, $arg{"Section"});
		foreach my $set (qw(SSL_ca_file SSL_cert_file SSL_key_file)) {
			delete $arg{$set} if (! defined $arg{$set});
		}
	};
	my $verifypeer = (defined($arg{"SSL_ca_file"}) ||
					  defined($arg{"SSL_ca_path"}));

	$arg{"PeerAddr"} = $host;
	$arg{"PeerPort"} ||= (delete($arg{"Port"}));
	$arg{"Proto"} = 'tcp';

	# We compute this by ourselves (according to whether we
	# were provided a CA or not).
	delete $arg{"SSL_use_cert"};
	delete $arg{"SSL_verify_mode"};

	$arg{"Timeout"} = 120 unless (defined $arg{"Timeout"});

	my $self;
	try {
		$self = SSLscope {
			$package->IO::Socket::SSL::new
				(%arg,
				 "SSL_use_cert" => (defined $arg{"SSL_cert_file"} ? 1 : 0),
				 "SSL_verify_mode" => ($verifypeer ? 3 : 0),
				 "SSL_cipher_list" => 'ALL') || throw IDX::Error::Network
				 ("Cannot establish SSL connection",
				  "-SSLeayErrors" => [_getSSLeayErrors()]);
		};
	} catch IDX::Error with {
		my $E = shift;
		$E->{"-arguments"} = \%arg;
		$E->throw ();
	};

	if ($verifypeer) {
		# We don' want to be fooled by a random certificate in
		# getPeerDN() in case we didn't get a CA from caller:
		${*$self}{SSL_cert_checked}=1;
	};

	if (defined $arg{PeerDN}) {
		my $peerdn=$self->getPeerDN();
		if (! ($IDX::PKI::DN::VERSION ?
			   IDX::PKI::DN::compare($arg{PeerDN}, $peerdn) :
			   ($arg{PeerDN} eq $peerdn)) ) {
			throw IDX::Error::Security("Unexpected DN on the remote end",
									   -remoteDN=>$peerdn);
		}
	}

	$self->autoflush(1);

	return $self;
}

=pod "

=item I<getPeerDN()>

Interface function for use in the IDEALX internal authentication
process (see L<IDX::Authentication>).  Returns the authenticated DN of
this connection's peer. If L<IDX::PKI::DN> is loaded, returns the
DN as a list in array context.

=cut "

sub getPeerDN {
	my ($self)=@_;
	return undef unless ${*$self}{'SSL_cert_checked'};
	my $cert=$self->get_peer_certificate();
	return undef if (! $cert);

	my $name=$cert->subject_name();
	throw IDX::Error::Internal("SHOULDNT_HAPPEN") unless
		($name =~ m|^/(.*)$|);

	return (wantarray && $IDX::PKI::DN::VERSION) ?
	  parse IDX::PKI::DN($name) : $name;
}

=pod "

=item debug($onoff)

Class method, turns SSL debugging on or off according to the boolean
value of $onoff. Debugging affects classes I<LWP>, I<Net::SSLeay> and
I<IO::Socket::SSL>.

=cut "

sub debug {
	my (undef,$onoff)=@_;

	if ($onoff) {
		import LWP::Debug ('+');
		$IO::Socket::SSL::DEBUG = 1;
		$Net::SSLeay::trace = 2;
	} else {
		import LWP::Debug ('-');
		$IO::Socket::SSL::DEBUG = 0;
		$Net::SSLeay::trace = 0;
	};
}

=pod "

=back

=head1 BUGS / TODO

LWP sanitization is not thoroughly tested. Net::SSLeay sanitization is
unimplemented (use IO::Socket::SSL instead).

IDX::SSLSocket's are only implemented / tested as clients.

=cut "

1;

####################### TEST SUITE #########################

require IDX::Test::Suite unless caller();

__END__

package IDX::SSLSocket;
use IDX::Test;
use IDX::Test::WebServer;

# IDX::SSLSocket->debug(1);

unless (`/usr/sbin/stunnel -V 2>&1` =~ m/^stunnel 3.22 /m) {
	warn <<MESSAGE;
####################################################
Unknown version of stunnel, tests will be unreliable
####################################################

MESSAGE
}


number_of_tests (23);

my (undef, $clicert, $clikey, $svrcert, $svrkey) = IDX::Test::certFiles();
my ($evilcertfile, $evilkeyfile) = IDX::Test::evilCertFiles();
my $cadir = new IDX::Test::Directory ();
{
	my ($cacert, $rootcacert) = (IDX::Test::certificates ())[0,6];
	my ($cahash, $rootcahash) = map {IDX::Test::getCertHash ($_)}
		($cacert, $rootcacert);
	local *FILE;
	open (FILE, ">", "$cadir/$cahash.0") ||
		die "Can't write to $cadir/$cahash.0: $!";
	print FILE $cacert;
	close (FILE);
	open (FILE, ">", "$cadir/$rootcahash.0") ||
		die "Can't write to $cadir/$rootcahash.0: $!";
	print FILE $rootcacert;
	close (FILE);
}

my ($server, $ssocket);

sub TESTreceive {
	my ($sock) = @_;
	$sock = $ssocket if (! defined $sock);
	my $msg = <$sock>;
	warn ("Got nothing from the socket!") if (! $msg);
	TEST($msg =~ m/hello/i) || warn "Unexpected message $msg";
};

test "setSSLcontext syntax" => sub {
	setSSLcontext('SSL_verify_mode' => 0);
	setSSLcontext({ 'SSL_verify_mode' => 0 });
};

test "setSSLcontext throws upon bogus args" => sub {
	try {
		setSSLcontext({'SSL_use_cert' => 1,
					   'SSL_key_file' => $clikey,
					   'SSL_verify_mode' => 0,
					  });
		TESTFAIL();
	} otherwise {
		TESTOK();
	};
};

################### HTTP/S tests ####################################
my $wwwserver=new IDX::Test::WebServer()->start();
my $sslserver=new IDX::Test::SSLServer(-server=>$wwwserver)->start();


use Net::HTTPS;
test "SSLSocket and Net::HTTPS: connect without a certificate" => sub {
	SSLscope {
		my $req=new Net::HTTPS("Host" => "localhost",
							   "PeerPort" => $sslserver->getPort(),
							  					   SSL_verify_mode=>0);
		$req->write_request("GET"=>"/index.html",
						   "User-Agent"=>"IDX-was-zere v1.0");
        my($code, $mesg, %h) = $req->read_response_headers();
		TEST($code eq 200) || warn "$code $mesg";

		my $buf;
		TEST($req->read_entity_body($buf, 1024));
		TEST($buf =~ m/IDX-was-zere/) || warn $buf;
	};
};



test "Bogus setSSLcontext (should not block subsequent tests)" => sub {
	setSSLcontext({'SSL_use_cert' => 1,
				   'SSL_key_file' => $svrkey,
				   'SSL_cert_file' => $svrcert,
				   'SSL_verify_mode' => 0,
				   });
	TESTOK();
};


test "new server (without server verification)" => sub {
	$server = new IDX::Test::SSLServer("-verify" => 0)->start();
};

test "new socket (without server verification)" => sub {
	$ssocket = new IDX::SSLSocket("localhost",
									   "Port" => $server->{"port"},
									   "SSL_ca_path" => $cadir,
									   );
};

test "receive one message" => \&TESTreceive;

my $peerDN;

test "peer certificate" => sub {
	$peerDN = $ssocket->getPeerDN;
	TEST($peerDN =~ m/test/i);
};

test "new socket (with client-side server verification)" => sub {
	my $ssocket = new IDX::SSLSocket("localhost",
										  "Port" => $server->{port},
										  "SSL_cert_file" => $clicert,
										  "SSL_key_file" => $clikey,
										  "SSL_ca_path" => $cadir);
	TESTreceive($ssocket);
};

test "new server (with server-side verification)" => sub {
	$server = new IDX::Test::SSLServer("-picky" => 1)->start();
};

test "new socket (with server-side verification)" => sub {
	$ssocket = new IDX::SSLSocket("localhost",
									   "Port" => $server->{port},
									   "SSL_cert_file" => $clicert,
									   "SSL_key_file" => $clikey,
									   "SSL_ca_path" => $cadir,
									   "PeerDN" => $peerDN);
};

test "new socket (with server verif but no check on our part)" => sub {
	my $client = new IDX::SSLSocket("localhost",
										 "Port" => $server->{port},
										 "SSL_cert_file" => $clicert,
										 "SSL_key_file" => $clikey);
	TESTreceive($client);
};

test "send/receive one message" => \&TESTreceive;

test "peer certificate" => sub {
	TEST($ssocket->getPeerDN =~ m/test/i);
};

test "tourist client" => sub {
	local $SIG{"PIPE"} = 'IGNORE';
	try {
		my $c = new IDX::SSLSocket("localhost",
										"Port" => $server->{port});
		TESTFAIL;
	} otherwise {
		TESTOK;
	};
};

test "setSSLcontext meaningful to IO::Socket::SSL::new" => sub {
	SSLscope {
		setSSLcontext({
			'SSL_verify_mode' => 3,
			'SSL_ca_path'     => $cadir,
			'SSL_use_cert'    => 1,
			'SSL_key_file'    => $clikey,
			'SSL_cert_file'   => $clicert,
		});
		my $c=new IO::Socket::SSL(PeerHost=>"localhost",
								  PeerPort => $server->{port});
		TESTreceive($c);
	};

};


# Check reentrance part 1

test "check reentrance (part 1)" => sub {
	TEST($ssocket = new IDX::SSLSocket("localhost",
											"Port" => $server->{port},
											"SSL_cert_file" => $clicert,
											"SSL_key_file" => $clikey,
											"SSL_ca_path" => $cadir,
											"PeerDN" => $peerDN));
};

test "evil client" => sub {
	local $SIG{"PIPE"} = 'IGNORE';
	try {
		my $c = new IDX::SSLSocket("localhost",
										"Port" => $server->{port},
										"SSL_cert_file" => $evilcertfile,
										"SSL_key_file" => $evilkeyfile);
		TESTFAIL;
	} otherwise {
		TESTOK;
	};
};

test "evil server" => sub {
	my $evilserver = new IDX::Test::SSLServer (-evil => 1)->start();

	try {
		my $c = new IDX::SSLSocket("localhost",
										"Port" => $evilserver->{port},
										"SSL_cert_file" => $clicert,
										"SSL_key_file" => $clikey,
										"SSL_ca_path" => $cadir);
		warn "new IDX::SSLSocket worked (w/ params)!";
		TESTFAIL;
	} catch IDX::Error::Network with {
		my $E = shift;
		TEST($E->{"-SSLeayErrors"}->[0] =~ m/GET_SERVER_CERTIFICATE/) ||
			warn $E;
	};

};

test "redo setSSLcontext test w/ correct server again" => sub {
	SSLscope {
		setSSLcontext({
			'SSL_verify_mode' => 3,
			'SSL_ca_path'     => $cadir,
			'SSL_use_cert'    => 1,
			'SSL_key_file'    => $clikey,
			'SSL_cert_file'   => $clicert,
		});
		my $c = new IO::Socket::SSL("PeerHost" => "localhost",
									"PeerPort" => $server->{port});
		TESTreceive($c);
	};
};

test "correct client and server again (checks reentrance part 2)"
	=> \&TESTreceive;

$IDX::Authentication::_forced_auth_result=0;

test "subtly evil server" => sub {
	try {
		$ssocket = new IDX::SSLSocket("localhost",
										   "Port" => $server->{port},
										   "SSL_cert_file" => $clicert,
										   "SSL_key_file" => $clikey,
										   "SSL_ca_path" => $cadir,
										   "PeerDN" => "bogusDN");
		TESTFAIL;
	} catch IDX::Error::Security with {
		TESTOK;
	};
};

use LWP::UserAgent;
test "SSLSocket and LWP: connect without a certificate" => sub {
	my $wwwserver = new IDX::Test::WebServer()->start();
	my $sslserver = new IDX::Test::SSLServer("-server" => $wwwserver)->start();
	SSLscope {
		setSSLcontext({
			"SSL_verify_mode" => 0,
		});
		my $req = LWP::UserAgent->new()->get
			("https://localhost:".$sslserver->getPort()."/index.html");
		TEST($req->code() eq 200) || warn $req->as_string();
	};
};

# Fixes variable suicide problem in 5.6
undef $server;
undef $ssocket;

1;
