#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::Test> - Test cases for IDEALX projects

=head1 SYNOPSIS

 use IDX::Test;

 number_of_tests(3);

 test "network" => sub {
 	if (I_can_connect) {
 		if (I_can_make_request) {
 			TESTOK;
 		}
 	};
 	0; # Default will be to fail test
 }; # Don't forget the semicolon please

 test "symmetry flip / flop" => sub {
 	my $a = new A;
 	my $b = $a->flip(); TESTFAIL if (!defined $b);
 	my $c = $b->flop(); TESTFAIL if (!defined $c);

 	TEST($a eq $c);
 };

 test "This is a developper-friendly test" => sub {
   $DB::single = 1; # This is a breakpoint for the Perl debugger
   my $a = do_stuff();
   TEST($a->{somekey} eq "something") ||
      warn Data::Dumper::Dumper($a);
      # One can always assume that Data::Dumper is loaded in
      # tests; furthermore, messages on STDERR don't interfer
      # with the test harness.
 };

 1;

From inside a module, you can also use the following syntax:

 #!/usr/bin/perl -Tw
 package My::Package;
 sub myfunction {
 	return 1;
 }
 require IDX::Test::Suite unless caller();
 1;

 __END__

 # No need to specify number_of_test in this case, IDX::Test
 # computes the proper number for us.
 test "Some stupid test" => sub {
 	TEST (myfunction ());
 };
 1;

In order to call a module equipped with an IDX::Test suite :

 perl -T -I$PERL5LIB My/Module.pm

short form (more readable, exceptions are saved in a file for debugging):

 perl -MIDX::Test=logfile,/tmp/log -T -I$PERL5LIB My/Module.pm

=head1 DESCRIPTION

B<IDX::Test> is a convenient way of making test cases, for unit,
interoperability and regression tests.  It ls compatible with
B<Test::Harness> but better off than B<Test> from Perl5. It is
developper-friendly in a number of ways (see the SYNOPSIS below).

=cut "

package IDX::Test;

use POSIX;
use strict;
use Exporter;
use Data::Dumper; # Just for the convenience of testers.

use IO::File;
use IO::Pipe;

require 5.6.0;

our $VERSION="1.0";

use vars qw(@ISA @EXPORT @EXPORT_OK);
@ISA=qw(Exporter);
@EXPORT=qw(test integrationtest TESTOK TESTFAIL TESTFAILED TEST number_of_tests
	runCommand newLWPRequest);
@EXPORT_OK=@EXPORT;

our $failed; # modified by calls to "TEST()", "TESTOK() and "TESTFAIL()"
use vars qw($counter $lasttest); # test counter, and number of last test
$lasttest=0;
use vars qw($reversed); # All tests reversed - normally only useful for
                        # testing IDX::Test itself.
use vars qw($FD);     # file descriptor where test results are output

$FD=\*STDOUT;

# Never too careful with %ENV.
%ENV = (
		"PATH"  => "/usr/bin:/bin:/usr/local/bin:/usr/sbin:/sbin:".
		"/usr/local/sbin",
		"DEBUG" => $ENV{"DEBUG"} ? 1 : 0,
		);

#####################################################################

=pod "

=head1 FUNCTIONS

They are all exported by default.

=over 4

=item I<number_of_tests($num)>

Prepares for running the said number of tests.  This must be done
before the first test.  Sure this is a hassle for programmers, but
this allows the test harness to know beforehand how many tests it must
expect (so as to interrupt hanging test suites).

B<Note for the lazy>: you can now omit B<number_of_tests()>
altogether. In this case, B<IDX::Test> will count the number of tests
by parsing the __END__ block. This will only work when using the
recommended form for invoking tests (see L</EXAMPLES>), instead of the
"eval" form recommended in L<perlmodlib>, and then, oviously, only if
all tests are flat (no tests in a loop or conditional).

=cut "

sub number_of_tests {
	($lasttest) = @_;
	$lasttest += 0;
	print $FD ("1..$lasttest\n");

	unless ($] >= 5.006001) {
		$IDX::Error::Internal::inhibitTrace = 1; # OUCH! See
		# perldoc IDX::Error for info on this one.
		# Twice to shut off a "used only once" warning.
		$IDX::Error::Internal::inhibitTrace = 1;
	};
	$counter = 0;
}


=pod "

=item I<test($name,$code)>

Executes I<$code> in a controlled environment (exceptions are trapped
and cause the test to fail) and prints the result in about the same
format as the one used in B<Test::Harness(3)> (only the comments move
around).  If none of the functions below is called during execution of
the test, and no exception is raised, the test succeeds if and only if
the return value of I<$code> is true.

=cut "

sub test($&) {
	my ($name,$code)=@_;

	local $failed=undef;

	my $return;
	local $@=undef;
	my $isok=eval {$return=&$code(); 1};

	$counter++;

	if (! $isok) {
		$failed=1;
		my $exn = ( (ref($@) || (defined $@ && length($@) > 0)) ? $@ :
				   (defined($Error::THROWN) ? $Error::THROWN : undef));

		# Don't let the exception get lost
		if (our $logfd) {
			print $logfd ("Test $counter died with exception".
			  (! defined $exn ? "(undef)" : "\n$exn\n"));
			$name="test ``$name'' died - see log file";
		} else {
			# Output diagnostics on one line after the failure message
			my $message = (! defined $exn ? "an undefined exception" :
						   "$exn" ? "$exn" :
						   (UNIVERSAL::isa($exn, "IDX::Error") ?
							$exn->safe_dumper() : "an empty exception"));
			$message =~ s/\n/ /gs;
			$name="test ``$name'' died with $message";
		};
	}

	if (!defined $failed) {$failed=!($return);};

	# Output compatible with Test::Harness
	if ($counter > $lasttest) {
		print $FD "not ok $counter # Surnumerary test, ".
			"fix number_of_tests() in script\n";
	} else {
		print $FD (($failed xor $reversed) ? "not ok":
			"ok"," $counter # $name\n");
	};
};


=pod "

=item I<integrationtest($name,$code)>

Does exactly the same thing as I<test()> above, except that the test will
not run (and always return true) if launched in a unit test, that is,
if I<IDX::Config::bootstrapped()> is defined and returns true.

=cut "

sub integrationtest {
	unless ( UNIVERSAL::can("IDX::Config","is_bootstrapped") &&
			 IDX::Config::is_bootstrapped()) {
		$counter++;
		print "ok $counter # SKIPPING this integration test\n";
		return 1;
	};
	goto &test;
}

=pod "

=item I<TESTOK()>

When called from within the test snippet, causes the test to succeed
except if I<TESTFAIL()> or I<TEST(false)> is called in the same test
(before or after it). Returns a true value.

=cut "

sub TESTOK() {
	$failed = 0 if (!$failed);
	return 1;
};


=pod "

=item I<TESTFAIL()>

When called from within a code snippet passed to I<test()>, causes the
test to fail. Returns a false value.

=cut "

sub TESTFAIL() {
	$failed = 1;
	return 0;
};


=pod "

=item I<TESTFAILED()>

True if test already failed. Handy in some developper-friendly code such as

   warn Data::Dumper::Dumper($stuff) if TESTFAILED();

=cut "

sub TESTFAILED() {
	$failed;
}

=pod "

=item I<TEST($condition)>

Behaves like I<TESTFAIL()> or I<TESTOK()>, according to the boolean value of
I<$condition>.

=cut "

sub TEST($) {
	my ($value) = @_;
	$failed ||= !$value;
	return $value;
};

=pod "

=item I<import()>

This function does the importing of symbols into packages that say
C<use IDX::Test>. Additionnally, this module recognizes extra
parameters (e.g. C<use IDX::Test logfile => "/tmp/log">, or C<perl
-MIDX::Test=logfile,/tmp/log>, as this feature is intended for):

=over 2

=item I<logfile>

Dumps the test status messages to the next argument.

=back

These settings are process-wide.

=cut "

sub import {
	while(@_ >= 2) {
		if ($_[1] eq "logfile") {
			my (undef, $logfile)=splice @_,1,2;
			our $logfd=new IO::File("> $logfile");
			die "Cannot open $logfile" unless (defined $logfd);
			next;
		};
		last;
	};
	goto &Exporter::import;
}

=pod "

=item I<IDX::Test::inUnitTest ()>

Returns true if and only if the top frame of the call stack is a test
suite.

=cut "

sub inUnitTest {
	my $level=0;
	while(my @caller=caller($level)) {
		return 1 if ($caller[0] eq "IDX::Test::UnitTestSuite" &&
					 $caller[3] eq "IDX::Test::UnitTestSuite::run");
		# New-style invocation
		$level++;
	};

	my @topcaller=caller($level-1);
	return 1 if ($topcaller[3] eq "(eval)" &&
				 defined $topcaller[7]); # Old-style

	return undef;
};

=pod "

=back

=head1 VARIABLES

=over 4

=item I<$IDX::Test::reversed>

If set to true, all tests are reversed.  Only useful for testing
B<IDX::Test> itself; in every other case, you should rewrite
your test rather than use this variable.

=back

=head1 EXTERNAL PROGRAM TESTS

The following functions are I<@EXPORT_OK> and can be used to build
external unit tests for programs, such as CGI scripts:

=over 4

=item I<runCommand(@args)>

Runs the command just like I<system()> does, but C<< 2>/dev/null >>;
returns the text read on F<STDOUT>. I<$?> is set according to the usual
semantics (see L<perlvar>), except when the command is not found (in which
case I<$?> is set to C<<< 165<<8 >>> - this is just a "more random" value
than your average C<<< 255<<8 >>>).

=cut "

use IPC::Open3;

sub runCommand {
	my ($hisfdout);

	local *DEVNULLIN;
	local *DEVNULLOUT;
	local *IPC::Open3::croak = sub {POSIX::_exit(165)}; # Well yes I'm ashamed.
	#  Not for the dirty function override, though, but rather for the magic
	#  165 :-)

	open(DEVNULLOUT,">/dev/null") ||
		die "[runCommand] Cannot open /dev/null?!? ($!)\n";

	open(DEVNULLIN,"</dev/null") ||
		die "[runCommand] Cannot open /dev/null?!? ($!)\n";

	my $pid = open3("<&DEVNULLIN",$hisfdout,">&DEVNULLOUT",@_);

	my @lines = (wantarray ? <$hisfdout> : join('',<$hisfdout>));

	close($hisfdout);
	waitpid($pid,0); # Sets $?

	return wantarray ? @lines : $lines[0];
}


=pod "

=back

=head1 TEST CERTIFICATES AND KEYS

IDEALX develops, among other things, a PKI and security devices
obeying the X.509 standard. Of course, this requires certificates and
private keys.  Here are two functions that emit a tuple of
certificates, keys and CRLs (for two communicating principals and a CA):

=over 4

=cut "

# Well those are always the same, and after 20 years the tests will
# cease to work :-).

# Root CA.
# Certificate:
#     Data:
#         Version: 3 (0x2)
#         Serial Number: 0 (0x0)
#         Signature Algorithm: sha1WithRSAEncryption
#         Issuer: O=Test IDX-PKI Installation, CN=IDX-PKI Root CA
#         Validity
#             Not Before: Mar 21 13:25:59 2003 GMT
#             Not After : Mar 21 13:25:59 2023 GMT
#         Subject: O=Test IDX-PKI Installation, CN=IDX-PKI Root CA
#         Subject Public Key Info:
#             Public Key Algorithm: rsaEncryption
#             RSA Public Key: (512 bit)
#                 Modulus (512 bit):
#                     00:af:4c:70:e1:0b:7c:6b:50:da:2e:86:2c:86:11:
#                     af:0d:5c:94:13:0b:9f:e2:8f:d0:7c:bd:78:6a:8e:
#                     bc:13:e0:4d:f5:a6:26:54:c5:4b:01:52:cb:70:c4:
#                     4b:dc:fd:76:cb:03:91:80:76:bf:30:ff:3a:f8:0f:
#                     27:48:01:07:d3
#                 Exponent: 65537 (0x10001)
#         X509v3 extensions:
#             X509v3 Subject Key Identifier:
#                 E8:CA:2C:37:C4:E1:5A:A6:11:3A:00:80:76:32:0C:46:EB:24:17:FD
#             X509v3 Authority Key Identifier:
#                 keyid:E8:CA:2C:37:C4:E1:5A:A6:11:3A:00:80:76:32:0C:46:EB:24:17:FD
#
#             X509v3 Basic Constraints: critical
#                 CA:TRUE
#             X509v3 Key Usage: critical
#                 Certificate Sign, CRL Sign
#     Signature Algorithm: sha1WithRSAEncryption
#         6f:9c:79:e5:a2:bc:91:71:1e:6b:65:da:b3:18:ad:61:60:f1:
#         0f:24:7a:42:f5:6a:b5:de:b3:8c:6e:a4:7d:44:b1:1f:7f:82:
#         72:a6:f3:5d:e8:fb:14:1d:6f:eb:52:0f:c0:ed:f5:e3:1e:22:
#         45:dc:07:c8:de:d8:bc:7f:98:c3

my $rootcacert = << 'MYDATA';
-----BEGIN CERTIFICATE-----
MIIB0DCCAXqgAwIBAgIBADANBgkqhkiG9w0BAQUFADA+MSIwIAYDVQQKExlUZXN0
IElEWC1QS0kgSW5zdGFsbGF0aW9uMRgwFgYDVQQDEw9JRFgtUEtJIFJvb3QgQ0Ew
HhcNMDMwMzIxMTMyNTU5WhcNMjMwMzIxMTMyNTU5WjA+MSIwIAYDVQQKExlUZXN0
IElEWC1QS0kgSW5zdGFsbGF0aW9uMRgwFgYDVQQDEw9JRFgtUEtJIFJvb3QgQ0Ew
XDANBgkqhkiG9w0BAQEFAANLADBIAkEAr0xw4Qt8a1DaLoYshhGvDVyUEwuf4o/Q
fL14ao68E+BN9aYmVMVLAVLLcMRL3P12ywORgHa/MP86+A8nSAEH0wIDAQABo2Mw
YTAdBgNVHQ4EFgQU6MosN8ThWqYROgCAdjIMRuskF/0wHwYDVR0jBBgwFoAU6Mos
N8ThWqYROgCAdjIMRuskF/0wDwYDVR0TAQH/BAUwAwEB/zAOBgNVHQ8BAf8EBAMC
AQYwDQYJKoZIhvcNAQEFBQADQQBvnHnloryRcR5rZdqzGK1hYPEPJHpC9Wq13rOM
bqR9RLEff4JypvNd6PsUHW/rUg/A7fXjHiJF3AfI3ti8f5jD
-----END CERTIFICATE-----
MYDATA

my (undef) = << 'MYDATA';
-----BEGIN RSA PRIVATE KEY-----
MIIBOwIBAAJBAK9McOELfGtQ2i6GLIYRrw1clBMLn+KP0Hy9eGqOvBPgTfWmJlTF
SwFSy3DES9z9dssDkYB2vzD/OvgPJ0gBB9MCAwEAAQJAfpO5piY/DtZw1Ms5dw1p
00IYAbCRIG0r5AaD3p12HPrWIeES/Jt9NjdniHFsfJQ91sM9KOQKDjDDvO8dQepK
EQIhANQtlr7UihDxlvRR+SCgAvDQKgOtECc+5x9eJJ5iypttAiEA04Dv53FwQ9J9
+c7zkFipaEknzQ3fpdCljSoMfwAy6D8CIAdqzJFBNs5e06sYzn+weEnYzQ8x1pW4
ojW7gcYzp+eFAiEAkLCqbmjaZmt/pv/+/AbLjf5N7ifJkkQUnWfguJlIOj0CIQDG
HrsLe3Xe0BBbYBzULIpdo/fCNr4x2KTALAT/8/4Pkw==
-----END RSA PRIVATE KEY-----
MYDATA

# Certificate Revocation List (CRL):
#         Version 1 (0x0)
#         Signature Algorithm: md5WithRSAEncryption
#         Issuer: /O=Test IDX-PKI Installation/CN=IDX-PKI Root CA
#         Last Update: Mar 21 13:25:59 2003 GMT
#         Next Update: Mar 20 13:25:59 2023 GMT
# No Revoked Certificates.
#     Signature Algorithm: md5WithRSAEncryption
#         31:45:31:1a:1a:09:83:eb:be:62:a0:27:e7:29:6e:a6:66:86:
#         14:01:78:73:d8:aa:c7:e7:76:c5:72:0c:31:bd:39:3f:5b:6e:
#         fd:ff:34:8f:6a:75:f4:dc:e7:3d:4a:bf:5e:8b:b5:ef:36:37:
#         fc:33:f4:d3:bb:bb:bc:08:f9:e7

my $rootcrl = << 'MYDATA';
-----BEGIN X509 CRL-----
MIHBMG0wDQYJKoZIhvcNAQEEBQAwPjEiMCAGA1UEChMZVGVzdCBJRFgtUEtJIElu
c3RhbGxhdGlvbjEYMBYGA1UEAxMPSURYLVBLSSBSb290IENBFw0wMzAzMjExMzI1
NTlaFw0yMzAzMjAxMzI1NTlaMA0GCSqGSIb3DQEBBAUAA0EAMUUxGhoJg+u+YqAn
5ylupmaGFAF4c9iqx+d2xXIMMb05P1tu/f80j2p19NznPUq/Xou17zY3/DP007u7
vAj55w==
-----END X509 CRL-----
MYDATA

# Operational CA.
# Certificate:
#     Data:
#         Version: 3 (0x2)
#         Serial Number: 1 (0x1)
#         Signature Algorithm: sha1WithRSAEncryption
#         Issuer: O=Test IDX-PKI Installation, CN=IDX-PKI Root CA
#         Validity
#             Not Before: Mar 21 13:25:59 2003 GMT
#             Not After : Mar 20 13:25:59 2023 GMT
#         Subject: O=Test IDX-PKI Installation, CN=IDX-PKI Operational CA
#         Subject Public Key Info:
#             Public Key Algorithm: rsaEncryption
#             RSA Public Key: (512 bit)
#                 Modulus (512 bit):
#                     00:d9:8b:d7:be:42:70:b5:fa:40:e5:a8:b2:d4:1b:
#                     a1:15:ca:7c:29:a4:ad:dc:14:48:53:1e:bd:ee:6b:
#                     d3:f1:c6:e0:5b:ad:8d:15:a0:65:d7:24:80:2e:7b:
#                     68:ec:99:94:eb:a3:c1:9f:eb:de:b1:df:5e:b8:1f:
#                     4e:28:67:b2:8f
#                 Exponent: 65537 (0x10001)
#         X509v3 extensions:
#             X509v3 Subject Key Identifier:
#                 CB:8F:4E:71:87:1D:65:AE:78:E9:6C:0F:6A:A7:60:5B:48:C9:9F:D6
#             X509v3 Authority Key Identifier:
#                 keyid:E8:CA:2C:37:C4:E1:5A:A6:11:3A:00:80:76:32:0C:46:EB:24:17:FD
#
#             X509v3 Basic Constraints: critical
#                 CA:TRUE
#             X509v3 Key Usage: critical
#                 Certificate Sign, CRL Sign
#             X509v3 CRL Distribution Points:
#                 URI:http://pkidevel.ird.idealx.com:24329/RootCRL.crl
#
#     Signature Algorithm: sha1WithRSAEncryption
#         9d:64:fd:a8:d0:80:7d:3c:0b:8f:fe:c1:c7:a3:55:c8:b9:9f:
#         cb:b5:e2:c3:6c:92:93:b6:9c:e6:86:dd:b3:19:83:6d:15:4b:
#         77:a0:b5:ed:c4:d2:bc:67:d4:55:51:c4:e4:26:0a:58:44:cd:
#         8b:d8:ed:02:f5:a9:b5:3d:e0:d8

my $cacert = << 'MYDATA';
-----BEGIN CERTIFICATE-----
MIICHDCCAcagAwIBAgIBATANBgkqhkiG9w0BAQUFADA+MSIwIAYDVQQKExlUZXN0
IElEWC1QS0kgSW5zdGFsbGF0aW9uMRgwFgYDVQQDEw9JRFgtUEtJIFJvb3QgQ0Ew
HhcNMDMwMzIxMTMyNTU5WhcNMjMwMzIwMTMyNTU5WjBFMSIwIAYDVQQKExlUZXN0
IElEWC1QS0kgSW5zdGFsbGF0aW9uMR8wHQYDVQQDExZJRFgtUEtJIE9wZXJhdGlv
bmFsIENBMFwwDQYJKoZIhvcNAQEBBQADSwAwSAJBANmL175CcLX6QOWostQboRXK
fCmkrdwUSFMeve5r0/HG4FutjRWgZdckgC57aOyZlOujwZ/r3rHfXrgfTihnso8C
AwEAAaOBpzCBpDAdBgNVHQ4EFgQUy49OcYcdZa546WwPaqdgW0jJn9YwHwYDVR0j
BBgwFoAU6MosN8ThWqYROgCAdjIMRuskF/0wDwYDVR0TAQH/BAUwAwEB/zAOBgNV
HQ8BAf8EBAMCAQYwQQYDVR0fBDowODA2oDSgMoYwaHR0cDovL3BraWRldmVsLmly
ZC5pZGVhbHguY29tOjI0MzI5L1Jvb3RDUkwuY3JsMA0GCSqGSIb3DQEBBQUAA0EA
nWT9qNCAfTwLj/7Bx6NVyLmfy7Xiw2ySk7ac5obdsxmDbRVLd6C17cTSvGfUVVHE
5CYKWETNi9jtAvWptT3g2A==
-----END CERTIFICATE-----
MYDATA

my (undef) = << 'MYDATA';
-----BEGIN RSA PRIVATE KEY-----
MIIBOgIBAAJBANmL175CcLX6QOWostQboRXKfCmkrdwUSFMeve5r0/HG4FutjRWg
ZdckgC57aOyZlOujwZ/r3rHfXrgfTihnso8CAwEAAQJAXuO6hQih9R5lNEdzSUWL
699YbSmhlD9clLTyD48E8Y4J6E2ye/5P2j1ts6Iq2fdvxRX2c3pzPuD0MVUBjWXB
sQIhAO59ucm1mmVo/uwBu3J/rjCRYleyvJlErWBcB24a/zuLAiEA6YR5Gj7dAU2O
Mlqjf3XWJu54Z37hqCzW+piMsyoElY0CIQDOguXWyT85ovJjEJjs66y895ctk2MG
YNa0gtwYNp5eQwIgBc3qKLHZUnf4Cd5g5AEs0ojo60GnIEQFD/OMw8NOm90CIF+O
65K5bYg4GEfsekTSYsmDtmf7SqXisuHBpLniLsPh
-----END RSA PRIVATE KEY-----
MYDATA

# Certificate Revocation List (CRL):
#         Version 1 (0x0)
#         Signature Algorithm: sha1WithRSAEncryption
#         Issuer: /O=Test IDX-PKI Installation/CN=IDX-PKI Operational CA
#         Last Update: Mar 21 13:26:34 2003 GMT
#         Next Update: Mar 19 13:26:34 2023 GMT
# No Revoked Certificates.
#     Signature Algorithm: sha1WithRSAEncryption
#         5b:dd:09:2e:b1:4a:78:cf:e5:5b:4f:7f:03:62:a6:9d:07:db:
#         3c:66:2c:fd:48:72:5d:68:78:62:33:30:5f:b8:55:96:0a:0b:
#         d6:33:ce:8e:f2:70:22:46:f6:6c:5f:65:73:25:78:15:38:e8:
#         b7:e8:43:fa:c1:90:8b:a5:dd:52

my $crl = << 'MYDATA';
-----BEGIN X509 CRL-----
MIHIMHQwDQYJKoZIhvcNAQEFBQAwRTEiMCAGA1UEChMZVGVzdCBJRFgtUEtJIElu
c3RhbGxhdGlvbjEfMB0GA1UEAxMWSURYLVBLSSBPcGVyYXRpb25hbCBDQRcNMDMw
MzIxMTMyNjM0WhcNMjMwMzE5MTMyNjM0WjANBgkqhkiG9w0BAQUFAANBAFvdCS6x
SnjP5VtPfwNipp0H2zxmLP1Icl1oeGIzMF+4VZYKC9Yzzo7ycCJG9mxfZXMleBU4
6LfoQ/rBkIul3VI=
-----END X509 CRL-----
MYDATA


# First cert, named "client" (but can be of any use).
# Certificate:
#     Data:
#         Version: 3 (0x2)
#         Serial Number: 369 (0x171)
#         Signature Algorithm: sha1WithRSAEncryption
#         Issuer: O=Test IDX-PKI Installation, CN=IDX-PKI Operational CA
#         Validity
#             Not Before: Mar 21 13:26:46 2003 GMT
#             Not After : Mar 19 13:26:46 2023 GMT
#         Subject: O=Test IDX-PKI Installation, OU=IDX-PKI, CN=Administrator
#         Subject Public Key Info:
#             Public Key Algorithm: rsaEncryption
#             RSA Public Key: (512 bit)
#                 Modulus (512 bit):
#                     00:a1:80:6a:cb:32:1b:90:f6:8f:e7:a1:be:cb:73:
#                     32:ea:d8:f9:05:b3:27:56:db:b9:b1:7a:12:21:f4:
#                     d4:c2:40:3b:72:45:f5:00:a6:5d:f4:8b:cc:69:74:
#                     4f:54:7b:02:aa:fb:3a:1d:60:ab:0b:ae:63:b1:04:
#                     37:b4:87:83:af
#                 Exponent: 65537 (0x10001)
#         X509v3 extensions:
#             X509v3 Subject Key Identifier:
#                 4F:FE:AD:C7:1E:34:27:CA:DA:15:B5:15:EB:69:B3:27:E9:8E:AA:62
#             X509v3 Authority Key Identifier:
#                 keyid:CB:8F:4E:71:87:1D:65:AE:78:E9:6C:0F:6A:A7:60:5B:48:C9:9F:D6
#
#             X509v3 Extended Key Usage:
#                 E-mail Protection, TLS Web Client Authentication
#             Netscape Cert Type:
#                 SSL Client, S/MIME
#             X509v3 Key Usage: critical
#                 Digital Signature, Key Encipherment
#     Signature Algorithm: sha1WithRSAEncryption
#         5b:41:da:35:28:15:1f:dd:46:a6:b0:e8:ab:fc:b6:6d:5f:2c:
#         e5:37:32:71:51:61:ca:45:77:3b:2b:a2:c3:a3:3d:2c:f2:21:
#         25:1f:05:55:7c:28:e6:4b:a1:69:f2:a9:d6:d2:13:6e:0c:42:
#         13:54:33:85:60:40:c6:4e:f9:0a

my $cert1 = << 'MYDATA';
-----BEGIN CERTIFICATE-----
MIICCzCCAbWgAwIBAgICAXEwDQYJKoZIhvcNAQEFBQAwRTEiMCAGA1UEChMZVGVz
dCBJRFgtUEtJIEluc3RhbGxhdGlvbjEfMB0GA1UEAxMWSURYLVBLSSBPcGVyYXRp
b25hbCBDQTAeFw0wMzAzMjExMzI2NDZaFw0yMzAzMTkxMzI2NDZaME4xIjAgBgNV
BAoTGVRlc3QgSURYLVBLSSBJbnN0YWxsYXRpb24xEDAOBgNVBAsTB0lEWC1QS0kx
FjAUBgNVBAMTDUFkbWluaXN0cmF0b3IwXDANBgkqhkiG9w0BAQEFAANLADBIAkEA
oYBqyzIbkPaP56G+y3My6tj5BbMnVtu5sXoSIfTUwkA7ckX1AKZd9IvMaXRPVHsC
qvs6HWCrC65jsQQ3tIeDrwIDAQABo4GFMIGCMB0GA1UdDgQWBBRP/q3HHjQnytoV
tRXrabMn6Y6qYjAfBgNVHSMEGDAWgBTLj05xhx1lrnjpbA9qp2BbSMmf1jAdBgNV
HSUEFjAUBggrBgEFBQcDBAYIKwYBBQUHAwIwEQYJYIZIAYb4QgEBBAQDAgWgMA4G
A1UdDwEB/wQEAwIFoDANBgkqhkiG9w0BAQUFAANBAFtB2jUoFR/dRqaw6Kv8tm1f
LOU3MnFRYcpFdzsrosOjPSzyISUfBVV8KOZLoWnyqdbSE24MQhNUM4VgQMZO+Qo=
-----END CERTIFICATE-----
MYDATA

my $key1 = << 'MYDATA';
-----BEGIN RSA PRIVATE KEY-----
MIIBOgIBAAJBAKGAassyG5D2j+ehvstzMurY+QWzJ1bbubF6EiH01MJAO3JF9QCm
XfSLzGl0T1R7Aqr7Oh1gqwuuY7EEN7SHg68CAwEAAQJAdgHOhuD7MYZoCjmunxxy
1RVriFWq5hWxbtQikbtDBxgc7imY8y7St04SlhFj5EQGxcqvggUvrjJYe8Tidzid
4QIhAM01y2JBX7Kjvq9as2Zbp6LH0+5ZpGIMQrIPgiTzbuafAiEAyXk7eufSGhvu
ZHZaLCJ0bO1O7HQrIQeUEsP/fLyOmPECIFeSEEPSwALcq5/rJcmVbVkiy0bYjDSn
JMMVqSUZmQbjAiEAoNLPMgq+4eyN3U/EJxrRyOmdaI5KpzD/eANQe+egBGECIBIQ
0jSsvZfdjXYiL9ivRlbEIBBzmdrV7qpxGSZCQuAO
-----END RSA PRIVATE KEY-----
MYDATA

# Second certificate, named "server".
# Certificate:
#     Data:
#         Version: 3 (0x2)
#         Serial Number: 341 (0x155)
#         Signature Algorithm: sha1WithRSAEncryption
#         Issuer: O=Test IDX-PKI Installation, CN=IDX-PKI Operational CA
#         Validity
#             Not Before: Mar 21 13:26:23 2003 GMT
#             Not After : Mar 19 13:26:23 2023 GMT
#         Subject: O=Test IDX-PKI Installation, OU=IDX-PKI, CN=pkidevel.ird.idealx.com
#         Subject Public Key Info:
#             Public Key Algorithm: rsaEncryption
#             RSA Public Key: (512 bit)
#                 Modulus (512 bit):
#                     00:a3:e3:63:b4:09:5d:0a:19:68:13:d3:fe:f5:bf:
#                     6f:93:63:ed:ec:2f:9e:be:e0:90:cc:70:fe:60:67:
#                     a8:ad:3e:d8:5b:38:de:d6:e6:33:79:12:1d:3b:91:
#                     6f:a5:53:01:5a:5a:93:5e:f6:38:33:10:fe:0b:45:
#                     36:c3:53:a0:9f
#                 Exponent: 65537 (0x10001)
#         X509v3 extensions:
#             X509v3 Subject Key Identifier:
#                 59:EB:19:F8:43:17:CB:5B:0D:FD:0F:8C:B6:BC:90:96:13:65:63:C4
#             X509v3 Authority Key Identifier:
#                 keyid:CB:8F:4E:71:87:1D:65:AE:78:E9:6C:0F:6A:A7:60:5B:48:C9:9F:D6
#
#             X509v3 Extended Key Usage:
#                 E-mail Protection, TLS Web Client Authentication, TLS Web Server Authentication
#             Netscape Cert Type:
#                 SSL Client, SSL Server, S/MIME
#             X509v3 Key Usage: critical
#                 Digital Signature, Key Encipherment
#     Signature Algorithm: sha1WithRSAEncryption
#         3e:be:f0:9f:07:c7:c3:98:09:a9:78:02:b4:8c:f1:98:ff:f2:
#         68:56:b8:e9:2e:b9:fd:33:0e:01:d9:ec:0b:30:41:3e:7f:fb:
#         6b:65:ee:e7:31:05:ec:1d:6b:58:c0:7d:fb:21:bb:fc:6e:ac:
#         47:3e:e6:07:03:56:07:50:6a:65

my $cert2 = << 'MYDATA';
-----BEGIN CERTIFICATE-----
MIICHzCCAcmgAwIBAgICAVUwDQYJKoZIhvcNAQEFBQAwRTEiMCAGA1UEChMZVGVz
dCBJRFgtUEtJIEluc3RhbGxhdGlvbjEfMB0GA1UEAxMWSURYLVBLSSBPcGVyYXRp
b25hbCBDQTAeFw0wMzAzMjExMzI2MjNaFw0yMzAzMTkxMzI2MjNaMFgxIjAgBgNV
BAoTGVRlc3QgSURYLVBLSSBJbnN0YWxsYXRpb24xEDAOBgNVBAsTB0lEWC1QS0kx
IDAeBgNVBAMTF3BraWRldmVsLmlyZC5pZGVhbHguY29tMFwwDQYJKoZIhvcNAQEB
BQADSwAwSAJBAKPjY7QJXQoZaBPT/vW/b5Nj7ewvnr7gkMxw/mBnqK0+2Fs43tbm
M3kSHTuRb6VTAVpak172ODMQ/gtFNsNToJ8CAwEAAaOBjzCBjDAdBgNVHQ4EFgQU
WesZ+EMXy1sN/Q+MtryQlhNlY8QwHwYDVR0jBBgwFoAUy49OcYcdZa546WwPaqdg
W0jJn9YwJwYDVR0lBCAwHgYIKwYBBQUHAwQGCCsGAQUFBwMCBggrBgEFBQcDATAR
BglghkgBhvhCAQEEBAMCBeAwDgYDVR0PAQH/BAQDAgWgMA0GCSqGSIb3DQEBBQUA
A0EAPr7wnwfHw5gJqXgCtIzxmP/yaFa46S65/TMOAdnsCzBBPn/7a2Xu5zEF7B1r
WMB9+yG7/G6sRz7mBwNWB1BqZQ==
-----END CERTIFICATE-----
MYDATA

my $key2 = << 'MYDATA';
-----BEGIN RSA PRIVATE KEY-----
MIIBPAIBAAJBAKPjY7QJXQoZaBPT/vW/b5Nj7ewvnr7gkMxw/mBnqK0+2Fs43tbm
M3kSHTuRb6VTAVpak172ODMQ/gtFNsNToJ8CAwEAAQJAHNVCk0gacE5r+yEO+BlH
/fxUvXrbSOWlU/KFLlzOVjwyJMHnfk+oUAEbiV0zcPk4hCsFYRSwfiJGP8kWaKb5
YQIhANNC5f+7FVt+SQsHypAR95kSSORAjW3iX/ZHYplarmYpAiEAxphBjDK0JMPS
5R6q7JUpun2LRwdzMNSD/ocE+xEL2YcCIQClkU50B4kLdYOFU4vbMVmc64mY6UZu
EfYnK3cIvNrlUQIhAJdz7ik5UeDn5qINbVTXvOcYb2tP5y1KJQ+rcMT4Dw1pAiEA
mCX0SFMYovBkQIvRwKKxeTWw8R7sLJDRGEnRWdfbRlg=
-----END RSA PRIVATE KEY-----
MYDATA

# Used in getCertDN() below. Keep them in the "right" order please
# (reversed from openssl), comma-separated and with no extra spaces.
my %DNs = (
	$rootcacert => 'CN=IDX-PKI Root CA,O=Test IDX-PKI Installation',
	$cacert     => 'CN=IDX-PKI Operational CA,O=Test IDX-PKI Installation',
    $cert1      => 'CN=Administrator,OU=IDX-PKI,O=Test IDX-PKI Installation',
    $cert2      => 'CN=pkidevel.ird.idealx.com,OU=IDX-PKI,'.
		   'O=Test IDX-PKI Installation',
);


=pod "

=item I<IDX::Test::certificates()>

Generates a CA certificate, a server certificate and associated key, and a
client certificate and associated key.  Usage example:

 my ($ca, $clicert, $clikey, $srvcert, $srvkey, $crl,
 	$rootca, $rootcrl) = IDX::Test::certificates();

All values will be PEM-encoded in strings.  It is guaranteed that all
invocations of this function from the same process will return the same
result (and thus they can be used to test client/server interoperation).

=cut "

sub certificates {
	return ($cacert, $cert1, $key1, $cert2, $key2, $crl,
			$rootcacert, $rootcrl);
}


=pod "

=item I<IDX::Test::certFiles()>

Writes certificates and keys (all produced by the last function) into
files.

The files will be created using B<IDX::Test::Filename>.

=cut "

sub certFiles {
	return _associatesCertsAndFiles (certificates());
}


=pod "

=item I<IDX::Test::_associatesCertsAndFiles(@certs)>

Takes PEM certificates and returns the array of the associated files.

=cut "

sub _associatesCertsAndFiles {
	my @pems = @_;

	local *PEM;

	return map {
		my $file = new IDX::Test::Filename();
		open(PEM,">$file") || die "Cannot write in $file: $!\n";
		print PEM $_;
		close(PEM);
		"$file";
	} @pems;
}


=pod "

=item I<IDX::Test::evilCerts()>

=item I<IDX::Test::evilCertFiles()>

Usage examples:

 my ($evilcerttxt, $evilkeytxt) = IDX::Test::evilCerts();
 my ($evilcertfile, $evilkeyfile) =
 	IDX::Test::evilCertFiles();

Returns a certificate-and-key that looks just like a regular one (validity
et al), except that it is signed by a different CA.  If connections fail
with the evil certificate but not with the right one, this should capture
a fairly complete part of our code (assuming that openssl itself is
bug-free, of course).

=cut "

my $evilcert = << 'MYDATA';
-----BEGIN CERTIFICATE-----
MIICCzCCAbWgAwIBAgICAnAwDQYJKoZIhvcNAQEFBQAwRTEiMCAGA1UEChMZVGVz
dCBJRFgtUEtJIEluc3RhbGxhdGlvbjEfMB0GA1UEAxMWSURYLVBLSSBPcGVyYXRp
b25hbCBDQTAeFw0wMzAzMjExNDQyMzFaFw0yMzAzMTkxNDQyMzFaME4xIjAgBgNV
BAoTGVRlc3QgSURYLVBLSSBJbnN0YWxsYXRpb24xEDAOBgNVBAsTB0lEWC1QS0kx
FjAUBgNVBAMTDUFkbWluaXN0cmF0b3IwXDANBgkqhkiG9w0BAQEFAANLADBIAkEA
0oetM0A89rYtkSFgZuMoog9gn7zBO8/jtOkSnE7TaG2vFfUpfqmIGIKD0n7GfVmJ
1bC5sPG3xhnJ92SfRXB7ewIDAQABo4GFMIGCMB0GA1UdDgQWBBRiBDyWOH05XpJD
In1jPI2EoO7FqTAfBgNVHSMEGDAWgBTvhhFArQiqUycobS26gdwjB/cYtjAdBgNV
HSUEFjAUBggrBgEFBQcDBAYIKwYBBQUHAwIwEQYJYIZIAYb4QgEBBAQDAgWgMA4G
A1UdDwEB/wQEAwIFoDANBgkqhkiG9w0BAQUFAANBAGS+EgT0++bl0Oa4FrrSz+tt
KQWYXDr+9qFo6NYx8wxwc7wh5JGJPZXbqotVkKm9p3xdYCOY1gMoCPJmW4Q40J0=
-----END CERTIFICATE-----
MYDATA

my $evilkey = << 'MYDATA';
-----BEGIN RSA PRIVATE KEY-----
MIIBOwIBAAJBANKHrTNAPPa2LZEhYGbjKKIPYJ+8wTvP47TpEpxO02htrxX1KX6p
iBiCg9J+xn1ZidWwubDxt8YZyfdkn0Vwe3sCAwEAAQJBAMM0JfXlVCF1HZ75kYcU
jPJmy7KQs6KfyYROr4Da3c5uZevxr04A/0Bl3rT+pa68nbD5xWJM+sOw+u9Msosq
rHECIQDzmOYYlNrpV/tCOxjv7o9APWFVFGnvKlDrAcTiRg8oZwIhAN0/xYD+f+BF
+rON/Suge6gMrBVVR9YE/40vrdXSCTfNAiEAlB7Cuuu4eO7AnzMkJLCgdmVcS8mP
6JeIgH2g5B3Jlb0CIDedgi4sy0Q7Ewvzk0JWMv9JQDCUoVwhIYQCCjaHd525AiAj
Tnz4rDq8gciF7m+7beqIwBscgyHLacM26sxC7mmNEA==
-----END RSA PRIVATE KEY-----
MYDATA

$DNs{$evilcert} = 'CN=Administrator,OU=IDX-PKI,O=Test IDX-PKI Installation';


sub evilCerts {
	return ($evilcert, $evilkey);
}


sub evilCertFiles {
	return _associatesCertsAndFiles (evilCerts());
}

=pod "

=item I<IDX::Test::getCertDN ($cert)>

Returns the DN of a certificate, in string form.  It is only
guaranteed to work with certificates created from one of the above
functions (but it works with both files, strings and objects).

If called in an array context, returns the list of the RDN keys and values
(from smallest to biggest, like (C<CN>, C<my CN>, C<O>, C<my
organisation>, C<C>, C<my country>)), or the empty list upon failure. In
scalar context, returns the full DN as a string (same order: C<CN=my
CN,O=my organisation,C=my country>).

=cut "

sub getCertDN {
	my ($cert_or_file) = @_;

	$cert_or_file = $cert_or_file->serialize() if (ref ($cert_or_file));
	if (! $DNs{$cert_or_file}) {
		local *FILE;
		open (FILE, "<", $cert_or_file) || die "Can't read $cert_or_file: $!";
		$cert_or_file = join ('', <FILE>);
		close FILE;
	}

	if ($DNs{$cert_or_file}) {
		return wantarray ? split (m/\s*[,\/\|=]\s*/, $DNs{$cert_or_file}) :
			$DNs{$cert_or_file};
	} else {
		die "Unknown certificate for getCertDN(): $cert_or_file";
	}
}

=pod "

=item I<IDX::Test::getCertHash ($cert)>

Returns the certificate hash, if known (only the operational CA and the
root CA are known).

=cut "

my %hashs = (
	$rootcacert => '227329c4',
	$cacert     => 'b7b84e18',
);

sub getCertHash {
	my ($cert_or_file) = @_;

	$cert_or_file = $cert_or_file->serialize() if (ref ($cert_or_file));
	if (! $hashs{$cert_or_file}) {
		local *FILE;
		open (FILE, "<", $cert_or_file) || die "Can't read $cert_or_file: $!";
		$cert_or_file = join ('', <FILE>);
		close FILE;
	}

	$hashs{$cert_or_file} ? return $hashs{$cert_or_file} :
		die "Unknown certificate for getCertHash(): $cert_or_file";
}

=pod "

=item I<IDX::Test::newLWPRequest ($protocol, $server, $port, $uri)>

Convenience method, returns an object of class B<HTTP::Request>, ready
to use in a B<LWP::UserAgent> (see L<Crypt::SSLeay>, quite
unexpectedly, for an example).

=cut "

sub newLWPRequest {
	my ($proto, $host, $port, $uri) = @_;
	die "Missing argument in newLWPRequest" unless ($proto && $host &&
		$port && $uri);
	die "Unsupported protocol $proto" unless ($proto eq 'http');

	require HTTP::Request::Common;

	my $r = HTTP::Request->new ('GET', "$proto://$host".
		($port ? ":$port" : "").($uri =~ m|^/| ? $uri : "/$uri"));
	$r->header ('Host', $host) if ($proto =~ m/^http/ &&
		! defined ($r->header ('Host')));
	return $r;
}


######################### TEST RESOURCE MANAGER ###################

=pod "

=back

=head1 RESOURCE ALLOCATION IN TESTS

Additionnally, it is possible from within B<IDX::Test> to obtain
resources that are useful for the tests.  These resources are standardized
and part of the test suite.  At the moment, the following
classes are available:

=over 4

=item B<IDX::Test::Directory>

 my $directory = new IDX::Test::Directory();

Returns the name of a newly created directory.  You can do whatever you
want inside.  It will be removed with its contents when I<$directory> falls
out of scope (typically at the end of the tests).

=cut "

package IDX::Test::Directory;

use vars qw($tmpdir $counter); # Directory where test files are created

$tmpdir="/tmp";
$counter=0;

use overload '""' => \&getname;

sub new {
	my ($class)=@_;
	my $self=bless({
		"name"=>"$tmpdir/IDX-test-$$-$counter"
	},$class);
	$counter++;

	mkdir($self->{name});
	(-d $self->{name}) || die "Unable to create $tmpdir for temp files: $!\n";
	return $self;
};


sub getname {
	my $self=shift;
	return $self->{name};
}


sub DESTROY {
	my ($self)=@_;
	system("/bin/rm","-rf",$self->{name});
};


=pod "

=item B<IDX::Test::Filename>

 my $file = new IDX::Test::Filename();

Returns the name of a temporary file.  All temporary files allocated
through this call are in the same temporary directory.  This is a
filename, not a filehandle: You still have to open it (but since the
holding directory is created with B<IDX::Test::Directory>, you may
safely forget to unlink it).

=cut "

package IDX::Test::Filename;

use overload '""' => \&getname;

use vars qw($tmpdir $counter);
$counter=0;

sub new {
	my ($class)=@_;
	$tmpdir=new IDX::Test::Directory unless
		(defined $tmpdir && -d "$tmpdir");
	my $self=bless({
		"dir"=>$tmpdir, # Creates a reference
		"name"=>"$tmpdir/test-$counter",
	},$class);
	$counter++;

	# Creation is up to caller (this is a filename, not a file).

	return $self;
};

sub getname {
	my $self=shift;
	return $self->{name};
}


=pod "

=item B<IDX::Test::PortRange>

 my $baseport = new IDX::Test::PortRange($n);

Returns a range of I<$n> currently inoccupied network ports.  No
allocation is made on them, though, so this is not at all
race-condition-proof.

=cut "

package IDX::Test::PortRange;

sub new {
	my (undef,$n)=@_;

	my $netstat=new IO::Pipe;
	$netstat->reader("/bin/netstat","-ant") ||
	  die "Can't fork a netstat command: $!\n";

	my %openports;
	while(<$netstat>) {
		next unless (m/LISTEN$/);
		my (undef,undef,undef,$local,undef,undef)=split m/\s/;
		my ($address,$port)=split m/:/,$local;
		$openports{$port}=1;
	}
	$netstat->close();

	my $baseport;
	TRY:do {
		$baseport=int(rand(40000)+4000);
		for(my $i=0; $i<$n; $i++) {
			redo TRY if ($openports{$baseport+$i});
		};
	};
	return $baseport;
}


=pod "

=item B<IDX::Test::NetworkServer>

This class is an ancestor for all test network servers.  A test network
server is handy for unit testing classes that talk to the network, such as
LDAP/S and HTTP/S clients.

Any B<IDX::Test::NetworkServer> will be shut down, and all its
resources freed, when its last reference falls out of scope.

 my $server=new IDX::Test::NetworkServer();

 my $server=new IDX::Test::NetworkServer(
 	-command=>["echo","goodbye"]);

Returns a newly created network server.  It serves on localhost
(C<127.0.0.1>), its port is I<< $server->{port} >> (determined at
random) and a new directory, I<< $server->{dir} >>, is available for
perusal.  If you pass a I<-port> option, you may specify the port as
you like; If you pass a I<-sock> option, you get a server for
a UNIX domain socket instead.

If the I<-command> parameter is set, and is a list reference, the list
will be cast into a shell command and run every time someone
connects (see below).

If the I<-nohalf> parameter is set to a true value, the connection is
unable to handle the TCP half-open state (this is the case over SSL,
see RFC2246 section 7.2.1) and the demo server should refrain from
waiting for the termination of the input stream (see L</_answer>).

=cut "

package IDX::Test::NetworkServer;

sub new {
	die("WRONG_NUMBER_ARGS") unless (@_ % 2);
	my($class,%opts)=@_;

	my $self;
	if (ref($class)) {
		$self=$class;
	} else {
		$self=bless {},$class;
	};

	foreach my $field (qw(command port sock nohalf)) {
		$self->{$field}=$opts{-$field} if ($opts{-$field});
	};

	$self->{dir} ||= new IDX::Test::Directory();
	if (! $self->{sock}) {
		$self->{port} ||= new IDX::Test::PortRange(1);
	};
	return $self;
}

=pod "

 $server->start();

Starts the server and makes it ready to answer network queries.
This method may to be overridden in subclasses; in the
superclass, it starts a forked mini-daemon in Perl that repeatedly
runs I<$server->_answer()> on every request.  Returns I<$self>.

=cut "

sub start {
	my ($self)=@_;

	require IO::Pipe;
	require Socket; import Socket;

	local *Server; local *Client;
	die("No server endpoint specified") unless
	  ($self->{port} || $self->{sock});

	socket(Server, ($self->{port}?PF_INET():PF_UNIX()),
		   SOCK_STREAM(),
		   ($self->{port} ? getprotobyname("tcp") : 0)) ||
			 die("cannot socket(): $!\n");
	setsockopt(Server, SOL_SOCKET(), SO_REUSEADDR(),
			   pack("l", 1)) ||
				 die("cannot setsockopt(): $!\n");

	bind(Server,($self->{port} ?
				 pack_sockaddr_in($self->{port}, INADDR_ANY()) :
				 pack_sockaddr_un($self->{sock}))
		 ) ||
	  die("cannot bind(): $!\n");

	listen(Server,SOMAXCONN()) ||
	  die("cannot listen(): $!\n");

	die("Cannot fork() a Perl test server daemon")
	  unless (defined(my $pid=fork()));
	if ($pid) {
		close Server;
		$self->{pid}=$pid;
		return $self;
	}

	# Ye olde main loop.
	while(my $paddr = accept(Client,Server)) {
		eval {
			$self->_answer(\*Client);
			1;
		} || do { require POSIX; POSIX::_exit(1); };
		close Client;
	};
	require POSIX; POSIX::_exit(0); # Huh?
	# Should never return in the child.
}

=pod "

 $server->_answer($sockref);

Answer an incoming network request open on $sockref. This method
should not be called directly; it may be overridden in subclasses. In
this class, if the I<-command> parameter was set in the constructor,
then the request will be answered with a shell command whose args are
the value of the I<-command> option; by default, the server just says
"hello" and optionally copies the input over (except if the I<-nohalf>
option was specified at server construction time).

=cut "

sub _answer {
	my ($self,$sockref)=@_;
	print $sockref ("hello\n",($self->{nohalf} ? "" : join('',<$sockref>)));
}

=pod "

 $server->DESTROY();

This method is called automatically when the last reference scopes out,
and should never be called directly (not even from another DESTROY
method).  It calls the I<$self->{sysv}> command with argument C<stop>.

=cut "

sub DESTROY {
	my ($self)=@_;
	return undef if (!defined $self->{dir}); # "protection" against
	# bogus global destruction sequences
	if (defined $self->{sysv}) {
		system("sh","$self->{sysv}","stop");
	} elsif (defined $self->{pid}) {
		kill KILL => $self->{pid}; # We don't have anything useful
		# in the test server, now do we? :-) Furthermore, this
		# ensures that a forked Perl daemon doesn't try to do global
		# destruction.
	} elsif (defined $self->{pidfile}) {
		local *PIDFILE;
		open(PIDFILE,"<",$self->{pidfile}) && do {
			my $pid=<PIDFILE>; $pid =~ m/^(\d+)$/; $pid=$1;
			close(PIDFILE);
			kill INT => $pid;		# Be nicer with foreign programs
		};
	};
}

=pod "

=item I<getPort()>

Returns the port number this server operates at. This routine is best
used by other classes instead of I<< $self->{port} >> (yeah, let's do
our encapsulation homework right down to the test module :-)

=cut "

sub getPort {
	my ($self)=@_;
	return $self->{port};
}



=pod "

=item B<IDX::Test::SSLServer>

Wraps an existing TCP server into an SSL shroud. This class is a
wrapper for all SSL-enabled network servers; it provides interface
for querying its X.509 identity (see below).

Usage examples:

 my $server = new IDX::Test::SSLServer (
 	-arg1 => val1, ...);

 my $server = new IDX::Test::SSLServer (-server=>$plainserver,
 	-evil=>1);

The I<-server> argument specifies the port (or the server object) to
SSLize.  An C<stunnel> will be launched so that the far end of the
tunnel reaches that port. (Yes I know this gratuitously adds stunnel
as a dependency for making tests. If you want to reimplement this in
Perl using IO::Socket::SSL in a forked child, be my guest!). If no
I<-server> is specified, "new IDX::Test::NetworkServer(-nohalf=>1)"
will be your talking party.

If I<-evil> is specified to a true value, the SSL personality of the
server will be that of L</evilCerts>. If not, the I<second> user
certificate from L</certificates> will be used. In both cases, $server
will contain the fields named I<$server->{mycert}>, I<$server->{.mykey}>,
and in case the server is not evil, I<$server->{ca}> (operational CA
file), I<$server->{rootca}> (root CA file), I<$server->{capath}>
(directory to hashed CAs), I<$server->{acceptedcert}> and
I<$server->{acceptedkey}>. The latter parameters are adequate for
establishing a secure connection with this server (that is, they can be
queried from code using this class).

You may specify your own certificates and CA stores with I<-cert>,
I<-key>, I<-cafile> and I<-cadir> options. By default, sensible values
will be chosen according to the setting of I<-evil> (see above).

If I<-picky> is specified to a true value, the SSL server will perform
X509 client authentication: a valid SSL certificate w.r.t.
$server->{ca} is therefore mandatory for connecting. The default is
to accept any connection, with or without a user certificate.

The stunnel will emit logs into I<$server->logfile()>, which is handy
for debugging.  This transport server will be shut down, and its
resources freed, when I<$server> falls out of scope.

=cut "

package IDX::Test::SSLServer;

@IDX::Test::SSLServer::ISA = qw(IDX::Test::NetworkServer);

use IO::File;

sub new {
	my ($class, %params) = @_;

	my $self = IDX::Test::NetworkServer::new($class);

	if (defined (my $plainserver = $params{-server})) {
		$self->{"wrapport"} = (ref($plainserver) ?
							   $plainserver->getPort():
							   $plainserver);
	} else {
		$self->{"wrapserver"} = new IDX::Test::NetworkServer(-nohalf=>1)
			->start();
		$self->{"wrapport"} = $self->{"wrapserver"}->getPort();
	}

	if ($params{"-key"} && $params{"-cert"}) {
		($self->{"mycert"}, $self->{".mykey"}) =
			($params{"-cert"}, $params{"-key"});
	} elsif ($params{-evil}) {
		($self->{"mycert"}, $self->{".mykey"}) = IDX::Test::evilCerts();
	} else {
		($self->{"ca"}, $self->{"acceptedcert"}, $self->{"acceptedkey"},
		 $self->{"mycert"}, $self->{".mykey"}, undef, $self->{"rootca"}) =
			 IDX::Test::certificates();
	}

	$self->{"pidfile"} = $self->{"dir"}."/stunnel.pid";
	$self->{"logfile"} = $self->{"dir"}."/stunnel.log";

	open (FILE, ">", "$self->{dir}/bundle.cert") || die
		"Cannot open $self->{dir}/bundle.cert: $!\n";
	print FILE $self->{".mykey"};
	print FILE $self->{"mycert"};
	close FILE;
	chmod (0600, "$self->{dir}/bundle.cert");

	$self->{"picky"} = 1 if ($params{"-picky"});

	if ($params{"-cafile"}) {
		$self->{"cafile"} = $params{"-cafile"};
	} elsif ($params{"-cadir"}) {
		$self->{"cadir"} = $params{"-cadir"};
	} elsif ($self->{"ca"}) {
		$self->{"cadir"} = new IDX::Test::Directory ();
		$self->{"cafile"} = "$self->{cadir}/ca-bundle.crt";

		local *BUNDLE;
		open(BUNDLE, ">", $self->{cafile}) or
		  die "Cannot open $self->{cafile}: $!";

		my ($cahash, $rootcahash) = map {IDX::Test::getCertHash ($_)}
			($self->{"ca"}, $self->{"rootca"});
		my %files = ("ca"     => "$self->{'cadir'}/$cahash.0",
					 "rootca" => "$self->{'cadir'}/$rootcahash.0",
					 );
		foreach my $entity ("ca", "rootca") {
			local *FILE;
			open (FILE, ">", $files{$entity}) || die
				"Cannot open $files{$entity}: $!\n";
			print FILE $self->{$entity};
			close FILE;

			print BUNDLE $self->{$entity} or
			  die "Cannot print $self->{cafile}: $!";
		}

		close(BUNDLE) || die "Cannot close $self->{cafile}: $!";

	};
	return $self;
}

=pod "

=item I<start()>

See L</IDX::Test::NetworkServer> - this is an overloaded method.

=cut "

sub start {
	my ($self) = @_;
	my @verifparams;
	if ($self->{"picky"}) {
		@verifparams = ("-v", "2");
		if (defined $self->{"cafile"}) {
			push (@verifparams, "-A", $self->{"cafile"});
		} elsif (defined $self->{"cadir"}) {
		    push (@verifparams, "-a", $self->{"cadir"});
		} else {
			die "How can I be picky without a CA?";
		}
	}

	system("/usr/sbin/stunnel",
		   "-D","debug","-P", $self->{'pidfile'}, @verifparams,
		   "-p","$self->{dir}/bundle.cert","-d",$self->{port},
		   "-r",$self->{wrapport}, "-o", $self->{logfile});
	sleep(1); # Yet another daemon that fork()s too early.
	          # ANYBODY IN THERE GROKKING UNIX?!
	return $self;
}

sub getDN {
	my ($self)=@_;
	return IDX::Test::getCertDN($self->{mycert});
}

sub logfile { return shift->{logfile}; }



=pod "

=item I<IDX::Test::IOIsolatedSystem($command)>

Launches a command with F<STDIN> and F<STDOUT> redirected to newly-created
pipes, so that misbehaving daemons don't keep our stdin or stdout open.

=cut "

sub IOIsolatedSystem {
	my $command=join(" ",@_);

	local *BLAH;

	open(BLAH,"$command </dev/null 2>&1 |");

	eval {
		local $SIG{CHLD}=sub { die; };
		while(<BLAH>) { print STDERR $_ };
	};

	close(BLAH); return ( ($? & 127) ? 201 : ($? >> 8) );
}

=pod "

=back

=cut "

package IDX::Test;
$INC{"IDX/Test.pm"}=1; # Breaks an require loop - IDX::Test::Suite use()s
# IDX::Test
require IDX::Test::Suite unless caller();

1;

__END__

# Of course, this module has a
######################## TEST CASE ###################################

use strict;

number_of_tests (30); # There are false occurences below, so the autodetect
# fails.

################### Tests of the test framework #######################

test "syntax 1" => sub {
	TESTFAIL if (1==0);
	TESTFAIL if (0==1);
	TEST(4 == 4);
}; # Don't forget the semicolon please

test "syntax 2" => sub {
	if (1) {
		if (1) {
			TESTOK;
		};
	};
	0; # Default is to fail test
};

package Bogus::Printable::Class;

use overload ('""' => 'stringify');

sub stringify {return "this is a stringified exception"};

package IDX::Test;

my $testretval;

test "TEST(true) returns true" => sub {
	TEST(TEST(1));
};

test "TESTOK() returns true" => sub {
	TEST(TESTOK());
};

# Reversed tests to test the test package. :-)
do {
	local $IDX::Test::reversed = 1;
	test "die is not fatal to tests" => sub {
		die
	};

	test "TESTFAIL cancels TESTOK (1/2)" => sub {
		TESTOK;
		TESTFAIL;
	};

	test "TESTFAIL cancels TESTOK (2/2)" => sub {
		TESTFAIL;
		TESTOK;
	};

	test "false return fails" => sub {
		0
	};

	test "exceptional return fails" => sub {
		die bless({},'Bogus::Printable::Class')
	};

	my $false;
	test "TEST(false) returns false (1/2)" => sub {
		$false = TEST(undef);
	};

	test "TEST(false) returns false (2/2)" => sub {
		TEST($false);
	};

	test "TESTFAIL() returns false (1/2)" => sub {
		$false=TESTFAIL();
	};

	test "TESTFAIL() returns false (1/2)" => sub {
		TEST($false);
	};
};

test "inUnitTest" => sub {
	TEST(IDX::Test::inUnitTest ()) || warn "false negative 1";
	my $scriptwithend=<<'SCRIPT';
#!/usr/bin/perl -Tw
use strict;

open(STDOUT,">","/dev/null"); # Do not clobber test
# output of IDX::Test itself

require IDX::Test::Suite unless caller();
1;
__END__

die unless IDX::Test::inUnitTest ();

1;
SCRIPT
	my $scriptfile=new IDX::Test::Filename();
	local *FILE;
	open(FILE,">",$scriptfile); print FILE $scriptwithend; close FILE;
	TEST(! system("/usr/bin/perl","-Tw","-I",join(":",@INC),$scriptfile)) ||
				  warn "false negative 2";

	##### Other calling convention.
	$scriptwithend = <<'SCRIPT';
#!/usr/bin/perl -Tw
use strict;
# Do not clobber test output of IDX::Test itself
open (STDOUT, ">", "/dev/null");
eval join ('', <main::DATA>) || die $@ unless (caller());
1;
__END__
use IDX::Test;
die unless IDX::Test::inUnitTest ();
1;
SCRIPT
	open (FILE, ">", $scriptfile) || die "Can't write to $scriptfile: $!";
	print FILE $scriptwithend;
	close FILE;
	TEST (! system ("/usr/bin/perl", "-Tw", "-I", join(":",@INC),
					$scriptfile)) || warn "False negative 3";

	##### Script which already uses IDX::Test.
	$scriptwithend = <<'SCRIPT';
#!/usr/bin/perl -Tw
use strict;
use IDX::Test;
# Do not clobber test output of IDX::Test itself
open (STDOUT, ">", "/dev/null");
eval join ('', <main::DATA>) || die $@ unless (caller());
1;
__END__
use IDX::Test;
die unless IDX::Test::inUnitTest ();
1;
SCRIPT
	open (FILE, ">", $scriptfile) || die "Can't write to $scriptfile: $!";
	print FILE $scriptwithend;
	close FILE;
	TEST (! system ("/usr/bin/perl", "-Tw", "-I", join(":",@INC), $scriptfile))
		|| warn "False negative with early use of IDX::Test";

	##### Not unit tests.
	TEST(! system("/usr/bin/perl","-Tw","-I",join(":",@INC),
				  "-MIDX::Test","-e",
				  'exit(1) if IDX::Test::inUnitTest (); exit 0')) ||
				  warn "false positive";
};


# Tests over the __END__ test section for real modules

my %testdecls=("old style" => sprintf(<<'OLDSTYLE',"number_of","_tests(2);"),
eval join("",<main::DATA>) || die $@ unless caller();

1;

__END__

use IDX::Test;
import Fake::Module;

%s%s
OLDSTYLE
			   "new style" => <<'NEWSTYLE');
require IDX::Test::Suite unless caller();

1;

__END__

use Fake::Module;

NEWSTYLE

foreach my $style ("old style", "new style") {
	test "Syntactic sugar for end-of-package tests, $style" => sub {
		my $fakemoduletext=<<"FAKEMODULE";
#!/usr/bin/perl -Tw

package Fake::Module;
use strict;

use base 'Exporter';
our \@EXPORT=qw(zoinx);

sub zoinx { 1; }

package Fake::Module::Sub;

$testdecls{$style}

test "In package 'main' for tests" => sub {
  TEST(__PACKAGE__ eq "main") || warn "package is ".__PACKAGE__;
};

test "Symbol zoinx is defined" => sub {
  TEST(zoinx());
};

${$style eq "new style" ? \"0;": \"1;"}
FAKEMODULE
		die "Oops, wrong quoting" unless ($fakemoduletext =~ m/PACKAGE/);

		my $fakemodule=new IDX::Test::Filename;
		(open(FAKEMODULE,">",$fakemodule) &&
		 (print FAKEMODULE $fakemoduletext) && (close FAKEMODULE)) || die;

		use IPC::Open3;
		use FindBin qw($Bin);
		my ($hisstdin,$hisstdout);
		my $pid=open3($hisstdin,$hisstdout,0,
					  '/usr/bin/perl','-Tw','-I',"$Bin/..",$fakemodule);
		close($hisstdin);
		my $result=join("",<$hisstdout>);

		my $failed=0;

		waitpid($pid,0); TEST($? == 0) || do {
			warn "Exited with return code $?\n";
			$failed++;
		};
		TEST($result =~ m/1..2/) || do {
			warn "Counting of tests failed\n";
			$failed++;
		};
		TEST($result !~ m/not ok/) || do {
			warn "Some subtest failed\n";
			$failed++;
		};

		if ($failed) {
			$result =~ s/^/:  /gm; # Do not interfere with our harness
			warn $result;
		};
	};
};

test "dumb tests for integration primitives" => sub {
	TEST(IDX::Test->can("newLWPRequest"));
};


##################### File allocation #################################

test "file allocation" => sub {
	my $testfile=new IDX::Test::Filename;
	TESTFAIL() unless (open(NADA,">$testfile"));
	TESTOK();
};

test "file allocation occurs in same dir" => sub {
	my $testfile1=new IDX::Test::Filename;
	$testfile1 =~ m|^(.*/)[^/]*$| || TESTFAIL;
	my $testdir1=$1;
	my $testfile2=new IDX::Test::Filename;
	$testfile2 =~ m|^(.*/)[^/]*$| || TESTFAIL;
	TEST($1 eq $testdir1);
};

##################### Server allocation #################################
require Socket; import Socket;

test "Network server" => sub {
	my $server=new IDX::Test::NetworkServer()->start();

	local *SOCK;
	socket(SOCK, PF_INET(), SOCK_STREAM(), getprotobyname("tcp"))  ||
	  die "socket: $!";
	connect(SOCK, pack_sockaddr_in($server->{port},inet_aton("127.0.0.1"))) ||
			die "connect: $!";
	SOCK->autoflush(1);
	print SOCK "world\n"; shutdown(SOCK,1);
	my $read=join('',<SOCK>);
	TEST($read =~ m/hello.*world/is) || die Data::Dumper::Dumper($read);
};


test "Network server over UNIX domain sockets" => sub {
	my $sock=new IDX::Test::Filename();
	my $server=new IDX::Test::NetworkServer(-sock=>$sock)->start();

	local *SOCK;
	socket(SOCK, PF_UNIX(), SOCK_STREAM(), 0)  ||
	  die "socket: $!";
	connect(SOCK, pack_sockaddr_un($sock)) ||
			die "connect to $sock: $!";
	shutdown(SOCK,1);
	my $read=<SOCK>;
	TEST($read =~ m/hello/i) || die Data::Dumper::Dumper($read);
};


# Tests whether the port $port says hello over SSL. @extraargs nominates
# extra parameters to pass to stunnel.
sub TESThello {
	my ($port, @extraargs)=@_;
	use IPC::Open2;

    local *IN; local *OUT;

    open2(\*OUT, \*IN, "/usr/sbin/stunnel", "-f", "-c", "-r",
       "localhost:$port", @extraargs);

    local $_=join("", <OUT>);
    TEST(m/^hello/mi) || warn "read on sock returned <<$_>>";

    close(IN); close(OUT);
}

test "SSL server" => sub {
 	my $server=new IDX::Test::SSLServer()->start();

	TESThello($server->getPort());
	TEST(-f $server->logfile());
};

test "picky SSL server" => sub {
 	my $server=new IDX::Test::SSLServer(-picky=>1)->start();

	my $bundle=new IDX::Test::Filename();
	local *BUNDLE;

	open(BUNDLE, ">", $bundle) or die "Cannot open $bundle: $!";
	print BUNDLE $server->{acceptedcert} or die "Cannot write to $bundle: $!";
	print BUNDLE $server->{acceptedkey} or die "Cannot write to $bundle: $!";
	close(BUNDLE) or die "Cannot write to $bundle: $!";

	TESThello($server->getPort(), -p => "$bundle");
};

########################### Unsorted #############################

do {
	my $text=<<TEXT;
Cat was here.
TEXT

	my $file = new IDX::Test::Filename;
	open(FILE,">$file") || die;
	print FILE $text;
	close(FILE);

	test "simple runCommand" => sub {
        my $out=runCommand("/bin/cat",$file);
		TEST($? == 0) || warn "wrong return code: $?";
		TEST($out eq $text) || warn $out;
	};

	test "runCommand for a command that returns false" => sub {
		TEST(runCommand("/usr/bin/perl","-e","exit 42") eq "") ||
                warn "got text?";
		TEST($? eq (42<<8));
	};

	test "runCommand for a non-existing command" => sub {
		my $text=runCommand("/no/such/command",$file);
		TEST($text eq "");
		TEST($? eq (165<<8));
	};
};

test "certificates and certFiles" => sub {
	my @data=certificates();
	my @files=certFiles();

	while(@data) {
		my $data=shift @data;
		my $file=shift @files;
		TEST(! ref $file);
		local *FILE; local $/=undef;
		open(FILE,"<",$file);
		TEST(<FILE> eq $data);
		close(FILE);
	};
};

test "evilCerts and evilCertFiles" => sub {
	my @data=evilCerts();
	my @files=evilCertFiles();

	while(@data) {
		my $data=shift @data;
		my $file=shift @files;
		TEST(! ref $file);
		local *FILE; local $/=undef;
		open(FILE,"<",$file);
		TEST(<FILE> eq $data);
		close(FILE);
	};
};

test "getCertDN" => sub {
	my ($cafile, $cert1file, undef, $cert2file, undef, undef, $rootcafile) =
		certFiles();
	my ($ca, $cert1, undef, $cert2, undef, undef, $rootca) = certificates();
	TEST ((getCertDN ($cafile) =~ m/^cn=/i) &&
		  (getCertDN ($ca) =~ m/^cn=/i) &&
		  (getCertDN ($cafile) eq getCertDN ($ca))) ||
		warn "Error with CA cert: ".getCertDN ($cafile)." (file), ".
			getCertDN ($ca)." (cert)";
	TEST ((getCertDN ($cert1file) =~ m/^cn=/i) &&
		  (getCertDN ($cert1) =~ m/^cn=/i) &&
		  (getCertDN ($cert1file) eq getCertDN ($cert1))) ||
		warn "Error with client cert: ".getCertDN ($cert1file)." (file), ".
			getCertDN ($cert1)." (cert)";
	TEST ((getCertDN ($cert2file) =~ m/^cn=/i) &&
		  (getCertDN ($cert2) =~ m/^cn=/i) &&
		  (getCertDN ($cert2file) eq getCertDN ($cert2))) ||
		warn "Error with server cert: ".getCertDN ($cert2file)." (file), ".
			getCertDN ($cert2)." (cert)";
	TEST ((getCertDN ($rootcafile) =~ m/^cn=/i) &&
		  (getCertDN ($rootca) =~ m/^cn=/i) &&
		  (getCertDN ($rootcafile) eq getCertDN ($rootca))) ||
		warn "Error with root CA cert: ".getCertDN ($rootcafile)." (file), ".
			getCertDN ($rootca)." (cert)";
	my ($evilcertfile, undef) = evilCertFiles();
	my ($evilcert, undef) = evilCerts();
	TEST ((getCertDN ($evilcertfile) =~ m/^cn=/i) &&
		  (getCertDN ($evilcert) =~ m/^cn=/i) &&
		  (getCertDN ($evilcertfile) eq getCertDN ($evilcert))) ||
		warn "Error with evil cert: ".getCertDN ($evilcertfile)." (file), ".
			getCertDN ($evilcert)." (cert)";
	TEST (getCertDN ($evilcert) eq getCertDN ($cert1)) ||
		warn "Evil and client cert have different DNs: ".
		getCertDN ($evilcert)." (evil), ".getCertDN ($cert1)." (client)";
};

# Emulate a non-bootstrapped config.
undef $IDX::Config::PATH;
undef $IDX::Config::APPCLASS;
undef *IDX::Config::_default;

integrationtest "integrationtest (should be skipped)" => sub {
   warn "Are you sure you want to tests tests in integration tests?";
   warn "appclass is $IDX::Config::APPCLASS";
   TESTFAIL();
};


1;


