#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: LDAPServer.pm,v 1.7 2003/02/13 16:54:21 laura Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/Test/LDAPServer.pm,v $

use strict;

=pod "

=head1 NAME

IDX::Test::LDAPServer - Test LDAP server for networked unit tests

=head1 SYNOPSIS

 use IDX::Test::LDAPServer;

 my $server = new IDX::Test::LDAPServer::InPerl(sub {
 	return undef});

 my $server = new IDX::Test::LDAPServer::Stateful(
 	<<"LDIF");
 dn: ou=foo,O=Idealx
 objectClass: top
 objectClass: organizationalUnit
 organizationalUnitName: foo
 businessCategory: 1st class

 dn: ou=bar,O=Idealx
 objectClass: top
 objectClass: organizationalUnit
 organizationalUnitName: bar
 businessCategory: 2nd class
 LDIF

=head1 DESCRIPTION

Test LDAP server for networked unit tests.  This actually is a lot of
effort for a very thin result... but at least unit tests really are now.

This package defines two classes, B<IDX::Test::LDAPServer::InPerl>
and B<IDX::Test::LDAPServer::Stateful>, that both inherit from
B<IDX::Test::NetworkServer> (see L<IDX::Test>).  Its purpose is
to provide a testbed for LDAP accessor classes such as B<IDX::PKI::LDAP>.

B<IDX::Test::LDAPServer::InPerl> accepts a reference to a
function, that maps the text representation of an LDAP answer to the
text representation of a query. This class may thus respond virtually
anything to a given request, which makes it ideally suited for
stress-testing LDAP client implementations.

B<IDX::Test::LDAPServer::Stateful>, on the other hand, is
targeted at applications that treat the LDAP directory as a database.
It accepts both reading and writing, and the initial contents of the
database is specified at constructor invocation.

=head2 B<IDX::Test::LDAPServer::InPerl> class

B<IDX::Test::LDAPServer::InPerl> is implemented using the "shell"
backend of OpenLDAP and some UNIX pipework:

        ----------------------------
        | main Perl program        |
        | (calls the constructors) |
        ----------------------------
               /               \
              /                 \     INET socket
             /                   \        ||
    -----------------------   ------------||-----
    | forked Perl daemon  |   | OpenLDAP server |
    | (answers OpenLDAP's |   -------------------
    | shell queries)      |          |
    ----------------||-----          |
                    ||        -------------------------
                     #========= Shell backend process |
            UNIX socket       | (perl scriptlet, one  |
                              | per LDAP request)     |
                              -------------------------

Contrast this with B<IDX::Test::LDAPServer::Stateful> below.

All the stuff above will be properly destroyed when the last reference
to the server objects fall out of scope.

=over 4

=cut "

package IDX::Test::LDAPServer::InPerl;

use strict;

use IO::File;
use IO::Pipe;
use IDX::Test;

our @ISA=qw(IDX::Test::LDAPServer);


=pod "

=item I<new($handlerref)>

=item I<new($handlerref, $suffix, $acl)>

Creates and starts a new LDAP server. It listens both on a UNIX domain
socket (whose path is available through L<getSock()>) and on a
TCP port (which can be queried with L<IDX::Test/getPort()>).

The LDAP hierarchy is rooted at I<$suffix>, or C<O=Idealx> if I<$suffix>
is not provided.

One may specify an acl through $acl, using L<slapd.conf>. By default,
everyone will have write access everywhere.

I<$handlerref> is a Perl subroutine that will be called to handle all
incoming requests.  It will be called with a text version of OpenLDAP's
request, and will need to return a text version of the response.  Here
is an example for a query:

 SEARCH
 msgid: 12345
 suffix: O=Idealx
 suffix: O=AnotherSuffixOptional
 base: O=Idealx
 scope: 1
 deref: 0
 sizelimit: 300
 timelimit: 3600
 filter: (objectClass=inetOrgPerson)
 attrsonly: 1
 attrs: all

I<attrsonly> is a boolean; adequate values for scope and deref are
available in the I<%IDX::Test::LDAPServer::numeric_scope> and
I<%IDX::Test::LDAPServer::numeric_deref> hashes (which need to be
reverse()d for parsing).  The keys of those hashes are similar to
B<Net::LDAP>.  I<attrs> will be the empty string, or C<all> (according
to our RTFS session, that is - specs and documentation for OpenLDAP's
shell backend just don't exist).

Here are examples for valid responses: This is a failure,

 RESULT
 code: 53
 info: Go Away
 # Comments are OK in your response
 DEBUG: you may tell something here too.

and this is a success returning LDIF things (for a search for example):

 dn: cn=Myself, O=Idealx
 objectclass: top
 objectclass: person
 cn: Myself
 uid: 1234
 # Don't forget the newline below.

 RESULT
 code: 0
 info: in fact you may put any LDIF you like
   in there too.

Return codes are as in B<Net::LDAP>.  Informational texts are passed back
to the client and may be any text.  The semantics of the rest of the
fields is unknown, and is best determined by trial and error :-).

=cut "

sub new {
	my ($class, $handlerref, $suffix, $acl) = @_;

	my $self=IDX::Test::LDAPServer::_new($class,$acl);

	$self->{commssocket}=$self->{dir}."/backendsock";
	$self->{commsscript}=$self->{dir}."/socketme";
	$self->{responder}=new IDX::Test::LDAPServer::Responder
	  ($self->{commssocket},$handlerref,
	   IO::File->new("> $self->{dir}/commslog"))->start();

	$self->{'suffix'} = defined $suffix ? $suffix : "O=Idealx";

	my $endofconfig=<<"CONFIG";
# This pipes all requests back to the forked Perl, via a trivial
# UNIX-socket-chat proggie.
database	shell
suffix		"$self->{'suffix'}"
bind        $self->{commsscript}
unbind        $self->{commsscript}
search        $self->{commsscript}
compare        $self->{commsscript}
modify        $self->{commsscript}
modrdn        $self->{commsscript}
add        $self->{commsscript}
delete        $self->{commsscript}
CONFIG

	local *CONFIG;
	(open(CONFIG,">>",$self->{config}) and (print CONFIG $endofconfig)
	 and close CONFIG) ||
	   die ("Cannot finish up $self->{config}");

	# Here comes the proggie.
	my $scripttxt=<<"SCRIPT";
#!/usr/bin/perl -w
use Socket;
\$SIG{PIPE}="IGNORE";
socket(SOCK, PF_UNIX(), SOCK_STREAM(), 0)  ||
	  die "socket: \$!";
connect(SOCK, pack_sockaddr_un("$self->{commssocket}")) ||
			die "connect to $self->{commssocket}: \$!";
select(SOCK);\$|=1;select(STDOUT);
while(<STDIN>) { print SOCK \$_ || die "writing to socket: \$!"};
shutdown(SOCK,1);
while(<SOCK>) { print || die "Cannot print; $!"; };
SCRIPT

	local *SCRIPT;
	(open(SCRIPT,">",$self->{commsscript}) and (print SCRIPT $scripttxt)
	 and close SCRIPT) ||
	   die ("Cannot create $self->{commsscript}");
	chmod(0755,$self->{commsscript});

	return $self;
}

=pod "

=back

=head2 B<IDX::Test::LDAPServer::Stateful> class

The I<Stateful> implementation is more straightforward:

        ----------------------------
        | main Perl program        |
        | (calls the constructors) |
        ----------------------------
                               \
                                \     INET socket
                                 \        ||
                              ------------||-----
                              | OpenLDAP server |
                              -------------------
                                     |
                                     |
                                    / \
                                   /   \
                                  /LDBM \
                                  \     /
                                   \   /
                                    \_/

I<Note:> B<IDX::Test::LDAPServer::Stateful> needs a bootstrapped
configuration to work, so you have to use the proper B<IDX::MyApp::Config>
before this module. Tests using a B<IDX::Test::LDAPServer::Stateful> LDAP
server are always integration tests.

=over 4

=item I<new($ldif)>

=item I<new($ldif, $suffix, $acl)>

Constructor, returns an object that remembers the entries written into
it.  I<$ldif> is the initial LDIF contents, empty lines separate
entries.

The LDAP hierarchy is rooted at I<$suffix>, or C<O=Idealx> if I<$suffix>
is not provided.

One may specify an acl through $acl, using the syntax of
L<slapd.conf>. By default, everyone will have write access everywhere.

=cut "

package IDX::Test::LDAPServer::Stateful;

use strict;
use IDX::Test;
use IDX::Config;

our @ISA=qw(IDX::Test::LDAPServer);


sub new {
	my ($class, $ldif, $suffix, $acl) = @_;

	my $self=IDX::Test::LDAPServer::_new($class,$acl);
	$self->{ldif}=$ldif;

	mkdir("$self->{dir}/ldbm");

	$self->{'suffix'} = defined $suffix ? $suffix : "O=Idealx";
	$self->{'rootdn'} = "CN=root".
		($self->{'suffix'} ? ",$self->{'suffix'}" : "");
	$self->{'rootpw'} = "root";

	my $endofconfig=<<"CONFIG";
# An LDBM database for testing
# LDBM database definitions.
database	ldbm
suffix      "$self->{'suffix'}"
directory	$self->{dir}/ldbm

# Aaargh, the wide-open access in the main config file still is not
# enough to change the suffix node!
rootdn "$self->{'rootdn'}"
rootpw "$self->{'rootpw'}"

CONFIG

	local *CONFIG;
	(open(CONFIG,">>",$self->{config}) and (print CONFIG $endofconfig)
	 and close CONFIG) ||
	   die ("Cannot finish up $self->{config}");

	return $self;
}

=pod "

=item I<start()>

This method is overloaded from the parent class in order to load the
LDIF data into the LDAP server at startup time.

=cut "

sub start {
	my ($self)=@_;
	$self->SUPER::start();

	return $self if (! $self->{ldif});
	my $pipe=new IO::Pipe;
	my $config = the IDX::Config ();
	$pipe->writer($config->getval ("paths", "ldapmodify"), "-a",
				  "-H", "ldap://127.0.0.1:$self->{port}/", "-D",
				  $self->{'rootdn'}, "-w", $self->{'rootpw'}, "-x");
	$pipe->print($self->{ldif});
	$pipe->close(); # I don't know if this sets $? correctly??
	die "ldapmodify failed with error ".($?>>8) if ($?);
	return $self;
}

=pod "

=back

=head2 Package B<IDX::Test::LDAPServer>

Catch-all for generic LDAP stuff that spans the two operative classes
above. This package acts as a superclass to the other two, but this is
something you don't need (nor want) to know.

=over 4

=cut "

package IDX::Test::LDAPServer;

use strict;
use Errno qw(ECONNREFUSED);
use Socket;

our @ISA=qw(IDX::Test::NetworkServer);

=pod "

=item I<%IDX::Test::LDAPServer::numeric_scope>

=item I<%IDX::Test::LDAPServer::numeric_deref>

Mappings from symbolic to numeric values for the I<scope> and I<deref>
parameters of an LDAP search.  Copy-pasted from the implementation of
B<Net::LDAP>.

=cut "

use vars qw(%numeric_scope %numeric_deref);
%numeric_scope = qw(base  0 one    1 single 1 sub    2 subtree 2);
%numeric_deref = qw(never 0 search 1 find   2 always 3);

=pod "

=item I<IDX::Test::LDAPServer::result($code,$explain)>

This is not a method, just a function: It takes an error code (for
example I<LDAP_UNWILLING_TO_PERFORM()>, see L<Net::LDAP> on how to import
this constant in your own package) and an explanation, and returns a
properly formatted piece of text that will hopefully tell OpenLDAP what
to tell to the client.  If called without parameters, returns a success,
that must be appended at the end of the search results in LDIF, if any.

=cut "

sub result {
	my ($code,$explain)=@_;
	$code||=0;
	my $result=<<"RESULT1";
RESULT
code: $code
RESULT1
	$result.=<<"RESULT2" if $explain;
info: $explain
RESULT2
	return $result;
}

=pod "

=item I<_new($acl)>

Shared code between the two constructors of the class.  This
constructor is not meant to be called directly: It must be inherited
from to do anything useful.  Specifically, I<_new()> creates the object
data, blesses it, and creates a working directory with a PID file, an
args file and a config file that has no database definitions inside
and thus needs to be appended to in the subclass' constructor.

One may specify an acl through $acl, using the syntax of
L<slapd.conf>. By default, everyone will have write access everywhere.

=cut "

sub _new {
	my ($class,$acl)=@_;
	my $self=IDX::Test::NetworkServer::new($class);

	$self->{config}=$self->{dir}."/slapd.conf";
	$self->{pidfile}=$self->{dir}."/slapd.pid";
	$self->{sockpath}=$self->{dir}."/sock";

	# Now create the environment for OpenLDAP.

	my $config = the IDX::Config ();
	my $schemadir = $config->getval ("paths", "schemadir");
	#my $idxschemadir=IDX::Test::whereami()."/src/etc/ldap";
	my $idxincludes = "";
	if ($config->getval ("paths", "configdir"))
	{
		# Warning, this works only with an installed software.
		my $idxschemadir = $config->getval ("paths", "configdir")."/schema";
		local *SCHEMADIR;
		opendir (SCHEMADIR, $idxschemadir);
		foreach my $schema (readdir (SCHEMADIR))
		{
			next if ($schema =~ m/^\./);
			next unless ($schema =~ m/\.schema$/);
			$idxincludes .= "include		$idxschemadir/$schema\n";
		}
		closedir (SCHEMADIR);
	}

	$acl=<<"ACL" if (!defined $acl);
access to * by * write
access to * by anonymous write
defaultaccess write
ACL

	my $configtxt=<<"CONFIG";
# Configuration file for an IDEALX test LDAP server.
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# See slapd.conf(5) for details on configuration options.

pidfile		$self->{pidfile}
argsfile	$self->{dir}/slapd.args

include		$schemadir/core.schema
include		$schemadir/cosine.schema
include		$schemadir/inetorgperson.schema
include		$schemadir/nis.schema
include		$schemadir/misc.schema
include		$schemadir/openldap.schema
$idxincludes

schemacheck	off
$acl

# LDBM or shell database definitions go below.
CONFIG

	local *CONFIG;
	(open(CONFIG,">",$self->{config}) and (print CONFIG $configtxt)
	 and close CONFIG) ||
	   die ("Cannot create $self->{config}");

	return $self;
}


=pod "

=item I<start()>

Starts the LDAP server. Its configuration must be finished up
beforehand (see L</_new()>).

=cut "

sub start {
	my ($self)=@_;

	defined(my $pid=fork()) || die "Can't fork()";
	if (!$pid) { # Child process
		# This stupid slapd cannot bind to a UNIX socket except if
		# it is in the current directory...
		chdir($self->{dir});

#		my $logfile=$self->getLogFile();
#		open(STDOUT,">$logfile") ||
#			die "Cannot open $logfile: $!\n";
#		open(STDERR,">&STDOUT") ||
#			die "Cannot dup STDERR to STDOUT: $!\n";

		my $config = the IDX::Config ();
		exec ($config->getval ("paths", "slapd"), "-f", $self->{config},
#			  "-d", "65535", "-h",
			  "-h", "ldap://127.0.0.1:$self->{port}/ ldapi://sock/")  || do {
				  warn "Could not launch slapd (return code $?)\n";
				  require POSIX;
				  POSIX::_exit(42);
			  };
	};

	# OpenLDAP does not synchronize with caller by fork()ing at the
	# appropriate time:
	local *SOCK;
	socket(SOCK, PF_INET, SOCK_STREAM, getprotobyname("tcp")) ||
		die "socket: $!";
	for (my $i = 0; $i < 5; $i++) {
		$! = 0;
		connect(SOCK,pack_sockaddr_in($self->{port},inet_aton("127.0.0.1")));
		last if ($! != ECONNREFUSED);
		sleep(1);
	};
	die "Connect: $!" if ($!);
	close(SOCK);
	return $self;
}

=pod "

=item I<getSock()>

Returns the path to the UNIX domain socket this LDAP server (either
InPerl or Stateful) is listening to.

=cut "

sub getSock {
	my ($self)=@_;
	return $self->{sockpath};
}

=pod "

=item I<getLogFile()>

Returns the path of a logfile this LDAP server (either InPerl or
Stateful) is writing to.

=cut "

sub getLogFile {
	return shift->{dir}."/log";
}

=pod "

=back

=head2 Ancillary class B<IDX::Test::LDAPServer::Responder>

This class is also a descendant of B<IDX::Test::NetworkServer>,
but this time in UNIX-domain style. It is used to respond to the
requests that arrive to OpenLDAP and are piped back to us through a
shell (actually Perl) scriptlet that we taught OpenLDAP to use.  A
forked Perl will stay around, oracling back the answers to the main
Perl process which can proceed to the client side of the tests.

=over 4

=item I<new($sock,$handler_coderef)>

=item I<new($sock,$handler_coderef, $logfd)>

Initializes the responder server. I<$sock> is the path to the UNIX domain
socket.  I<$handler_coderef> is a reference to a function that will be
called to answer each request.

If I<$logfd> is passed to the constructor, all incoming messages are
logged to it (separated with a comment that starts with exactly three C<#>
signs).

=back

=cut "

package IDX::Test::LDAPServer::Responder;

use strict;
use POSIX;
use Net::LDAP qw(:all);
use IDX::Test;

our @ISA=qw(IDX::Test::NetworkServer);

sub new {
	my ($class,$sock,$handler_coderef,$logfd)=@_;

	my $self=IDX::Test::NetworkServer::new($class,-sock=>$sock);
	if ($logfd) {
		unless (UNIVERSAL::isa($logfd,"IO::Handle")) {
			my $h=new IO::Handle;
			$h->fdopen(fileno($logfd));
			$logfd=$h;
		};
		$self->{log}=$logfd;
	};

	$self->{handler}=$handler_coderef;
	return $self;
}

sub _answer {
	my ($self,$sockref)=@_;

	local *SOCK=$sockref;

	my $txt=join('',<SOCK>);

	my $ans=&{$self->{handler}}($txt);
	$ans=IDX::Test::LDAPServer::result(LDAP_UNWILLING_TO_PERFORM())
	  if (!defined $ans);

	if ($self->{log}) {
		$self->{log}->printf("### Message received at %s\n",
		  POSIX::strftime("%c",localtime(time)));
		$self->{log}->print($txt);
		$self->{log}->print("### Answer:\n"),
		$self->{log}->print($ans);
		$self->{log}->flush();
	};
	print SOCK $ans;
}

eval join('',<main::DATA>) || die $@ unless caller();

1;


__END__

####################### TEST SUITE ###########################

package main;

use strict;

use IDX::Test;
use IDX::PKI::Error;
use Net::LDAP;

# All tests are integration tests.
use IDX::Config;
unless (UNIVERSAL::can("IDX::Config", "is_bootstrapped") &&
		IDX::Config::is_bootstrapped()) {
	number_of_tests (0);
	return 1;
}

number_of_tests(10);

sub bogusanswer {
	my ($request) = @_;
	return undef unless ($request =~ m/^SEARCH$/m);
	return undef unless ($request =~ m/^base: (.*)$/m);
	my $base = $1;

	return (<<"BOGUSDATA".IDX::Test::LDAPServer::result());
dn: ou=foo,$base
objectClass: top
objectClass: organizationalUnit
organizationalUnitName: foo
businessCategory: 1st class

dn: ou=bar,$base
objectClass: top
objectClass: organizationalUnit
organizationalUnitName: bar
businessCategory: 2nd class

BOGUSDATA
}


my $server;

test "result()" => sub {
	TEST(IDX::Test::LDAPServer::result() eq <<'THERESULT');
RESULT
code: 0
THERESULT
};
test "new()" => sub {
	$server=new IDX::Test::LDAPServer::InPerl(\&bogusanswer);
	TEST(-S $server->{commssocket});
};

test "socket script syntactically correct" => sub {
	TEST($server->{commsscript}) || die "No commsscript?";
	system("/usr/bin/perl","-c",$server->{commsscript});
	TEST(! $?) || sleep 60;
};

test "socket script direct call" => sub {
	my $out=new IDX::Test::Filename();
	my $pipe=new IO::Pipe;
	$pipe->writer("/bin/sh","-c","$server->{commsscript} > $out");
	# Yes I know this is an inelegant and shameful dodging of the
	# taint mechanism. We are in the tests right?
	local $SIG{PIPE}="IGNORE";
	$pipe->print(<<BOGUSREQUEST);
SEARCH
scope: 0
base: OU=Zoinx,C=IDEALX
BOGUSREQUEST

	$pipe->close();
	if (my $errcode=$?>>8) {
		warn "Could not launch $server->{commsscript}, return code $?";
		warn `cat $out`;
	};
	local *BACK;
	open(BACK,"<",$out) || die "Could not read back $out: $!";
	my $result=join('',<BACK>);
	TEST($result =~ m/Zoinx/) ||
	  warn $result;
	close(BACK);
};

test "start()" => sub {
	TEST (ref($server->start()) eq "IDX::Test::LDAPServer::InPerl") ||
		warn "Start failed";

#	TEST (-f $server->getLogFile());
};

test "making a request" => sub {
	my $cnx=new Net::LDAP("localhost",port=>(0+$server->{port}))
	  or die "Failed connection to $server->{port}: $@\n";
	$cnx->bind();
	my $msg=$cnx->search ( base=>"OU=Nowhere,O=Idealx",
						 filter=>"(objectClass=*)");
	$msg->code && die ($msg->code . " " . $msg->error);
	my ($foo,$bar)=$msg->all_entries;
	TEST($foo->get_value("businessCategory") eq "1st class") ||
	  $foo->dump();
	TEST($bar->get_value("businessCategory") eq "2nd class") ||
	  $bar->dump();
};

test "attempting to write (should fail)" => sub {
	my $cnx=new Net::LDAP("localhost",port=>(0+$server->{port}),onerror=>"die")
	  or die "Failed connection to $server->{port}: $@\n";
#	$cnx->debug(15);
	$cnx->bind();
	my $msg=$cnx->search ( base=>"OU=Nowhere,O=Idealx",
						 filter=>"(objectClass=*)");
	my ($foo)=$msg->all_entries;
	$foo->add("description","this is a test");
	try {
		$foo->update($cnx);
		TESTFAIL;
	} catch Error::Simple with {
		TEST(shift =~ m/unwilling/i);
	};
};

use Net::LDAPS;
test "attempting write over SSL" => sub {
	# This test fails under Net::LDAP 0.23 due to a synchronism bug
	my $sslserver=new IDX::Test::SSLServer(-server=>$server)->start();
	my $cnx=new Net::LDAPS("localhost",port=>(0+$sslserver->{port}),
						   onerror=>"die")
	  or die "Failed connection to $server->{port}: $@\n";
#	$cnx->debug(15);
	$cnx->bind();
	my $msg=$cnx->search ( base=>"OU=Nowhere,O=Idealx",
						 filter=>"(objectClass=*)");
	my ($foo)=$msg->all_entries;
	$foo->add("description","this is a test");
	try {
		$foo->update($cnx);
		TESTFAIL;
	} catch Error::Simple with {
		TEST(shift =~ m/unwilling/i);
	};
};

# my $port=$server->{port};
# 
# warn "please point your gq at $port and have a glance";
# sleep(1800);

my $baseLDIF=<<LDIF;
dn: O=Idealx
objectClass: organization
organizationName: Idealx

dn: ou=foo,O=Idealx
objectClass: top
objectClass: organizationalUnit
organizationalUnitName: foo
businessCategory: 1st class

dn: ou=bar,O=Idealx
objectClass: top
objectClass: organizationalUnit
organizationalUnitName: bar
businessCategory: 2nd class

LDIF


test "stateful LDAP server" => sub {
	$server=new IDX::Test::LDAPServer::Stateful($baseLDIF)->start();

#	TEST(-f $server->getLogFile());

	my $cnx=new Net::LDAP("localhost",port=>($server->{port}))
	  or die "Failed connection to $server->{port}: $!\n";
	$cnx->bind();
	my $msg=$cnx->search ( base=>"O=Idealx",
						 filter=>"(organizationalUnitName=bar)");
	$msg->code && die ($msg->code . " " . $msg->error);
	TEST($msg->count==1) || warn "msg->count = $msg->count";
};

test "ACLs" => sub {
	my $blockingACL = <<"ACL";
access to * by * none
access to * by anonymous none
defaultaccess none
ACL
	$server = new IDX::Test::LDAPServer::Stateful($baseLDIF, "O=Idealx",
												  $blockingACL)->start();

	my $cnx = new Net::LDAP ("localhost", port => ($server->{port}))
		or die "Failed connection to $server->{port}: $!\n";
	$cnx->bind();
	my $msg = $cnx->search (base => "O=Idealx",
							filter => "(organizationalUnitName=bar)");

	TEST($msg->code() || ! $msg->count()); # OpenLDAP 2.0.23 returns zero
	# search results in this case instead of an error. Is he right? I doubt
	# so. But it suffices to prove that the ACL mechanism works.
};

# This ensures that the server is DESTROY()ed before the temporary files
# (otherwise it won't be killed).
$server = undef;
1;
