#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<IDX::Test::Suite> - invoke a test suite run at the end of a script.

=head1 SYNOPSIS

 package MyPackage;

 <the text of the package goes here>

 # This is the ritual stanza, which will only kick in if
 # `MyPackage.pm' is invoked directly (`perl MyPackage.pm')
 # instead of being `use'd. This consists in `eval'ing
 # everything after __END__; the test case won't even be
 # compiled in production.

 require IDX::Test::Suite unless caller();

 1;

 __END__

 use MyPackage; # Remember, tests occur in package "main"
 use IDX::Error; # If you want to trap exceptions in tests

(and there you are with your test suite - see L<IDX::Test>)

=head1 DESCRIPTION

IDX::Test::Suite is a helper for the test framework provided in
L<IDX::Test>. The C<< require IDX::Test::Suite unless caller(); >>
idiomatics (see L</SYNOPSIS>) triggers the running of the test suite
if the Perl module (C<.pm> file) is run standalone. Due to the way
this construct abuses the Perl module mechanism, it needs to be
isolated in its own package.

=head2 Support for the old permodlib style

L<perlmodlib> recommends the 'eval unless caller' form instead for
tests:

 package MyPackage;

 <the text of the package goes here>

 eval join('',<main::DATA>) || die $@ unless caller();

 1;

 __END__

 package main;

 use IDX::Test;
 number_of_tests(42);

 BEGIN { import MyPackage; } # You can't "use" yourself in perlmodlib
                             # style or every function will appear to
                             # be defined twice.
 use IDX::Error;

 <The test suite goes there.>

 1; # The '|| die $@' form relies on your package test suite returning
    # true too.

I<IDX::Test> supports this too, but with much less comfort. The
differences between the two syntaxes are:

=over 4

=item Tests are not counted automatically

In require style, the test suite is parsed and tests are automatically
counted. For the simple cases (e.g. you run all tests exactly once),
you don't have to include a I<number_of_tests()> call at the top of
the package.


=item Package handling is up to you

With the perlmodlib style, you have to say C<use IDX::Test> (this is
done implicitly with the require style), and you will not be placed
automatically in package "main" (you have to switch there using a
"package main;" stanza). In order to import the package you are
testing into your test suite (which is probably what you want if the
package exports things), you must use "import" in a BEGIN block
instead of "use" (see below). In require style, all this is
transparent, even the %INC is fiddled with so that "use MyPackage"
does what you want.

=item Line counting for the debugger does not work

You can step through the test using with a GUI debugger (e.g. perldb
in Emacs) if using the require form. In perlmodlib style, the line
numbers returned by the Perl debugger are of the form '(eval 136) line
155', which confuses Emacs.

=back

=cut "

package IDX::Test::Suite;

use strict;
use IDX::Test;

if (my $testsuite=do { no warnings; scalar <main::DATA> }) {

	$testsuite.=join('',<main::DATA>);
	# Provide comfort to user by not imposing her to dot he 'use
	# IDX::Test;' thing in her tests.
	IDX::Test->export('main',@IDX::Test::EXPORT);

	# Allow user to tell 'use My::Package' and not really do disk access
	# to her module.

	my ($package,$packfilename,$packline)=caller(0);
	if (!defined $package) { # Handle self-testing case
		$package="IDX::Test::Suite";
		$0 =~ m/^(.*)$/; $packfilename = $1;
	}
	if (defined $package && defined $packfilename) {
		for(;$package; $package =~ s/(::|^)[^:]*$//) {
			my $filename=$package;
			$filename =~ s|::|/|g;
			$filename .= ".pm";
			$INC{$filename}=1;
			last if ($packfilename =~ m/\Q$filename\E$/); # e.g. do
			# not clobber "My" if testing "My::Package".
		};
	};

	# Add a preamble to get the line numbers correct. This requires
	# parsing the source file :-)
	local *SOURCEFILE; local $_;
	if (defined($packfilename) && open(SOURCEFILE,"<",$packfilename)) {
		while(<SOURCEFILE>) {
			next if (defined($packline) && $. <= $packline); # Be sure to
			# catch the first marker *after* the require directive, and
			# mind the self-test case too.
			next unless (m/^__(END|DATA)__/);
			$testsuite="#line ".($.+1)." \"$packfilename\"\n".$testsuite;
			last;
		};
		close(SOURCEFILE);
	};

	$testsuite="package main;\n".$testsuite;

	# Accomodate test suites without a number of tests specified.
	unless ($testsuite =~ m/^\s*number_of_tests/m) {
		my @tests=($testsuite =~ m/^\s*(?:integration)?test\W/gm);
		IDX::Test::number_of_tests(scalar @tests);
	};

	eval $testsuite; die($@) if $@;
	1;
};

1;

__END__

# Yes, even this module has a
######################## TEST SUITE ###################################

use IDX::Test;
use strict;

number_of_tests(2);

# Tests over the __END__ test section for real modules

my %testdecls=("old style" => <<'OLDSTYLE',
eval join("",<main::DATA>) || die $@ unless caller();

1;

__END__

use IDX::Test;
import Fake::Module;

number_of_tests(2);
OLDSTYLE
			   "new style" => <<'NEWSTYLE');
require IDX::Test::Suite unless caller();

1;

__END__

use Fake::Module;

NEWSTYLE

foreach my $style ("old style", "new style") {
	test "Syntactic sugar for end-of-package tests, $style" => sub {
		my $fakemoduletext=<<"FAKEMODULE";
#!/usr/bin/perl -Tw

package Fake::Module;
use strict;

use base 'Exporter';
our \@EXPORT=qw(zoinx);

sub zoinx { 1; }

package Fake::Module::Sub;

$testdecls{$style}

test "In package 'main' for tests" => sub {
  TEST(__PACKAGE__ eq "main") || warn "package is ".__PACKAGE__;
};

test "Symbol zoinx is defined" => sub {
  TEST(zoinx());
};

${$style eq "new style" ? \"0;": \"1;"}
FAKEMODULE
		die "Oops, wrong quoting" unless ($fakemoduletext =~ m/PACKAGE/);

		my $fakemodule=new IDX::Test::Filename;
		(open(FAKEMODULE,">",$fakemodule) &&
		 (print FAKEMODULE $fakemoduletext) && (close FAKEMODULE)) || die;

		use IPC::Open3;
		use FindBin qw($Bin);
		my ($hisstdin,$hisstdout);
		my $pid=open3($hisstdin,$hisstdout,0,
					  '/usr/bin/perl','-Tw','-I',"$Bin/../..",$fakemodule);
		close($hisstdin);
		my $result=join("",<$hisstdout>);

		my $failed=0;

		waitpid($pid,0); TEST($? == 0) || do {
			warn "Exited with return code $?\n";
			$failed++;
		};
		TEST($result =~ m/1..2/) || do {
			warn "Counting of tests failed\n";
			$failed++;
		};
		TEST($result !~ m/not ok/) || do {
			warn "Some subtest failed\n";
			$failed++;
		};

		if ($failed) {
			$result =~ s/^/:  /gm; # Do not interfere with our harness
			warn $result;
		};
	};
};

