#!/usr/bin/perl -wT
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $Id: WebServer.pm,v 1.4 2003/07/23 13:20:47 mbrossard Exp $
# $Source: /opt/cvs/idxpki/sources/src/perllib/IDX/Test/WebServer.pm,v $

use strict;

package IDX::Test::WebServer;

=pod "

=head1 NAME

IDX::Test::WebServer - Test HTTP server for networked unit tests

=head1 SYNOPSIS

 my $server = new IDX::Test::WebServer ();

=head1 DESCRIPTION

Returns a ready-to-perform web server by using the configured values.
This web server will be shut down when I<$server> falls out of
scope. By default, it can serve only one page, "/index.html".  it also
has a 404 (not found) error document.

=cut "

use IDX::Test;
use HTTP::Daemon; # From LWP

@IDX::Test::WebServer::ISA = qw (IDX::Test::NetworkServer);


=pod "

=head1 METHODS

=over 4

=item I<new (-arg1=>val1,...)>

Creates the WWW server (but does not start it; see
L<IDX::Test/IDX::Test::NetworkServer::start>). Recognized
named arguments are:

=over 4

=item -page=>[$uri,$action]

Adds a page to the server's namespace. $uri is the URI (relative to
the server base) in which it should appear; $action is either a text
to return to the client (including HTTP headers), or a function that
takes the text of the request as the only argument and must return a
text (still with HTTP headers).

=item -activepage=>[$uri,$action]

Adds a page to the server's namespace like above, but I<$action> is a
function which will be called like this:

   &$action($socket,$request);

where I<$socket> is the input socket file descriptor (an
B<HTTP::Daemon::ClientConn> object) and I<$request> an B<HTTP::Request>
returned as if by I<< $socket->get_request(1) >> (see
L<HTTP::Daemon/get_request>). I<$request> holds the I<complete> request,
including the POSTed document if any. Use methods in B<HTTP::Request>
to get at its contents.

You have to directly print data onto I<$socket> rather than I<return()>ing
it if you want it to be output to the client.

=back

=cut "

sub new {
	my ($class, @args) = @_;

	my $self = IDX::Test::NetworkServer::new ($class, @args);

	$self->{pages}={"/index.html"=>sub {
						my ($input)=@_;
						return <<"INDEX";
HTTP/1.1 200 OK
Content-Type: text/html

<html>
  <head>
    <title>Little web page</title>
  </head>
  <body>
    <p>
      A very little web page.
    </p>
    <p>
      You said: <pre>$input</pre>
    </p>
  </body>
</html>
INDEX
					},
				   };
	$self->{404}=<<'ERRORPAGE';
HTTP/1.1 404 Not Found
Content-Type: text/html

<html>
  <head>
    <title>Inexistent page</title>
  </head>
  <body>
    <p>
      The page '%s' does not exist.
    </p>
    <p>
      You said: <pre>%s</pre>
    </p>
  </body>
</html>
ERRORPAGE

	$self->{'validpage'} = '/index.html';

	ARG: while(my ($key,$val)=splice @args,0,2) {
		($key eq "-page") && do {
			$self->{pages}->{$val->[0]}=$val->[1];
			next ARG;
		};
		($key eq "-activepage") && do {
			$self->{pages}->{$val->[0]}=$val->[1];
			$self->{pagehandler}->{$val->[0]}="activepage";
			next ARG;
		};
	};

	return $self;
}

=pod "

=item I<start ()>

Overloaded from I<IDX::Test::NetworkServer>, starts a network
server using LWP's L<HTTP::Daemon>.

=cut "

sub start {
	my ($self)=@_;

	my $d=new HTTP::Daemon(LocalPort=>$self->{port});
	if (!defined $d) {
		require POSIX; POSIX::_exit(42);
	};

	die("Cannot fork() a Perl test server daemon")
	  unless (defined(my $pid=fork()));
	if ($pid) {
		$self->{pid}=$pid;
		return $self;
	}

	# Ye olde main loop.
	while(my $c = $d->accept()) {
		eval {
			$self->_answer($c);
			1;
		} || do { warn $@; };
		$c->close();
	};
	require POSIX; POSIX::_exit(0); # Huh ?
	# Should never return in the child.

}


=pod "

=item I<_answer ($c)>

Answers the requests in HTTP from the main loop in start(). $c is
an I<HTTP::Daemon::ClientConn> object.

=cut "

sub _answer {
	my ($self, $sockref) = @_;

	my $req=$sockref->get_request();
	die "Input socket gives no request" if (!defined $req);
	my $page=$req->uri();

	if (defined $self->{pagehandler}->{$page}) {
		if ($self->{pagehandler}->{$page} eq "activepage") {
			return $self->{pages}->{$page}->($sockref,$req);
		};
		die "Unrecognized handler $self->{pagehandler}->{$page}";
	};

	my $input = $req->as_string();
	if (exists $self->{pages}->{$page}) {
		my $handler=$self->{pages}->{$page};
		if (! ref($handler)) {
			print $sockref $handler;
		} elsif (ref($handler) eq "CODE") {
			print $sockref (&$handler($input));
		} else {
			die "Unrecognized static handler $handler";
		}
	} else {
		print $sockref sprintf($self->{404},$page,$input);
	};
}

=pod "

=back

=cut "


eval join('',<main::DATA>) || die $@ unless caller();

1;

__END__

####################  TEST CASE  ####################

package IDX::Test::WebServer;
use strict;

use IDX::Test;
use LWP::UserAgent;
use IO::Socket::INET;
use Net::HTTP;

number_of_tests(7);

my $server;

test "New webserver" => sub {
	TEST ($server = new IDX::Test::WebServer (-page=>["/zoinx.html"=>
"HTTP/1.0 200 OK
Connection: close

Zoinx!
"],
	-activepage=>["/active.cgi" => sub {
					  my ($sock,$req)=@_;
					  print $sock "HTTP/1.1 200 OK
Connection: close

";
					  print $sock $req->header("User-Agent");

}]
)->start());
};

test "Connection to an existent page" => sub {
	my $request = &newLWPRequest ("http", "localhost", $server->{'port'},
								  $server->{'validpage'});
	my $agent = new LWP::UserAgent ();
	$agent->agent ("IDEALX test suite");
	my $reply = $agent->request ($request) ||
		die "Couln't execute the request: $!";

	TEST ($reply->code () == 200) ||
		warn "Code for $server->{'validpage'}: ".$reply->code ();
	TEST ($reply->content () =~ m/little web page/i) ||
		warn "Contents of $server->{'validpage'}: ".$reply->content ();

	$request = &newLWPRequest ("http", "localhost", $server->{'port'},
								  "/zoinx.html");
	$reply = $agent->request ($request) ||
		die "Couln't execute the request: $!";

	TEST ($reply->code () == 200) ||
		warn "Code for /zoinx.html: ".$reply->code ();
	TEST ($reply->content () =~ m/zoinx/i) ||
		warn "Contents of /zoinx.html: ".$reply->content ();
};

test "Connection to an inexistent page" => sub {
	my $request = &newLWPRequest ("http", "localhost", $server->{'port'},
								  "/ThisPageDoesNotExist.html");
	my $agent = new LWP::UserAgent ();
	$agent->agent ("IDEALX test suite");
	my $reply = $agent->request ($request) ||
		die "Couln't execute the request: $!";

	TEST ($reply->code () == 404);
};

test "bogus request does not kill server" => sub {
	my $sock=new
	  IO::Socket::INET(PeerHost=>"localhost",PeerPort=>$server->{'port'});
	TEST($sock) || die "Could not open sock: $!";
	$sock->close();

	my $request = &newLWPRequest ("http", "localhost", $server->{'port'},
								  $server->{'validpage'});
	my $agent = new LWP::UserAgent ();
	$agent->agent ("IDEALX test suite");
	my $reply = $agent->request ($request) ||
		die "Couln't execute the request: $!";

	TEST ($reply->code () == 200);
	TEST ($reply->content () =~ m/little web page/i) ||
	  warn $reply->content ();
};

test "active page" => sub {
	my $request = &newLWPRequest ("http", "localhost", $server->{'port'},
								  "/active.cgi");
	my $agent = new LWP::UserAgent ();
	$agent->agent ("IDEALX test suite");
	my $reply = $agent->request ($request) ||
		die "Couln't execute the request: $!";

	TEST ($reply->code () == 200);
	TEST($reply->content() =~ m/IDEALX test suite/);
};

open(SERVICES,"/etc/services");
my $services=join('',<SERVICES>);
close(SERVICES);

test "Posting big amounts of data to request page (LWP)" => sub {
	my $agent = new LWP::UserAgent ();
	my $request=new HTTP::Request
	  ("POST","http://localhost:$server->{port}/active.cgi",
	   new HTTP::Headers("Content-Type" => "text/plain"),
	   $services);
	$agent->agent ("IDEALX test suite");
	my $reply = $agent->request($request) ||
		die "Couln't execute the request: $!";
	(TEST($reply->code () == 200) &&
	 TEST($reply->content() =~ m/IDEALX test suite/)) ||
	   warn $reply->as_string();
};

test "Posting big amounts of data to request page (chunked Net::HTTP)" => sub {
	my $sock = Net::HTTP->new(Host=>"localhost",
							  PeerPort => $server->getPort());
	$sock->write_request(POST => "/active.cgi",
						 "Content-Type" => "text/plain",
						 "Transfer-Encoding" => "chunked",
						"User-Agent" => "IDEALX test suite");
	while(my $chunk=substr($services,0,4096,"")) {
		$sock->write_chunk($chunk);
	};
	$sock->write_chunk_eof();
	my ($code,$mess,%headers)=$sock->read_response_headers();

	TEST($code == 200) || warn "\$code = $code\n";

	my $buf="";
	1 until $sock->read_entity_body($buf,4096);
	TEST($buf =~ m/IDEALX test suite/) || warn $buf;
};

1;
