#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<auto-ca.pl> - Script for batch operation of the CA entity

=head1 SYNOPSIS

B<auto-ca.pl>

=head1 EXAMPLES

Invoke with no arguments from the crontab:

 */5 * * * * auto-ca.pl

=head1 DESCRIPTION

B<auto-ca.pl> scans the incoming CSR and CRR transport pools and (if the
CA is in automatic mode) attempts to accept them all. No output is
performed except in case of an error.

B<auto-ca.pl> also searches for unfinished certificate creation steps,
like for example a failed attempt to send a central-mode PKCS#12 to its
recipient, and retries the operation. This is useful in case of network
errors, so that the generated data are not lost.

=cut "

use strict;

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Log;
use IDX::PKI::CA;
use IDX::PKI::Principal;
use IDX::PKI::Transport;
use IDX::PKI::Message::Simple;
use IDX::PKI::Crypto::PrivateKey;
use IDX::PKI::Crypto::X509;
use IDX::PKI::Crypto::CSR;
use IDX::PKI::Crypto::PKCS12;

use Carp;
# For reading incomplete operation messages.
use Fcntl qw(:DEFAULT :flock);
# For checking return values of syscalls to ensure atomicity.
use POSIX qw(:errno_h);
use IO::File;

my $config = the IDX::PKI::Config ();
my $log    = new IDX::PKI::Log ();
my $ca     = new IDX::PKI::CA ($log);

my $waitdir    = $config->getval ("CA", "waiting_dir");
my $cakeyfile  = $config->getval ("CA", "key");
my $cacertfile = $config->getval ("CA", "certificate");
my $sectioncsr = $config->getval ("CA", "csr_transport_in");
my $sectioncrr = $config->getval ("CA", "crr_transport_in");
my $cacertsdir = $config->getval ("paths", "cacertsdir");

my $cakey  = load IDX::PKI::Crypto::PrivateKey ($cakeyfile);
my $cacert = load IDX::PKI::Crypto::X509       ($cacertfile);

####################  Failed Operations  ####################

MESSAGE: foreach my $id (IDX::PKI::Transport::Unspooler::list_dir
						 (undef, $waitdir)) {
	my $lock     = new IO::File;
	my $filename = "$waitdir/$id";
	sysopen ($lock, $filename, O_RDWR) || do {
		# No such file or directory: Someone was faster than us.
		next MESSAGE if ($! == ENOENT);
		carp "Can't open $filename: $!";
	};

	# File opened but lock failed -- we have a concurrent faster than we
	# are, give up.
	next MESSAGE unless (flock ($lock, LOCK_EX | LOCK_NB));

	# For Postfix, non-executable files indicate unfinished transmission.
	# We keep this convention.
	next MESSAGE unless (-x $lock);

	my $message = load IDX::PKI::Message::Simple ($filename);
	my $x509 = $message->getObjects ("IDX::PKI::Crypto::X509");
	my ($enccsr, $encp12) =
		$message->getObjects ("IDX::PKI::Crypto::PKCS7::EnvelopedData");
	my ($state, $attempts) = map {scalar ($message->getData ($_))}
		qw (state attempts);
	$attempts ++;

	$enccsr->unlock ([$cakey, $cacert]);
	my $csr = deserialize IDX::PKI::Crypto::CSR ($enccsr->getData ());

	my $p12obj;
	if ($encp12) {
		$encp12->unlock ([$cakey, $cacert]);
		$p12obj = parse IDX::PKI::Crypto::PKCS12 ($encp12->getData ());
	}


	# Last undone step was P12_DISTRIBUTION.
	if ($state eq "P12_DISTRIBUTION") {
		try {
			$log->attempt ({
				"-subject" => "CERT_SIGN",
				"-details" => "Sending the PKCS#12 to its recipient.",
			}, sub {
				$ca->_handle_centralized ($csr, $x509, $p12obj);
				$state = "PUBLISH";				# Next step.
				$attempts = 0;					# Reset the counter.
			});
		} catch IDX::PKI::Error with {
			$ca->_dump_state ($state, $attempts, $csr, $x509, $p12obj);
			unlink ($filename);	# Remove the old message.
			close ($lock);		# Release the lock.
			no warnings;
			next MESSAGE;
		};
	}


	# We are done.
	unlink ($filename);	# Remove the message.
	close ($lock);		# Release the lock.
	next MESSAGE;
}


####################  Automatic CSR & CRR  ####################

my $mode = $config->val ("CA", "mode");
exit (0) if ((! $mode) || ($mode !~ m/auto/i));

# Pending CRRs.
my $transportcrr = new IDX::PKI::Transport::Unspooler ("TRANSPORT-IN-".
													   $sectioncrr);
my @list_of_crr = $transportcrr->list();

foreach my $id (@list_of_crr) {
	try {
		my $crr = pullCRR ($transportcrr, $id) ||
			do {warn "Cannot get message id $id (lost race?)\n" ; next;};
		$ca->crr_accept ($crr);
		$transportcrr->expunge();
	} otherwise {
		my ($E) = @_;
		warn $E;
		$log->alert ("CRR_ACCEPT", "-details" => $id, "-attachment" => $E);
	};
}

# Pending CSRs.
my $transportcsr = new IDX::PKI::Transport::Unspooler ("TRANSPORT-IN-".
													   $sectioncsr);
my @list_of_csr = $transportcsr->list();

foreach my $id (@list_of_csr) {
	try {
		my $signedcsr = pullSignedBlob ($transportcsr, $id) ||
			do {warn "Cannot get message id $id (lost race?)\n"; next;};
		$signedcsr->verify ("-cadir" => $cacertsdir);
		my $csr = deserialize IDX::PKI::Crypto::CSR ($signedcsr->get_object());
		$log->clearTracking();
		$log->addTracking ($csr->getFormEntry ("tracking"));
		$ca->csr_accept ($csr);
		$transportcsr->expunge();
	} otherwise {
		my ($E) = @_;
		warn $E;
		$log->alert ("CSR_SIGN", "-details" => $id, "-attachment" => $E);
	};
}

exit (0);
