#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<ca.pl> - Script for by hand operation of the CA entity.

=head1 SYNOPSIS

B<ca.pl> [B<--cn> I<CN>] [B<--bootstrap> I<file>] [B<--keysize> I<size>]
[B<--outfile> I<file>]

=head1 OPTIONS

=over 4

=item B<--cn>

The CN of the certificate(s) to generate, for a centralised private key
generation.

=item B<--keysize>

Key size for a PKCS#12 file.

=item B<--bootstrap>

A bootstrap SPKAC (form entries have a bootstrap-specific meaning in this
case, see the man page).

=item B<--outfile>

Write output to file instead printing it.

=back

=head1 EXAMPLES

In the IDXPKI bootstrap phase, the SPKAC files contain a DN prefix set
up by the requestor. They are accepted this way:

 ca.pl --bootstrap file.csr > mycert.pem

To issue a centralized certificate in PKCS#12 form (without a password):

 ca.pl --cn "Administrator" --keysize 1024 > admin.p12

=head1 DESCRIPTION

B<ca.pl> generates new certificates, and/or private keys, in PEM or
PKCS#12 format. The result is either printed to standard output (by
default) or written it to a file (with the B<--outfile> switch).

The B<--cn> option is mandatory for central-mode key generation; it sets
the common name of the certificate(s) to be generated. For SPKAC signing,
the CN is taken from the SPKAC itself. The certificate DN is C<< CN=<cn>,
<idxpki_subdn>, <rootdn> >>, where I<< <rootdn> >> and I<< <idxpki_subdn>
>> are taken from the configuration.

If the B<--keysize> option is passed with an integer, one centralized
certificate will be generated

If the B<--bootstrap> option is passed with a filename argument, then this
file is expected to be a SPKAC request with specially formatted request
fields that go like this:

=over 4

=item I<DNS=some.domain.com>

I<some.domain.com> will be set as a FQDN subjectAltName in the
resulting certificate. Additionnally, the certificate will be supposed
to be a B<server certificate>, and the key usage flags will be set
accordingly.

=item I<URI=ldaps://some.domain.com/>

I<ldaps://some.domain.com/> will be set as an URI subjectAltName in
the resulting certificate.  Additionnally, the certificate will be
supposed to be a B<server certificate>.

=item I<email=some@domain.com>

I<some@domain.com> will be set as an UFQDN subjectAltName in
the resulting certificate.

=item I<CN=some value>

=item I<0.CN=some value>

=item I<1.CN=some value> ...

The bootstrap SPKAC gets to specify its CN. The rest of the DN will be
appended from I<idxpki-subdn> and I<rootdn>, both taken from the
configuration (so that the bootstrapped PKI members do not need to know
their full DN at request time). The first form member that looks like
C<CN> or C<something.CN> will be used, the others will be discarded.

=back

=head1 BUGS

The whole command line stuff is limited to the bare minimum needed for
supporting the IDX-PKI bootstrap: PKCS#10 is not implemented, SPKAC
support is ad-hoc, and no key usage flags are set on certificates (this
should be no problem since certificates concerned by the bootstrap are
only ever seen as WWW or LDAP servers and thus do not need to be selected
among a certificate store, which would require them having extensions).
The missing features won't be implemented anytime soon, unless there is
another specific need for CA tasks outside the HTTP interface.

=cut "

use strict;

use IDX::PKI::Error;
use IDX::PKI::Log;
use IDX::PKI::Config;
use IDX::PKI::CA;
use IDX::PKI::Crypto::PKCS12;
use IDX::PKI::Crypto::SPKAC;
use IDX::PKI::Crypto::Tools;
use IDX::PKI::DN;

use Carp;
use Getopt::Long;

my $config = the IDX::PKI::Config ();
my $log    = new IDX::PKI::Log ();
my $ca     = new IDX::PKI::CA ($log);

############################# MANUAL OPERATION #####################

my ($cn, $bootstrapspkac, $keysize, @extensions, $email, %sortedexts,
	$outfile);

sub usage {
	my ($message) = @_;
	require Pod::Usage;
	Pod::Usage::pod2usage({-verbose => 1, -exitval => 2,
						   ($message ? (-message => $message) : ())});
}

Getopt::Long::GetOptions
	("cn=s"        => \$cn,
	 "bootstrap=s" => \$bootstrapspkac,
	 "pkcs10=s"    => sub {die "PKCS10 support is not implemented"},
	 "spkac=s"     => sub {die "Non-bootstrap SPKAC support is not ".
							   "implemented"},
	 "keysize=i"   => \$keysize,
	 "email=s"     => \$email,
	 "extension=s" => \@extensions,
	 "outfile=s"   => \$outfile,
	 ) || usage();

usage ("--cn option is mandatory in central mode") if ($keysize && (! $cn));
my $dn = $keysize ? "cn=$cn" : "";
my $subdn  = $config->getval ("dn", "idxpki_subdn");
my $rootdn = $config->getval ("dn", "rootdn");
$dn .= ($dn ? "," : "") . $subdn . ($rootdn ? ",$rootdn" : "");
my @dn = parse IDX::PKI::DN ($dn)->canonical ();

usage ("--keysize and --bootstrap are mutually incompatible") if
	($keysize && $bootstrapspkac);

foreach my $type (keys (%IDX::PKI::Crypto::X509::keyUsageExtensions)) {
	foreach my $ext (@{$IDX::PKI::Crypto::X509::keyUsageExtensions{$type}}) {
		push (@{$sortedexts{$type}}, grep {$_ eq $ext} @extensions);
	}
}

my %altnameopts;
$altnameopts{"email"} = $email if ($email);

if ($keysize) {
	my ($key, $req) = $ca->_generate_private_key ($keysize, \@dn);
	throw IDX::PKI::Error::Internal ("INCORRECT_ARGS", "-key" => $key,
									 "-req" => $req) unless ($key && $req);

	my $transaction = new IDX::PKI::CertDB::CA ("CA", 1);
	my $x509 =
		$ca->_sign_cert (
						 "req"        => $req,
						 "type"       => "PKCS10",
						 "dn"         => \@dn,
						 "validity"   => $config->getval ("CA",
														  "default_validity"),
						 "extensions" => \%sortedexts,
						 "altnames"   => \%altnameopts,
						 "serial"     => $transaction->nextSerial (),
						 );
	$transaction->addCert ($x509);
	$transaction->commit ();

	my $pkcs12 = build IDX::PKI::Crypto::PKCS12 ($x509, $key)->serialize ();

	if ($outfile) {
		IDX::PKI::Crypto::Tools::string2file ($pkcs12, $outfile);
	} else {
		print $pkcs12;
	}
	exit (0);
}

if ($bootstrapspkac) {
	my $spkac = load IDX::PKI::Crypto::SPKAC ($bootstrapspkac);
	my %form = $spkac->getForm();
	foreach my $try (keys %form) {
		next unless ($try =~ m/(?:^|\.)CN$/i);
		unshift (@dn, "CN" => $form{$try});
		last;
	}

	foreach my $try (keys %form) {
		next unless ($try =~ m/(?:^|\.)(DNS|URI|email)$/i);
		my $alt = $1;
		if (my $val = $form{$try}) {
			$altnameopts{$alt} = [] unless defined ($altnameopts{$alt});
			push (@{$altnameopts{$alt}}, $val);
		}
	}

	my $transaction = new IDX::PKI::CertDB::CA ("CA", 1);
	my $x509 =
		$ca->_sign_cert (
						 "req"        => $spkac,
						 "type"       => "SPKAC",
						 "dn"         => \@dn,
						 "validity"   => $config->getval ("CA",
														  "default_validity"),
						 "extensions" => \%sortedexts,
						 "altnames"   => \%altnameopts,
						 "serial"     => $transaction->nextSerial (),
						 );
	$transaction->addCert ($x509);
	$transaction->commit ();

	my $cert = $x509->serialize("PEM");
	if ($outfile) {
		IDX::PKI::Crypto::Tools::string2file ($cert, $outfile);
	} else {
		print $cert;
	}

	exit (0);
}

usage ("Need one of --bootstrap or --keysize options");
