#!/usr/bin/perl -w
#
# Download the CRL periodically, and kick Apache to have it reload it.
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<CRL-get.pl> - Script to download the CRL from the CA

=head1 SYNOPSIS

B<CRL-get.pl> [B<--rootca>] [B<--crlname> I<crl name>] [B<--entity>
I<entity>] [B<--uri> I<uri>] [--apache-cannot-reread-crl]

=head1 DESCRIPTION

This script downloads the CRL from the CA (or the root CA) and optionnaly
restarts Apache.  Should be invoked from the crontab, with
idxpki-www rights (or whatever the owner of the CRL is).

=head1 OPTIONS

=over 4

=item B<--rootca>

If set, tells that we want to manage the root CA CRL.  It is equivalent as
C<--crlname=RootCRL --entity=rootCA>.

=item B<--crlname> I<crl name>

Name of the CRL file(s) on this host (see L<IPRFC-07>).  The F<.crl>
extension will be appended for the DER format, and F<.pem> for the PEM
format.  B<--rootca> and B<--crlname> are mutually exclusive: If both are
specified, B<--crlname> takes precedence.  It defaults to a value computed
from the configuration parameter I<crlurl> in section I<rootCA> if
B<--rootca> is set, in section I<CA> otherwise.

=item B<--entity> I<entity>

Name of the entity in the configuration file: It is used to get the URI
from the C<crlurl> option.  B<--rootca> and B<--entity> are mutually
exclusive: If both are specified, B<--entity> takes precedence.  It
defaults to C<rootCA> if B<--rootca> is set, C<CA> otherwise.

=item B<--uri> I<uri>

URI from which to fetch the CRL.  If B<--rootca> or B<--entity> are also
defined, B<--uri> takes precedence.  Its default value is taken from the
configuration file, in the I<entity> C<crlurl> section.

=item B<--apache-cannot-reread-crl>

If this option is activated, Apache is restarted every time a new CRL is
received. This is mandatory for stock mod_ssl which cannot re-read its
CRL. With IDEALX's patch, one only needs to restart Apache the very first
time a CRL is received (because the httpd command line has to be updated),
but it works only with a CRL file, not a CRL directory. The default
assumes that Apache has to be restarted.

=back

=cut "

use strict;

use IDX::PKI::Error;
use IDX::PKI::Log;
use IDX::PKI::Config;
use LWP::UserAgent;
use Getopt::Long;
use Pod::Usage;

my $config = the IDX::PKI::Config ();

my $rootca = 0;
my $crlname;
my $entity;
my $uri;
my $restart = $config->getval ("webserver", "restart");

GetOptions ('rootca!'   => \$rootca,
			'crlname=s' => \$crlname,
			'entity=s'  => \$entity,
			'uri=s'     => \$uri,
			'apache-cannot-reread-crl' => \$restart,
			) || pod2usage ('-verbose' => 1, '-exitval' => 1);

# Command-line options have priority over default values.
$entity = ($rootca ? 'rootCA'  : 'CA')     unless (defined $entity);
$uri = $config->getval ($entity, 'crlurl') unless (defined $uri);
unless (defined $crlname) {
	$crlname = $uri;
	$crlname =~ s|.*/||;		# Strip path.
	$crlname =~ s|\.[a-z]+$||;	# Strip extension.
}

our $oldcrl = $config->getval ("paths", "crldir")."/$crlname.crl";
our $newcrl = $config->getval ("paths", "crldir")."/$crlname.crl-NEW$$";
our $crlpem = $config->getval ("paths", "crldir")."/$crlname.pem";
our $bundle = $config->getval ("paths", "crldir")."/crl-bundle.pem";

##########################################################################

my $log = new IDX::PKI::Log ();

# Cleans old failures to avoid filling up the directory.
unlink glob ("$oldcrl-NEW*");
unlink glob ("$crlpem-NEW*");
unlink glob ("$bundle-NEW*");

my $ua = new LWP::UserAgent;

my $req = new HTTP::Request (GET => $uri);

my $res = $ua->request ($req, $newcrl);

do {
	unlink $newcrl;
	my $err = $res->status_line();
	die "Failed getting the CRL from the CA:\n$uri\n($! - LWP result: $err)\n";
	exit;
} if (! $res->is_success());

# Now compare the two CRL timestamp, and in case of changes, rotate
# them and kick any relevant daemons (just Apache for now).

# Returns an integer that only grows with new CRLs
sub version_of_crl {
	my ($crlfile)=@_;

	local *OPENSSL;
	my $cmdline = "openssl crl -text -inform der -in $crlfile -noout -CApath ".
		$config->getval ("paths", "cacertsdir")." 2>&1 |";
	open (OPENSSL, $cmdline);
	my @openssl_output = <OPENSSL>;
	close (OPENSSL);
	my ($verifyOK,$seconds);
	foreach (@openssl_output) {
		$verifyOK++ if m/verify OK/;

		m/Last Update:(.*)$/ && do {
			# FIXME: this is very clever but not portable.
			$seconds=`date +%s -d "$1" 2>/dev/null`; chomp $seconds;
		};

	};
	die "Wrong signature on $crlfile - left new CRL $newcrl\n".
		"The openssl command line was:\n$cmdline\n\n".
		"Openssl output:\n".join ("", @openssl_output)
		unless $verifyOK;
	return $seconds;
}

if (-f $oldcrl) { # CRL update, compare dates
	my ($olddate,$newdate)=(version_of_crl($oldcrl),version_of_crl($newcrl));

	die "CRL travels backwards in time - left new CRL $newcrl\n"
		unless ($olddate <= $newdate);

	do {
		unlink($newcrl);
		exit 0;
	} if ($olddate == $newdate);
} else { # first CRL transfer, just check signature
	version_of_crl($newcrl);
	$restart++;
};

rename($newcrl,$oldcrl);

system ("openssl", "crl", "-inform", "der", "-outform", "pem",
		"-in", $oldcrl, "-out", "$crlpem-NEW$$");
rename ("$crlpem-NEW$$", $crlpem) if ("$crlpem-NEW$$");

# mod_ssl is a plague! It fails miserably to use a CRL directory although
# it is documented. Fortunately a "crl-bundle.pem" file does the job.
local *BUNDLE;
open (BUNDLE, ">", "$bundle-NEW$$") ||
	die "Can't write crl-bundle.pem-NEW$$: $!";
foreach my $file (glob ($config->getval ("paths", "crldir")."/*.pem")) {
	next if ($file =~ m/crl-bundle/);
	local *CRLFILE;
	open (CRLFILE, "<", $file) || die "Can't read $file: $!";
	print BUNDLE join ("", <CRLFILE>);
	close (CRLFILE);
}
close (BUNDLE);
rename ("$bundle-NEW$$", $bundle);

# Maybe we are not running a standalone web server. In this case the
# task of updating the CRL is left to the web server's administrator.
if ($restart &&
	-x $config->getval ("paths", "sbindir")."/reload-CRL-in-Apache.pl") {
	system ($config->getval ("paths", "sbindir")."/reload-CRL-in-Apache.pl");
}
