#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<ee-certs-search.cgi> - Searches for certificates in the PKI repository LDAP server

=head1 DESCRIPTION

This scripts can search both user certificates and server certificates.
In the case of user certificates, it returns only those that aren't
revoked.

=cut "

use strict;

use IDX::PKI::Config;
use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Principal qw (zulu_datenow);
use IDX::PKI::CertDB;
use IDX::PKI::Crypto::X509;
use IDX::PKI::Profile;

=pod "

=head1 PARAMETERS

At least one of the following parameters must be present:

=over 4

=item *

C<idxpkiSubjectID>: "User identifier" filter (see
L<IDX::PKI::Crypto::X509/getUserID> for what a "user identifier" is);

=item *

C<idxpkiSubjectEmail>: Email address filter;

=item *

C<idxpkiSerial>: Serial number filter.

=back

=cut "

# We don't want change the API, but CADB require other variable name
my %compat = ( "idxpkiSubjectID"    => "ID",
			   "idxpkiSubjectEmail" => "email",
			   "idxpkiSerial"       => "serial" );

$cgi->setupViews ("ee-certs-search.php");
$cgi->authorize ("__NONE__");

############### Configuration informations ####################
my $limit = $config->val ("EE", "max_search_count");
$limit ||= 0;

################ Call parameters ###############################
# Boolean validation can't fail, can it?
my $revoking = $cgi->validate
  ("revoking", \&IDX::PKI::ValidateInput::boolean, "-default" => undef);

# assign values passed by users
my %criteria = ();
foreach my $param (keys (%compat)) {
	my $value =
	  $cgi->validate ($param, \&IDX::PKI::ValidateInput::LDAP_wildcard,
					  "-default" => undef);
	if ($cgi->validateFailures() && $cgi->internalparam ("ERROR_$param")) {
		$cgi->internalparam ("ERROR", $param);
		$cgi->error (INPUT_FORMAT);
	} else {
		$criteria{$compat{$param}} = $value if (defined $value);
	}
}
$cgi->error (INPUT_EMPTY) unless (%criteria);

$criteria{'!+revocation'} = "R" if ($revoking);

my $onlyvalid = 1;
# Rationale: If we search for a serial number, show the certificate
# even if it is expired or revoked.
$onlyvalid = 0 if (exists $criteria{'serial'} &&
				   $criteria{'serial'} =~ /^[a-z0-9]+$/i);
if ($onlyvalid) {
	$criteria{'validAt'}    = "now";
	$criteria{'+revocation'} = "V";
}

################ Certificate search ###############################
my @certs;
try {
	my $reader = new IDX::PKI::CertDB::CA ("EE");
	my $now = zulu_datenow();
	
		CERTS: foreach my $cert ($reader->searchCerts (%criteria)) {
		my $profile = $reader->getCertAttr ($cert, "profile");
		next CERTS unless ($profile);
		

		my %certValues;
		$certValues{"idxpkiSubject"} = $cert->getSubjectDN ();
		$certValues{"idxpkiSubjectID"} = $cert->getUserID ();
		$certValues{"idxpkiSubjectEmail"} = $cert->getEmail ();

		#GUI friendly
		if ($reader->getCertAttr($cert, "revocation") eq "V") {
			$certValues{"idxpkiStatus"} = "valid";
		} elsif ($reader->getCertAttr($cert, "revocation") eq "R") {
			$certValues{"idxpkiStatus"} = "revoked";
		} elsif ($cert->getAttribute("not_after_zulu") < $now ||
				 $cert->getAttribute("not_before_zulu") > $now ) {
			$certValues{"idxpkiStatus"} = "expired";
		} else {
			$certValues{"idxpkiStatus"} = "unknown";
		}

		$certValues{"idxpkiSerial"} = $cert->getSerial();
		$certValues{"idxpkiCertPublicationTime"} =
		  $cert->getAttribute("not_before");
		$certValues{"idxpkiCertExpirationTime"} =
		  $cert->getAttribute("not_after");

		push (@certs, \%certValues);
	}
} catch IDX::PKI::Error::Network with {
	$log->alert ("REPOSITORY_CONNECT", "-details" => shift);
	$cgi->failure (REPOSITORY_CONNECT);
};

my %different_dns;
foreach my $entry (@certs) {
	$different_dns{$entry->{'idxpkiSubject'}} = 1;
}

# No more than $limit, if not null.
($limit && scalar (keys (%different_dns)) > $limit) ?
  $cgi->error (REPOSITORY_TOOMANY) : $cgi->success (\@certs);
