#!/usr/bin/perl
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<ra-certs-search.cgi> - Searches for certificates in the PKI repository LDAP server

=head1 DESCRIPTION

Search for valid certificates through a LDAP repository according to some
criteria.

=cut "

use strict;

use IDX::PKI::CommonVars;
use IDX::PKI::Config;
use IDX::PKI::Log;
use IDX::PKI::Authentication qw(authorize is_authorized);
use IDX::PKI::Registry;
use IDX::PKI::Error;
use IDX::PKI::Profile;
use IDX::PKI::Principal;
use IDX::PKI::CertDB;

=pod "

=head1 PARAMETERS

Waits for at least one of the following CGI parameters:

=over 4

=item *

C<idxpkiSubjectID>: "User identifier" filter (see
L<IDX::PKI::Crypto::X509/getUserID> for what a "user identifier" is);

=item *

C<idxpkiSubjectEmail>: Email address filter;

=item *

C<idxpkiSerial>: Serial number filter.

=back

If the C<revoking> parameter is supplied and true, returns only
certificates which have not been revoked.

If the C<revoking> or C<recovery> parameters are supplied and true,
returns only certificates which may be recovered by the connected user
(that is, certificates corresponding to profiles he is allowed to use).

=cut "

# we don't want change the API, but CADB require other variable name
my %compat = (
			  "idxpkiSubjectID"    => "ID",
			  "idxpkiSubjectEmail" => "email",
			  "idxpkiSerial"       => "serial",
			 );

############### CGI init ####################
$cgi->setupViews ("ra-certs-search.php");

############### Access Restriction ####################
$cgi->authorize("PRIV_USE", "ACL-".$config->getval ('RA', 'web_acl'));

############### Call parameters ####################
my $revoking = $cgi->validate
  ("revoking", \&IDX::PKI::ValidateInput::boolean, "-default" => undef);

# assign values passed by users
my %criteria = ();
foreach my $p (keys (%compat)) {
	my $value =
	  $cgi->validate ($p, \&IDX::PKI::ValidateInput::LDAP_wildcard,
					  "-default" => undef);
	if (defined ($value) && ($value !~ m/^[\s]*$/)) {
		# check if value is well formed
		my $wf_value = &{$default_validate_search} ($value);
		if (defined ($wf_value)) {
			$cgi->param ($p, $wf_value);
			$criteria{$compat{$p}} = $wf_value;
		} else {
			$cgi->internalparam ('ERROR', $p);
			$cgi->error (INPUT_FORMAT);
		}
	}
}
$cgi->error (INPUT_EMPTY) unless (%criteria);

$criteria{'!+revocation'} = "R" if ($revoking);

my @certs;
try {
	my $reader = new IDX::PKI::CertDB::CA ("RA");
	foreach my $cert ($reader->searchCerts (%criteria)) {
		my %certValues;
		$certValues{"idxpkiProfile"} =
		  $reader->getCertAttr($cert, "profile");
		$certValues{"idxpkiSubject"} = $cert->getSubjectDN;
		$certValues{"idxpkiSubjectID"} = $cert->getUserID;
		$certValues{"idxpkiSubjectEmail"} = $cert->getEmail();

		#GUI friendly
		if ($reader->getCertAttr($cert, "revocation") eq "V") {
			$certValues{"idxpkiStatus"} = "valid";
		} elsif ($reader->getCertAttr($cert, "revocation") eq "R") {
			$certValues{"idxpkiStatus"} = "revoked";
			$certValues{"reason"} = $reader->getCertAttr($cert, "reason");
		} elsif ($cert->getAttribute("not_after_zulu") < zulu_datenow() ||
				 $cert->getAttribute("not_before_zulu") > zulu_datenow()) {
			$certValues{"idxpkiStatus"} = "expired";
		} else {
			$certValues{"idxpkiStatus"} = "unknown";
		}

		$certValues{"idxpkiSerial"} = $cert->getSerial;
		$certValues{"idxpkiCertPublicationTime"} =
		  $cert->getAttribute("not_before");
		$certValues{"idxpkiCertExpirationTime"} =
		  $cert->getAttribute("not_after");

		push (@certs, \%certValues);
	}
} catch IDX::PKI::Error::Network with {
	$cgi->failure (REPOSITORY_CONNECT);
};


my %different_dns;
foreach my $entry (@certs) {
	$different_dns{$entry->{'idxpkiSubject'}} = 1;
}

# No more than $limit, if not null.
my $limit = $config->getPath ("RA", "max_search_count");
($limit && scalar (keys (%different_dns)) > $limit) ?
  $cgi->error (REPOSITORY_TOOMANY) : $cgi->success (\@certs);
