#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<publish.pl> - IDX-PKI publication module.

=head1 DESCRIPTION

This script is supposed to be run periodically. It receives messages and
publishes the corresponding data (user / server certificates, CA
certificates or CRLs along with additional informations), sending them to
their recipients by email, adding them into some LDAP directory, or other
kind of things.

=cut "

use strict;

use IDX::PKI::Config;
use IDX::PKI::Log;
use IDX::PKI::Error;
use IDX::PKI::Transport;
use IDX::PKI::Message::Publish;
use IDX::PKI::Crypto::X509;
use IDX::PKI::Crypto::Tools qw(file2string);
use IDX::PKI::Crypto::CRL;
use IDX::PKI::Profile;
use IDX::PKI::DN;
use IDX::PKI::Principal;
use IDX::PKI::Publish;

use Carp;

our $config = the IDX::PKI::Config ();

sub usercert_message {
	my ($url) = @_;
	my $message = <<"EOF";
[French translation at the end. / Traduction en franais  la fin.]

Your certificate has been published.  You can fetch it at the following
URL:
$url

          ----------------------------------------

Votre certificat a t publi. Vous pouvez le rcuprer  l'URL suivante:
$url
EOF
#'

	return $message;
}

sub servercert_message {
	my ($server) = @_;
	my $message = << "EOF";
[French translation at the end. / Traduction en franais  la fin.]

The certificate for $server has been created.

          ----------------------------------------

Le certificat pour $server a t cr.
EOF
}

my $log = new IDX::PKI::Log ();
my $publish = new IDX::PKI::Publish();
my $admin = $config->getval ("EE", "admin_email");

# Retreives the pending message list and treats them.
my $section = "TRANSPORT-IN-".$publish->getTransport();
my $transport = new IDX::PKI::Transport::Unspooler($section);
my @messages_list;
try {
	@messages_list = $transport->list();
} otherwise {
	my ($E) = @_;
	$log->alert('TRANSPORT_LIST', -details => "Transport: ".$section,
				-attachment => $E);
	croak $E;
};

MESSAGE: foreach my $id (@messages_list) {
	my $message;
	try {
		$message = $transport->pullByID ($id, 'IDX::PKI::Message::Publish');
	} otherwise {
		my ($E) = @_;
		$log->alert('TRANSPORT_MSG', -details =>
					["Transport: ".$section, "Message ID: $id"],
					"-attachment" => $E);
		warn $E;
		no warnings;
		next MESSAGE;
	};
	my $type = $message->getType ();

	####################  User Certificate  ####################
	if ($type eq 'certificate') {

		# Get certificate to publish and profile informations.
		my $x509 = $message->getObject ();
		my $dnobject = build IDX::PKI::DN ($x509->getSubjectDN ());
		my $profilename = $message->getProfile();
		unless (defined $profilename && $profilename ne '') {
			$transport->expunge ();
			next MESSAGE;
		}
		my $profile = new IDX::PKI::Profile ($profilename);
		my %form = $message->getForm ();

		# Send an email to the user, in decentralised mode.  If we don't
		# have access to a profile, we consider that we don't have to do
		# this!
		if ($profile && $profile->getType () eq 'user') {
			my $serial = $x509->getSerial;
			my $url = sprintf (scalar ($publish->getEEURL()), $serial);
			my $email = $x509->getEmail();
			unless ($email) {
				$email = ($form{'emailAddress'} ?
						  (ref($form{'emailAddress'}) ?
						   $form{'emailAddress'}->[0] :
						   $form{'emailAddress'}) :
						   $dnobject->getRDNValue ('Email'));
			}
			&IDX::PKI::Principal::sendmail
				($admin, $email,
				 "Your Certificate is Ready / Votre certificat est pret",
				 usercert_message ($url)) if ($email);
		}
	}
	####################  End of User Certificate  ####################

	####################  CA Certificate  ####################
	elsif ($type eq 'ca-certificate') {
			$transport->expunge ();
			next MESSAGE;
	}
####################  End of CA Certificate  ####################

####################  CRL  ####################
	elsif ($type eq 'crl') {
			$transport->expunge ();
			next MESSAGE;
	}
####################  End of CRL  ####################

####################  Unknown Type  ####################
	else {
		$log->alert("PUBLISH_TYPE_UNKNOWN", -attachment => $message);
		warn "Unknown data type to publish '$type'";
		$transport->release ();
		next MESSAGE;
	}
####################  End of Message Management  ####################

	$transport->expunge ();
}

