#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod

=head1 NAME

B<renewal.pl> - Sends certificate renewal reminder messages to users

=cut

require 5.6.0;

use strict;

use IDX::PKI::Config;
use IDX::PKI::Error;
use IDX::PKI::Log;
use IDX::PKI::CertDB;
use IDX::PKI::Principal;
use IDX::PKI::FileManip;

use Carp qw (croak);
use POSIX qw (mktime);

# A 24 hours delta for date comparison.
use constant DELTA => 24 * 3600;

my $config = the IDX::PKI::Config ();
my $log    = new IDX::PKI::Log ();

# Open a connection to the repository.
my $reader;
try {
	$reader = new IDX::PKI::CertDB::CA ("renewal");
} catch IDX::PKI::Error::Internal with {
	my ($E) = @_;
	$log->alert ("Error", "-details" => "Error connecting to the repository",
				 "-attachment" => $E);
	croak "Error connecting to the repository: $E";
};

# Get the days from the configuration.
my @days_list     = $config->getPath ("renewal", 'days');
my @active_list   = $config->getPath ("renewal", 'active');
my @textfile_list = $config->getPath ("renewal", 'textfile');

# Get the admin email address
my $from = $config->getPath ('EE', 'admin_email');

POOL: for (my $i = 0; $i <= $#textfile_list; $i++) {
	# Active or not
	next POOL unless ($active_list[$i]);

	# Get the reminder message.
	my $text = &IDX::PKI::FileManip::file_read ($textfile_list[$i]);

	# Expiration time in seconds.
	my $exptime = time() + $days_list[$i] * 24 * 3600;
	my $mindate = epoch2ldaptime ($exptime - DELTA / 2);
	my $maxdate = epoch2ldaptime ($exptime + DELTA / 2);

	my @list;
	# Get certificates' serials here.
	my @criteria = ("+revocation", "V",# don't get the revoked certificates
					"+expirationTime", ">=$mindate",
					"+expirationTime", "<=$maxdate"
					);
	try {
		@list = $reader->searchCerts (@criteria);
	} catch IDX::PKI::Error::Network with {
		my ($E) = @_;
		$log->alert ("Error", "-details" =>
					 "Error searching for expiring certificates",
					 "-attachment" => $E);
		warn $E;
		next POOL;
	};

	# If the expiration date matches
	LOOP: foreach my $x509 (@list) {
		# Get certificate informations.
		my $dn_subject = $x509->getSubjectDN ();
		my $identity   = $x509->getUserID ();
		my $email      = $x509->getEmail ();
		my $serial     = $x509->getSerial ();

		next LOOP unless ($email && ($email =~ m/\@/));

		try {
			my $expdate = cert2timeen ($x509->getAttribute ('not_after'));

			# Replace variables
			my $message = $text;
			$message =~ s/\[IDENTITY\]/$identity/gx;
			$message =~ s/\[EXPIRATIONDATE\]/$expdate/gx;
			$message =~ s/\[SERIALNUMBER\]/$serial/gx;

			# Send the email
			&IDX::PKI::Principal::sendmail
				($from, $email,
				 "Certificate Renewal / Renouvellement de certificat",
				 $message);

		} catch IDX::PKI::Error::IO with {
			my ($E) = @_;
			$log->alert ("Error", "-details" => "Error sending email to ".
						 "$dn_subject ($email)", "-attachment" => $E);
			warn $E;
			next LOOP;
		};
	}
}

exit 0;

# Convert an epoch time to a ldap time.
sub epoch2ldaptime {
	my ($time) = @_;
	my @timet = gmtime ($time);

	return sprintf("%04d%02d%02d%02d%02dZ", $timet[5] + 1900, $timet[4] + 1,
				   $timet[3], $timet[2], $timet[1]);
}


# Convert a date like Jun 21 16:04:14 2002 GMT
# to June, 21 2002 16h04 14s
sub cert2timeen {
	my ($time) = @_;
	my %mon = (
			   "Jan" => 'January',
			   "Feb" => 'February',
			   "Mar" => 'March',
			   "Apr" => 'April',
			   "May" => 'May',
			   "Jun" => 'June',
			   "Jul" => 'July',
			   "Aug" => 'August',
			   "Sep" => 'September',
			   "Oct" => 'October',
			   "Nov" => 'November',
			   "Dec" => 'December',
			   );

	my @date = ($time =~ m/^([^ ]+)\s+(\d+)\s+(\d+?):(\d+?):(\d+)\s+(\d{4})/);

	return undef if (!@date);

	return sprintf ("%s, %d %d %dh%d %ds", $mon{$date[0]}, $date[1],
					$date[5], $date[2], $date[3], $date[4]);
}
