#!/usr/bin/perl -Tw
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

B<BuildTimeConfig> - Support module for the configuration script (B<configure.pl>)

=head1 SYNOPSIS

 # Small trick to get the @INC right:
 use FindBin;
 use lib "$FindBin::Bin/some/path/in/source/tree";

 use BuildTimeConfig;

 our @opttemplates = (
 	# Much stuff here
 );

 our %opts = runOpts ();	# Sets %opts.

 our @loctemplates = (
 	# Ditto.
 );

 %opts = runLocate ();		# Update %opts.

 our %needed_perl_modules = (
 	# Idem.
 );

 our %needed_software = (
 	# Again.
 );

 runVersions ();	# Writes final config.status.


=head1 DESCRIPTION

This module is support code for B<configure.pl>. It is only used there,
but it is shared among several unrelated IDEALX projects.

The purpose is to ease the writing of the configure script, much like
autoconf does in the C world.

=head2 Options

B<configure.pl> can be programmed to accept a number of configure-time
options. They will be written into the F<config.status> file, and may be
used to take compile-time decisions (such as embedding various
subcomponents, or creating the default configuration file - see
L<IDX::Config>). Options have a type, which is either integer, boolean or
string. They can be set on the command line of B<configure.pl>, or
interactively by answering questions. Options can depend of each other
forcibly or just for their default value, but both the user and the
B<configure.pl> programmer can specify them in any order.

The order of precendece is: First, command-line options, then values from
a previous F<config.status> file, then default values. TODO: There should
be an exception if a default value uses I<valueof()>: Then the recomputed
value should be used rather than a previous configured one from
F<config.status>. If the B<--batch> option is used, then values from a
previous F<config.status> are never used.

Adding a new option is easy: One just adds an option template to the
I<@opttemplates> global variable (exported by default). Here is an example
for the format for an option template:

 'host-crl' =>
 {
 	'mandatory' => 0,
 	'type'      => "string",
 	'question'  => "which host should we fetch the ".
 		"CRL from?",
 	'default'   => sub {valueof ("host-ca")},
 },

The first thing is the name of the option (in command-line style, as
shown). Parameters I<default>, I<question> or I<mandatory> are either
literal values or subs computing said values, using I<valueof()> to get
the values of other variables (see below). The recognized parameters are
as follows:

=over 4

=item I<type>

The type of this option: C<boolean> (or C<bool>), C<integer> (or C<int>),
or C<string>. The default is C<boolean> for all options that begin with
C<--with>, C<--use>, C<--install> and C<--test>, and C<string> for all
others. Boolean values are C<"1"> and C<""> (the empty string).

=item I<default>

If a scalar, the default value. If a subroutine, it should compute the
value, maybe using I<valueof()> to fetch the value of another option.
Default may be C<undef> or return C<undef>.

=item I<mandatory>

If true, the script will try very hard to get a value for this one. If
false, failure to get a value simply causes the option to be omitted in
the resulting F<config.status>.

=item I<question>

The question to ask to the user. If C<undef>, no question will be asked
(even in interactive mode) and the default value will be used instead.

=item I<temporary>

If set, this variable will be erased after configuration and won't appear
in the F<config.status>. The side effect is that this value will be lost
for a further invocation of B<configure.pl>, so don't use it without a
good reason.

=back

=head2 On the Good Use of I<valueof()>

This section applies to all function references that want to make a
decision depending on the value of other variables.

In order for automatic inter-variable dependency tracking to take
place, all required values should be read at least once --- the
following is incorrect:

 'default' => sub {valueof ("test-ca") ? "localhost" :
 	valueof ("host-ca")},

and this is the correct version:

 'default' => sub {my $hostca = valueof ("host-ca");
 	valueof ("test-ca") ? "localhost" : $hostca},

The I<sub()> must be purely functional, and not use any side-effects such
as counters, global variables, etc.

I<valueof()> cannot be called outside this kind of anonymous subs (this
would not make sense anyway).

=head2 Directory and File Names

To be written. The variable is I<@loctemplates>, and it is a hash of
hashes similar to above, only with different per-location fields.

 'with-openssl-root' =>
 {
 	'default'   => "",
 	'directory' => ["/usr", "/usr/local"],
 },

 'with-openssl' =>
 {
 	'default'    => "",
 	'executable' => "openssl",
 },

=head2 Perl Dependencies

To be written. The key variables are I<%needed_perl_modules> and
I<%needed_software>.

=cut "

package BuildTimeConfig;

use strict;

use Getopt::Long;
use Pod::Usage;
use Exporter;
use Carp;

our @ISA=qw(Exporter);

use vars (
	'$DEBUG',				# If true, debug statements are printed
	'$configstatusfile',	# The path to the config.status file
	'@opttemplates',		# All option templates, set by user.
	'%opttemplates',		# Ditto, computed by module
	'@loctemplates',		# All filepath templates, set by user.
	'%needed_perl_modules',	# All required modules, set by user.
	'%needed_software',		# All required 3rd party software, set by user.
	'%opts',				# Options to write into config.status
	'%oldopts',				# Old options (may be used as default values)
	'%valueof_opts',		# Options touched by valueof() - horrible
							# side effect, must be reset by user code :-)
	'$valueof_active',		# True if valueof() can be used - to prevent
							# mistakes
);

=pod "

=head1 REFERENCE

=head2 Global Variables

Those global variables are exported by default.

=over 4

=item I<$configstatusfile>

The location of the F<config.status> file, defaults to C<./config.status>.

=item I<@opttemplates>

The templates for configure-time options (see L</Options>). Despite it
being an array, it must be filled like a hash, where odd items are keys
and even items are values. This trick is used to ask the options to the
user in a sensible order (barring inter-variable dependencies).

=item I<@loctemplates>

The templates for directories in which required pieces of software reside
(see L</Directory and File Names>). Despite it being an array, it must be
filled like a hash, where odd items are keys and even items are values.

=item I<%needed_perl_modules>

=item I<%needed_software>

=back

=cut "

our @EXPORT=qw($configstatusfile @opttemplates @loctemplates
			   %needed_perl_modules %needed_software
			   valueof install_all_implies test_enables
			   if_installed credentials_if_installed
			   runConfig runOpts runLocate runVersions);

=pod "

=head2 Functions

I<valueof()>, I<install_all_implies()>, I<test_enables()>,
I<if_installed()>, I<credentials_if_installed()>, I<runConfig()>,
I<runOpts()>, I<runLocate()> and I<runVersions()> are exported by default.
Most of them allow, or ease, the writing of option templates. The
I<runOpts()> and I<runLocate()> functions must respectively be called
after having filled I<@opttemplates> and I<@loctemplates>. The same goes
for I<runVersions()> with I<%needed_perl_modules> and I<%needed_software>.

=cut "

$DEBUG = $ENV{"DEBUG"} ? 1 : 0;
$configstatusfile = "./config.status";

=pod "

=head3 Hash Calculation Functions

I<Warning: You can't specify subs for parameters when you use these
functions; you have to give values.>

=over 4

=item I<install_all_implies ($question)>

Functional form that computes the whole hashref record for one option.
Expected usage is something like:

 "install-squid" => install_all_implies
 	("install IDX-PKI's Squid on this machine"),

The meaning is that the I<install-squid> option is boolean, and should be
set to true if I<install-all> is true (and asked to the user if not): It
could also have been written as:

 "install-squid" => {
 	"mandatory" => 1,
 	"question"  => sub {valueof("install-all") ?
 		undef : "install IDX-PKI's Squid ".
 			"on this machine"},
 	"default"   => 1,
 },

=cut "

sub install_all_implies {
	my ($question) = @_;
	return {
		"default"   => 1,
		"mandatory" => 1,
		"question"  => sub {valueof("install-all") ? undef : $question}
	};
}

=pod "

=item I<test_enables ($question)>

Functional form that computes the whole hashref record for one option (see
above). The meaning of a test-implied option is that this option should be
set to false if I<test> is false (and asked to the user if not).

For example:

 "test-privs" => test_enables
 	("unprivileged installation"),

is the same as:

 "test-privs" => {
 	"mandatory" => sub {valueof("test") ? 1 : undef},
 	"question"  => sub {valueof("test") ?
 		"unprivileged installation" : undef},
 	"default"   => sub {valueof("test") ? 1 : undef},
 },

=cut "

sub test_enables {
	my ($question) = @_;
	return {
		"default"   => sub {valueof("test") ? 1 : undef},
		"mandatory" => sub {valueof("test") ? 1 : undef},
		"question"  => sub {valueof("test") ? $question : undef},
	};
}

=pod "

=item I<if_installed ($whatinstalled, %keyvalues)>

Functional form that computes the whole hashref record for one option (see
above). If C<valueof ($whatinstalled)> is false, all parameters indicated
in I<%keyvalues> (except I<type>) are set to C<undef>, otherwise keep the
given values. The purpose of this function is to ask questions only if
some modules are to be installed on this host.

 "port-https" => if_installed ("install-webserver",
 	"type"     => "int",
 	"question" => "port for the HTTPS server",
 	"default"  => 443,
 ),

really means:

 "port-https" => {
 	"type"     => "int",
 	"question" => sub {valueof ("install-webserver") ?
 		"port for the HTTPS server" : undef},
 	"default"  => sub {valueof ("install-webserver") ?
 		443 : undef},
 },

=cut "

sub if_installed {
	my ($whatinstalled, %keyvalues) = @_;

	my %kcopy;

	while (my ($k, $v) = each %keyvalues) {
		$kcopy{$k} = ($k eq "type") ? $v :
			sub {return valueof ($whatinstalled) ? $v : undef;};
	}

	return \%kcopy;
}

=pod "

=item I<credentials_if_installed ($whatinstalled, %keyvalues)>

Same as I<if_installed()>, except that the values are also disabled if
I<test-privs> is set. The purpose of this function is to wrap around
options asking for user or group credentials.

=cut "

sub credentials_if_installed {
	my ($whatinstalled, %keyvalues) = @_;

	my %kcopy;

	while (my ($k,$v) = each %keyvalues) {
		$kcopy{$k} = ($k eq "type") ? $v :
			sub {return (valueof ($whatinstalled) &&
						 ! valueof ("test-privs")) ? $v : undef;};

		if ($k eq "type") {
			$kcopy{$k} = $v;
        } else {
			$kcopy{$k} = sub {
				my $t1 = valueof("test-privs");
				my $t2 = valueof($whatinstalled);
				return ($t2 && ! $t1) ? $v : undef;
			};
		};
	};

	return \%kcopy;
}

=pod

=back

All I<recurs_*> functions are mutually recursive and avoid loops by
keeping track of which variable(s) were needed to reach a decision.

=over 4

=item I<recurs_compute_field_coderef ($comment, $value_or_code,
@keys_to_not_use)>

Computes the value of a field that may be a coderef computing valueof().
Returns the computed value.

I<$comment> is only used to display an message in case of errors.
I<$value_or_code> may be a plain scalar (in which case it is directly
returned) or a sub reference (in which case the returned code is
returned).

=cut
sub recurs_compute_field_coderef {
	my ($comment, $value_or_code, @keys_to_not_use) = @_;
	if (! ref $value_or_code) {
		return $value_or_code;
	} elsif (ref($value_or_code) eq "CODE") {
		local $valueof_active = 1;
		local %valueof_opts = ();

		&$value_or_code(); # Dry run to get dependencies

		foreach my $k (@keys_to_not_use) {
			croak "Loop dependency detected for computation of $comment ".
				"for variable $k\n" if ($valueof_opts{$k});
		}

		# Make all required variables spring into existence
		foreach my $subkey (sort like_opttemplates (keys %valueof_opts)) {
			recurs_compute_value($subkey, @keys_to_not_use);
		}

		# Real thing now
		return &$value_or_code();
	} else {
		croak("Incorrect type for argument 2");
	}
}

=pod

=item I<recurs_compute_value ($key, @keys_to_not_use)>

Computes a needed value and puts it in I<%opts>. Dies if computation
cannot take place and we are in batch mode. In interactive mode, always
ask the user to at least confirm stuff.

=cut
sub recurs_compute_value {
	my ($key, @keys_to_not_use) = @_;
	if (exists $opts{$key})
	{
		&debug ("$key: already known", $opts{$key});
		return;
	}

	# Value taken from the old config.status or default value
	&debug ("$key: previously configured value", $oldopts{$key});
	my $defval = ((! $opts{"batch"}) && exists ($oldopts{$key})) ?
		$oldopts{$key} :
		recurs_compute_field_coderef
		("default value", $opttemplates{$key}->{default},
		 @keys_to_not_use, $key);

	my $mandatory = ((exists $opttemplates{$key}->{mandatory}) ?
					 recurs_compute_field_coderef
					 ("mandatoryness", $opttemplates{$key}->{mandatory},
					  @keys_to_not_use, $key) :
					 0); # By default nothing is mandatory.

	my $question = recurs_compute_field_coderef
		("interactive question", $opttemplates{$key}->{question},
		 @keys_to_not_use, $key);

	if ($opts{batch} || (! $question)) {
		# Do not ask, use default
		croak "Cannot compute a value for mandatory option $key\n"
			if ($mandatory && (!defined $defval));
		$opts{$key} = $defval;
		&debug ("$key: batch or no question", $opts{$key});
	} else {
		# Ask question to user
		do {
			debug("$key: proposed value", $defval);
			$opts{$key} = &ask_question ($defval, $question, typeofkey($key));
		} until ((! $mandatory) || (defined $opts{$key}));
		&debug ("$key: interactive", $opts{$key});
	}
}

#####################################################################
#####################################################################
#####################################################################
#####################################################################
#####################################################################

=pod "

=back

=head3 General Functions

=over 4

=item I<runConfig ()>

Runs I<runOpts ()> and I<runLocate ()> at a time.

=cut

sub runConfig {
	runOpts ();
	return runLocate ();
}

=pod

=item I<runOpts ()>

Performs the command line and old configuration file parsing, asks
questions to the user if appropriate and checks dependencies.

=cut "

sub runOpts {

	# The hash of all options.
	our %opttemplates = @opttemplates;
	&debug ("Default opts", \%opttemplates);

	croak "Two or more options have the same name"
		unless (2 * (scalar keys %opttemplates) == (scalar @opttemplates));

	# GetOptions gets to check the type of the options (boolean, integer
	# or string).
	do {
		my @typeopts;
		my %type2getopt = ("string" => "=s", "int" => "=i", "bool" => "!");
		while (my ($key, $val) = each (%opttemplates))
		{
			push (@typeopts, $key . $type2getopt{typeofkey($key)});
		}

		GetOptions (\%opts, "batch", @typeopts) ||
			pod2usage ('-verbose' => 1, '-exitval' => 1);
		&debug ("Getopt", \%opts);
	};


	unless ($opts{batch}) {
		&read_old_config_status (\%oldopts, $configstatusfile);
		&debug ("Previous config", \%oldopts);
	}

	# Now if something is missing and undef, the user must tell us or we
	# must die.

	###################### COMPUTING MISSING OPTIONS #####################

	do {
		my @opttemplates_copy = @opttemplates;
		while (my ($k, $v) = splice (@opttemplates_copy, 0, 2)) {
			recurs_compute_value($k);
		}
		&debug ("With computed values", \%opts);
		@opttemplates_copy = @opttemplates;
		while (my ($k, $v) = splice (@opttemplates_copy, 0, 2)) {
			delete ($opts{$k}) if ($v->{temporary});
		}

		&print_message ("Done\n");
	};

	&debug ("Final values", \%opts);
	&print_message ("Done\n");

	return %opts;
}

=pod

=item I<runLocate ()>

Looks for directories and binaries, according to the contents of
I<@loctemplates>.

=cut

sub runLocate {

	####################  PLATFORM AUTO-DETECTION  ####################

	&debug ("#### PLATFORM AUTO-DETECTION ####");

	my %loctemplates = @loctemplates;

	&print_message ("\nLooking for file paths:\n-----------------------\n");
	LOCATE: foreach my $key (keys %loctemplates)
	{
		&debug ("Considered option", $key, $opts{$key});
		(my $label = $key) =~ s/^with-//;

		# If we already have a supplied value for this one, verify it,
		# otherwise search for the right location.
		if (defined ($opts{$key}) && ($opts{$key} ne ""))
		{
			croak "Error: The supplied value for $key does not exist:\n".
				"'$opts{$key}'." if (! -e $opts{$key});
			&print_message ("* $label... $opts{$key}\n");
			next;
		}
		elsif ($loctemplates{$key}{'executable'})	# Locate the file.
		{
			require DB;
			$DB::single = 1;
			&print_message ("* $label... ");
			my @multi = ref $loctemplates{$key}{'executable'} ?
				@{$loctemplates{$key}{'executable'}} :
				($loctemplates{$key}{'executable'});

			foreach my $filename (@multi)
			{
				my $result = &which ($filename);
				if (defined $result)
				{
					# First found, first chosen.
					$opts{$key} = $result;
					&print_message ("$result\n");
					next LOCATE;
				}
			}
			&print_message ("not found\n\nCouldn't find a suitable path for ".
							"$label.\n\n");
			&write_new_config_status (\%opts, $configstatusfile);
			exit 1;
		}
		elsif ($loctemplates{$key}{'directory'})	# Locate the directory.
		{
			&print_message ("* $label... ");
			my @multi = ref $loctemplates{$key}{'directory'} ?
				@{$loctemplates{$key}{'directory'}} :
				($loctemplates{$key}{'directory'});

			foreach my $filename (@multi)
			{
				$filename = &whichdir ($filename) unless
					($filename =~ m|^/| && -d $filename);
				if (defined ($filename) && -d $filename)
				{
					# First found, first chosen.
					$opts{$key} = $filename;
					&print_message ("$filename\n");
					next LOCATE;
				}
			}
			&print_message ("not found\n\nCouldn't find a suitable path for ".
							"$label.\n\n");
			&write_new_config_status (\%opts, $configstatusfile);
			exit 1;
		}

		&debug ("Choosen value", $key, $opts{$key});
	}

	&debug ("Directories and paths values", \%opts);
	&print_message ("Done\n");

	return %opts;
}

=pod

=item I<runVersions()>

Checks for software versions and for perl modules and writes the new
configuration file.

=cut

sub runVersions {
	####################  CHECKING VERSIONS  ####################

	&debug ("#### CHECKING VERSIONS ####");

	&print_message ("\nChecking versions:\n------------------\n");

	my %versions;

	######### Horribly ad-hoc stuff for some well-known software

	# Perl all versions
	$versions{'perl'} = $];

	# Suidperl
	$versions{'suidperl'} = `suidperl -v`;
	$versions{'suidperl'} =~
		s/.*This is perl, v(\d\.\d\.\d).*/$1/is;

	# Slapd & slapadd 2.0.x
	if ($opts{'with-slapd'}) {
		$versions{'slapd'} = `$opts{'with-slapd'} -d 1 2>&1 | head -1`;
		$versions{'slapd'} =~
			s/^.*slapd.*?([0-9][0-9a-z]*(\.[0-9a-z]+)*).*/$1/is;
	}

	# Apache 1.3.x
	if ($opts{'with-apache'}) {
		$versions{'apache'} = `$opts{'with-apache'} -v`;
		$versions{'apache'} =~
			s/^.*?version.*?([0-9][0-9a-z]*(\.[0-9a-z]+)*).*/$1/is;
	}

	# Openssl 0.9.6*
	if ($opts{'with-openssl'}) {
		$versions{'openssl'} = `$opts{'with-openssl'} version`;
		$versions{'openssl'} =~ s/^.*?([0-9](\.[0-9a-z]+)*).*/$1/is;
	}

	while (my ($name, $version) = each (%needed_software)) {
		&compare_versions ($name, $versions{$name}, $version);
	}

	# Perl modules
	&print_message ("\nPerl modules:\n-------------\n");

	open (SAVEDERR, ">&STDERR");
	(undef) = fileno(SAVEDERR); #Shuts warning
	open (STDERR, '>/dev/null');

	foreach my $package (sort (keys (%needed_perl_modules)))
	{
		# Will be undef if the package isn't installed.
		$versions{$package} = eval "use $package; ".
			"\$${package}::VERSION || 'unknown';";
		&compare_versions ($package, $versions{$package},
						   $needed_perl_modules{$package});
	}
	open (STDERR, ">&SAVEDERR");

	&print_message ("Done\n");


	####################  UPDATE CONFIG.STATUS  ####################

	&write_new_config_status (\%opts, $configstatusfile);
	my $end_message = <<EOM;


The configuration values have been written in $configstatusfile.
You can now build IDX-PKI by typing `make`.

EOM
	&print_message ($end_message);

}

####################  SUPPORT FUNCTIONS  ####################

=pod "

=item I<hostname()>

Returns the local hostname, used as a default in many situations.

=item I<username()>

Returns the username of the user running the configure script, used
as a default in many situations.

=cut "

sub hostname {
	chomp (my $hostname = `hostname -f`);
	return $hostname;
}

sub username {return getpwuid ($>);}

=pod "

=item I<read_old_config_status ($configopts, $file)>

Reads the old configuration file F<$file> and puts the values into the
I<$configopts> (which shall be a reference to a hash).

=cut "
sub read_old_config_status
{
	my ($configopts, $config_status) = @_;
	croak "Internal error in read_old_config_status: \$opts is not a hash ref."
		unless (ref ($configopts) eq "HASH");
	if (-f $config_status)
	{
		print_message (<< 'MESSAGE');

Reconfiguration starting from the previous answers given to "./configure.pl".
To suppress this behaviour, remove the config.status file.

MESSAGE
	}
	else
	{
		return 1;
	}

	open (CONFIG_STATUS, $config_status) || croak "Can't open $config_status.";
	while (my $line = <CONFIG_STATUS>)
	{
		next if ($line =~ m/^\s*$/);	# Empty lines
		if ($line =~ m/^\#\s*CONFIG_(.*) is not set$/)	# ... is not set
		{
			my $option = lc ($1);
			$option =~ s/_/-/g;
			$configopts->{$option} = "";
			next;
		}
		next if ($line =~ m/^\#/);	# Comments
		my ($option, $value) = ($line =~ m/^\s*CONFIG_(.*?)\s*=\s*(.*)\s*$/);
		$option = lc ($option);
		$option =~ s/_/-/g;
		$configopts->{$option} = $value;
	}
	close (CONFIG_STATUS);
	return (wantarray ? %$configopts : $configopts);
}


=pod "

=item I<write_new_config_status ($opts, $file)>

Write the chosen configuration values into the file F<$file>.  If already
present, renames it into F<$file.old>.

=cut "
sub write_new_config_status
{
	my ($opts, $config_status) = @_;
	croak "Internal error in read_old_config_status: \$opts is not a hash ref."
		unless (ref ($opts) eq "HASH");
	if (-f $config_status)
	{
		rename ($config_status, $config_status.".old");
	}

	open (CONFIG_STATUS, ">$config_status") ||
		croak "Can't open $config_status for writing.";
	print CONFIG_STATUS <<ENDCOMMENT;
# Automatically generated by $0.  Some values are calculated
# from others; don't modify this unless you know what you are doing.

ENDCOMMENT
	foreach my $key (sort (keys (%$opts)))
	{
		my $val = $opts->{$key};
		$key =~ s/-/_/g;
		$key = "CONFIG_".uc($key);
		my $line = ((defined $val && $val ne "" && $val ne "0") ?
			"$key=$val\n" : "# $key is not set\n");
		print CONFIG_STATUS $line;
	}
	close (CONFIG_STATUS);
	return 1;
}


=pod "

=item I<which ($filename)>

Looks for the executable I<$filename> in the path (extended with the
standard F<sbin> directories).  If not found, then calls F<locate>.

=cut "
sub which ($)
{
	my ($filename) = @_;
	my $result;

	if ($filename =~ m|^/|)
	{
		return (-f $filename && -x _) ? $filename : undef;
	}

	local %ENV = ("PATH" => "/usr/local/sbin:/usr/sbin:/sbin:$ENV{PATH}");
	$result = `which $filename 2>/dev/null`;
	if (! $?)
	{
		chomp ($result);
		return $result;
	}

	my @results = `locate $filename 2>/dev/null`;
	if (! $?)
	{
		foreach my $attempt (@results)
		{
			chomp ($attempt);
			next unless ($attempt =~ m|/$filename$|);
			return $attempt if (-f $attempt && -x _);
		}
	}

	return undef;
}


=pod "

=item I<whichdir ($dirname)>

Looks for the directory I<$dirname> by calling B<locate(1)>.

=cut "
sub whichdir ($)
{
	my ($dirname) = @_;
	my $result;

	if ($dirname =~ m|^/|)
	{
		return (-d $dirname) ? $dirname : undef;
	}

	my @results = `locate $dirname 2>/dev/null`;
	if (! $?)
	{
		foreach my $attempt (@results)
		{
			chomp ($attempt);
			next unless ($attempt =~ m|/$dirname$|);
			return $attempt if (-d $attempt);
		}
	}

	return undef;
}


=pod "

=item I<compare ($current, $needed)>

Compare two versions of a software.  Returns true if the current version
is recognised as newer or equal than the needed version, false otherwise.

=cut "
sub compare
{
	my ($current, $needed) = @_;
	$current = "" if (! defined $current);
	$needed  = "" if (! defined $needed);

	# Same version.  Easy.
	return 1 if ($current eq $needed);

	# Numerical comparison.
	return ($current >= $needed) if ($current !~ /\D/ && $needed !~ /\D/);

	(my $curr_first = $current) =~ s/\..*//;
	(my $need_first = $needed)  =~ s/\..*//;
	(my $curr_last  = $current) =~ s/.*?\.//;
	(my $need_last  = $needed)  =~ s/.*?\.//;

	# If the first components are equals, compares the rest.
	return &compare ($curr_last, $need_last) if ($curr_first eq $need_first);

	# Numerical or alphanumerical comparison.
	return (($curr_first !~ /\D/ && $need_first !~ /\D/) ?
		($curr_first >= $need_first) : ($curr_first ge $need_first));
}


=pod "

=item I<compare_versions ($name, $current, $needed)>

Compares the current and needed versions of the software I<$name> and
prints a message accordingly. Exits immediately upon mismatch.

=cut "

sub compare_versions
{
	my ($soft, $curr, $need) = @_;

	&print_message ("* $soft (".($need ? $need :"unknown").")... ");

	if (! defined $curr)
	{
		&print_message ("==> not OK <==\n\n\u$soft doesn't appear to be ".
			"installed on the system.\n\n");
		&write_new_config_status (\%opts, $configstatusfile);
		exit 1;
	}

	if (! &compare ($curr, $need))
	{
		&print_message ("==> not OK <==\n\n\u$soft version at least $need ".
			"is needed (you have $curr).\n\n");
		&write_new_config_status (\%opts, $configstatusfile);
		exit 1;
	}
	else
	{
		&print_message ("$curr OK\n");
	}
}

=pod "

=item I<like_opttemplates ()>

Compare global variables I<$a> and I<$b> (results just as the C<< <=> >>
operator) w.r.t. their position in I<@opttemplates>. The expected usage is
to sort a list of keys so that they are in the same order as this list,
hence the idiom:

 my @sorted = sort like_opttemplates (@listofkeys);

The purpose of such an idiom is to ask questions in the same order as said
I<@opttemplates>, when possible (e.g. when computing dependencies don't
get in the way).

=cut "

sub like_opttemplates {
	return 0 if (!defined $a && !defined $b);
	return 1 if (defined $a); return -1 if (!defined $b);
	return 0 if ($a eq $b);
	foreach my $item (@opttemplates) {
		return -1 if ($a eq $_);
		return 1 if ($b eq $_);
	};
	return 0; # Unknown elements get stacked at the end of the list.
}

=pod "

=item I<typeofkey($keyname)>

Returns the data type expected for I<$keyname> (C<bool>, C<string> or
C<int>).

=cut "

sub typeofkey {
	my ($keyname) = @_;

	my $keytype = $opttemplates{$keyname}->{"type"};

	if (! defined $keytype) {
		return (($keyname =~ m/^with-|use-|test-|install-/) ? "bool" :
				"string");
	} elsif ($keytype =~ m/bool/) {
		return "bool";
	} elsif ($keytype =~ m/int/) {
		return "int";
	} elsif ($keytype =~ m/str/) {
		return "string";
	} else {
		croak "Unknown data type $keytype";
	}
}

=pod "

=item I<ask_question ($default, $question, $type)>

Asks the question I<$question> with default value I<$default>, according
to its type I<$type> (see L<ADDING NEW CONFIGURATION OPTIONS>), with
default value I<$default>. C<undef> is a valid value both for default and
for answer (the string composed of two double quotes translates into the
C<undef> value when input, whereas the empty string translates to the
default).

=cut "

sub ask_question
{
	my ($default, $question, $type) = @_;
	my $precision = "";

	if ($type =~ m/bool/)		# Yes/no questions
	{
		$default = ($default ? "y" : "n");
		$precision = "(y/n) ";
	}

	printf("\u$question $precision\[%s\]?\n",($default or ""));
	chomp (my $val = <STDIN>);
	if ($val eq "") {
		$val = $default;
	} elsif ($val eq '""') {
		$val = "";
	}

	if ($type =~ m/bool/) {
		$val = &ask_question (@_)
			unless ($val && ($val eq "y" || $val eq "n"));
		$val = ($val eq "y" || $val eq "1"); # 1 or ""
	}

	&debug ("Answered value", $val);
	return $val;
}


=pod "

=item I<print_message ($message)>

Prints the message I<$message> on the standard output.

=cut "

sub print_message
{
	my ($message) = @_;
	print $message;
	return 1;
}


=pod "

=item I<debug ($title, @args)>

Recursively shows the content of @args if the global $DEBUG variable is
set.

=cut "

sub debug
{
	return unless ($DEBUG);
	my ($title, @args) = @_;
	croak "Internal error in debug: Wrong arguments" unless (defined $title);
	print STDERR "\n#===#  $title  #===#\n";
	use Data::Dumper;
	print STDERR Data::Dumper::Dumper (@args) . "\n";
}

=pod "

=item I<valueof ($option)>

Ensures that option I<$option> will be computed before options which use
I<valueof ($option)>, and returns the value of I<$option>.

See L</DESCRIPTION/On the Good Use of valueof()>.

=cut "

sub valueof {
	croak "You cannot call valueof() except whithin an anonymous ".
		"subroutine" unless ($valueof_active);

	my ($key) = @_;
	croak "valueof(undef) called" if (!defined $key);

	$valueof_opts{$key}++;
	return $opts{$key};
}


=pod "

=back

=head1 BUGS / TODO

=over 4

=item *

"int"-ness is not checked for user inputs.

=item *

You can't specify subs for parameters when you use functions defined in
L</REFERENCE/Functions/Hash Calculation Functions>; you have to give
values.

=item *

An extensible framework for developper-specified data types should be
provided (using a sub reference as the value of the I<type> field).

=item *

Common code to several IDEALX projects should be modularized.

=item *

When choosing the computed value, there should be an exception if the
corresponding default value uses I<valueof()>: Then the recomputed value
should be used rather than a previous configured one from
F<config.status>.

=cut "

unless (caller()) {
	require FindBin;
	$FindBin::Bin = $FindBin::Bin; # Avoids a "used only once" warning.
	my $perllib = "$FindBin::Bin/../src/perllib";
	$perllib =~ m|^([a-z0-9/\-_.]*)$|i ||
		croak "Strange path $perllib when trying to find IDX::Test";
	croak "Can't find IDX::Test module in $1" unless (-f "$1/IDX/Test.pm");
	unshift (@INC, $1);
	require IDX::Test::Suite;
}

1;
__END__

use strict;

use BuildTimeConfig;

$configstatusfile = new IDX::Test::Filename ();

{
	no warnings;
	# Avoids verbose messages.
	sub BuildTimeConfig::print_message {1;};
}

sub cleanvars
{
	# Avoids environment pollution, as BuildTimeConfig uses global
	# variables everywhere.
	@BuildTimeConfig::opttemplates = ();
	%BuildTimeConfig::opttemplates = ();
	@BuildTimeConfig::loctemplates = ();
	%BuildTimeConfig::needed_perl_modules = ();
	%BuildTimeConfig::needed_software = ();
    %BuildTimeConfig::opts = ();
    %BuildTimeConfig::oldopts = ();
    %BuildTimeConfig::valueof_opts = ();
	$BuildTimeConfig::valueof_active = undef;
}

test "Empty config.status" => sub {
	cleanvars ();
	runConfig ();
	runVersions ();
	TEST (-f $configstatusfile && -s _) || warn "No config.status or ".
		"empty size (should contain at least comments";
	# Should fail.
	TEST (system ("grep", "-q", "^CONFIG", $configstatusfile)) ||
		warn "There are some CONFIG lines in config.status: ".
		`cat $configstatusfile`;
};

test "Dependancies (default values)" => sub {
	unlink $configstatusfile || die "Can't unlink $configstatusfile: $!";
	cleanvars ();
	@opttemplates =
		(
		 "activates"  =>
		 {
			 "type"    => "bool",
			 "default" => 0,
		 },

		 "value" =>
		 {
			 "default"   => sub {
				 valueof ("activates") ? "isactivated" : "isnotactivated";
			 },
		 },
		 );
	runConfig ();
	runVersions ();
	TEST (! system ("grep", "-q", "^CONFIG_VALUE", $configstatusfile)) ||
		warn "There are no CONFIG_VALUE lines in config.status: ".
		`cat $configstatusfile`;
};

test "Dependancies (pseudo-command-line values)" => sub {
	unlink $configstatusfile || die "Can't unlink $configstatusfile: $!";
	cleanvars ();
	@opttemplates =
		(
		 "activates"  =>
		 {
			 "type"    => "bool",
			 "default" => 0,
		 },

		 "value" =>
		 {
			 "default"   => sub {
				 valueof ("activates") ? "isactivated" : "isnotactivated";
			 },
		 },
		 );
	# Emulates command-line options.
	%BuildTimeConfig::opts = ("activates" => 1);
	runConfig ();
	runVersions ();
	my @lines = split (/\n/, `grep "^CONFIG_VALUE" "$configstatusfile"`);
	TEST (@lines == 1) || warn "More than one matching line: ".
		join ("", @lines);
	TEST ($lines[0] eq "CONFIG_VALUE=isactivated") ||
		warn "Didn't take command-line value: $lines[0]";
};

test "Dependancies (pseudo-command-line values overrides defaut)" => sub {
	unlink $configstatusfile || die "Can't unlink $configstatusfile: $!";
	cleanvars ();
	@opttemplates =
		(
		 "activates"  =>
		 {
			 "type"    => "bool",
			 "default" => 0,
		 },

		 "value" =>
		 {
			 "default"   => sub {
				 valueof ("activates") ? "isactivated" : "isnotactivated";
			 },
		 },
		 );
	# Emulates command-line options.
	%BuildTimeConfig::opts = ("value" => "surprise");
	runConfig ();
	runVersions ();
	my @lines = split (/\n/, `grep "^CONFIG_VALUE" "$configstatusfile"`);
	TEST (@lines == 1) || warn "More than one matching line: ".
		join ("", @lines);
	TEST ($lines[0] eq "CONFIG_VALUE=surprise") ||
		warn "Didn't take command-line value: $lines[0]";
};

test "Dependancies (previously configured values overrides defaut)" => sub {
	cleanvars ();
	@opttemplates =
		(
		 "activates"  =>
		 {
			 "type"    => "bool",
			 "default" => 0,
		 },

		 "value" =>
		 {
			 "default"   => sub {
				 valueof ("activates") ? "isactivated" : "isnotactivated";
			 },
		 },
		 );
	runConfig ();
	runVersions ();
	TEST (! system ("diff", "$configstatusfile.old", $configstatusfile)) ||
		warn "The two configuration files differ";
};

1;
