#!/usr/bin/perl -w
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

=pod "

=head1 NAME

InstallLib.pm - Perl library for IDX-PKI's installation scripts (in Perl, of course)

=head1 SYNOPSIS

It is possible to C<use InstallLib;> but B<InstallLib> will typically not
be in your I<@INC> variable.  So insert the following boilerplate in your
script instead (use the right number of C<..>'s according to its position
in the source tree):

 use FindBin;
 use vars qw ($topdir);
 $topdir = "$FindBin::RealBin/../../";
 push (@INC, "$topdir/tools");
 require InstallLib;
 import InstallLib ();

As a side effect, I<$InstallLib::topdir> will be set to the absolute path
of IDX-PKI's root directory.  You can then use the functions described
below to perform the required installation tasks.

=cut "

package InstallLib;

use strict;
use locale;

use vars qw (@ISA @EXPORT);

require Exporter;
@ISA = qw (Exporter);
@EXPORT = qw (&makeSysVScript &parseConfigStatus &getConfig &installcron
	&locateexe &locatedir);

use vars qw($topdir);

=pod "

=head1 VARIABLES

I<$topdir> is the top-level directory of IDX-PKI's source tree.

=head1 FUNCTIONS

=over 4

=item I<getConfig()>

 my %config = getConfig(\@argv);

Fetches the configuration elements that will be useful to any install
script, both from F<$InstallLib::topdir/config.status> and from I<@argv>.
Option C<--ConfigStatus> (with this capitalization) asks for
interpretation of the F<config.status> I<at this point>.  All other
command line switches in I<@argv> must be in long format, and expect a
string parameter which is affected to this switch in I<%config>, but
uppercased and with dashes converted into underscores
(e.g. C<--alt-directory=3> gives C<< ALT_DIRECTORY => 3 >> in I<%config>).
I<@argv> is modified in-place; after execution of I<getConfig()>, it
contains all non-options contained in I<@argv>.

If an error is encountered during parsing of I<@argv>, I<getConfig()>
returns the empty list, so that the customary idiom is permitted:

 do {%config = getConfig(\@argv);} || die;

=cut "

sub getConfig {
	# No, we can't use Getopt::Long (it doesn't support our situation, where
	# we don't know which arguments to expect beforehand).
	my ($argvref)=@_;
	my @saveargv;

	my %conf;

	ARGV: while(@$argvref) {
		my ($switch,$val);
		CASE: for ($argvref->[0]) {
			# This is a "case" statement, and $_ is the matched variable.
			(m/^--ConfigStatus$/) && do {
				shift @$argvref;
				my %conffromfile=parseConfigStatus($topdir);
				while(my ($k,$v)=each %conffromfile) { $conf{$k}=$v; };
				last CASE;
			};
			(m/^--help$/) && do {
				return ();
			};
			(m/--([^=]*)=(.*)$/) && do {
				($switch,$val)=($1,$2); shift @$argvref;
				last CASE;
			};
			(m/^--(.*)$/) && do {
				$switch=$1;shift @$argvref;
				$val=shift @$argvref; if ($val =~ m/^--/) {
					warn "Incorrect syntax in script arguments\n";
					return ();
				};
				last CASE;
			};
			do { # Default case
				push @saveargv,(shift @$argvref);
				last CASE;
			};
		};
		next ARGV if (!defined $switch) || (!defined $val);
		$switch = uc($switch);
		$switch =~ tr/-/_/;
		$conf{"CONFIG_".$switch} = $val;
	};

	@$argvref=@saveargv;
	return %conf;
}


=pod "

=item I<parseConfigStatus()>

 my %config = parseConfigStatus($dir);

Parses the F<config.status> file in F<$dir>, and returns it as an
associative array. If F<$dir> is omitted, use F<$InstallLib::topdir>
instead.

=cut "

sub parseConfigStatus {
	my ($dir) = @_;

	$dir ||= $topdir;

	local *CONFIGSTATUS;

	my %CONFIG;

	# It's not an error to have no config.status at this point.
	open(CONFIGSTATUS,"<$dir/config.status") || return ();

	while(<CONFIGSTATUS>) {
		if (m/^\#\s*(CONFIG_.*) is not set$/) {
			my $key = $1;
			$CONFIG{$key} = "";
			next;
		}

		next if (m/^\#/ || m/^\s*$/);

		my ($key,$val) = m/^\s*(CONFIG_[^=]*)=(.*)$/;
		$CONFIG{$key} = $val;
	};

	close(CONFIGSTATUS);

	%CONFIG;
}


=pod "

=item I<< installcron (C<arg1> => value1, ...) >>

Installs a crontab entry.  Available arguments are:

=over 4

=item C<command>

Command to launch in the crontab entry.

=item C<freq>

Frequency at which to launch the command (see L<crontab(5)> for frequency
syntax).

=item C<label>

Label to include in the descriptive comment, if present.

=item C<user>

User for whom the crontab has to be installed (current user if not
specified).

=item C<remove>

If true, removes the line from the crontrab file instead.

=back

=cut "

sub installcron {
	my %args = @_;
	my $command = $args{'command'} || die "Missing 'command' in installcron.";
	my $freq    = $args{'freq'}    || die "Missing 'freq' in installcron.";
	my $label   = defined ($args{'label'}) ? $args{'label'} : "";
	my $user    = $args{'user'};
	my $remove  = $args{'remove'};

	open (OLDCRONTAB, "-|", "crontab".($user ? " -u $user" : "")." -l") ||
		die "Couldn't get the old crontab.";
	my @crontab = <OLDCRONTAB>;
	close (OLDCRONTAB);
	# RedHat specific: Remove cruft at top.
	if ($crontab[0] && $crontab[1] && $crontab[2] &&
		$crontab[0] =~ m/^\# DO NOT EDIT THIS FILE/ &&
		$crontab[1] =~ m/^\# / && $crontab[2] =~ m/^\# /) {
		splice (@crontab, 0, 3);
	}
	my $crontab = join ("", @crontab);

	# Cosmetic for alignment!
	my $tab;
	if ($freq =~ m/^\@[a-z]*$/) {
		# '@blah' format.
		$tab = "\t\t";
	} elsif ($freq =~ m/^[^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+$/) {
		# '* * * * *' format.
		$tab = "\t";
	} else {
		die "Invalid crontab frequence value: '$freq'\n";
	}
	my $comment     = "# Automatically installed by IDX-PKI";
	my $commenttest = quotemeta ($comment);
	my $freqtest    = quotemeta ($freq);
	my $cmdtest     = quotemeta ($command);
	my $freqregexp  = '(@[a-z]+)|([^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+\s+[^\s]+)';

	if (! $remove) {		# Add line.
		if ($crontab =~ m/^($freqregexp\s+$cmdtest)\s*$/m) {
			# Already present, maybe with a different frequency.
			return 1;
		} else {
			$crontab .= $comment.($label ? ": \u$label" : "").".\n".
				"$freq$tab$command\n";
		}
	} else {				# Remove line.
		if ($crontab !~ m/^$freqregexp\s*$cmdtest\s*$/m) {
			# Not present, even with a different frequency.
			return 1;
		} else {
			$crontab =~  s/(^|\n)
				($commenttest[^\n]*\n)?
				$freqtest\s*$cmdtest\s*\n?/$1/sx;
		}
	}

	open (NEWCRONTAB, "|-", "crontab".($user ? " -u $user" : "")." -") ||
		die "Couldn't set the new crontab.";
	print NEWCRONTAB $crontab;
	close (NEWCRONTAB);

	return 1;
}


=pod "

=item I<locateexe ($first_guess, $second_guess, ...)> or I<locateexe
([$first_guess, $second_guess, ...])>

Locates an executable: Looks for the executable I<$filename> in the path
(extended with the standard F<sbin> directories).  If not found, then
calls F<locate>.

=cut "

sub locateexe ($) {
	my @guesses = @_;
	@guesses = @{$guesses[0]} if (@guesses == 1 &&
		ref ($guesses[0]) eq "ARRAY");

	my $result;

	local $ENV{'PATH'} = "/usr/local/sbin:/usr/sbin:/sbin:$ENV{PATH}";
	GUESS: foreach my $filename (@guesses) {
		if ($filename =~ m|^/| && -f $filename && -x _) {
			# If they just exist as is...
			$result = $filename;
			last GUESS;
		} else {
			my $attempt = `which $filename 2>/dev/null`;
			if (! $?) {
				# ... or they are in the PATH...
				chomp ($result = $attempt);
				last GUESS;
			} else {
				# Otherwise, use locate.
				my @results = `locate $filename 2>/dev/null`;
				next GUESS if ($?);
				foreach my $attempt (@results) {
					chomp ($attempt);
					if ($attempt =~ m|/$filename$| && -f $attempt && -x _) {
						$result = $attempt;
						last GUESS;
					}
				}
			}
		}
	}

	return $result;
}


=pod "

=item I<locatedir ($first_guess, $second_guess, ...)> or I<locatedir
([$first_guess, $second_guess, ...])>

Locates a directory: Looks for the first found directory among supplied
guesses by calling B<locate(1)>, and returns it.

=cut "

sub locatedir ($) {
	my @guesses = @_;
	@guesses = @{$guesses[0]} if (@guesses == 1 &&
		ref ($guesses[0]) eq "ARRAY");

	my $result;

	GUESS: foreach my $dirname (@guesses) {
		if (($dirname =~ m|^/|) && (-d $dirname)) {
			# If they just exist as is...
			$result = $dirname;
			last GUESS;
		} else {
			# Otherwise, use locate.
			my @results = `locate $dirname 2>/dev/null`;
			next GUESS if ($?);
			foreach my $attempt (@results) {
				chomp ($attempt);
				if ($attempt =~ m|/$dirname$| && -d $attempt) {
					$result = $attempt;
					last GUESS;
				}
			}
		}
	}

	return $result;
}


############# Utility functions

=pod "

=back

=head2 Utility Functions

Expand a command to the point where it is useable standalone in a function
like I<start()> or I<stop()>.

=over 4

=item I<_shellEscape()>

 my $text=_shellEscape($progname,@args);

Takes a putative ARGV array, returns a string quoted in a way that allows
a Bourne shell to restitute exactly each and every argument to the called
program.  So that C<system(_shellEscape($prog,@args))> launches the very
same command line as C<system($prog,@args)> (well, except the father is
Perl instead of a shell, of course).

=cut "

sub _shellEscape {
	my @quoted;

	foreach my $element (@_) {
		#TODO "make install" complains about an uninitialized value in the
		#  following line:
		my @sqpieces = split m/\'/,$element,-1;

		push(@quoted,join("\\'",map {
			scalar(m|^[-[:alnum:]_,./@+]*$|) ?
				$_ :    # Make it more readable for the simple case
				qq{'$_'}
		} @sqpieces));
	};
	return join(" ",@quoted);
}


=pod "

=item I<command2shell()>

 my $command = command2shell ($ref_or_text, ['arg1' => val1,
 	'arg2' => val2, ...]);

The main magic in the script; does pretty much everything to insert real
commands into the SysV script which is pretty much boilerplate everywhere
else.

I<$ref_or_text> must be a shell command (properly quoted) or reference to
an array (unquoted text allowed but see L</BUGS>).  Just like I<system()>
in the Perl standard library, a passed string is subject to shell
expansion (so it will not be quoted in the output) whereas a reference on
an array means that the arguments of the program are correctly identified
(so it will be properly quoted to prevent the shell from messing it up).

Allowed arguments are:

=over 4

=item C<user>

The user to run the command as.  The B<su(1)> command will be used.

=item C<async>

If set, starts the job asynchronously (by appending an ampersand).

=back

=cut "

sub command2shell {
	my ($ref_or_text, %args) = @_;

	die "Missing or indefined parameter in command2shell."
		if (!defined $ref_or_text);

	# Towards quote level 1: Text command to be passed to a shell.
	my $cmd;
	if (ref $ref_or_text) {
		my @arglist = @$ref_or_text;
		$cmd = _shellEscape (@arglist);
	} else {
		$cmd = $ref_or_text;
		chomp $cmd;
	}

	# Towards quote level 2, if necessary: Text command to be passed to a
	# shell-in-the-shell.
	$cmd = "su ".$args{'user'}." -s /bin/sh -c "._shellEscape("exec $cmd")
		if ($args{'user'});

	$cmd .= " &" if ($args{'async'});

	return $cmd;
}


####################### The main thing ############################

=pod "

=back

=head2 The Main Thing

=over 4

=item I<makeSysVScript()>

 my $fulltext = makeSysVScript ('arg1' => value1, 'arg2' => value2, ...)

This function creates a SysV-style init script for a service.  The full
text of the script is returned as a string.  The following features are
provided:

=over 4

=item *

Fully configurable (see below);

=item *

Full RH7.1 compliance, including PID file and colorful OK/FAILED messages
:-);

=item *

Support for using with or without root privileges.

=back

Recognized named parameters are:

=over 4

=item C<name> I<(mandatory)>

The name of the service: This will be the filename of the init script and
the name under which it appears in B<initlog(8)>.

=item C<description>

A short description of the service.  Defaults to none.

=item C<comment>

The long comment to insert inside the text.  Will be properly
paragraph-filled, no need to worry about this.  Defaults to none.

=item C<init>

The command(s) in this option will be executed when the script starts up.
This is useful to set up variables, exit prematurely if some preconditions
are not met, etc.

=item C<start> I<(mandatory)>

The command to use to start the service.  It can be a list of arguments or
a string.

=item C<stop>

The command to use to stop the service.  Defaults to C<< kill -TERM `cat
$pidfile` >> but is mandatory if C<pidfile> is not supplied.

=item C<reload>

The command to use to reload the service configuration file.  Defaults to
restart.

=item C<check>

The command to check for up-ness of the service.  Substitutions occur
again.  Defaults to checking for the existence of the daemon via it's PID
file, or C<true> if there is no PID.

=item C<details>

A string that will be appended to the status line every time the script is
started or stopped (a handy example is C<on port [num]>).

=item C<user>

The user under which the daemon will run.  If it is C<undef>, no
privileges are set and the script is protected from being run by root,
otherwise only root will be able to run the script.  Default is C<undef>.

=item C<env>

Anonymous hash of the special environment variables to pass to the
program.  Defaults to none.  The script takes care of erasing all the
environment, except this one and a few standard variables (I<PATH>) before
launching the programs.

=item C<pidfile>

The filename containing the PID.

=item C<asyncstart>

If true, runs the start command asynchronously.  In this case, success of
the start phase cannot be tested.

=back

=cut "

sub makeSysVScript {
	my (%params) = @_;

	# Name.
	die "'name' parameter undefined in makeSysVScript.\n"
		if (!defined $params{'name'});
	# Description.
	$params{'description'} = "" if (! defined $params{'description'});
	# Comment.
	$params{'comment'} = defined ($params{'comment'}) ?
		"\u$params{'comment'}" : "";
	# Start.
	die "'start' parameter undefined in makeSysVScript.\n"
		if (!defined $params{'start'});
	# Stop.
	die "Neither 'stop' nor 'pidfile' parameters defined in ".
		"makeSysVScript.\n" if ((! defined $params{'stop'}) &&
		(! defined $params{'pidfile'}));
	$params{'stop'} = "kill -TERM `cat $params{'pidfile'} 2>/dev/null` ".
			"2>/dev/null && rm -f $params{'pidfile'}"
			if (! defined $params{'stop'});
	# Check.
	if (! defined $params{'check'}) {
		if ($params{'pidfile'}) {
			$params{'check'} = << "DEFAULT_CHECK_CMD";
local failed=0
	pid=`cat $params{'pidfile'} 2>/dev/null` || failed=1
	ps \$pid >/dev/null || failed=1
	[ "\$failed" = 0 ]
DEFAULT_CHECK_CMD
		} else {
			$params{'check'} = "true";
		}
	}
	# Details.
	$params{'details'} = "" if (! defined $params{'details'});
	$params{'details'} =~ s/^\s*/ /s if ($params{'details'} ne "");
	# Env.
	$params{'env'} = {} if (! defined $params{'env'});
	die "'env' parameter must be a hash reference in makeSysVScript.\n"
		if (ref ($params{'env'}) ne "HASH");

	# Antibozo: Command to enforce root/non-root rule wrt. privileges.
	my $antibozo = (defined ($params{'user'}) ?
		'[ `id -u` = 0 ] || { echo "This script must be run as root." >&2; '.
			'exit 1; }' :
		'[ `id -u` = 0 ] && { echo "Do NOT run this script as root!" >&2; '.
			'exit 1; }'
	);

	# Commands to check if the service is down or up.
	my ($precheckcmd, $postcheckcmd);
	if ($params{'check'} eq "true") {
		# Impossible to reliably use status() to test for up-ness.  We
		# could try and use the return value of the start command, but
		# this fails with su(1) - see BUGS.  We could work around this
		# with touch(1)ed files or some equally crude hack, but we don't
		# want to.
		$precheckcmd  = "false";
		$postcheckcmd = "true";
	} else {
		$precheckcmd  = "status >/dev/null 2>&1";
		$postcheckcmd = "status >/dev/null 2>&1";
	}


	my $fulltext = "";

	##### File header
	do {
		# Comments are word-wrapped. Perl is such a wonderful language for
		# this stuff !
		my @commentwords = split (m/\s+/, $params{"comment"});

		# Warning: "description" and "chkconfig" must be written
		# lower-case or chkconfig won't recognise them.
		my $descriptiontext = "";
		while (@commentwords) {
			my $word = shift (@commentwords);
			next if ($word eq "");
			my $descriptionline = $descriptiontext ? "\n#     $word" :
				"# description: \u$word";

			WORD: while (length ($descriptionline) +
				length ($commentwords[0]) + 1 < 75) {
				my $word = shift (@commentwords);
				next WORD if ($word eq "");
				$descriptionline .= " $word";
				last WORD if (! @commentwords);
			}
			$descriptiontext .= $descriptionline;
		}

		my $pidstring = $params{'pidfile'} ?
			"# pidfile:     $params{'pidfile'}\n" : "";

		$fulltext .= << "BOILERPLATE";
#!/bin/sh
#
# This code was developped by IDEALX (http://IDEALX.org/).
# 
# Copyright  2000-2003 IDEALX
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

# $params{'name'} -- $params{'description'}
#
# chkconfig:   2345 90 60
$pidstring$descriptiontext

# Install this file in /etc/init.d and run "chkconfig $params{'name'} on" and
# "chkconfig --add $params{'name'}".

BOILERPLATE

		if (-f "/etc/init.d/functions") {
			$fulltext .= << 'TOOLS';
# Source function library.
. /etc/init.d/functions

TOOLS
		} else {
			$fulltext .= << 'TOOLS';
# Imported functions from RedHat, to see the result with nice colors.
RES_COL=60
MOVE_TO_COL="echo -e -n \\033[${RES_COL}G"
SETCOLOR_SUCCESS="echo -e -n \\033[1;32m"
SETCOLOR_FAILURE="echo -e -n \\033[1;31m"
SETCOLOR_WARNING="echo -e -n \\033[1;33m"
SETCOLOR_NORMAL="echo -e -n \\033[0;39m"

success() {
	$MOVE_TO_COL
	echo -n "[  "
	$SETCOLOR_SUCCESS
	echo -n $"OK"
	$SETCOLOR_NORMAL
	echo -n "  ]"
	echo -n -e "\r"
	return 0
}

failure() {
	$MOVE_TO_COL
	echo -n "["
	$SETCOLOR_FAILURE
	echo -n $"FAILED"
	$SETCOLOR_NORMAL
	echo -n "]"
	echo -n -e "\r"
	return 1
}

passed() {
	$MOVE_TO_COL
	echo -n "["
	$SETCOLOR_WARNING
	echo -n $"PASSED"
	$SETCOLOR_NORMAL
	echo -n "]"
	echo -n -e "\r"
	return 1
}

TOOLS
#"#'
		}
	};

	##### Startup shell snippets
	if ($params{'init'}) { $fulltext.=$params{'init'}."\n\n";};

	##### Start command
	do {
		# Environment must be reset by the start script.
		my $cmd = 'env - PATH=$PATH ' . join (" ", map {
				"$_=" . '"' . $params{'env'}->{$_} . '"'
			} (keys %{$params{'env'}})) . " \\\n" . (" "x12);

		$params{'user'} = undef if ($params{'user'} &&
			$params{'user'} eq "root");

		$cmd .= command2shell ($params{'start'}, 'user' => $params{'user'},
			'async' => $params{'asyncstart'});

		$fulltext .= << "STARTCOMMAND";
start() {
	echo -n "Starting $params{'name'}$params{'details'}: "
	$precheckcmd && { failure; echo; return 1; }
	$cmd
	for i in 1 2 3 4 5 6 7 8; do
		$postcheckcmd && {
			success
			echo
			return 0
		}
		sleep 1
	done
	failure
	echo
	return 1
}

STARTCOMMAND
	};


	##### Stop command
	do {
		my $cmd = command2shell ($params{'stop'});
		$fulltext .= << "STOPCOMMAND";
stop() {
	local failed=0
	echo -n "Stopping $params{'name'}$params{'details'}: "
	$cmd
	failed=\$?
	[ "\$failed" = "0" ] && success || failure
	echo
	return \$failed
}

STOPCOMMAND
	};


	##### Status command
	do {
		my $cmd = command2shell ($params{'check'});
		$fulltext .= << "STATUSCOMMAND";
status() {
	local failed=0
	$cmd
	failed=\$?
	[ "\$failed" = 0 ] && \\
		echo "$params{name} is up" || \\
		echo "$params{name} is down"
	return \$failed
}

STATUSCOMMAND
	};


	##### Reload command
	do {
		if ($params{'reload'}) {
			my $cmd = command2shell ($params{'reload'});
			$fulltext .= << "RELOADCOMMAND";
reload() {
	local failed=0
	$cmd
	failed=\$?
	[ "\$failed" = 0 ] && \\
		echo "$params{name} is up" || \\
		echo "$params{name} is down"
	return \$failed
}

RELOADCOMMAND
		} else {
			$fulltext .= << "RELOADCOMMAND";
reload() {
	restart
}

RELOADCOMMAND
		}
	};


	##### Footer
	do {
		$fulltext .= << "FOOTER";
restart() {
	stop
	start
}

case "\$1" in
	start)
		$antibozo
		start
	;;
	stop)
		stop
	;;
	restart)
		$antibozo
		restart
	;;
	condrestart)
		# Only restart if it is already running.
		$antibozo
		$precheckcmd && restart
	;;
	reload)
		$antibozo
		reload
	;;
	status)
		status
	;;
	*)
		echo "Usage: $params{'name'} {start|stop|restart|condrestart|reload|status}"
		exit 1
	;;
esac

exit \$?
FOOTER
	};

	return $fulltext;
} # End of makeSysVScript

#######################

=pod "

=back

=head1 BUGS

Single quotes in arguments are still not passed along correctly when using
B<su>.  This should be fixed.

B<su(1)> doesn't pass error values, so commands launched with special
privileges will always appear to succeed if C<-check> doesn't do something
useful.

Scripts that do not provide a correct check method (the default used when
a PID file is in use is OK) are not correctly protected against double
starts -- i.e. the F</var/lock/subsys> mechanism is not implemented here.

=cut "

1;

################## Initialization block ###############################

do {
	my $whoami = sub {caller()};
	my (undef, $filename, undef) = &$whoami();
	$filename ||= $0;
	$filename =~ s|/[^/]*$||;
	$topdir = $filename."/..";
	$topdir = `cd $topdir && /bin/pwd`;
	chomp $topdir;
};

my $bogustoshutwarning = *main::DATA;
eval join('',<main::DATA>) || die $@ unless caller();
1;
__END__

############################# TEST SUITE ##############################

use IDX::Test;

number_of_tests (17);

my ($file, $testdir) = (new IDX::Test::Filename,
	new IDX::Test::Directory);

test 'Initialisation of $InstallLib::topdir' => sub {
	-f "$topdir/IDX-PKI-global.mk";
};

# This tests really captures a variety of situations.
test "getConfig and parseConfigStatus" => sub {
	local $topdir = $testdir."";
	open(CONFIG_STATUS,">$testdir/config.status") ||
		die "Can't create $testdir/config.status: $!";
	print CONFIG_STATUS <<CONFIG_STATUS_TEST;
CONFIG_THIS_VARIABLE=321
CONFIG_THAT_VARIABLE=aaabbbccc
CONFIG_ONLY_CONFIGFILE=hello
CONFIG_STATUS_TEST
	close CONFIG_STATUS;
	my @argv = (
		"--passthrough"   => "hi",
		"--that-VARiable" => "will be overridden",
		"bogus",
		"--ConfigStatus",
		"inside",
		"--this-variable" => "123",
		"--debug=1"
	);
	my %conf = getConfig(\@argv);
};

{
	my @crontab = `crontab -l`;
	# RedHat specific: Remove cruft at top.
	if ($crontab[0] && $crontab[1] && $crontab[2] &&
		$crontab[0] =~ m/^\# DO NOT EDIT THIS FILE/ &&
		$crontab[1] =~ m/^\# / && $crontab[2] =~ m/^\# /) {
		splice (@crontab, 0, 3);
	}
	my $origcrontab = join ("", @crontab);

	test "installcron" => sub {
		&installcron ('command' => "/some/command", 'freq' => "* * * * *");
		TEST (`crontab -l` =~ m|/some/command|s);
	};

	test "installcron cleaning" => sub {
		&installcron ('command' => "/some/command", 'freq' => "* * * * *",
					  'remove' => 1);
		my $newcrontab = `crontab -l`;
		(my $oldcrontab = $origcrontab) =~ s/^(\#.*|\s*)$ \n//gmx;
		$newcrontab =~ s/^(\#.*|\s*)$ \n//gmx;
		TEST ($newcrontab eq $oldcrontab);
	};

	test "installcron cleans only the appropriate line" => sub {
		&installcron ('command' => "/some/command", 'freq' => "* * * * *");
		&installcron ('command' => "/some/other/command",
			'freq' => "* * * * *");
		&installcron ('command' => "/yet/another/command",
			'freq' => "* * * * *");
		&installcron ('command' => "/some/other/command",
			'freq' => "* * * * *", 'remove' => 1);
		TEST (`crontab -l` =~ m|/some/command|s);
		TEST (`crontab -l` =~ m|/yet/another/command|s);
	};

	test "installcron idempotence" => sub {
		my $oldcrontab = `crontab -l`;
		&installcron ('command' => "/some/command", 'freq' => "* * * * *");
		&installcron ('command' => "/some/command",
			'freq' => "1-59/5 * * * *");
		&installcron ('command' => "/some/command",
			'freq' => "2-59/5 * * * *");
		TEST (`crontab -l | grep /some/command | wc -l` == 1);
	};

	open (CRONTAB, "|-", "crontab -");
	print CRONTAB $origcrontab;
	close (CRONTAB);
}	# End of scope for $origcrontab.

test "shellEscape" => sub {
	my @commandset = ("echo", "#!/bin/sh '", qq{"I'm there!"});
	my $cmd = _shellEscape (@commandset);
	local *HANDLE1;
	open (HANDLE1, "$cmd |");
	local $/ = undef;
	my $result = <HANDLE1>;
	close (HANDLE1);

	shift @commandset;
	TEST ($result eq (join (" ", @commandset)."\n"));
};

test "cpmmand2shell" => sub {
	my @commandset = ("echo", "#!/bin/sh '", qq{"I'm there!"});
	my $cmd = _shellEscape (@commandset);
	local *HANDLE1;
	open (HANDLE1, "$cmd |");
	local $/ = undef;
	my $result = <HANDLE1>;
	close (HANDLE1);

	shift @commandset;
	TEST ($result eq (join (" ", @commandset)."\n"));
};

test "script creation" => sub {
	my $pidfile = $testdir."/testservice.pid";
	local *TESTFILE;
	open (TESTFILE, ">$file") || die "Can't create $file";
	print TESTFILE makeSysVScript (
		'name'        => "testservice",
		'description' => "Test SysV script",
		'comment'     => "",
		'start'       =>
			"bash -c 'echo \$\$ >$pidfile; exec sleep 200' >/dev/null 2>&1",
		'pidfile'     => $pidfile,
		'asyncstart'  => 1,
	);
	close (TESTFILE);

	TEST (-f $file);
	TEST (-s _);
};

test "script start" => sub {
	my $ret = system ("bash", $file, "start");
	TEST (0 == $ret>>8);
};
test "status (must be on)" => sub {
	my $ret = system ("bash", $file, "status");
	TEST (0 == $ret>>8);
};
test "script double-start (must fail)" => sub {
	my $ret = system ("bash", $file, "start");
	TEST (0 != $ret>>8);
};
test "status (must be on)" => sub {
	my $ret = system ("bash", $file, "status");
	TEST (0 == $ret>>8);
};
test "script stop" => sub {
	my $ret = system ("bash", $file, "stop");
	TEST (0 == $ret>>8);
};
test "status (must be off)" => sub {
	my $ret = system ("bash", $file, "status");
	TEST (0 != $ret>>8);
};
test "script double-stop (must fail)" => sub {
	my $ret = system ("bash", $file, "stop");
	TEST (0 != $ret>>8);
};
test "status (must be off)" => sub {
	my $ret = system ("bash", $file, "status");
	TEST (0 != $ret>>8);
};
